@@ -43,6 +43,7 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #endif
 
 #include <string>
+#include "../../common/EmUtils.h"
 #include "../IGC/Compiler/CodeGenPublicEnums.h"
 // TODO: add all external declarations so that external projects only need
 // to include this file only.
@@ -86,7 +87,8 @@ typedef struct {
 typedef struct {
     CG_CTX_STATS_t  m_stats;              // record what simd has been generated
     void*           m_pixelShaderGen;     // Generated pixel shader output
-    std::string     m_savedBitcodeString; // Serialized Bitcode
+    char*           m_savedBitcodeCharArray; // Serialized Bitcode
+    unsigned int    m_savedBitcodeCharArraySize;
     void*           m_savedInstrTypes;
     SHADER_STATS_t  m_savedShaderStats;
 } CG_CTX_t;
@@ -107,38 +109,47 @@ typedef enum {
     FLAG_CG_ALL_SIMDS = 0,
     FLAG_CG_STAGE1_FAST_COMPILE = 1,
     FLAG_CG_STAGE1_BEST_PERF = 2,
+    FLAG_CG_STAGE1_FASTEST_COMPILE = 3,
 } CG_FLAG_t;
 
 #define IsStage2Available(ctx_ptr) (ctx_ptr != nullptr)
 
 #define IsStage2RestSIMDs(prev_ctx_ptr) (prev_ctx_ptr != nullptr)
 #define IsStage1FastCompile(flag, prev_ctx_ptr) (!IsStage2RestSIMDs(prev_ctx_ptr) && flag == FLAG_CG_STAGE1_FAST_COMPILE)
+#define IsStage1FastestCompile(flag, prev_ctx_ptr) (!IsStage2RestSIMDs(prev_ctx_ptr) && flag == FLAG_CG_STAGE1_FASTEST_COMPILE)
 #define IsStage1BestPerf(flag, prev_ctx_ptr)    (!IsStage2RestSIMDs(prev_ctx_ptr) && flag == FLAG_CG_STAGE1_BEST_PERF)
 #define IsAllSIMDs(flag, prev_ctx_ptr)          (!IsStage2RestSIMDs(prev_ctx_ptr) && flag == FLAG_CG_ALL_SIMDS)
 #define IsStage1(pCtx)   (IsStage1BestPerf(pCtx->m_CgFlag, pCtx->m_StagingCtx) || \
                           IsStage1FastCompile(pCtx->m_CgFlag, pCtx->m_StagingCtx))
 #define HasSavedIR(pCtx) (pCtx && IsStage2RestSIMDs(pCtx->m_StagingCtx) && \
-                          pCtx->m_StagingCtx->m_savedBitcodeString.size() > 0)
+                          pCtx->m_StagingCtx->m_savedBitcodeCharArraySize > 0)
 
 #define DoSimd32Stage2(prev_ctx_ptr) (IsStage2RestSIMDs(prev_ctx_ptr) && DoSimd32(prev_ctx_ptr->m_stats))
 #define DoSimd16Stage2(prev_ctx_ptr) (IsStage2RestSIMDs(prev_ctx_ptr) && DoSimd16(prev_ctx_ptr->m_stats))
 
-// We don't need compile continuation if no staged compilation enabled denoted by RegKeys.
+#define ContinueFastCompileStage1(flag, prev_ctx_ptr, stats) ( \
+    IsStage1FastCompile(flag, prev_ctx_ptr) && \
+    (IsRetry(stats) || DoSimd16(stats)))
+
 // If the staged compilation enabled, we don't need compile continuation when SIMD8 is spilled.
-#define HasCompileContinuation(flag, prev_ctx_ptr, stats) ( \
-    IGC_IS_FLAG_ENABLED(StagedCompilation) && \
-    ((IsStage1FastCompile(flag, prev_ctx_ptr) && \
-     !(!IsRetry(stats) && !DoSimd16(stats))) || \
-    ((IsStage1BestPerf(flag, prev_ctx_ptr)) && \
-     ( IGC_IS_FLAG_ENABLED(ExtraRetrySIMD16) && !HasSimdSpill(8, stats)) || \
-     (!IGC_IS_FLAG_ENABLED(ExtraRetrySIMD16) && (!HasSimd(8, stats) || DoSimd32(stats))))) \
-    )
+#define ContinueBestPerfStage1(flag, prev_ctx_ptr, stats) ( \
+    IsStage1BestPerf(flag, prev_ctx_ptr) && \
+     (( IGC_IS_FLAG_ENABLED(ExtraRetrySIMD16) && !HasSimdSpill(8, stats)) || \
+      (!IGC_IS_FLAG_ENABLED(ExtraRetrySIMD16) && (!HasSimd(8, stats) || DoSimd32(stats)))))
+
+// We don't need compile continuation if no staged compilation enabled denoted by RegKeys.
+#define HasCompileContinuation(Ail, flag, prev_ctx_ptr, stats) ( \
+    (IGC_IS_FLAG_ENABLED(StagedCompilation) || Ail) && \
+    (ContinueFastCompileStage1(flag, prev_ctx_ptr, stats) || \
+     ContinueBestPerfStage1(flag, prev_ctx_ptr, stats)))
 
 // Return true when simd MODE has been generated previously
 #define AvoidDupStage2(MODE, flag, prev_ctx_ptr)      (IsStage2RestSIMDs(prev_ctx_ptr) && HasSimdNoSpill(MODE, prev_ctx_ptr->m_stats))
 
 // Fast CG always returns simd 8
 #define ValidFastModes(flag, prev_ctx_ptr, stats)     (IsStage1FastCompile(flag, prev_ctx_ptr) && HasSimd(8, stats) && !HasSimd(16, stats) && !HasSimd(32, stats))
+// Fastest CG always returns simd 8
+#define ValidFastestModes(flag, prev_ctx_ptr, stats)     (IsStage1FastestCompile(flag, prev_ctx_ptr) && HasSimd(8, stats) && !HasSimd(16, stats) && !HasSimd(32, stats))
 // Best CG returns simd 8 or 16
 #define ValidBestModes(flag, prev_ctx_ptr, stats)     (IsStage1BestPerf(flag, prev_ctx_ptr) && (HasSimd(8, stats) || HasSimd(16, stats)) && !HasSimd(32, stats))
 // ALL_SIMDS CG must have simd 8 in any case
@@ -155,6 +166,7 @@ typedef enum {
     )
 
 #define ValidGeneratedModes(flag, prev_ctx_ptr, stats) ( \
+    ValidFastestModes(flag, prev_ctx_ptr, stats) || \
     ValidFastModes(flag, prev_ctx_ptr, stats) || \
     ValidBestModes(flag, prev_ctx_ptr, stats) || \
     ValidAllSimdsModes(flag, prev_ctx_ptr, stats) || \
