@@ -28,11 +28,12 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "Compiler/Optimizer/OpenCLPasses/KernelArgs.hpp"
 
 #include "common/LLVMWarningsPush.hpp"
-#include <llvm/IR/DerivedTypes.h>
-#include <llvmWrapper/IR/Function.h>
+#include <llvmWrapper/IR/DerivedTypes.h>
+#include <llvm/IR/Function.h>
 #include <llvm/IR/Metadata.h>
 #include <llvm/IR/Module.h>
 #include "common/LLVMWarningsPop.hpp"
+#include "Probe/Assertion.h"
 
 using namespace llvm;
 using namespace IGC;
@@ -102,7 +103,7 @@ Type* ImplicitArg::getLLVMType(LLVMContext& context) const
         baseType = Type::getInt8Ty(context)->getPointerTo(ADDRESS_SPACE_GLOBAL);
         break;
     default:
-        assert(0 && "Unrecognized implicit argument type");
+        IGC_ASSERT_MESSAGE(0, "Unrecognized implicit argument type");
         return nullptr;
     }
 
@@ -112,7 +113,7 @@ Type* ImplicitArg::getLLVMType(LLVMContext& context) const
     }
     else
     {
-        return VectorType::get(baseType, m_nbElement);
+        return IGCLLVM::FixedVectorType::get(baseType, m_nbElement);
     }
 }
 
@@ -145,7 +146,8 @@ VISA_Type ImplicitArg::getVISAType(const DataLayout& DL) const
     case PRIVATEPTR:
         return getPointerSize(DL) == 4 ? VISA_Type::ISA_TYPE_UD : VISA_Type::ISA_TYPE_UQ;
     default:
-        assert(0 && "Unrecognized implicit argument type");
+        IGC_ASSERT_MESSAGE(0, "Unrecognized implicit argument type");
+        break;
     }
 
     return VISA_Type::ISA_TYPE_UD;
@@ -157,7 +159,8 @@ unsigned int ImplicitArg::getPointerSize(const DataLayout& DL) const {
   case PRIVATEPTR:  return DL.getPointerSize(ADDRESS_SPACE_PRIVATE);
   case GLOBALPTR:   return DL.getPointerSize(ADDRESS_SPACE_GLOBAL);
   default:
-    assert(false && "Unrecognized pointer type");
+    IGC_ASSERT_MESSAGE(0, "Unrecognized pointer type");
+    break;
   }
   return 0;
 }
@@ -170,12 +173,13 @@ IGC::e_alignment ImplicitArg::getAlignType(const DataLayout& DL) const
            return IGC::EALIGN_DWORD;
         case ALIGN_QWORD:
             return IGC::EALIGN_QWORD;
-        case ALIGN_GRF:
-           return IGC::EALIGN_GRF;
+        case ALIGN_GRF: //According to old implementation, EALIGN_GRF = EALIGN_HWORD, the correpsonding alignmentSize is 32, so EALIGN_HWORD will not change the old define.
+           return IGC::EALIGN_HWORD;  //FIXME: But, the ALIGN_GRF is really GRF aligned? If so, there is bug here.
         case ALIGN_PTR:
           return getPointerSize(DL) == 4 ? IGC::EALIGN_DWORD : IGC::EALIGN_QWORD;
         default:
-            assert(0 && "Uknown alignment");
+            IGC_ASSERT_MESSAGE(0, "Uknown alignment");
+            break;
     }
 
     return IGC::EALIGN_DWORD;
@@ -218,7 +222,8 @@ unsigned int ImplicitArg::getAllocateSize(const DataLayout& DL) const
         elemSize = getPointerSize(DL);
         break;
     default:
-        assert(0 && "Unrecognized implicit argument type");
+        IGC_ASSERT_MESSAGE(0, "Unrecognized implicit argument type");
+        break;
     }
 
     return m_nbElement *  elemSize;
@@ -305,32 +310,29 @@ ImplicitArgs::ImplicitArgs(const llvm::Function& func , const MetaDataUtils* pMd
 
         IMPLICIT_ARGS.push_back(ImplicitArg(ImplicitArg::SYNC_BUFFER, "syncBuffer", ImplicitArg::GLOBALPTR, WIAnalysis::UNIFORM, 1, ImplicitArg::ALIGN_PTR, false));
 
-        assert(IMPLICIT_ARGS.size() == ImplicitArg::NUM_IMPLICIT_ARGS && "Mismatch in NUM_IMPLICIT_ARGS and IMPLICIT_ARGS vector");
+        IGC_ASSERT_MESSAGE((IMPLICIT_ARGS.size() == ImplicitArg::NUM_IMPLICIT_ARGS), "Mismatch in NUM_IMPLICIT_ARGS and IMPLICIT_ARGS vector");
 
-#ifdef _DEBUG
-        // Note: the order that implicit args are added here must match the
-        // order of the ImplicitArg::Argtype enum.  Let's check that they match:
-        uint32_t CurArgId = ImplicitArg::START_ID;
-        for (auto &Arg : IMPLICIT_ARGS)
         {
-            if (Arg.getArgType() != CurArgId++)
+            // Note: the order that implicit args are added here must match the
+            // order of the ImplicitArg::Argtype enum.  Let's check that they match:
+            uint32_t CurArgId = ImplicitArg::START_ID;
+            for (auto& Arg : IMPLICIT_ARGS)
             {
-                assert(0 && "enum and vector out of sync!");
+                IGC_ASSERT_MESSAGE((Arg.getArgType() == CurArgId++), "enum and vector out of sync!");
             }
         }
-#endif // _DEBUG
     }
 
     m_funcInfoMD = pMdUtils->getFunctionsInfoItem(const_cast<llvm::Function*>(&func));
 }
 
 const ImplicitArg& ImplicitArgs::operator[](unsigned int i) const {
-    assert(IMPLICIT_ARGS.size() == ImplicitArg::NUM_IMPLICIT_ARGS && "Mismatch in NUM_IMPLICIT_ARGS and IMPLICIT_ARGS vector");
+    IGC_ASSERT_MESSAGE((IMPLICIT_ARGS.size() == ImplicitArg::NUM_IMPLICIT_ARGS), "Mismatch in NUM_IMPLICIT_ARGS and IMPLICIT_ARGS vector");
     return IMPLICIT_ARGS[getArgType(i)];
 }
 
 unsigned int ImplicitArgs::getArgIndex(ImplicitArg::ArgType argType) {
-    assert(this->size() > 0 && "There are no implicit arguments!");
+    IGC_ASSERT_MESSAGE((this->size() > 0), "There are no implicit arguments!");
 
     // Find the first appearance of the given implicit arg type
     unsigned int implicitArgIndex = 0;
@@ -343,7 +345,7 @@ unsigned int ImplicitArgs::getArgIndex(ImplicitArg::ArgType argType) {
         }
     }
 
-    assert(implicitArgIndex < this->size() && "Implicit argument not found!");
+    IGC_ASSERT_MESSAGE((implicitArgIndex < this->size()), "Implicit argument not found!");
 
     return implicitArgIndex;
 }
@@ -387,12 +389,12 @@ bool ImplicitArgs::isImplicitArgExist(
 }
 
 unsigned int ImplicitArgs::getImageArgIndex(ImplicitArg::ArgType argType, const Argument* image) {
-    assert(isImplicitImage(argType) && "Non image/sampler implicit arg!");
+    IGC_ASSERT_MESSAGE(isImplicitImage(argType), "Non image/sampler implicit arg!");
     return getNumberedArgIndex(argType, image->getArgNo());
 }
 
 unsigned int ImplicitArgs::getNumberedArgIndex(ImplicitArg::ArgType argType, int argNum) {
-    assert((argNum >= 0) && "objectNum cannot be less than 0");
+    IGC_ASSERT_MESSAGE((argNum >= 0), "objectNum cannot be less than 0");
 
     for (int i = 0, e = m_funcInfoMD->size_ImplicitArgInfoList() ; i < e; ++i)
     {
@@ -403,11 +405,16 @@ unsigned int ImplicitArgs::getNumberedArgIndex(ImplicitArg::ArgType argType, int
         }
     }
 
-    assert(false && "No implicit argument for the given type & argNum");
+    IGC_ASSERT_MESSAGE(0, "No implicit argument for the given type & argNum");
     return m_funcInfoMD->size_ImplicitArgInfoList();
 }
 
 void ImplicitArgs::addImplicitArgs(llvm::Function& F, SmallVectorImpl<ImplicitArg::ArgType>& implicitArgs, MetaDataUtils* pMdUtils) {
+    // Stack calls does not support implicit arguments!
+    // Just return for now. TODO: Each pass should check if it's inserting implicit arg to stackcall function
+    if (F.hasFnAttribute("visaStackCall"))
+        return;
+
     // Add implicit args metadata for the given function
     FunctionInfoMetaDataHandle funcInfo = pMdUtils->getFunctionsInfoItem(&F);
     for (auto arg : implicitArgs)
@@ -419,7 +426,6 @@ void ImplicitArgs::addImplicitArgs(llvm::Function& F, SmallVectorImpl<ImplicitAr
             funcInfo->addImplicitArgInfoListItem(argMD);
         }
     }
-    pMdUtils->save(F.getParent()->getContext());
 }
 
 void ImplicitArgs::addImageArgs(llvm::Function& F, ImplicitArg::ArgMap& argMap, MetaDataUtils* pMdUtils)
@@ -436,7 +442,6 @@ void ImplicitArgs::addImageArgs(llvm::Function& F, ImplicitArg::ArgMap& argMap,
             funcInfo->addImplicitArgInfoListItem(argMD);
         }
     }
-    pMdUtils->save(F.getParent()->getContext());
 }
 
 void ImplicitArgs::addStructArgs(llvm::Function& F, const Argument* A, const ImplicitArg::StructArgList& S, MetaDataUtils* pMdUtils)
@@ -453,8 +458,6 @@ void ImplicitArgs::addStructArgs(llvm::Function& F, const Argument* A, const Imp
         argMD->setStructArgOffset(offset);
         funcInfo->addImplicitArgInfoListItem(argMD);
     }
-
-    pMdUtils->save(F.getParent()->getContext());
 }
 
 void ImplicitArgs::addNumberedArgs(llvm::Function& F, ImplicitArg::ArgMap& argMap, IGCMD::MetaDataUtils* pMdUtils)
@@ -472,7 +475,6 @@ void ImplicitArgs::addNumberedArgs(llvm::Function& F, ImplicitArg::ArgMap& argMa
       funcInfo->addImplicitArgInfoListItem(argMD);
     }
   }
-  pMdUtils->save(F.getParent()->getContext());
 }
 
 // Add one implicit argument for each pointer argument to global or constant buffer.
@@ -483,7 +485,11 @@ void ImplicitArgs::addBufferOffsetArgs(llvm::Function& F, IGCMD::MetaDataUtils*
     FunctionInfoMetaDataHandle funcInfoMD =
         pMdUtils->getFunctionsInfoItem(const_cast<Function*>(&F));
 
-    assert(modMD->FuncMD.find(&F) != modMD->FuncMD.end());
+    IGC_ASSERT(modMD->FuncMD.find(&F) != modMD->FuncMD.end());
+
+    // StatelessToStatefull optimization is not applied on non-kernel functions.
+    if (!isEntryFunc(pMdUtils, &F))
+        return;
 
     FunctionMetaData* funcMD = &modMD->FuncMD.find(&F)->second;
     for (auto& Arg : F.args() )
@@ -535,7 +541,7 @@ bool ImplicitArgs::isImplicitStruct(ImplicitArg::ArgType argType)
 }
 
 ImplicitArg::ArgType ImplicitArgs::getArgType(unsigned int index) const {
-    assert(index < size() && "Index out of range");
+    IGC_ASSERT_MESSAGE((index < size()), "Index out of range");
     ArgInfoMetaDataHandle argInfo = m_funcInfoMD->getImplicitArgInfoListItem(index);
     return (ImplicitArg::ArgType)argInfo->getArgId();
 }
@@ -567,10 +573,10 @@ int32_t ImplicitArgs::getStructArgOffset(unsigned int index) const
 
 TODO("Refactor code to avoid code triplication for getArgInFunc(), getImplicitArg() and WIFuncResolution::getImplicitArg()")
 Argument* ImplicitArgs::getArgInFunc(llvm::Function& F, ImplicitArg::ArgType argType) {
-    assert(IGCLLVM::GetFuncArgSize(F) >= size() && "Invalid number of argumnents in the function!");
+    IGC_ASSERT_MESSAGE((F.arg_size() >= size()), "Invalid number of argumnents in the function!");
 
     unsigned int argIndex       =  getArgIndex(argType);
-    unsigned int argIndexInFunc = IGCLLVM::GetFuncArgSize(F) - size() + argIndex;
+    unsigned int argIndexInFunc = F.arg_size() - size() + argIndex;
     Function::arg_iterator arg  = F.arg_begin();
     for (unsigned int i = 0; i < argIndexInFunc; ++i,  ++arg);
 
@@ -584,7 +590,7 @@ Argument* ImplicitArgs::getImplicitArg(llvm::Function& F, ImplicitArg::ArgType a
         return nullptr;
     unsigned int implicitArgIndex = this->getArgIndex(argType);
 
-    unsigned int implicitArgIndexInFunc = IGCLLVM::GetFuncArgSize(F) - numImplicitArgs + implicitArgIndex;
+    unsigned int implicitArgIndexInFunc = F.arg_size() - numImplicitArgs + implicitArgIndex;
 
     Function::arg_iterator arg = F.arg_begin();
     for (unsigned int i = 0; i < implicitArgIndexInFunc; ++i, ++arg);
@@ -594,14 +600,14 @@ Argument* ImplicitArgs::getImplicitArg(llvm::Function& F, ImplicitArg::ArgType a
 
 Argument* ImplicitArgs::getNumberedImplicitArg(llvm::Function& F, ImplicitArg::ArgType argType, int argNum)
 {
-    assert(IGCLLVM::GetFuncArgSize(F) >= size() && "Invalid number of arguments in the function!");
+    IGC_ASSERT_MESSAGE((F.arg_size() >= size()), "Invalid number of arguments in the function!");
 
     unsigned int numImplicitArgs = size();
     unsigned int implicitArgIndex = this->getNumberedArgIndex(argType, argNum);
     if (implicitArgIndex == numImplicitArgs)
       return nullptr;
 
-    unsigned int implicitArgIndexInFunc = IGCLLVM::GetFuncArgSize(F) - numImplicitArgs + implicitArgIndex;
+    unsigned int implicitArgIndexInFunc = F.arg_size() - numImplicitArgs + implicitArgIndex;
 
     Function::arg_iterator arg = F.arg_begin();
     for (unsigned int i = 0; i < implicitArgIndexInFunc; ++i, ++arg);
