@@ -25,6 +25,7 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 ======================= end_copyright_notice ==================================*/
 
 #include "ProcessFuncAttributes.h"
+#include "Compiler/CISACodeGen/EstimateFunctionSize.h"
 #include "Compiler/MetaDataApi/IGCMetaDataHelper.h"
 #include "Compiler/MetaDataUtilsWrapper.h"
 #include "Compiler/IGCPassSupport.h"
@@ -34,7 +35,8 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 #include "common/LLVMWarningsPush.hpp"
 
-#include "llvmWrapper/IR/Attributes.h"
+#include "llvm/IR/Attributes.h"
+#include "llvmWrapper/IR/Instructions.h"
 
 #include <llvm/Pass.h>
 #include <llvm/IR/Module.h>
@@ -58,11 +60,12 @@ class ProcessFuncAttributes : public ModulePass
 {
 public:
     static char ID;
-    virtual void getAnalysisUsage(llvm::AnalysisUsage &AU) const
+    virtual void getAnalysisUsage(llvm::AnalysisUsage& AU) const
     {
         AU.setPreservesCFG();
         AU.addRequired<MetaDataUtilsWrapper>();
         AU.addRequired<CodeGenContextWrapper>();
+        AU.addRequired<EstimateFunctionSize>();
     }
 
     ProcessFuncAttributes();
@@ -79,6 +82,7 @@ public:
 private:
     bool isGASPointer(Value* arg);
 
+    bool shouldForceToInline(const Function* F);
 };
 
 } // namespace
@@ -90,6 +94,8 @@ private:
 #define PASS_ANALYSIS false
 IGC_INITIALIZE_PASS_BEGIN(ProcessFuncAttributes, PASS_FLAG, PASS_DESCRIPTION, PASS_CFG_ONLY, PASS_ANALYSIS)
 IGC_INITIALIZE_PASS_DEPENDENCY(MetaDataUtilsWrapper)
+IGC_INITIALIZE_PASS_DEPENDENCY(CodeGenContextWrapper)
+IGC_INITIALIZE_PASS_DEPENDENCY(EstimateFunctionSize)
 IGC_INITIALIZE_PASS_END(ProcessFuncAttributes, PASS_FLAG, PASS_DESCRIPTION, PASS_CFG_ONLY, PASS_ANALYSIS)
 
 char ProcessFuncAttributes::ID = 0;
@@ -187,6 +193,27 @@ static DenseSet<Function*> collectMemPoolUsage(const Module &M)
     return FuncsToInline;
 }
 
+bool ProcessFuncAttributes::shouldForceToInline(const Function* F)
+{
+    for (auto& arg : F->args())
+    {
+        if (containsOpaque(arg.getType()))
+        {
+            return true;
+        }
+    }
+
+    // SPIR-V image functions don't contain opaque types for images,
+    // they use i64 values instead.
+    // We need to detect them based on function name.
+    if (F->getName().startswith(spv::kLLVMName::builtinPrefix) &&
+        F->getName().contains("Image")) {
+        return true;
+    }
+
+    return false;
+}
+
 bool ProcessFuncAttributes::runOnModule(Module& M)
 {
     MetaDataUtilsWrapper &mduw = getAnalysis<MetaDataUtilsWrapper>();
@@ -194,6 +221,7 @@ bool ProcessFuncAttributes::runOnModule(Module& M)
     ModuleMetaData *modMD = mduw.getModuleMetaData();
     auto MemPoolFuncs = collectMemPoolUsage(M);
     CodeGenContext* pCtx = getAnalysis<CodeGenContextWrapper>().getCodeGenContext();
+    EstimateFunctionSize& efs = getAnalysis<EstimateFunctionSize>();
 
     std::set<llvm::Function *> fastMathFunct;
     GlobalVariable *gv_fastMath = M.getGlobalVariable("__FastRelaxedMath", true);
@@ -215,25 +243,6 @@ bool ProcessFuncAttributes::runOnModule(Module& M)
         }
     }
 
-    // lambda for setting indirect function attributes
-    auto SetIndirectFuncAttributes = [this](Function* F)->void
-    {
-        F->addFnAttr("IndirectlyCalled");
-        F->addFnAttr("visaStackCall");
-
-        if (!F->isDeclaration())
-        {
-            // Require global relocation if any global values are used in indirect functions, since we cannot pass implicit args
-            F->addFnAttr("EnableGlobalRelocation");
-
-            if (IGC_GET_FLAG_VALUE(FunctionControl) == FLAG_FCALL_FORCE_INDIRECTCALL)
-            {
-                F->removeFnAttr(llvm::Attribute::AlwaysInline);
-                F->addFnAttr(llvm::Attribute::NoInline);
-            }
-        }
-    };
-
     // 1. Set function's linkage type to InternalLinkage (C's static) so that
     //    LLVM can remove the dead functions asap, which saves compiling time.
     //    Only non-kernel function with function bodies are set.
@@ -243,71 +252,43 @@ bool ProcessFuncAttributes::runOnModule(Module& M)
     bool Changed = false;
     for (Module::iterator I = M.begin(), E = M.end(); I != E; ++I)
     {
-        Function *F = &(*I);
+        Function* F = &(*I);
         if (F->isDeclaration())
         {
             if (F->getName() == "__translate_sampler_initializer")
                 F->addFnAttr(llvm::Attribute::ReadOnly);
             if (F->hasFnAttribute("referenced-indirectly"))
             {
+                // External function not defined in current module
                 pCtx->m_enableFunctionPointer = true;
-                SetIndirectFuncAttributes(F);
+                F->addFnAttr("IndirectlyCalled");
+                F->addFnAttr("visaStackCall");
             }
 
             // It is not a defined function
             continue;
         }
 
-        // If EnableOCLNoInlineAttr is on and F does have
-        // NoInline, do not reset it.
-        if (IGC_IS_FLAG_ENABLED(EnableOCLNoInlineAttr) &&
-            pCtx->type == ShaderType::OPENCL_SHADER &&
-            F->hasFnAttribute(llvm::Attribute::NoInline) &&
-            !F->hasFnAttribute(llvm::Attribute::Builtin))
-        {
-            continue;
-        }
-
         // Do not reset it for critical section builtins
         if (F->hasFnAttribute("KMPLOCK"))
         {
             continue;
         }
-
-        // Remove noinline attr if present.
-        F->removeFnAttr(llvm::Attribute::NoInline);
-
-        if (IGC_IS_FLAG_ENABLED(DisableAddingAlwaysAttribute))
+        // Do not reset attributes for SYCL unmasked functions.
+        if (IGC_IS_FLAG_ENABLED(EnableUnmaskedFunctions) && F->hasFnAttribute("sycl-unmasked"))
         {
-            // Add always attribute if function has an argument with opaque type
-            // Curently, ExtensionArgAnalysis assumes that all functions with image arguments to be inlined
-            // This patch makes sure that we add always inline for such cases
-            for (auto& arg : F->args())
-            {
-                if (containsOpaque(arg.getType()))
-                {
-                    F->addFnAttr(llvm::Attribute::AlwaysInline);
-                    break;
-                }
-            }
-
-            // Add always attribtue if function is a builtin
-            if (F->hasFnAttribute(llvm::Attribute::Builtin))
-            {
-                F->addFnAttr(llvm::Attribute::AlwaysInline);
-            }
-        }
-        else
-        {
-            // Add AlwaysInline attribute to force inlining all calls.
-            F->addFnAttr(llvm::Attribute::AlwaysInline);
+            continue;
         }
 
         // Go through call sites and remove NoInline atrributes.
         for (auto I : F->users()) {
             if (CallInst* callInst = dyn_cast<CallInst>(&*I)) {
                 if (callInst->hasFnAttr(llvm::Attribute::NoInline)) {
-                    callInst->removeAttribute(IGCLLVM::AttributeSet::FunctionIndex, llvm::Attribute::NoInline);
+                    callInst->removeAttribute(AttributeList::FunctionIndex, llvm::Attribute::NoInline);
+                }
+                if (getAnalysis<MetaDataUtilsWrapper>().getModuleMetaData()->compOpt.OptDisable &&
+                    callInst->hasFnAttr(llvm::Attribute::AlwaysInline)) {
+                    callInst->removeAttribute(AttributeList::FunctionIndex, llvm::Attribute::AlwaysInline);
                 }
             }
         }
@@ -315,8 +296,7 @@ bool ProcessFuncAttributes::runOnModule(Module& M)
         // set function attributes according to build options so
         // inliner doesn't conservatively turn off unsafe optimizations
         // when inlining BIFs (see mergeAttributesForInlining() in inliner).
-
-        const auto &opts = modMD->compOpt;
+        const auto& opts = modMD->compOpt;
 
         if (opts.MadEnable)
             F->addFnAttr("less-precise-fpmad", "true");
@@ -330,55 +310,6 @@ bool ProcessFuncAttributes::runOnModule(Module& M)
             F->addFnAttr("no-nans-fp-math", "true");
         }
 
-        // F is not a kernel
-        // it is builtin, or user function
-        const bool notKernel =  pMdUtils->findFunctionsInfoItem(F) == pMdUtils->end_FunctionsInfo();
-
-        if (notKernel)
-        {
-            F->setLinkage(GlobalValue::InternalLinkage);
-            Changed = true;
-        }
-
-        // inline all OCL math functions if __FastRelaxedMath is set
-        if (fastMathFunct.find(F) != fastMathFunct.end()) continue;
-
-        // The following subroutine check is added to disable two-phase-inlining
-        // when we do not enable subroutines.
-        bool keepAlwaysInline = (MemPoolFuncs.count(F) != 0);
-        if (IGC_GET_FLAG_VALUE(FunctionControl) != FLAG_FCALL_FORCE_INLINE)
-        {
-            if (!keepAlwaysInline)
-            {
-                for (auto &arg : F->args())
-                {
-                    // If argument contains an opaque type e.g. image, then always inline it.
-                    // If argument is a pointer to GAS, always inline it for perf reason.
-                    //
-                    // Note that this workaround should be removed.
-                    if (containsOpaque(arg.getType()) || isSupportedAggregateArgument(&arg) ||
-                        isGASPointer(&arg))
-                    {
-                        keepAlwaysInline = true;
-                        break;
-                    }
-                }
-
-                // SPIR-V image functions don't contain opaque types for images,
-                // they use i64 values instead.
-                // We need to detect them based on function name.
-                if (F->getName().startswith(spv::kLLVMName::builtinPrefix) &&
-                    F->getName().contains("Image")) {
-                  keepAlwaysInline = true;
-                }
-            }
-
-            if (!keepAlwaysInline)
-            {
-                F->removeFnAttr(llvm::Attribute::AlwaysInline);
-            }
-        }
-
         // Add Optnone to user functions but not on builtins. This allows to run
         // optimizations on builtins.
         if (getAnalysis<MetaDataUtilsWrapper>().getModuleMetaData()->compOpt.OptDisable)
@@ -390,59 +321,201 @@ bool ProcessFuncAttributes::runOnModule(Module& M)
         }
 
         bool istrue = false;
-        if (notKernel || istrue)
-        {
-            if (!keepAlwaysInline)
-            {
-                bool forceSubroutine = IGC_GET_FLAG_VALUE(FunctionControl) == FLAG_FCALL_FORCE_SUBROUTINE;
-                bool forceStackCall = IGC_GET_FLAG_VALUE(FunctionControl) == FLAG_FCALL_FORCE_STACKCALL;
 
-                if ((forceSubroutine || forceStackCall) && (istrue == false))
-                {
-                    // add the following line in order to stress-test
-                    // subroutine call or stack call
-                    F->removeFnAttr(llvm::Attribute::AlwaysInline);
-                    F->addFnAttr(llvm::Attribute::NoInline);
-                    if (forceStackCall)
-                    {
-                        F->addFnAttr("visaStackCall");
+        // set hasVLA function attribute
+        bool isSet = false;
+        for (auto& BB : F->getBasicBlockList()) {
+            for (auto& Inst : BB.getInstList()) {
+                if (AllocaInst * AI = dyn_cast<AllocaInst>(&Inst)) {
+                    if (!isa<ConstantInt>(AI->getArraySize())) {
+                        F->addFnAttr("hasVLA");
+                        isSet = true;
+                        break;
                     }
                 }
             }
+            if (isSet)
+                break;
+        }
 
-            if (IGC_IS_FLAG_ENABLED(EnableFunctionPointer))
+        const bool isKernel = isEntryFunc(pMdUtils, F);
+        if (isKernel && !istrue)
+        {
+            // No need to process kernel funcs any further
+            continue;
+        }
+        else if (!isKernel)
+        {
+            F->setLinkage(GlobalValue::InternalLinkage);
+            Changed = true;
+        }
+
+        // Add function attribute for indirectly called functions
+        if (IGC_IS_FLAG_ENABLED(EnableFunctionPointer))
+        {
+            // Check if the function can be called either from
+            // externally or as a function pointer
+            bool isExtern = (F->hasFnAttribute("referenced-indirectly"));
+            bool isIndirect = false;
+            for (auto u = F->user_begin(), e = F->user_end(); u != e; u++)
             {
-                // Check if the function can be called either from
-                // externally or as a function pointer
-                bool isExtern = (F->hasFnAttribute("referenced-indirectly"));
-                bool isIndirect = false;
+                CallInst* call = dyn_cast<CallInst>(*u);
 
-                for (auto u = F->user_begin(), e = F->user_end(); u != e; u++)
+                if (!call || IGCLLVM::getCalledValue(call) != F)
                 {
-                    CallInst* call = dyn_cast<CallInst>(*u);
+                    isIndirect = true;
+                }
+            }
 
-                    if (!call || call->getCalledValue() != F)
+            // Add indirect call function attributes
+            if (isExtern || isIndirect)
+            {
+                pCtx->m_enableFunctionPointer = true;
+
+                F->addFnAttr("IndirectlyCalled");
+                if (!istrue)
+                {
+                    F->addFnAttr("visaStackCall");
+                }
+                if (isExtern)
+                {
+                    F->setLinkage(GlobalValue::ExternalLinkage);
+                }
+                Changed = true;
+            }
+        }
+
+        if (isKernel)
+            continue;
+
+        // Flag for function calls where alwaysinline must be true
+        bool mustAlwaysInline = false;
+
+        // Add always attribute if function has an argument with opaque type
+        for (auto& arg : F->args())
+        {
+            if (containsOpaque(arg.getType()))
+            {
+                mustAlwaysInline = true;
+                break;
+            }
+        }
+        // Add always attribtue if function is a builtin
+        if (F->hasFnAttribute(llvm::Attribute::Builtin) ||
+            F->getName().startswith(spv::kLLVMName::builtinPrefix))
+        {
+            mustAlwaysInline = true;
+        }
+        // inline all OCL math functions if __FastRelaxedMath is set
+        else if (fastMathFunct.find(F) != fastMathFunct.end())
+        {
+            mustAlwaysInline = true;
+        }
+        // Enable inlining for -O0 in order to preserve debug info. This may be removed when debug stack call support is enabled.
+        else if (getAnalysis<MetaDataUtilsWrapper>().getModuleMetaData()->compOpt.OptDisable &&
+            IGC_GET_FLAG_VALUE(FunctionControl) == FLAG_FCALL_DEFAULT)
+        {
+            mustAlwaysInline = true;
+        }
+        if (mustAlwaysInline)
+        {
+            F->removeFnAttr(llvm::Attribute::NoInline);
+            F->addFnAttr(llvm::Attribute::AlwaysInline);
+
+            Changed = true;
+            continue;
+        }
+
+        // Fixme: Can we support user noinline attrib from non-OCL shader?
+        if (pCtx->type != ShaderType::OPENCL_SHADER)
+        {
+            F->removeFnAttr(llvm::Attribute::NoInline);
+        }
+
+        // Set default inline mode
+        auto FCtrl = IGC_GET_FLAG_VALUE(FunctionControl);
+        if (FCtrl == FLAG_FCALL_DEFAULT)
+        {
+            const bool forceStackCall = F->hasFnAttribute("igc-force-stackcall");
+            // If this flag is enabled, default function call mode will be stackcall.
+            // Otherwise subroutines are used.
+            if (IGC_IS_FLAG_ENABLED(EnableStackCallFuncCall) || forceStackCall)
+            {
+                pCtx->m_enableStackCall = true;
+                F->addFnAttr("visaStackCall");
+                if (forceStackCall) {
+                    F->removeFnAttr(llvm::Attribute::AlwaysInline);
+                    F->addFnAttr(llvm::Attribute::NoInline);
+                }
+            }
+
+            if (!F->hasFnAttribute(llvm::Attribute::NoInline) &&
+                IGC_IS_FLAG_DISABLED(DisableAddingAlwaysAttribute))
+            {
+                bool shouldAlwaysInline = (MemPoolFuncs.count(F) != 0);
+
+                if (!shouldAlwaysInline)
+                {
+                    for (auto& arg : F->args())
                     {
-                        isIndirect = true;
+                        // If argument is a pointer to GAS or aggregate type, always inline it for perf reasons
+                        if (isSupportedAggregateArgument(&arg) || isGASPointer(&arg))
+                        {
+                            shouldAlwaysInline = true;
+                            break;
+                        }
                     }
                 }
-
-                if (isExtern || isIndirect)
+                if (shouldAlwaysInline)
                 {
-                    pCtx->m_enableFunctionPointer = true;
-                    SetIndirectFuncAttributes(F);
-
-                    if(istrue)
-                        F->removeFnAttr("visaStackCall");
-
-                    if (isExtern)
+                    if (IGC_IS_FLAG_ENABLED(ControlKernelTotalSize) &&
+                        pCtx->m_enableSubroutine &&
+                        efs.isTrimmedFunction(F))
                     {
-                        F->setLinkage(GlobalValue::ExternalLinkage);
+                        if( ( IGC_GET_FLAG_VALUE( PrintControlKernelTotalSize ) & 0x4 ) != 0 )
+                        {
+                            std::cout << "Trimmed function " << F->getName().str() << std::endl;
+                        }
+
+                        if (IGC_IS_FLAG_ENABLED(AddNoInlineToTrimmedFunctions))
+                        {
+                            F->addFnAttr(llvm::Attribute::NoInline);
+                        }
+                    }
+                    else
+                    {
+                        F->addFnAttr(llvm::Attribute::AlwaysInline);
                     }
                 }
             }
         }
-        Changed = true;
+        else if (FCtrl == FLAG_FCALL_FORCE_INLINE)
+        {
+            // Forced inlining all functions
+            F->removeFnAttr(llvm::Attribute::NoInline);
+            F->addFnAttr(llvm::Attribute::AlwaysInline);
+        }
+        else
+        {
+            // Forcing subroutines/stack-call/indirect-call
+            // Do not add alwaysinline
+            bool forceSubroutine = FCtrl == FLAG_FCALL_FORCE_SUBROUTINE;
+            bool forceStackCall = FCtrl == FLAG_FCALL_FORCE_STACKCALL;
+            bool forceIndirectCall = F->hasFnAttribute("IndirectlyCalled") &&
+                (FCtrl == FLAG_FCALL_FORCE_INDIRECTCALL || F->hasFnAttribute("IFCALL_BUILTIN"));
+
+            if (forceSubroutine || forceStackCall || forceIndirectCall)
+            {
+                F->removeFnAttr(llvm::Attribute::AlwaysInline);
+                F->addFnAttr(llvm::Attribute::NoInline);
+                if (forceStackCall)
+                {
+                    pCtx->m_enableStackCall = true;
+                    F->addFnAttr("visaStackCall");
+                }
+                Changed = true;
+            }
+        }
     }
     return Changed;
 }
@@ -516,7 +589,7 @@ bool ProcessBuiltinMetaData::runOnModule(Module& M)
     bool Changed = false;
     for (Module::iterator I = M.begin(), E = M.end(); I != E; ++I)
     {
-        Function *F = &(*I);
+        Function* F = &(*I);
         if (!F || F->isDeclaration()) continue;
 
         // add AlwaysInline for functions. It will be handle in optimization phase
@@ -533,6 +606,7 @@ bool ProcessBuiltinMetaData::runOnModule(Module& M)
         }
         Changed = true;
     }
+
     return Changed;
 }
 
@@ -549,9 +623,115 @@ void ProcessBuiltinMetaData::updateBuiltinFunctionMetaData(llvm::Function* pFunc
         llvm::raw_string_ostream x(typeStr);
         arg->getType()->print(x);
 
-        funcMD->m_OpenCLArgNames.push_back(arg->getName());
+        funcMD->m_OpenCLArgNames.push_back(arg->getName().str());
         funcMD->m_OpenCLArgAccessQualifiers.push_back("none");
         funcMD->m_OpenCLArgBaseTypes.push_back(x.str());
     }
     m_pMdUtil->setFunctionsInfoItem(pFunc, fHandle);
 }
+
+
+//
+// InsertDummyKernelForSymbolTable
+//
+namespace {
+
+    class InsertDummyKernelForSymbolTable : public ModulePass
+    {
+    public:
+        static char ID;
+        virtual void getAnalysisUsage(llvm::AnalysisUsage& AU) const
+        {
+            AU.setPreservesCFG();
+            AU.addRequired<MetaDataUtilsWrapper>();
+            AU.addRequired<CodeGenContextWrapper>();
+        }
+
+        InsertDummyKernelForSymbolTable();
+
+        ~InsertDummyKernelForSymbolTable() {}
+
+        virtual bool runOnModule(Module& M);
+
+        virtual llvm::StringRef getPassName() const
+        {
+            return "InsertDummyKernelForSymbolTable";
+        }
+    private:
+    };
+
+} // namespace
+
+// Register pass to igc-opt
+#define PASS_FLAG3 "igc-insert-dummy-kernel-for-symbol-table"
+#define PASS_DESCRIPTION3 "If symbol table is needed, insert a dummy kernel to attach it to"
+#define PASS_CFG_ONLY3 false
+#define PASS_ANALYSIS3 false
+IGC_INITIALIZE_PASS_BEGIN(InsertDummyKernelForSymbolTable, PASS_FLAG3, PASS_DESCRIPTION3, PASS_CFG_ONLY3, PASS_ANALYSIS3)
+IGC_INITIALIZE_PASS_DEPENDENCY(MetaDataUtilsWrapper)
+IGC_INITIALIZE_PASS_DEPENDENCY(CodeGenContextWrapper)
+IGC_INITIALIZE_PASS_END(InsertDummyKernelForSymbolTable, PASS_FLAG3, PASS_DESCRIPTION3, PASS_CFG_ONLY3, PASS_ANALYSIS3)
+
+char InsertDummyKernelForSymbolTable::ID = 0;
+
+InsertDummyKernelForSymbolTable::InsertDummyKernelForSymbolTable() : ModulePass(ID)
+{
+    initializeInsertDummyKernelForSymbolTablePass(*PassRegistry::getPassRegistry());
+}
+
+ModulePass* createInsertDummyKernelForSymbolTablePass()
+{
+    return new InsertDummyKernelForSymbolTable();
+}
+
+bool InsertDummyKernelForSymbolTable::runOnModule(Module& M)
+{
+    MetaDataUtilsWrapper& mduw = getAnalysis<MetaDataUtilsWrapper>();
+    MetaDataUtils* pMdUtils = mduw.getMetaDataUtils();
+    ModuleMetaData* modMD = mduw.getModuleMetaData();
+    CodeGenContext* pCtx = getAnalysis<CodeGenContextWrapper>().getCodeGenContext();
+
+    bool needDummyKernel = false;
+
+    // Creates an empty dummy kernel.
+    // This kernel will only be used for creating the symbol table.
+    // All indirectly called functions will also be attached to this kernel's binary.
+    if (IGC_IS_FLAG_ENABLED(EnableFunctionPointer) &&
+        pCtx->type == ShaderType::OPENCL_SHADER)
+    {
+        if (pCtx->m_enableFunctionPointer)
+        {
+            // Symbols are needed for external functions and function pointers
+            needDummyKernel = true;
+        }
+        else if (!modMD->inlineProgramScopeOffsets.empty())
+        {
+            // Create one also if global variables are present and require symbols
+            needDummyKernel = true;
+        }
+    }
+
+    if (needDummyKernel)
+    {
+        // Create empty kernel function
+        IGC_ASSERT(IGC::getIntelSymbolTableVoidProgram(&M) == nullptr);
+        Type* voidTy = Type::getVoidTy(M.getContext());
+        FunctionType* fTy = FunctionType::get(voidTy, false);
+        Function* pNewFunc = Function::Create(fTy, GlobalValue::ExternalLinkage, IGC::INTEL_SYMBOL_TABLE_VOID_PROGRAM, &M);
+        BasicBlock* entry = BasicBlock::Create(M.getContext(), "entry", pNewFunc);
+        IRBuilder<> builder(entry);
+        builder.CreateRetVoid();
+
+        // Set spirv calling convention and kernel metadata
+        pNewFunc->setCallingConv(llvm::CallingConv::SPIR_KERNEL);
+        IGCMD::FunctionInfoMetaDataHandle fHandle = IGCMD::FunctionInfoMetaDataHandle(IGCMD::FunctionInfoMetaData::get());
+        FunctionMetaData* funcMD = &modMD->FuncMD[pNewFunc];
+        funcMD->functionType = IGC::FunctionTypeMD::KernelFunction;
+        fHandle->setType(FunctionTypeMD::KernelFunction);
+        pMdUtils->setFunctionsInfoItem(pNewFunc, fHandle);
+        pMdUtils->save(M.getContext());
+
+        return true;
+    }
+    return false;
+}
