@@ -23,8 +23,11 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 
 ======================= end_copyright_notice ==================================*/
+#include "llvmWrapper/Support/Alignment.h"
 #include "TypesLegalizationPass.hpp"
 #include "../Compiler/IGCPassSupport.h"
+#include "Probe/Assertion.h"
+
 using namespace llvm;
 
 // Register pass to igc-opt
@@ -210,7 +213,7 @@ TypesLegalizationPass::ResolveValue( Instruction *ip,Value *val,SmallVector<unsi
     dyn_cast<InsertElementInst>(val))
   {
     IRBuilder<> builder( ie );
-    assert( indices.size() == 1 );
+    IGC_ASSERT(indices.size() == 1);
     return builder.CreateExtractElement( ie,builder.getInt32( indices[0] ) );
   }
   else if(LoadInst* ld = dyn_cast<LoadInst>(val))
@@ -220,7 +223,7 @@ TypesLegalizationPass::ResolveValue( Instruction *ip,Value *val,SmallVector<unsi
     unsigned alignment = ld->getAlignment();
     unsigned pointerTypeSize = gep->getType()->getPointerElementType()->getScalarSizeInBits() / 8;
     if ( alignment && pointerTypeSize == alignment )
-      return builder.CreateAlignedLoad( gep, alignment );
+      return builder.CreateAlignedLoad( gep, IGCLLVM::getAlign(alignment) );
     return builder.CreateLoad( gep );
   }
   else if(Constant *c = dyn_cast<Constant>(val)) {
@@ -270,7 +273,7 @@ TypesLegalizationPass::ResolveValue( Instruction *ip,Value *val,SmallVector<unsi
   }
   else if (SelectInst* select = dyn_cast<SelectInst>(val))
   {
-    assert(3 == select->getNumOperands());
+    IGC_ASSERT(3 == select->getNumOperands());
 
     Value* condition = select->getOperand(0);
     Value* whenTrue = select->getOperand(1);
@@ -284,7 +287,7 @@ TypesLegalizationPass::ResolveValue( Instruction *ip,Value *val,SmallVector<unsi
   }
 
   // What other kind of instruction can we have here?
-  assert( !"Unresolved instruction!" );
+  IGC_ASSERT_MESSAGE(0, "Unresolved instruction!");
 
   // Fallback by creating an ExtractValueInstr.
   IRBuilder<> builder( ip );
