@@ -38,11 +38,10 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "common/igc_regkeys.hpp"
 #include "common/SysUtils.hpp"
 #include "common/secure_string.h" // strcpy_s()
+#include "Probe/Assertion.h"
 
 #if defined(IGC_DEBUG_VARIABLES)
-
 #include "3d/common/iStdLib/File.h"
-
 #endif
 
 
@@ -135,7 +134,7 @@ namespace IGC
                 CASE(VISA_DOTALL);
                 CASE(VISA_SLOWPATH);
                 CASE(VISA_NOBXMLENCODER);
-            default : assert( 0 && "unknown DebugFlag" ); return "<unknown>";
+            default : IGC_ASSERT_EXIT_MESSAGE(0, "unknown DebugFlag"); return "<unknown>";
             }
 #undef CASE
         }
@@ -160,7 +159,7 @@ namespace IGC
                 CASE(LLVM_OPT_STAT_TEXT);
                 CASE(TIME_STATS_TEXT);
                 CASE(TIME_STATS_CSV);
-            default : assert( 0 && "unknown DumpType" ); return "<unknown>";
+            default : IGC_ASSERT_EXIT_MESSAGE(0, "unknown DumpType"); return "<unknown>";
             }
 #undef CASE
         }
@@ -172,7 +171,7 @@ namespace IGC
             {
                 CASE(ODS);
                 CASE(FILE);
-            default : assert( 0 && "unknown DumpLoc" ); return "<unknown>";
+            default : IGC_ASSERT_EXIT_MESSAGE(0, "unknown DumpLoc"); return "<unknown>";
             }
 #undef CASE
         }
@@ -259,10 +258,10 @@ namespace IGC
 
         void IGC_DEBUG_API_CALL SetDebugFlag( DebugFlag flag, bool enabled )
         {
+            IGC_ASSERT_EXIT_MESSAGE((0 <= static_cast<int>(flag)), "range sanity check");
+            IGC_ASSERT_EXIT_MESSAGE((static_cast<int>(flag) < static_cast<int> (DebugFlag::END)), "range sanity check");
+
 #if defined( _DEBUG ) || defined( _INTERNAL )
-            assert( 0 <= static_cast<int>(flag) &&
-                        static_cast<int>(flag) < static_cast<int>( DebugFlag::END ) &&
-                        "range sanity check" );
             g_debugFlags[ static_cast<int>( flag ) ] = enabled;
 #else
             (void) flag;
@@ -272,6 +271,9 @@ namespace IGC
 
         bool IGC_DEBUG_API_CALL GetDebugFlag( DebugFlag flag )
         {
+            IGC_ASSERT_EXIT_MESSAGE((0 <= static_cast<int>(flag)), "range sanity check");
+            IGC_ASSERT_EXIT_MESSAGE((static_cast<int>(flag) < static_cast<int> (DebugFlag::END)), "range sanity check");
+
 #if defined( _DEBUG ) || defined( _INTERNAL )
 
 #if defined(_WIN32 )|| defined( _WIN64 )
@@ -288,9 +290,6 @@ namespace IGC
 
             }
 #endif
-            assert( 0 <= static_cast<int>(flag) &&
-                        static_cast<int>(flag) < static_cast<int>( DebugFlag::END ) &&
-                        "range sanity check" );
             return g_debugFlags[ static_cast<int>(flag) ];
 #else
             (void) flag;
@@ -300,15 +299,15 @@ namespace IGC
 
         void IGC_DEBUG_API_CALL SetDumpFlag( DumpType type, DumpLoc loc, bool enabled )
         {
+            IGC_ASSERT_EXIT_MESSAGE((0 <= static_cast<int>(type)), "range sanity check");
+            IGC_ASSERT_EXIT_MESSAGE((static_cast<int>(type) < static_cast<int> (DumpType::END)), "range sanity check");
+
 #if defined( _DEBUG ) || defined( _INTERNAL )
-            assert( 0 <= static_cast<int>(type) &&
-                        static_cast<int>(type) < static_cast<int>( DumpType::END ) &&
-                        "range sanity check" );
             switch (loc)
             {
             case DumpLoc::ODS  : g_dumpFlags[ static_cast<int>(type) ].dumpODS  = enabled; break;
             case DumpLoc::FILE : g_dumpFlags[ static_cast<int>(type) ].dumpFile = enabled; break;
-            default            : assert( 0 && "unreachable" ); break;
+            default            : IGC_ASSERT_EXIT_MESSAGE(0, "unreachable"); break;
             }
 #else
             (void) type;
@@ -319,10 +318,11 @@ namespace IGC
 
         bool IGC_DEBUG_API_CALL GetDumpFlag( DumpType type, DumpLoc loc )
         {
+            IGC_ASSERT_EXIT_MESSAGE((0 <= static_cast<int>(type)), "range sanity check");
+            IGC_ASSERT_EXIT_MESSAGE((static_cast<int>(type) < static_cast<int> (DumpType::END)), "range sanity check");
+
 #if defined( _DEBUG ) || defined( _INTERNAL )
-            assert( 0 <= static_cast<int>(type) &&
-                        static_cast<int>(type) < static_cast<int>( DumpType::END ) &&
-                        "range sanity check" );
+
 #if defined(_WIN32 )|| defined( _WIN64 )
             //Disable Dump  for OS Applications
             if (GetModuleHandleA("dwm.exe") || GetModuleHandleA("explorer.exe"))
@@ -334,7 +334,7 @@ namespace IGC
             {
             case DumpLoc::ODS  : return g_dumpFlags[ static_cast<int>(type) ].dumpODS;
             case DumpLoc::FILE : return g_dumpFlags[ static_cast<int>(type) ].dumpFile;
-            default            : assert( 0 && "unreachable" ); return false;
+            default            : IGC_ASSERT_EXIT_MESSAGE(0, "unreachable"); return false;
             }
 #else
             (void) type;
