@@ -38,10 +38,6 @@ Abstract:  Defines the types used for ELF headers/sections.  This uses
     #include <inttypes.h>
 #endif
 
-#if defined(__linux__)
-    #include "elf.h"
-#endif
-
 #include <stddef.h>
 
 namespace CLElfLib
