@@ -63,7 +63,7 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 #include <algorithm>
 #include <cstring>
-#include "Probe.h"
+#include "Probe/Assertion.h"
 
 using namespace IGC;
 using namespace IGC::IGCMD;
@@ -223,8 +223,9 @@ struct SStateProcessorContextGen8_0
 
     } Surface;
 };
-CGen8OpenCLStateProcessor::CGen8OpenCLStateProcessor(PLATFORM platform)
-    : m_Platform(platform)
+CGen8OpenCLStateProcessor::CGen8OpenCLStateProcessor(PLATFORM platform, const IProgramContext& ContextProvider)
+    : m_Platform(platform),
+      m_Context(ContextProvider)
 {
     G6HWC::InitializeCapsGen8( &m_HWCaps );
 }
@@ -318,7 +319,7 @@ void CGen8OpenCLStateProcessor::CreateKernelBinary(
     if (IGC_IS_FLAG_ENABLED(EnableCosDump))
     {
         auto name = DumpName(IGC::Debug::GetShaderOutputName())
-            .Hash(m_Context ? m_Context->hash : ShaderHash())
+            .Hash(m_Context.getProgramHash())
             .Type(ShaderType::OPENCL_SHADER)
             .PostFix("kernel_" + annotations.m_kernelName + std::to_string( annotations.m_executionEnivronment.CompiledSIMDSize))
             .Extension("cos");
@@ -349,15 +350,17 @@ void CGen8OpenCLStateProcessor::CreateProgramScopePatchStream(const IGC::SOpenCL
         patch.Token = iOpenCL::PATCH_TOKEN_ALLOCATE_CONSTANT_MEMORY_SURFACE_PROGRAM_BINARY_INFO;
         patch.Size = sizeof( patch );
         patch.ConstantBufferIndex = DEFAULT_CONSTANT_BUFFER_INDEX;
-        patch.InlineDataSize = (DWORD)iter->InlineData.size();
+        patch.InlineDataSize = (DWORD)iter->AllocSize;
 
         retValue = AddPatchItem(
             patch,
             membuf );
 
         // And now write the actual data
-        membuf.Write((char*)iter->InlineData.data(),
-                     iter->InlineData.size());
+        membuf.Write((char*)iter->InlineData.data(), iter->InlineData.size());
+        // Pad the end with zeros
+        unsigned zeroPadding = iter->AllocSize - iter->InlineData.size();
+        membuf.AddPadding(zeroPadding);
     }
 
     for (auto &iter : annotations.m_initGlobalAnnotation)
@@ -369,15 +372,17 @@ void CGen8OpenCLStateProcessor::CreateProgramScopePatchStream(const IGC::SOpenCL
         patch.Size = sizeof( patch );
         patch.Type = iOpenCL::GLOBAL_BUFFER_TYPE_INLINE;
         patch.GlobalBufferIndex = 0;
-        patch.InlineDataSize = (DWORD)iter->InlineData.size();
+        patch.InlineDataSize = (DWORD)iter->AllocSize;
 
         retValue = AddPatchItem(
             patch,
             membuf );
 
         // And now write the actual data
-        membuf.Write((char*)iter->InlineData.data(),
-                     iter->InlineData.size());
+        membuf.Write((char*)iter->InlineData.data(), iter->InlineData.size());
+        // Pad the end with zeros
+        unsigned zeroPadding = iter->AllocSize - iter->InlineData.size();
+        membuf.AddPadding(zeroPadding);
     }
 
 
@@ -548,19 +553,9 @@ RETVAL CGen8OpenCLStateProcessor::CreateKernelHeap(
     RETVAL retValue = g_cInitRetValue;
 
     // Add the system kernel if required.
-    if (m_Context)
+    if (retValue.Success && m_Context.needsSystemKernel())
     {
-        const auto options = m_Context->m_InternalOptions;
-        if (retValue.Success &&
-            (options.IncludeSIPCSR ||
-             options.IncludeSIPKernelDebug ||
-             options.IncludeSIPKernelDebugWithLocalMemory))
-        {
-            retValue = AddSystemKernel(
-                pSystemThreadKernelOutput,
-                context,
-                membuf);
-        }
+        retValue = AddSystemKernel(pSystemThreadKernelOutput, context, membuf);
     }
 
     if (retValue.Success)
@@ -603,7 +598,7 @@ RETVAL CGen8OpenCLStateProcessor::CreateSurfaceStateHeap(
     std::map<unsigned, SurfaceState> SurfaceStates;
 
     // First, System Thread Surface
-    if (retValue.Success && m_Context && m_Context->m_InternalOptions.KernelDebugEnable)
+    if (retValue.Success && m_Context.isProgramDebuggable())
     {
         unsigned int bti = layout.GetSystemThreadBindingTableIndex();
 
@@ -801,7 +796,7 @@ RETVAL CGen8OpenCLStateProcessor::CreateSurfaceStateHeap(
                 }
             }
 
-            IGC_ASSERT(found && "GTPIN_IGC_OCL Error: Fail to find a free BTI for GT-Pin surface (i)");
+            IGC_ASSERT_MESSAGE(found, "GTPIN_IGC_OCL Error: Fail to find a free BTI for GT-Pin surface (i)");
         }
         const int res = GTPIN_IGC_OCL_UpdateKernelInfo(0, btiAssigned[0], btiAssigned[1]);
 
@@ -890,7 +885,7 @@ RETVAL CGen8OpenCLStateProcessor::CreateDynamicStateHeap(
     if( numSamplers && retValue.Success )
     {
         // Handle border color state:
-        if (m_Context && m_Context->m_DriverInfo.ProgrammableBorderColorInCompute())
+        if (m_Context.hasProgrammableBorderColor())
         {
             // Indirect states for argument samplers:
             for (DWORD i = 0; i < numArgumentSamplers && retValue.Success; i++)
@@ -934,7 +929,7 @@ RETVAL CGen8OpenCLStateProcessor::CreateDynamicStateHeap(
         context.Dynamic.SamplerBorderColorStateOffset = borderColorOffsets[0];
 
         DWORD borderColorIndex = 0;
-        DWORD borderColorStep = (m_Context && m_Context->m_DriverInfo.ProgrammableBorderColorInCompute()) ? 1 : 0;
+        DWORD borderColorStep = m_Context.hasProgrammableBorderColor() ? 1 : 0;
 
         // First handle the sampler arguments
         for (auto i = annotations.m_samplerArgument.begin();
@@ -956,7 +951,7 @@ RETVAL CGen8OpenCLStateProcessor::CreateDynamicStateHeap(
                 bool    normalizedCoords = true;
                 bool    enable = true;
 
-                IGC_ASSERT((m_Context && m_Context->m_DriverInfo.ProgrammableBorderColorInCompute()) || borderColorIndex == 0);
+                IGC_ASSERT(m_Context.hasProgrammableBorderColor() || borderColorIndex == 0);
 
                 retValue = AddSamplerState(
                     enable,
@@ -973,7 +968,7 @@ RETVAL CGen8OpenCLStateProcessor::CreateDynamicStateHeap(
 
                 if( !retValue.Success )
                 {
-                    IGC_ASSERT(0 && "Error during adding sampler state for sampler argument (index: samplerAnnotation->SamplerTableIndex, type: samplerAnnotation->SamplerType)");
+                    IGC_ASSERT_MESSAGE(0, "Error during adding sampler state for sampler argument (index: samplerAnnotation->SamplerTableIndex, type: samplerAnnotation->SamplerType)");
                     break;
                 }
             }
@@ -996,7 +991,7 @@ RETVAL CGen8OpenCLStateProcessor::CreateDynamicStateHeap(
 
         if( retValue.Success )
         {
-            IGC_ASSERT(!(m_Context && m_Context->m_DriverInfo.ProgrammableBorderColorInCompute()) || borderColorIndex == numArgumentSamplers);
+            IGC_ASSERT(!m_Context.hasProgrammableBorderColor() || borderColorIndex == numArgumentSamplers);
 
             // And then the inline samplers
             for (auto i = annotations.m_samplerInput.begin();
@@ -1011,7 +1006,7 @@ RETVAL CGen8OpenCLStateProcessor::CreateDynamicStateHeap(
 
                 bool enable = true;
 
-                IGC_ASSERT((m_Context && m_Context->m_DriverInfo.ProgrammableBorderColorInCompute()) || borderColorIndex == 0);
+                IGC_ASSERT(m_Context.hasProgrammableBorderColor() || borderColorIndex == 0);
 
                 retValue = AddSamplerState(
                     enable,
@@ -1028,7 +1023,7 @@ RETVAL CGen8OpenCLStateProcessor::CreateDynamicStateHeap(
 
                 if( !retValue.Success )
                 {
-                    IGC_ASSERT(0 && "Error during adding sampler state for inline sampler (index: samplerAnnotation->SamplerTableIndex, type: samplerAnnotation->SamplerType)");
+                    IGC_ASSERT_MESSAGE(0, "Error during adding sampler state for inline sampler (index: samplerAnnotation->SamplerTableIndex, type: samplerAnnotation->SamplerType)");
                     break;
                 }
             }
@@ -1433,7 +1428,7 @@ RETVAL CGen8OpenCLStateProcessor::CreatePatchList(
     // Patch for Surface State Scratch Space
 
     // Patch for ALLOCATE_SIP_SURFACE
-    if (retValue.Success && m_Context && m_Context->m_InternalOptions.KernelDebugEnable)
+    if (retValue.Success && m_Context.isProgramDebuggable())
     {
         unsigned int bti = layout.GetSystemThreadBindingTableIndex();
 
@@ -1749,6 +1744,8 @@ RETVAL CGen8OpenCLStateProcessor::CreatePatchList(
                     patch.Token = iOpenCL::PATCH_TOKEN_ALLOCATE_STATELESS_PRIVATE_MEMORY;
                     patch.Size = sizeof( patch );
                     patch.SurfaceStateHeapOffset = context.Surface.SurfaceOffset[ bti ];
+                    //FIXME: IGC currently set PerThreadPrivateMemorySize with size assumed to be per-simt-thread by setting IsSimtThread==1
+                    patch.IsSimtThread = 1;
                     patch.PerThreadPrivateMemorySize =
                             std::max((DWORD)IGC_GET_FLAG_VALUE(ForcePerThreadPrivateMemorySize),
                                      ptrArg->PerThreadPrivateMemorySize);
@@ -2031,6 +2028,8 @@ RETVAL CGen8OpenCLStateProcessor::CreatePatchList(
         patch.HasGlobalAtomics = annotations.m_executionEnivronment.HasGlobalAtomics;
 
 
+        patch.UseBindlessMode = annotations.m_executionEnivronment.UseBindlessMode;
+        patch.SIMDInfo = annotations.m_executionEnivronment.SIMDInfo;
 
         retValue = AddPatchItem(
             patch,
@@ -2143,6 +2142,12 @@ RETVAL CGen8OpenCLStateProcessor::CreatePatchList(
             buffer = program->simd32.m_gtpinBuffer;
             size = program->simd32.m_gtpinBufferSize;
         }
+        // CM kernels are dispatched with CompiledSIMDSize == 1
+        else if (annotations.m_executionEnivronment.CompiledSIMDSize == 1)
+        {
+            buffer = program->simd1.m_gtpinBuffer;
+            size = program->simd1.m_gtpinBufferSize;
+        }
 
         if (size > 0)
         {
@@ -2188,6 +2193,12 @@ RETVAL CGen8OpenCLStateProcessor::CreatePatchList(
             size = program->simd32.m_funcSymbolTableSize;
             entries = program->simd32.m_funcSymbolTableEntries;
         }
+        else if (annotations.m_executionEnivronment.CompiledSIMDSize == 1)
+        {
+            buffer = program->simd1.m_funcSymbolTable;
+            size = program->simd1.m_funcSymbolTableSize;
+            entries = program->simd1.m_funcSymbolTableEntries;
+        }
 
         if (size > 0)
         {
@@ -2242,6 +2253,14 @@ RETVAL CGen8OpenCLStateProcessor::CreatePatchList(
             size = program->simd32.m_funcRelocationTableSize;
             entries = program->simd32.m_funcRelocationTableEntries;
         }
+        // CM kernels are dispatched with CompiledSIMDSize == 1, this is just a contract
+        // between igcmc and patch token generator, shouldn't break existing scenarios for IGC
+        else if (annotations.m_executionEnivronment.CompiledSIMDSize == 1)
+        {
+            buffer = program->simd1.m_funcRelocationTable;
+            size = program->simd1.m_funcRelocationTableSize;
+            entries = program->simd1.m_funcRelocationTableEntries;
+        }
 
         if (size > 0)
         {
