@@ -39,9 +39,10 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 #include "../Patch/patch_parser.h"
 #include "inc/common/Compiler/API/SurfaceFormats.h"
+#include "common/shaderHash.hpp"
 #include "usc.h"
 #include "sp_debug.h"
-#include "Probe.h"
+#include "Probe/Assertion.h"
 
 namespace IGC
 {
@@ -79,11 +80,23 @@ private:
     DisallowCopy& operator=(const DisallowCopy&);
 };
 
+/// CGen8OpenCLStateProcessor - Provides services to create (legacy) patch token
+/// based binary from given SProgramOutput information
 class CGen8OpenCLStateProcessor : DisallowCopy
 {
 
 public:
-    explicit CGen8OpenCLStateProcessor(PLATFORM platform);
+    class IProgramContext {
+    public:
+      virtual ShaderHash getProgramHash() const = 0;
+      virtual bool needsSystemKernel() const  = 0;
+      virtual bool isProgramDebuggable() const = 0;
+      virtual bool hasProgrammableBorderColor() const = 0;
+
+      virtual ~IProgramContext () {}
+    };
+
+    explicit CGen8OpenCLStateProcessor(PLATFORM platform, const IProgramContext& Ctx);
     virtual ~CGen8OpenCLStateProcessor( void );
 
     virtual void CreateKernelBinary(
@@ -107,9 +120,7 @@ public:
         const std::string& kernelName,
         Util::BinaryStream& kernelDebugData);
 
-    // Optional OpeneCL program context.
-    const IGC::OpenCLProgramContext *m_Context = nullptr;
-
+    const IProgramContext& m_Context;
     std::string m_oclStateDebugMessagePrintOut;
 
 private:
@@ -253,7 +264,7 @@ inline DWORD CGen8OpenCLStateProcessor::GetSamplerStateSizeMultiplier(
     case SAMPLER_OBJECT_SAMPLE_8X8_BOOL_CENTROID:   multiplier =   0; break;
     case SAMPLER_OBJECT_SAMPLE_8X8_BOOL_SUM:        multiplier =   0; break;
     default:
-        IGC_ASSERT(0 && "Unknown sampler type");
+        IGC_ASSERT_MESSAGE(0, "Unknown sampler type");
         multiplier = 0;
         break;
     }
