@@ -101,6 +101,9 @@ namespace spv {
             FlagIsLValueReference = 1 << 11,
             FlagIsRValueReference = 1 << 12,
             FlagIsOptimized = 1 << 13,
+            FlagIsEnumClass = 1 << 14,
+            FlagTypePassByValue = 1 << 15,
+            FlagTypePassByReference = 1 << 16,
         };
 
         enum EncodingTag {
@@ -137,7 +140,330 @@ namespace spv {
             Xderef = 6,
             StackValue = 7,
             Constu = 8,
-            Fragment = 9
+            Fragment = 9,
+            Convert = 10,
+            Addr = 11,
+            Const1u = 12,
+            Const1s = 13,
+            Const2u = 14,
+            Const2s = 15,
+            Const4u = 16,
+            Const4s = 17,
+            Const8u = 18,
+            Const8s = 19,
+            Consts = 20,
+            Dup = 21,
+            Drop = 22,
+            Over = 23,
+            Pick = 24,
+            Rot = 25,
+            Abs = 26,
+            And = 27,
+            Div = 28,
+            Mod = 29,
+            Mul = 30,
+            Neg = 31,
+            Not = 32,
+            Or = 33,
+            Shl = 34,
+            Shr = 35,
+            Shra = 36,
+            Xor = 37,
+            Bra = 38,
+            Eq = 39,
+            Ge = 40,
+            Gt = 41,
+            Le = 42,
+            Lt = 43,
+            Ne = 44,
+            Skip = 45,
+            Lit0 = 46,
+            Lit1 = 47,
+            Lit2 = 48,
+            Lit3 = 49,
+            Lit4 = 50,
+            Lit5 = 51,
+            Lit6 = 52,
+            Lit7 = 53,
+            Lit8 = 54,
+            Lit9 = 55,
+            Lit10 = 56,
+            Lit11 = 57,
+            Lit12 = 58,
+            Lit13 = 59,
+            Lit14 = 60,
+            Lit15 = 61,
+            Lit16 = 62,
+            Lit17 = 63,
+            Lit18 = 64,
+            Lit19 = 65,
+            Lit20 = 66,
+            Lit21 = 67,
+            Lit22 = 68,
+            Lit23 = 69,
+            Lit24 = 70,
+            Lit25 = 71,
+            Lit26 = 72,
+            Lit27 = 73,
+            Lit28 = 74,
+            Lit29 = 75,
+            Lit30 = 76,
+            Lit31 = 77,
+            Reg0 = 78,
+            Reg1 = 79,
+            Reg2 = 80,
+            Reg3 = 81,
+            Reg4 = 82,
+            Reg5 = 83,
+            Reg6 = 84,
+            Reg7 = 85,
+            Reg8 = 86,
+            Reg9 = 87,
+            Reg10 = 88,
+            Reg11 = 89,
+            Reg12 = 90,
+            Reg13 = 91,
+            Reg14 = 92,
+            Reg15 = 93,
+            Reg16 = 94,
+            Reg17 = 95,
+            Reg18 = 96,
+            Reg19 = 97,
+            Reg20 = 98,
+            Reg21 = 99,
+            Reg22 = 100,
+            Reg23 = 101,
+            Reg24 = 102,
+            Reg25 = 103,
+            Reg26 = 104,
+            Reg27 = 105,
+            Reg28 = 106,
+            Reg29 = 107,
+            Reg30 = 108,
+            Reg31 = 109,
+            Breg0 = 110,
+            Breg1 = 111,
+            Breg2 = 112,
+            Breg3 = 113,
+            Breg4 = 114,
+            Breg5 = 115,
+            Breg6 = 116,
+            Breg7 = 117,
+            Breg8 = 118,
+            Breg9 = 119,
+            Breg10 = 120,
+            Breg11 = 121,
+            Breg12 = 122,
+            Breg13 = 123,
+            Breg14 = 124,
+            Breg15 = 125,
+            Breg16 = 126,
+            Breg17 = 127,
+            Breg18 = 128,
+            Breg19 = 129,
+            Breg20 = 130,
+            Breg21 = 131,
+            Breg22 = 132,
+            Breg23 = 133,
+            Breg24 = 134,
+            Breg25 = 135,
+            Breg26 = 136,
+            Breg27 = 137,
+            Breg28 = 138,
+            Breg29 = 139,
+            Breg30 = 140,
+            Breg31 = 141,
+            Regx = 142,
+            Fbreg = 143,
+            Bregx = 144,
+            Piece = 145,
+            DerefSize = 146,
+            XderefSize = 147,
+            Nop = 148,
+            PushObjectAddress = 149,
+            Call2 = 150,
+            Call4 = 151,
+            CallRef = 152,
+            FormTlsAddress = 153,
+            CallFrameCfa = 154,
+            ImplicitValue = 155,
+            ImplicitPointer = 156,
+            Addrx = 157,
+            Constx = 158,
+            EntryValue = 159,
+            ConstTypeOp = 160,
+            RegvalType = 161,
+            DerefType = 162,
+            XderefType = 163,
+            Reinterpret = 164
+        };
+
+        static std::unordered_map<ExpressionOpCode, unsigned> OpCountMap{
+        { Deref,              1 },
+        { Plus,               1 },
+        { Minus,              1 },
+        { PlusUconst,         2 },
+        { BitPiece,           3 },
+        { Swap,               1 },
+        { Xderef,             1 },
+        { StackValue,         1 },
+        { Constu,             2 },
+        { Fragment,           3 },
+        { Convert,            3 },
+        { Addr,               2 },
+        { Const1u,            2 },
+        { Const1s,            2 },
+        { Const2u,            2 },
+        { Const2s,            2 },
+        { Const4u,            2 },
+        { Const4s,            2 },
+        { Const8u,            2 },
+        { Const8s,            2 },
+        { Consts,             2 },
+        { Dup,                1 },
+        { Drop,               1 },
+        { Over,               1 },
+        { Pick,               1 },
+        { Rot,                1 },
+        { Abs,                1 },
+        { And,                1 },
+        { Div,                1 },
+        { Mod,                1 },
+        { Mul,                1 },
+        { Neg,                1 },
+        { Not,                1 },
+        { Or,                 1 },
+        { Shl,                1 },
+        { Shr,                1 },
+        { Shra,               1 },
+        { Xor,                1 },
+        { Bra,                2 },
+        { Eq,                 1 },
+        { Ge,                 1 },
+        { Gt,                 1 },
+        { Le,                 1 },
+        { Lt,                 1 },
+        { Ne,                 1 },
+        { Skip,               2 },
+        { Lit0,               1 },
+        { Lit1,               1 },
+        { Lit2,               1 },
+        { Lit3,               1 },
+        { Lit4,               1 },
+        { Lit5,               1 },
+        { Lit6,               1 },
+        { Lit7,               1 },
+        { Lit8,               1 },
+        { Lit9,               1 },
+        { Lit10,              1 },
+        { Lit11,              1 },
+        { Lit12,              1 },
+        { Lit13,              1 },
+        { Lit14,              1 },
+        { Lit15,              1 },
+        { Lit16,              1 },
+        { Lit17,              1 },
+        { Lit18,              1 },
+        { Lit19,              1 },
+        { Lit20,              1 },
+        { Lit21,              1 },
+        { Lit22,              1 },
+        { Lit23,              1 },
+        { Lit24,              1 },
+        { Lit25,              1 },
+        { Lit26,              1 },
+        { Lit27,              1 },
+        { Lit28,              1 },
+        { Lit29,              1 },
+        { Lit30,              1 },
+        { Lit31,              1 },
+        { Reg0,               1 },
+        { Reg1,               1 },
+        { Reg2,               1 },
+        { Reg3,               1 },
+        { Reg4,               1 },
+        { Reg5,               1 },
+        { Reg6,               1 },
+        { Reg7,               1 },
+        { Reg8,               1 },
+        { Reg9,               1 },
+        { Reg10,              1 },
+        { Reg11,              1 },
+        { Reg12,              1 },
+        { Reg13,              1 },
+        { Reg14,              1 },
+        { Reg15,              1 },
+        { Reg16,              1 },
+        { Reg17,              1 },
+        { Reg18,              1 },
+        { Reg19,              1 },
+        { Reg20,              1 },
+        { Reg21,              1 },
+        { Reg22,              1 },
+        { Reg23,              1 },
+        { Reg24,              1 },
+        { Reg25,              1 },
+        { Reg26,              1 },
+        { Reg27,              1 },
+        { Reg28,              1 },
+        { Reg29,              1 },
+        { Reg30,              1 },
+        { Reg31,              1 },
+        { Breg0,              2 },
+        { Breg1,              2 },
+        { Breg2,              2 },
+        { Breg3,              2 },
+        { Breg4,              2 },
+        { Breg5,              2 },
+        { Breg6,              2 },
+        { Breg7,              2 },
+        { Breg8,              2 },
+        { Breg9,              2 },
+        { Breg10,             2 },
+        { Breg11,             2 },
+        { Breg12,             2 },
+        { Breg13,             2 },
+        { Breg14,             2 },
+        { Breg15,             2 },
+        { Breg16,             2 },
+        { Breg17,             2 },
+        { Breg18,             2 },
+        { Breg19,             2 },
+        { Breg20,             2 },
+        { Breg21,             2 },
+        { Breg22,             2 },
+        { Breg23,             2 },
+        { Breg24,             2 },
+        { Breg25,             2 },
+        { Breg26,             2 },
+        { Breg27,             2 },
+        { Breg28,             2 },
+        { Breg29,             2 },
+        { Breg30,             2 },
+        { Breg31,             2 },
+        { Regx,               2 },
+        { Fbreg,              1 },
+        { Bregx,              3 },
+        { Piece,              2 },
+        { DerefSize,          2 },
+        { XderefSize,         2 },
+        { Nop,                1 },
+        { PushObjectAddress,  1 },
+        { Call2,              2 },
+        { Call4,              2 },
+        { CallRef,            2 },
+        { FormTlsAddress,     1 },
+        { CallFrameCfa,       1 },
+        { ImplicitValue,      3 },
+        { ImplicitPointer,    3 },
+        { Addrx,              2 },
+        { Constx,             2 },
+        { EntryValue,         3 },
+        { ConstTypeOp,        4 },
+        { RegvalType,         3 },
+        { DerefType,          3 },
+        { XderefType,         3 },
+        { Reinterpret,        2 },
         };
 
         enum ImportedEntityTag {
@@ -868,5 +1194,22 @@ namespace spv {
         SPIRVWord getLine() { return arg<SPIRVWord>(SPIRVDebug::Operand::TemplateTemplateParameter::LineIdx); }
         SPIRVWord getColumn() { return arg<SPIRVWord>(SPIRVDebug::Operand::TemplateTemplateParameter::ColumnIdx); }
     };
+
+    class OpDebugExpression : OpDebugInfoBase
+    {
+    public:
+        OpDebugExpression(SPIRVExtInst* extInst) : OpDebugInfoBase(extInst) {}
+        SPIRVId getOperation(unsigned int idx) { return arg<SPIRVId>(idx); }
+        unsigned int getNumOperations() { return getNumArgs(); }
+    };
+
+    class OpDebugOperation : OpDebugInfoBase
+    {
+    public:
+        OpDebugOperation(SPIRVExtInst* extInst) : OpDebugInfoBase(extInst) {}
+        unsigned int getNumLiterals() { return getNumArgs() - SPIRVDebug::Operand::Operation::OpCodeIdx - 1; }
+        unsigned int getLiteral(unsigned int idx) { return arg<SPIRVWord>(idx + SPIRVDebug::Operand::Operation::OpCodeIdx); }
+        SPIRVWord getOperation() { return arg<SPIRVWord>(SPIRVDebug::Operand::Operation::OpCodeIdx); }
+    };
 }
 #endif
