@@ -67,7 +67,7 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #define SPIRVDECORATE_HPP_
 
 #include "SPIRVEntry.h"
-#include "Probe.h"
+#include "Probe/Assertion.h"
 
 namespace spv{
 class SPIRVDecorationGroup;
