@@ -379,8 +379,11 @@ _SPIRV_OP(SubgroupImageMediaBlockWriteINTEL, 5581)
 _SPIRV_OP(AsmTargetINTEL, 5609)
 _SPIRV_OP(AsmINTEL, 5610)
 _SPIRV_OP(AsmCallINTEL, 5611)
+// optimization_hints
+_SPIRV_OP(AssumeTrueINTEL, 5630)
+_SPIRV_OP(ExpectINTEL, 5631)
 // Function pointers
-_SPIRV_OP(FunctionPointerINTEL, 5600)
+_SPIRV_OP(ConstFunctionPointerINTEL, 5600)
 _SPIRV_OP(FunctionPointerCallINTEL, 5601)
 // device_side_avc_motion_estimation extension
 _SPIRV_OP(VmeImageINTEL, 5699)
@@ -501,3 +504,6 @@ _SPIRV_OP(SubgroupAvcSicGetIpeChromaModeINTEL, 5813)
 _SPIRV_OP(SubgroupAvcSicGetPackedSkcLumaCountThresholdINTEL, 5814)
 _SPIRV_OP(SubgroupAvcSicGetPackedSkcLumaSumThresholdINTEL, 5815)
 _SPIRV_OP(SubgroupAvcSicGetInterRawSadsINTEL, 5816)
+_SPIRV_OP(VariableLengthArrayINTEL, 5818)
+_SPIRV_OP(SaveMemoryINTEL, 5819)
+_SPIRV_OP(RestoreMemoryINTEL, 5820)
