@@ -48,6 +48,7 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 #include "AdaptorCommon/AddImplicitArgs.hpp"
 #include "AdaptorCommon/ProcessFuncAttributes.h"
+#include "AdaptorCommon/LegalizeFunctionSignatures.h"
 #include "common/LLVMUtils.h"
 
 #include "Compiler/CISACodeGen/ShaderCodeGen.hpp"
@@ -85,7 +86,6 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "Compiler/Optimizer/OpenCLPasses/OpenCLPrintf/OpenCLPrintfAnalysis.hpp"
 #include "Compiler/Optimizer/OpenCLPasses/OpenCLPrintf/OpenCLPrintfResolution.hpp"
 #include "Compiler/Optimizer/OpenCLPasses/AggregateArguments/AggregateArguments.hpp"
-#include "Compiler/Optimizer/OpenCLPasses/UnreachableHandling/UnreachableHandling.hpp"
 #include "Compiler/Optimizer/OCLBIConverter.h"
 #include "Compiler/Optimizer/OpenCLPasses/SetFastMathFlags/SetFastMathFlags.hpp"
 #include "Compiler/Optimizer/OpenCLPasses/CorrectlyRoundedDivSqrt/CorrectlyRoundedDivSqrt.hpp"
@@ -97,6 +97,7 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "Compiler/Optimizer/OpenCLPasses/SubGroupFuncs/SubGroupFuncsResolution.hpp"
 #include "Compiler/Optimizer/OpenCLPasses/BIFTransforms/BIFTransforms.hpp"
 #include "Compiler/Optimizer/OpenCLPasses/BreakdownIntrinsic.h"
+#include "Compiler/Optimizer/OpenCLPasses/TransformUnmaskedFunctionsPass.h"
 #include "Compiler/Optimizer/OpenCLPasses/StatelessToStatefull/StatelessToStatefull.hpp"
 #include "Compiler/Optimizer/OpenCLPasses/KernelFunctionCloning.h"
 #include "Compiler/Legalizer/TypeLegalizerPass.h"
@@ -123,9 +124,11 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 #include <iStdLib/utility.h>
 
-#include "Compiler/DebugInfo/VISADebugEmitter.hpp"
 #include "Compiler/CISACodeGen/DebugInfo.hpp"
 #include "Compiler/CISACodeGen/TimeStatsCounter.h"
+#include "Compiler/DebugInfo/ScalarVISAModule.h"
+#include "DebugInfo/VISADebugEmitter.hpp"
+#include "DebugInfo/DebugInfoUtils.hpp"
 
 #include <string>
 #include <algorithm>
@@ -246,11 +249,15 @@ static void CommonOCLBasedPasses(
     CompilerOpts.DisableA64WA =
         pContext->m_InternalOptions.IntelDisableA64WA;
 
+    CompilerOpts.ForceEnableA64WA =
+        pContext->m_InternalOptions.IntelForceEnableA64WA;
+
     CompilerOpts.HasBufferOffsetArg =
         pContext->m_InternalOptions.IntelHasBufferOffsetArg;
 
     CompilerOpts.PreferBindlessImages =
-        pContext->m_InternalOptions.PreferBindlessImages;
+        pContext->m_InternalOptions.PreferBindlessImages ||
+        pContext->m_InternalOptions.UseBindlessMode;
 
     if (CompilerOpts.PreferBindlessImages) {
         pContext->getModuleMetaData()->UseBindlessImage = true;
@@ -259,6 +266,9 @@ static void CommonOCLBasedPasses(
     CompilerOpts.EnableTakeGlobalAddress =
         pContext->m_Options.EnableTakeGlobalAddress;
 
+    CompilerOpts.EnableZEBinary =
+        pContext->m_InternalOptions.EnableZEBinary;
+
     // right now we don't support any standard function in the code gen
     // maybe we want to support some at some point to take advantage of LLVM optimizations
     TargetLibraryInfoImpl TLI;
@@ -279,12 +289,15 @@ static void CommonOCLBasedPasses(
     mpm.add(new MetaDataUtilsWrapper(pMdUtils, pContext->getModuleMetaData()));
     mpm.add(new CodeGenContextWrapper(pContext));
 
+    if (IGC_IS_FLAG_ENABLED(EnableUnmaskedFunctions))
+    {
+        mpm.add(new TransformUnmaskedFunctionsPass());
+    }
+
     mpm.add(new ClampLoopUnroll(256));
 
     mpm.add(new MoveStaticAllocas());
 
-    mpm.add(new UnreachableHandling());
-
     // Skip this pass if OCL version < 2.0
     if (!(OCLMajor < 2))
     {
@@ -331,6 +344,7 @@ static void CommonOCLBasedPasses(
 
     mpm.add(new CatchAllLineNumber());
 
+
     // OCL has built-ins so it always need to run inlining
     {
         mpm.add(createProcessFuncAttributesPass());
@@ -356,6 +370,16 @@ static void CommonOCLBasedPasses(
         mpm.add(new PurgeMetaDataUtils());
     }
 
+    // Fix illegal argument/return types in function calls not already inlined.
+    // Structs/arrays are not allowed to be passed by value.
+    // Return types are not allowed to be more than 64-bits.
+    // This pass changes all illegal function signatures to be passed by pointer instead.
+    // NOTE: SPIR-V adaptor already handles this for struct types
+    if (pContext->m_instrTypes.hasSubroutines)
+    {
+        mpm.add(new LegalizeFunctionSignatures());
+    }
+
     // OpenCL WI + image function resolution
 
     // OCLTODO : do another DCE that will get rid of unused WI func calls before this?
@@ -433,6 +457,10 @@ static void CommonOCLBasedPasses(
     mpm.add(createDeadCodeEliminationPass());
 
     mpm.add(createBuiltinsConverterPass());
+
+    // check for unsupported intrinsics
+    mpm.add(new ErrorCheck());
+
     mpm.add(new ImageFuncResolution());
     mpm.add(new Image3dToImage2darray());
 
@@ -468,6 +496,10 @@ static void CommonOCLBasedPasses(
     // "false" to createScalarizerPass() means that vector load/stores are NOT scalarized
     mpm.add(createScalarizerPass(false));
 
+    // Create a dummy kernel to attach the symbol table if necessary
+    // Only needed if function pointers, externally linked functions, or relocatable global variables are present
+    mpm.add(createInsertDummyKernelForSymbolTablePass());
+
     // Add SetFastMathFalgs pass at the end so that we don't need to worry about previous passes that
     // forget setting math flags. We expect the generic llvm optimizations after the unification
     // (optimizeIR()) will fully take advantage of the flags.
@@ -517,4 +549,5 @@ void UnifyIRSPIR(
 {
     CommonOCLBasedPasses(pContext, std::move(BuiltinGenericModule), std::move(BuiltinSizeModule));
 }
+
 }
