@@ -24,9 +24,11 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 ======================= end_copyright_notice ==================================*/
 
+#include "IGC/common/StringMacros.hpp"
 #include "common/LLVMWarningsPush.hpp"
-#include <llvm/Support/ScaledNumber.h>
-#include <llvm/Support/CommandLine.h>
+#include "llvm/Config/llvm-config.h"
+#include "llvm/Support/ScaledNumber.h"
+#include "llvm/Support/CommandLine.h"
 #include "common/LLVMWarningsPop.hpp"
 
 #include <cstring>
@@ -39,7 +41,6 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "AdaptorOCL/OCL/BuiltinResource.h"
 #include "AdaptorOCL/OCL/TB/igc_tb.h"
 
-#include "AdaptorOCL/Upgrader/Upgrader.h"
 #include "AdaptorOCL/UnifyIROCL.hpp"
 #include "AdaptorOCL/DriverInfoOCL.hpp"
 
@@ -54,8 +55,13 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "usc.h"
 
 #include "AdaptorOCL/OCL/sp/gtpin_igc_ocl.h"
-#include "AdaptorOCL/igcmc.h"
-#include "AdaptorOCL/cmc.h"
+
+#if !defined(WDDM_LINUX) && (!defined(IGC_VC_DISABLED) || !IGC_VC_DISABLED)
+#include "common/LLVMWarningsPush.hpp"
+#include "vc/igcdeps/TranslationInterface.h"
+#include "vc/Support/StatusCode.h"
+#include "common/LLVMWarningsPop.hpp"
+#endif // !defined(WDDM_LINUX) && (!defined(IGC_VC_DISABLED) || !IGC_VC_DISABLED)
 
 #include <iStdLib/MemCopy.h>
 
@@ -78,7 +84,7 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 #include <sstream>
 #include <iomanip>
-#include "Probe.h"
+#include "Probe/Assertion.h"
 
 //In case of use GT_SYSTEM_INFO in GlobalData.h from inc/umKmInc/sharedata.h
 //We have to do this temporary defines
@@ -233,7 +239,9 @@ bool CIGCTranslationBlock::Translate(
     const STB_TranslateInputArgs* pInputArgs,
     STB_TranslateOutputArgs* pOutputArgs )
 {
-  // Create a copy of input arguments that can be modified
+    LoadRegistryKeys();
+
+    // Create a copy of input arguments that can be modified
     STB_TranslateInputArgs InputArgsCopy = *pInputArgs;
 
     IGC::CPlatform IGCPlatform(m_Platform);
@@ -263,9 +271,6 @@ bool CIGCTranslationBlock::Translate(
     pOutputArgs->pDebugData = nullptr;
     pOutputArgs->DebugDataSize = 0;
 
-
-    LoadRegistryKeys();
-
     if (m_DataFormatInput == TB_DATA_FORMAT_ELF)
     {
         // Handle TB_DATA_FORMAT_ELF input as a result of a call to
@@ -293,7 +298,7 @@ bool CIGCTranslationBlock::Translate(
     }
     else
     {
-        IGC_ASSERT(0 && "Unsupported input format");
+        IGC_ASSERT_MESSAGE(0, "Unsupported input format");
         return false;
     }
     return false;
@@ -316,18 +321,77 @@ void GenerateCompilerOptionsMD(llvm::LLVMContext &C, llvm::Module &M, llvm::Stri
 {
     llvm::SmallVector<llvm::StringRef, 8> flags;
     llvm::StringRef sep(" ");
-    options.split(flags, sep);
+    options.split(flags, sep, -1, false);
 
     std::vector<llvm::Metadata*> ValueVec;
     for (auto flag : flags) {
         flag = flag.trim();
-        if (!flag.empty())
+        flag = flag.rtrim(0);  // make sure no ending 0
+        // flags : C string (ended with 0)
+        if (!flag.empty() && flag.front() != 0)
             ValueVec.push_back(llvm::MDString::get(C, flag));
     }
     llvm::NamedMDNode* NamedMD = M.getOrInsertNamedMetadata("opencl.compiler.options");
     NamedMD->addOperand(llvm::MDNode::get(C, ValueVec));
 }
 
+// Dump shader (binary or text), to output directory.
+// Create directory if it doesn't exist.
+// Works for all OSes.
+// ext - file name suffix (optional) and extension.
+void DumpShaderFile(
+    const std::string& dstDir,
+    const char* pBuffer,
+    const UINT bufferSize,
+    const QWORD hash,
+    const std::string& ext)
+{
+    if (pBuffer && bufferSize > 0)
+    {
+        std::ostringstream fullPath(dstDir, std::ostringstream::ate);
+        fullPath << "OCL_asm"
+            << std::hex
+            << std::setfill('0')
+            << std::setw(sizeof(hash) * CHAR_BIT / 4)
+            << hash
+            << std::dec
+            << std::setfill(' ')
+            << ext;
+
+        FILE* pFile = NULL;
+        fopen_s(&pFile, fullPath.str().c_str(), "wb");
+        if (pFile)
+        {
+            fwrite(pBuffer, 1, bufferSize, pFile);
+            fclose(pFile);
+        }
+    }
+}
+
+#if defined(IGC_SPIRV_TOOLS_ENABLED)
+spv_result_t DisassembleSPIRV(const char* pBuffer, UINT bufferSize, spv_text* outSpirvAsm)
+{
+    const spv_target_env target_env = SPV_ENV_UNIVERSAL_1_3;
+    spv_context context = spvContextCreate(target_env);
+    const uint32_t* const binary = reinterpret_cast<const uint32_t*> (pBuffer);
+    const size_t word_count = (bufferSize / sizeof(uint32_t));
+    const uint32_t options = (SPV_BINARY_TO_TEXT_OPTION_FRIENDLY_NAMES | SPV_BINARY_TO_TEXT_OPTION_INDENT | SPV_BINARY_TO_TEXT_OPTION_SHOW_BYTE_OFFSET);
+    spv_diagnostic diagnostic = nullptr;
+
+    const spv_result_t result = spvBinaryToText(
+        context,
+        binary,
+        word_count,
+        options,
+        outSpirvAsm,
+        &diagnostic);
+
+    spvContextDestroy(context);
+    spvDiagnosticDestroy(diagnostic);
+    return result;
+}
+#endif // defined(IGC_SPIRV_TOOLS_ENABLED)
+
 bool ProcessElfInput(
   STB_TranslateInputArgs &InputArgs,
   STB_TranslateOutputArgs &OutputArgs,
@@ -361,6 +425,7 @@ bool ProcessElfInput(
         {
             pElfReader->GetSectionData(i, pData, dataSize);
             InputArgs.pSpecConstantsIds = reinterpret_cast<const uint32_t*>(pData);
+            InputArgs.SpecConstantsSize = static_cast<uint32_t>(dataSize / sizeof(uint32_t));
         }
 
         if (pSectionHeader->Type == CLElfLib::SH_TYPE_SPIRV_SC_VALUES)
@@ -385,7 +450,7 @@ bool ProcessElfInput(
               llvm::Module* pKernelModule = nullptr;
 #if defined(IGC_SPIRV_ENABLED)
               Context.setAsSPIRV();
-              std::istringstream IS(buf);
+              std::istringstream IS(buf.str());
               std::string stringErrMsg;
               std::unordered_map<uint32_t, uint64_t> specIDToSpecValueMap = UnpackSpecConstants(
                                                                                   InputArgs.pSpecConstantsIds,
@@ -401,6 +466,11 @@ bool ProcessElfInput(
               std::string stringErrMsg{ "SPIRV consumption not enabled for the TARGET." };
               bool success = false;
 #endif
+              // unset specialization constants, to avoid using them by subsequent SPIR-V modules
+              InputArgs.pSpecConstantsIds = nullptr;
+              InputArgs.pSpecConstantsValues = nullptr;
+              InputArgs.SpecConstantsSize = 0;
+
               if (success)
               {
                   InputModule.reset(pKernelModule);
@@ -420,7 +490,7 @@ bool ProcessElfInput(
                       llvm::SMDiagnostic(pInputBuffer->getBufferIdentifier(), llvm::SourceMgr::DK_Error,
                           EIB.message());
                   });
-                  IGC_ASSERT(errMsg.empty() && "parsing bitcode failed");
+                  IGC_ASSERT_MESSAGE(errMsg.empty(), "parsing bitcode failed");
               }
 
               InputModule = std::move(errorOrModule.get());
@@ -472,10 +542,48 @@ bool ProcessElfInput(
             // destroying it
             OutputArgs.OutputSize = OutputString.size();
             OutputArgs.pOutput = pBufResult;
+
+#if defined(IGC_SPIRV_ENABLED)
+            if (IGC_IS_FLAG_ENABLED(ShaderDumpEnable))
+            {
+                // Dumping SPIRV files needs to be done after linking process, as we need to know
+                // the output parameters to prepare the correct file hash
+                for (unsigned i = 1; i < pHeader->NumSectionHeaderEntries; i++)
+                {
+                    const CLElfLib::SElf64SectionHeader* pSectionHeader = pElfReader->GetSectionHeader(i);
+                    IGC_ASSERT(pSectionHeader != NULL);
+                    if (pSectionHeader->Type == CLElfLib::SH_TYPE_SPIRV)
+                    {
+                        char* pSPIRVBitcode = NULL;
+                        size_t size = 0;
+                        pElfReader->GetSectionData(i, pSPIRVBitcode, size);
+                        QWORD hash = ShaderHashOCL((const UINT*)OutputArgs.pOutput, OutputArgs.OutputSize / 4).getAsmHash();
+
+                        // beyond of general hash, each SPIR-V module needs to have it's own hash
+                        QWORD spvHash = ShaderHashOCL((const UINT*)pSPIRVBitcode, size / 4).getAsmHash();
+                        std::ostringstream spvHashSuffix("_", std::ostringstream::ate);
+                        spvHashSuffix << std::hex << std::setfill('0') << std::setw(sizeof(spvHash)* CHAR_BIT / 4) << spvHash;
+                        const std::string suffix = spvHashSuffix.str();
+
+                        const char* pOutputFolder = IGC::Debug::GetShaderOutputFolder();
+                        DumpShaderFile(pOutputFolder, pSPIRVBitcode, size, hash, suffix + ".spv");
+
+#if defined(IGC_SPIRV_TOOLS_ENABLED)
+                        spv_text spirvAsm = nullptr;
+                        if (DisassembleSPIRV(pSPIRVBitcode, size, &spirvAsm) == SPV_SUCCESS)
+                        {
+                            DumpShaderFile(pOutputFolder, spirvAsm->str, spirvAsm->length, hash, suffix + ".spvasm");
+                        }
+                        spvTextDestroy(spirvAsm);
+#endif // defined(IGC_SPIRV_TOOLS_ENABLED)
+                    }
+                }
+            }
+#endif // defined(IGC_SPIRV_ENABLED)
           }
           else
           {
-            IGC_ASSERT(0 && "Unrecognized output format when processing ELF input");
+            IGC_ASSERT_MESSAGE(0, "Unrecognized output format when processing ELF input");
             success = false;
           }
         }
@@ -568,13 +676,11 @@ bool ParseInput(
         }
     }
 
-    // BEGIN HACK
-    // Upgrade BC to LLVM 3.5.1+ from LLVM 3.4+
     if (inputDataFormatTemp == TB_DATA_FORMAT_LLVM_BINARY) {
         std::unique_ptr<llvm::MemoryBuffer> Buf =
             llvm::MemoryBuffer::getMemBuffer(strInput, "<origin>", false);
         llvm::Expected<std::unique_ptr<llvm::Module>> MOE =
-            upgrader::upgradeAndParseBitcodeFile(Buf->getMemBufferRef(), oclContext);
+            llvm::parseBitcodeFile(Buf->getMemBufferRef(), oclContext);
         if (llvm::Error E = MOE.takeError())
         {
             llvm::handleAllErrors(std::move(E), [&](llvm::ErrorInfoBase &EIB) {
@@ -588,11 +694,10 @@ bool ParseInput(
             pKernelModule = MOE->release();
         }
     }
-    // END HACK
     else if (inputDataFormatTemp == TB_DATA_FORMAT_SPIR_V) {
 #if defined(IGC_SPIRV_ENABLED)
         //convert SPIR-V binary to LLVM module
-        std::istringstream IS(strInput);
+        std::istringstream IS(strInput.str());
         std::string stringErrMsg;
         std::unordered_map<uint32_t, uint64_t> specIDToSpecValueMap = UnpackSpecConstants(
                                                                             pInputArgs->pSpecConstantsIds,
@@ -625,7 +730,7 @@ bool ParseInput(
     if (pKernelModule == nullptr)
     {
         err.print(nullptr, llvm::errs(), false);
-        IGC_ASSERT(false && "Parsing module failed!");
+        IGC_ASSERT_MESSAGE(0, "Parsing module failed!");
     }
     if (pKernelModule == nullptr)
     {
@@ -649,7 +754,7 @@ bool ReadSpecConstantsFromSPIRV(std::istream &IS, std::vector<std::pair<uint32_t
     for (auto& SC : SPV)
     {
         SPIRVType *type = SC->getType();
-        uint32_t spec_size = type->getBitWidth() / 8;
+        uint32_t spec_size = type->isTypeBool() ? 1 : type->getBitWidth() / 8;
 
         if (SC->hasDecorate(DecorationSpecId))
         {
@@ -663,7 +768,7 @@ bool ReadSpecConstantsFromSPIRV(std::istream &IS, std::vector<std::pair<uint32_t
             }
             else
             {
-                IGC_ASSERT(0 && "Wrong instruction opcode, shouldn't be here!");
+                IGC_ASSERT_MESSAGE(0, "Wrong instruction opcode, shouldn't be here!");
                 return false;
             }
         }
@@ -694,7 +799,7 @@ void overrideOCLProgramBinary(OpenCLProgramContext &Ctx, char *&binaryOutput, in
     char *newBinaryOutput = new char[newBinarySize];
     f.read(newBinaryOutput, newBinarySize);
 
-    IGC_ASSERT(f && "Not fully read!");
+    IGC_ASSERT_MESSAGE(f.good(), "Not fully read!");
 
     delete[] binaryOutput;
     binaryOutput = newBinaryOutput;
@@ -717,45 +822,12 @@ void dumpOCLProgramBinary(OpenCLProgramContext &Ctx, char *binaryOutput, int bin
 #endif
 }
 
-// Dump shader (binary or text), to default directory.
-// Create directory if it doesn't exist.
-// Works for all OSes.
-// pExt - file name suffix (optional) and extenstion.
-void DumpShaderFile(const char *pOutputFolder, const char * pBuffer, UINT bufferSize, QWORD hash, const char * pExt )
-{
-    using namespace std;
-
-    stringstream ss;
-
-    if (pBuffer && bufferSize > 0)
-    {
-        ss << pOutputFolder;
-        ss << "OCL_"
-            << "asm"
-            << std::hex
-            << std::setfill('0')
-            << std::setw(sizeof(hash) * CHAR_BIT / 4)
-            << hash
-            << std::dec
-            << std::setfill(' ')
-            << pExt;
-
-        FILE* pFile = NULL;
-        fopen_s(&pFile, ss.str().c_str(), "wb");
-        if (pFile)
-        {
-            fwrite(pBuffer, 1, bufferSize, pFile);
-            fclose(pFile);
-        }
-    }
+static std::unique_ptr<llvm::MemoryBuffer> GetGenericModuleBuffer() {
+    char Resource[5] = {'-'};
+    _snprintf(Resource, sizeof(Resource), "#%d", OCL_BC);
+    return std::unique_ptr<llvm::MemoryBuffer>{llvm::LoadBufferFromResource(Resource, "BC")};
 }
 
-static bool TranslateBuildCM(const STB_TranslateInputArgs* pInputArgs,
-    STB_TranslateOutputArgs* pOutputArgs,
-    TB_DATA_FORMAT inputDataFormatTemp,
-    const IGC::CPlatform& IGCPlatform,
-    float profilingTimerResolution);
-
 bool TranslateBuild(
     const STB_TranslateInputArgs* pInputArgs,
     STB_TranslateOutputArgs* pOutputArgs,
@@ -763,15 +835,18 @@ bool TranslateBuild(
     const IGC::CPlatform& IGCPlatform,
     float profilingTimerResolution)
 {
+#if !defined(WDDM_LINUX) && (!defined(IGC_VC_DISABLED) || !IGC_VC_DISABLED)
     if (pInputArgs->pOptions) {
-        static const char* CMC = "-cmc";
-        if (strstr(pInputArgs->pOptions, CMC) != nullptr)
-            return TranslateBuildCM(pInputArgs,
-                                    pOutputArgs,
-                                    inputDataFormatTemp,
-                                    IGCPlatform,
-                                    profilingTimerResolution);
+        std::error_code Status =
+            vc::translateBuild(pInputArgs, pOutputArgs, inputDataFormatTemp,
+                               IGCPlatform, profilingTimerResolution);
+        if (!Status)
+            return true;
+        // If vc codegen option was not specified, then vc was not called.
+        if (static_cast<vc::errc>(Status.value()) != vc::errc::not_vc_codegen)
+            return false;
     }
+#endif // !defined(WDDM_LINUX) && (!defined(IGC_VC_DISABLED) || !IGC_VC_DISABLED)
 
     // Disable code sinking in instruction combining.
     // This is a workaround for a performance issue caused by code sinking
@@ -813,36 +888,14 @@ bool TranslateBuild(
         else if (inputDataFormatTemp == TB_DATA_FORMAT_SPIR_V)
         {
             DumpShaderFile(pOutputFolder, (char *)pInputArgs->pInput, pInputArgs->InputSize, hash, ".spv");
-
-            #ifdef IGC_SPIRV_TOOLS_ENABLED
-
-            const spv_target_env target_env = SPV_ENV_UNIVERSAL_1_3;
-            spv_context context = spvContextCreate(target_env);
-            const uint32_t* const binary = reinterpret_cast<const uint32_t*> (pInputArgs->pInput);
-            const size_t word_count = (pInputArgs->InputSize / sizeof(uint32_t));
-            const uint32_t options = (SPV_BINARY_TO_TEXT_OPTION_FRIENDLY_NAMES | SPV_BINARY_TO_TEXT_OPTION_INDENT | SPV_BINARY_TO_TEXT_OPTION_SHOW_BYTE_OFFSET);
-            spv_text text = nullptr;
-            spv_diagnostic diagnostic = nullptr;
-
-            const spv_result_t result = spvBinaryToText(
-                context,
-                binary,
-                word_count,
-                options,
-                &text,
-                &diagnostic);
-
-            spvContextDestroy(context);
-
-            if (SPV_SUCCESS == result)
+#if defined(IGC_SPIRV_TOOLS_ENABLED)
+            spv_text spirvAsm = nullptr;
+            if (DisassembleSPIRV(pInputArgs->pInput, pInputArgs->InputSize, &spirvAsm) == SPV_SUCCESS)
             {
-                DumpShaderFile(pOutputFolder, text->str, text->length, hash, ".spvasm");
+                DumpShaderFile(pOutputFolder, spirvAsm->str, spirvAsm->length, hash, ".spvasm");
             }
-
-            spvTextDestroy(text);
-            spvDiagnosticDestroy(diagnostic);
-
-            #endif
+            spvTextDestroy(spirvAsm);
+#endif // defined(IGC_SPIRV_TOOLS_ENABLED)
         }
 
         DumpShaderFile(pOutputFolder, (char *)pInputArgs->pInternalOptions, pInputArgs->InternalOptionsSize, hash, "_internal_options.txt");
@@ -940,10 +993,7 @@ bool TranslateBuild(
             {
                 COMPILER_TIME_START(&oclContext, TIME_OCL_LazyBiFLoading);
 
-                char Resource[5] = { '-' };
-                _snprintf(Resource, sizeof(Resource), "#%d", OCL_BC);
-
-                pGenericBuffer.reset(llvm::LoadBufferFromResource(Resource, "BC"));
+                pGenericBuffer = GetGenericModuleBuffer();
 
                 if (pGenericBuffer == NULL)
                 {
@@ -986,22 +1036,21 @@ bool TranslateBuild(
                     _snprintf(ResNumber, sizeof(ResNumber), "#%d", OCL_BC_64);
                     break;
                 default:
-                    IGC_ASSERT(0 && "Unknown bitness of compiled module");
+                    IGC_ASSERT_MESSAGE(0, "Unknown bitness of compiled module");
                 }
 
                 // the MemoryBuffer becomes owned by the module and does not need to be managed
                 pSizeTBuffer.reset(llvm::LoadBufferFromResource(ResNumber, "BC"));
-                IGC_ASSERT(pSizeTBuffer && "Error loading builtin resource");
+                IGC_ASSERT_MESSAGE(pSizeTBuffer, "Error loading builtin resource");
 
                 llvm::Expected<std::unique_ptr<llvm::Module>> ModuleOrErr =
                     getLazyBitcodeModule(pSizeTBuffer->getMemBufferRef(), *oclContext.getLLVMContext());
                 if (llvm::Error EC = ModuleOrErr.takeError())
-                    IGC_ASSERT(0 && "Error lazily loading bitcode for size_t builtins");
+                    IGC_ASSERT_MESSAGE(0, "Error lazily loading bitcode for size_t builtins");
                 else
                     BuiltinSizeModule = std::move(*ModuleOrErr);
 
-                IGC_ASSERT(BuiltinSizeModule
-                    && "Error loading builtin module from buffer");
+                IGC_ASSERT_MESSAGE(BuiltinSizeModule, "Error loading builtin module from buffer");
             }
 
             BuiltinGenericModule->setDataLayout(BuiltinSizeModule->getDataLayout());
@@ -1064,18 +1113,52 @@ bool TranslateBuild(
         }
     } while (retry);
 
-    // Create the binary streams for each compiled kernel
-    oclContext.m_programOutput.CreateKernelBinaries();
-
-    unsigned int pointerSizeInBytes = (PtrSzInBits == 64) ? 8 : 4;
+    if (!(oclContext.oclErrorMessage.empty()))
+    {
+        //The error buffer returned will be deleted when the module is unloaded so
+        //a copy is necessary
+        if (const char* pErrorMsg = oclContext.oclErrorMessage.c_str())
+        {
+            SetErrorMessage(oclContext.oclErrorMessage, *pOutputArgs);
+        }
+        return false;
+    }
 
     // Prepare and set program binary
-    Util::BinaryStream programBinary;
-    oclContext.m_programOutput.GetProgramBinary(programBinary, pointerSizeInBytes);
+    unsigned int pointerSizeInBytes = (PtrSzInBits == 64) ? 8 : 4;
 
-    int binarySize = static_cast<int>(programBinary.Size());
-    char* binaryOutput = new char[binarySize];
-    memcpy_s(binaryOutput, binarySize, (char*)programBinary.GetLinearPointer(), binarySize);
+    // FIXME: zebin currently only support program output itself, will add debug info
+    // into it
+    int binarySize = 0;
+    char* binaryOutput = nullptr;
+
+    if (!IGC_IS_FLAG_ENABLED(EnableZEBinary) &&
+        !oclContext.getModuleMetaData()->compOpt.EnableZEBinary) {
+        Util::BinaryStream programBinary;
+        // Patch token based binary format
+        oclContext.m_programOutput.CreateKernelBinaries();
+        oclContext.m_programOutput.GetProgramBinary(programBinary, pointerSizeInBytes);
+        binarySize = static_cast<int>(programBinary.Size());
+        binaryOutput = new char[binarySize];
+        memcpy_s(binaryOutput, binarySize, (char*)programBinary.GetLinearPointer(), binarySize);
+    } else {
+        // ze binary foramt
+        llvm::SmallVector<char, 64> buf;
+        llvm::raw_svector_ostream llvm_os(buf);
+        const char* spv_data = nullptr;
+        uint32_t spv_size = 0;
+        if (inputDataFormatTemp == TB_DATA_FORMAT_SPIR_V) {
+            spv_data = pInputArgs->pInput;
+            spv_size = pInputArgs->InputSize;
+        }
+        oclContext.m_programOutput.GetZEBinary(llvm_os, pointerSizeInBytes,
+            spv_data, spv_size);
+
+        // FIXME: try to avoid memory copy here
+        binarySize = buf.size();
+        binaryOutput = new char[binarySize];
+        memcpy_s(binaryOutput, binarySize, buf.data(), buf.size());
+    }
 
     if (IGC_IS_FLAG_ENABLED(ShaderDumpEnable))
         dumpOCLProgramBinary(oclContext, binaryOutput, binarySize);
@@ -1181,7 +1264,7 @@ bool CIGCTranslationBlock::Initialize(
         (m_DataFormatInput == TB_DATA_FORMAT_SPIR_V) &&
         isDeviceBinaryFormat(m_DataFormatOutput);
 
-    IGC_ASSERT(validTBChain && "Invalid TB Chain");
+    IGC_ASSERT_MESSAGE(validTBChain, "Invalid TB Chain");
 
     return validTBChain;
 }
@@ -1245,141 +1328,4 @@ TRANSLATION_BLOCK_API void Delete(
     CIGCTranslationBlock::Delete(pIGCTranslationBlock);
 }
 
-// Generate cmc compile options.
-static std::string getCommandLine(const STB_TranslateInputArgs* pInputArgs,
-    TB_DATA_FORMAT inputDataFormatTemp,
-    const IGC::CPlatform& IGCPlatform)
-{
-    std::string cmd;
-
-    // Set the input file type.
-    if (inputDataFormatTemp == TB_DATA_FORMAT::TB_DATA_FORMAT_SPIR_V)
-        cmd.append(" -filetype=spv");
-    else
-        llvm_unreachable("not implemented yet");
-
-    // Set the HW platform.
-    std::string platform = " -platform=";
-    platform += cmc::getPlatformStr(IGCPlatform.getPlatformInfo());
-    cmd.append(platform);
-
-    // Option to dump vISA
-    if (strstr(pInputArgs->pInternalOptions, "-dumpvisa") != nullptr)
-        cmd.append(" -dumpvisa");
-
-    if (strstr(pInputArgs->pOptions, "-no_vector_decomposition") != nullptr)
-        cmd.append(" -no_vector_decomposition");
-
-    const char StackMemSzOpt[] = "-stack-mem-size=";
-    if (const char *OptBegin = strstr(pInputArgs->pOptions, StackMemSzOpt)) {
-        const char *OptEnd = strstr(OptBegin, " ");
-        unsigned Count = 0;
-        if (OptEnd)
-            Count = static_cast<unsigned>(OptEnd - OptBegin);
-        else
-            Count = strlen(OptBegin);
-        cmd += " ";
-        cmd.append(OptBegin, Count);
-    }
-
-    return move(cmd);
-}
-
-// Generate vISA compile options from the input option string
-//
-// -visaopts='-dumpcommonisa,-noschedule'
-//
-static void getvISACompileOpts(const STB_TranslateInputArgs* pInputArgs,
-    std::vector<std::string>& optstrings,
-    std::vector<const char*>& opts)
-{
-    do {
-        llvm::StringRef Opts(pInputArgs->pOptions, pInputArgs->OptionsSize);
-        size_t pos = Opts.find_first_of("-visaopts");
-        if (pos == llvm::StringRef::npos)
-            break;
-
-        size_t beginPos = Opts.find_first_of("'", pos);
-        if (beginPos == llvm::StringRef::npos)
-            break;
-        ++beginPos;
-        size_t endPos = Opts.find_first_of("'", beginPos);
-        if (endPos == llvm::StringRef::npos)
-            break;
-
-        // vISA options are in a form '-dumpcommonisa,-noschedule'
-        llvm::StringRef vISAOpts = Opts.substr(beginPos, endPos - beginPos);
-        const char* delim = ", ";
-
-        // vISA crashes on illegal options.
-        size_t curPos = 0, nextPos = 0;
-        do {
-            nextPos = vISAOpts.find_first_of(delim, curPos);
-            if (nextPos == llvm::StringRef::npos) {
-                // last argument
-                llvm::StringRef O = vISAOpts.substr(curPos);
-                O = O.trim();
-                if (!O.empty())
-                    optstrings.push_back(O);
-                break;
-            } else {
-                llvm::StringRef O = vISAOpts.substr(curPos, nextPos - curPos);
-                O = O.trim();
-                if (!O.empty())
-                    optstrings.push_back(O);
-                curPos = nextPos + 1;
-            }
-        } while (curPos != llvm::StringRef::npos);
-    } while (false /* dummy loop to allow break inside */);
-
-    for (auto& s : optstrings) {
-        // Make sure this s.data() can be used as a c-string.
-        s.push_back('\0');
-        opts.push_back(s.data());
-    }
-}
-
-// When an internal otion "-cmc" is present, compile the input as a CM program.
-static bool TranslateBuildCM(const STB_TranslateInputArgs* pInputArgs,
-    STB_TranslateOutputArgs* pOutputArgs,
-    TB_DATA_FORMAT inputDataFormatTemp,
-    const IGC::CPlatform& IGCPlatform,
-    float profilingTimerResolution)
-{
-    cmc::CMCLibraryLoader Loader;
-    if (!Loader.isValid()) {
-        SetErrorMessage(Loader.ErrMsg, *pOutputArgs);
-        return false;
-    }
-
-    cmc_compile_info* output = nullptr;
-    const std::string cmd = getCommandLine(pInputArgs, inputDataFormatTemp, IGCPlatform);
-    int32_t status = Loader.compileFn(pInputArgs->pInput, pInputArgs->InputSize, cmd.c_str(), &output);
-    if (status == 0 && output) {
-        iOpenCL::CGen8CMProgram CMProgram(IGCPlatform.getPlatformInfo());
-        std::vector<std::string> optstrings;
-        std::vector<const char*> opts;
-        getvISACompileOpts(pInputArgs, optstrings, opts);
-        cmc::vISACompile(output, CMProgram, opts);
-
-        // Prepare and set program binary
-        Util::BinaryStream programBinary;
-        CMProgram.GetProgramBinary(programBinary, output->pointer_size_in_bytes);
-        size_t binarySize = static_cast<size_t>(programBinary.Size());
-        char* binaryOutput = new char[binarySize];
-        memcpy_s(binaryOutput, binarySize, (char*)programBinary.GetLinearPointer(), binarySize);
-        pOutputArgs->OutputSize = static_cast<uint32_t>(binarySize);
-        pOutputArgs->pOutput = binaryOutput;
-
-        // Free the resource allocated in the dll side.
-        Loader.freeFn(output);
-        return true;
-    }
-
-    // Set the error message.
-    const char* Err = "compilation error";
-    SetErrorMessage(Err, *pOutputArgs);
-    return false;
-}
-
 } // namespace TC
