@@ -209,12 +209,16 @@ struct SPatchExecutionEnvironment :
     uint32_t    NumGRFRequired;
     uint32_t    WorkgroupWalkOrderDims; // dim0 : [0 : 1]; dim1 : [2 : 3]; dim2 : [4 : 5]
     uint32_t    HasGlobalAtomics;
-    uint32_t    reserved;
+    uint32_t    reserved1;
+    uint32_t    reserved2;
+    uint32_t    reserved3;
     uint32_t    StatelessWritesCount;
+    uint32_t    UseBindlessMode;
+    uint64_t    SIMDInfo;
 };
 
 // Update CURRENT_ICBE_VERSION when modifying the patch list
-static_assert(sizeof(SPatchExecutionEnvironment) == (100 + sizeof(SPatchItemHeader)), "The size of SPatchExecutionEnvironment is not what is expected");
+static_assert(sizeof(SPatchExecutionEnvironment) == (120 + sizeof(SPatchItemHeader)), "The size of SPatchExecutionEnvironment is not what is expected");
 
 /*****************************************************************************\
 STRUCT: SPatchString
@@ -465,10 +469,13 @@ struct SPatchAllocateStatelessPrivateSurface :
     uint32_t   DataParamOffset;
     uint32_t   DataParamSize;
     uint32_t   PerThreadPrivateMemorySize;
+    // if IsSimtThread is true, API assumes it is allocated per-simt-thread
+    // else, API assumes it's allocated per-hardware-thread
+    uint32_t   IsSimtThread;
 };
 
 // Update CURRENT_ICBE_VERSION when modifying the patch list
-static_assert( sizeof( SPatchAllocateStatelessPrivateSurface ) == ( 16 + sizeof( SPatchItemHeader ) ) , "The size of SPatchAllocateStatelessPrivateSurface is not what is expected" );
+static_assert( sizeof( SPatchAllocateStatelessPrivateSurface ) == ( 20 + sizeof( SPatchItemHeader ) ) , "The size of SPatchAllocateStatelessPrivateSurface is not what is expected" );
 
 /*****************************************************************************\
 STRUCT: SPatchMediaVFEState
