@@ -7,7 +7,7 @@
 //
 #if defined (float_atomics_enable)
 
-#if (__OPENCL_C_VERSION__ >= CL_VERSION_2_0)
+#ifdef __opencl_c_generic_address_space
 float __attribute__((overloadable)) atomic_fetch_min(volatile generic atomic_float *object, float operand);
 float __attribute__((overloadable)) atomic_fetch_min_explicit(volatile generic atomic_float *object, float operand, memory_order order);
 float __attribute__((overloadable)) atomic_fetch_min_explicit(volatile generic atomic_float *object, float operand, memory_order order, memory_scope scope);
@@ -15,7 +15,7 @@ float __attribute__((overloadable)) atomic_fetch_min_explicit(volatile generic a
 float __attribute__((overloadable)) atomic_fetch_max(volatile generic atomic_float *object, float operand);
 float __attribute__((overloadable)) atomic_fetch_max_explicit(volatile generic atomic_float *object, float operand, memory_order order);
 float __attribute__((overloadable)) atomic_fetch_max_explicit(volatile generic atomic_float *object, float operand, memory_order order, memory_scope scope);
-#endif // CL_VERSION_2_0
+#endif // __opencl_c_generic_address_space
 
 // atom_min
 float __attribute__((overloadable)) atom_min(volatile __global float *p, float val);
@@ -1463,6 +1463,137 @@ float __attribute__((overloadable)) intel_atomic_xchg   (image2d_depth_t image,
 float __attribute__((overloadable)) intel_atomic_xchg   (image2d_array_depth_t image, int4 coord, float val);
 #endif
 
+#if defined(cl_intel_bit_instructions)
+// bfi i8
+uchar   __attribute__((overloadable)) intel_bfi(uchar   base, uchar   insert, uint   offset, uint   count);
+uchar2  __attribute__((overloadable)) intel_bfi(uchar2  base, uchar2  insert, uint2  offset, uint2  count);
+uchar3  __attribute__((overloadable)) intel_bfi(uchar3  base, uchar3  insert, uint3  offset, uint3  count);
+uchar4  __attribute__((overloadable)) intel_bfi(uchar4  base, uchar4  insert, uint4  offset, uint4  count);
+uchar8  __attribute__((overloadable)) intel_bfi(uchar8  base, uchar8  insert, uint8  offset, uint8  count);
+uchar16 __attribute__((overloadable)) intel_bfi(uchar16 base, uchar16 insert, uint16 offset, uint16 count);
+
+// bfi i16
+ushort   __attribute__((overloadable)) intel_bfi(ushort   base, ushort   insert, uint   offset, uint   count);
+ushort2  __attribute__((overloadable)) intel_bfi(ushort2  base, ushort2  insert, uint2  offset, uint2  count);
+ushort3  __attribute__((overloadable)) intel_bfi(ushort3  base, ushort3  insert, uint3  offset, uint3  count);
+ushort4  __attribute__((overloadable)) intel_bfi(ushort4  base, ushort4  insert, uint4  offset, uint4  count);
+ushort8  __attribute__((overloadable)) intel_bfi(ushort8  base, ushort8  insert, uint8  offset, uint8  count);
+ushort16 __attribute__((overloadable)) intel_bfi(ushort16 base, ushort16 insert, uint16 offset, uint16 count);
+
+// bfi i32
+uint   __attribute__((overloadable)) intel_bfi(uint   base, uint   insert, uint   offset, uint   count);
+uint2  __attribute__((overloadable)) intel_bfi(uint2  base, uint2  insert, uint2  offset, uint2  count);
+uint3  __attribute__((overloadable)) intel_bfi(uint3  base, uint3  insert, uint3  offset, uint3  count);
+uint4  __attribute__((overloadable)) intel_bfi(uint4  base, uint4  insert, uint4  offset, uint4  count);
+uint8  __attribute__((overloadable)) intel_bfi(uint8  base, uint8  insert, uint8  offset, uint8  count);
+uint16 __attribute__((overloadable)) intel_bfi(uint16 base, uint16 insert, uint16 offset, uint16 count);
+
+// bfi i64
+ulong   __attribute__((overloadable)) intel_bfi(ulong   base, ulong   insert, uint   offset, uint   count);
+ulong2  __attribute__((overloadable)) intel_bfi(ulong2  base, ulong2  insert, uint2  offset, uint2  count);
+ulong3  __attribute__((overloadable)) intel_bfi(ulong3  base, ulong3  insert, uint3  offset, uint3  count);
+ulong4  __attribute__((overloadable)) intel_bfi(ulong4  base, ulong4  insert, uint4  offset, uint4  count);
+ulong8  __attribute__((overloadable)) intel_bfi(ulong8  base, ulong8  insert, uint8  offset, uint8  count);
+ulong16 __attribute__((overloadable)) intel_bfi(ulong16 base, ulong16 insert, uint16 offset, uint16 count);
+
+// sbfe i8
+char   __attribute__((overloadable)) intel_sbfe(char   base, uint   offset, uint   count);
+char2  __attribute__((overloadable)) intel_sbfe(char2  base, uint2  offset, uint2  count);
+char3  __attribute__((overloadable)) intel_sbfe(char3  base, uint3  offset, uint3  count);
+char4  __attribute__((overloadable)) intel_sbfe(char4  base, uint4  offset, uint4  count);
+char8  __attribute__((overloadable)) intel_sbfe(char8  base, uint8  offset, uint8  count);
+char16 __attribute__((overloadable)) intel_sbfe(char16 base, uint16 offset, uint16 count);
+
+// sbfe i16
+short   __attribute__((overloadable)) intel_sbfe(short   base, uint   offset, uint   count);
+short2  __attribute__((overloadable)) intel_sbfe(short2  base, uint2  offset, uint2  count);
+short3  __attribute__((overloadable)) intel_sbfe(short3  base, uint3  offset, uint3  count);
+short4  __attribute__((overloadable)) intel_sbfe(short4  base, uint4  offset, uint4  count);
+short8  __attribute__((overloadable)) intel_sbfe(short8  base, uint8  offset, uint8  count);
+short16 __attribute__((overloadable)) intel_sbfe(short16 base, uint16 offset, uint16 count);
+
+// sbfe i32
+int   __attribute__((overloadable)) intel_sbfe(int   base, uint   offset, uint   count);
+int2  __attribute__((overloadable)) intel_sbfe(int2  base, uint2  offset, uint2  count);
+int3  __attribute__((overloadable)) intel_sbfe(int3  base, uint3  offset, uint3  count);
+int4  __attribute__((overloadable)) intel_sbfe(int4  base, uint4  offset, uint4  count);
+int8  __attribute__((overloadable)) intel_sbfe(int8  base, uint8  offset, uint8  count);
+int16 __attribute__((overloadable)) intel_sbfe(int16 base, uint16 offset, uint16 count);
+
+// sbfe i64
+long   __attribute__((overloadable)) intel_sbfe(long   base, uint   offset, uint   count);
+long2  __attribute__((overloadable)) intel_sbfe(long2  base, uint2  offset, uint2  count);
+long3  __attribute__((overloadable)) intel_sbfe(long3  base, uint3  offset, uint3  count);
+long4  __attribute__((overloadable)) intel_sbfe(long4  base, uint4  offset, uint4  count);
+long8  __attribute__((overloadable)) intel_sbfe(long8  base, uint8  offset, uint8  count);
+long16 __attribute__((overloadable)) intel_sbfe(long16 base, uint16 offset, uint16 count);
+
+
+// ubfe i8
+uchar   __attribute__((overloadable)) intel_ubfe(uchar   base, uint   offset, uint   count);
+uchar2  __attribute__((overloadable)) intel_ubfe(uchar2  base, uint2  offset, uint2  count);
+uchar3  __attribute__((overloadable)) intel_ubfe(uchar3  base, uint3  offset, uint3  count);
+uchar4  __attribute__((overloadable)) intel_ubfe(uchar4  base, uint4  offset, uint4  count);
+uchar8  __attribute__((overloadable)) intel_ubfe(uchar8  base, uint8  offset, uint8  count);
+uchar16 __attribute__((overloadable)) intel_ubfe(uchar16 base, uint16 offset, uint16 count);
+
+// ubfe i16
+ushort   __attribute__((overloadable)) intel_ubfe(ushort   base, uint   offset, uint   count);
+ushort2  __attribute__((overloadable)) intel_ubfe(ushort2  base, uint2  offset, uint2  count);
+ushort3  __attribute__((overloadable)) intel_ubfe(ushort3  base, uint3  offset, uint3  count);
+ushort4  __attribute__((overloadable)) intel_ubfe(ushort4  base, uint4  offset, uint4  count);
+ushort8  __attribute__((overloadable)) intel_ubfe(ushort8  base, uint8  offset, uint8  count);
+ushort16 __attribute__((overloadable)) intel_ubfe(ushort16 base, uint16 offset, uint16 count);
+
+// ubfe i32
+uint   __attribute__((overloadable)) intel_ubfe(uint   base, uint   offset, uint   count);
+uint2  __attribute__((overloadable)) intel_ubfe(uint2  base, uint2  offset, uint2  count);
+uint3  __attribute__((overloadable)) intel_ubfe(uint3  base, uint3  offset, uint3  count);
+uint4  __attribute__((overloadable)) intel_ubfe(uint4  base, uint4  offset, uint4  count);
+uint8  __attribute__((overloadable)) intel_ubfe(uint8  base, uint8  offset, uint8  count);
+uint16 __attribute__((overloadable)) intel_ubfe(uint16 base, uint16 offset, uint16 count);
+
+// ubfe i64
+ulong   __attribute__((overloadable)) intel_ubfe(ulong   base, uint   offset, uint   count);
+ulong2  __attribute__((overloadable)) intel_ubfe(ulong2  base, uint2  offset, uint2  count);
+ulong3  __attribute__((overloadable)) intel_ubfe(ulong3  base, uint3  offset, uint3  count);
+ulong4  __attribute__((overloadable)) intel_ubfe(ulong4  base, uint4  offset, uint4  count);
+ulong8  __attribute__((overloadable)) intel_ubfe(ulong8  base, uint8  offset, uint8  count);
+ulong16 __attribute__((overloadable)) intel_ubfe(ulong16 base, uint16 offset, uint16 count);
+
+// bfrev i8
+uchar   __attribute__((overloadable)) intel_bfrev(uchar   base);
+uchar2  __attribute__((overloadable)) intel_bfrev(uchar2  base);
+uchar3  __attribute__((overloadable)) intel_bfrev(uchar3  base);
+uchar4  __attribute__((overloadable)) intel_bfrev(uchar4  base);
+uchar8  __attribute__((overloadable)) intel_bfrev(uchar8  base);
+uchar16 __attribute__((overloadable)) intel_bfrev(uchar16 base);
+
+// bfrev i16
+ushort   __attribute__((overloadable)) intel_bfrev(ushort   base);
+ushort2  __attribute__((overloadable)) intel_bfrev(ushort2  base);
+ushort3  __attribute__((overloadable)) intel_bfrev(ushort3  base);
+ushort4  __attribute__((overloadable)) intel_bfrev(ushort4  base);
+ushort8  __attribute__((overloadable)) intel_bfrev(ushort8  base);
+ushort16 __attribute__((overloadable)) intel_bfrev(ushort16 base);
+
+// bfrev i32
+uint   __attribute__((overloadable)) intel_bfrev(uint   base);
+uint2  __attribute__((overloadable)) intel_bfrev(uint2  base);
+uint3  __attribute__((overloadable)) intel_bfrev(uint3  base);
+uint4  __attribute__((overloadable)) intel_bfrev(uint4  base);
+uint8  __attribute__((overloadable)) intel_bfrev(uint8  base);
+uint16 __attribute__((overloadable)) intel_bfrev(uint16 base);
+
+// bfrev i64
+ulong   __attribute__((overloadable)) intel_bfrev(ulong   base);
+ulong2  __attribute__((overloadable)) intel_bfrev(ulong2  base);
+ulong3  __attribute__((overloadable)) intel_bfrev(ulong3  base);
+ulong4  __attribute__((overloadable)) intel_bfrev(ulong4  base);
+ulong8  __attribute__((overloadable)) intel_bfrev(ulong8  base);
+ulong16 __attribute__((overloadable)) intel_bfrev(ulong16 base);
+#endif // defined(cl_intel_bit_instructions)
+
 #ifdef cl_intel_subgroups_ballot
 uint intel_sub_group_ballot(bool p);
 #endif
