@@ -26,15 +26,14 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "common/LLVMWarningsPush.hpp"
 #include <llvm/Support/ScaledNumber.h>
 #include "common/LLVMWarningsPop.hpp"
-
 #include "Compiler/CISACodeGen/ComputeShaderCodeGen.hpp"
 #include "Compiler/CISACodeGen/messageEncoding.hpp"
 #include "common/allocator.h"
 #include "common/secure_mem.h"
 #include <iStdLib/utility.h>
 #include <iStdLib/FloatUtil.h>
-
 #include <algorithm>
+#include "Probe/Assertion.h"
 
 using namespace llvm;
 
@@ -47,6 +46,8 @@ namespace IGC
         , m_hasSLM(false)
         , m_tileY(false)
         , m_walkOrder(WO_XYZ)
+        , m_threadGroupModifier_X(0)
+        , m_threadGroupModifier_Y(0)
     {
     }
 
@@ -189,9 +190,12 @@ namespace IGC
     {
         uint offset = 0;
 
+        ComputeShaderContext* pctx =
+            static_cast<ComputeShaderContext*>(GetContext());
+
         // R0 is used as a Predefined variable so that vISA doesn't free it later. In CS, we expect the
         // thread group id's in R0.
-        assert(GetR0());
+        IGC_ASSERT(GetR0());
 
         // We use predefined variables so offset has to be added for R0.
         offset += getGRFSize();
@@ -199,15 +203,24 @@ namespace IGC
         bool bZeroIDs = !GetNumberOfId();
         // for indirect threads data payload hardware doesn't allow empty per thread buffer
         // so we allocate a dummy thread id in case no IDs are used
-        if (bZeroIDs)
+        if (pctx->m_DriverInfo.UsesIndirectPayload() && bZeroIDs)
         {
             CreateThreadIDinGroup(THREAD_ID_IN_GROUP_X);
         }
 
+        if (!pctx->m_DriverInfo.UsesIndirectPayload())
+        {
+            // Cross-thread constant data.
+            AllocateNOSConstants(offset);
+        }
+
         AllocatePerThreadConstantData(offset);
 
         // Cross-thread constant data.
-        AllocateNOSConstants(offset);
+        if (pctx->m_DriverInfo.UsesIndirectPayload())
+        {
+            AllocateNOSConstants(offset);
+        }
     }
 
 
@@ -237,8 +250,7 @@ namespace IGC
                 }
                 else
                 {
-                    // should not compile again if already got a non spill kernel
-                    assert(false);
+                    IGC_ASSERT_MESSAGE(0, "should not compile again if already got a non spill kernel");
                 }
             }
         }
@@ -256,8 +268,7 @@ namespace IGC
                 }
                 else
                 {
-                    // should not compile again if already got a non spill kernel
-                    assert(false);
+                    IGC_ASSERT_MESSAGE(0, "should not compile again if already got a non spill kernel");
                 }
             }
         }
@@ -275,8 +286,7 @@ namespace IGC
                 }
                 else
                 {
-                    // should not compile again if already got a non spill kernel
-                    assert(false);
+                    IGC_ASSERT_MESSAGE(0, "should not compile again if already got a non spill kernel");
                 }
             }
         }
@@ -284,6 +294,9 @@ namespace IGC
 
     void CComputeShader::FillProgram(SComputeShaderKernelProgram* pKernelProgram)
     {
+        ComputeShaderContext* pctx =
+            static_cast<ComputeShaderContext*>(GetContext());
+
         CreateGatherMap();
         CreateConstantBufferOutput(pKernelProgram);
 
@@ -325,9 +338,12 @@ namespace IGC
 
         pKernelProgram->ThreadGroupSize = m_threadGroupSize;
 
+        pKernelProgram->ThreadGroupModifier_X = m_threadGroupModifier_X;
+        pKernelProgram->ThreadGroupModifier_Y = m_threadGroupModifier_Y;
+
         pKernelProgram->CSHThreadDispatchChannel = 0;
 
-        pKernelProgram->CompiledForIndirectPayload = 0;
+        pKernelProgram->CompiledForIndirectPayload = pctx->m_DriverInfo.UsesIndirectPayload();
 
         pKernelProgram->DispatchAlongY = m_dispatchAlongY;
 
@@ -355,13 +371,24 @@ namespace IGC
         m_threadGroupSize_Z = int_cast<uint>(llvm::cast<llvm::ConstantInt>(pGlobal->getInitializer())->getZExtValue());
 
         m_threadGroupSize = m_threadGroupSize_X * m_threadGroupSize_Y * m_threadGroupSize_Z;
+
+        pGlobal = module->getGlobalVariable("ThreadGroupModifier_X");
+        if ((pGlobal != nullptr) && pGlobal->hasInitializer()) {
+            m_threadGroupModifier_X = int_cast<uint>(llvm::cast<llvm::ConstantInt>(pGlobal->getInitializer())->getZExtValue());
+        }
+        pGlobal = module->getGlobalVariable("ThreadGroupModifier_Y");
+        if ((pGlobal != nullptr) && pGlobal->hasInitializer()) {
+            m_threadGroupModifier_Y = int_cast<uint>(llvm::cast<llvm::ConstantInt>(pGlobal->getInitializer())->getZExtValue());
+        }
     }
 
     void CComputeShader::PreCompile()
     {
         CreateImplicitArgs();
 
-        if (IGC_IS_FLAG_ENABLED(DispatchGPGPUWalkerAlongYFirst) && (m_num2DAccesses > m_num1DAccesses))
+        if (IGC_IS_FLAG_ENABLED(DispatchGPGPUWalkerAlongYFirst) &&
+            (m_num2DAccesses > m_num1DAccesses) &&
+            GetContext()->m_DriverInfo.SupportsDispatchGPGPUWalkerAlongYFirst())
         {
             m_dispatchAlongY = true;
         }
@@ -401,6 +428,12 @@ namespace IGC
         bool hasSimd16 = simd16Program && simd16Program->ProgramOutput()->m_programSize > 0;
         bool hasSimd32 = simd32Program && simd32Program->ProgramOutput()->m_programSize > 0;
 
+        if (!ctx->m_retryManager.IsFirstTry())
+        {
+            ctx->ClearSIMDInfo(simdMode, ShaderDispatchMode::NOT_APPLICABLE);
+            ctx->SetSIMDInfo(SIMD_RETRY, simdMode, ShaderDispatchMode::NOT_APPLICABLE);
+        }
+
         ////////
         // dynamic rules
         ////////
@@ -415,6 +448,7 @@ namespace IGC
         if (simdMode == SIMDMode::SIMD32 && simd16Program &&
             simd16Program->m_spillSize > 0)
         {
+            ctx->SetSIMDInfo(SIMD_SKIP_SPILL, simdMode, ShaderDispatchMode::NOT_APPLICABLE);
             return false;
         }
 
@@ -449,11 +483,20 @@ namespace IGC
                 {
                     return true;
                 }
+
+                ctx->SetSIMDInfo(SIMD_SKIP_THGRPSIZE, simdMode, ShaderDispatchMode::NOT_APPLICABLE);
             }
             else    // SIMD8
             {
                 if (simd16Program->m_spillCost <= ctx->GetSpillThreshold())
                 {
+                    ctx->SetSIMDInfo(SIMD_SKIP_PERF, simdMode, ShaderDispatchMode::NOT_APPLICABLE);
+                    return false;
+                }
+                else if (!ctx->m_retryManager.IsLastTry() && ctx->instrStat[LICM_STAT][EXCEED_THRESHOLD])
+                {
+                    // skip SIMD8 if LICM threshold is met, unless it's lastTry
+                    ctx->SetSIMDInfo(SIMD_SKIP_REGPRES, simdMode, ShaderDispatchMode::NOT_APPLICABLE);
                     return false;
                 }
                 else
