@@ -26,12 +26,11 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #pragma once
 
 #include "llvm/Config/llvm-config.h"
-
 #include "common/LLVMWarningsPush.hpp"
 #include "common/LLVMWarningsPop.hpp"
-
 #include "Compiler/IGCPassSupport.h"
-#include "Compiler/DebugInfo/VISAModule.hpp"
+#include "DebugInfo/VISAModule.hpp"
+#include "DebugInfo/VISAIDebugEmitter.hpp"
 #include "ShaderCodeGen.hpp"
 #include "common/allocator.h"
 #include "common/debug/Dump.hpp"
@@ -40,12 +39,12 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "Compiler/CISACodeGen/helper.h"
 #include "common/secure_mem.h"
 #include "iStdLib/File.h"
-#include "Compiler/DebugInfo/VISAIDebugEmitter.hpp"
 #include "GenISAIntrinsics/GenIntrinsicInst.h"
 #include "Compiler/IGCPassSupport.h"
 #include "Types.hpp"
 #include "ShaderCodeGen.hpp"
 #include "llvm/IR/DIBuilder.h"
+#include "Probe/Assertion.h"
 
 using namespace llvm;
 using namespace IGC;
@@ -66,9 +65,7 @@ namespace IGC
 
         void addVISAModule(llvm::Function* F, VISAModule* m)
         {
-            auto it = m_VISAModules.find(F);
-            if (it != m_VISAModules.end())
-                assert(false && "Reinserting VISA module for function");
+            IGC_ASSERT_MESSAGE(m_VISAModules.find(F) == m_VISAModules.end(), "Reinserting VISA module for function");
 
             m_VISAModules.insert(std::make_pair(F, m));
         }
@@ -124,4 +121,4 @@ namespace IGC
             AU.setPreservesAll();
         }
     };
-};
\ No newline at end of file
+};
