@@ -23,6 +23,7 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 
 ======================= end_copyright_notice ==================================*/
+
 #include "Compiler/CISACodeGen/helper.h"
 #include "Compiler/CISACodeGen/ShaderUnits.hpp"
 #include "Compiler/CISACodeGen/ShaderCodeGen.hpp"
@@ -30,6 +31,7 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "DomainShaderLowering.hpp"
 #include "Compiler/IGCPassSupport.h"
 #include "common/IGCIRBuilder.h"
+#include "Probe/Assertion.h"
 
 using namespace IGC;
 using namespace IGC::IGCMD;
@@ -247,7 +249,9 @@ namespace IGC
                             default:
                                 break;
                             }
-                            e_interpolation interpolant = DSDualPatchEnabled(pCtx) ? EINTERPOLATION_UNDEFINED : EINTERPOLATION_CONSTANT;
+                            const e_interpolation interpolant =
+                                DSDualPatchEnabled(pCtx) ?
+                            EINTERPOLATION_UNDEFINED : EINTERPOLATION_CONSTANT;
                             Value* arguments[] =
                             {
                                 builder.getInt32(index),
@@ -273,8 +277,11 @@ namespace IGC
                 }
             }
         }
+
+        const CPlatform& platform = getAnalysis<CodeGenContextWrapper>().getCodeGenContext()->platform;
+
         Value* undef = llvm::UndefValue::get(Type::getFloatTy(F.getContext()));
-        if (getAnalysis<CodeGenContextWrapper>().getCodeGenContext()->platform.WaForceDSToWriteURB())
+        if (platform.WaForceDSToWriteURB())
         {
             unsigned int numPhaseWritten = 0;
             for (unsigned int i = 0; i < m_maxNumOfOutput + m_headerSize.Count(); i += 2)
@@ -312,7 +319,8 @@ namespace IGC
             }
         }
         //URB padding to 32Byte offsets
-        for (unsigned int i = 0; i < m_maxNumOfOutput + m_headerSize.Count(); i++)
+        bool addURBPaddingTo32Bytes = true;
+        for (unsigned int i = 0; addURBPaddingTo32Bytes && i < m_maxNumOfOutput + m_headerSize.Count(); i++)
         {
             //If not aligned to 32Byte offset and has valid data
             if (offsetInst[i])
@@ -441,8 +449,9 @@ namespace IGC
         {
             Value* index = builder.getInt32(currentElementIndex + i);
             auto pCtx = getAnalysis<CodeGenContextWrapper>().getCodeGenContext();
-            e_interpolation interpolant =
-                DSDualPatchEnabled(pCtx) ? EINTERPOLATION_UNDEFINED : EINTERPOLATION_CONSTANT;
+            const e_interpolation interpolant =
+                DSDualPatchEnabled(pCtx) ?
+                EINTERPOLATION_UNDEFINED : EINTERPOLATION_CONSTANT;
             Value* interpolation = builder.getInt32(interpolant);
             channels[i] = builder.CreateCall2(shaderInputVecFn, index, interpolation);
         }
@@ -527,7 +536,8 @@ namespace IGC
 
         // Compute the offset for the given attribute and usage.
         QuadEltUnit offset = GetURBOffset(usage, attributeIndex);
-        assert(offset.Count() < (m_maxNumOfOutput + m_headerSize.Count()) && "Index is out of bound");
+        IGC_ASSERT(offset.Count() < offsetInst.size());
+        IGC_ASSERT(offset.Count() < (m_maxNumOfOutput + m_headerSize.Count()));
         offsetInst[offset.Count()] = inst;
 
         uint offsetWritten = int_cast<unsigned int>(offset.Count() + (mask <= 0xF ? 1 : 2));
@@ -568,7 +578,7 @@ namespace IGC
             return QuadEltUnit(0);
             break;
         default:
-            assert(!"unknown DS output type");
+            IGC_ASSERT_MESSAGE(0, "unknown DS output type");
             break;
         }
         return QuadEltUnit(0);
@@ -583,7 +593,7 @@ namespace IGC
         llvm::Value* data[8],
         llvm::Instruction* prev)
     {
-        assert(mask < 256 && "mask is an 8-bit bitmask and has to be in range 0..255");
+        IGC_ASSERT_MESSAGE(mask < 256, "mask is an 8-bit bitmask and has to be in range 0..255");
         Value* arguments[] =
         {
             offset,
