@@ -47,6 +47,9 @@ namespace IGC
         /// Driver supports Simple Push Mechanism only.
         virtual bool SupportsSimplePushOnly() const { return false; }
 
+        /// Driver supports Gather Constant Mechanism only.
+        virtual bool SupportsGatherConstantOnly() const { return false; }
+
         /// Driver supports resource streamer if HW supportes it, otherwise simple push
         virtual bool SupportsHWResourceStreameAndSimplePush() const { return false; }
 
@@ -112,9 +115,6 @@ namespace IGC
         /// API supports IEEE min/max
         virtual bool SupportsIEEEMinMax() const { return false; }
 
-        /// Supports precise math
-        virtual bool SupportsPreciseMath() const { return false; }
-
         virtual bool NeedCountSROA() const { return false; }
 
         /// Can we always contract mul and add
@@ -182,6 +182,7 @@ namespace IGC
 
         /// Need to know if the driver can accept more than one SIMD mode for compute shaders
         virtual bool sendMultipleSIMDModes() const { return false; }
+
         /// pick behavior whether we need to keep discarded helper pixels to calculate
         /// gradient correctly for sampler or we need to force early out discarded pixels
         virtual bool KeepDiscardHelperPixels() const { return false; }
@@ -224,9 +225,6 @@ namespace IGC
         /// disable some optimizations for front end which sends IR with unresolved NOS function when optimizing
         virtual bool WaNOSNotResolved() const { return false; }
 
-        /// Temporarary disable GS attr const interpolation promotion
-        virtual bool WADisableConstInterpolationPromotion() const { return false; }
-
         /// WA for APIs where frc generates a different precision than x - rndd(x) for small negative values
         /// Needs to switch to use fast math flags
         virtual bool DisableMatchFrcPatternMatch() const { return false; }
@@ -267,6 +265,31 @@ namespace IGC
         /// Check for flush to zero for DP emulated function
         virtual bool DPEmulationFlushToZero() const { return false; }
 
+        // Maximum id that can be used by simple push constant buffers. The default is maximum unsigned int (no restriction)
+        virtual unsigned int MaximumSimplePushBufferID() const { return std::numeric_limits<unsigned int>::max(); }
+
+        /// Use first VB to send vertex&base instance and second for draw index
+        virtual bool UsesVertexBuffersToSendShaderDrawParameters() const { return false; }
+
+        /// Use indirect payload in CS
+        virtual bool UsesIndirectPayload() const { return true; }
+
+        virtual bool SupportsDispatchGPGPUWalkerAlongYFirst() const { return true; }
+
+        /// Check if integer mad is enabled
+        virtual bool EnableIntegerMad() const { return false; }
+
+        /// add shader hash code after EOT for debug purposes
+        virtual bool EnableShaderDebugHashCodeInKernel() const { return false; }
+
+        // The size of output printf buffer is 4 MB by default by agreement with Runtime.
+        virtual uint32_t getPrintfBufferSize() const
+        {
+            return 4 * sizeof(MEGABYTE);
+        }
+
+        // Limits simple push constants based on pushed inputs
+        virtual bool EnableSimplePushRestriction() const { return false; }
 
 
     };
