@@ -24,8 +24,7 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 ======================= end_copyright_notice ==================================*/
 
-// vim:ts=2:sw=2:et:
-
+#include "IGC/common/StringMacros.hpp"
 #include "common/LLVMWarningsPush.hpp"
 #include "llvm/ADT/ArrayRef.h"
 #include "llvm/ADT/DenseMap.h"
@@ -43,25 +42,20 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "llvm/IR/Type.h"
 #include "llvm/IR/Value.h"
 #include "llvm/Pass.h"
-#include "llvm/PassAnalysisSupport.h"
 #include "llvm/Analysis/TargetFolder.h"
-
+#include "llvmWrapper/IR/DerivedTypes.h"
 #include "llvmWrapper/IR/Instructions.h"
 #include "llvmWrapper/IR/Intrinsics.h"
 #include "llvmWrapper/Support/Alignment.h"
-
 #include "common/LLVMWarningsPop.hpp"
-
 #include "common/LLVMUtils.h"
-
 #include "common/IGCIRBuilder.h"
-
 #include "GenISAIntrinsics/GenIntrinsics.h"
 #include "Compiler/CISACodeGen/ShaderCodeGen.hpp"
 #include "Compiler/IGCPassSupport.h"
 #include "Compiler/MetaDataUtilsWrapper.h"
-
 #include "Compiler/CISACodeGen/Emu64OpsPass.h"
+#include "Probe/Assertion.h"
 
 using namespace llvm;
 using namespace IGC;
@@ -125,8 +119,11 @@ namespace {
         LLVMContext* getContext() const { return TheContext; }
         Module* getModule() const { return TheModule; }
         Function* getFunction() const { return TheFunction; }
-
-        bool hasPtr64() const { return DL->getPointerSizeInBits() == 64; }
+        bool hasPtr64() const {
+            return (DL->getPointerSizeInBits() == 64 ||
+                DL->getPointerSizeInBits(ADDRESS_SPACE_GLOBAL) == 64 ||
+                DL->getPointerSizeInBits(ADDRESS_SPACE_CONSTANT) == 64);
+        }
         bool isPtr64(const PointerType* PtrTy) const {
             return CGC->getRegisterPointerSizeInBits(PtrTy->getAddressSpace()) == 64;
         }
@@ -136,7 +133,7 @@ namespace {
         bool isArg64Cast(BitCastInst* BC) const { return Arg64Casts.count(BC) != 0; }
 
         Type* getV2Int32Ty(unsigned NumElts = 1) const {
-            return VectorType::get(IRB->getInt32Ty(), NumElts * 2);
+            return IGCLLVM::FixedVectorType::get(IRB->getInt32Ty(), NumElts * 2);
         }
 
         ValuePair getExpandedValues(Value* V);
@@ -162,22 +159,22 @@ namespace {
         }
 
         void dupMemoryAttribute(LoadInst* NewLD, LoadInst* RefLD, unsigned Off) const {
-            unsigned Align = getAlignment(RefLD);
+            unsigned alignment = getAlignment(RefLD);
 
             NewLD->setVolatile(RefLD->isVolatile());
-            NewLD->setAlignment(MaybeAlign(unsigned(MinAlign(Align, Off))));
+            NewLD->setAlignment(IGCLLVM::getAlign(unsigned(MinAlign(alignment, Off))));
             NewLD->setOrdering(RefLD->getOrdering());
-            IGCLLVM::CopySyncScopeID(NewLD, RefLD);
+            NewLD->setSyncScopeID(RefLD->getSyncScopeID());
             copyKnownMetadata(NewLD, RefLD);
         }
 
         void dupMemoryAttribute(StoreInst* NewST, StoreInst* RefST, unsigned Off) const {
-            unsigned Align = getAlignment(RefST);
+            unsigned alignment = getAlignment(RefST);
 
             NewST->setVolatile(RefST->isVolatile());
-            NewST->setAlignment(MaybeAlign(unsigned(MinAlign(Align, Off))));
+            NewST->setAlignment(IGCLLVM::getAlign(unsigned(MinAlign(alignment, Off))));
             NewST->setOrdering(RefST->getOrdering());
-            IGCLLVM::CopySyncScopeID(NewST, RefST);
+            NewST->setSyncScopeID(RefST->getSyncScopeID());
             copyKnownMetadata(NewST, RefST);
         }
 
@@ -299,20 +296,22 @@ namespace {
                         UE = II->user_end(); UI != UE; /*EMPTY*/) {
                         User* U = *UI++;
                         ExtractValueInst* Ex = cast<ExtractValueInst>(U);
-                        assert(Ex->getNumIndices() == 1);
+                        IGC_ASSERT(nullptr != Ex);
+                        IGC_ASSERT(Ex->getNumIndices() == 1);
 
                         unsigned Idx = *Ex->idx_begin();
-                        assert(Idx == 0 || Idx == 1);
+                        IGC_ASSERT(Idx == 0 || Idx == 1);
                         Ex->replaceAllUsesWith((Idx == 0) ? Res : Overflow);
                         Ex->eraseFromParent();
                     }
-                    assert(II->user_empty());
+                    IGC_ASSERT(II->user_empty());
                     ++BI;
                     II->eraseFromParent();
                     Changed = true;
                 }
             }
             // Preprocess non-LOAD/-STORE pointer usage if there's 64-bit pointer.
+            IGC_ASSERT(nullptr != Emu);
             if (Emu->hasPtr64()) {
                 for (auto& BB : F) {
                     SmallVector<Instruction*, 16> LocalDeadInsts;
@@ -322,6 +321,7 @@ namespace {
                             break;
                         case Instruction::ICmp: {
                             ICmpInst* Cmp = cast<ICmpInst>(BI);
+                            IGC_ASSERT(nullptr != Cmp);
                             PointerType* PtrTy =
                                 dyn_cast<PointerType>(Cmp->getOperand(0)->getType());
                             if (!PtrTy || !Emu->isPtr64(PtrTy))
@@ -475,6 +475,7 @@ IGC_INITIALIZE_PASS_END(Emu64Ops, PASS_FLAG, PASS_DESC, PASS_CFG_ONLY, PASS_ANAL
 bool Emu64Ops::runOnFunction(Function& F) {
     // Skip non-kernel function.
     MetaDataUtils* MDU = getAnalysis<MetaDataUtilsWrapper>().getMetaDataUtils();
+    IGC_ASSERT(nullptr != MDU);
     auto FII = MDU->findFunctionsInfoItem(&F);
     if (FII == MDU->end_FunctionsInfo())
         return false;
@@ -544,8 +545,7 @@ ValuePair Emu64Ops::getExpandedValues(Value* V) {
         return VMI->second;
     }
 
-    errs() << "V = " << *V << '\n';
-    llvm_unreachable("TODO: NOT IMPLEMENTED!");
+    IGC_ASSERT_EXIT_MESSAGE(0, "TODO: NOT IMPLEMENTED!");
 }
 
 void Emu64Ops::setExpandedValues(Value* V, Value* Lo, Value* Hi) {
@@ -661,23 +661,34 @@ bool InstExpander::expand(Instruction* I) {
 }
 
 bool InstExpander::visitInstruction(Instruction& I) {
-#if 1
-    errs() << "I = " << I << '\n';
-#endif
-    llvm_unreachable("UNKNOWN INSTRUCTION is BEING EXPANDED!");
+    IGC_ASSERT_MESSAGE(0, "UNKNOWN INSTRUCTION is BEING EXPANDED!");
     return false;
 }
 
 bool InstExpander::visitRet(ReturnInst& RI) {
-    if (Value * V = RI.getReturnValue())
+    if (Value* V = RI.getReturnValue())
+    {
+        IGC_ASSERT(nullptr != Emu);
         if (Emu->isInt64(V))
-            // TODO: Add 64-bit return value support when function/subroutine call is
-            // supported.
-            llvm_unreachable("TODO: NOT IMPLEMENTED YET!");
+        {
+            Value* Lo = nullptr, *Hi = nullptr;
+            std::tie(Lo, Hi) = Emu->getExpandedValues(V);
+            Type* V2I32Ty = Emu->getV2Int32Ty();
+            Value* NewVal = UndefValue::get(V2I32Ty);
+            NewVal = IRB->CreateInsertElement(NewVal, Lo, IRB->getInt32(0));
+            NewVal = IRB->CreateInsertElement(NewVal, Hi, IRB->getInt32(1));
+            NewVal = IRB->CreateBitCast(NewVal, IRB->getInt64Ty());
+
+            IRB->SetInsertPoint(&RI);
+            (void) IRB->CreateRet(NewVal);
+            return true;
+        }
+    }
     return false;
 }
 
 bool InstExpander::visitAdd(BinaryOperator& BinOp) {
+    IGC_ASSERT(nullptr != Emu);
     if (!Emu->isInt64(&BinOp))
         return false;
 
@@ -688,6 +699,7 @@ bool InstExpander::visitAdd(BinaryOperator& BinOp) {
 
     GenISAIntrinsic::ID GIID = GenISAIntrinsic::GenISA_add_pair;
     Function* IFunc = GenISAIntrinsic::getDeclaration(Emu->getModule(), GIID);
+    IGC_ASSERT(nullptr != IRB);
     Value* V = IRB->CreateCall4(IFunc, L0, H0, L1, H1);
     Value* Lo = IRB->CreateExtractValue(V, 0);
     Value* Hi = IRB->CreateExtractValue(V, 1);
@@ -697,6 +709,7 @@ bool InstExpander::visitAdd(BinaryOperator& BinOp) {
 }
 
 bool InstExpander::visitSub(BinaryOperator& BinOp) {
+    IGC_ASSERT(nullptr != Emu);
     if (!Emu->isInt64(&BinOp))
         return false;
 
@@ -707,6 +720,7 @@ bool InstExpander::visitSub(BinaryOperator& BinOp) {
 
     GenISAIntrinsic::ID GIID = GenISAIntrinsic::GenISA_sub_pair;
     Function* IFunc = GenISAIntrinsic::getDeclaration(Emu->getModule(), GIID);
+    IGC_ASSERT(nullptr != IRB);
     Value* V = IRB->CreateCall4(IFunc, L0, H0, L1, H1);
     Value* Lo = IRB->CreateExtractValue(V, 0);
     Value* Hi = IRB->CreateExtractValue(V, 1);
@@ -716,6 +730,7 @@ bool InstExpander::visitSub(BinaryOperator& BinOp) {
 }
 
 bool InstExpander::visitMul(BinaryOperator& BinOp) {
+    IGC_ASSERT(nullptr != Emu);
     if (!Emu->isInt64(&BinOp))
         return false;
 
@@ -726,6 +741,7 @@ bool InstExpander::visitMul(BinaryOperator& BinOp) {
 
     GenISAIntrinsic::ID GIID = GenISAIntrinsic::GenISA_mul_pair;
     Function* IFunc = GenISAIntrinsic::getDeclaration(Emu->getModule(), GIID);
+    IGC_ASSERT(nullptr != IRB);
     Value* V = IRB->CreateCall4(IFunc, L0, H0, L1, H1);
     Value* Lo = IRB->CreateExtractValue(V, 0);
     Value* Hi = IRB->CreateExtractValue(V, 1);
@@ -735,34 +751,31 @@ bool InstExpander::visitMul(BinaryOperator& BinOp) {
 }
 
 bool InstExpander::visitSDiv(BinaryOperator& BinOp) {
-    if (!Emu->isInt64(&BinOp))
-        return false;
-    llvm_unreachable("There should not be `sdiv` which is already emulated by library call.");
+    IGC_ASSERT(nullptr != Emu);
+    IGC_ASSERT_MESSAGE(false == Emu->isInt64(&BinOp), "There should not be `sdiv` which is already emulated by library call.");
     return false;
 }
 
 bool InstExpander::visitUDiv(BinaryOperator& BinOp) {
-    if (!Emu->isInt64(&BinOp))
-        return false;
-    llvm_unreachable("There should not be `udiv` which is already emulated by library call.");
+    IGC_ASSERT(nullptr != Emu);
+    IGC_ASSERT_MESSAGE(false == Emu->isInt64(&BinOp), "There should not be `udiv` which is already emulated by library call.");
     return false;
 }
 
 bool InstExpander::visitSRem(BinaryOperator& BinOp) {
-    if (!Emu->isInt64(&BinOp))
-        return false;
-    llvm_unreachable("There should not be `srem` which is already emulated by library call.");
+    IGC_ASSERT(nullptr != Emu);
+    IGC_ASSERT_MESSAGE(false == Emu->isInt64(&BinOp), "There should not be `srem` which is already emulated by library call.");
     return false;
 }
 
 bool InstExpander::visitURem(BinaryOperator& BinOp) {
-    if (!Emu->isInt64(&BinOp))
-        return false;
-    llvm_unreachable("There should not be `urem` which is already emulated by library call.");
+    IGC_ASSERT(nullptr != Emu);
+    IGC_ASSERT_MESSAGE(false == Emu->isInt64(&BinOp), "There should not be `urem` which is already emulated by library call.");
     return false;
 }
 
 bool InstExpander::visitShl(BinaryOperator& BinOp) {
+    IGC_ASSERT(nullptr != Emu);
     if (!Emu->isInt64(&BinOp))
         return false;
 
@@ -846,7 +859,8 @@ bool InstExpander::visitShl(BinaryOperator& BinOp) {
         H = IRB->CreateOr(H, T0);
 
         if (InnerTBB) {
-            assert(isa<PHINode>(InnerResLo) && isa<PHINode>(InnerResHi));
+            IGC_ASSERT(isa<PHINode>(InnerResLo));
+            IGC_ASSERT(isa<PHINode>(InnerResHi));
             cast<PHINode>(InnerResLo)->addIncoming(L, InnerTBB);
             cast<PHINode>(InnerResHi)->addIncoming(H, InnerTBB);
         }
@@ -865,7 +879,8 @@ bool InstExpander::visitShl(BinaryOperator& BinOp) {
         Value* H = IRB->CreateShl(Lo, Amt);
 
         if (InnerFBB) {
-            assert(isa<PHINode>(InnerResLo) && isa<PHINode>(InnerResHi));
+            IGC_ASSERT(isa<PHINode>(InnerResLo));
+            IGC_ASSERT(isa<PHINode>(InnerResHi));
             cast<PHINode>(InnerResLo)->addIncoming(L, InnerFBB);
             cast<PHINode>(InnerResHi)->addIncoming(H, InnerFBB);
         }
@@ -880,6 +895,7 @@ bool InstExpander::visitShl(BinaryOperator& BinOp) {
 }
 
 bool InstExpander::visitLShr(BinaryOperator& BinOp) {
+    IGC_ASSERT(nullptr != Emu);
     if (!Emu->isInt64(&BinOp))
         return false;
 
@@ -963,7 +979,8 @@ bool InstExpander::visitLShr(BinaryOperator& BinOp) {
         L = IRB->CreateOr(L, T0);
 
         if (InnerTBB) {
-            assert(isa<PHINode>(InnerResLo) && isa<PHINode>(InnerResHi));
+            IGC_ASSERT(isa<PHINode>(InnerResLo));
+            IGC_ASSERT(isa<PHINode>(InnerResHi));
             cast<PHINode>(InnerResLo)->addIncoming(L, InnerTBB);
             cast<PHINode>(InnerResHi)->addIncoming(H, InnerTBB);
         }
@@ -982,7 +999,8 @@ bool InstExpander::visitLShr(BinaryOperator& BinOp) {
         Value* L = IRB->CreateLShr(Hi, Amt);
 
         if (InnerFBB) {
-            assert(isa<PHINode>(InnerResLo) && isa<PHINode>(InnerResHi));
+            IGC_ASSERT(isa<PHINode>(InnerResLo));
+            IGC_ASSERT(isa<PHINode>(InnerResHi));
             cast<PHINode>(InnerResLo)->addIncoming(L, InnerFBB);
             cast<PHINode>(InnerResHi)->addIncoming(H, InnerFBB);
         }
@@ -997,6 +1015,7 @@ bool InstExpander::visitLShr(BinaryOperator& BinOp) {
 }
 
 bool InstExpander::visitAShr(BinaryOperator& BinOp) {
+    IGC_ASSERT(nullptr != Emu);
     if (!Emu->isInt64(&BinOp))
         return false;
 
@@ -1080,7 +1099,8 @@ bool InstExpander::visitAShr(BinaryOperator& BinOp) {
         L = IRB->CreateOr(L, T0);
 
         if (InnerTBB) {
-            assert(isa<PHINode>(InnerResLo) && isa<PHINode>(InnerResHi));
+            IGC_ASSERT(isa<PHINode>(InnerResLo));
+            IGC_ASSERT(isa<PHINode>(InnerResHi));
             cast<PHINode>(InnerResLo)->addIncoming(L, InnerTBB);
             cast<PHINode>(InnerResHi)->addIncoming(H, InnerTBB);
         }
@@ -1099,7 +1119,8 @@ bool InstExpander::visitAShr(BinaryOperator& BinOp) {
         Value* L = IRB->CreateAShr(Hi, Amt);
 
         if (InnerFBB) {
-            assert(isa<PHINode>(InnerResLo) && isa<PHINode>(InnerResHi));
+            IGC_ASSERT(isa<PHINode>(InnerResLo));
+            IGC_ASSERT(isa<PHINode>(InnerResHi));
             cast<PHINode>(InnerResLo)->addIncoming(L, InnerFBB);
             cast<PHINode>(InnerResHi)->addIncoming(H, InnerFBB);
         }
@@ -1114,6 +1135,7 @@ bool InstExpander::visitAShr(BinaryOperator& BinOp) {
 }
 
 bool InstExpander::visitAnd(BinaryOperator& BinOp) {
+    IGC_ASSERT(nullptr != Emu);
     if (!Emu->isInt64(&BinOp))
         return false;
 
@@ -1130,6 +1152,7 @@ bool InstExpander::visitAnd(BinaryOperator& BinOp) {
 }
 
 bool InstExpander::visitOr(BinaryOperator& BinOp) {
+    IGC_ASSERT(nullptr != Emu);
     if (!Emu->isInt64(&BinOp))
         return false;
 
@@ -1146,6 +1169,7 @@ bool InstExpander::visitOr(BinaryOperator& BinOp) {
 }
 
 bool InstExpander::visitXor(BinaryOperator& BinOp) {
+    IGC_ASSERT(nullptr != Emu);
     if (!Emu->isInt64(&BinOp))
         return false;
 
@@ -1162,6 +1186,7 @@ bool InstExpander::visitXor(BinaryOperator& BinOp) {
 }
 
 bool InstExpander::visitLoad(LoadInst& LD) {
+    IGC_ASSERT(nullptr != Emu);
     if (!Emu->isInt64(&LD))
         return false;
 
@@ -1183,6 +1208,7 @@ bool InstExpander::visitLoad(LoadInst& LD) {
 
 bool InstExpander::visitStore(StoreInst& ST) {
     Value* V = ST.getValueOperand();
+    IGC_ASSERT(nullptr != Emu);
     if (!Emu->isInt64(V))
         return false;
 
@@ -1207,28 +1233,32 @@ bool InstExpander::visitStore(StoreInst& ST) {
 
 bool InstExpander::visitAtomicCmpXchg(AtomicCmpXchgInst& ACXI) {
     Value* V = ACXI.getCompareOperand();
-    if (!Emu->isInt64(V))
-        return false;
-    llvm_unreachable("TODO: NOT IMPLEMENTED YET!");
+    IGC_ASSERT(nullptr != V);
+    IGC_ASSERT(nullptr != Emu);
+    IGC_ASSERT_MESSAGE(false == Emu->isInt64(V), "TODO: NOT IMPLEMENTED YET!");
+
     return false;
 }
 
 bool InstExpander::visitAtomicRMW(AtomicRMWInst& RMW) {
-    if (!Emu->isInt64(&RMW))
-        return false;
-    llvm_unreachable("TODO: NOT IMPLEMENTED YET!");
+    IGC_ASSERT(nullptr != Emu);
+    IGC_ASSERT_MESSAGE(false == Emu->isInt64(&RMW), "TODO: NOT IMPLEMENTED YET!");
     return false;
 }
 
 bool InstExpander::visitTrunc(TruncInst& TI) {
     Value* Src = TI.getOperand(0);
+    IGC_ASSERT(nullptr != Emu);
+    IGC_ASSERT(nullptr != Src);
     if (!Emu->isInt64(Src))
         return false;
 
     Value* Lo = nullptr;
     std::tie(Lo, std::ignore) = Emu->getExpandedValues(Src);
-    assert(Lo->getType()->getScalarSizeInBits() >=
-        TI.getType()->getScalarSizeInBits());
+    IGC_ASSERT(nullptr != Lo);
+    IGC_ASSERT(nullptr != Lo->getType());
+    IGC_ASSERT(nullptr != TI.getType());
+    IGC_ASSERT(Lo->getType()->getScalarSizeInBits() >= TI.getType()->getScalarSizeInBits());
 
     if (Lo->getType()->getScalarSizeInBits() !=
         TI.getType()->getScalarSizeInBits())
@@ -1239,13 +1269,15 @@ bool InstExpander::visitTrunc(TruncInst& TI) {
 }
 
 bool InstExpander::visitSExt(SExtInst& SEI) {
+    IGC_ASSERT(nullptr != Emu);
     if (!Emu->isInt64(&SEI))
         return false;
 
     Value* Src = SEI.getOperand(0);
     Type* SrcTy = SEI.getSrcTy();
+    IGC_ASSERT(nullptr != SrcTy);
     unsigned SrcWidth = SrcTy->getIntegerBitWidth();
-    assert(SrcWidth <= 32);
+    IGC_ASSERT(SrcWidth <= 32);
 
     Value* Lo = Src;
     if (SrcWidth < 32)
@@ -1257,13 +1289,15 @@ bool InstExpander::visitSExt(SExtInst& SEI) {
 }
 
 bool InstExpander::visitZExt(ZExtInst& ZEI) {
+    IGC_ASSERT(nullptr != Emu);
     if (!Emu->isInt64(&ZEI))
         return false;
 
     Value* Src = ZEI.getOperand(0);
     Type* SrcTy = ZEI.getSrcTy();
+    IGC_ASSERT(nullptr != SrcTy);
     unsigned SrcWidth = SrcTy->getIntegerBitWidth();
-    assert(SrcWidth <= 32);
+    IGC_ASSERT(SrcWidth <= 32);
 
     Value* Lo = Src;
     if (SrcWidth < 32)
@@ -1275,12 +1309,15 @@ bool InstExpander::visitZExt(ZExtInst& ZEI) {
 }
 
 bool InstExpander::visitFPToUI(FPToUIInst& F2U) {
+    IGC_ASSERT(nullptr != Emu);
     if (!Emu->isInt64(&F2U))
         return false;
 
     IGCLLVM::Intrinsic IID;
     Value* Src = F2U.getOperand(0);
+    IGC_ASSERT(nullptr != Src);
     Type* SrcTy = Src->getType();
+    IGC_ASSERT(nullptr != SrcTy);
 
     if (SrcTy->isHalfTy()) {
         // Convert half directly into 32-bit integer.
@@ -1327,12 +1364,15 @@ bool InstExpander::visitFPToUI(FPToUIInst& F2U) {
 }
 
 bool InstExpander::visitFPToSI(FPToSIInst& F2S) {
+    IGC_ASSERT(nullptr != Emu);
     if (!Emu->isInt64(&F2S))
         return false;
 
     IGCLLVM::Intrinsic IID;
     Value* Src = F2S.getOperand(0);
+    IGC_ASSERT(nullptr != Src);
     Type* SrcTy = Src->getType();
+    IGC_ASSERT(nullptr != SrcTy);
 
     if (SrcTy->isHalfTy()) {
         // Convert half directly into 32-bit integer.
@@ -1362,7 +1402,7 @@ bool InstExpander::visitFPToSI(FPToSIInst& F2S) {
         Sign = IRB->CreateExtractElement(Sign, IRB->getInt32(1));
     }
     else {
-        assert(SrcTy->isFloatTy() && "Unknown float type!");
+        IGC_ASSERT_MESSAGE(SrcTy->isFloatTy(), "Unknown float type!");
         Sign = IRB->CreateBitCast(Src, IRB->getInt32Ty());
     }
     Sign = IRB->CreateAShr(Sign, 31);
@@ -1414,6 +1454,8 @@ bool InstExpander::visitFPToSI(FPToSIInst& F2S) {
 // method is called.
 //
 Value* InstExpander::convertUIToFP32(Type* DstTy, Value* Lo, Value* Hi, Instruction* Pos) {
+    IGC_ASSERT(nullptr != Pos);
+    IGC_ASSERT(nullptr != IRB);
     BuilderType::InsertPointGuard Guard(*IRB);
     IRB->SetInsertPoint(Pos);
 
@@ -1426,6 +1468,7 @@ Value* InstExpander::convertUIToFP32(Type* DstTy, Value* Lo, Value* Hi, Instruct
     Value* NE = IRB->CreateICmpNE(ShAmt, IRB->getInt32(32));
 
     BasicBlock* OldBB = Pos->getParent();
+    IGC_ASSERT(nullptr != OldBB);
     BasicBlock* JointBB = OldBB->splitBasicBlock(Pos);
     PHINode* Res = PHINode::Create(IRB->getInt32Ty(), 2, ".u2f.outer.merge", Pos);
 
@@ -1505,6 +1548,7 @@ Value* InstExpander::convertUIToFP32(Type* DstTy, Value* Lo, Value* Hi, Instruct
 
 bool InstExpander::visitUIToFP(UIToFPInst& U2F) {
     Value* Src = U2F.getOperand(0);
+    IGC_ASSERT(nullptr != Emu);
     if (!Emu->isInt64(Src))
         return false;
 
@@ -1548,6 +1592,7 @@ bool InstExpander::visitUIToFP(UIToFPInst& U2F) {
 
 bool InstExpander::visitSIToFP(SIToFPInst& S2F) {
     Value* Src = S2F.getOperand(0);
+    IGC_ASSERT(nullptr != Emu);
     if (!Emu->isInt64(Src))
         return false;
 
@@ -1607,6 +1652,7 @@ bool InstExpander::visitSIToFP(SIToFPInst& S2F) {
 }
 
 bool InstExpander::visitPtrToInt(PtrToIntInst& P2I) {
+    IGC_ASSERT(nullptr != Emu);
     if (!Emu->isInt64(&P2I))
         return false;
 
@@ -1624,6 +1670,7 @@ bool InstExpander::visitPtrToInt(PtrToIntInst& P2I) {
 
 bool InstExpander::visitIntToPtr(IntToPtrInst& I2P) {
     Value* Src = I2P.getOperand(0);
+    IGC_ASSERT(nullptr != Emu);
     if (!Emu->isInt64(Src))
         return false;
 
@@ -1641,6 +1688,7 @@ bool InstExpander::visitIntToPtr(IntToPtrInst& I2P) {
 
 bool InstExpander::visitBitCast(BitCastInst& BC) {
     Value* Src = BC.getOperand(0);
+    IGC_ASSERT(nullptr != Emu);
     if (!Emu->isInt64(&BC) && !Emu->isInt64(Src))
         return false;
 
@@ -1653,7 +1701,7 @@ bool InstExpander::visitBitCast(BitCastInst& BC) {
         return true;
     }
 
-    assert(Emu->isInt64(Src));
+    IGC_ASSERT(Emu->isInt64(Src));
 
     // Skip argument which is already prepared specially.
     if (Emu->isArg64Cast(&BC))
@@ -1673,6 +1721,7 @@ bool InstExpander::visitBitCast(BitCastInst& BC) {
 }
 
 bool InstExpander::visitICmp(ICmpInst& Cmp) {
+    IGC_ASSERT(nullptr != Emu);
     if (!Emu->isInt64(Cmp.getOperand(0)))
         return false;
 
@@ -1684,9 +1733,6 @@ bool InstExpander::visitICmp(ICmpInst& Cmp) {
 
     Value* T0 = nullptr, * T1 = nullptr, * T2 = nullptr, * T3 = nullptr, * Res = nullptr;
     switch (Pred) {
-    default:
-        llvm_unreachable("Invalid ICmp predicate");
-        break;
     case CmpInst::ICMP_EQ:
         T0 = IRB->CreateICmpEQ(L0, L1);
         T1 = IRB->CreateICmpEQ(H0, H1);
@@ -1753,19 +1799,25 @@ bool InstExpander::visitICmp(ICmpInst& Cmp) {
         T3 = IRB->CreateICmpSLT(H0, H1);
         Res = IRB->CreateOr(T2, T3);
         break;
+    default:
+        IGC_ASSERT_EXIT_MESSAGE(0, "Invalid ICmp predicate");
+        break;
     }
-    assert(Res != nullptr);
+
+    IGC_ASSERT(nullptr != Res);
 
     Cmp.replaceAllUsesWith(Res);
     return true;
 }
 
 bool InstExpander::visitPHI(PHINode& PN) {
+    IGC_ASSERT(nullptr != Emu);
     if (!Emu->isInt64(&PN))
         return false;
     Value* Lo = nullptr, * Hi = nullptr;
     std::tie(Lo, Hi) = Emu->getExpandedValues(&PN);
-    assert(Lo != nullptr && Hi != nullptr);
+    IGC_ASSERT(nullptr != Lo);
+    IGC_ASSERT(nullptr != Hi);
     return false;
 }
 
@@ -1774,7 +1826,8 @@ bool InstExpander::visitCall(CallInst& Call) {
     // lambdas for splitting and combining i64 to <2 x i32>
     auto Combine2xi32Toi64 = [this](Value* val)->Value *
     {
-        assert(Emu->isInt64(val));
+        IGC_ASSERT(nullptr != Emu);
+        IGC_ASSERT(Emu->isInt64(val));
         Value* InputLo = nullptr, * InputHi = nullptr;
         std::tie(InputLo, InputHi) = Emu->getExpandedValues(val);
         Type* V2I32Ty = Emu->getV2Int32Ty();
@@ -1786,7 +1839,8 @@ bool InstExpander::visitCall(CallInst& Call) {
     };
     auto Spliti64To2xi32 = [this](Value* retVal, Value*& OutputLo, Value*& OutputHi)->void
     {
-        assert(Emu->isInt64(retVal));
+        IGC_ASSERT(nullptr != Emu);
+        IGC_ASSERT(Emu->isInt64(retVal));
         Value* V = IRB->CreateBitCast(retVal, Emu->getV2Int32Ty());
         OutputLo = IRB->CreateExtractElement(V, IRB->getInt32(0));
         OutputHi = IRB->CreateExtractElement(V, IRB->getInt32(1));
@@ -1812,6 +1866,7 @@ bool InstExpander::visitCall(CallInst& Call) {
             return false;
         }
     }
+    IGC_ASSERT(nullptr != Emu);
     if (!Emu->isInt64(&Call)) {
         for (auto& Op : Call.operands()) {
             if (Emu->isInt64(Op.get()))
@@ -1834,6 +1889,7 @@ Emu64BitCall:
         case GenISAIntrinsic::GenISA_simdGetMessagePhase:
         case GenISAIntrinsic::GenISA_RuntimeValue:
         case GenISAIntrinsic::GenISA_simdBlockRead:
+        case GenISAIntrinsic::GenISA_simdMediaBlockRead:
         {
             auto* GenCopy = Call.clone();
             GenCopy->insertBefore(&Call);
@@ -1884,8 +1940,10 @@ Emu64BitCall:
         case GenISAIntrinsic::GenISA_uitof_rtp:
         case GenISAIntrinsic::GenISA_uitof_rtz:
         case GenISAIntrinsic::GenISA_simdBlockWrite:
+        case GenISAIntrinsic::GenISA_simdMediaBlockWrite:
         {
             auto* GenCopy = Call.clone();
+            IGC_ASSERT(nullptr != GenCopy);
             GenCopy->insertBefore(&Call);
             IRB->SetInsertPoint(GenCopy);
             uint opNum = 0;
@@ -1905,6 +1963,7 @@ Emu64BitCall:
         case GenISAIntrinsic::GenISA_WavePrefix:
         {
             auto* GenCopy = Call.clone();
+            IGC_ASSERT(nullptr != GenCopy);
             GenCopy->insertBefore(&Call);
             IRB->SetInsertPoint(GenCopy);
 
@@ -1924,10 +1983,13 @@ Emu64BitCall:
             break;
         }
     }
-    // Support for subroutine calls
-    else if (F->hasFnAttribute("UserSubroutine"))
+    // Support for stack/indirect/subroutine calls
+    // Note: should use enableFunctionCall() without using attr checking.
+    else if (   !F || F->hasFnAttribute("visaStackCall") || F->hasFnAttribute("UserSubroutine")
+             || Emu->CGC->enableFunctionCall())
     {
         auto* CallCopy = Call.clone();
+        IGC_ASSERT(nullptr != CallCopy);
         CallCopy->insertBefore(&Call);
         IRB->SetInsertPoint(CallCopy);
         unsigned argNo = 0;
@@ -1957,6 +2019,7 @@ Emu64BitCall:
 }
 
 bool InstExpander::visitSelect(SelectInst& SI) {
+    IGC_ASSERT(nullptr != Emu);
     if (!Emu->isInt64(&SI))
         return false;
 
@@ -1966,6 +2029,7 @@ bool InstExpander::visitSelect(SelectInst& SI) {
     Value* L1 = nullptr, * H1 = nullptr;
     std::tie(L1, H1) = Emu->getExpandedValues(SI.getOperand(2));
 
+    IGC_ASSERT(nullptr != IRB);
     Value* Lo = IRB->CreateSelect(Cond, L0, L1);
     Value* Hi = IRB->CreateSelect(Cond, H0, H1);
 
@@ -1974,15 +2038,15 @@ bool InstExpander::visitSelect(SelectInst& SI) {
 }
 
 bool InstExpander::visitVAArg(VAArgInst& VAAI) {
-    if (!Emu->isInt64(&VAAI))
-        return false;
     // TODO: Add i64 emulation support.
-    llvm_unreachable("TODO: NOT IMPLEMENTED YET!");
+    IGC_ASSERT(nullptr != Emu);
+    IGC_ASSERT_MESSAGE(false == Emu->isInt64(&VAAI), "TODO: NOT IMPLEMENTED YET!");
     return false;
 }
 
 bool InstExpander::visitExtractElement(ExtractElementInst& EEI) {
     // Fix index operand if necessary.
+    IGC_ASSERT(nullptr != Emu);
     if (Emu->isInt64(EEI.getIndexOperand())) {
         Value* L = nullptr;
         std::tie(L, std::ignore) = Emu->getExpandedValues(EEI.getIndexOperand());
@@ -1997,10 +2061,11 @@ bool InstExpander::visitExtractElement(ExtractElementInst& EEI) {
     // later.
 
     Value* V = EEI.getVectorOperand();
-    unsigned NumElts = V->getType()->getVectorNumElements();
+    unsigned NumElts = (unsigned)cast<VectorType>(V->getType())->getNumElements();
     V = IRB->CreateBitCast(V, Emu->getV2Int32Ty(NumElts));
     // Re-calculate indices to Lo and Hi parts.
     Value* Idx = EEI.getIndexOperand();
+    Idx = IRB->CreateZExt(Idx, IRB->getInt32Ty());
     Idx = IRB->CreateMul(Idx, IRB->getInt32(2));
     Value* IdxLo = IRB->CreateAdd(Idx, IRB->getInt32(0));
     Value* IdxHi = IRB->CreateAdd(Idx, IRB->getInt32(1));
@@ -2013,6 +2078,7 @@ bool InstExpander::visitExtractElement(ExtractElementInst& EEI) {
 
 bool InstExpander::visitInsertElement(InsertElementInst& IEI) {
     // Fix index operand if necessary.
+    IGC_ASSERT(nullptr != Emu);
     if (Emu->isInt64(IEI.getOperand(2))) {
         Value* L = nullptr;
         std::tie(L, std::ignore) = Emu->getExpandedValues(IEI.getOperand(2));
@@ -2032,7 +2098,7 @@ bool InstExpander::visitInsertElement(InsertElementInst& IEI) {
 
     // Create the emulated vector.
     Value* NewVal = IEI.getOperand(0);
-    unsigned NumElts = NewVal->getType()->getVectorNumElements();
+    unsigned NumElts = (unsigned)cast<VectorType>(NewVal->getType())->getNumElements();
     NewVal = IRB->CreateBitCast(NewVal, Emu->getV2Int32Ty(NumElts));
     // Re-calculate indices to Lo and Hi parts.
     Value* Idx = IEI.getOperand(2);
@@ -2049,25 +2115,24 @@ bool InstExpander::visitInsertElement(InsertElementInst& IEI) {
 }
 
 bool InstExpander::visitExtractValue(ExtractValueInst& EVI) {
-    if (!Emu->isInt64(&EVI))
-        return false;
     // TODO: Add i64 emulation support.
-    llvm_unreachable("TODO: NOT IMPLEMENTED YET!");
+    IGC_ASSERT(nullptr != Emu);
+    IGC_ASSERT_MESSAGE(false == Emu->isInt64(&EVI), "TODO: NOT IMPLEMENTED YET!");
     return false;
 }
 
 bool InstExpander::visitInsertValue(InsertValueInst& IVI) {
-    if (!Emu->isInt64(IVI.getOperand(1)))
-        return false;
     // TODO: Add i64 emulation support.
-    llvm_unreachable("TODO: NOT IMPLEMENTED YET!");
+    IGC_ASSERT(nullptr != Emu);
+    IGC_ASSERT(0 < IVI.getNumOperands());
+    IGC_ASSERT_MESSAGE(false == Emu->isInt64(IVI.getOperand(1)), "TODO: NOT IMPLEMENTED YET!");
     return false;
 }
 
 bool InstExpander::visitLandingPad(LandingPadInst& LPI) {
-    if (!Emu->isInt64(LPI.getOperand(1)))
-        return false;
     // TODO: Add i64 emulation support.
-    llvm_unreachable("TODO: NOT IMPLEMENTED YET!");
+    IGC_ASSERT(nullptr != Emu);
+    IGC_ASSERT(0 < LPI.getNumOperands());
+    IGC_ASSERT_MESSAGE(false == Emu->isInt64(LPI.getOperand(1)), "TODO: NOT IMPLEMENTED YET!");
     return false;
 }
