@@ -25,12 +25,12 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 ======================= end_copyright_notice ==================================*/
 
 #pragma once
-
 #include "common/LLVMWarningsPush.hpp"
 #include "llvm/ADT/DenseMap.h"
 #include "llvm/Pass.h"
 #include "common/LLVMWarningsPop.hpp"
 #include <cstddef>
+#include "Probe/Assertion.h"
 
 namespace IGC {
 
@@ -63,14 +63,20 @@ namespace IGC {
 
         bool hasRecursion() const { return HasRecursion; }
 
+        bool isTrimmedFunction( llvm::Function* F);
+
     private:
         void analyze();
         void checkSubroutine();
         void clear();
 
+        bool funcIsGoodtoTrim( llvm::Function* F );
+        void reduceKernelSize();
+        size_t findKernelTotalSize(llvm::Function* Kernel, uint32_t uk, uint32_t &up);
+
         /// \brief Return the associated opaque data.
         template <typename T> T* get(llvm::Function* F) {
-            assert(ECG.count(F));
+            IGC_ASSERT(ECG.count(F));
             return static_cast<T*>(ECG[F]);
         }
 
