@@ -24,13 +24,13 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 ======================= end_copyright_notice ==================================*/
 #pragma once
-
 #include "Compiler/MetaDataApi/MetaDataApi.h"
 #include "common/LLVMWarningsPush.hpp"
 #include "llvm/IR/ValueHandle.h"
 #include "llvm/Pass.h"
 #include "llvm/Analysis/CallGraph.h"
 #include "common/LLVMWarningsPop.hpp"
+#include "Probe/Assertion.h"
 
 namespace IGC {
     class GenXFunctionGroupAnalysis;
@@ -98,16 +98,21 @@ namespace IGC {
         bool hasStackCall() {
             return m_hasStackCall;
         }
+        /// \brief Function group has a variable length alloca
+        bool hasVariableLengthAlloca() {
+            return m_hasVaribleLengthAlloca;
+        }
 
         void replaceGroupHead(llvm::Function* OH, llvm::Function* NH) {
             auto headSG = Functions[0];
             llvm::AssertingVH<llvm::Function>& HVH = (*headSG)[0];
-            assert(&(*HVH) == OH && "Group's head isn't set up correctly!");
+            IGC_ASSERT_MESSAGE(&(*HVH) == OH, "Group's head isn't set up correctly!");
             HVH = NH;
         }
 
     private:
         bool m_hasStackCall = false;
+        bool m_hasVaribleLengthAlloca = false;
     };
 
     class GenXFunctionGroupAnalysis : public llvm::ImmutablePass {
@@ -163,7 +168,7 @@ namespace IGC {
         /// \brief Get the group head for the group to which F belongs.
         llvm::Function* getGroupHead(llvm::Function* F) {
             auto FG = getGroup(F);
-            assert(FG);
+            IGC_ASSERT(nullptr != FG);
             return FG->getHead();
         }
         /// \brief Get the subgroup head for the subgroup to which F belongs
@@ -184,6 +189,9 @@ namespace IGC {
             }
         }
 
+        /// \brief Check whether a group contains variable length alloca
+        void setHasVariableLengthAlloca(llvm::Module* pModule);
+
         /// \brief Check whether this is a group header.
         bool isGroupHead(llvm::Function* F) {
             return getGroupForHead(F) != nullptr;
@@ -193,7 +201,7 @@ namespace IGC {
         /// order is also reflected in the module function list.
         bool isGroupTail(llvm::Function* F) {
             FunctionGroup* FG = getGroup(F);
-            assert(FG && "not in function group");
+            IGC_ASSERT_MESSAGE(nullptr != FG, "not in function group");
             return F == FG->back();
         }
 
