@@ -26,14 +26,12 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 // vim:ts=2:sw=2:et:
 #include "common/LLVMUtils.h"
-
 #include "Compiler/CISACodeGen/GenIRLowering.h"
 #include "Compiler/CISACodeGen/OpenCLKernelCodeGen.hpp"
 #include "Compiler/CISACodeGen/ShaderCodeGen.hpp"
 #include "Compiler/CodeGenContextWrapper.hpp"
 #include "Compiler/MetaDataUtilsWrapper.h"
 #include "Compiler/IGCPassSupport.h"
-
 #include "common/LLVMWarningsPush.hpp"
 #include <llvm/Pass.h>
 #include <llvm/IR/DataLayout.h>
@@ -42,16 +40,17 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <llvm/IR/PatternMatch.h>
 #include <llvm/Analysis/TargetFolder.h>
 #include "llvm/IR/GetElementPtrTypeIterator.h"
-
 #include "llvmWrapper/IR/Intrinsics.h"
-
+#include "llvmWrapper/IR/DerivedTypes.h"
 #include "common/LLVMWarningsPop.hpp"
 #include "GenISAIntrinsics/GenIntrinsics.h"
 #include "common/IGCIRBuilder.h"
+#include "Probe/Assertion.h"
 
 using namespace llvm;
 using namespace IGC;
 using namespace IGC::IGCMD;
+using IGCLLVM::FixedVectorType;
 
 namespace {
     class GenIRLowering : public FunctionPass {
@@ -451,8 +450,19 @@ bool GEPLowering::runOnFunction(Function& F) {
 
 Value* GEPLowering::getSExtOrTrunc(Value* Val, Type* NewTy) const {
     Type* OldTy = Val->getType();
-    unsigned OldWidth = OldTy->getIntegerBitWidth();
-    unsigned NewWidth = NewTy->getIntegerBitWidth();
+    unsigned OldWidth;
+    unsigned NewWidth;
+
+    IGC_ASSERT_MESSAGE(OldTy->isIntOrIntVectorTy(), "Index should be Integer or vector of Integer!");
+
+    if (auto OldVecTy = dyn_cast<VectorType>(OldTy)) {
+        OldWidth = (unsigned)OldVecTy->getNumElements() * OldVecTy->getElementType()->getIntegerBitWidth();
+        NewWidth = (unsigned)OldVecTy->getNumElements() * NewTy->getIntegerBitWidth();
+    }
+    else {
+        OldWidth = OldTy->getIntegerBitWidth();
+        NewWidth = NewTy->getIntegerBitWidth();
+    }
 
     if (OldWidth < NewWidth) { // SExt
         return Builder->CreateSExt(Val, NewTy);
@@ -577,10 +587,10 @@ Value* GenIRLowering::rearrangeAdd(Value* val, Loop* loop) const
 
 bool GEPLowering::lowerGetElementPtrInst(GetElementPtrInst* GEP) const
 {
-    Value* PtrOp = GEP->getPointerOperand();
-    PointerType* PtrTy = dyn_cast<PointerType>(PtrOp->getType());
-
-    assert(PtrTy && "Only accept scalar pointer!");
+    Value* const PtrOp = GEP->getPointerOperand();
+    IGC_ASSERT(nullptr != PtrOp);
+    PointerType* const PtrTy = dyn_cast<PointerType>(PtrOp->getType());
+    IGC_ASSERT_MESSAGE(nullptr != PtrTy, "Only accept scalar pointer!");
 
     unsigned pointerSizeInBits = m_ctx->getRegisterPointerSizeInBits(PtrTy->getAddressSpace());
     unsigned pointerMathSizeInBits = pointerSizeInBits;
@@ -797,6 +807,13 @@ bool GEPLowering::lowerGetElementPtrInst(GetElementPtrInst* GEP) const
                 }
                 else
                 {
+                    if (auto NewIdxVT = dyn_cast<VectorType>(NewIdx->getType())) {
+                        Value* result = llvm::UndefValue::get(FixedVectorType::get(PtrMathTy, (unsigned)NewIdxVT->getNumElements()));
+                        for (uint32_t j = 0; j < (uint32_t)NewIdxVT->getNumElements(); j++) {
+                            result = Builder->CreateInsertElement(result, PointerValue, Builder->getInt32(j));
+                        }
+                        PointerValue = result;
+                    }
                     PointerValue = Builder->CreateAdd(PointerValue, NewIdx);
                     if (COffset)
                     {
@@ -809,7 +826,7 @@ bool GEPLowering::lowerGetElementPtrInst(GetElementPtrInst* GEP) const
 
     if (reducePointerArith)
     {
-        assert(GEP->isInBounds() && "we can only do a zext if the GEP is inbound");
+        IGC_ASSERT_MESSAGE(GEP->isInBounds(), "we can only do a zext if the GEP is inbound");
         if (!canReduceNegativeOffset)
         {
             PointerValue = Builder->CreateZExt(PointerValue, BasePointer->getType());
