@@ -23,7 +23,9 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 
 ======================= end_copyright_notice ==================================*/
+
 #include "common/LLVMWarningsPush.hpp"
+#include "llvm/Config/llvm-config.h"
 #include <llvm/ADT/STLExtras.h>
 #include <llvmWrapper/Analysis/MemoryLocation.h>
 #include <llvmWrapper/Analysis/TargetLibraryInfo.h>
@@ -37,17 +39,19 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <llvm/IR/GlobalAlias.h>
 #include <llvm/IR/IRBuilder.h>
 #include <llvm/Pass.h>
+#include <llvmWrapper/Support/Alignment.h>
+#include <llvmWrapper/IR/DerivedTypes.h>
 #include <llvm/Support/Debug.h>
 #include <llvm/Support/DebugCounter.h>
 #include <llvm/Support/raw_ostream.h>
 #include <llvm/Transforms/Utils/Local.h>
 #include "common/LLVMWarningsPop.hpp"
-
 #include "Compiler/CISACodeGen/ShaderCodeGen.hpp"
 #include "Compiler/IGCPassSupport.h"
 #include "Compiler/MetaDataUtilsWrapper.h"
 #include "Compiler/CISACodeGen/WIAnalysis.hpp"
 #include "Compiler/CISACodeGen/MemOpt.h"
+#include "Probe/Assertion.h"
 
 using namespace llvm;
 using namespace IGC;
@@ -123,7 +127,7 @@ namespace {
             MemRefListTy& MemRefs, TrivialMemRefListTy& ToOpt);
 
         unsigned getNumElements(Type* Ty) const {
-            return Ty->isVectorTy() ? Ty->getVectorNumElements() : 1;
+            return Ty->isVectorTy() ? (unsigned)cast<VectorType>(Ty)->getNumElements() : 1;
         }
 
         MemoryLocation getLocation(Instruction* I) const {
@@ -154,12 +158,6 @@ namespace {
             if (V->getType() == DestTy)
                 return V;
 
-            if (V->getType()->isPointerTy() && DestTy->isIntegerTy())
-                return Builder.CreatePtrToInt(V, DestTy);
-
-            if (V->getType()->isIntegerTy() && DestTy->isPointerTy())
-                return Builder.CreateIntToPtr(V, DestTy);
-
             if (V->getType()->isPointerTy() && DestTy->isPointerTy()) {
                 PointerType* SrcPtrTy = cast<PointerType>(V->getType());
                 PointerType* DstPtrTy = cast<PointerType>(DestTy);
@@ -168,6 +166,30 @@ namespace {
                     return Builder.CreateAddrSpaceCast(V, DestTy);
             }
 
+            if (V->getType()->isPointerTy()) {
+                if (DestTy->isIntegerTy()) {
+                    return Builder.CreatePtrToInt(V, DestTy);
+                }
+                else if (DestTy->isFloatingPointTy()) {
+                    uint32_t Size = (uint32_t)DestTy->getPrimitiveSizeInBits();
+                    Value* Cast = Builder.CreatePtrToInt(
+                        V, Builder.getIntNTy(Size));
+                    return Builder.CreateBitCast(Cast, DestTy);
+                }
+            }
+
+            if (DestTy->isPointerTy()) {
+                if (V->getType()->isIntegerTy()) {
+                    return Builder.CreateIntToPtr(V, DestTy);
+                }
+                else if (V->getType()->isFloatingPointTy()) {
+                    uint32_t Size = (uint32_t)V->getType()->getPrimitiveSizeInBits();
+                    Value* Cast = Builder.CreateBitCast(
+                        V, Builder.getIntNTy(Size));
+                    return Builder.CreateIntToPtr(Cast, DestTy);
+                }
+            }
+
             return Builder.CreateBitCast(V, DestTy);
         }
 
@@ -217,20 +239,27 @@ namespace {
         {
             if (inst->getAlignment() < 4 && WI->whichDepend(inst) != WIAnalysis::UNIFORM)
             {
+                llvm::Type* dataType = isa<LoadInst>(inst) ? inst->getType() : inst->getOperand(0)->getType();
+                unsigned scalarTypeSizeInBytes = unsigned(DL->getTypeSizeInBits(dataType->getScalarType()) / 8);
+
                 // Need the first offset value (not necessarily zero)
-                auto firstOffset = std::get<1>(AccessIntrs[0]);
+                int64_t firstOffset = std::get<1>(AccessIntrs[0]);
+                int64_t mergedSize = 0;
                 for (auto rit = AccessIntrs.rbegin(),
                     rie = AccessIntrs.rend(); rit != rie; ++rit)
                 {
-                    unsigned accessSize = 0;
-                    auto cur_offset = std::get<1>(*rit);
+                    int64_t accessSize = 0;
+                    int64_t cur_offset = std::get<1>(*rit);
                     auto acessInst = std::get<0>(*rit);
                     if (isa<LoadInst>(acessInst))
-                        accessSize = unsigned(DL->getTypeSizeInBits(acessInst->getType())) / 8;
+                        accessSize = int64_t(DL->getTypeSizeInBits(acessInst->getType())) / 8;
                     else
-                        accessSize = unsigned(DL->getTypeSizeInBits(acessInst->getOperand(0)->getType())) / 8;
-                    if ((cur_offset - firstOffset + accessSize) > 4)
+                        accessSize = int64_t(DL->getTypeSizeInBits(acessInst->getOperand(0)->getType())) / 8;
+                    mergedSize = cur_offset - firstOffset + accessSize;
+                    if (mergedSize > 4)
                         AccessIntrs.pop_back();
+                    else
+                        break;
                 }
 
                 if (AccessIntrs.size() < 2)
@@ -245,8 +274,7 @@ namespace {
 
                 // Need to subtract the last offset by the first offset and add one to
                 // get the new size of the vector
-                auto lastOffset = std::get<1>(AccessIntrs[AccessIntrs.size() - 1]);
-                NumElts = unsigned(lastOffset - firstOffset) + 1;
+                NumElts = unsigned(mergedSize / scalarTypeSizeInBytes);
             }
             return true;
         }
@@ -673,8 +701,7 @@ bool MemOpt::mergeLoad(LoadInst* LeadingLoad,
 
     // Start to merge loads.
 
-    assert(NumElts > 1
-        && "It's expected to merge into at least 2-element vector!");
+    IGC_ASSERT_MESSAGE(1 < NumElts, "It's expected to merge into at least 2-element vector!");
 
     // Try to find the profitable vector length first.
     unsigned s = LoadsToMerge.size();
@@ -710,9 +737,7 @@ bool MemOpt::mergeLoad(LoadInst* LeadingLoad,
     // new pointer.
     LoadInst* FirstLoad = std::get<0>(LoadsToMerge.front());
     int64_t FirstOffset = std::get<1>(LoadsToMerge.front());
-    assert(FirstOffset <= 0 &&
-        "The 1st load should be either the leading load or "
-        "load with smaller offset!");
+    IGC_ASSERT_MESSAGE(FirstOffset <= 0, "The 1st load should be either the leading load or load with smaller offset!");
 
     // Next we need to check alignment
     if (!checkAlignmentBeforeMerge(FirstLoad, LoadsToMerge, NumElts))
@@ -730,7 +755,8 @@ bool MemOpt::mergeLoad(LoadInst* LeadingLoad,
     if (FirstOffset < 0) {
         // If the first load is not the leading load, re-calculate the pointer
         // from the pointer of the leading load.
-        assert(FirstOffset % LdScalarSize == 0 && "Remainder is expected to be 0!");
+        IGC_ASSERT(LdScalarSize);
+        IGC_ASSERT_MESSAGE(FirstOffset % LdScalarSize == 0, "Remainder is expected to be 0!");
 
         Value* Idx = Builder.getInt64(FirstOffset / LdScalarSize);
         Type* Ty =
@@ -746,12 +772,12 @@ bool MemOpt::mergeLoad(LoadInst* LeadingLoad,
             Ptr = Builder.CreateGEP(Ptr, Idx);
     }
 
-    Type* NewLoadType = VectorType::get(LeadingLoadScalarType, NumElts);
+    Type* NewLoadType = IGCLLVM::FixedVectorType::get(LeadingLoadScalarType, NumElts);
     Type* NewPointerType =
         PointerType::get(NewLoadType, LeadingLoad->getPointerAddressSpace());
     Value* NewPointer = Builder.CreateBitCast(Ptr, NewPointerType);
     LoadInst* NewLoad =
-        Builder.CreateAlignedLoad(NewPointer, FirstLoad->getAlignment());
+        Builder.CreateAlignedLoad(NewPointer, IGCLLVM::getAlign(FirstLoad->getAlignment()));
     NewLoad->setDebugLoc(LeadingLoad->getDebugLoc());
 
     // Unpack the load value to their uses. For original vector loads, extracting
@@ -763,7 +789,7 @@ bool MemOpt::mergeLoad(LoadInst* LeadingLoad,
     for (auto& I : LoadsToMerge) {
         Type* Ty = std::get<0>(I)->getType();
         Type* ScalarTy = Ty->getScalarType();
-        assert(hasSameSize(ScalarTy, LeadingLoadScalarType));
+        IGC_ASSERT(hasSameSize(ScalarTy, LeadingLoadScalarType));
 
         mdLoadInv = std::get<0>(I)->getMetadata(LLVMContext::MD_invariant_load);
         if (!mdLoadInv)
@@ -774,16 +800,15 @@ bool MemOpt::mergeLoad(LoadInst* LeadingLoad,
         Pos = unsigned((std::get<1>(I) - FirstOffset) / LdScalarSize);
 
         if (Ty->isVectorTy()) {
-            if (Pos + Ty->getVectorNumElements() > NumElts) {
+            if (Pos + cast<VectorType>(Ty)->getNumElements() > NumElts) {
                 // This implies we're trying to extract an element from our new load
-                // with an index > the size of the new load.  This shouldn't happen,
-                // but we'll generate correct code if it does since we don't remove the
+                // with an index > the size of the new load.  If this happens,
+                // we'll generate correct code if it does since we don't remove the
                 // original load for this element.
-                assert(0 && "Trying to merge a load with an offset bigger than the load");
                 continue;
             }
             Value* Val = UndefValue::get(Ty);
-            for (unsigned i = 0, e = Ty->getVectorNumElements(); i != e; ++i) {
+            for (unsigned i = 0, e = (unsigned)cast<VectorType>(Ty)->getNumElements(); i != e; ++i) {
                 Value* Ex = Builder.CreateExtractElement(NewLoad, Builder.getInt32(Pos + i));
                 Ex = createBitOrPointerCast(Ex, ScalarTy, Builder);
                 Val = Builder.CreateInsertElement(Val, Ex, Builder.getInt32(i));
@@ -792,7 +817,6 @@ bool MemOpt::mergeLoad(LoadInst* LeadingLoad,
         }
         else {
             if (Pos > NumElts) {
-                assert(0 && "Trying to merge a load with an offset bigger than the load");
                 continue;
             }
             Value* Val = Builder.CreateExtractElement(NewLoad,
@@ -1014,8 +1038,7 @@ bool MemOpt::mergeStore(StoreInst* LeadingStore,
         NumElts += getNumElements(Ty);
     }
 
-    assert(NumElts > 1 &&
-        "It's expected to merge into at least 2-element vector!");
+    IGC_ASSERT_MESSAGE(1 < NumElts, "It's expected to merge into at least 2-element vector!");
 
     // Try to find the profitable vector length first.
     unsigned s = StoresToMerge.size();
@@ -1040,7 +1063,16 @@ bool MemOpt::mergeStore(StoreInst* LeadingStore,
     StoresToMerge.resize(s);
     std::sort(StoresToMerge.begin(), StoresToMerge.end(), less_tuple<1>());
 
-    Type* NewStoreType = VectorType::get(LeadingStoreScalarType, NumElts);
+    // Stores to be merged will be merged into the tailing store. However, the
+    // pointer from the first store (with the minimal offset) will be used as the
+    // new pointer.
+    StoreInst* FirstStore = std::get<0>(StoresToMerge.front());
+
+    // Next we need to check alignment
+    if (!checkAlignmentBeforeMerge(FirstStore, StoresToMerge, NumElts))
+        return false;
+
+    Type* NewStoreType = IGCLLVM::FixedVectorType::get(LeadingStoreScalarType, NumElts);
     Value* NewStoreVal = UndefValue::get(NewStoreType);
 
     // Pack the store value from their original store values. For original vector
@@ -1051,10 +1083,10 @@ bool MemOpt::mergeStore(StoreInst* LeadingStore,
         Value* Val = std::get<0>(I)->getValueOperand();
         Type* Ty = Val->getType();
         Type* ScalarTy = Ty->getScalarType();
-        assert(hasSameSize(ScalarTy, LeadingStoreScalarType));
+        IGC_ASSERT(hasSameSize(ScalarTy, LeadingStoreScalarType));
 
         if (Ty->isVectorTy()) {
-            for (unsigned i = 0, e = Ty->getVectorNumElements(); i != e; ++i) {
+            for (unsigned i = 0, e = (unsigned)cast<VectorType>(Ty)->getNumElements(); i != e; ++i) {
                 Value* Ex = Builder.CreateExtractElement(Val, Builder.getInt32(i));
                 Ex = createBitOrPointerCast(Ex, LeadingStoreScalarType, Builder);
                 NewStoreVal = Builder.CreateInsertElement(NewStoreVal, Ex,
@@ -1088,15 +1120,6 @@ bool MemOpt::mergeStore(StoreInst* LeadingStore,
         }
     }
 
-    // Stores to be merged will be merged into the tailing store. However, the
-    // pointer from the first store (with the minimal offset) will be used as the
-    // new pointer.
-    StoreInst* FirstStore = std::get<0>(StoresToMerge.front());
-
-    // Next we need to check alignment
-    if (!checkAlignmentBeforeMerge(FirstStore, StoresToMerge, NumElts))
-        return false;
-
     if (!DebugCounter::shouldExecute(MergeStoreCounter))
         return false;
 
@@ -1109,7 +1132,7 @@ bool MemOpt::mergeStore(StoreInst* LeadingStore,
         Builder.CreateBitCast(FirstStore->getPointerOperand(), NewPointerType);
     StoreInst* NewStore =
         Builder.CreateAlignedStore(NewStoreVal, NewPointer,
-            FirstStore->getAlignment());
+            IGCLLVM::getAlign(FirstStore->getAlignment()));
     NewStore->setDebugLoc(TailingStore->getDebugLoc());
 
     // Replace the list to be optimized with the new store.
@@ -1119,13 +1142,28 @@ bool MemOpt::mergeStore(StoreInst* LeadingStore,
     for (auto& I : StoresToMerge) {
         StoreInst* ST = cast<StoreInst>(std::get<0>(I));
         Value* Ptr = ST->getPointerOperand();
+        // Stores merged in the previous iterations can get merged again, so we need
+        // to update ToOpt vector to avoid null instruction in there
+        ToOpt.erase(std::remove(ToOpt.begin(), ToOpt.end(), ST), ToOpt.end());
         ST->eraseFromParent();
         RecursivelyDeleteTriviallyDeadInstructions(Ptr);
-        // Mark it as already merged.
-        std::get<2>(I)->first = nullptr;
-        // Checking zero distance is intentionally omitted here due to the first
-        // memory access won't be able to be checked again.
-        std::get<2>(I)->second -= 1;
+
+        if (std::get<2>(I)->first == TailingStore)
+            // Writing NewStore to MemRefs for correct isSafeToMergeLoad working.
+            // For example if MemRefs contains this sequence: S1, S2, S3, L5, L6, L7, S4, L4
+            // after stores merge MemRefs contains : L5, L6, L7, S1234, L4 and loads are
+            // merged to L567, final instructions instructions sequence is L567, S1234, L4.
+            // Otherwise the sequence could be merged to sequence L4567, S1234 with
+            // unordered L4,S4 accesses.
+            std::get<2>(I)->first = NewStore;
+        else {
+            // Mark it as already merged.
+            std::get<2>(I)->first = nullptr;
+            // Checking zero distance is intentionally omitted here due to the first
+            // memory access won't be able to be checked again.
+            std::get<2>(I)->second -= 1;
+        }
+
     }
 
     return true;
@@ -1295,11 +1333,14 @@ bool MemOpt::canonicalizeGEP64(Instruction* I) const {
             Value* RHS = BinOp->getOperand(1);
             LHS = Builder.CreateCast(CastOpcode, LHS, IdxTy);
             RHS = Builder.CreateCast(CastOpcode, RHS, IdxTy);
-            auto BO = cast<BinaryOperator>(Builder.CreateBinOp(BinOpcode, LHS, RHS));
-            if (BinOp->hasNoUnsignedWrap())
-                BO->setHasNoUnsignedWrap();
-            if (BinOp->hasNoSignedWrap())
-                BO->setHasNoSignedWrap();
+            auto BO = Builder.CreateBinOp(BinOpcode, LHS, RHS);
+            // BO can be a constant if both sides are constants
+            if (auto BOP = dyn_cast<BinaryOperator>(BO)) {
+                if (BinOp->hasNoUnsignedWrap())
+                    BOP->setHasNoUnsignedWrap();
+                if (BinOp->hasNoSignedWrap())
+                    BOP->setHasNoSignedWrap();
+            }
             U->set(BO);
             RecursivelyDeleteTriviallyDeadInstructions(ExtOp);
             Changed = true;
@@ -1397,7 +1438,9 @@ Value*
 SymbolicPointer::getLinearExpression(Value* V, APInt& Scale, APInt& Offset,
     ExtensionKind& Extension, unsigned Depth,
     const DataLayout* DL) {
-    assert(V->getType()->isIntegerTy() && "Not an integer value");
+    IGC_ASSERT(nullptr != V);
+    IGC_ASSERT(nullptr != V->getType());
+    IGC_ASSERT_MESSAGE(V->getType()->isIntegerTy(), "Not an integer value");
 
     // Limit our recursion depth.
     if (Depth == 16) {
