@@ -84,6 +84,14 @@ namespace IGC
         // Creates appropriate annotation based on the kernel arg
         void CreateAnnotations(IGC::KernelArg* kernelArg, uint payloadPosition);
 
+        // Fill SOpenCLKernelInfo::m_zePayloadArgs
+        // Return true: if the argument is supported in ZEBinary and it's created successfully
+        // Return false: if the argument cannot be supported by ZEBinary
+        bool CreateZEPayloadArguments(IGC::KernelArg* kernelArg, uint payloadPosition);
+
+        // a helper function to get image type from kernelArg
+        iOpenCL::IMAGE_MEMORY_OBJECT_TYPE getImageTypeFromKernelArg(const KernelArg& kernelArg);
+
         // Creates annotations for inline sampler_t objects
         void CreateInlineSamplerAnnotations();
 
