@@ -171,7 +171,6 @@ namespace IGC
         void visitFPToUIInst(llvm::FPToUIInst& I);
         void visitDbgInfoIntrinsic(llvm::DbgInfoIntrinsic& I);
         void visitExtractValueInst(llvm::ExtractValueInst& I);
-        void visitInsertValueInst(llvm::InsertValueInst& I);
         void visitBranchInst(llvm::BranchInst& I);
 
     public:
@@ -205,13 +204,15 @@ namespace IGC
 
         Pattern* Match(llvm::Instruction& inst);
         bool MatchAbsNeg(llvm::Instruction& I);
-        bool MatchSatModifier(llvm::Instruction& I);
+        bool MatchFloatingPointSatModifier(llvm::Instruction& I);
+        bool MatchIntegerSatModifier(llvm::Instruction& I);
         bool MatchPredicate(llvm::SelectInst& I);
         bool MatchSelectModifier(llvm::SelectInst& I);
         bool MatchPow(llvm::IntrinsicInst& I);
         bool MatchCondModifier(llvm::CmpInst& I);
         bool MatchBoolOp(llvm::BinaryOperator& I);
         bool MatchRotate(llvm::Instruction& I);
+        bool MatchDp4a(llvm::GenIntrinsicInst& I);
         bool MatchLogicAlu(llvm::BinaryOperator& I);
         bool MatchRsqrt(llvm::BinaryOperator& I);
         bool MatchLoadStorePointer(llvm::Instruction& I, llvm::Value& ptrVal);
@@ -229,7 +230,7 @@ namespace IGC
         std::tuple<llvm::Value*, unsigned, VISA_Type> isFPToIntegerSatWithExactConstant(llvm::CastInst* I);
         std::tuple<llvm::Value*, unsigned, VISA_Type> isFPToSignedIntSatWithInexactConstant(llvm::SelectInst* I);
         std::tuple<llvm::Value*, unsigned, VISA_Type> isFPToUnsignedIntSatWithInexactConstant(llvm::SelectInst* I);
-        bool MatchIntegerSatModifier(llvm::SelectInst& I);
+        bool MatchIntegerTruncSatModifier(llvm::SelectInst& I);
         std::tuple<llvm::Value*, bool, bool> isIntegerSatTrunc(llvm::SelectInst*);
 
         bool MatchSIToFPZExt(llvm::SIToFPInst* S2FI);
@@ -239,8 +240,7 @@ namespace IGC
         bool matchMulPair(llvm::ExtractValueInst*);
         bool matchPtrToPair(llvm::ExtractValueInst*);
 
-        bool MatchCopyToStruct(llvm::InsertValueInst*);
-        bool MatchCopyFromStruct(llvm::ExtractValueInst*);
+        bool MatchUnmaskedRegionBoundary(llvm::Instruction& I, bool start);
 
         void AddPattern(Pattern* P)
         {
@@ -293,13 +293,6 @@ namespace IGC
         typedef llvm::DenseMap<llvm::Value*, PairOutputTy> PairOutputMapTy;
         PairOutputMapTy PairOutputMap;
 
-        // Maps the final insertvalue instruction to a struct type to:
-        // The initial value of the struct
-        // A vector of sources and the corresponding index into the struct
-        // [InsertValueInst] -> [Constant Init Value][vector<[Source][Idx]>]
-        typedef llvm::DenseMap<llvm::InsertValueInst*, std::pair<llvm::Constant*, std::vector<std::pair<SSource, unsigned>>>> StructValueInsertMapTy;
-        StructValueInsertMapTy StructValueInsertMap;
-
         llvm::Instruction* m_root;
         Pattern* m_currentPattern;
 
@@ -334,7 +327,7 @@ namespace IGC
     bool IsNegate(llvm::BinaryOperator& modifier, llvm::Value*& negateSource);
     bool IsZero(llvm::Value* v);
     bool isAbs(llvm::Value* abs, e_modifier& mod, llvm::Value*& source);
-    bool isSat(llvm::Instruction* sat, llvm::Value*& source);
+    bool isSat(llvm::Instruction* sat, llvm::Value*& source, bool& isUnsigned);
     bool isOne(llvm::Value* zero);
     bool isMinOrMax(llvm::Value* inst, llvm::Value*& source0, llvm::Value*& source1, bool& isMin, bool& isUnsigned);
     bool isCandidateForConstantPool(llvm::Value * val);
