@@ -23,19 +23,18 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 
 ======================= end_copyright_notice ==================================*/
+
 #include "common/LLVMWarningsPush.hpp"
+#include "llvmWrapper/IR/DerivedTypes.h"
 #include <llvm/IR/Module.h>
 #include <llvm/IR/Function.h>
 #include <llvm/IR/Instructions.h>
 #include "common/LLVMWarningsPop.hpp"
-
 #include "common/LLVMUtils.h"
-
 #include "LLVMWarningsPush.hpp"
 #include "llvm/IR/Instruction.h"
 #include "llvm/IR/Function.h"
 #include "LLVMWarningsPop.hpp"
-
 #include "Compiler/CISACodeGen/GenCodeGenModule.h"
 #include "Compiler/CISACodeGen/PushAnalysis.hpp"
 #include "Compiler/CISACodeGen/ShaderCodeGen.hpp"
@@ -46,10 +45,9 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "Compiler/IGCPassSupport.h"
 #include "LLVM3DBuilder/MetadataBuilder.h"
 #include "Compiler/MetaDataUtilsWrapper.h"
-
 #include "common/debug/Debug.hpp"
-
 #include <list>
+#include "Probe/Assertion.h"
 
 /***********************************************************************************
 This File contains the logic to decide for each inputs and constant if the data
@@ -136,16 +134,10 @@ namespace IGC
 
     bool PushAnalysis::IsStatelessCBLoad(
         llvm::Instruction* inst,
-        unsigned int& GRFOffset,
+        int& pushableAddressGrfOffset,
+        int& pushableOffsetGrfOffset,
         unsigned int& offset)
     {
-        /*
-            %12 = call i64 @llvm.genx.GenISA.RuntimeValue(i32 2)
-            %13 = add i64 %12, 16
-            %14 = inttoptr i64 %13 to <3 x float> addrspace(2)*
-            %15 = load <3 x float> addrspace(2)* %14, align 16
-        */
-
         if (!llvm::isa<llvm::LoadInst>(inst))
             return false;
 
@@ -169,7 +161,8 @@ namespace IGC
         if (GenIntrinsicInst * genIntr = dyn_cast<GenIntrinsicInst>(pAddress))
         {
             /*
-            find 64 bit case
+            Examples of patterns matched on platforms without 64bit type support:
+            1. Pushable 64bit address + immediate offset case:
             %29 = call { i32, i32 } @llvm.genx.GenISA.ptr.to.pair.p2v3f32(<3 x float> addrspace(2)* %runtime_value_6)
             %30 = extractvalue { i32, i32 } %29, 1
             %31 = extractvalue { i32, i32 } %29, 0
@@ -182,6 +175,8 @@ namespace IGC
             %37 = extractvalue { i32, i32 } %36, 0
             %38 = extractvalue { i32, i32 } %36, 1
             %39 = call <2 x i32> addrspace(2)* @llvm.genx.GenISA.pair.to.ptr.p2v2i32(i32 %37, i32 %38)
+            2. TODO: add support for pushable 64bit address + pushable 32bit offset + immediate
+               offset pattern.
             */
             if (genIntr->getIntrinsicID() == GenISAIntrinsic::GenISA_pair_to_ptr)
             {
@@ -219,76 +214,186 @@ namespace IGC
                     }
                 }
             }
+            if (IsPushableAddress(
+                inst,
+                pAddress,
+                pushableAddressGrfOffset,
+                pushableOffsetGrfOffset))
+            {
+                IGC_ASSERT(pushableAddressGrfOffset >= 0);
+                IGC_ASSERT_MESSAGE(pushableOffsetGrfOffset == -1, "Pushable 32bit offset not supported yet!");
+                return true;
+            }
         }
         else
         {
             /*
-            find 32 bit case
-            % 33 = ptrtoint <4 x float> addrspace(2)* %runtime_value_4 to i64
-            % 34 = add i64 % 33, 368
-            % chunkPtr = inttoptr i64 % 34 to <2 x i32> addrspace(2)*
+            Examples of patterns matched on platforms with 64bit type support:
+            1. Pushable 64bit address + immediate offset case:
+               %33 = ptrtoint <4 x float> addrspace(2)* %runtime_value_4 to i64
+               %34 = add i64 %33, 368
+               %chunkPtr = inttoptr i64 % 34 to <2 x i32> addrspace(2)*
+
+            2. Pushable 64bit address + pushable 32bit offset + immediate
+               offset case:
+               %0 = call i32 @llvm.genx.GenISA.RuntimeValue.i32(i32 3)
+               %1 = add i32 %0, 4
+               %2 = call i64 @llvm.genx.GenISA.RuntimeValue.i64(i32 1)
+               %3 = zext i32 %1 to i64
+               %4 = add i64 %2, %3
+               %5 = inttoptr i64 %4 to i8 addrspace(1507329)* addrspace(2)*
             */
 
             offset = 0;
-            // % 13 = add i64 % 12, 16
-            // This add might or might not be present necessarily.
-            if (BinaryOperator * pAdd = dyn_cast<BinaryOperator>(pAddress))
+            SmallVector<Value*, 4> potentialPushableAddresses;
+            std::function<void(Value*)> GetPotentialPushableAddresses;
+            GetPotentialPushableAddresses = [&potentialPushableAddresses, &offset, &GetPotentialPushableAddresses](
+                Value* pAddress)->void
             {
-                if (pAdd->getOpcode() == llvm::Instruction::Add)
+                BinaryOperator* pAdd = dyn_cast<BinaryOperator>(pAddress);
+                if (pAdd && pAdd->getOpcode() == llvm::Instruction::Add)
                 {
-                    ConstantInt* pConst = dyn_cast<llvm::ConstantInt>(pAdd->getOperand(1));
-                    if (!pConst)
-                        return false;
-
-                    pAddress = pAdd->getOperand(0);
-                    offset = (uint)pConst->getZExtValue();
+                    GetPotentialPushableAddresses(pAdd->getOperand(0));
+                    GetPotentialPushableAddresses(pAdd->getOperand(1));
                 }
-            }
+                else if (isa<ZExtInst>(pAddress))
+                {
+                    GetPotentialPushableAddresses(
+                        cast<ZExtInst>(pAddress)->getOperand(0));
+                }
+                else if (isa<ConstantInt>(pAddress))
+                {
+                    ConstantInt* pConst = cast<ConstantInt>(pAddress);
+                    offset += int_cast<uint>(pConst->getZExtValue());
+                }
+                else
+                {
+                    potentialPushableAddresses.push_back(pAddress);
+                }
+            };
 
+            GetPotentialPushableAddresses(pAddress);
+            if (potentialPushableAddresses.size() == 1 ||
+                potentialPushableAddresses.size() == 2)
+            {
+                for (Value* potentialAddress : potentialPushableAddresses)
+                {
+                    bool isPushable = IsPushableAddress(
+                        inst,
+                        potentialAddress,
+                        pushableAddressGrfOffset,
+                        pushableOffsetGrfOffset);
+                    if (!isPushable)
+                    {
+                        return false;
+                    }
+                }
+                IGC_ASSERT(pushableAddressGrfOffset >= 0);
+                return true;
+            }
         }
+
+        return false;
+    }
+
+
+    bool PushAnalysis::IsPushableAddress(
+        llvm::Instruction* inst,
+        llvm::Value* pAddress,
+        int& pushableAddressGrfOffset,
+        int& pushableOffsetGrfOffset) const
+    {
         // skip casts
-        while (1)
+        while (
+            isa<IntToPtrInst>(pAddress) ||
+            isa<PtrToIntInst>(pAddress) ||
+            isa<BitCastInst>(pAddress))
         {
-            if (isa<IntToPtrInst>(pAddress) || isa<PtrToIntInst>(pAddress) || isa<BitCastInst>(pAddress))
-            {
-                pAddress = cast<Instruction>(pAddress)->getOperand(0);
-            }
-            else
-            {
-                break;
-            }
+            pAddress = cast<Instruction>(pAddress)->getOperand(0);
         }
 
         llvm::GenIntrinsicInst* pRuntimeVal = llvm::dyn_cast<llvm::GenIntrinsicInst>(pAddress);
+        uint runtimeval0;
 
-        if (pRuntimeVal == nullptr ||
-            pRuntimeVal->getIntrinsicID() != llvm::GenISAIntrinsic::GenISA_RuntimeValue)
+        bool isRuntimeValFound = false;
+        if (pRuntimeVal == nullptr)
+        {
+            auto it = std::find(m_argList.begin(), m_argList.end(), pAddress);
+            if (it != m_argList.end())
+            {
+                int argIndex = (int) std::distance(m_argList.begin(), it);
+                PushInfo& pushInfo = m_context->getModuleMetaData()->pushInfo;
+                for (auto index_it = pushInfo.constantReg.begin(); index_it != pushInfo.constantReg.end(); ++index_it)
+                {
+                    if (index_it->second == argIndex)
+                    {
+                        runtimeval0 = index_it->first;
+                        isRuntimeValFound = true;
+                    }
+                }
+            }
+            if (!isRuntimeValFound)
+                return false;
+        }
+        else if(pRuntimeVal->getIntrinsicID() != llvm::GenISAIntrinsic::GenISA_RuntimeValue)
             return false;
+        if (!isRuntimeValFound)
+            runtimeval0 = (uint)llvm::cast<llvm::ConstantInt>(pRuntimeVal->getOperand(0))->getZExtValue();
+
+        uint runtimevalSize = GetSizeInBits(pAddress->getType());
+        IGC_ASSERT(32 == runtimevalSize ||
+            64 == runtimevalSize);
+        const bool is64Bit = 64 == runtimevalSize;
 
-        uint runtimeval0 = (uint)llvm::cast<llvm::ConstantInt>(pRuntimeVal->getOperand(0))->getZExtValue();
         PushInfo& pushInfo = m_context->getModuleMetaData()->pushInfo;
 
         // then check for static flag so that we can do push safely
         for (auto it : pushInfo.pushableAddresses)
         {
-            if ((runtimeval0 * 4 == it.addressOffset) && (IGC_IS_FLAG_ENABLED(DisableStaticCheck) || it.isStatic))
+            if (runtimeval0 * 4 != it.addressOffset)
             {
-                GRFOffset = runtimeval0;
-                return true;
+                continue;
+            }
+
+            if (IGC_IS_FLAG_ENABLED(DisableStaticCheck) ||
+                it.isStatic ||
+                IsSafeToPushNonStaticBufferLoad(inst))
+            {
+                // only a single 64bit pushable address and a single 32bit
+                // pushable offset is supported.
+                if (is64Bit && pushableAddressGrfOffset == -1)
+                {
+                    pushableAddressGrfOffset = runtimeval0;
+                    return true;
+                }
+                else if (!is64Bit && pushableOffsetGrfOffset == -1)
+                {
+                    pushableOffsetGrfOffset = runtimeval0;
+                    return true;
+                }
             }
         }
 
-        // otherwise the descriptor could bound to uninitialized buffer and we
-        // need to avoid pushing in control flow
-        // Find the return BB or the return BB before discard lowering.
 
+        return false;
+    }
+
+    bool PushAnalysis::IsSafeToPushNonStaticBufferLoad(llvm::Instruction* inst) const
+    {
+        // Find the return BB or the return BB before discard lowering.
         bool searchForRetBBBeforeDiscard = false;
         BasicBlock* retBB = m_PDT->getRootNode()->getBlock();
         if (!retBB)
         {
+#if LLVM_VERSION_MAJOR <= 10
             auto& roots = m_PDT->getRoots();
-            assert(roots.size() == 1 && "Unexpected multiple roots");
+            IGC_ASSERT_MESSAGE(roots.size() == 1, "Unexpected multiple roots");
             retBB = roots[0];
+#else
+            auto roots = m_PDT->root_begin();
+            IGC_ASSERT_MESSAGE(m_PDT->root_size() == 1, "Unexpected multiple roots");
+            retBB = *roots;
+#endif
         }
 
         for (auto& II : m_pFunction->getEntryBlock())
@@ -316,14 +421,7 @@ namespace IGC
 
         if (m_DT->dominates(inst->getParent(), retBB))
         {
-            for (auto it : pushInfo.pushableAddresses)
-            {
-                if (runtimeval0 * 4 == it.addressOffset)
-                {
-                    GRFOffset = runtimeval0;
-                    return true;
-                }
-            }
+            return true;
         }
 
         return false;
@@ -401,8 +499,8 @@ namespace IGC
             const Instruction* andInst = cast<Instruction>(offsetValue);
             ConstantInt* src1 = dyn_cast<ConstantInt>(andInst->getOperand(1));
             if (src1 &&
-                (int_cast<uint>(src1->getZExtValue()) == 0xFFFFFFE0 || int_cast<uint>(src1->getZExtValue()) == 0xFFFFFFF0) &&
-                isa<BitCastInst>(andInst->getOperand(0)))
+                (int_cast<uint>(src1->getZExtValue()) == 0xFFFFFFE0 ||
+                 int_cast<uint>(src1->getZExtValue()) == 0xFFFFFFF0))
             {
                 uint offset = 0;
                 if (GetConstantOffsetForDynamicUniformBuffer(bufferId, andInst->getOperand(0), offset) &&
@@ -436,33 +534,30 @@ namespace IGC
         }
         else if (BitCastInst * bitCast = dyn_cast<BitCastInst>(offsetValue))
         {
-            if (GenIntrinsicInst * genIntr = dyn_cast<GenIntrinsicInst>(bitCast->getOperand(0)))
-            {
-                if (genIntr->getIntrinsicID() == GenISAIntrinsic::GenISA_RuntimeValue)
-                {
-                    if (MDNode * bufIdMd = genIntr->getMetadata("dynamicBufferOffset.bufferId"))
-                    {
-                        ConstantInt* bufIdMdVal = mdconst::extract<ConstantInt>(bufIdMd->getOperand(0));
-                        if (bufferId == int_cast<uint>(bufIdMdVal->getZExtValue()))
-                        {
-                            relativeOffsetInBytes = 0;
-                            return true;
-                        }
-                    }
-                }
-            }
+            return GetConstantOffsetForDynamicUniformBuffer(bufferId, bitCast->getOperand(0), relativeOffsetInBytes);
         }
         else if (IntToPtrInst * i2p = dyn_cast<IntToPtrInst>(offsetValue))
         {
             return GetConstantOffsetForDynamicUniformBuffer(bufferId, i2p->getOperand(0), relativeOffsetInBytes);
         }
+        else if (m_dynamicBufferOffsetArgs.find(offsetValue) != m_dynamicBufferOffsetArgs.end())
+        {
+            relativeOffsetInBytes = 0;
+            return true;
+        }
 
         return false;
     }
 
     /// The constant-buffer id and element id must be compile-time immediate
     /// in order to be added to thread-payload
-    bool PushAnalysis::IsPushableShaderConstant(Instruction* inst, uint& bufIdOrGRFOffset, uint& eltId, bool& isStateless)
+    bool PushAnalysis::IsPushableShaderConstant(
+        Instruction* inst,
+        unsigned int& cbIdx,
+        int& pushableAddressGrfOffset,
+        int& pushableOffsetGrfOffset,
+        unsigned int& eltId,
+        bool& isStateless)
     {
         Value* eltPtrVal = nullptr;
         if (!llvm::isa<llvm::LoadInst>(inst))
@@ -470,21 +565,21 @@ namespace IGC
 
         if (inst->getType()->isVectorTy())
         {
-            if (!(inst->getType()->getVectorElementType()->isFloatTy() ||
-                inst->getType()->getVectorElementType()->isIntegerTy(32)))
+            if (!(cast<VectorType>(inst->getType())->getElementType()->isFloatTy() ||
+                cast<VectorType>(inst->getType())->getElementType()->isIntegerTy(32)))
                 return false;
         }
         else
         {
-            if (!(inst->getType()->isFloatTy() || inst->getType()->isIntegerTy(32)))
+            if (!(inst->getType()->isFloatTy() || inst->getType()->isIntegerTy(32) || inst->getType()->isPointerTy()))
                 return false;
         }
 
         // \todo, not support vector-load yet
-        if (IsLoadFromDirectCB(inst, bufIdOrGRFOffset, eltPtrVal))
+        if (IsLoadFromDirectCB(inst, cbIdx, eltPtrVal))
         {
             isStateless = false;
-            if (bufIdOrGRFOffset == getAnalysis<MetaDataUtilsWrapper>().getModuleMetaData()->pushInfo.inlineConstantBufferSlot)
+            if (cbIdx == getAnalysis<MetaDataUtilsWrapper>().getModuleMetaData()->pushInfo.inlineConstantBufferSlot)
             {
                 return false;
             }
@@ -509,7 +604,7 @@ namespace IGC
             if (m_context->m_DriverInfo.SupportsDynamicUniformBuffers() && IGC_IS_FLAG_DISABLED(DisableSimplePushWithDynamicUniformBuffers))
             {
                 unsigned int relativeOffsetInBytes = 0; // offset in bytes starting from dynamic buffer offset
-                if (GetConstantOffsetForDynamicUniformBuffer(bufIdOrGRFOffset, eltPtrVal, relativeOffsetInBytes))
+                if (GetConstantOffsetForDynamicUniformBuffer(cbIdx, eltPtrVal, relativeOffsetInBytes))
                 {
                     eltId = relativeOffsetInBytes;
                     if ((eltId % 4) == 0)
@@ -520,8 +615,14 @@ namespace IGC
             }
             return false;
         }
-        else if (IsStatelessCBLoad(inst, bufIdOrGRFOffset, eltId))
+        else if (IsStatelessCBLoad(
+            inst,
+            pushableAddressGrfOffset,
+            pushableOffsetGrfOffset,
+            eltId))
         {
+            IGC_ASSERT(pushableAddressGrfOffset >= 0);
+            cbIdx = 0;
             isStateless = true;
             return true;
         }
@@ -670,11 +771,31 @@ namespace IGC
         return 0;
     }
 
-    unsigned int PushAnalysis::AllocatePushedConstant(
-        Instruction* load, unsigned int cbIdxOrGRFOffset, unsigned int offset, unsigned int maxSizeAllowed, bool isStateless)
+    unsigned int PushAnalysis::GetSizeInBits(Type* type) const
     {
-        unsigned int size = (unsigned int)load->getType()->getPrimitiveSizeInBits() / 8;
-        assert(isa<LoadInst>(load) && "Expected a load instruction");
+        unsigned int size = (unsigned int)type->getPrimitiveSizeInBits();
+        if (type->isPointerTy())
+        {
+            size = m_DL->getPointerSizeInBits(type->getPointerAddressSpace());
+        }
+        return size;
+    }
+
+    unsigned int PushAnalysis::AllocatePushedConstant(
+        Instruction* load,
+        unsigned int cbIdx,
+        int pushableAddressGrfOffset,
+        int pushableOffsetGrfOffset,
+        unsigned int offset,
+        unsigned int maxSizeAllowed,
+        bool isStateless)
+    {
+        if (cbIdx > m_context->m_DriverInfo.MaximumSimplePushBufferID())
+        {
+            return 0;
+        }
+        unsigned int size = GetSizeInBits(load->getType()) / 8;
+        IGC_ASSERT_MESSAGE(isa<LoadInst>(load), "Expected a load instruction");
         PushInfo& pushInfo = m_context->getModuleMetaData()->pushInfo;
 
         bool canPromote = false;
@@ -682,20 +803,31 @@ namespace IGC
         // greedy allocation for now
         // first check if we are already pushing from the buffer
         unsigned int piIndex;
-        bool cbIdxFound = false;
+        bool regionFound = false;
 
         //For stateless buffers, we store the GRFOffset into the simplePushInfoArr[piIndex].cbIdx
         //For stateful buffers, we store the bufferId into the simplePushInfoArr[piIndex].cbIdx
         //We traverse the array and identify which CB we're in based on either GFXOffset or bufferId
         for (piIndex = 0; piIndex < pushInfo.simplePushBufferUsed; piIndex++)
         {
-            if (pushInfo.simplePushInfoArr[piIndex].cbIdx == cbIdxOrGRFOffset)
+            const SimplePushInfo& info = pushInfo.simplePushInfoArr[piIndex];
+            if (info.isStateless == isStateless &&
+                info.isStateless == false &&
+                info.cbIdx == cbIdx)
             {
-                cbIdxFound = true;
+                regionFound = true;
+                break;
+            }
+            else if (info.isStateless == isStateless &&
+                info.isStateless == true &&
+                info.pushableAddressGrfOffset == pushableAddressGrfOffset &&
+                info.pushableOffsetGrfOffset == pushableOffsetGrfOffset)
+            {
+                regionFound = true;
                 break;
             }
         }
-        if (cbIdxFound)
+        if (regionFound)
         {
             unsigned int newStartOffset = iSTD::RoundDown(std::min(offset, pushInfo.simplePushInfoArr[piIndex].offset), getGRFSize());
             unsigned int newEndOffset = iSTD::Round(std::max(offset + size, pushInfo.simplePushInfoArr[piIndex].offset + pushInfo.simplePushInfoArr[piIndex].size), getGRFSize());
@@ -728,10 +860,20 @@ namespace IGC
                 sizeGrown = newSize;
 
                 piIndex = pushInfo.simplePushBufferUsed;
-                pushInfo.simplePushInfoArr[piIndex].cbIdx = cbIdxOrGRFOffset;
-                pushInfo.simplePushInfoArr[piIndex].offset = newStartOffset;
-                pushInfo.simplePushInfoArr[piIndex].size = newSize;
-                pushInfo.simplePushInfoArr[piIndex].isStateless = isStateless;
+                SimplePushInfo& info = pushInfo.simplePushInfoArr[piIndex];
+                if (isStateless)
+                {
+                    info.pushableAddressGrfOffset = pushableAddressGrfOffset;
+                    info.pushableOffsetGrfOffset = pushableOffsetGrfOffset;
+                }
+                else
+                {
+                    info.cbIdx = cbIdx;
+                }
+
+                info.offset = newStartOffset;
+                info.size = newSize;
+                info.isStateless = isStateless;
                 pushInfo.simplePushBufferUsed++;
             }
         }
@@ -748,16 +890,22 @@ namespace IGC
     {
         unsigned int num_elms = 1;
         llvm::Type* pTypeToPush = load->getType();
+        if (pTypeToPush->isPointerTy())
+        {
+            pTypeToPush = IntegerType::get(
+                load->getContext(),
+                m_DL->getPointerSizeInBits(pTypeToPush->getPointerAddressSpace()));
+        }
         llvm::Value* pReplacedInst = nullptr;
         llvm::Type* pScalarTy = pTypeToPush;
 
         if (pTypeToPush->isVectorTy())
         {
-            num_elms = pTypeToPush->getVectorNumElements();
-            pTypeToPush = pTypeToPush->getVectorElementType();
-            llvm::Type* pVecTy = llvm::VectorType::get(pTypeToPush, num_elms);
+            num_elms = (unsigned)cast<VectorType>(pTypeToPush)->getNumElements();
+            pTypeToPush = cast<VectorType>(pTypeToPush)->getElementType();
+            llvm::Type* pVecTy = IGCLLVM::FixedVectorType::get(pTypeToPush, num_elms);
             pReplacedInst = llvm::UndefValue::get(pVecTy);
-            pScalarTy = pVecTy->getVectorElementType();
+            pScalarTy = cast<VectorType>(pVecTy)->getElementType();
         }
 
         SmallVector< SmallVector<ExtractElementInst*, 1>, 4> extracts(num_elms);
@@ -771,7 +919,7 @@ namespace IGC
             if (it != info.simplePushLoads.end())
             {
                 // Value is already getting pushed
-                assert((it->second <= m_argIndex) && "Function arguments list and metadata are out of sync!");
+                IGC_ASSERT_MESSAGE((it->second <= m_argIndex), "Function arguments list and metadata are out of sync!");
                 value = m_argList[it->second];
                 if (pTypeToPush != value->getType())
                     value = CastInst::CreateZExtOrBitCast(value, pTypeToPush, "", load);
@@ -802,6 +950,15 @@ namespace IGC
             }
             else
             {
+                if (load->getType()->isPointerTy())
+                {
+                    value = IntToPtrInst::Create(
+                        Instruction::IntToPtr,
+                        value,
+                        load->getType(),
+                        load->getName(),
+                        load);
+                }
                 pReplacedInst = value;
             }
 
@@ -815,8 +972,24 @@ namespace IGC
 
     void PushAnalysis::BlockPushConstants()
     {
-        // push up to 31 GRF
-        static const unsigned int cthreshold = m_pullConstantHeuristics->getPushConstantThreshold(m_pFunction) * getGRFSize();
+        auto& inputs = m_context->getModuleMetaData()->pushInfo.inputs;
+        typedef const std::map<unsigned int, SInputDesc>::value_type& inputPairType;
+        unsigned int largestIndex = 0;
+        if (m_context->m_DriverInfo.EnableSimplePushRestriction())
+        {
+            auto largestPair = std::max_element(inputs.begin(), inputs.end(),
+                [](inputPairType a, inputPairType b) { return a.second.index < b.second.index; });
+            largestIndex = largestPair != inputs.end() ? largestPair->second.index : 0;
+        }
+        const unsigned int maxPushedGRFs = 96;
+        if (largestIndex >= maxPushedGRFs)
+        {
+            return;
+        }
+        // push up to 31 GRFs of constants
+        // The maximum number of GRFs used for all pushed data is 96.
+        const unsigned int cthreshold =
+            std::min(m_pullConstantHeuristics->getPushConstantThreshold(m_pFunction), maxPushedGRFs - largestIndex) * getGRFSize();
         unsigned int sizePushed = 0;
         m_entryBB = &m_pFunction->getEntryBlock();
 
@@ -825,13 +998,27 @@ namespace IGC
         {
             for (auto i = bb->begin(), ie = bb->end(); i != ie; ++i)
             {
-                unsigned int cbIdOrGRFOffset = 0;
+                unsigned int cbIdx = 0;
+                int pushableAddressGrfOffset = -1;
+                int pushableOffsetGrfOffset = -1;
                 unsigned int offset = 0;
                 bool isStateless = false;
-                if (IsPushableShaderConstant(&(*i), cbIdOrGRFOffset, offset, isStateless))
+                if (IsPushableShaderConstant(
+                    &(*i),
+                    cbIdx,
+                    pushableAddressGrfOffset,
+                    pushableOffsetGrfOffset,
+                    offset,
+                    isStateless))
                 {
-                    // convert offset in bytes
-                    sizePushed += AllocatePushedConstant(&(*i), cbIdOrGRFOffset, offset, cthreshold - sizePushed, isStateless);
+                    sizePushed += AllocatePushedConstant(
+                        &(*i),
+                        cbIdx,
+                        pushableAddressGrfOffset,
+                        pushableOffsetGrfOffset,
+                        offset,
+                        cthreshold - sizePushed,
+                        isStateless);
                 }
             }
         }
@@ -839,31 +1026,59 @@ namespace IGC
 
     PushConstantMode PushAnalysis::GetPushConstantMode()
     {
-        PushConstantMode pushConstantMode = PushConstantMode::NO_PUSH_CONSTANT;
-        if (CanPushConstants())
+        PushConstantMode pushConstantMode = PushConstantMode::DEFAULT;
+
+        if(!CanPushConstants())
         {
-            if (IGC_IS_FLAG_ENABLED(forcePushConstantMode))
+            // Hardware can not do push constant mode or the registry has been set in a way to disable push altogether
+            pushConstantMode = PushConstantMode::NONE;
+        }
+        else
+        {
+            // Priority order of how the push constant mode is determined:
+            //   1.) Registry Keys
+            //   2.) Compiler Input
+            //   3.) Default Logic dependent on platform and attributes of the shader
+
+            // 1.) Check registry keys
+            if(pushConstantMode == PushConstantMode::DEFAULT)
             {
-                pushConstantMode = (PushConstantMode)IGC_GET_FLAG_VALUE(forcePushConstantMode);
+                if(IGC_IS_FLAG_ENABLED(forcePushConstantMode))
+                {
+                    pushConstantMode = (PushConstantMode)IGC_GET_FLAG_VALUE(forcePushConstantMode);
+                }
             }
-            else if (m_context->m_DriverInfo.SupportsSimplePushOnly())
+            
+            // 2.) Check compiler input
+            if (pushConstantMode == PushConstantMode::DEFAULT)
             {
-                pushConstantMode = PushConstantMode::SIMPLE_PUSH;
+                pushConstantMode = m_context->m_pushConstantMode;
             }
-            else if (m_context->platform.supportsHardwareResourceStreamer())
+
+            // 3.) Default Logic dependent on platform and attributes of the shader
+            if (pushConstantMode == PushConstantMode::DEFAULT)
             {
-                pushConstantMode = PushConstantMode::GATHER_CONSTANT;
-            }
-            else if (m_context->m_DriverInfo.SupportsHWResourceStreameAndSimplePush())
-            {
-                pushConstantMode = PushConstantMode::SIMPLE_PUSH;
-            }
-            else
-            {
-                //CPU copy
-                pushConstantMode = PushConstantMode::GATHER_CONSTANT;
+                if (m_context->m_DriverInfo.SupportsSimplePushOnly())
+                {
+                    pushConstantMode = PushConstantMode::SIMPLE;
+                }
+                else if (m_context->platform.supportsHardwareResourceStreamer() || m_context->m_DriverInfo.SupportsGatherConstantOnly())
+                {
+                    pushConstantMode = PushConstantMode::GATHER;
+                }
+                else if (m_context->m_DriverInfo.SupportsHWResourceStreameAndSimplePush())
+                {
+                    pushConstantMode = PushConstantMode::SIMPLE;
+                }
+                else
+                {
+                    // CPU copy
+                    pushConstantMode = PushConstantMode::GATHER;
+                }
             }
         }
+
+        IGC_ASSERT_MESSAGE(pushConstantMode != PushConstantMode::DEFAULT, "GetPushConstantMode should resolve a non-DEFAULT mode!");
         return pushConstantMode;
     }
 
@@ -888,8 +1103,8 @@ namespace IGC
             return;
         }
 
-        unsigned int num_elms =
-            inst->getType()->isVectorTy() ? inst->getType()->getVectorNumElements() : 1;
+        unsigned num_elms =
+            inst->getType()->isVectorTy() ? (unsigned)cast<VectorType>(inst->getType())->getNumElements() : 1;
         llvm::Type* pTypeToPush = inst->getType();
         llvm::Value* replaceVector = nullptr;
         unsigned int numberChannelReplaced = 0;
@@ -898,7 +1113,7 @@ namespace IGC
         if (inst->getType()->isVectorTy())
         {
             allExtract = LoadUsedByConstExtractOnly(cast<LoadInst>(inst), extracts);
-            pTypeToPush = inst->getType()->getVectorElementType();
+            pTypeToPush = cast<VectorType>(inst->getType())->getElementType();
         }
 
         for (unsigned int i = 0; i < num_elms; ++i)
@@ -936,8 +1151,7 @@ namespace IGC
                 }
                 else
                 {
-                    assert((it->second <= m_argIndex) &&
-                        "Function arguments list and metadata are out of sync!");
+                    IGC_ASSERT_MESSAGE((it->second <= m_argIndex), "Function arguments list and metadata are out of sync!");
                     value = m_argList[it->second];
                     if (pTypeToPush != value->getType())
                         value = CastInst::CreateZExtOrBitCast(value, pTypeToPush, "", inst);
@@ -997,7 +1211,7 @@ namespace IGC
                 auto it = pushInfo.inputs.find(input.index);
                 if (it == pushInfo.inputs.end())
                 {
-                    assert(inst->getType()->isHalfTy() || inst->getType()->isFloatTy());
+                    IGC_ASSERT(inst->getType()->isHalfTy() || inst->getType()->isFloatTy());
                     llvm::Type* floatTy = Type::getFloatTy(m_pFunction->getContext());
                     addArgumentAndMetadata(floatTy, VALUE_NAME(std::string("input_") + to_string(input.index)), uniformInput ? WIAnalysis::UNIFORM : WIAnalysis::RANDOM);
                     input.argIndex = m_argIndex;
@@ -1005,7 +1219,7 @@ namespace IGC
                 }
                 else
                 {
-                    assert((it->second.argIndex <= m_argIndex) && "Function arguments list and metadata are out of sync!");
+                    IGC_ASSERT_MESSAGE((it->second.argIndex <= m_argIndex), "Function arguments list and metadata are out of sync!");
                     input.argIndex = it->second.argIndex;
                 }
                 llvm::Value* replacementValue = m_argList[input.argIndex];
@@ -1057,7 +1271,7 @@ namespace IGC
                             }
                             else
                             {
-                                assert((it->second.argIndex <= m_argIndex) && "Function arguments list and metadata are out of sync!");
+                                IGC_ASSERT_MESSAGE((it->second.argIndex <= m_argIndex), "Function arguments list and metadata are out of sync!");
                                 input.argIndex = it->second.argIndex;
                             }
 
@@ -1070,7 +1284,7 @@ namespace IGC
             {
                 // This should never happen for geometry shader since we leave GS specific
                 // intrinsic if we want pull model earlier in GS lowering.
-                assert(m_context->type != ShaderType::GEOMETRY_SHADER);
+                IGC_ASSERT(m_context->type != ShaderType::GEOMETRY_SHADER);
             }
         }
     }
@@ -1136,7 +1350,7 @@ namespace IGC
                             }
                             else
                             {
-                                assert((it->second.argIndex <= m_argIndex) && "Function arguments list and metadata are out of sync!");
+                                IGC_ASSERT_MESSAGE((it->second.argIndex <= m_argIndex), "Function arguments list and metadata are out of sync!");
                                 input.argIndex = it->second.argIndex;
                             }
                             extract->replaceAllUsesWith(m_argList[input.argIndex]);
@@ -1147,7 +1361,7 @@ namespace IGC
         }
     }
 
-    // process runtimve value, update PushInfo.constantReg
+    // process runtime value, update PushInfo.constantReg
     void PushAnalysis::processRuntimeValue(GenIntrinsicInst* intrinsic)
     {
         PushInfo& pushInfo = m_context->getModuleMetaData()->pushInfo;
@@ -1169,11 +1383,19 @@ namespace IGC
                 VALUE_NAME(std::string("runtime_value_") + to_string(index)),
                 WIAnalysis::UNIFORM);
             pushInfo.constantReg[index] = m_argIndex;
+
+            if (m_context->m_DriverInfo.SupportsDynamicUniformBuffers() &&
+                IGC_IS_FLAG_DISABLED(DisableSimplePushWithDynamicUniformBuffers) &&
+                pushInfo.dynamicBufferInfo.numOffsets > 0 &&
+                index >= pushInfo.dynamicBufferInfo.firstIndex &&
+                index < pushInfo.dynamicBufferInfo.firstIndex + pushInfo.dynamicBufferInfo.numOffsets)
+            {
+                m_dynamicBufferOffsetArgs.insert(arg);
+            }
         }
         else
         {
-            assert((it->second <= m_argIndex) &&
-                "Function arguments list and metadata are out of sync!");
+            IGC_ASSERT_MESSAGE(it->second <= m_argIndex, "Function arguments list and metadata are out of sync!");
             arg = m_argList[it->second];
             while (arg->getType() != runtimeValue->getType() &&
                 runtimeValue->hasOneUse() &&
@@ -1220,7 +1442,7 @@ namespace IGC
         }
 
         PushConstantMode pushConstantMode = GetPushConstantMode();
-
+        
         if (m_context->type == ShaderType::DOMAIN_SHADER)
         {
             auto valueU = addArgumentAndMetadata(Type::getFloatTy(m_pFunction->getContext()), VALUE_NAME("DS_U"), WIAnalysis::RANDOM);
@@ -1231,11 +1453,6 @@ namespace IGC
             m_dsProps->SetDomainPointWArgu(valueW);
         }
 
-        if (pushConstantMode == PushConstantMode::SIMPLE_PUSH)
-        {
-            BlockPushConstants();
-        }
-
         for (auto BB = m_pFunction->begin(), E = m_pFunction->end(); BB != E; ++BB)
         {
             llvm::BasicBlock::InstListType& instructionList = BB->getInstList();
@@ -1273,9 +1490,11 @@ namespace IGC
                 }
                 // code to push constant-buffer value into thread-payload
                 uint bufId, eltId;
+                int pushableAddressGrfOffset = -1;
+                int pushableOffsetGrfOffset = -1;
                 bool isStateless = false;
-                if (pushConstantMode == PushConstantMode::GATHER_CONSTANT &&
-                    IsPushableShaderConstant(inst, bufId, eltId, isStateless) &&
+                if (pushConstantMode == PushConstantMode::GATHER &&
+                    IsPushableShaderConstant(inst, bufId, pushableAddressGrfOffset, pushableOffsetGrfOffset, eltId, isStateless) &&
                     !isStateless)
                 {
                     processGather(inst, bufId, eltId);
@@ -1305,6 +1524,12 @@ namespace IGC
                 }
             }
         }
+
+        if (pushConstantMode == PushConstantMode::SIMPLE)
+        {
+            BlockPushConstants();
+        }
+
         // WA: Gen11+ HW doesn't work correctly if doubles are on vertex shader input and the input has unused components,
         // so ElementComponentEnableMask is not full => packing occurs
         // Code below fills gaps in inputs, so the ElementComponentEnableMask if full even if we don't use all
@@ -1407,7 +1632,7 @@ namespace IGC
                 funcsMapping[pFunc] = pNewFunc;
 
                 // This is a kernel function, so there should not be any call site
-                assert(pFunc->use_empty());
+                IGC_ASSERT(pFunc->use_empty());
             }
             m_pFuncUpgrade.Clean();
         }
@@ -1440,7 +1665,7 @@ namespace IGC
         {
             Function* pFunc = I->first;
 
-            assert(pFunc->use_empty() && "Assume all user function are inlined at this point");
+            IGC_ASSERT_MESSAGE(pFunc->use_empty(), "Assume all user function are inlined at this point");
 
             if (FGA) {
                 FGA->replaceEntryFunc(pFunc, I->second);
