@@ -25,12 +25,10 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 ======================= end_copyright_notice ==================================*/
 
 #include "common/LLVMUtils.h"
-
 #include "Compiler/CISACodeGen/SLMConstProp.hpp"
 #include "Compiler/CISACodeGen/ShaderCodeGen.hpp"
 #include "Compiler/CodeGenPublic.h"
 #include "Compiler/IGCPassSupport.h"
-
 #include "common/LLVMWarningsPush.hpp"
 #include <llvm/Analysis/LoopInfo.h>
 #include <llvm/Analysis/ScalarEvolution.h>
@@ -44,8 +42,8 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <llvm/Support/Debug.h>
 #include <llvm/Support/MathExtras.h>
 #include "common/LLVMWarningsPop.hpp"
-
 #include <vector>
+#include "Probe/Assertion.h"
 
 using namespace llvm;
 using namespace IGC;
@@ -148,7 +146,7 @@ namespace
 
 void SymbolicEvaluation::dump_symbols()
 {
-    assert((int)m_symInfos.size() <= m_nextValueID &&
+    IGC_ASSERT_MESSAGE((int)m_symInfos.size() <= m_nextValueID,
         "ValueInfoMap has the incorrect number of entries!");
 
     // for sorting symbols in increasing ID.
@@ -156,7 +154,8 @@ void SymbolicEvaluation::dump_symbols()
     for (auto II : m_symInfos) {
         Value* V = II.first;
         ValueSymInfo* VSI = II.second;
-        assert(VSI->ID < m_nextValueID && "Incorrect value ID!");
+        IGC_ASSERT(nullptr != VSI);
+        IGC_ASSERT_MESSAGE(VSI->ID < m_nextValueID, "Incorrect value ID!");
         Vals[VSI->ID] = V;
     }
 
@@ -165,6 +164,7 @@ void SymbolicEvaluation::dump_symbols()
         Value* V = Vals[i];
         // V should not be nullptr, but check it
         // for safety.
+        IGC_ASSERT(nullptr != V);
         if (V) {
             dbgs() << "\n    V"
                 << i
@@ -423,7 +423,7 @@ int SymbolicEvaluation::cmp(const SymProd* T0, const SymProd* T1)
             }
         }
         else {
-            assert(false && "V0 and/or V1 should be in the map!");
+            IGC_ASSERT_MESSAGE(0, "V0 and/or V1 should be in the map!");
         }
     }
     return 0;
@@ -637,7 +637,7 @@ bool SLMConstProp::isFloatType(Type* Ty)
         {
             return false;
         }
-        Ty = vTy->getVectorElementType();
+        Ty = cast<VectorType>(vTy)->getElementType();
     }
     return Ty->isFloatTy();
 }
@@ -696,8 +696,7 @@ bool SLMConstProp::find_PATTERN_REPEAT_WITH_STRIDE(PatternInfo& aPattern)
         }
     }
 
-    assert(ix_1st != -1 &&
-        "Constant store expected, but not found!");
+    IGC_ASSERT_MESSAGE(-1 != ix_1st, "constant store expected but not found");
 
     SymExpr* basePtr = m_storeInfos[0].SE;
     StoreInst* lastSI = m_storeInsts[m_storeInfos[nStores - 1].ix];
@@ -1211,4 +1210,4 @@ bool SLMConstProp::runOnFunction(Function& F)
     }
 #endif
     return true;
-}
\ No newline at end of file
+}
