@@ -23,12 +23,14 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 
 ======================= end_copyright_notice ==================================*/
+
 #include "Compiler/IGCPassSupport.h"
 #include "common/LLVMWarningsPush.hpp"
 #include "llvm/IR/InstIterator.h"
 #include "common/LLVMWarningsPop.hpp"
 #include "VertexShaderLowering.hpp"
 #include <array>
+#include "Probe/Assertion.h"
 
 using namespace IGC::IGCMD;
 using namespace llvm;
@@ -89,6 +91,12 @@ namespace IGC
     {
         unsigned int index = bInsertAfterLastUsedSlot ? GetUnusedInputSlotAFterLastUsedOne() : GetUnusedInputSlot();
         m_inputUsed[index] = true;
+        InsertSGVRead(sgv, index);
+        return index;
+    }
+
+    void VertexShaderLowering::InsertSGVRead(Instruction* sgv, uint index)
+    {
         uint usage =
             (uint)llvm::cast<llvm::ConstantInt>(sgv->getOperand(0))->getZExtValue();
 
@@ -121,10 +129,8 @@ namespace IGC
         Value* channel = ConstantInt::get(Type::getInt32Ty(sgv->getContext()), index % 4);
         Instruction* newExt = ExtractElementInst::Create(urbRead, channel, "", sgv);
         sgv->replaceAllUsesWith(newExt);
-        return index;
     }
 
-
     unsigned int VertexShaderLowering::GetUnusedInputSlot()
     {
         for (unsigned int i = 0; i < ARRAY_COUNT(m_inputUsed); i++)
@@ -134,7 +140,7 @@ namespace IGC
                 return i;
             }
         }
-        assert(0 && "All input slots are already used, cannot find an empty one");
+        IGC_ASSERT_MESSAGE(0, "All input slots are already used, cannot find an empty one");
         return 32 * 4 - 1;
     }
 
@@ -149,7 +155,7 @@ namespace IGC
             }
         }
 
-        assert(idx < ARRAY_COUNT(m_inputUsed) && "All input slots are already used, cannot find an empty one");
+        IGC_ASSERT_MESSAGE(idx < ARRAY_COUNT(m_inputUsed), "All input slots are already used, cannot find an empty one");
         return idx;
     }
 
@@ -268,6 +274,7 @@ namespace IGC
                             };
                             AddURBWrite(offset, 0xF, data, inst);
                         }
+                        m_isHeaderPresent = hasHeaderData;
                         instructionToRemove.push_back(inst);
                     }
                     else if (IID == GenISAIntrinsic::GenISA_DCL_ShaderInputVec)
@@ -324,7 +331,8 @@ namespace IGC
                         case XP0:
                         case XP1:
                         case XP2:
-                            assert(m_context->platform.supportsDrawParametersSGVs());
+                            IGC_ASSERT(m_context->platform.supportsDrawParametersSGVs() || 
+                                m_context->m_DriverInfo.UsesVertexBuffersToSendShaderDrawParameters());
                             vertexFetchSGVExtendedParameters.at(usage - XP0) = inst;
                             break;
 
@@ -339,27 +347,104 @@ namespace IGC
         {
             AddURBWrite(headerOffset, 0xF, headerData, prevInst);
         }
-        if (vertexId)
+        if (m_context->m_DriverInfo.UsesVertexBuffersToSendShaderDrawParameters())
         {
-            unsigned int slot = InsertInEmptySlot(vertexId);
-            m_vsPropsPass->SetVertexIdSlot(slot);
-        }
-        for (size_t paramIndex = 0; paramIndex < vertexFetchSGVExtendedParameters.size(); ++paramIndex)
-        {
-            if (vertexFetchSGVExtendedParameters[paramIndex])
+            llvm::Instruction* baseVertex = vertexFetchSGVExtendedParameters[0];
+            llvm::Instruction* baseInstance = vertexFetchSGVExtendedParameters[1];
+            llvm::Instruction* drawIndex = vertexFetchSGVExtendedParameters[2];
+
+            if (baseVertex || baseInstance || drawIndex || vertexId || InstanceId)
             {
-                unsigned int slot = InsertInEmptySlot(vertexFetchSGVExtendedParameters[paramIndex]);
-                m_vsPropsPass->SetShaderDrawParameter(paramIndex, slot);
+                // Find first free location at the end i.e. after all user inputs. 
+                unsigned int drawParametersIndex = (ARRAY_COUNT(m_inputUsed) - 1);
+                for (int index = drawParametersIndex; index >= 0; --index)
+                {
+                    if (m_inputUsed[index] == true)
+                    {
+                        drawParametersIndex = ((index + 4) / 4) * 4;
+                        break;
+                    }
+                    if (index == 0)
+                    {
+                        drawParametersIndex = 0;
+                    }
+                }
+
+                /// UMD has to limit the number of user inputs in order 
+                /// to be sure there are 2 free inputs at the end.
+                assert(drawParametersIndex < (ARRAY_COUNT(m_inputUsed) - 8));
+
+                if (baseVertex || baseInstance)
+                {
+                    /// First VB contains base vertex and base instance.
+                    /// UMD always sends both.
+                    m_inputUsed[drawParametersIndex] = true;
+                    m_inputUsed[drawParametersIndex + 1] = true;
+                    if (baseVertex)
+                    {
+                        InsertSGVRead(baseVertex, drawParametersIndex);
+                        m_vsPropsPass->SetShaderDrawParameter(0, drawParametersIndex);
+                    }
+                    if (baseInstance)
+                    {
+                        InsertSGVRead(baseInstance, drawParametersIndex + 1);
+                        m_vsPropsPass->SetShaderDrawParameter(1, drawParametersIndex + 1);
+                    }
+                }
+
+                if (vertexId)
+                {
+                    m_inputUsed[drawParametersIndex + 2] = true;
+                    InsertSGVRead(vertexId, drawParametersIndex + 2);
+                    m_vsPropsPass->SetVertexIdSlot(drawParametersIndex + 2);
+                }
+
+                if (InstanceId)
+                {
+                    m_inputUsed[drawParametersIndex + 3] = true;
+                    InsertSGVRead(InstanceId, drawParametersIndex + 3);
+                    m_vsPropsPass->SetInstanceIdSlot(drawParametersIndex + 3);
+                }
+
+                if (drawIndex)
+                {
+                    if (baseVertex || baseInstance || vertexId || InstanceId)
+                    {
+                        drawParametersIndex += 4;
+                    }
+                    /// Second VB contains draw index.
+                    m_inputUsed[drawParametersIndex] = true;
+                    InsertSGVRead(drawIndex, drawParametersIndex);
+                    m_vsPropsPass->SetShaderDrawParameter(2, drawParametersIndex);
+                }
             }
         }
-        if (InstanceId)
+        else
         {
-            unsigned int slot = InsertInEmptySlot(InstanceId, (!IGC_IS_FLAG_ENABLED(DisableMovingInstanceIDIndexOfVS)));
-            m_vsPropsPass->SetInstanceIdSlot(slot);
+            for (size_t paramIndex = 0; paramIndex < vertexFetchSGVExtendedParameters.size(); ++paramIndex)
+            {
+                if (vertexFetchSGVExtendedParameters[paramIndex])
+                {
+                    unsigned int slot = InsertInEmptySlot(vertexFetchSGVExtendedParameters[paramIndex]);
+                    m_vsPropsPass->SetShaderDrawParameter(paramIndex, slot);
+                }
+            }
+
+            if (vertexId)
+            {
+                unsigned int slot = InsertInEmptySlot(vertexId);
+                m_vsPropsPass->SetVertexIdSlot(slot);
+            }
+            if (InstanceId)
+            {
+                unsigned int slot = InsertInEmptySlot(InstanceId, (!IGC_IS_FLAG_ENABLED(DisableMovingInstanceIDIndexOfVS)));
+                m_vsPropsPass->SetInstanceIdSlot(slot);
+            }
         }
 
         //URB padding to 32Byte offsets
-        for (unsigned int i = 0; i < MaxNumOfOutput + m_headerSize.Count(); i++)
+        bool addURBPaddingTo32Bytes = true;
+        for (unsigned int i = 0; addURBPaddingTo32Bytes && i < MaxNumOfOutput + m_headerSize.Count(); i++)
         {
             //If not aligned to 32Byte offset and has valid data
             if (offsetInst[i])
@@ -426,7 +511,7 @@ namespace IGC
             }
             else
             {
-                assert(0 && "This case of indirect accessing of attribute  shouldn't be hit");
+                IGC_ASSERT_MESSAGE(0, "This case of indirect accessing of attribute  shouldn't be hit");
             }
         }
         case SHADER_OUTPUT_TYPE_POINTWIDTH:
@@ -438,7 +523,7 @@ namespace IGC
             m_vsPropsPass->DeclareRTAI();
             return 0;
         default:
-            assert(!"unknown VS output type");
+            IGC_ASSERT_MESSAGE(0, "unknown VS output type");
             break;
         }
         return 0;
@@ -495,7 +580,7 @@ namespace IGC
             offset
         };
         unsigned int inIndex = int_cast<unsigned int>(cast<ConstantInt>(offset)->getZExtValue());
-        assert(inIndex < MaxNumOfUserInputs);
+        IGC_ASSERT(inIndex < MaxNumOfUserInputs);
 
         Instruction* urbRead = GenIntrinsicInst::Create(
             GenISAIntrinsic::getDeclaration(m_module, GenISAIntrinsic::GenISA_URBRead),
@@ -558,7 +643,7 @@ namespace IGC
             m_isHeaderPresent = true;
             AddInitializedHeader(prev);
         }
-        assert(mask < 256 && "mask is an 8-bit bitmask and has to be in range 0..255");
+        IGC_ASSERT_MESSAGE(mask < 256, "mask is an 8-bit bitmask and has to be in range 0..255");
         Value* arguments[] =
         {
             offset,
@@ -667,7 +752,7 @@ namespace IGC
 
     void CollectVertexShaderProperties::SetShaderDrawParameter(size_t paramIndex, unsigned int slot)
     {
-        assert(paramIndex < ARRAY_COUNT(m_vsProps.m_VertexFetchSGVExtendedParameters.extendedParameters));
+        IGC_ASSERT(paramIndex < ARRAY_COUNT(m_vsProps.m_VertexFetchSGVExtendedParameters.extendedParameters));
         auto& parameter = m_vsProps.m_VertexFetchSGVExtendedParameters.extendedParameters[paramIndex];
         parameter.enabled = true;
         parameter.location = slot;
