@@ -25,7 +25,6 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 ======================= end_copyright_notice ==================================*/
 #pragma once
 
-
 #include "common/LLVMWarningsPush.hpp"
 #include <llvm/Analysis/AssumptionCache.h>
 #include <llvm/IR/Instruction.h>
@@ -50,6 +49,7 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "Compiler/MetaDataApi/MetaDataApi.h"
 #include "common/MDFrameWork.h"
 #include "common/Types.hpp"
+#include "Probe/Assertion.h"
 
 typedef unsigned int uint;
 
@@ -65,6 +65,8 @@ namespace IGC
     class CodeGenContext;
     struct SProgramOutput;
 
+    static const std::string INTEL_SYMBOL_TABLE_VOID_PROGRAM = "Intel_Symbol_Table_Void_Program";
+
 #ifdef _DEBUG
     template<typename T, size_t N>
     using smallvector = std::vector<T>;
@@ -106,6 +108,7 @@ namespace IGC
     bool IsMathIntrinsic(EOPCODE opcode);
     bool IsAtomicIntrinsic(EOPCODE opcode);
     bool IsGradientIntrinsic(EOPCODE opcode);
+    bool IsExtendedMathInstruction(llvm::Instruction* Inst);
     bool IsSubGroupIntrinsicWithSimd32Implementation(EOPCODE opcode);
     bool UsesTypedConstantBuffer(CodeGenContext* pContext);
 
@@ -158,7 +161,7 @@ namespace IGC
     bool GetGRFOffsetFromRTV(llvm::Value* pointerSrc, unsigned& GRFOffset);
     bool GetStatelessBufferInfo(llvm::Value* pointer, unsigned& bufIdOrGRFOffset, IGC::BufferType& bufferTy, llvm::Value*& bufferSrcPtr, bool& isDirectBuf);
     // try to evaluate the address if it is constant.
-    bool EvalConstantAddress(llvm::Value* address, unsigned int& offset, const llvm::DataLayout* pDL, llvm::Value* ptrSrc = nullptr);
+    bool EvalConstantAddress(llvm::Value* address, int& offset, const llvm::DataLayout* pDL, llvm::Value* ptrSrc = nullptr);
 
 
     bool isSampleLoadGather4InfoInstruction(llvm::Instruction* inst);
@@ -172,24 +175,33 @@ namespace IGC
     bool IsSIMDBlockIntrinsic(llvm::Instruction* inst);
     bool isSubGroupIntrinsic(const llvm::Instruction* I);
 
+    bool IsStatelessMemLoadIntrinsic(const llvm::GenIntrinsicInst& inst);
+    bool IsStatelessMemStoreIntrinsic(const llvm::GenIntrinsicInst& inst);
+    bool IsStatelessMemAtomicIntrinsic(const llvm::GenIntrinsicInst& inst);
+
     bool isURBWriteIntrinsic(const llvm::Instruction* inst);
 
+    llvm::Instruction* AdjustSystemValueCall(llvm::GenIntrinsicInst* inst);
+
     unsigned EncodeAS4GFXResource(
         const llvm::Value& bufIdx,
         BufferType bufType,
         unsigned uniqueIndAS);
 
     unsigned SetBufferAsBindless(unsigned addressSpaceOfPtr, BufferType bufferType);
+    bool isStatefulAddrSpace(unsigned AS);
 
     BufferType DecodeAS4GFXResource(unsigned addrSpace, bool& directIdx, unsigned& bufId);
     int getConstantBufferLoadOffset(llvm::LoadInst* ld);
 
+    bool isDummyBasicBlock(llvm::BasicBlock* BB);
+
     bool IsDirectIdx(unsigned addrSpace);
     bool isNaNCheck(llvm::FCmpInst& FC);
 
     inline bool IsBindless(BufferType t)
     {
-        return t == BINDLESS || t == BINDLESS_READONLY;
+        return t == BINDLESS || t == BINDLESS_CONSTANT_BUFFER || t == BINDLESS_TEXTURE;
     }
 
     bool IsUnsignedCmp(const llvm::CmpInst::Predicate Pred);
@@ -200,6 +212,15 @@ namespace IGC
     // isA64Ptr - Queries whether given pointer type requires 64-bit representation in vISA
     bool isA64Ptr(llvm::PointerType* PT, CodeGenContext* pContext);
 
+    inline llvm::Function* getIntelSymbolTableVoidProgram(llvm::Module* pM)
+    {
+        return pM->getFunction(INTEL_SYMBOL_TABLE_VOID_PROGRAM);
+    }
+    inline bool isIntelSymbolTableVoidProgram(llvm::Function* pF)
+    {
+        return (pF == getIntelSymbolTableVoidProgram(pF->getParent()));
+    }
+
     /// Return true if F is an entry function of a kernel or a shader.
     ///    A entry function must have an entry in FunctionInfoMetaData
     ///       with type KernelFunction;
@@ -213,7 +234,7 @@ namespace IGC
             return false;
 
         IGCMD::FunctionInfoMetaDataHandle Info = pM->getFunctionsInfoItem(F);
-        assert(Info->isTypeHasValue() && "FunctionInfoMetaData missing type!");
+        IGC_ASSERT_MESSAGE(Info->isTypeHasValue(), "FunctionInfoMetaData missing type!");
         return Info->getType() == FunctionTypeMD::KernelFunction;
     }
 
@@ -293,14 +314,17 @@ namespace IGC
         return &(*I);
     }
 
+    template <typename T>
     inline bool RTWriteHasSource0Alpha(
-        const llvm::RTWritIntrinsic* rtWrite,
+        const T* rtWrite,
         ModuleMetaData* md)
     {
-        return !llvm::isa<llvm::UndefValue>(rtWrite->getSource0Alpha());
+        return (nullptr != rtWrite->getSource0Alpha()) && !llvm::isa<llvm::UndefValue>(rtWrite->getSource0Alpha());
     }
+
+    template <typename T>
     inline bool DoesRTWriteSrc0AlphaBelongToHomogeneousPart(
-        const llvm::RTWritIntrinsic* rtWrite,
+        const T* rtWrite,
         ModuleMetaData* md)
     {
         return !rtWrite->hasMask() && RTWriteHasSource0Alpha(rtWrite, md);
@@ -348,13 +372,14 @@ namespace IGC
 
     bool DSDualPatchEnabled(class CodeGenContext* ctx);
 
+
     /// \brief Check whether inst precedes given position in one basic block
     inline bool isInstPrecede(
         const llvm::Instruction* inst,
         const llvm::Instruction* pos)
     {
         // must within same basic block
-        assert(inst->getParent() == pos->getParent());
+        IGC_ASSERT(inst->getParent() == pos->getParent());
         if (inst == pos)
         {
             return true;
@@ -388,15 +413,17 @@ namespace IGC
         case SIMDMode::SIMD16:  simdWidth = 16; break;
         case SIMDMode::SIMD32:  simdWidth = 32; break;
         default:
-            assert(false && "Invalid SIMD mode");
+            IGC_ASSERT_MESSAGE(0, "Invalid SIMD mode");
+            break;
         }
-        unsigned nThreadsPerTG = (threadGroupSize + simdWidth - 1) / simdWidth;
 
-        unsigned TGPerSubsliceNoSLM = hwThreadPerSubslice / nThreadsPerTG;
-        unsigned nTGDispatch = (slmSize == 0) ? TGPerSubsliceNoSLM : std::min(TGPerSubsliceNoSLM, slmSizePerSubSlice / slmSize);
-
-        float occupancy =
-            float(nTGDispatch * nThreadsPerTG) / float(hwThreadPerSubslice);
+        IGC_ASSERT(simdWidth);
+        const unsigned nThreadsPerTG = (threadGroupSize + simdWidth - 1) / simdWidth;
+        IGC_ASSERT(nThreadsPerTG);
+        const unsigned TGPerSubsliceNoSLM = hwThreadPerSubslice / nThreadsPerTG;
+        const unsigned nTGDispatch = (slmSize == 0) ? TGPerSubsliceNoSLM : std::min(TGPerSubsliceNoSLM, slmSizePerSubSlice / slmSize);
+        IGC_ASSERT(float(hwThreadPerSubslice));
+        const float occupancy = float(nTGDispatch * nThreadsPerTG) / float(hwThreadPerSubslice);
         return occupancy;
     }
 
@@ -436,4 +463,18 @@ namespace IGC
         }
     }
 
+    // Debug line info helper function
+    inline void updateDebugLoc(llvm::Instruction* pOrigin, llvm::Instruction* pNew)
+    {
+        IGC_ASSERT_MESSAGE(nullptr != pOrigin, "Expect valid instructions");
+        IGC_ASSERT_MESSAGE(nullptr != pNew, "Expect valid instructions");
+        pNew->setDebugLoc(pOrigin->getDebugLoc());
+    }
+
+    llvm::ConstantInt* getConstantSInt(llvm::IRBuilder<>& Builder, const int bitSize, int64_t val);
+    llvm::ConstantInt* getConstantUInt(llvm::IRBuilder<>& Builder, const int bitSize, uint64_t val);
+    llvm::Value* CreateMulhS64(llvm::IRBuilder<>& B, llvm::Value* const u, llvm::Value* const v);
+    llvm::Value* CreateMulhU64(llvm::IRBuilder<>& B, llvm::Value* const u, llvm::Value* const v);
+    llvm::Value* CreateMulh(llvm::Function& F, llvm::IRBuilder<>& B, const bool isSigned, llvm::Value* const u, llvm::Value* const v);
+
 } // namespace IGC
