@@ -283,6 +283,11 @@ namespace IGC
     \*****************************************************************************/
     static const unsigned int SLM_BTI = 254;
 
+    /*****************************************************************************\
+    CONST: STATELESS_NONCOHERENT_BTI
+    \*****************************************************************************/
+    static const unsigned int STATELESS_NONCOHERENT_BTI = 253;
+
     /*****************************************************************************\
     CONST: BINDLESS_BTI
     \*****************************************************************************/
