@@ -23,13 +23,14 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 
 ======================= end_copyright_notice ==================================*/
+
 #include "common/LLVMWarningsPush.hpp"
 #include <llvm/Support/ScaledNumber.h>
 #include "common/LLVMWarningsPop.hpp"
-
 #include "Compiler/CISACodeGen/ComputeShaderCodeGen.hpp"
 #include "Compiler/CISACodeGen/ShaderCodeGen.hpp"
 #include "Compiler/CodeGenPublic.h"
+#include "Probe/Assertion.h"
 
 namespace IGC
 {
@@ -54,7 +55,7 @@ namespace IGC
         memset(m_simdEntries, 0, sizeof(m_simdEntries));
         firstStateId = IGC_GET_FLAG_VALUE(RetryManagerFirstStateId);
         stateId = firstStateId;
-        assert(stateId < getStateCnt());
+        IGC_ASSERT(stateId < getStateCnt());
     }
 
     bool RetryManager::AdvanceState() {
@@ -62,32 +63,32 @@ namespace IGC
         {
             return false;
         }
-        assert(stateId < getStateCnt());
+        IGC_ASSERT(stateId < getStateCnt());
         stateId = RetryTable[stateId].nextState;
         return (stateId < getStateCnt());
     }
     bool RetryManager::AllowLICM() {
-        assert(stateId < getStateCnt());
+        IGC_ASSERT(stateId < getStateCnt());
         return RetryTable[stateId].allowLICM;
     }
     bool RetryManager::AllowPromotePrivateMemory() {
-        assert(stateId < getStateCnt());
+        IGC_ASSERT(stateId < getStateCnt());
         return RetryTable[stateId].allowPromotePrivateMemory;
     }
     bool RetryManager::AllowPreRAScheduler() {
-        assert(stateId < getStateCnt());
+        IGC_ASSERT(stateId < getStateCnt());
         return RetryTable[stateId].allowPreRAScheduler;
     }
     bool RetryManager::AllowCodeSinking() {
-        assert(stateId < getStateCnt());
+        IGC_ASSERT(stateId < getStateCnt());
         return RetryTable[stateId].allowCodeSinking;
     }
     bool RetryManager::AllowSimd32Slicing() {
-        assert(stateId < getStateCnt());
+        IGC_ASSERT(stateId < getStateCnt());
         return RetryTable[stateId].allowSimd32Slicing;
     }
     bool RetryManager::AllowLargeURBWrite() {
-        assert(stateId < getStateCnt());
+        IGC_ASSERT(stateId < getStateCnt());
         return RetryTable[stateId].allowLargeURBWrite;
     }
     bool RetryManager::IsFirstTry() {
@@ -121,7 +122,8 @@ namespace IGC
         case SIMDMode::SIMD16:  m_simdEntries[1] = shader;  break;
         case SIMDMode::SIMD32:  m_simdEntries[2] = shader;  break;
         default:
-            assert(false);
+            IGC_ASSERT(0);
+            break;
         }
     }
 
@@ -133,10 +135,11 @@ namespace IGC
         case SIMDMode::SIMD16:  return m_simdEntries[1];
         case SIMDMode::SIMD32:  return m_simdEntries[2];
         default:
-            assert(false);
+            IGC_ASSERT(0);
             return nullptr;
         }
     }
+
     RetryManager::~RetryManager()
     {
         for (unsigned i = 0; i < 3; i++)
@@ -168,7 +171,7 @@ namespace IGC
         }
         else
         {
-            assert(false && "TODO for other shader types");
+            IGC_ASSERT_MESSAGE(0, "TODO for other shader types");
             return true;
         }
     }
@@ -282,8 +285,8 @@ namespace IGC
                     return m_simdEntries[2];
                 }
                 // If SIMD32 doesn't spill, SIMD16 and SIMD8 shouldn't, if they exist
-                assert((m_simdEntries[0] == NULL) || simd8NoSpill == true);
-                assert((m_simdEntries[1] == NULL) || simd16NoSpill == true);
+                IGC_ASSERT((m_simdEntries[0] == NULL) || simd8NoSpill == true);
+                IGC_ASSERT((m_simdEntries[1] == NULL) || simd16NoSpill == true);
             }
 
             if (simd16NoSpill)
@@ -293,7 +296,7 @@ namespace IGC
                     simdMode = SIMDMode::SIMD16;
                     return m_simdEntries[1];
                 }
-                assert((m_simdEntries[0] == NULL) || simd8NoSpill == true); // If SIMD16 doesn't spill, SIMD8 shouldn't, if it exists
+                IGC_ASSERT_MESSAGE((m_simdEntries[0] == NULL) || simd8NoSpill == true, "If SIMD16 doesn't spill, SIMD8 shouldn't, if it exists");
             }
         }
 
@@ -382,8 +385,9 @@ namespace IGC
         {
             shader = static_cast<CComputeShader*>(
                 PickCSEntryFinally(simdMode));
-            assert(shader != nullptr);
+            IGC_ASSERT(shader != nullptr);
         }
+
         if (shader)
         {
             switch (simdMode)
@@ -391,22 +395,31 @@ namespace IGC
             case SIMDMode::SIMD8:
                 pKernelProgram->simd8 = *shader->ProgramOutput();
                 pKernelProgram->SimdWidth = USC::GFXMEDIA_GPUWALKER_SIMD8;
+                cgCtx->SetSIMDInfo(SIMD_SELECTED, simdMode,
+                    ShaderDispatchMode::NOT_APPLICABLE);
                 break;
 
             case SIMDMode::SIMD16:
                 pKernelProgram->simd16 = *shader->ProgramOutput();
                 pKernelProgram->SimdWidth = USC::GFXMEDIA_GPUWALKER_SIMD16;
+                cgCtx->SetSIMDInfo(SIMD_SELECTED, simdMode,
+                    ShaderDispatchMode::NOT_APPLICABLE);
                 break;
 
             case SIMDMode::SIMD32:
                 pKernelProgram->simd32 = *shader->ProgramOutput();
                 pKernelProgram->SimdWidth = USC::GFXMEDIA_GPUWALKER_SIMD32;
+                cgCtx->SetSIMDInfo(SIMD_SELECTED, simdMode,
+                    ShaderDispatchMode::NOT_APPLICABLE);
                 break;
 
             default:
-                assert(false && "Invalie SIMDMode");
+                IGC_ASSERT_MESSAGE(0, "Invalie SIMDMode");
+                break;
             }
             shader->FillProgram(pKernelProgram);
+            pKernelProgram->SIMDInfo = cgCtx->GetSIMDInfo();
+
 
             // free allocated memory for the remaining kernels
             FreeAllocatedMemForNotPickedCS(simdMode);
@@ -549,13 +562,13 @@ namespace IGC
         llvmCtxWrapper->AddRef();
     }
 
-    llvm::LLVMContext* CodeGenContext::getLLVMContext() {
+    llvm::LLVMContext* CodeGenContext::getLLVMContext() const {
         return llvmCtxWrapper;
     }
 
     IGC::IGCMD::MetaDataUtils* CodeGenContext::getMetaDataUtils()
     {
-        assert(m_pMdUtils && "Metadata Utils is not initialized");
+        IGC_ASSERT_MESSAGE(nullptr != m_pMdUtils, "Metadata Utils is not initialized");
         return m_pMdUtils;
     }
 
@@ -597,7 +610,7 @@ namespace IGC
 
     IGC::ModuleMetaData* CodeGenContext::getModuleMetaData() const
     {
-        assert(modMD && "Module Metadata is not initialized");
+        IGC_ASSERT_MESSAGE(nullptr != modMD, "Module Metadata is not initialized");
         return modMD;
     }
 
@@ -636,12 +649,7 @@ namespace IGC
 
     bool CodeGenContext::enableFunctionCall() const
     {
-        if (m_enableSubroutine || m_enableFunctionPointer)
-            return true;
-
-        int FCtrol = IGC_GET_FLAG_VALUE(FunctionControl);
-        return FCtrol == FLAG_FCALL_FORCE_SUBROUTINE ||
-            FCtrol == FLAG_FCALL_FORCE_STACKCALL;
+        return (m_enableSubroutine || m_enableStackCall || m_enableFunctionPointer);
     }
 
     void CodeGenContext::InitVarMetaData() {}
@@ -655,6 +663,7 @@ namespace IGC
     void CodeGenContext::clear()
     {
         m_enableSubroutine = false;
+        m_enableStackCall = false;
         m_enableFunctionPointer = false;
 
         delete modMD;
@@ -680,6 +689,11 @@ namespace IGC
         return;
     }
 
+    bool CodeGenContext::HasError() const
+    {
+        return !this->oclErrorMessage.empty();
+    }
+
     CompOptions& CodeGenContext::getCompilerOption()
     {
         return getModuleMetaData()->compOpt;
