@@ -30,6 +30,8 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 // and VerificationPass.cpp.
 //
 
+#include "llvm/Config/llvm-config.h"
+
 //=======================================================================================
 // TYPES
 
@@ -68,6 +70,7 @@ IGC_IR_VECTOR_TYPE(x32, 32)
 //------------------------------------
 IGC_IR_LLVM_INTRINSIC(sqrt)
 IGC_IR_LLVM_INTRINSIC(fabs)
+IGC_IR_LLVM_INTRINSIC(copysign)
 IGC_IR_LLVM_INTRINSIC(ceil)
 IGC_IR_LLVM_INTRINSIC(floor)
 IGC_IR_LLVM_INTRINSIC(trunc)
@@ -98,6 +101,8 @@ IGC_IR_LLVM_INTRINSIC(bswap)
 IGC_IR_LLVM_INTRINSIC(maxnum)
 IGC_IR_LLVM_INTRINSIC(minnum)
 IGC_IR_LLVM_INTRINSIC(canonicalize)
+IGC_IR_LLVM_INTRINSIC(stacksave)
+IGC_IR_LLVM_INTRINSIC(stackrestore)
 #if LLVM_VERSION_MAJOR >= 8
 IGC_IR_LLVM_INTRINSIC(fshl)
 IGC_IR_LLVM_INTRINSIC(fshr)
