@@ -30,16 +30,18 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "Compiler/CISACodeGen/helper.h"
 #include "Compiler/IGCPassSupport.h"
 #include "Compiler/MetaDataApi/MetaDataApi.h"
-
 #include "common/LLVMWarningsPush.hpp"
-#include <llvm/Support/CommandLine.h>
-#include <llvm/IR/Instructions.h>
-#include <llvm/IR/InstIterator.h>
-#include <llvm/Transforms/Utils/Local.h>
-#include <llvmWrapper/IR/InstrTypes.h>
+#include "llvm/Config/llvm-config.h"
+#include "llvmWrapper/IR/DerivedTypes.h"
+#include "llvmWrapper/IR/IRBuilder.h"
+#include "llvm/Support/CommandLine.h"
+#include "llvm/IR/Instructions.h"
+#include "llvm/IR/InstIterator.h"
+#include "llvm/Transforms/Utils/Local.h"
+#include "llvmWrapper/IR/InstrTypes.h"
 #include "common/LLVMWarningsPop.hpp"
-
 #include "GenISAIntrinsics/GenIntrinsicInst.h"
+#include "Probe/Assertion.h"
 
 using namespace llvm;
 using namespace IGC;
@@ -97,7 +99,7 @@ bool Legalization::runOnFunction(Function& F)
     {
         for (auto OI = I->op_begin(), OE = I->op_end(); OI != OE; ++OI)
         {
-            assert(!isa<ConstantExpr>(OI) && "Function must not contain constant expressions");
+            IGC_ASSERT_MESSAGE(!isa<ConstantExpr>(OI), "Function must not contain constant expressions");
         }
     }
 
@@ -256,7 +258,7 @@ void Legalization::visitBinaryOperator(llvm::BinaryOperator& I)
                     }
                     else if (BranchInst * br = dyn_cast<BranchInst>(*U))
                     {
-                        assert(br->isConditional());
+                        IGC_ASSERT(br->isConditional());
                         br->swapSuccessors();
                         br->setCondition(invert);
                     }
@@ -361,7 +363,7 @@ LegalizeGVNBitCastPattern(IRBuilder<>* Builder, const DataLayout* DL,
         return false;
     }
 
-    Type* EltTy = SrcTy->getVectorElementType();
+    Type* EltTy = cast<VectorType>(SrcTy)->getElementType();
     auto match1 = [=](Value* V, BinaryOperator*& BO, TruncInst*& TI,
         BitCastInst*& BI, int& Index)
     {
@@ -380,11 +382,14 @@ LegalizeGVNBitCastPattern(IRBuilder<>* Builder, const DataLayout* DL,
 
             // The shift amount shall be a multiple of base element.
             uint64_t ShAmt = CI->getZExtValue();
-            if (ShAmt % EltTy->getPrimitiveSizeInBits() != 0)
+            const unsigned int denominator = (const unsigned int)EltTy->getPrimitiveSizeInBits();
+            IGC_ASSERT(denominator);
+
+            if (ShAmt % denominator != 0)
                 return false;
 
             // Compute the index of the element to be extracted.
-            Index = int_cast<int>(ShAmt / EltTy->getPrimitiveSizeInBits());
+            Index = int_cast<int>(ShAmt / denominator);
         }
 
         // The second instruction is *not* optional.
@@ -508,7 +513,7 @@ LegalizeGVNBitCastPattern(IRBuilder<>* Builder, const DataLayout* DL,
 
             if (BI)
             {
-                assert(BI->getType() == EltTy);
+                IGC_ASSERT(BI->getType() == EltTy);
 
                 // BO, TI, and BI are dead.
                 BI->replaceAllUsesWith(V);
@@ -525,8 +530,7 @@ LegalizeGVNBitCastPattern(IRBuilder<>* Builder, const DataLayout* DL,
             }
             else
             {
-                assert(TI->getType()->getPrimitiveSizeInBits() ==
-                    EltTy->getPrimitiveSizeInBits());
+                IGC_ASSERT(TI->getType()->getPrimitiveSizeInBits() == EltTy->getPrimitiveSizeInBits());
                 if (V->getType() != TI->getType())
                     V = Builder->CreateBitCast(V, TI->getType());
 
@@ -578,11 +582,12 @@ LegalizeGVNBitCastPattern(IRBuilder<>* Builder, const DataLayout* DL,
             int dstSize = int_cast<int>(castType->getPrimitiveSizeInBits());
 
             // vecSize is 128/8 = 16 in above example
-            assert(srcSize % dstSize == 0);
+            IGC_ASSERT(dstSize);
+            IGC_ASSERT(srcSize % dstSize == 0);
             uint vecSize = srcSize / dstSize;
 
             Builder->SetInsertPoint(TI);
-            Value* BC = Builder->CreateBitCast(I.getOperand(0), VectorType::get(castType, vecSize));
+            Value* BC = Builder->CreateBitCast(I.getOperand(0), IGCLLVM::FixedVectorType::get(castType, vecSize));
             Value* EE = Builder->CreateExtractElement(BC, ConstantInt::get(Type::getInt32Ty(I.getContext()), Index));
 
             // BO and TI are dead
@@ -681,7 +686,7 @@ void Legalization::visitBitCastInst(llvm::BitCastInst& I)
             return;
 
         uint numElt = (unsigned int)pX->getType()->getPrimitiveSizeInBits() / (unsigned int)pEltTy->getPrimitiveSizeInBits();
-        auto* pBCType = VectorType::get(pEltTy, numElt);
+        auto* pBCType = IGCLLVM::FixedVectorType::get(pEltTy, numElt);
 
         SmallVector<uint32_t, 4> maskVals;
         for (uint i = 0; i < pVecTy->getNumElements(); i++)
@@ -732,14 +737,14 @@ void Legalization::visitSelectInst(SelectInst& I)
     }
     else if (I.getType()->isDoubleTy() &&
         (IGC_IS_FLAG_ENABLED(ForceDPEmulation) ||
-            !m_ctx->platform.supportFP64()))
+            m_ctx->platform.hasNoFP64Inst()))
     {
         // Split double select to i32 select.
 
         Value* lo[2];
         Value* hi[2];
         Type* intTy = Type::getInt32Ty(I.getContext());
-        VectorType* vec2Ty = VectorType::get(intTy, 2);
+        VectorType* vec2Ty = IGCLLVM::FixedVectorType::get(intTy, 2);
         Constant* Zero = ConstantInt::get(intTy, 0);
         Constant* One = ConstantInt::get(intTy, 1);
         m_builder->SetInsertPoint(&I);
@@ -762,7 +767,7 @@ void Legalization::visitSelectInst(SelectInst& I)
     }
     else if (I.getType()->isVectorTy())
     {
-        unsigned int vecSize = I.getType()->getVectorNumElements();
+        unsigned int vecSize = (unsigned)cast<VectorType>(I.getType())->getNumElements();
         Value* newVec = UndefValue::get(I.getType());
         m_builder->SetInsertPoint(&I);
         for (unsigned int i = 0; i < vecSize; i++)
@@ -813,8 +818,9 @@ static Value* GetMaskedValue(IRBuilder<>* IRB, bool Signed, Value* Src, Type* Ty
     IntegerType* SrcITy = dyn_cast<IntegerType>(Src->getType());
     IntegerType* ITy = dyn_cast<IntegerType>(Ty);
 
-    assert(SrcITy && ITy && SrcITy->getBitWidth() > ITy->getBitWidth() &&
-        "The source integer must be wider than the target integer.");
+    IGC_ASSERT_MESSAGE(SrcITy, "The source integer must be wider than the target integer.");
+    IGC_ASSERT_MESSAGE(ITy, "The source integer must be wider than the target integer.");
+    IGC_ASSERT_MESSAGE(SrcITy->getBitWidth() > ITy->getBitWidth(), "The source integer must be wider than the target integer.");
 
     if (!Signed) // For unsigned value, just mask off non-significant bits.
         return IRB->CreateAnd(Src, ITy->getBitMask());
@@ -1009,7 +1015,7 @@ void Legalization::visitFCmpInstUndorderedPredicate(FCmpInst& FC)
             }
             else if (BranchInst * br = dyn_cast<BranchInst>(*I))
             {
-                assert(br->isConditional());
+                IGC_ASSERT(br->isConditional());
                 br->swapSuccessors();
                 br->setCondition(invertedOrderedInst);
             }
@@ -1075,7 +1081,7 @@ CmpInst::Predicate getOrderedPredicate(CmpInst::Predicate pred)
     case CmpInst::FCMP_UGE: return CmpInst::FCMP_OGE;
     case CmpInst::FCMP_ULE: return CmpInst::FCMP_OLE;
     default:
-        assert(0 && "wrong predicate");
+        IGC_ASSERT_MESSAGE(0, "wrong predicate");
         break;
     }
     return pred;
@@ -1143,7 +1149,7 @@ void Legalization::visitStoreInst(StoreInst& I)
     if (ConstantDataVector * vec = dyn_cast<ConstantDataVector>(I.getOperand(0)))
     {
         Value* newVec = UndefValue::get(vec->getType());
-        unsigned int nbElement = vec->getType()->getVectorNumElements();
+        unsigned int nbElement = (unsigned)cast<VectorType>(vec->getType())->getNumElements();
         for (unsigned int i = 0; i < nbElement; i++)
         {
             Constant* cst = vec->getElementAsConstant(i);
@@ -1164,7 +1170,7 @@ void Legalization::visitStoreInst(StoreInst& I)
     else if (ConstantVector * vec = dyn_cast<ConstantVector>(I.getOperand(0)))
     {
         Value* newVec = UndefValue::get(vec->getType());
-        unsigned int nbElement = vec->getType()->getVectorNumElements();
+        unsigned int nbElement = (unsigned)cast<VectorType>(vec->getType())->getNumElements();
         for (unsigned int i = 0; i < nbElement; i++)
         {
             Constant* cst = vec->getOperand(i);
@@ -1185,7 +1191,7 @@ void Legalization::visitStoreInst(StoreInst& I)
     else if (ConstantAggregateZero * vec = dyn_cast<ConstantAggregateZero>(I.getOperand(0)))
     {
         Value* newVec = UndefValue::get(vec->getType());
-        unsigned int nbElement = vec->getType()->getVectorNumElements();
+        unsigned int nbElement = (unsigned)cast<VectorType>(vec->getType())->getNumElements();
         for (unsigned int i = 0; i < nbElement; i++)
         {
             Constant* cst = vec->getElementValue(i);
@@ -1237,11 +1243,14 @@ void Legalization::visitStoreInst(StoreInst& I)
         if (intSize == 0) // unaligned sizes not supported
             return;
 
-        Type* legalTy = VectorType::get(Type::getIntNTy(I.getContext(), intSize), srcWidth / intSize);
+        Type* legalTy = IGCLLVM::FixedVectorType::get(Type::getIntNTy(I.getContext(), intSize), srcWidth / intSize);
         Value* storeVal = BitCastInst::Create(Instruction::BitCast, I.getOperand(0), legalTy, "", &I);
         Value* storePtr = I.getPointerOperand();
 
-        assert(storePtr->getType()->getPointerElementType()->isIntegerTy(srcWidth));
+        IGC_ASSERT(nullptr != storePtr);
+        IGC_ASSERT(nullptr != storePtr->getType());
+        IGC_ASSERT(nullptr != storePtr->getType()->getPointerElementType());
+        IGC_ASSERT(storePtr->getType()->getPointerElementType()->isIntegerTy(srcWidth));
 
         PointerType* ptrTy = PointerType::get(legalTy, storePtr->getType()->getPointerAddressSpace());
         IntToPtrInst* intToPtr = dyn_cast<IntToPtrInst>(storePtr);
@@ -1333,7 +1342,7 @@ void Legalization::visitInsertElementInst(InsertElementInst& I)
     if (ConstantDataVector * vec = dyn_cast<ConstantDataVector>(I.getOperand(0)))
     {
         Value* newVec = UndefValue::get(vec->getType());
-        unsigned int nbElement = vec->getType()->getVectorNumElements();
+        unsigned int nbElement = (unsigned)cast<VectorType>(vec->getType())->getNumElements();
         for (unsigned int i = 0; i < nbElement; i++)
         {
             Constant* cst = vec->getElementAsConstant(i);
@@ -1353,7 +1362,7 @@ void Legalization::visitInsertElementInst(InsertElementInst& I)
     else if (ConstantVector * vec = dyn_cast<ConstantVector>(I.getOperand(0)))
     {
         Value* newVec = UndefValue::get(I.getType());
-        unsigned int nbElement = vec->getType()->getVectorNumElements();
+        unsigned int nbElement = (unsigned)cast<VectorType>(vec->getType())->getNumElements();
         for (unsigned int i = 0; i < nbElement; i++)
         {
             Constant* cst = vec->getOperand(i);
@@ -1373,7 +1382,7 @@ void Legalization::visitInsertElementInst(InsertElementInst& I)
     else if (ConstantAggregateZero * vec = dyn_cast<ConstantAggregateZero>(I.getOperand(0)))
     {
         Value* newVec = UndefValue::get(I.getType());
-        unsigned int nbElement = vec->getType()->getVectorNumElements();
+        unsigned int nbElement = (unsigned)cast<VectorType>(vec->getType())->getNumElements();
         for (unsigned int i = 0; i < nbElement; i++)
         {
             Constant* cst = vec->getElementValue(i);
@@ -1390,8 +1399,8 @@ void Legalization::visitInsertElementInst(InsertElementInst& I)
     else if (I.getOperand(1)->getType()->isIntegerTy(1))
     {
         // This promotes i1 insertelement to i32
-        unsigned int nbElement = I.getOperand(0)->getType()->getVectorNumElements();
-        Value* newVec = UndefValue::get(VectorType::get(m_builder->getInt32Ty(), nbElement));
+        unsigned int nbElement = (unsigned)cast<VectorType>(I.getOperand(0)->getType())->getNumElements();
+        Value* newVec = UndefValue::get(IGCLLVM::FixedVectorType::get(m_builder->getInt32Ty(), nbElement));
         PromoteInsertElement(&I, newVec);
     }
 }
@@ -1409,6 +1418,41 @@ void Legalization::visitShuffleVectorInst(ShuffleVectorInst& I)
     Value* src1 = I.getOperand(1);
     // The mask is guaranteed by the LLVM IR spec to be constant
     Constant* mask = cast<Constant>(I.getOperand(2));
+    // The two inputs are guaranteed to be of the same type
+    VectorType* inType = cast<VectorType>(src0->getType());
+    int inCount = int_cast<int>(inType->getNumElements());
+    int inBase = 2;  // 2 means using undef
+    // if inType == resType, use src0/src1 as the input
+    if (inType == resType)
+    {
+        int srcMatch0 = 0;
+        int srcMatch1 = 0;
+        for (unsigned int dstIndex = 0; dstIndex < resType->getNumElements(); ++dstIndex)
+        {
+            // The mask value can be either an integer or undef.
+            // If it's undef, do nothing.
+            // Otherwise, create an insert with the appropriate value.
+            ConstantInt* index = dyn_cast<ConstantInt>(mask->getAggregateElement(dstIndex));
+            if (index)
+            {
+                int indexVal = int_cast<int>(index->getZExtValue());
+                if (indexVal == dstIndex)
+                    srcMatch0++;
+                else if (indexVal == inCount + dstIndex)
+                    srcMatch1++;
+            }
+        }
+        if (srcMatch0 > srcMatch1 && srcMatch0 > 0)
+        {
+            newVec = src0;
+            inBase = 0;
+        }
+        else if (srcMatch1 > srcMatch0 && srcMatch1 > 0)
+        {
+            inBase = 1;
+            newVec = src1;
+        }
+    }
 
     for (unsigned int dstIndex = 0; dstIndex < resType->getNumElements(); ++dstIndex)
     {
@@ -1420,9 +1464,10 @@ void Legalization::visitShuffleVectorInst(ShuffleVectorInst& I)
         {
             int indexVal = int_cast<int>(index->getZExtValue());
 
-            // The two inputs are guaranteed to be of the same type
-            VectorType* inType = cast<VectorType>(src0->getType());
-            int inCount = int_cast<int>(inType->getNumElements());
+            if (inBase == 0 && indexVal == dstIndex)
+                continue;
+            else if (inBase == 1 && indexVal == dstIndex + inCount)
+                continue;
 
             Value* srcVector = nullptr;
             int srcIndex = 0;
@@ -1444,7 +1489,7 @@ void Legalization::visitShuffleVectorInst(ShuffleVectorInst& I)
             Constant* constSrc = dyn_cast<Constant>(srcVector);
             if (constSrc)
             {
-                srcVal = constSrc->getAggregateElement(dstIndex);
+                srcVal = constSrc->getAggregateElement(srcIndex);
             }
             else
             {
@@ -1510,7 +1555,7 @@ Value* Cast(Value* val, Type* type, Instruction* insertBefore)
     }
     else
     {
-        assert(0 && "unexpected type");
+        IGC_ASSERT_MESSAGE(0, "unexpected type");
     }
     return newVal;
 }
@@ -1580,13 +1625,17 @@ Type* Legalization::LegalAllocaType(Type* type) const
         break;
     case Type::ArrayTyID:
         legalType = ArrayType::get(
-            LegalAllocaType(type->getSequentialElementType()),
+            LegalAllocaType(cast<ArrayType>(type)->getElementType()),
             type->getArrayNumElements());
         break;
+#if LLVM_VERSION_MAJOR >= 11
+    case Type::FixedVectorTyID:
+#else
     case Type::VectorTyID:
-        legalType = VectorType::get(
-            LegalAllocaType(type->getSequentialElementType()),
-            type->getVectorNumElements());
+#endif
+        legalType = IGCLLVM::FixedVectorType::get(
+            LegalAllocaType(cast<VectorType>(type)->getElementType()),
+            (unsigned)cast<VectorType>(type)->getNumElements());
         break;
     case Type::StructTyID:
         return LegalStructAllocaType(type);
@@ -1596,7 +1645,7 @@ Type* Legalization::LegalAllocaType(Type* type) const
     case Type::PointerTyID:
         break;
     default:
-        assert(0 && "Alloca of unsupported type");
+        IGC_ASSERT_MESSAGE(0, "Alloca of unsupported type");
         break;
     }
     return legalType;
@@ -1619,7 +1668,7 @@ void Legalization::visitAlloca(AllocaInst& I)
 void Legalization::visitIntrinsicInst(llvm::IntrinsicInst& I)
 {
     m_ctx->m_instrTypes.numInsts++;
-    IRBuilder<> Builder(&I);
+    IGCLLVM::IRBuilder<> Builder(&I);
 
     auto intrinsicID = I.getIntrinsicID();
 
@@ -1641,7 +1690,7 @@ void Legalization::visitIntrinsicInst(llvm::IntrinsicInst& I)
         case Intrinsic::uadd_sat: OverflowIntrinID = Intrinsic::uadd_with_overflow; break;
         case Intrinsic::sadd_sat: OverflowIntrinID = Intrinsic::sadd_with_overflow; break;
 #endif
-        default: assert(0 && "Incorrect intrinsic"); break;
+        default: IGC_ASSERT_MESSAGE(0, "Incorrect intrinsic"); break;
         }
 
         int BitWidth = I.getType()->getIntegerBitWidth();
@@ -1676,7 +1725,9 @@ void Legalization::visitIntrinsicInst(llvm::IntrinsicInst& I)
         }
             break;
 #endif
-        default: assert(0 && "Incorrect intrinsic"); break;
+        default:
+            IGC_ASSERT_MESSAGE(0, "Incorrect intrinsic");
+            break;
         }
 
         Value* Saturated = Builder.CreateSelect(Overflow, Boundary, Result);
@@ -1690,6 +1741,8 @@ void Legalization::visitIntrinsicInst(llvm::IntrinsicInst& I)
     case Intrinsic::usub_with_overflow:
     case Intrinsic::ssub_with_overflow:
     case Intrinsic::uadd_with_overflow:
+    case Intrinsic::umul_with_overflow:
+    case Intrinsic::smul_with_overflow:
     {
         Value* src0 = I.getArgOperand(0);
         Value* src1 = I.getArgOperand(1);
@@ -1731,7 +1784,75 @@ void Legalization::visitIntrinsicInst(llvm::IntrinsicInst& I)
                 isOverflow = CmpInst::Create(Instruction::ICmp, CmpInst::ICMP_SLT, andOpt, zero, "", &I);
             }
             break;
-            default: assert(0 && "Incorrect intrinsic"); break;
+            case Intrinsic::umul_with_overflow:
+            case Intrinsic::smul_with_overflow:
+            {
+                IGC_ASSERT(nullptr != src0);
+                IGC_ASSERT(nullptr != src1);
+                IGC_ASSERT(src0->getType() == src1->getType());
+                IGC_ASSERT(src0->getType()->isIntegerTy());
+                bool isSigned = intrinsicID == Intrinsic::smul_with_overflow;
+                res = BinaryOperator::Create(Instruction::Mul, src0, src1, "", &I);
+                const unsigned int bitWidth = src0->getType()->getIntegerBitWidth();
+                if (bitWidth == 64 || bitWidth == 32 )
+                {
+                    BasicBlock* const bb = I.getParent();
+                    IGC_ASSERT(nullptr != bb);
+                    Function* const f = bb->getParent();
+                    IGC_ASSERT(nullptr != f);
+                    Value* hiDst = CreateMulh(*f, Builder, isSigned, src0, src1);
+                    if (isSigned)
+                    {
+                        // Signed a * b overflows if Mulh(a, b) != 0 or -1   and consequently
+                        // if Mulh(a, b) + 1 > 1 (for an unsigned comparison)
+                        Value* const one = ConstantInt::get(src0->getType(), 1, true);
+                        Value* const add1 = BinaryOperator::Create(Instruction::Add, hiDst, one, "", &I);
+                        isOverflow = CmpInst::Create(Instruction::ICmp, CmpInst::ICMP_UGT, add1, one, "", &I);
+                    }
+                    else
+                    {
+                        // Unsigned a * b overflows if Mulh(a, b) != 0
+                        Value* const zero = ConstantInt::get(src0->getType(), 0, isSigned);
+                        isOverflow = CmpInst::Create(Instruction::ICmp, CmpInst::ICMP_NE, hiDst, zero, "", &I);
+                    }
+                }
+                else
+                {
+                    IGC_ASSERT(bitWidth < 32);
+                    Value* ores = nullptr;
+                    if (isSigned)
+                    {
+                        // Signed a * b overflows if (a * b) + (1 << (bitWidth-1)) >= (1 << bitWidth)
+                        //   (for an unsigned comparison)
+                        // For example. If src0 is 0xFF (-1) and src0 is 0xFF (-1) then
+                        //   src0SExt is 0xFFFFFFFF
+                        //   src1SExt is 0xFFFFFFFF
+                        //   mulRes is 0x00000001   (0xFFFFFFFF * 0xFFFFFFFF or -1 * -1)
+                        //   oneShl is 0x00000080   (1 << 7)
+                        //   ores is 0x00000081     (1 + 128)
+                        //   overflowed is 0x00000100    (1 << 8)
+                        //   isOverflow is false    (129 >= 256)
+                        Value* const src0SExt = BitCastInst::CreateSExtOrBitCast(src0, Builder.getInt32Ty(), "", &I);
+                        Value* const src1SExt = BitCastInst::CreateSExtOrBitCast(src1, Builder.getInt32Ty(), "", &I);
+                        Value* const mulRes = BinaryOperator::Create(Instruction::Mul, src0SExt, src1SExt, "", &I);
+                        Value* const oneShl = ConstantInt::get(Builder.getInt32Ty(), 1LL << (bitWidth - 1), true);
+                        ores = BinaryOperator::Create(Instruction::Add, mulRes, oneShl, "", &I);
+                    }
+                    else
+                    {
+                        // Unsigned a * b overflows if a * b >= (1 << bitWidth) (for an unsigned comparison)
+                        Value* const src0ZExt = BitCastInst::CreateZExtOrBitCast(src0, Builder.getInt32Ty(), "", &I);
+                        Value* const src1ZExt = BitCastInst::CreateZExtOrBitCast(src1, Builder.getInt32Ty(), "", &I);
+                        ores = BinaryOperator::Create(Instruction::Mul, src0ZExt, src1ZExt, "", &I);
+                    }
+                    Value* const overflowed = ConstantInt::get(Builder.getInt32Ty(), 1LL << bitWidth, false);
+                    isOverflow = CmpInst::Create(Instruction::ICmp, CmpInst::ICMP_UGE, ores, overflowed, "", &I);
+                }
+            }
+            break;
+            default:
+                IGC_ASSERT_MESSAGE(0, "Incorrect intrinsic");
+                break;
         }
 
         // llvm.x.with.overflow returns a struct, where the first element is the operation result,
@@ -1742,7 +1863,7 @@ void Legalization::visitIntrinsicInst(llvm::IntrinsicInst& I)
             ExtractValueInst* extract = dyn_cast<ExtractValueInst>(*U);
             if (!extract)
             {
-                assert(0 && "Did not expect anything but an extract after uadd_with_overflow");
+                IGC_ASSERT_MESSAGE(0, "Did not expect anything but an extract after uadd_with_overflow");
                 continue;
             }
 
@@ -1757,7 +1878,7 @@ void Legalization::visitIntrinsicInst(llvm::IntrinsicInst& I)
             }
             else
             {
-                assert(0 && "Unexpected index when handling uadd_with_overflow");
+                IGC_ASSERT_MESSAGE(0, "Unexpected index when handling uadd_with_overflow");
             }
 
             m_instructionsToRemove.push_back(extract);
@@ -1769,33 +1890,74 @@ void Legalization::visitIntrinsicInst(llvm::IntrinsicInst& I)
     case Intrinsic::assume:
         m_instructionsToRemove.push_back(&I);
         break;
-    case Intrinsic::umul_with_overflow:
-    case Intrinsic::smul_with_overflow:
-        TODO("Handle the other with_overflow intrinsics");
-        assert(0 && "Unhandled llvm.x.with.overflow intrinsic");
-        break;
-    default:
-        break;
-    }
-    if (!m_ctx->platform.supportFP16Rounding() && I.getType()->isHalfTy()) {
-        // On platform lacking of FP16 rounding, promote them to FP32 and
-        // demote back.
-        Intrinsic::ID IID = I.getIntrinsicID();
-        switch (IID) {
-        default:
-            break;
-        case llvm::Intrinsic::floor:
-        case llvm::Intrinsic::ceil:
-        case llvm::Intrinsic::trunc: {
+    case Intrinsic::floor:
+    case Intrinsic::ceil:
+    case Intrinsic::trunc:
+    {
+        if (!m_ctx->platform.supportFP16Rounding() && I.getType()->isHalfTy())
+        {
+            // On platform lacking of FP16 rounding, promote them to FP32 and
+            // demote back.
             Value* Val = Builder.CreateFPExt(I.getOperand(0), Builder.getFloatTy());
-            Value* Callee = Intrinsic::getDeclaration(I.getParent()->getParent()->getParent(), IID, Builder.getFloatTy());
-            Val = Builder.CreateCall(Callee, Val);
+            Value* Callee = Intrinsic::getDeclaration(I.getParent()->getParent()->getParent(), intrinsicID, Builder.getFloatTy());
+            Val = Builder.CreateCall(Callee, ArrayRef<Value*>(Val));
             Val = Builder.CreateFPTrunc(Val, I.getType());
             I.replaceAllUsesWith(Val);
             I.eraseFromParent();
-            break;
         }
+    }
+    break;
+    case Intrinsic::copysign:
+    {
+        Value* const src0 = I.getArgOperand(0);
+        Value* const src1 = I.getArgOperand(1);
+        Type* const srcType = src0->getType();
+
+        IGC_ASSERT(nullptr != srcType);
+        IGC_ASSERT_MESSAGE(srcType->getScalarType()->isFloatingPointTy(), "llvm.copysign supports only floating-point type");
+
+        auto cpySign = [&Builder](Value* const src0, Value* const src1) {
+            Type* const srcType = src0->getType();
+            const unsigned int srcTypeSize = (const unsigned int)srcType->getPrimitiveSizeInBits();
+            const uint64_t signMask = (uint64_t)0x1 << (srcTypeSize - 1);
+
+            Value* const src0Int = Builder.CreateBitCast(src0, Builder.getIntNTy(srcTypeSize));
+            Value* const src1Int = Builder.CreateBitCast(src1, Builder.getIntNTy(srcTypeSize));
+
+            Value* const src0NoSign = Builder.CreateAnd(src0Int, Builder.getIntN(srcTypeSize, ~signMask));
+            Value* const src1Sign = Builder.CreateAnd(src1Int, Builder.getIntN(srcTypeSize, signMask));
+
+            Value* newValue = static_cast<Value*>(Builder.CreateOr(src0NoSign, src1Sign));
+            newValue = Builder.CreateBitCast(newValue, srcType);
+            return newValue;
+        };
+
+        Value* newValue = nullptr;
+        if (srcType->isVectorTy())
+        {
+            auto sourceVT = cast<VectorType>(srcType);
+            const unsigned int numElements = (uint32_t)sourceVT->getNumElements();
+            Value* dstVec = UndefValue::get(srcType);
+            for (unsigned int i = 0; i < numElements; ++i)
+            {
+                Value* const src0Scalar = Builder.CreateExtractElement(src0, i);
+                Value* const src1Scalar = Builder.CreateExtractElement(src1, i);
+                auto newValue = cpySign(src0Scalar, src1Scalar);
+                dstVec = Builder.CreateInsertElement(dstVec, newValue, i);
+            }
+            newValue = dstVec;
         }
+        else
+        {
+            newValue = cpySign(src0, src1);
+        }
+        IGC_ASSERT(nullptr != newValue);
+        I.replaceAllUsesWith(newValue);
+        I.eraseFromParent();
+    }
+    break;
+    default:
+        break;
     }
 }
 
@@ -1924,7 +2086,7 @@ void Legalization::visitTruncInst(llvm::TruncInst& I) {
 
     m_builder->SetInsertPoint(&I);
 
-    assert(Idx < 3 && "The initial index is out of range!");
+    IGC_ASSERT_MESSAGE(Idx < 3, "The initial index is out of range!");
 
     Value* NewVal =
         m_builder->CreateZExt(
@@ -2083,7 +2245,7 @@ void GenOptLegalizer::visitLoadInst(LoadInst& I) {
         // (RAUW)
         //
         m_Builder->SetInsertPoint(&I);
-        Type* I8x3Ty = VectorType::get(m_Builder->getInt8Ty(), 3);
+        Type* I8x3Ty = IGCLLVM::FixedVectorType::get(m_Builder->getInt8Ty(), 3);
         Type* I8x3PtrTy = PointerType::get(I8x3Ty, I.getPointerAddressSpace());
         Value* NewPtr = m_Builder->CreateBitCast(I.getPointerOperand(), I8x3PtrTy);
         Value* NewLD = IGC::cloneLoad(&I, NewPtr);
@@ -2130,7 +2292,7 @@ void GenOptLegalizer::visitStoreInst(StoreInst& I) {
             // %newdst = bitcast i24* %dst to <3 x i8>*
             // %1 = store <3 x i8> %0, <3 x i8>* %newdst
             //
-            Type* I8x3Ty = VectorType::get(m_Builder->getInt8Ty(), 3);
+            Type* I8x3Ty = IGCLLVM::FixedVectorType::get(m_Builder->getInt8Ty(), 3);
             Type* I8x3PtrTy = PointerType::get(I8x3Ty, LD->getPointerAddressSpace());
             // Replace load of i24 to load of <3 x i8>
             m_Builder->SetInsertPoint(LD);
@@ -2166,13 +2328,13 @@ void GenOptLegalizer::visitStoreInst(StoreInst& I) {
                 // store <3 x i8> %2, <3 x i8>* %0
                 //
                 m_Builder->SetInsertPoint(&I);
-                Type* I8x3Ty = VectorType::get(m_Builder->getInt8Ty(), 3);
+                Type* I8x3Ty = IGCLLVM::FixedVectorType::get(m_Builder->getInt8Ty(), 3);
                 Type* I8x3PtrTy = PointerType::get(I8x3Ty, I.getPointerAddressSpace());
 
                 // Convert i32 to <4 x i8>
                 Type* SrcTy = SV->getOperand(0)->getType();
                 unsigned numElements = (unsigned int)SrcTy->getPrimitiveSizeInBits() / 8;
-                Type* NewVecTy = VectorType::get(m_Builder->getInt8Ty(), numElements);
+                Type* NewVecTy = IGCLLVM::FixedVectorType::get(m_Builder->getInt8Ty(), numElements);
                 Value* NewVec = m_Builder->CreateBitCast(SV->getOperand(0), NewVecTy);
                 // Create shufflevector to select elements for <3 x i8>
                 SmallVector<uint32_t, 3> maskVals = { 0, 1, 2 };
