@@ -24,24 +24,23 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 ======================= end_copyright_notice ==================================*/
 
-// vim:ts=2:sw=2:et:
-
 #define DEBUG_TYPE "type-legalizer"
 #include "TypeLegalizer.h"
 #include "InstLegalChecker.h"
 #include "common/LLVMWarningsPush.hpp"
-
-#include "llvmWrapper/Support/Debug.h"
-
+#include "llvm/Config/llvm-config.h"
+#include "llvm/Support/Debug.h"
 #include "llvm/Support/raw_ostream.h"
 #include "common/LLVMWarningsPop.hpp"
+#include "Probe/Assertion.h"
+
 using namespace llvm;
 using namespace IGC::Legalizer;
 
 // By default, capture all missing instructions!
 LegalizeAction InstLegalChecker::visitInstruction(Instruction& I) {
     LLVM_DEBUG(dbgs() << "LEGAL-CHECK: " << I << '\n');
-    llvm_unreachable("UNKNOWN INSTRUCTION IS BEING LEGAL-CHECKED!");
+    IGC_ASSERT_EXIT_MESSAGE(0, "UNKNOWN INSTRUCTION IS BEING LEGAL-CHECKED!");
 }
 
 /// Terminator instructions
@@ -168,7 +167,7 @@ LegalizeAction InstLegalChecker::visitIntrinsicInst(IntrinsicInst& I) {
     case Intrinsic::exp:
     case Intrinsic::exp2:
     case Intrinsic::fabs:
-        // case Intrinsic::copysign:
+    case Intrinsic::copysign:
     case Intrinsic::floor:
     case Intrinsic::ceil:
     case Intrinsic::trunc:
@@ -262,7 +261,13 @@ LegalizeAction InstLegalChecker::visitShuffleVectorInst(ShuffleVectorInst& I) {
     if ((Act = TL->getTypeLegalizeAction(I.getOperand(0)->getType())) != Legal)
         return Act;
     // Check the constant mask.
-    return TL->getTypeLegalizeAction(I.getMask()->getType());
+    return TL->getTypeLegalizeAction(I.
+#if LLVM_VERSION_MAJOR <= 10
+        getMask
+#else
+        getShuffleMaskForBitcode
+#endif
+        ()->getType());
 }
 
 LegalizeAction InstLegalChecker::visitExtractValueInst(ExtractValueInst& I) {
