@@ -24,19 +24,17 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 ======================= end_copyright_notice ==================================*/
 
-// vim:ts=2:sw=2:et:
-
 #define DEBUG_TYPE "type-legalizer"
 #include "TypeLegalizer.h"
 #include "InstScalarizer.h"
 #include "common/LLVMWarningsPush.hpp"
-
-#include "llvmWrapper/Support/Debug.h"
-
+#include "llvm/Support/Debug.h"
+#include "llvmWrapper/IR/DerivedTypes.h"
 #include "llvm/Support/raw_ostream.h"
 #include "common/LLVMWarningsPop.hpp"
-
 #include "common/Types.hpp"
+#include "Probe/Assertion.h"
+
 using namespace llvm;
 using namespace IGC::Legalizer;
 
@@ -57,7 +55,7 @@ bool InstScalarizer::scalarize(Instruction* I) {
 // By default, capture all missing instructions!
 bool InstScalarizer::visitInstruction(Instruction& I) {
     LLVM_DEBUG(dbgs() << "SCALARIZE: " << I << '\n');
-    llvm_unreachable("UNKNOWN INSTRUCTION IS BEING SCALARIZED!");
+    IGC_ASSERT_EXIT_MESSAGE(0, "UNKNOWN INSTRUCTION IS BEING SCALARIZED!");
     return false;
 }
 
@@ -81,7 +79,7 @@ bool InstScalarizer::visitBinaryOperator(BinaryOperator& I) {
     ValueSeq Ops0Copy(*Ops0);
 
     std::tie(Ops1, std::ignore) = TL->getLegalizedValues(I.getOperand(1));
-    assert(Ops0Copy.size() == Ops1->size());
+    IGC_ASSERT(Ops0Copy.size() == Ops1->size());
 
     for (unsigned i = 0, e = Ops0Copy.size(); i != e; ++i) {
         Value* LHS = (Ops0Copy)[i];
@@ -108,7 +106,7 @@ bool InstScalarizer::visitLoadInst(LoadInst& I) {
 
     unsigned AS = I.getPointerAddressSpace();
 
-    Type* EltTy = OrigTy->getVectorElementType();
+    Type* EltTy = cast<VectorType>(OrigTy)->getElementType();
 
     if (TL->preferVectorLoad(OrigTy)) {
         // Skip if this load is already legalized.
@@ -132,10 +130,9 @@ bool InstScalarizer::visitLoadInst(LoadInst& I) {
 
         // NOTE: It's assumed the element in this case is byte-addressable;
         // otherwise, it's broken.
-        assert(TL->getTypeSizeInBits(EltTy) ==
-            TL->getTypeStoreSizeInBits(EltTy));
+        IGC_ASSERT(TL->getTypeSizeInBits(EltTy) == TL->getTypeStoreSizeInBits(EltTy));
 
-        unsigned NumElts = OrigTy->getVectorNumElements();
+        unsigned NumElts = (unsigned)cast<VectorType>(OrigTy)->getNumElements();
         unsigned Elt = 0;
 
         Type* NewPtrTy = PointerType::get(EltTy, AS);
@@ -149,7 +146,7 @@ bool InstScalarizer::visitLoadInst(LoadInst& I) {
         for (auto PLI = ProfitLengths.rbegin(),
             PLE = ProfitLengths.rend(); PLI != PLE; ++PLI) {
             unsigned PL = *PLI;
-            assert(PL > 0);
+            IGC_ASSERT(PL > 0);
 
             for (; NumElts >= PL; NumElts -= PL) {
                 Value* NewPtr =
@@ -157,7 +154,7 @@ bool InstScalarizer::visitLoadInst(LoadInst& I) {
                         Twine(OldPtr->getName(), getSuffix()) + Twine(Elt));
                 if (PL != 1) {
                     // Load <PL x EltTy>
-                    Type* VecTy = VectorType::get(EltTy, PL);
+                    Type* VecTy = IGCLLVM::FixedVectorType::get(EltTy, PL);
                     Type* VecPtrTy = PointerType::get(VecTy, AS);
                     NewPtr =
                         IRB->CreatePointerCast(NewPtr, VecPtrTy,
@@ -235,13 +232,13 @@ bool InstScalarizer::visitStoreInst(StoreInst& I) {
 
     ValueSeq* ValSeq;
     std::tie(ValSeq, std::ignore) = TL->getLegalizedValues(OrigVal);
-    assert(ValSeq->size() == OrigTy->getVectorNumElements());
+    IGC_ASSERT(ValSeq->size() == cast<VectorType>(OrigTy)->getNumElements());
 
     StringRef Name = OrigVal->getName();
 
     unsigned AS = I.getPointerAddressSpace();
 
-    Type* EltTy = OrigTy->getVectorElementType();
+    Type* EltTy = cast<VectorType>(OrigTy)->getElementType();
 
     if (TL->preferVectorStore(OrigTy)) {
         // If the type of store value prefer to be vector, prepare its
@@ -271,10 +268,9 @@ bool InstScalarizer::visitStoreInst(StoreInst& I) {
 
         // NOTE: It's assumed the element in this case is byte-addressable;
         // otherwise, it's broken.
-        assert(TL->getTypeSizeInBits(EltTy) ==
-            TL->getTypeStoreSizeInBits(EltTy));
+        IGC_ASSERT(TL->getTypeSizeInBits(EltTy) == TL->getTypeStoreSizeInBits(EltTy));
 
-        unsigned NumElts = OrigTy->getVectorNumElements();
+        unsigned NumElts = (unsigned)cast<VectorType>(OrigTy)->getNumElements();
         unsigned Elt = 0;
 
         Type* NewPtrTy = PointerType::get(EltTy, AS);
@@ -288,7 +284,7 @@ bool InstScalarizer::visitStoreInst(StoreInst& I) {
         for (auto PLI = ProfitLengths.rbegin(),
             PLE = ProfitLengths.rend(); PLI != PLE; ++PLI) {
             unsigned PL = *PLI;
-            assert(PL > 0);
+            IGC_ASSERT(PL > 0);
 
             for (; NumElts >= PL; NumElts -= PL) {
                 Value* NewPtr =
@@ -296,7 +292,7 @@ bool InstScalarizer::visitStoreInst(StoreInst& I) {
                         Twine(OldPtr->getName(), getSuffix()) + Twine(Elt));
                 if (PL != 1) {
                     // Store <PL x EltTy>
-                    Type* VecTy = VectorType::get(EltTy, PL);
+                    Type* VecTy = IGCLLVM::FixedVectorType::get(EltTy, PL);
                     // Prepare the shorter vector.
                     Value* Vec = UndefValue::get(VecTy);
                     for (unsigned i = 0; i != PL; ++i) {
@@ -401,7 +397,7 @@ bool InstScalarizer::visitBitCastInst(BitCastInst& I) {
     TL->repack(&Repacked, *TySeq, *ValSeq, I.getName() + getSuffix());
 
     if (Act == Legal) {
-        assert(Repacked.size() == 1);
+        IGC_ASSERT(Repacked.size() == 1);
 
         I.replaceAllUsesWith(Repacked.front());
         return true;
@@ -431,8 +427,7 @@ bool InstScalarizer::visitExtractElementInst(ExtractElementInst& I) {
         // %elt = load eT* %eptr
         //
         // It would be much more complicated if eT is not byte addressable.
-        llvm_unreachable(
-            "NON-CONSTANT IDX IN EXTRACT-ELEMENT IS NOT SUPPORTED YET!");
+        IGC_ASSERT_EXIT_MESSAGE(0, "NON-CONSTANT IDX IN EXTRACT-ELEMENT IS NOT SUPPORTED YET!");
     }
 
     ConstantInt* CI = cast<ConstantInt>(I.getIndexOperand());
@@ -477,8 +472,7 @@ bool InstScalarizer::visitInsertElementInst(InsertElementInst& I) {
         // %vec1 = load <N x eT>* %stk
         //
         // It would be much more complicated if eT is not byte addressable.
-        llvm_unreachable(
-            "NON-CONSTANT IDX IN INSERT-ELEMENT IS NOT SUPPORTED YET!");
+        IGC_ASSERT_EXIT_MESSAGE(0, "NON-CONSTANT IDX IN INSERT-ELEMENT IS NOT SUPPORTED YET!");
     }
 
     ConstantInt* CI = cast<ConstantInt>(I.getOperand(2));
@@ -492,7 +486,8 @@ bool InstScalarizer::visitInsertElementInst(InsertElementInst& I) {
     // and previously received ValueSeq objects will become invalid.
     ValueSeq VecSeqCopy(*VecSeq);
 
-    ValueSeq* EltSeq; LegalizeAction Act;
+    ValueSeq* EltSeq = nullptr;
+    LegalizeAction Act;
     std::tie(EltSeq, Act) = TL->getLegalizedValues(I.getOperand(1));
 
     ValueSeq LegalVal;
@@ -501,9 +496,11 @@ bool InstScalarizer::visitInsertElementInst(InsertElementInst& I) {
         EltSeq = &LegalVal;
     }
 
-    assert(VecSeqCopy.size() % EltSeq->size() == 0);
+    IGC_ASSERT(nullptr != EltSeq);
+    IGC_ASSERT(EltSeq->size());
+    IGC_ASSERT(VecSeqCopy.size() % EltSeq->size() == 0);
 
-    unsigned NumElts = I.getOperand(0)->getType()->getVectorNumElements();
+    unsigned NumElts = (unsigned)cast<VectorType>(I.getOperand(0)->getType())->getNumElements();
     unsigned i = 0;
     for (unsigned Elt = 0; Elt != NumElts; ++Elt) {
         if (Elt == Idx) {
