@@ -33,15 +33,14 @@ FIXME? ::
 4. StoreInst : Need to consider illegal type being stored directly without being cast back to a legal.
 */
 
-
-// vim:ts=2:sw=2:fdm=marker:et:
-
 #define DEBUG_TYPE "type-legalizer"
 #include "PeepholeTypeLegalizer.hpp"
 #include "common/LLVMWarningsPush.hpp"
+#include "llvmWrapper/IR/DerivedTypes.h"
 #include "common/LLVMWarningsPop.hpp"
-
 #include "Compiler/IGCPassSupport.h"
+#include "Probe/Assertion.h"
+
 using namespace llvm;
 using namespace IGC::Legalizer;
 
@@ -71,7 +70,7 @@ FunctionPass* createPeepholeTypeLegalizerPass() { return new PeepholeTypeLegaliz
 
 bool PeepholeTypeLegalizer::runOnFunction(Function& F) {
     DL = &F.getParent()->getDataLayout();
-    assert(DL->isLittleEndian() && "ONLY SUPPORT LITTLE ENDIANNESS!");
+    IGC_ASSERT_MESSAGE(DL->isLittleEndian(), "ONLY SUPPORT LITTLE ENDIANNESS!");
 
     llvm::IRBuilder<> builder(F.getContext());
     m_builder = &builder;
@@ -123,7 +122,7 @@ void PeepholeTypeLegalizer::visitInstruction(Instruction& I) {
         if (dyn_cast<PHINode>(&I)) {
             legalizePhiInstruction(I);  // phi nodes and all incoming values
         }
-        if (dyn_cast<UnaryInstruction>(&I)) {
+        else if (dyn_cast<UnaryInstruction>(&I)) {
             legalizeUnaryInstruction(I); // pointercast &/or load
         }
         else if (dyn_cast<ICmpInst>(&I) || dyn_cast<BinaryOperator>(&I) || dyn_cast<SelectInst>(&I)) {
@@ -145,7 +144,7 @@ void PeepholeTypeLegalizer::visitInstruction(Instruction& I) {
 
 void PeepholeTypeLegalizer::legalizePhiInstruction(Instruction& I)
 {
-    assert(isa<PHINode>(&I));
+    IGC_ASSERT(isa<PHINode>(&I));
 
     unsigned srcWidth = I.getType()->getScalarSizeInBits();
     if (!I.getType()->isIntOrIntVectorTy() || isLegalInteger(srcWidth) || srcWidth == 1) // nothing to legalize
@@ -159,8 +158,8 @@ void PeepholeTypeLegalizer::legalizePhiInstruction(Instruction& I)
 
     if (quotient > 1)
     {
-        unsigned numElements = I.getType()->isVectorTy() ? I.getType()->getVectorNumElements() : 1;
-        Type* newType = VectorType::get(Type::getIntNTy(I.getContext(), promoteToInt), quotient * numElements);
+        unsigned numElements = I.getType()->isVectorTy() ? (unsigned)cast<VectorType>(I.getType())->getNumElements() : 1;
+        Type* newType = IGCLLVM::FixedVectorType::get(Type::getIntNTy(I.getContext(), promoteToInt), quotient * numElements);
 
         PHINode* newPhi = m_builder->CreatePHI(newType, oldPhi->getNumIncomingValues());
         for (unsigned i = 0; i < oldPhi->getNumIncomingValues(); i++)
@@ -197,7 +196,7 @@ void PeepholeTypeLegalizer::legalizePhiInstruction(Instruction& I)
 
 void PeepholeTypeLegalizer::legalizeExtractElement(Instruction& I)
 {
-    assert(isa<ExtractElementInst>(&I));
+    IGC_ASSERT(isa<ExtractElementInst>(&I));
 
     // Handles ExtractElement from illegal vector types
 
@@ -227,16 +226,16 @@ void PeepholeTypeLegalizer::legalizeExtractElement(Instruction& I)
         m_builder->SetInsertPoint(&I);
 
         // Bitcast the illegal vector type to legal type
-        Type* newVecTy = VectorType::get(Type::getIntNTy(I.getContext(), promoteToInt), quotient * numElements);
+        Type* newVecTy = IGCLLVM::FixedVectorType::get(Type::getIntNTy(I.getContext(), promoteToInt), quotient * numElements);
         Value* legalVector = m_builder->CreateBitCast(extract->getOperand(0), newVecTy);
 
         unsigned extractIndex = (unsigned)cast<ConstantInt>(extract->getOperand(1))->getZExtValue();
-        Value* extractedVec = UndefValue::get(VectorType::get(Type::getIntNTy(I.getContext(), promoteToInt), quotient));
+        Value* extractedVec = UndefValue::get(IGCLLVM::FixedVectorType::get(Type::getIntNTy(I.getContext(), promoteToInt), quotient));
 
         for (unsigned i = 0; i < quotient; i++)
         {
             unsigned index = extractIndex * quotient + i;
-            assert(index < quotient * numElements);
+            IGC_ASSERT(index < quotient * numElements);
             Value* extractedVal = m_builder->CreateExtractElement(legalVector, m_builder->getInt32(index));
             extractedVec = m_builder->CreateInsertElement(extractedVec, extractedVal, m_builder->getInt32(i));
         }
@@ -331,10 +330,10 @@ void PeepholeTypeLegalizer::legalizeBinaryOperator(Instruction& I) {
     if (quotient > 1)
     {
         Value* NewLargeSrc1VecForm = m_builder->CreateBitCast(NewLargeSrc1,
-            llvm::VectorType::get(llvm::Type::getIntNTy(I.getContext(), promoteToInt), quotient));
+            IGCLLVM::FixedVectorType::get(llvm::Type::getIntNTy(I.getContext(), promoteToInt), quotient));
         Value* NewLargeSrc2VecForm = m_builder->CreateBitCast(NewLargeSrc2,
-            llvm::VectorType::get(llvm::Type::getIntNTy(I.getContext(), promoteToInt), quotient));
-        Value* NewLargeResVecForm = UndefValue::get(llvm::VectorType::get(llvm::Type::getIntNTy(I.getContext(), promoteToInt), quotient));
+            IGCLLVM::FixedVectorType::get(llvm::Type::getIntNTy(I.getContext(), promoteToInt), quotient));
+        Value* NewLargeResVecForm = UndefValue::get(IGCLLVM::FixedVectorType::get(llvm::Type::getIntNTy(I.getContext(), promoteToInt), quotient));
 
         bool instSupported = true;
         for (unsigned Idx = 0; Idx < quotient; Idx++)
@@ -366,25 +365,26 @@ void PeepholeTypeLegalizer::legalizeBinaryOperator(Instruction& I) {
                 }
                 else {
                     instSupported = false;
-                    assert(false && "Shift by amount is not a constant.");
+                    IGC_ASSERT_MESSAGE(0, "Shift by amount is not a constant.");
                 }
                 break;
             }
             case Instruction::Add:
                 instSupported = false;
-                assert(false && "Add Instruction seen with 'large' illegal int type. Legalization support missing.");
+                IGC_ASSERT_MESSAGE(0, "Add Instruction seen with 'large' illegal int type. Legalization support missing.");
                 break;
             case Instruction::ICmp:
                 instSupported = false;
-                assert(false && "ICmp Instruction seen with 'large' illegal int type. Legalization support missing.");
+                IGC_ASSERT_MESSAGE(0, "ICmp Instruction seen with 'large' illegal int type. Legalization support missing.");
                 break;
             case Instruction::Select:
                 instSupported = false;
-                assert(false && "Select Instruction seen with 'large' illegal int type. Legalization support missing.");
+                IGC_ASSERT_MESSAGE(0, "Select Instruction seen with 'large' illegal int type. Legalization support missing.");
                 break;
             default:
                 printf("Binary Instruction seen with illegal int type. Legalization support missing. Inst opcode:%d", I.getOpcode());
-                assert(false);
+                IGC_ASSERT_MESSAGE(0, "Binary Instruction seen with illegal int type. Legalization support missing.");
+                break;
             }
             if (instSupported)
                 NewLargeResVecForm = m_builder->CreateInsertElement(NewLargeResVecForm, NewInst, Idx);
@@ -426,7 +426,7 @@ void PeepholeTypeLegalizer::legalizeBinaryOperator(Instruction& I) {
             {
                 // Must use sext [note that NewLargeSrc1/2 are zext]
                 int shiftAmt = promoteToInt - Src1width;
-                assert(shiftAmt > 0 && "Should not happen, something wrong!");
+                IGC_ASSERT_MESSAGE(shiftAmt > 0, "Should not happen, something wrong!");
                 Value* V1 = m_builder->CreateShl(NewLargeSrc1, shiftAmt);
                 Value* PromotedSrc1 = m_builder->CreateAShr(V1, shiftAmt);
                 Value* V2 = m_builder->CreateShl(NewLargeSrc2, shiftAmt);
@@ -450,7 +450,8 @@ void PeepholeTypeLegalizer::legalizeBinaryOperator(Instruction& I) {
             NewLargeRes = m_builder->CreateLShr(NewLargeSrc1, NewLargeSrc2);
         default:
             printf("Binary Instruction seen with illegal int type. Legalization support missing. Inst opcode:%d", I.getOpcode());
-            assert(false);
+            IGC_ASSERT_MESSAGE(0, "Binary Instruction seen with illegal int type. Legalization support missing.");
+            break;
         }
 
         if (!NewIllegal)
@@ -503,7 +504,7 @@ void PeepholeTypeLegalizer::legalizeUnaryInstruction(Instruction& I) {
 
             if (quotient > 1)
             {
-                Type* I8xXTy = VectorType::get(m_builder->getInt8Ty(), quotient);
+                Type* I8xXTy = IGCLLVM::FixedVectorType::get(m_builder->getInt8Ty(), quotient);
                 Type* I8xXPtrTy = PointerType::get(I8xXTy, I.getType()->getPointerAddressSpace());
 
                 Value* newBitCastToVec = m_builder->CreateBitCast(I.getOperand(0), I8xXPtrTy);
@@ -563,7 +564,7 @@ void PeepholeTypeLegalizer::legalizeUnaryInstruction(Instruction& I) {
         promoteInt(loadSrcWidth, loadQuotient, loadPromoteToInt, 8); // hard coded to 8 since our hardware is bte addressable.
 
         if (loadQuotient > 1) {
-            Value* newBitCast = m_builder->CreatePointerCast(I.getOperand(0), llvm::VectorType::get(llvm::Type::getIntNTy(I.getContext(), loadPromoteToInt), loadQuotient));
+            Value* newBitCast = m_builder->CreatePointerCast(I.getOperand(0), IGCLLVM::FixedVectorType::get(llvm::Type::getIntNTy(I.getContext(), loadPromoteToInt), loadQuotient));
             Value* newLoadInst = m_builder->CreateLoad(newBitCast);
             // mask the extra bits loaded for type legalization
             unsigned maskCnt = (loadPromoteToInt * loadQuotient) - loadSrcWidth;
@@ -586,7 +587,7 @@ void PeepholeTypeLegalizer::legalizeUnaryInstruction(Instruction& I) {
             I.eraseFromParent();
         }
         else {
-            Value* newBitCast = m_builder->CreatePointerCast(I.getOperand(0), llvm::VectorType::get(llvm::Type::getIntNTy(I.getContext(), loadPromoteToInt), loadQuotient));
+            Value* newBitCast = m_builder->CreatePointerCast(I.getOperand(0), IGCLLVM::FixedVectorType::get(llvm::Type::getIntNTy(I.getContext(), loadPromoteToInt), loadQuotient));
             Value* newLoadInst = m_builder->CreateLoad(newBitCast);
             // mask the extra bits loaded for type legalization
             unsigned maskCnt = (loadPromoteToInt * loadQuotient) - loadSrcWidth;
@@ -624,7 +625,7 @@ void PeepholeTypeLegalizer::legalizeUnaryInstruction(Instruction& I) {
         }
         else
         {
-            assert(false && "SExt Instruction seen with illegal int type and BitWidth > 1. Legalization support missing.");
+            IGC_ASSERT_MESSAGE(0, "SExt Instruction seen with illegal int type and BitWidth > 1. Legalization support missing.");
         }
     }
     break;
@@ -662,8 +663,8 @@ void PeepholeTypeLegalizer::legalizeUnaryInstruction(Instruction& I) {
 
         unsigned numSrcElements = srcSize / promotedInt;
         unsigned numDstElements = dstSize / promotedInt;
-        Type* srcVecTy = VectorType::get(Type::getIntNTy(I.getContext(), promotedInt), numSrcElements);
-        Type* dstVecTy = VectorType::get(Type::getIntNTy(I.getContext(), promotedInt), numDstElements);
+        Type* srcVecTy = IGCLLVM::FixedVectorType::get(Type::getIntNTy(I.getContext(), promotedInt), numSrcElements);
+        Type* dstVecTy = IGCLLVM::FixedVectorType::get(Type::getIntNTy(I.getContext(), promotedInt), numDstElements);
 
         // Bitcast the illegal src type to a legal vector
         Value* srcVec = m_builder->CreateBitCast(I.getOperand(0), srcVecTy);
@@ -687,7 +688,7 @@ void PeepholeTypeLegalizer::legalizeUnaryInstruction(Instruction& I) {
         // 3. zext the incoming ILLEGAL to byte padded value
         // 4. OR the zext'ed value and masked load.
         // 5. store the OR'ed value into byte padded size pointer
-        assert(false && "Store Instruction seen with illegal int type. Legalization support missing.");
+        IGC_ASSERT_MESSAGE(0, "Store Instruction seen with illegal int type. Legalization support missing.");
         break;
     }
 }
@@ -747,8 +748,8 @@ void PeepholeTypeLegalizer::cleanupZExtInst(Instruction& I) {
                 */
                 if ((promoteToInt * quotient) == I.getType()->getScalarSizeInBits()) { // rhis is the case for all trunc-zext pairs generated by first step of this legalization pass
                     Value* truncSrcAsVec = m_builder->CreateBitCast(prevInst->getOperand(0),
-                        llvm::VectorType::get(llvm::Type::getIntNTy(I.getContext(), promoteToInt), quotient));
-                    Value* vecRes = UndefValue::get(llvm::VectorType::get(llvm::Type::getIntNTy(I.getContext(), promoteToInt), quotient));
+                        IGCLLVM::FixedVectorType::get(llvm::Type::getIntNTy(I.getContext(), promoteToInt), quotient));
+                    Value* vecRes = UndefValue::get(IGCLLVM::FixedVectorType::get(llvm::Type::getIntNTy(I.getContext(), promoteToInt), quotient));
                     Value* Elmt;
                     Value* maskedElmt;
 
@@ -828,15 +829,15 @@ void PeepholeTypeLegalizer::cleanupZExtInst(Instruction& I) {
         promoteInt(srcWidth, quotient, promoteToInt, DL->getLargestLegalIntTypeSizeInBits());
 
         if (quotient * promoteToInt != I.getType()->getScalarSizeInBits()) {
-            assert(false && "Target size of zext is also illegal and needs promotion to a legal int or vec of largest legal int. Support for this extra legalization is not implemented yet.");
+            IGC_ASSERT_MESSAGE(0, "Target size of zext is also illegal and needs promotion to a legal int or vec of largest legal int. Support for this extra legalization is not implemented yet.");
             return;
         }
 
         unsigned ipElmtSize = prevInst->getOperand(0)->getType()->getScalarSizeInBits();
-        unsigned ipVecSize = prevInst->getOperand(0)->getType()->getVectorNumElements();
+        unsigned ipVecSize = (unsigned)cast<VectorType>(prevInst->getOperand(0)->getType())->getNumElements();
         unsigned convFactor = promoteToInt / ipElmtSize;
 
-        Value* vecRes = UndefValue::get(llvm::VectorType::get(llvm::Type::getIntNTy(I.getContext(), promoteToInt), quotient));
+        Value* vecRes = UndefValue::get(IGCLLVM::FixedVectorType::get(llvm::Type::getIntNTy(I.getContext(), promoteToInt), quotient));
         Type* resElmtTy = Type::getIntNTy(I.getContext(), promoteToInt);
         unsigned Idx = 0;
 
@@ -867,9 +868,9 @@ void PeepholeTypeLegalizer::cleanupZExtInst(Instruction& I) {
         Changed = true;
     }
     break;
-    //default:
-    //    printf("Unhandled source to ZExt Instruction seen with illegal int type. Legalization support missing. Source Inst opcode:%d", prevInst->getOpcode());
-    //    assert(false);
+    default:
+        IGC_ASSERT_MESSAGE(0, "Unhandled source to ZExt Instruction seen with illegal int type. Legalization support missing.");
+        break;
     }
 }
 
@@ -944,7 +945,7 @@ void PeepholeTypeLegalizer::cleanupBitCastInst(Instruction& I) {
         }
         break;
     }
-    /*default:
-        assert(false && "Unhandled source to BitCast Instruction seen with illegal int type. Legalization support missing.");*/
+    default:
+        break;
     }
 }
