@@ -36,12 +36,14 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 // srem, urem, frem.
 //
 //===----------------------------------------------------------------------===//
-#include "common/LLVMWarningsPush.hpp"
 
+#include "common/LLVMWarningsPush.hpp"
 #include "InstCombineInternal.h"
 #include "llvm/Analysis/InstructionSimplify.h"
 #include "llvm/IR/IntrinsicInst.h"
 #include "llvm/IR/PatternMatch.h"
+#include "Probe/Assertion.h"
+
 using namespace llvm;
 using namespace PatternMatch;
 using namespace IGCombiner;
@@ -118,8 +120,7 @@ static bool MultiplyOverflows(const APInt &C1, const APInt &C2, APInt &Product,
 /// \brief True if C2 is a multiple of C1. Quotient contains C2/C1.
 static bool IsMultiple(const APInt &C1, const APInt &C2, APInt &Quotient,
                        bool IsSigned) {
-  assert(C1.getBitWidth() == C2.getBitWidth() &&
-         "Inconsistent width of constants!");
+  IGC_ASSERT_MESSAGE(C1.getBitWidth() == C2.getBitWidth(), "Inconsistent width of constants!");
 
   // Bail if we will divide by zero.
   if (C2.isMinValue())
@@ -584,7 +585,7 @@ static bool isFMulOrFDivWithConstant(Value *V) {
 ///
 Value *InstCombiner::foldFMulConst(Instruction *FMulOrDiv, Constant *C,
                                    Instruction *InsertBefore) {
-  assert(isFMulOrFDivWithConstant(FMulOrDiv) && "V is invalid");
+  IGC_ASSERT_MESSAGE(isFMulOrFDivWithConstant(FMulOrDiv), "V is invalid");
 
   Value *Opnd0 = FMulOrDiv->getOperand(0);
   Value *Opnd1 = FMulOrDiv->getOperand(1);
@@ -1083,9 +1084,8 @@ static Instruction *foldUDivShl(Value *Op0, Value *Op1, const BinaryOperator &I,
     ShiftLeft = Op1;
 
   const APInt *CI = nullptr;
-  Value *N;
-  if (!match(ShiftLeft, m_Shl(m_APInt(CI), m_Value(N))))
-    llvm_unreachable("match should never fail here!");
+  Value *N = nullptr;
+  IGC_ASSERT_EXIT_MESSAGE(match(ShiftLeft, m_Shl(m_APInt(CI), m_Value(N))), "match should never fail here!");
   if (*CI != 1)
     N = IC.Builder->CreateAdd(N,
                               ConstantInt::get(N->getType(), CI->logBase2()));
