@@ -59,13 +59,11 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "llvm/IR/User.h"
 #include "llvm/IR/Value.h"
 #include "llvm/Support/Casting.h"
-#include "llvm/Support/ErrorHandling.h"
 #include "llvm/Transforms/InstCombine/InstCombineWorklist.h"
-
-#include <cassert>
 #include <cstdint>
 #include <iterator>
 #include <utility>
+#include "Probe/Assertion.h"
 
 using namespace llvm;
 using namespace PatternMatch;
@@ -324,8 +322,7 @@ Instruction* InstCombiner::visitExtractElementInst(ExtractElementInst& EI) {
 /// RHS, return the shuffle mask and true. Otherwise, return false.
 static bool collectSingleShuffleElements(Value* V, Value* LHS, Value* RHS,
     SmallVectorImpl<Constant*>& Mask) {
-    assert(LHS->getType() == RHS->getType() &&
-        "Invalid CollectSingleShuffleElements");
+    IGC_ASSERT_MESSAGE(LHS->getType() == RHS->getType(), "Invalid CollectSingleShuffleElements");
     unsigned NumElts = V->getType()->getVectorNumElements();
 
     if (isa<UndefValue>(V)) {
@@ -383,7 +380,7 @@ static bool collectSingleShuffleElements(Value* V, Value* LHS, Value* RHS,
                                     ExtractedIdx);
                         }
                         else {
-                            assert(EI->getOperand(0) == RHS);
+                            IGC_ASSERT(EI->getOperand(0) == RHS);
                             Mask[InsertedIdx % NumElts] =
                                 ConstantInt::get(Type::getInt32Ty(V->getContext()),
                                     ExtractedIdx + NumLHSElts);
@@ -489,7 +486,7 @@ static ShuffleOps collectShuffleElements(Value* V,
     SmallVectorImpl<Constant*>& Mask,
     Value* PermittedRHS,
     InstCombiner& IC) {
-    assert(V->getType()->isVectorTy() && "Invalid shuffle!");
+    IGC_ASSERT_MESSAGE(V->getType()->isVectorTy(), "Invalid shuffle!");
     unsigned NumElts = V->getType()->getVectorNumElements();
 
     if (isa<UndefValue>(V)) {
@@ -520,7 +517,7 @@ static ShuffleOps collectShuffleElements(Value* V,
                 if (EI->getOperand(0) == PermittedRHS || PermittedRHS == nullptr) {
                     Value* RHS = EI->getOperand(0);
                     ShuffleOps LR = collectShuffleElements(VecOp, Mask, RHS, IC);
-                    assert(LR.second == nullptr || LR.second == RHS);
+                    IGC_ASSERT(LR.second == nullptr || LR.second == RHS);
 
                     if (LR.first->getType() != RHS->getType()) {
                         // Although we are giving up for now, see if we can create extracts
@@ -793,7 +790,7 @@ static Instruction* foldConstantInsEltIntoShuffle(InsertElementInst& InsElt) {
         // into new value/mask vectors.
         for (uint64_t I : InsertIdx) {
             if (!Values[I]) {
-                assert(!Mask[I]);
+                IGC_ASSERT(!Mask[I]);
                 Values[I] = *ValI;
                 Mask[I] = ConstantInt::get(Type::getInt32Ty(InsElt.getContext()),
                     NumElts + I);
@@ -803,7 +800,7 @@ static Instruction* foldConstantInsEltIntoShuffle(InsertElementInst& InsElt) {
         // Remaining values are filled with 'undef' values.
         for (unsigned I = 0; I < NumElts; ++I) {
             if (!Values[I]) {
-                assert(!Mask[I]);
+                IGC_ASSERT(!Mask[I]);
                 Values[I] = UndefValue::get(InsElt.getType()->getElementType());
                 Mask[I] = ConstantInt::get(Type::getInt32Ty(InsElt.getContext()), I);
             }
@@ -995,7 +992,7 @@ static Value* buildNew(Instruction* I, ArrayRef<Value*> NewOps) {
     case Instruction::Or:
     case Instruction::Xor: {
         BinaryOperator* BO = cast<BinaryOperator>(I);
-        assert(NewOps.size() == 2 && "binary operator with #ops != 2");
+        IGC_ASSERT_MESSAGE(NewOps.size() == 2, "binary operator with #ops != 2");
         BinaryOperator* New =
             BinaryOperator::Create(cast<BinaryOperator>(I)->getOpcode(),
                 NewOps[0], NewOps[1], "", BO);
@@ -1011,11 +1008,11 @@ static Value* buildNew(Instruction* I, ArrayRef<Value*> NewOps) {
         return New;
     }
     case Instruction::ICmp:
-        assert(NewOps.size() == 2 && "icmp with #ops != 2");
+        IGC_ASSERT_MESSAGE(NewOps.size() == 2, "icmp with #ops != 2");
         return new ICmpInst(I, cast<ICmpInst>(I)->getPredicate(),
             NewOps[0], NewOps[1]);
     case Instruction::FCmp:
-        assert(NewOps.size() == 2 && "fcmp with #ops != 2");
+        IGC_ASSERT_MESSAGE(NewOps.size() == 2, "fcmp with #ops != 2");
         return new FCmpInst(I, cast<FCmpInst>(I)->getPredicate(),
             NewOps[0], NewOps[1]);
     case Instruction::Trunc:
@@ -1032,7 +1029,7 @@ static Value* buildNew(Instruction* I, ArrayRef<Value*> NewOps) {
         Type* DestTy =
             VectorType::get(I->getType()->getScalarType(),
                 NewOps[0]->getType()->getVectorNumElements());
-        assert(NewOps.size() == 1 && "cast with #ops != 1");
+        IGC_ASSERT_MESSAGE(NewOps.size() == 1, "cast with #ops != 1");
         return CastInst::Create(cast<CastInst>(I)->getOpcode(), NewOps[0], DestTy,
             "", I);
     }
@@ -1045,14 +1042,14 @@ static Value* buildNew(Instruction* I, ArrayRef<Value*> NewOps) {
         return GEP;
     }
     }
-    llvm_unreachable("failed to rebuild vector instructions");
+    IGC_ASSERT_EXIT_MESSAGE(0, "failed to rebuild vector instructions");
 }
 
 Value*
 InstCombiner::EvaluateInDifferentElementOrder(Value* V, ArrayRef<int> Mask) {
     // Mask.size() does not need to be equal to the number of vector elements.
 
-    assert(V->getType()->isVectorTy() && "can't reorder non-vector elements");
+    IGC_ASSERT_MESSAGE(V->getType()->isVectorTy(), "can't reorder non-vector elements");
     Type* EltTy = V->getType()->getScalarType();
     if (isa<UndefValue>(V))
         return UndefValue::get(VectorType::get(EltTy, Mask.size()));
@@ -1142,7 +1139,7 @@ InstCombiner::EvaluateInDifferentElementOrder(Value* V, ArrayRef<int> Mask) {
             Builder.getInt32(Index), "", I);
     }
     }
-    llvm_unreachable("failed to reorder elements of vector instruction!");
+    IGC_ASSERT_EXIT_MESSAGE(0, "failed to reorder elements of vector instruction!");
 }
 
 static void recognizeIdentityMask(const SmallVectorImpl<int>& Mask,
@@ -1222,7 +1219,7 @@ static BinopElts getAlternateBinop(BinaryOperator* BO, const DataLayout& DL) {
 }
 
 static Instruction* foldSelectShuffleWith1Binop(ShuffleVectorInst& Shuf) {
-    assert(Shuf.isSelect() && "Must have select-equivalent shuffle");
+    IGC_ASSERT_MESSAGE(Shuf.isSelect(), "Must have select-equivalent shuffle");
 
     // Are we shuffling together some value and that same value after it has been
     // modified by a binop with a constant?
@@ -1311,12 +1308,12 @@ static Instruction* foldSelectShuffle(ShuffleVectorInst& Shuf,
         if (Opc0 == Instruction::Shl || Opc1 == Instruction::Shl)
             DropNSW = true;
         if (BinopElts AltB0 = getAlternateBinop(B0, DL)) {
-            assert(isa<Constant>(AltB0.Op1) && "Expecting constant with alt binop");
+            IGC_ASSERT_MESSAGE(isa<Constant>(AltB0.Op1), "Expecting constant with alt binop");
             Opc0 = AltB0.Opcode;
             C0 = cast<Constant>(AltB0.Op1);
         }
         else if (BinopElts AltB1 = getAlternateBinop(B1, DL)) {
-            assert(isa<Constant>(AltB1.Op1) && "Expecting constant with alt binop");
+            IGC_ASSERT_MESSAGE(isa<Constant>(AltB1.Op1), "Expecting constant with alt binop");
             Opc1 = AltB1.Opcode;
             C1 = cast<Constant>(AltB1.Op1);
         }
@@ -1503,7 +1500,7 @@ Instruction* InstCombiner::visitShuffleVectorInst(ShuffleVectorInst& SVI) {
         VectorType* SrcTy = cast<VectorType>(V->getType());
         unsigned VecBitWidth = SrcTy->getBitWidth();
         unsigned SrcElemBitWidth = DL.getTypeSizeInBits(SrcTy->getElementType());
-        assert(SrcElemBitWidth && "vector elements must have a bitwidth");
+        IGC_ASSERT_MESSAGE(SrcElemBitWidth, "vector elements must have a bitwidth");
         unsigned SrcNumElems = SrcTy->getNumElements();
         SmallVector<BitCastInst*, 8> BCs;
         DenseMap<Type*, Value*> NewBCs;
@@ -1539,7 +1536,7 @@ Instruction* InstCombiner::visitShuffleVectorInst(ShuffleVectorInst& SVI) {
                 BegIdx = 0;
             }
             unsigned SrcElemsPerTgtElem = TgtElemBitWidth / SrcElemBitWidth;
-            assert(SrcElemsPerTgtElem);
+            IGC_ASSERT(SrcElemsPerTgtElem);
             BegIdx /= SrcElemsPerTgtElem;
             bool BCAlreadyExists = NewBCs.find(CastSrcTy) != NewBCs.end();
             auto* NewBC =
@@ -1699,8 +1696,7 @@ Instruction* InstCombiner::visitShuffleVectorInst(ShuffleVectorInst& SVI) {
                 // If the value selected is an undef value, explicitly specify it
                 // with a -1 mask value.
                 if (eltMask >= (int)RHSOp0Width) {
-                    assert(isa<UndefValue>(RHSShuffle->getOperand(1))
-                        && "should have been check above");
+                    IGC_ASSERT_MESSAGE(isa<UndefValue>(RHSShuffle->getOperand(1)), "should have been check above");
                     eltMask = -1;
                 }
             }
