@@ -23,15 +23,14 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 
 ======================= end_copyright_notice ==================================*/
+
 #include "MCSOptimization.hpp"
 #include "IGCPassSupport.h"
 #include "GenISAIntrinsics/GenIntrinsicInst.h"
 #include "Compiler/CodeGenPublic.h"
 #include "Compiler/WorkaroundAnalysisPass.h"
 #include "Compiler/CISACodeGen/ShaderCodeGen.hpp"
-
 #include <set>
-
 #include "common/LLVMWarningsPush.hpp"
 #include "llvm/IR/Function.h"
 #include <llvm/IR/InstVisitor.h>
@@ -41,7 +40,7 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "common/LLVMWarningsPop.hpp"
 #include "common/IGCIRBuilder.h"
 #include "common/igc_regkeys.hpp"
-
+#include "Probe/Assertion.h"
 
 using namespace llvm;
 using namespace IGC;
@@ -131,7 +130,7 @@ void MCSOptimization::visitCallInst(llvm::CallInst& I)
             const unsigned int shaderResourceViewMcsMaskIndex = textureIndex / BITS_PER_QWORD;
             const unsigned long long resourceViewMcsMaskElement = ctx->getModuleMetaData()->m_ShaderResourceViewMcsMask[shaderResourceViewMcsMaskIndex];
             const unsigned int resourceViewMaskTextureBit = textureIndex % BITS_PER_QWORD;
-            assert(textureIndex <= 127 && "Texture index is incorrectly extracted from ld_mcs");
+            IGC_ASSERT_MESSAGE(textureIndex <= 127, "Texture index is incorrectly extracted from ld_mcs");
 
             unsigned long long resultBit = resourceViewMcsMaskElement >> resourceViewMaskTextureBit;
             if ((resultBit & 1) == 0)
