@@ -106,29 +106,36 @@ bool MarkReadOnlyLoad::runOnFunction(Function& F)
 
 void MarkReadOnlyLoad::visitLoadInst(LoadInst& LI)
 {
-    Value* ldPtr = LI.getPointerOperand();
     bool isRO = false;
 
-    Value* srcPtr = TracePointerSource(ldPtr);
-    if (srcPtr && isa<GenIntrinsicInst>(srcPtr))
+    if (LI.getPointerAddressSpace() == ADDRESS_SPACE_CONSTANT)
     {
-        unsigned bufId;
-        BufferType bufTy;
-        BufferAccessType accTy;
-        bool needBufferOffset; // Unused
-
-        // check whether we are doing read only access on buffer (e.g. on UAV)
-        if (GetResourcePointerInfo(srcPtr, bufId, bufTy, accTy, needBufferOffset))
+        isRO = true;
+    }
+    else if (auto *srcPtr = TracePointerSource(LI.getPointerOperand()))
+    {
+        if (isa<GenIntrinsicInst>(srcPtr))
         {
-            if (accTy == BufferAccessType::ACCESS_READ)
+            unsigned bufId;
+            BufferType bufTy;
+            BufferAccessType accTy;
+            bool needBufferOffset; // Unused
+
+            // check whether we are doing read only access on buffer (e.g. on UAV)
+            if (GetResourcePointerInfo(
+                srcPtr, bufId, bufTy, accTy, needBufferOffset))
             {
-                isRO = true;
+                if (accTy == BufferAccessType::ACCESS_READ)
+                {
+                    isRO = true;
+                }
             }
         }
     }
 
     if (isRO)
     {
+        m_changed = true;
         LI.setMetadata(LLVMContext::MD_invariant_load, m_mdNode);
     }
 }
\ No newline at end of file
