@@ -24,9 +24,11 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 ======================= end_copyright_notice ==================================*/
 
+#include "llvm/Config/llvm-config.h"
 #include "Compiler/Optimizer/OpenCLPasses/AddressSpaceAliasAnalysis/AddressSpaceAliasAnalysis.h"
 #include "Compiler/CodeGenPublic.h"
 #include "Compiler/IGCPassSupport.h"
+#include "Probe/Assertion.h"
 
 using namespace llvm;
 using namespace IGC;
@@ -98,7 +100,7 @@ namespace {
             /// For some client APIs (e.g. vulkan) compiler is free to assume that
             /// resources bound to two different bindings points never alias unless a
             /// resource is explicitly marked as being aliased.
-            assert(CGC.getModuleMetaData());
+            IGC_ASSERT(CGC.getModuleMetaData());
             if (AS1 > ADDRESS_SPACE_NUM_ADDRESSES &&
                 AS2 > ADDRESS_SPACE_NUM_ADDRESSES &&
                 CGC.getModuleMetaData()->statefullResourcesNotAliased)
@@ -176,14 +178,16 @@ namespace {
         }
 
         bool doInitialization(Module& M) override {
-            Result.reset(new AddressSpaceAAResult(
-            getAnalysis<TargetLibraryInfoWrapperPass>().getTLI(
+            if(M.size() > 0)
+            {
+                Result.reset(new AddressSpaceAAResult(
+                getAnalysis<TargetLibraryInfoWrapperPass>().getTLI(
 #if LLVM_VERSION_MAJOR >= 10
-                // LLVM_UPGRADE_TODO - check how to feed this place with correct Function ref
-                *M.begin()
+                    *M.begin()
 #endif
-                ),
-                *getAnalysis<CodeGenContextWrapper>().getCodeGenContext()));
+                    ),
+                    *getAnalysis<CodeGenContextWrapper>().getCodeGenContext()));
+            }
             return false;
         }
 
