@@ -30,15 +30,17 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "Compiler/CodeGenPublicEnums.h"
 #include "Compiler/IGCPassSupport.h"
 #include "Compiler/MetaDataUtilsWrapper.h"
-
 #include "common/LLVMWarningsPush.hpp"
+#include "llvmWrapper/Support/Alignment.h"
 #include <llvm/IR/Module.h>
 #include <llvm/IR/Instructions.h>
 #include <llvm/IR/DataLayout.h>
 #include "common/LLVMWarningsPop.hpp"
+#include "Probe/Assertion.h"
 
 using namespace llvm;
 using namespace IGC;
+using IGCLLVM::getAlign;
 
 namespace {
     class GenericAddressAnalysis : public FunctionPass {
@@ -68,7 +70,7 @@ namespace {
 
 // Register pass
 #define PASS_FLAG2 "igc-generic-address-analysis"
-#define PASS_DESCRIPTION2 "Inserts implicit arguments when generic pointers are used"
+#define PASS_DESCRIPTION2 "Finds when generic pointers are used"
 #define PASS_CFG_ONLY2 false
 #define PASS_ANALYSIS2 false
 IGC_INITIALIZE_PASS_BEGIN(GenericAddressAnalysis, PASS_FLAG2, PASS_DESCRIPTION2, PASS_CFG_ONLY2, PASS_ANALYSIS2)
@@ -134,8 +136,9 @@ namespace {
         Type* getPointerAsIntType(LLVMContext& Ctx, unsigned AS);
         void resolveGAS(Instruction& I, Value* pointerOperand);
         void resolveGASWithoutBranches(Instruction& I, Value* pointerOperand);
+        bool allowArithmeticOnGenericAddressSpace(Function& F);
+        AddrSpaceCastInst* findAddressSpaceCastDef(Instruction* instruction, unsigned step);
     };
-
 } // namespace
 
 
@@ -157,6 +160,8 @@ bool GenericAddressDynamicResolution::runOnFunction(Function& F)
     bool modified = false;
     bool changed = false;
 
+    modified = allowArithmeticOnGenericAddressSpace(F);
+
     // iterate for all the intrinisics used by to_local, to_global, and to_private
     do {
         changed = false;
@@ -220,7 +225,7 @@ bool GenericAddressDynamicResolution::visitLoadStoreInst(Instruction& I)
         pointerAddressSpace = store->getPointerAddressSpace();
     }
     else {
-        report_fatal_error("Unable to resolve generic address space pointer");
+        IGC_ASSERT_EXIT_MESSAGE(0, "Unable to resolve generic address space pointer");
     }
 
     if (pointerAddressSpace == ADDRESS_SPACE_GENERIC) {
@@ -277,11 +282,11 @@ void GenericAddressDynamicResolution::resolveGAS(Instruction& I, Value* pointerO
 
         if (LoadInst* LI = dyn_cast<LoadInst>(&I))
         {
-            privateLoad = privateBuilder.CreateAlignedLoad(privatePtr, LI->getAlignment(), LI->isVolatile(), "privateLoad");
+            privateLoad = privateBuilder.CreateAlignedLoad(privatePtr, getAlign(LI->getAlignment()), LI->isVolatile(), "privateLoad");
         }
         else if (StoreInst* SI = dyn_cast<StoreInst>(&I))
         {
-            privateBuilder.CreateAlignedStore(I.getOperand(0), privatePtr, SI->getAlignment(), SI->isVolatile());
+            privateBuilder.CreateAlignedStore(I.getOperand(0), privatePtr, getAlign(SI->getAlignment()), SI->isVolatile());
         }
         privateBuilder.CreateBr(convergeBlock);
     }
@@ -297,11 +302,11 @@ void GenericAddressDynamicResolution::resolveGAS(Instruction& I, Value* pointerO
             Value* localPtr = localBuilder.CreateAddrSpaceCast(pointerOperand, localPtrType);
             if (LoadInst* LI = dyn_cast<LoadInst>(&I))
             {
-                localLoad = localBuilder.CreateAlignedLoad(localPtr, LI->getAlignment(), LI->isVolatile(), "localLoad");
+                localLoad = localBuilder.CreateAlignedLoad(localPtr, getAlign(LI->getAlignment()), LI->isVolatile(), "localLoad");
             }
             else if (StoreInst* SI = dyn_cast<StoreInst>(&I))
             {
-                localBuilder.CreateAlignedStore(I.getOperand(0), localPtr, SI->getAlignment(), SI->isVolatile());
+                localBuilder.CreateAlignedStore(I.getOperand(0), localPtr, getAlign(SI->getAlignment()), SI->isVolatile());
             }
             localBuilder.CreateBr(convergeBlock);
         }
@@ -316,11 +321,11 @@ void GenericAddressDynamicResolution::resolveGAS(Instruction& I, Value* pointerO
 
         if (LoadInst* LI = dyn_cast<LoadInst>(&I))
         {
-            globalLoad = globalBuilder.CreateAlignedLoad(globalPtr, LI->getAlignment(), LI->isVolatile(), "globalLoad");
+            globalLoad = globalBuilder.CreateAlignedLoad(globalPtr, getAlign(LI->getAlignment()), LI->isVolatile(), "globalLoad");
         }
         else if (StoreInst* SI = dyn_cast<StoreInst>(&I))
         {
-            globalBuilder.CreateAlignedStore(I.getOperand(0), globalPtr, SI->getAlignment(), SI->isVolatile());
+            globalBuilder.CreateAlignedStore(I.getOperand(0), globalPtr, getAlign(SI->getAlignment()), SI->isVolatile());
         }
         globalBuilder.CreateBr(convergeBlock);
     }
@@ -377,11 +382,11 @@ void GenericAddressDynamicResolution::resolveGASWithoutBranches(Instruction& I,
 
     if (LoadInst* LI = dyn_cast<LoadInst>(&I))
     {
-        nonLocalLoad = builder.CreateAlignedLoad(globalPtr, LI->getAlignment(), LI->isVolatile(), "globalOrPrivateLoad");
+        nonLocalLoad = builder.CreateAlignedLoad(globalPtr, getAlign(LI->getAlignment()), LI->isVolatile(), "globalOrPrivateLoad");
     }
     else if (StoreInst* SI = dyn_cast<StoreInst>(&I))
     {
-        builder.CreateAlignedStore(I.getOperand(0), globalPtr, SI->getAlignment(), SI->isVolatile());
+        builder.CreateAlignedStore(I.getOperand(0), globalPtr, getAlign(SI->getAlignment()), SI->isVolatile());
     }
 
     if (nonLocalLoad != nullptr)
@@ -406,7 +411,7 @@ bool GenericAddressDynamicResolution::visitIntrinsicCall(CallInst& I)
     if ((funcName == "__builtin_IB_to_private") || (funcName == "__builtin_IB_to_local")
         || (funcName == "__builtin_IB_to_global"))
     {
-        assert(I.getNumArgOperands() == 1);
+        IGC_ASSERT(I.getNumArgOperands() == 1);
         Value* arg = I.getArgOperand(0);
         PointerType* dstType = dyn_cast<PointerType>(I.getType());
         const unsigned targetAS = cast<PointerType>(I.getType())->getAddressSpace();
@@ -479,6 +484,88 @@ bool GenericAddressDynamicResolution::visitIntrinsicCall(CallInst& I)
     return changed;
 }
 
+// Add metadata to skip tagging generic pointers that are used for pointer arithmetic
+// We identify dependency of addrspacecast->inttoptr to recover the corresponding tag
+// after the arithmetic is done.
+bool GenericAddressDynamicResolution::allowArithmeticOnGenericAddressSpace(Function& F)
+{
+    LLVMContext& C = F.getContext();
+
+    bool modified = false;
+
+    SmallVector<AddrSpaceCastInst*, 8> ASCInsts;
+    SmallVector<IntToPtrInst*, 8> ITPInsts;
+
+    // iterate for all addrspacecast to generic pointers
+    for (inst_iterator i = inst_begin(F); i != inst_end(F); ++i)
+    {
+        AddrSpaceCastInst* addrSpaceCastInst = llvm::dyn_cast<AddrSpaceCastInst>(&*i);
+        IntToPtrInst* intToPtrInst = llvm::dyn_cast<IntToPtrInst>(&*i);
+        if (addrSpaceCastInst && addrSpaceCastInst->getDestAddressSpace() == ADDRESS_SPACE_GENERIC)
+        {
+            for (auto ui : addrSpaceCastInst->users())
+            {
+                Instruction* useInst = dyn_cast<Instruction>(ui);
+                PtrToIntInst* ptiInst = llvm::dyn_cast<PtrToIntInst>(&*useInst);
+                if (ptiInst && ptiInst->getPointerAddressSpace() == ADDRESS_SPACE_GENERIC)
+                {
+                    // add metadata to avoid tagging when emitting
+                    MDNode* N = MDNode::get(C, MDString::get(C, "generic.arith"));
+                    addrSpaceCastInst->setMetadata("generic.arith", N);
+                    ASCInsts.push_back(addrSpaceCastInst);
+                    modified = true;
+                }
+            }
+        }
+        else if (intToPtrInst && intToPtrInst->getAddressSpace() == ADDRESS_SPACE_GENERIC)
+        {
+            ITPInsts.push_back(intToPtrInst);
+        }
+    }
+
+    // for every IntToPtr find its origin Addrspacecast and source AS if it exists.
+    // Assumption: the first Addrspacecast found is used to determine the original
+    // source AS as arithmetic of pointers with different AS is not allowed.
+    for (auto I2P : ITPInsts)
+    {
+        AddrSpaceCastInst* ASCDef = findAddressSpaceCastDef(I2P, 8);
+        if (ASCDef)
+        {
+            for (auto ASC : ASCInsts)
+            {
+                if (ASC == ASCDef)
+                {
+                    MDNode* N = MDNode::get(C, ConstantAsMetadata::get(
+                        ConstantInt::get(Type::getInt32Ty(C), ASC->getSrcAddressSpace())));
+                    I2P->setMetadata("generic.arith", N);
+                    break;
+                }
+            }
+        }
+    }
+
+    return modified;
+}
+
+AddrSpaceCastInst* GenericAddressDynamicResolution::findAddressSpaceCastDef(Instruction* inst, unsigned step)
+{
+    if (AddrSpaceCastInst* addrSpaceCastInst = llvm::dyn_cast<AddrSpaceCastInst>(inst))
+        return addrSpaceCastInst;
+    if (step == 0)
+        return nullptr;
+
+    for (auto operand = inst->operands().begin();
+        operand != inst->operands().end(); ++operand)
+    {
+        auto* instOp = dyn_cast<llvm::Instruction>(operand->get());
+        if (instOp != nullptr)
+        {
+            return findAddressSpaceCastDef(instOp, step - 1);
+        }
+    }
+    return nullptr;
+}
+
 namespace IGC {
     FunctionPass* createGenericAddressAnalysisPass()
     {
