@@ -23,6 +23,7 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 
 ======================= end_copyright_notice ==================================*/
+
 #pragma once
 
 #include "common/LLVMWarningsPush.hpp"
@@ -30,9 +31,9 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <llvm/IR/InstVisitor.h>
 #include <llvm/IR/DataLayout.h>
 #include "common/LLVMWarningsPop.hpp"
-
 #include <vector>
 #include <map>
+#include "Probe/Assertion.h"
 
 namespace IGC
 {
@@ -79,8 +80,8 @@ namespace IGC
         /// @return offset in private buffer of given alloca instruction.
         unsigned int getBufferOffset(llvm::Function* pFunc, llvm::AllocaInst* pAI)
         {
-            assert(m_privateInfoMap.count(pFunc) && "Function should have private buffer info");
-            assert(m_privateInfoMap[pFunc].m_bufferOffsets.count(pAI) && "AllocalInst should have buffer offset info");
+            IGC_ASSERT_MESSAGE(m_privateInfoMap.count(pFunc), "Function should have private buffer info");
+            IGC_ASSERT_MESSAGE(m_privateInfoMap[pFunc].m_bufferOffsets.count(pAI), "AllocalInst should have buffer offset info");
             return m_privateInfoMap[pFunc].m_bufferOffsets[pAI];
         }
 
@@ -90,8 +91,8 @@ namespace IGC
         /// @return buffer size of given alloca instruction.
         unsigned int getBufferSize(llvm::Function* pFunc, llvm::AllocaInst* pAI)
         {
-            assert(m_privateInfoMap.count(pFunc) && "Function should have private buffer info");
-            assert(m_privateInfoMap[pFunc].m_bufferSizes.count(pAI) && "AllocalInst should have buffer size info");
+            IGC_ASSERT_MESSAGE(m_privateInfoMap.count(pFunc), "Function should have private buffer info");
+            IGC_ASSERT_MESSAGE(m_privateInfoMap[pFunc].m_bufferSizes.count(pAI), "AllocalInst should have buffer size info");
             return m_privateInfoMap[pFunc].m_bufferSizes[pAI];
         }
 
@@ -100,7 +101,7 @@ namespace IGC
         /// @return alloca instructions of given function.
         std::vector<llvm::AllocaInst*>& getAllocaInsts(llvm::Function* pFunc)
         {
-            assert(m_privateInfoMap.count(pFunc) && "Function should have private buffer info");
+            IGC_ASSERT_MESSAGE(m_privateInfoMap.count(pFunc), "Function should have private buffer info");
             return m_privateInfoMap[pFunc].m_allocaInsts;
         }
 
@@ -109,7 +110,7 @@ namespace IGC
         /// @return total private memory buffer size of given function.
         unsigned int getTotalPrivateMemPerWI(llvm::Function* pFunc)
         {
-            assert(m_privateInfoMap.count(pFunc) && "Function should have private buffer info");
+            IGC_ASSERT_MESSAGE(m_privateInfoMap.count(pFunc), "Function should have private buffer info");
             return m_privateInfoMap[pFunc].m_bufferTotalSize;
         }
 
