@@ -31,12 +31,13 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "Compiler/IGCPassSupport.h"
 #include "Compiler/CISACodeGen/GenCodeGenModule.h"
 #include "Compiler/CISACodeGen/LowerGEPForPrivMem.hpp"
-
 #include "common/LLVMWarningsPush.hpp"
+#include "llvm/Transforms/Utils/Local.h"
 #include "llvm/IR/DataLayout.h"
 #include "llvm/IR/DebugInfoMetadata.h"
 #include "llvm/IR/Dominators.h"
 #include "common/LLVMWarningsPop.hpp"
+#include "Probe/Assertion.h"
 
 using namespace llvm;
 using namespace IGC;
@@ -77,20 +78,22 @@ namespace IGC {
         bool runOnModule(llvm::Module& M) override;
 
         /// @brief  Resolve collected alloca instructions.
+        /// @param privateOnStack: whether the private variables are allocated on the stack
         /// @return true if there were resolved alloca, false otherwise.
-        bool resolveAllocaInstructions(bool stackCall);
+        bool resolveAllocaInstructions(bool privateOnStack);
 
         /// Initialize setup like UseScratchSpacePrivateMemory.
         bool safeToUseScratchSpace(llvm::Module& M) const;
 
     private:
-
         struct arrayIndex
         {
             llvm::GetElementPtrInst* gep;
             unsigned int operandIndex;
         };
 
+        static bool testTransposedMemory(const Type* pTmpType, const Type* const pTypeOfAccessedObject, uint64_t tmpAllocaSize, const uint64_t bufferSizeLimit);
+
         /// @brief  The module level alloca information
         ModuleAllocaInfo* m_ModAllocaInfo;
 
@@ -125,13 +128,17 @@ namespace IGC {
         ModuleAllocaInfo& operator=(const ModuleAllocaInfo&) = delete;
 
         /// \brief Return the offset of alloca instruction in private memory buffer.
-        unsigned getBufferOffset(AllocaInst* AI) const {
+        //  This function should not be called when AI is variable length alloca
+        unsigned getConstBufferOffset(AllocaInst* AI) const {
+            IGC_ASSERT(isa<ConstantInt>(AI->getArraySize()));
             Function* F = AI->getParent()->getParent();
             return getFuncAllocaInfo(F)->AllocaDesc[AI].first;
         }
 
         /// \brief Return the size of alloca instruction in private memory buffer.
-        unsigned getBufferSize(AllocaInst* AI) const {
+        //  This function should not be called when AI is variable length alloca
+        unsigned getConstBufferSize(AllocaInst* AI) const {
+            IGC_ASSERT(isa<ConstantInt>(AI->getArraySize()));
             Function* F = AI->getParent()->getParent();
             return getFuncAllocaInfo(F)->AllocaDesc[AI].second;
         }
@@ -208,7 +215,7 @@ namespace IGC {
 
 void ModuleAllocaInfo::analyze() {
     if (FGA && FGA->getModule()) {
-        assert(FGA->getModule() == M);
+        IGC_ASSERT(FGA->getModule() == M);
         for (auto FG : *FGA)
             analyze(FG);
     }
@@ -283,6 +290,7 @@ void ModuleAllocaInfo::analyze(Function* F, unsigned& Offset,
             Alignment = DL->getABITypeAlignment(AI->getAllocatedType());
         return Alignment;
     };
+
     std::sort(Allocas.begin(), Allocas.end(),
         [=](AllocaInst* AI1, AllocaInst* AI2) {
         return getAlignment(AI1) < getAlignment(AI2);
@@ -297,9 +305,13 @@ void ModuleAllocaInfo::analyze(Function* F, unsigned& Offset,
         if (Alignment > MaxAlignment)
             MaxAlignment = Alignment;
 
-        // Compute alloca size.
-        assert(isa<ConstantInt>(AI->getArraySize()));
-        ConstantInt* SizeVal = cast<ConstantInt>(AI->getArraySize());
+        // Compute alloca size. We don't know the variable length
+        // alloca size so skip it.
+        if (!isa<ConstantInt>(AI->getArraySize())) {
+            continue;
+        }
+        ConstantInt* const SizeVal = cast<ConstantInt>(AI->getArraySize());
+        IGC_ASSERT(nullptr != SizeVal);
         unsigned CurSize = (unsigned)(SizeVal->getZExtValue() *
             DL->getTypeAllocSize(AI->getAllocatedType()));
         AllocaInfo->setAllocaDesc(AI, Offset, CurSize);
@@ -492,9 +504,10 @@ bool PrivateMemoryResolution::runOnModule(llvm::Module& M)
         {
             continue;
         }
-        bool hasStackCall = (FGA && FGA->getGroup(m_currFunction)->hasStackCall());
+        bool hasStackCall = (FGA && FGA->getGroup(m_currFunction)->hasStackCall()) || m_currFunction->hasFnAttribute("visaStackCall");
+        bool hasVLA = (FGA && FGA->getGroup(m_currFunction)->hasVariableLengthAlloca()) || m_currFunction->hasFnAttribute("hasVLA");
         // Resolve collected alloca instructions for current function
-        changed |= resolveAllocaInstructions(hasStackCall);
+        changed |= resolveAllocaInstructions(hasStackCall || hasVLA);
     }
 
     if (changed)
@@ -539,7 +552,7 @@ bool PrivateMemoryResolution::runOnModule(llvm::Module& M)
 // [7] bar(j)
 //
 static void sinkAllocas(SmallVectorImpl<AllocaInst*>& Allocas) {
-    assert(!Allocas.empty());
+    IGC_ASSERT(false == Allocas.empty());
     DominatorTree DT;
     bool Calcuated = false;
 
@@ -615,7 +628,8 @@ public:
     }
     void handleLoadInst(LoadInst* pLoad, Value* pScalarizedIdx)
     {
-        assert(pLoad->isSimple());
+        IGC_ASSERT(nullptr != pLoad);
+        IGC_ASSERT(pLoad->isSimple());
         IRBuilder<> IRB(pLoad);
         if (isa<Instruction>(pLoad->getPointerOperand()))
         {
@@ -629,10 +643,12 @@ public:
         if (!vectorIO && pLoad->getType()->isVectorTy())
         {
             Type* scalarType = pLoad->getPointerOperand()->getType()->getPointerElementType()->getScalarType();
+            IGC_ASSERT(nullptr != scalarType);
             Type* scalarptrTy = PointerType::get(scalarType, pLoad->getPointerAddressSpace());
-            assert(scalarType->getPrimitiveSizeInBits() / 8 == elementSize);
+            IGC_ASSERT(scalarType->getPrimitiveSizeInBits() / 8 == elementSize);
             Value* vec = UndefValue::get(pLoad->getType());
-            for (unsigned i = 0, e = pLoad->getType()->getVectorNumElements(); i < e; ++i)
+            auto pLoadVT = cast<VectorType>(pLoad->getType());
+            for (unsigned i = 0, e = (unsigned)pLoadVT->getNumElements(); i < e; ++i)
             {
                 Value* ptr = IRB.CreateIntToPtr(address, scalarptrTy);
                 Value* v = IRB.CreateLoad(ptr);
@@ -650,7 +666,8 @@ public:
     }
     void handleStoreInst(StoreInst* pStore, Value* pScalarizedIdx)
     {
-        assert(pStore->isSimple());
+        IGC_ASSERT(nullptr != pStore);
+        IGC_ASSERT(pStore->isSimple());
         IRBuilder<> IRB(pStore);
         if (isa<Instruction>(pStore->getPointerOperand()))
         {
@@ -664,10 +681,13 @@ public:
         if (!vectorIO && pStore->getValueOperand()->getType()->isVectorTy())
         {
             Type* scalarType = pStore->getPointerOperand()->getType()->getPointerElementType()->getScalarType();
+            IGC_ASSERT(nullptr != scalarType);
             Type* scalarptrTy = PointerType::get(scalarType, pStore->getPointerAddressSpace());
-            assert(scalarType->getPrimitiveSizeInBits() / 8 == elementSize);
+            IGC_ASSERT(scalarType->getPrimitiveSizeInBits() / 8 == elementSize);
             Value* vec = pStore->getValueOperand();
-            for (unsigned i = 0, e = pStore->getValueOperand()->getType()->getVectorNumElements(); i < e; ++i)
+
+            unsigned vecNumElts = (unsigned)cast<VectorType>(vec->getType())->getNumElements();
+            for (unsigned i = 0; i < vecNumElts; ++i)
             {
                 Value* ptr = IRB.CreateIntToPtr(address, scalarptrTy);
                 IRB.CreateStore(IRB.CreateExtractElement(vec, IRB.getInt32(i)), ptr);
@@ -683,16 +703,91 @@ public:
     }
 };
 
+bool PrivateMemoryResolution::testTransposedMemory(const Type* pTmpType, const Type* const pTypeOfAccessedObject, uint64_t tmpAllocaSize, const uint64_t bufferSizeLimit)
+{
+    // verify that the size of transposed memory fits into the allocated scratch region
 
-bool PrivateMemoryResolution::resolveAllocaInstructions(bool stackCall)
+    bool ok = true;
+
+    if(ok)
+    {
+        ok = (nullptr != pTmpType);
+        IGC_ASSERT(ok);
+    }
+
+    if(ok)
+    {
+        ok = (nullptr != pTypeOfAccessedObject);
+        IGC_ASSERT(ok);
+    }
+
+    if(ok)
+    {
+        ok = (0 < tmpAllocaSize);
+        IGC_ASSERT(ok);
+    }
+
+    if(ok)
+    {
+        ok = (tmpAllocaSize <= bufferSizeLimit);
+        IGC_ASSERT(ok);
+    }
+
+    while(ok && (pTypeOfAccessedObject != pTmpType))
+    {
+        if(pTmpType->isStructTy() && (pTmpType->getStructNumElements() == 1))
+        {
+            pTmpType = pTmpType->getStructElementType(0);
+            ok = (nullptr != pTmpType);
+            IGC_ASSERT(ok);
+        }
+        else if(pTmpType->isArrayTy())
+        {
+            tmpAllocaSize *= pTmpType->getArrayNumElements();
+            pTmpType = pTmpType->getContainedType(0);
+            ok = (nullptr != pTmpType);
+            IGC_ASSERT(ok);
+        }
+        else if(pTmpType->isVectorTy())
+        {
+            auto pTmpVType = cast<VectorType>(pTmpType);
+            tmpAllocaSize *= pTmpVType->getNumElements();
+            pTmpType = pTmpType->getContainedType(0);
+            ok = (nullptr != pTmpType);
+            IGC_ASSERT(ok);
+        }
+        else
+        {
+            // unsupported type for memory transposition
+            ok = false;
+            IGC_ASSERT(ok);
+        }
+    }
+
+    if(ok)
+    {
+        ok = (0 < tmpAllocaSize);
+        IGC_ASSERT(ok);
+    }
+
+    if(ok)
+    {
+        ok = (tmpAllocaSize <= bufferSizeLimit);
+        IGC_ASSERT(ok);
+    }
+
+    return ok;
+}
+
+bool PrivateMemoryResolution::resolveAllocaInstructions(bool privateOnStack)
 {
     // It is possible that there is no alloca instruction in the caller but there
     // is alloca in the callee. Save the total private memory to the metadata.
     unsigned int totalPrivateMemPerWI = m_ModAllocaInfo->getTotalPrivateMemPerWI(m_currFunction);
 
     // This change is only till the FuncMD is ported to new MD framework
-    ModuleMetaData* modMD = getAnalysis<MetaDataUtilsWrapper>().getModuleMetaData();
-    assert(modMD && "Invalid metadata utils wrapper");
+    ModuleMetaData* const modMD = getAnalysis<MetaDataUtilsWrapper>().getModuleMetaData();
+    IGC_ASSERT(nullptr != modMD);
     modMD->FuncMD[m_currFunction].privateMemoryPerWI = totalPrivateMemPerWI;
     modMD->privateMemoryPerWI = totalPrivateMemPerWI;//redundant ?
 
@@ -703,7 +798,15 @@ bool PrivateMemoryResolution::resolveAllocaInstructions(bool stackCall)
         return false;
     }
 
-    sinkAllocas(allocaInsts);
+    // Disable sinking of alloca when optimizations are disabled. allocas (and associated bitcasts) dont
+    // contain any DebugLoc. Emitting low/high_pc for inlined subprograms becomes messy as allocas appear
+    // in subprogram prolog without any debug info. This makes debug info assume alloca code belongs to
+    // main program causing a break in low/high_pc of such subprograms. Not sinking allocas makes them
+    // appear in entry block of entry function. All such allocas get treated as being in prolog of
+    // entry function. This results in live-range extension, but with -O0 this is not a problem as debugging
+    // experience is more important than extra spills.
+    if(!modMD->compOpt.OptDisable)
+        sinkAllocas(allocaInsts);
     // If there are N+1 private buffers, and M+1 threads,
     // the layout representing the private memory will look like this:
 
@@ -752,10 +855,9 @@ bool PrivateMemoryResolution::resolveAllocaInstructions(bool stackCall)
     IF_DEBUG_INFO_IF(DISubprogram *subprogram = m_currFunction->getSubprogram(), entryDebugLoc = DebugLoc::get(subprogram->getLine(), 0, subprogram););
     IF_DEBUG_INFO(entryBuilder.SetCurrentDebugLocation(entryDebugLoc));
 
-    if (stackCall)
+    if (privateOnStack)
     {
         // Creates intrinsics that will be lowered in the CodeGen and will handle the stack-pointer
-        Function* stackAllocaFunc = GenISAIntrinsic::getDeclaration(m_currFunction->getParent(), GenISAIntrinsic::GenISA_StackAlloca);
         Instruction* simdLaneId16 = entryBuilder.CreateCall(simdLaneIdFunc, llvm::None, VALUE_NAME("simdLaneId16"));
         Value* simdLaneId = entryBuilder.CreateIntCast(simdLaneId16, typeInt32, false, VALUE_NAME("simdLaneId"));
         Instruction* simdSize = entryBuilder.CreateCall(simdSizeFunc, llvm::None, VALUE_NAME("simdSize"));
@@ -765,16 +867,49 @@ bool PrivateMemoryResolution::resolveAllocaInstructions(bool stackCall)
             llvm::IRBuilder<> builder(pAI);
             IF_DEBUG_INFO(builder.SetCurrentDebugLocation(entryDebugLoc));
 
-            // SP will be adjusted to include all the alloca space, therefore offset need to be adjusted back
-            int scalarBufferOffset = m_ModAllocaInfo->getBufferOffset(pAI) - totalPrivateMemPerWI;
-            unsigned int bufferSize = m_ModAllocaInfo->getBufferSize(pAI);
+            // buffer of this private var
+            Value* privateBuffer = nullptr;
+            if (!isa<ConstantInt>(pAI->getArraySize()))
+            {
+                // vla array must be AOS layout on stack
+                Value* increment = isUniform ? builder.getInt32(0) : simdLaneId;
+                // truncate alloca size to i32
+                Value* arraySize = builder.CreateTrunc(pAI->getArraySize(), increment->getType(), VALUE_NAME("TruncVLASize"));
+                Value* sizeWithType = builder.CreateMul(arraySize,
+                    builder.getInt32(static_cast<uint32_t>(m_currFunction->getParent()->getDataLayout().getTypeAllocSize(pAI->getAllocatedType()))),
+                    VALUE_NAME("VLASizeWithType"));
+                Value* perLaneOffset = builder.CreateMul(increment, sizeWithType, VALUE_NAME("VLAPerLaneOffset"));
+                // Create VLAStackAlloca intrinsic which will set private buffer offset to "SP + laneOffset",
+                // and set SP to "SP + buffer_size" in visa emitPass
+                Value* intrinArgs[] = { perLaneOffset, sizeWithType };
+                Function* stackAllocaFunc = GenISAIntrinsic::getDeclaration(m_currFunction->getParent(), GenISAIntrinsic::GenISA_VLAStackAlloca);
+                Value* stackAlloca = builder.CreateCall(stackAllocaFunc, intrinArgs , VALUE_NAME("VLAStackAlloca"));
+                privateBuffer = builder.CreatePointerCast(stackAlloca, pAI->getType(), VALUE_NAME(pAI->getName() + ".privateBuffer"));
+            }
+            else
+            {
+                int scalarBufferOffset = m_ModAllocaInfo->getConstBufferOffset(pAI);
+                unsigned int bufferSize = m_ModAllocaInfo->getConstBufferSize(pAI);
+
+                Value* bufferOffset = builder.CreateMul(simdSize, ConstantInt::get(typeInt32, scalarBufferOffset), VALUE_NAME(pAI->getName() + ".SIMDBufferOffset"));
+                Value* increment = isUniform ? builder.getInt32(0) : simdLaneId;
+                Value* perLaneOffset = builder.CreateMul(increment, ConstantInt::get(typeInt32, bufferSize), VALUE_NAME("perLaneOffset"));
+                Value* totalOffset = builder.CreateAdd(bufferOffset, perLaneOffset, VALUE_NAME(pAI->getName() + ".totalOffset"));
+                Function* stackAllocaFunc = GenISAIntrinsic::getDeclaration(m_currFunction->getParent(), GenISAIntrinsic::GenISA_StackAlloca);
+                Value* stackAlloca = builder.CreateCall(stackAllocaFunc, totalOffset, VALUE_NAME("stackAlloca"));
+                privateBuffer = builder.CreatePointerCast(stackAlloca, pAI->getType(), VALUE_NAME(pAI->getName() + ".privateBuffer"));
+                auto DbgUses = llvm::FindDbgAddrUses(pAI);
+                for (auto Use : DbgUses)
+                {
+                    if (auto DbgDcl = dyn_cast_or_null<DbgDeclareInst>(Use))
+                    {
+                        // Attach metadata to instruction containing offset of storage
+                        auto OffsetMD = MDNode::get(builder.getContext(), ConstantAsMetadata::get(builder.getInt32(scalarBufferOffset)));
+                        DbgDcl->setMetadata("StorageOffset", OffsetMD);
+                    }
+                }
+            }
 
-            Value* bufferOffset = builder.CreateMul(simdSize, ConstantInt::get(typeInt32, scalarBufferOffset), VALUE_NAME(pAI->getName() + ".SIMDBufferOffset"));
-            Value* increment = isUniform ? builder.getInt32(0) : simdLaneId;
-            Value* perLaneOffset = builder.CreateMul(increment, ConstantInt::get(typeInt32, bufferSize), VALUE_NAME("perLaneOffset"));
-            Value* totalOffset = builder.CreateAdd(bufferOffset, perLaneOffset, VALUE_NAME(pAI->getName() + ".totalOffset"));
-            Value* stackAlloca = builder.CreateCall(stackAllocaFunc, totalOffset, VALUE_NAME("stackAlloca"));
-            Value* privateBuffer = builder.CreatePointerCast(stackAlloca, pAI->getType(), VALUE_NAME(pAI->getName() + ".privateBuffer"));
             // Replace all uses of original alloca with the bitcast
             pAI->replaceAllUsesWith(privateBuffer);
             pAI->eraseFromParent();
@@ -820,7 +955,7 @@ bool PrivateMemoryResolution::resolveAllocaInstructions(bool stackCall)
             IF_DEBUG_INFO(builder.SetCurrentDebugLocation(entryDebugLoc));
 
             // Get buffer information from the analysis
-            unsigned int scalarBufferOffset = m_ModAllocaInfo->getBufferOffset(pAI);
+            unsigned int scalarBufferOffset = m_ModAllocaInfo->getConstBufferOffset(pAI);
 
             // If we can use SOA layout transpose the memory
             Type* pTypeOfAccessedObject = nullptr;
@@ -831,42 +966,11 @@ bool PrivateMemoryResolution::resolveAllocaInstructions(bool stackCall)
             {
                 auto DL = &m_currFunction->getParent()->getDataLayout();
                 bufferSize = (unsigned)DL->getTypeAllocSize(pTypeOfAccessedObject);
-
-#if defined(_DEBUG)
-                {
-                    // Debug code to verify that the size of transposed memory
-                    // fits into the allocated scratch region.
-                    Type* pTmpType = pAI->getType()->getPointerElementType();
-                    uint64_t tmpAllocaSize = bufferSize;
-                    while (pTmpType != pTypeOfAccessedObject)
-                    {
-                        if (pTmpType->isStructTy() && pTmpType->getStructNumElements() == 1)
-                        {
-                            pTmpType = pTmpType->getStructElementType(0);
-                        }
-                        else if (pTmpType->isArrayTy())
-                        {
-                            tmpAllocaSize *= pTmpType->getArrayNumElements();
-                            pTmpType = pTmpType->getSequentialElementType();
-                        }
-                        else if (pTmpType->isVectorTy())
-                        {
-                            tmpAllocaSize *= pTmpType->getVectorNumElements();
-                            pTmpType = pTmpType->getSequentialElementType();
-                        }
-                        else
-                        {
-                            assert(!"Unsupported type for memory transposition.");
-                            break;
-                        }
-                    }
-                    assert(tmpAllocaSize <= m_ModAllocaInfo->getBufferSize(pAI));
-                }
-#endif
+                IGC_ASSERT(testTransposedMemory((pAI->getType()->getPointerElementType()), pTypeOfAccessedObject, bufferSize, (m_ModAllocaInfo->getConstBufferSize(pAI))));
             }
             else
             {
-                bufferSize = m_ModAllocaInfo->getBufferSize(pAI);
+                bufferSize = m_ModAllocaInfo->getConstBufferSize(pAI);
             }
 
 
@@ -920,25 +1024,23 @@ bool PrivateMemoryResolution::resolveAllocaInstructions(bool stackCall)
     Value* simdLaneId = entryBuilder.CreateIntCast(simdLaneId16, typeInt32, false, VALUE_NAME("simdLaneId"));
     Instruction* simdSize = entryBuilder.CreateCall(simdSizeFunc, llvm::None, VALUE_NAME("simdSize"));
     Value* totalPrivateMemPerThread = entryBuilder.CreateMul(simdSize, totalPrivateMemPerWIValue, VALUE_NAME("totalPrivateMemPerThread"));
-    Value* r0_5 = entryBuilder.CreateExtractElement(r0Arg, ConstantInt::get(typeInt32, 5), VALUE_NAME("r0.5"));
-    ConstantInt* FFTIDMask = ConstantInt::get(typeInt32, Ctx.platform.getFFTIDBitMask());
-    Value* threadId = entryBuilder.CreateAnd(r0_5, FFTIDMask, VALUE_NAME("threadId"));
+
+    Function* pHWTIDFunc = GenISAIntrinsic::getDeclaration(m_currFunction->getParent(), GenISAIntrinsic::GenISA_hw_thread_id_alloca, Type::getInt32Ty(C));
+    Value* threadId = entryBuilder.CreateCall(pHWTIDFunc);
+
+    if (Ctx.platform.supportTwoStackTSG() && IGC_IS_FLAG_ENABLED(EnableGen11TwoStackTSG))
     {
-        if (Ctx.platform.supportTwoStackTSG() && IGC_IS_FLAG_ENABLED(EnableGen11TwoStackTSG))
-        {
-            // Gen11 , 2 - stack configuration : (FFTID[9:0] << 1) | FFSID[0]) * scratch_size
-            Value* shlThreadID = entryBuilder.CreateShl(threadId, ConstantInt::get(typeInt32, 1), VALUE_NAME("shlThreadID"));
+        // Gen11 , 2 - stack configuration : (FFTID[9:0] << 1) | FFSID[0]) * scratch_size
+        Value* shlThreadID = entryBuilder.CreateShl(threadId, ConstantInt::get(typeInt32, 1), VALUE_NAME("shlThreadID"));
 
-            // FFSID - r0.0 bit 16
-            Value* r0_0 = entryBuilder.CreateExtractElement(r0Arg, ConstantInt::get(typeInt32, 0), VALUE_NAME("r0.0"));
-            Value* FFSIDbit = entryBuilder.CreateLShr(r0_0, ConstantInt::get(typeInt32, 16), VALUE_NAME("FFSIDbit"));
-            Value* FFSID = entryBuilder.CreateAnd(FFSIDbit, ConstantInt::get(typeInt32, 1), VALUE_NAME("FFSID"));
+        // FFSID - r0.0 bit 16
+        Value* r0_0 = entryBuilder.CreateExtractElement(r0Arg, ConstantInt::get(typeInt32, 0), VALUE_NAME("r0.0"));
+        Value* FFSIDbit = entryBuilder.CreateLShr(r0_0, ConstantInt::get(typeInt32, 16), VALUE_NAME("FFSIDbit"));
+        Value* FFSID = entryBuilder.CreateAnd(FFSIDbit, ConstantInt::get(typeInt32, 1), VALUE_NAME("FFSID"));
 
-            threadId = entryBuilder.CreateOr(FFSID, shlThreadID, VALUE_NAME("threadId"));
-        }
+        threadId = entryBuilder.CreateOr(FFSID, shlThreadID, VALUE_NAME("threadId"));
     }
 
-
     Value* perThreadOffset = entryBuilder.CreateMul(threadId, totalPrivateMemPerThread, VALUE_NAME("perThreadOffset"));
 
     for (auto pAI : allocaInsts)
@@ -954,8 +1056,8 @@ bool PrivateMemoryResolution::resolveAllocaInstructions(bool stackCall)
         IF_DEBUG_INFO(builder.SetCurrentDebugLocation(entryDebugLoc));
         bool isUniform = pAI->getMetadata("uniform") != nullptr;
         // Get buffer information from the analysis
-        unsigned int scalarBufferOffset = m_ModAllocaInfo->getBufferOffset(pAI);
-        unsigned int bufferSize = m_ModAllocaInfo->getBufferSize(pAI);
+        unsigned int scalarBufferOffset = m_ModAllocaInfo->getConstBufferOffset(pAI);
+        unsigned int bufferSize = m_ModAllocaInfo->getConstBufferSize(pAI);
 
         Value* bufferOffset = builder.CreateMul(simdSize, ConstantInt::get(typeInt32, scalarBufferOffset), VALUE_NAME(pAI->getName() + ".SIMDBufferOffset"));
         Value* bufferOffsetForThread = builder.CreateAdd(perThreadOffset, bufferOffset, VALUE_NAME(pAI->getName() + ".bufferOffsetForThread"));
@@ -965,6 +1067,17 @@ bool PrivateMemoryResolution::resolveAllocaInstructions(bool stackCall)
         Value* privateBufferGEP = builder.CreateGEP(privateMemArg, totalOffset, VALUE_NAME(pAI->getName() + ".privateBufferGEP"));
         Value* privateBuffer = builder.CreatePointerCast(privateBufferGEP, pAI->getType(), VALUE_NAME(pAI->getName() + ".privateBuffer"));
 
+        auto DbgUses = llvm::FindDbgAddrUses(pAI);
+        for (auto Use : DbgUses)
+        {
+            if (auto DbgDcl = dyn_cast_or_null<DbgDeclareInst>(Use))
+            {
+                // Attach metadata to instruction containing offset of storage
+                auto OffsetMD = MDNode::get(builder.getContext(), ConstantAsMetadata::get(builder.getInt32(scalarBufferOffset)));
+                DbgDcl->setMetadata("StorageOffset", OffsetMD);
+            }
+        }
+
         // Replace all uses of original alloca with the bitcast
         pAI->replaceAllUsesWith(privateBuffer);
         pAI->eraseFromParent();
