@@ -27,12 +27,12 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "AdaptorCommon/ImplicitArgs.hpp"
 #include "Compiler/Optimizer/OpenCLPasses/ProgramScopeConstants/ProgramScopeConstantAnalysis.hpp"
 #include "Compiler/IGCPassSupport.h"
-
 #include "common/LLVMWarningsPush.hpp"
 #include <llvm/IR/Module.h>
 #include <llvm/IR/Function.h>
 #include <llvm/Analysis/ValueTracking.h>
 #include "common/LLVMWarningsPop.hpp"
+#include "Probe/Assertion.h"
 
 using namespace llvm;
 using namespace IGC;
@@ -59,9 +59,6 @@ bool ProgramScopeConstantAnalysis::runOnModule(Module& M)
     bool hasInlineConstantBuffer = false;
     bool hasInlineGlobalBuffer = false;
 
-    unsigned globalBufferAlignment = 0;
-    unsigned constantBufferAlignment = 0;
-
     BufferOffsetMap inlineProgramScopeOffsets;
 
     // maintains pointer information so we can patch in
@@ -74,12 +71,14 @@ bool ProgramScopeConstantAnalysis::runOnModule(Module& M)
     MetaDataUtils* mdUtils = getAnalysis<MetaDataUtilsWrapper>().getMetaDataUtils();
     ModuleMetaData* modMd = getAnalysis<MetaDataUtilsWrapper>().getModuleMetaData();
 
+    SmallVector<GlobalVariable*, 32> zeroInitializedGlobals;
+
     for (Module::global_iterator I = M.global_begin(), E = M.global_end(); I != E; ++I)
     {
         GlobalVariable* globalVar = &(*I);
 
-        PointerType* ptrType = cast<PointerType>(globalVar->getType());
-        assert(ptrType && "The type of a global variable must be a pointer type");
+        PointerType* const ptrType = cast<PointerType>(globalVar->getType());
+        IGC_ASSERT_MESSAGE(nullptr != ptrType, "The type of a global variable must be a pointer type");
 
         // Pointer's address space should be either constant or global
         // The ?: is a workaround for clang bug, clang creates string constants with private address sapce!
@@ -97,14 +96,14 @@ bool ProgramScopeConstantAnalysis::runOnModule(Module& M)
         if (AS != ADDRESS_SPACE_CONSTANT &&
             AS != ADDRESS_SPACE_GLOBAL)
         {
-            assert(0 && "program scope variable with unexpected address space");
+            IGC_ASSERT_MESSAGE(0, "program scope variable with unexpected address space");
             continue;
         }
 
         // The only way to get a null initializer is via an external variable.
         // Linking has already occurred; everything should be resolved.
         Constant* initializer = globalVar->getInitializer();
-        assert(initializer && "Constant must be initialized");
+        IGC_ASSERT_MESSAGE(initializer, "Constant must be initialized");
         if (!initializer)
         {
             continue;
@@ -122,13 +121,13 @@ bool ProgramScopeConstantAnalysis::runOnModule(Module& M)
         }
 
         DataVector* inlineProgramScopeBuffer = nullptr;
-        unsigned& bufferAlignment = (AS == ADDRESS_SPACE_GLOBAL) ? globalBufferAlignment : constantBufferAlignment;
         if (AS == ADDRESS_SPACE_GLOBAL)
         {
             if (!hasInlineGlobalBuffer)
             {
                 InlineProgramScopeBuffer ilpsb;
-                ilpsb.alignment = bufferAlignment;
+                ilpsb.alignment = 0;
+                ilpsb.allocSize = 0;
                 modMd->inlineGlobalBuffers.push_back(ilpsb);
                 hasInlineGlobalBuffer = true;
             }
@@ -138,23 +137,63 @@ bool ProgramScopeConstantAnalysis::runOnModule(Module& M)
         {
             if (!hasInlineConstantBuffer)
             {
+                // General constants
                 InlineProgramScopeBuffer ilpsb;
-                ilpsb.alignment = bufferAlignment;
+                ilpsb.alignment = 0;
+                ilpsb.allocSize = 0;
                 modMd->inlineConstantBuffers.push_back(ilpsb);
+
+                // String literals
+                InlineProgramScopeBuffer ilpsbString;
+                ilpsbString.alignment = 0;
+                ilpsbString.allocSize = 0;
+                modMd->inlineConstantBuffers.push_back(ilpsbString);
                 hasInlineConstantBuffer = true;
             }
-            inlineProgramScopeBuffer = &modMd->inlineConstantBuffers.back().Buffer;
+
+            // When ZeBin is enabled, constant variables that are string literals
+            // will be stored in the seconf const buffer
+            ConstantDataSequential* cds = dyn_cast<ConstantDataSequential>(initializer);
+            bool isStringConst = cds && (cds->isCString() || cds->isString());
+            if ((IGC_IS_FLAG_ENABLED(EnableZEBinary) || modMd->compOpt.EnableZEBinary) &&
+                isStringConst)
+            {
+                inlineProgramScopeBuffer = &modMd->inlineConstantBuffers[1].Buffer;
+            }
+            else
+            {
+                inlineProgramScopeBuffer = &modMd->inlineConstantBuffers[0].Buffer;
+            }
+        }
+
+        if (initializer->isZeroValue())
+        {
+            // For zero initialized values, we dont need to copy the data, just tell driver how much to allocate
+            // However, if it's used as a pointer value, we need to do patching and therefore cannot defer the offset calculation
+            bool hasPointerUser = false;
+            for (auto UI : globalVar->users())
+            {
+                if (isa<Constant>(UI) && UI->getType()->isPointerTy())
+                {
+                    hasPointerUser = true;
+                    break;
+                }
+            }
+            if (!hasPointerUser)
+            {
+                zeroInitializedGlobals.push_back(globalVar);
+                continue;
+            }
         }
 
         // Align the buffer.
-        // If this is the first constant, set the initial alignment, otherwise add padding.
-        if (!bufferAlignment)
-        {
-            bufferAlignment = m_DL->getPreferredAlignment(globalVar);
-        }
-        else
+        if (inlineProgramScopeBuffer->size() != 0)
         {
+#if LLVM_VERSION_MAJOR < 11
             alignBuffer(*inlineProgramScopeBuffer, m_DL->getPreferredAlignment(globalVar));
+#else
+            alignBuffer(*inlineProgramScopeBuffer, m_DL->getPreferredAlign(globalVar).value());
+#endif
         }
 
         // Ok, buffer is aligned, remember where this inline variable starts.
@@ -164,6 +203,29 @@ bool ProgramScopeConstantAnalysis::runOnModule(Module& M)
         addData(initializer, *inlineProgramScopeBuffer, pointerOffsetInfoList, inlineProgramScopeOffsets, AS);
     }
 
+    // Set the needed allocation size to the actual buffer size
+    if (hasInlineGlobalBuffer)
+        modMd->inlineGlobalBuffers.back().allocSize = modMd->inlineGlobalBuffers.back().Buffer.size();
+    if (hasInlineConstantBuffer)
+    {
+        modMd->inlineConstantBuffers[0].allocSize = modMd->inlineConstantBuffers[0].Buffer.size();
+        modMd->inlineConstantBuffers[1].allocSize = modMd->inlineConstantBuffers[1].Buffer.size();
+    }
+    // Calculate the correct offsets for zero-initialized globals/constants
+    // Total allocation size in runtime needs to include zero-init values, but data copied to compiler output can ignore them
+    for (auto globalVar : zeroInitializedGlobals)
+    {
+        unsigned AS = cast<PointerType>(globalVar->getType())->getAddressSpace();
+        unsigned &offset = (AS == ADDRESS_SPACE_GLOBAL) ? modMd->inlineGlobalBuffers.back().allocSize : modMd->inlineConstantBuffers[0].allocSize;
+#if LLVM_VERSION_MAJOR < 11
+        offset = iSTD::Align(offset, m_DL->getPreferredAlignment(globalVar));
+#else
+        offset = iSTD::Align(offset, m_DL->getPreferredAlign(globalVar).value());
+#endif
+        inlineProgramScopeOffsets[globalVar] = offset;
+        offset += (unsigned)(m_DL->getTypeAllocSize(globalVar->getType()->getPointerElementType()));
+    }
+
     if (inlineProgramScopeOffsets.size())
     {
         // Add globals tracked in metadata to the "llvm.used" list so they won't be deleted by optimizations
@@ -176,41 +238,45 @@ bool ProgramScopeConstantAnalysis::runOnModule(Module& M)
         IGC::appendToUsed(M, globalArray);
     }
 
-    if (hasInlineConstantBuffer)
+    // Always rely on relocation for ZEBinary, so only need to generate these
+    // implcit args when disabling ZEBinary
+    if (IGC_IS_FLAG_DISABLED(EnableZEBinary) &&
+        !modMd->compOpt.EnableZEBinary)
     {
-        // Just add the implicit argument to each function if a constant
-        // buffer has been created.  This will technically burn a patch
-        // token on kernels that don't actually use the buffer but it saves
-        // us having to walk the def-use chain (we can't just check if a
-        // constant is used in the kernel; for example, a global buffer
-        // may contain pointers that in turn point into the constant
-        // address space).
-        for (auto& pFunc : M)
+        if (hasInlineConstantBuffer)
         {
-            if (pFunc.isDeclaration()) continue;
-            // Don't add implicit arg if doing relocation
-            if (pFunc.hasFnAttribute("EnableGlobalRelocation")) continue;
+            // Just add the implicit argument to each function if a constant
+            // buffer has been created.  This will technically burn a patch
+            // token on kernels that don't actually use the buffer but it saves
+            // us having to walk the def-use chain (we can't just check if a
+            // constant is used in the kernel; for example, a global buffer
+            // may contain pointers that in turn point into the constant
+            // address space).
+            for (auto& pFunc : M)
+            {
+                if (pFunc.isDeclaration()) continue;
+                // Don't add implicit arg if doing relocation
+                if (pFunc.hasFnAttribute("visaStackCall")) continue;
 
-            SmallVector<ImplicitArg::ArgType, 1> implicitArgs;
-            implicitArgs.push_back(ImplicitArg::CONSTANT_BASE);
-            ImplicitArgs::addImplicitArgs(pFunc, implicitArgs, mdUtils);
+                SmallVector<ImplicitArg::ArgType, 1> implicitArgs;
+                implicitArgs.push_back(ImplicitArg::CONSTANT_BASE);
+                ImplicitArgs::addImplicitArgs(pFunc, implicitArgs, mdUtils);
+            }
         }
-        mdUtils->save(C);
-    }
 
-    if (hasInlineGlobalBuffer)
-    {
-        for (auto& pFunc : M)
+        if (hasInlineGlobalBuffer)
         {
-            if (pFunc.isDeclaration()) continue;
-            // Don't add implicit arg if doing relocation
-            if (pFunc.hasFnAttribute("EnableGlobalRelocation")) continue;
+            for (auto& pFunc : M)
+            {
+                if (pFunc.isDeclaration()) continue;
+                // Don't add implicit arg if doing relocation
+                if (pFunc.hasFnAttribute("visaStackCall")) continue;
 
-            SmallVector<ImplicitArg::ArgType, 1> implicitArgs;
-            implicitArgs.push_back(ImplicitArg::GLOBAL_BASE);
-            ImplicitArgs::addImplicitArgs(pFunc, implicitArgs, mdUtils);
+                SmallVector<ImplicitArg::ArgType, 1> implicitArgs;
+                implicitArgs.push_back(ImplicitArg::GLOBAL_BASE);
+                ImplicitArgs::addImplicitArgs(pFunc, implicitArgs, mdUtils);
+            }
         }
-        mdUtils->save(C);
     }
 
     // Setup the metadata for pointer patch info to be utilized during
@@ -243,11 +309,9 @@ bool ProgramScopeConstantAnalysis::runOnModule(Module& M)
             }
             else
             {
-                assert(0 && "trying to patch unsupported address space");
+                IGC_ASSERT_MESSAGE(0, "trying to patch unsupported address space");
             }
         }
-
-        mdUtils->save(C);
     }
 
     const bool changed = !inlineProgramScopeOffsets.empty();
@@ -256,6 +320,7 @@ bool ProgramScopeConstantAnalysis::runOnModule(Module& M)
         modMd->inlineProgramScopeOffsets[offset.first] = offset.second;
     }
 
+    // Update LLVM metadata based on IGC MetadataUtils
     if (changed)
     {
         mdUtils->save(C);
@@ -283,7 +348,7 @@ void ProgramScopeConstantAnalysis::alignBuffer(DataVector& buffer, unsigned int
 //
 static unsigned WalkCastsToFindNamedAddrSpace(const Value* val)
 {
-    assert(isa<PointerType>(val->getType()));
+    IGC_ASSERT(isa<PointerType>(val->getType()));
 
     const unsigned currAddrSpace = cast<PointerType>(val->getType())->getAddressSpace();
 
@@ -346,7 +411,7 @@ void ProgramScopeConstantAnalysis::addData(Constant* initializer,
         {
             const unsigned pointedToAddrSpace = WalkCastsToFindNamedAddrSpace(initializer);
 
-            assert(addressSpace == ADDRESS_SPACE_GLOBAL || addressSpace == ADDRESS_SPACE_CONSTANT);
+            IGC_ASSERT(addressSpace == ADDRESS_SPACE_GLOBAL || addressSpace == ADDRESS_SPACE_CONSTANT);
 
             // We can only patch global and constant pointers.
             if ((pointedToAddrSpace == ADDRESS_SPACE_GLOBAL ||
@@ -355,7 +420,7 @@ void ProgramScopeConstantAnalysis::addData(Constant* initializer,
                     addressSpace == ADDRESS_SPACE_CONSTANT))
             {
                 auto iter = inlineProgramScopeOffsets.find(ptrBase);
-                assert(iter != inlineProgramScopeOffsets.end());
+                IGC_ASSERT(iter != inlineProgramScopeOffsets.end());
 
                 const uint64_t pointeeOffset = iter->second + offset;
 
@@ -413,13 +478,13 @@ void ProgramScopeConstantAnalysis::addData(Constant* initializer,
             }
             else
             {
-                assert(0 && "unknown constant expression");
+                IGC_ASSERT_MESSAGE(0, "unknown constant expression");
             }
         }
         else
         {
             // What other shapes can pointers take at the program scope?
-            assert(0 && "unknown pointer shape encountered");
+            IGC_ASSERT_MESSAGE(0, "unknown pointer shape encountered");
         }
     }
     else if (const UndefValue * UV = dyn_cast<UndefValue>(initializer))
@@ -441,8 +506,9 @@ void ProgramScopeConstantAnalysis::addData(Constant* initializer,
         }
         else if (ce->getOpcode() == Instruction::IntToPtr)
         {
-            ConstantExpr* opExpr = dyn_cast<ConstantExpr>(ce->getOperand(0));
-            assert(opExpr && opExpr->getOpcode() == Instruction::PtrToInt && "Unexpected operand of IntToPtr");
+            ConstantExpr* const opExpr = dyn_cast<ConstantExpr>(ce->getOperand(0));
+            IGC_ASSERT_MESSAGE(nullptr != opExpr, "Unexpected operand of IntToPtr");
+            IGC_ASSERT_MESSAGE(opExpr->getOpcode() == Instruction::PtrToInt, "Unexpected operand of IntToPtr");
             addData(opExpr->getOperand(0), inlineProgramScopeBuffer, pointerOffsetInfoList, inlineProgramScopeOffsets, addressSpace);
         }
         else if (ce->getOpcode() == Instruction::PtrToInt)
@@ -451,7 +517,7 @@ void ProgramScopeConstantAnalysis::addData(Constant* initializer,
         }
         else
         {
-            assert(0 && "Unexpected constant expression type");
+            IGC_ASSERT_MESSAGE(0, "Unexpected constant expression type");
         }
     }
     else if (ConstantDataSequential * cds = dyn_cast<ConstantDataSequential>(initializer))
@@ -469,13 +535,17 @@ void ProgramScopeConstantAnalysis::addData(Constant* initializer,
     // If this is an sequential type which is not a CDS or zero, have to collect the values
     // element by element. Note that this is not exclusive with the two cases above, so the
     // order of ifs is meaningful.
-    else if (dyn_cast<CompositeType>(initializer->getType()))
+    else if (
+        initializer->getType()->isArrayTy() ||
+        initializer->getType()->isStructTy() ||
+        initializer->getType()->isVectorTy()
+    )
     {
         const int numElts = initializer->getNumOperands();
         for (int i = 0; i < numElts; ++i)
         {
             Constant* C = initializer->getAggregateElement(i);
-            assert(C && "getAggregateElement returned null, unsupported constant");
+            IGC_ASSERT_MESSAGE(C, "getAggregateElement returned null, unsupported constant");
             // Since the type may not be primitive, extra alignment is required.
             addData(C, inlineProgramScopeBuffer, pointerOffsetInfoList, inlineProgramScopeOffsets, addressSpace);
         }
@@ -494,13 +564,14 @@ void ProgramScopeConstantAnalysis::addData(Constant* initializer,
         }
         else
         {
-            assert(0 && "Unsupported constant type");
+            IGC_ASSERT_MESSAGE(0, "Unsupported constant type");
         }
 
-        int bitWidth = intVal.getBitWidth();
-        assert((bitWidth % 8 == 0) && (bitWidth <= 64) && "Unsupported bitwidth");
+        const int bitWidth = intVal.getBitWidth();
+        IGC_ASSERT_MESSAGE((bitWidth % 8 == 0), "Unsupported bitwidth");
+        IGC_ASSERT_MESSAGE((bitWidth <= 64), "Unsupported bitwidth");
 
-        const uint64_t* val = intVal.getRawData();
+        const uint64_t* const val = intVal.getRawData();
         inlineProgramScopeBuffer.insert(inlineProgramScopeBuffer.end(), (char*)val, ((char*)val) + (bitWidth / 8));
     }
 
