@@ -28,15 +28,14 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "Compiler/Optimizer/OpenCLPasses/ProgramScopeConstants/ProgramScopeConstantResolution.hpp"
 #include "Compiler/CodeGenPublic.h"
 #include "Compiler/IGCPassSupport.h"
-
 #include "common/LLVMWarningsPush.hpp"
-#include <llvmWrapper/IR/Function.h>
+#include <llvm/IR/Function.h>
 #include <llvm/IR/Module.h>
 #include <llvm/IR/Instructions.h>
 #include "common/LLVMWarningsPop.hpp"
-
 #include <vector>
 #include <map>
+#include "Probe/Assertion.h"
 
 using namespace llvm;
 using namespace IGC;
@@ -86,6 +85,12 @@ bool ProgramScopeConstantResolution::runOnModule(Module& M)
         // There are no constants, or no constants are used, so we have nothing to do.
         return false;
     }
+    if (IGC_IS_FLAG_ENABLED(EnableZEBinary) ||
+        modMD->compOpt.EnableZEBinary)
+    {
+        // ZEBinary always relies on relocation, so we can ignore this pass
+        return false;
+    }
 
     if (RunCautiously) {
         if (!needRunConservatively(M))
@@ -110,7 +115,7 @@ bool ProgramScopeConstantResolution::runOnModule(Module& M)
     {
         GlobalVariable* pGlobalVar = &(*I);
         PointerType* ptrType = cast<PointerType>(pGlobalVar->getType());
-        assert(ptrType && "The type of a global variable must be a pointer type");
+        IGC_ASSERT_MESSAGE(ptrType, "The type of a global variable must be a pointer type");
 
         // Pointer's address space should be either constant or global
         const unsigned AS = ptrType->getAddressSpace();
@@ -127,12 +132,12 @@ bool ProgramScopeConstantResolution::runOnModule(Module& M)
             // This is a workaround for clang bug, clang creates string constants with private address sapce!
             AS != ADDRESS_SPACE_PRIVATE)
         {
-            assert(0 && "program scope variable with unexpected address space");
+            IGC_ASSERT_MESSAGE(0, "program scope variable with unexpected address space");
             continue;
         }
 
         Constant* initializer = pGlobalVar->getInitializer();
-        assert(initializer && "Constant must be initialized");
+        IGC_ASSERT_MESSAGE(initializer, "Constant must be initialized");
         if (!initializer)
         {
             continue;
@@ -173,20 +178,20 @@ bool ProgramScopeConstantResolution::runOnModule(Module& M)
             Function* userFunc = user->getParent()->getParent();
 
             // Don't have implicit arg if doing relocation
-            if (userFunc->hasFnAttribute("EnableGlobalRelocation"))
+            if (userFunc->hasFnAttribute("visaStackCall"))
                 continue;
 
             // Skip unused internal functions.
             if (mdUtils->findFunctionsInfoItem(userFunc) == mdUtils->end_FunctionsInfo())
             {
-                assert(userFunc->use_empty() && Function::isDiscardableIfUnused(userFunc->getLinkage()));
+                IGC_ASSERT(userFunc->use_empty() && Function::isDiscardableIfUnused(userFunc->getLinkage()));
                 continue;
             }
 
             ImplicitArgs implicitArgs(*userFunc, mdUtils);
 
             // Find the implicit argument representing this constant.
-            unsigned int ImplicitArgsBaseIndex = IGCLLVM::GetFuncArgSize(userFunc) - implicitArgs.size();
+            unsigned int ImplicitArgsBaseIndex = userFunc->arg_size() - implicitArgs.size();
             unsigned int implicitArgIndex = implicitArgs.getArgIndex(argType);
             unsigned int implicitArgIndexInFunc = ImplicitArgsBaseIndex + implicitArgIndex;
             Function::arg_iterator bufArg = userFunc->arg_begin();
@@ -206,7 +211,7 @@ bool ProgramScopeConstantResolution::runOnModule(Module& M)
             }
 
             Value* bc = funcToVarSet[userFunc][pGlobalVar];
-            assert(bc != nullptr && "Program Scope buffer handling is broken!");
+            IGC_ASSERT_MESSAGE(bc != nullptr, "Program Scope buffer handling is broken!");
 
             // And actually use the bitcast.
             user->replaceUsesOfWith(pGlobalVar, bc);
