@@ -28,23 +28,25 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "Compiler/CodeGenPublic.h"
 #include "Compiler/IGCPassSupport.h"
 #include "common/igc_regkeys.hpp"
-
 #include "common/LLVMWarningsPush.hpp"
-
+#include "llvm/Config/llvm-config.h"
+#include "llvmWrapper/IR/DerivedTypes.h"
 #include "llvmWrapper/IR/Instructions.h"
-
-#include <llvm/IR/DerivedTypes.h>
-#include <llvm/IR/Module.h>
-#include <llvm/IR/Function.h>
-#include <llvm/IR/Instructions.h>
-#include <llvm/IR/Intrinsics.h>
-#include <llvm/IR/IntrinsicInst.h>
+#include "llvmWrapper/Support/Alignment.h"
+#include "llvmWrapper/Support/TypeSize.h"
+#include "llvm/IR/DerivedTypes.h"
+#include "llvm/IR/Module.h"
+#include "llvm/IR/Function.h"
+#include "llvm/IR/Instructions.h"
+#include "llvm/IR/Intrinsics.h"
+#include "llvm/IR/IntrinsicInst.h"
 #include "common/LLVMWarningsPop.hpp"
-
 #include <map>
+#include "Probe/Assertion.h"
 
 using namespace llvm;
 using namespace IGC;
+using IGCLLVM::getAlign;
 
 // Register pass to igc-opt
 #define PASS_FLAG "igc-replace-unsupported-intrinsics"
@@ -92,7 +94,7 @@ namespace {
 
         Type* Tys[] = { Dst->getType(), Src->getType(), Size->getType() };
         auto* M = MM->getParent()->getParent()->getParent();
-        Value* TheFn = Intrinsic::getDeclaration(M, Intrinsic::memcpy, Tys);
+        auto TheFn = Intrinsic::getDeclaration(M, Intrinsic::memcpy, Tys);
 
         return cast<MemCpyInst>(MemCpyInst::Create(TheFn, args));
     }
@@ -194,7 +196,9 @@ namespace {
         Type* ETy = VTy ? VTy->getElementType() : Ty;
         uint32_t sBits = (unsigned int)ScalarVal->getType()->getPrimitiveSizeInBits();
         uint32_t nBits = (unsigned int)ETy->getPrimitiveSizeInBits();
-        assert((nBits % sBits) == 0 && nBits <= 64 && "Type mismatch in replicateScalar!");
+        IGC_ASSERT(sBits);
+        IGC_ASSERT_MESSAGE((nBits % sBits) == 0, "Type mismatch in replicateScalar!");
+        IGC_ASSERT_MESSAGE(nBits <= 64, "Type mismatch in replicateScalar!");
         uint32_t ratio = nBits / sBits;
 
         IRBuilder<> Builder(InsertBefore);
@@ -291,21 +295,21 @@ namespace {
         Type* TyI8 = Type::getInt8Ty(C);
 
         uint32_t n = 0;
-        Vecs[n++] = VectorType::get(TyI32, 8);
+        Vecs[n++] = IGCLLVM::FixedVectorType::get(TyI32, 8);
         // CntI32 range [0, 7]
         if (CntI32 >= 4)
         {
-            Vecs[n++] = VectorType::get(TyI32, 4);
+            Vecs[n++] = IGCLLVM::FixedVectorType::get(TyI32, 4);
             CntI32 -= 4;
         }
         if (CntI32 == 3 && Align >= 4)
         {
-            Vecs[n++] = VectorType::get(TyI32, 3);
+            Vecs[n++] = IGCLLVM::FixedVectorType::get(TyI32, 3);
             CntI32 -= 3;
         }
         if (CntI32 >= 2)
         {
-            Vecs[n++] = VectorType::get(TyI32, 2);
+            Vecs[n++] = IGCLLVM::FixedVectorType::get(TyI32, 2);
             CntI32 -= 2;
         }
         if (CntI32 > 0)
@@ -313,12 +317,12 @@ namespace {
             Vecs[n++] = TyI32;
             CntI32 -= 1;
         }
-        assert(CntI32 == 0 && "Did not handle all types of i32");
+        IGC_ASSERT_MESSAGE(CntI32 == 0, "Did not handle all types of i32");
 
         // CntI8 range [0, 3]
         if (CntI8 >= 2)
         {
-            Vecs[n++] = VectorType::get(TyI8, 2);
+            Vecs[n++] = IGCLLVM::FixedVectorType::get(TyI8, 2);
             CntI8 -= 2;
         }
         if (CntI8 > 0)
@@ -326,7 +330,7 @@ namespace {
             Vecs[n++] = TyI8;
             CntI8 -= 1;
         }
-        assert(CntI8 == 0 && "Did not handle all types of i8");
+        IGC_ASSERT_MESSAGE(CntI8 == 0, "Did not handle all types of i8");
 
         L = n;
     }
@@ -370,7 +374,7 @@ namespace {
         Value* Dst = MC->getRawDest();
         Value* Src = MC->getRawSource();
         Value* LPCount = MC->getLength();
-        uint32_t Align = IGCLLVM::getDestAlignment(MC);
+        uint32_t Align = MC->getDestAlignment();
         const bool IsVolatile = MC->isVolatile();
         const uint32_t SrcAS = MC->getSourceAddressSpace();
         const uint32_t DstAS = MC->getDestAddressSpace();
@@ -417,8 +421,8 @@ namespace {
                     {
                         Value* tSrc = Builder.CreateConstGEP1_32(vSrc, i);
                         Value* tDst = Builder.CreateConstGEP1_32(vDst, i);
-                        LoadInst* L = Builder.CreateAlignedLoad(tSrc, Align, IsVolatile);
-                        (void)Builder.CreateAlignedStore(L, tDst, Align, IsVolatile);
+                        LoadInst* L = Builder.CreateAlignedLoad(tSrc, getAlign(Align), IsVolatile);
+                        (void)Builder.CreateAlignedStore(L, tDst, getAlign(Align), IsVolatile);
                     }
                 }
                 else
@@ -429,8 +433,8 @@ namespace {
                         IRBuilder<> B(&(*++BasicBlock::iterator(IV)));
                         Value* tSrc = B.CreateGEP(vSrc, IV);
                         Value* tDst = B.CreateGEP(vDst, IV);
-                        LoadInst* L = B.CreateAlignedLoad(tSrc, Align, IsVolatile);
-                        (void)B.CreateAlignedStore(L, tDst, Align, IsVolatile);
+                        LoadInst* L = B.CreateAlignedLoad(tSrc, getAlign(Align), IsVolatile);
+                        (void)B.CreateAlignedStore(L, tDst, getAlign(Align), IsVolatile);
                     }
                 }
 
@@ -455,8 +459,8 @@ namespace {
                 NewDst = BOfst > 0 ? Builder.CreateConstGEP1_32(Dst, BOfst) : Dst;
                 vSrc = Builder.CreateBitCast(SkipBitCast(NewSrc), PointerType::get(VecTys[i], SrcAS), "memcpy_rem");
                 vDst = Builder.CreateBitCast(SkipBitCast(NewDst), PointerType::get(VecTys[i], DstAS), "memcpy_rem");
-                LoadInst* L = Builder.CreateAlignedLoad(vSrc, Align, IsVolatile);
-                (void)Builder.CreateAlignedStore(L, vDst, Align, IsVolatile);
+                LoadInst* L = Builder.CreateAlignedLoad(vSrc, getAlign(Align), IsVolatile);
+                (void)Builder.CreateAlignedStore(L, vDst, getAlign(Align), IsVolatile);
                 BOfst += SZ;
             }
         }
@@ -470,8 +474,8 @@ namespace {
                 IRBuilder<> B(&(*++BasicBlock::iterator(IV)));
                 Value* tSrc = B.CreateGEP(Src, IV);
                 Value* tDst = B.CreateGEP(Dst, IV);
-                LoadInst* L = B.CreateAlignedLoad(tSrc, Align, IsVolatile);
-                (void)B.CreateAlignedStore(L, tDst, Align, IsVolatile);
+                LoadInst* L = B.CreateAlignedLoad(tSrc, getAlign(Align), IsVolatile);
+                (void)B.CreateAlignedStore(L, tDst, getAlign(Align), IsVolatile);
             }
         }
         MC->eraseFromParent();
@@ -483,7 +487,7 @@ namespace {
         Value* Dst = MM->getRawDest();
         Value* Src = MM->getRawSource();
         Value* LPCount = MM->getLength();
-        uint32_t Align = IGCLLVM::getDestAlignment(MM);
+        uint32_t Align = MM->getDestAlignment();
         if (Align == 0)
             Align = 1;
         const bool IsVolatile = MM->isVolatile();
@@ -595,8 +599,8 @@ namespace {
 
                     auto* vSrc = B.CreateBitCast(SkipBitCast(tSrc), PointerType::get(VecTys[i], SrcAS), "memcpy_rem");
                     auto* vDst = B.CreateBitCast(SkipBitCast(tDst), PointerType::get(VecTys[i], DstAS), "memcpy_rem");
-                    LoadInst* L = B.CreateAlignedLoad(vSrc, newAlign, IsVolatile);
-                    (void)B.CreateAlignedStore(L, vDst, newAlign, IsVolatile);
+                    LoadInst* L = B.CreateAlignedLoad(vSrc, getAlign(newAlign), IsVolatile);
+                    (void)B.CreateAlignedStore(L, vDst, getAlign(newAlign), IsVolatile);
                 }
 
                 // now emit the <8 x i32> stores
@@ -613,8 +617,8 @@ namespace {
                         unsigned idx = NewCount - 1 - i;
                         auto* tSrc = B.CreateConstGEP1_32(vSrc, idx);
                         auto* tDst = B.CreateConstGEP1_32(vDst, idx);
-                        LoadInst* L = B.CreateAlignedLoad(tSrc, newAlign, IsVolatile);
-                        (void)B.CreateAlignedStore(L, tDst, newAlign, IsVolatile);
+                        LoadInst* L = B.CreateAlignedLoad(tSrc, getAlign(newAlign), IsVolatile);
+                        (void)B.CreateAlignedStore(L, tDst, getAlign(newAlign), IsVolatile);
                     }
                 }
                 else
@@ -625,8 +629,8 @@ namespace {
                         IRBuilder<> B(&(*++BasicBlock::iterator(IV)));
                         Value* tSrc = B.CreateGEP(vSrc, IV);
                         Value* tDst = B.CreateGEP(vDst, IV);
-                        LoadInst* L = B.CreateAlignedLoad(tSrc, newAlign, IsVolatile);
-                        (void)B.CreateAlignedStore(L, tDst, newAlign, IsVolatile);
+                        LoadInst* L = B.CreateAlignedLoad(tSrc, getAlign(newAlign), IsVolatile);
+                        (void)B.CreateAlignedStore(L, tDst, getAlign(newAlign), IsVolatile);
                     }
                 }
             }
@@ -649,8 +653,8 @@ namespace {
                     IRBuilder<> B(&(*++BasicBlock::iterator(IV)));
                     Value* tSrc = B.CreateGEP(i8Src, IV);
                     Value* tDst = B.CreateGEP(i8Dst, IV);
-                    LoadInst* L = B.CreateAlignedLoad(tSrc, 1, IsVolatile);
-                    (void)B.CreateAlignedStore(L, tDst, 1, IsVolatile);
+                    LoadInst* L = B.CreateAlignedLoad(tSrc, getAlign(1), IsVolatile);
+                    (void)B.CreateAlignedStore(L, tDst, getAlign(1), IsVolatile);
                 }
             }
 
@@ -672,7 +676,7 @@ namespace {
         Value* Dst = MS->getRawDest();
         Value* Src = MS->getValue();
         Value* LPCount = MS->getLength();
-        uint32_t Align = IGCLLVM::getDestAlignment(MS);
+        uint32_t Align = MS->getDestAlignment();
         const bool IsVolatile = MS->isVolatile();
         const uint32_t AS = MS->getDestAddressSpace();
 
@@ -715,7 +719,7 @@ namespace {
                     for (unsigned i = 0; i < NewCount; ++i)
                     {
                         Value* tDst = Builder.CreateConstGEP1_32(vDst, i);
-                        (void)Builder.CreateAlignedStore(vSrc, tDst, Align, IsVolatile);
+                        (void)Builder.CreateAlignedStore(vSrc, tDst, getAlign(Align), IsVolatile);
                     }
                 }
                 else
@@ -725,7 +729,7 @@ namespace {
                     {
                         IRBuilder<> B(&(*++BasicBlock::iterator(IV)));
                         Value* tDst = B.CreateGEP(vDst, IV);
-                        (void)B.CreateAlignedStore(vSrc, tDst, Align, IsVolatile);
+                        (void)B.CreateAlignedStore(vSrc, tDst, getAlign(Align), IsVolatile);
                     }
                 }
 
@@ -750,7 +754,7 @@ namespace {
                 NewDst = BOfst > 0 ? Builder.CreateConstGEP1_32(Dst, BOfst) : Dst;
                 vSrc = replicateScalar(Src, VecTys[i], MS);
                 vDst = Builder.CreateBitCast(SkipBitCast(NewDst), PTy, "memset_rem");
-                (void)Builder.CreateAlignedStore(vSrc, vDst, Align, IsVolatile);
+                (void)Builder.CreateAlignedStore(vSrc, vDst, getAlign(Align), IsVolatile);
                 BOfst += SZ;
             }
         }
@@ -762,7 +766,7 @@ namespace {
             {
                 IRBuilder<> B(&(*++BasicBlock::iterator(IV)));
                 Value* tDst = B.CreateGEP(Dst, IV);
-                (void)B.CreateAlignedStore(Src, tDst, Align, IsVolatile);
+                (void)B.CreateAlignedStore(Src, tDst, getAlign(Align), IsVolatile);
             }
         }
         MS->eraseFromParent();
@@ -786,13 +790,13 @@ namespace {
         %r = or i8 %shlRes, %shrRes    // compose the final result
     */
     void replaceFunnelShift(IntrinsicInst * I) {
-        assert(I->getIntrinsicID() == Intrinsic::fshl ||
+        IGC_ASSERT(I->getIntrinsicID() == Intrinsic::fshl ||
             I->getIntrinsicID() == Intrinsic::fshr);
         IRBuilder<> Builder(I);
         unsigned sizeInBits = I->getArgOperand(0)->getType()->getScalarSizeInBits();
         Value* numBits = Builder.getIntN(sizeInBits, sizeInBits);
-        if (I->getType()->isVectorTy()) {
-            numBits = ConstantVector::getSplat(I->getType()->getVectorNumElements(), cast<Constant>(numBits));
+        if (auto IVT = dyn_cast<VectorType>(I->getType())) {
+            numBits = ConstantVector::getSplat(IGCLLVM::getElementCount(IVT->getNumElements()), cast<Constant>(numBits));
         }
         auto shiftModulo = Builder.CreateURem(I->getArgOperand(2), numBits);
         auto negativeShift = Builder.CreateSub(numBits, shiftModulo);
