@@ -24,27 +24,23 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 ======================= end_copyright_notice ==================================*/
 
+#include "Compiler/CISACodeGen/helper.h"
+#include "Compiler/IGCPassSupport.h"
 #include "Compiler/Optimizer/OCLBIUtils.h"
 #include "Compiler/Optimizer/CodeAssumption.hpp"
-#include "Compiler/IGCPassSupport.h"
-
 #include "Compiler/Optimizer/OpenCLPasses/StatelessToStatefull/StatelessToStatefull.hpp"
-
 #include "common/Stats.hpp"
 #include "common/secure_string.h"
-
 #include "common/LLVMWarningsPush.hpp"
-
 #include "llvmWrapper/IR/Instructions.h"
 #include "llvmWrapper/Support/Alignment.h"
-
-#include <llvmWrapper/IR/Function.h>
+#include <llvm/IR/Function.h>
 #include <llvm/IR/Instructions.h>
 #include <llvm/IR/GetElementPtrTypeIterator.h>
 #include <llvm/Analysis/ValueTracking.h>
 #include "common/LLVMWarningsPop.hpp"
-
 #include <string>
+#include "Probe/Assertion.h"
 
 using namespace llvm;
 using namespace IGC;
@@ -207,7 +203,7 @@ bool StatelessToStatefull::runOnFunction(llvm::Function& F)
 Argument* StatelessToStatefull::getBufferOffsetArg(Function* F, uint32_t ArgNumber)
 {
     uint32_t nImplicitArgs = m_pImplicitArgs->size();
-    uint32_t totalArgs = (uint32_t)IGCLLVM::GetFuncArgSize(F);
+    uint32_t totalArgs = (uint32_t)F->arg_size();
     uint32_t nExplicitArgs = (totalArgs - nImplicitArgs);
     uint32_t implicit_ix = m_pImplicitArgs->getNumberedArgIndex(ImplicitArg::BUFFER_OFFSET, ArgNumber);
     uint32_t arg_ix = nExplicitArgs + implicit_ix;
@@ -215,7 +211,7 @@ Argument* StatelessToStatefull::getBufferOffsetArg(Function* F, uint32_t ArgNumb
     for (; AI != AE && AI->getArgNo() != arg_ix; ++AI);
     if (AI == AE)
     {
-        assert(false && "Implicit arg for BUFFER_OFFSET is out of range!");
+        IGC_ASSERT_MESSAGE(0, "Implicit arg for BUFFER_OFFSET is out of range!");
         return nullptr;
     }
     Argument* arg = &*AI;
@@ -277,7 +273,7 @@ bool StatelessToStatefull::getOffsetFromGEP(
         Value* PtrOp = GEP->getPointerOperand();
         PointerType* PtrTy = dyn_cast<PointerType>(PtrOp->getType());
 
-        assert(PtrTy && "Only accept scalar pointer!");
+        IGC_ASSERT_MESSAGE(PtrTy, "Only accept scalar pointer!");
 
         Type* Ty = PtrTy;
         gep_type_iterator GTI = gep_type_begin(GEP);
@@ -335,6 +331,44 @@ bool StatelessToStatefull::getOffsetFromGEP(
     return true;
 }
 
+const KernelArg* StatelessToStatefull::gepIsFromKernelArgument(const PointerType& ptrType, GetElementPtrInst* gep)
+{
+    // skip if no gep at all
+    if (gep == nullptr)
+        return nullptr;
+
+    Value* base = gep->getPointerOperand()->stripPointerCasts();
+
+    // stripPointerCasts might skip addrSpaceCast, thus check if AS is still
+    // the original one.
+    unsigned int ptrAS = ptrType.getAddressSpace();
+    if (cast<PointerType>(base->getType())->getAddressSpace() == ptrAS && !isa<Instruction>(base))
+    {
+        if (const KernelArg* arg = getKernelArg(base))
+            return arg;
+    }
+    return nullptr;
+}
+
+bool StatelessToStatefull::pointerIsFromKernelArgument(Value& ptr)
+{
+    // find the last gep
+    Value* base = ptr.stripPointerCasts();
+    // gep : the last gep of pointer address, null if no GEP at all.
+    GetElementPtrInst* gep = nullptr;
+    while (isa<GetElementPtrInst>(base)) {
+        gep = static_cast<GetElementPtrInst*>(base);
+        base = gep->getPointerOperand()->stripPointerCasts();
+    }
+
+    if (gep == nullptr)
+        return false;
+
+    if (gepIsFromKernelArgument(*dyn_cast<PointerType>(ptr.getType()), gep) != nullptr)
+        return true;
+    return false;
+}
+
 bool StatelessToStatefull::pointerIsPositiveOffsetFromKernelArgument(
     Function* F, Value* V, Value*& offset, unsigned int& argNumber)
 {
@@ -355,17 +389,15 @@ bool StatelessToStatefull::pointerIsPositiveOffsetFromKernelArgument(
     AssumptionCache* AC = getAC(F);
 
     PointerType* ptrType = dyn_cast<PointerType>(V->getType());
-    assert(ptrType && "Expected scalar Pointer (No support to vector of pointers");
+    IGC_ASSERT_MESSAGE(ptrType, "Expected scalar Pointer (No support to vector of pointers");
     if (!ptrType || (ptrType->getAddressSpace() != ADDRESS_SPACE_GLOBAL &&
         ptrType->getAddressSpace() != ADDRESS_SPACE_CONSTANT))
     {
         return false;
     }
-    unsigned int ptrAS = ptrType->getAddressSpace();
 
     SmallVector<GetElementPtrInst*, 4> GEPs;
     Value* base = V->stripPointerCasts();
-
     // gep : the last gep of pointer address, null if no GEP at all.
     GetElementPtrInst* gep = nullptr;
     while (isa<GetElementPtrInst>(base)) {
@@ -374,71 +406,67 @@ bool StatelessToStatefull::pointerIsPositiveOffsetFromKernelArgument(
         base = gep->getPointerOperand()->stripPointerCasts();
     }
 
-    // stripPointerCasts might skip addrSpaceCast, thus check if AS is still
-    // the original one. Also, if base is still instruction, skip.
-    if (gep && cast<PointerType>(base->getType())->getAddressSpace() == ptrAS && !isa<Instruction>(base))
+    // if the last gep is from kerenl argument
+    if (const KernelArg * arg = gepIsFromKernelArgument(*ptrType, gep))
     {
-        if (const KernelArg * arg = getKernelArg(base))
+        // base is the argument!
+        argNumber = arg->getAssociatedArgNo();
+        bool gepProducesPositivePointer = true;
+
+        // An address needs to be DW-aligned in order to be a base
+        // in a surface state.  In another word, a unaligned argument
+        // cannot be used as a surface base unless buffer_offset is
+        // used, in which "argument + buffer_offset" is instead used
+        // as a surface base. (argument + buffer_offset is the original
+        // base of buffer created on host side, the original buffer is
+        // guarantted to be DW-aligned.)
+        //
+        // Note that implicit arg is always aligned.
+        bool isAlignedPointee =
+            (IGC_IS_FLAG_DISABLED(UseSubDWAlignedPtrArg) || arg->isImplicitArg())
+            ? true
+            : (getPointeeAlign(DL, base) >= 4);
+
+        // If m_hasBufferOffsetArg is true, the offset argument is added to
+        // the final offset to make it definitely positive. Thus skip checking
+        // if an offset is positive.
+        //
+        // Howerver, if m_hasoptionalBufferOffsetArg is true, the buffer offset
+        // is not generated if all offsets can be proven positive (this has
+        // performance benefit as adding buffer offset is an additional add).
+        // Also, if an argument is unaligned, buffer offset must be ON and used;
+        // otherwise, no stateful conversion for the argument can be carried out.
+        //
+        // Note that offset should be positive for any implicit ptr argument,
+        // so no need to prove it!
+        if (!arg->isImplicitArg() &&
+            isAlignedPointee &&
+            (!m_hasBufferOffsetArg || m_hasOptionalBufferOffsetArg) &&
+            IGC_IS_FLAG_DISABLED(SToSProducesPositivePointer))
         {
-            // base is the argument!
-            argNumber = arg->getAssociatedArgNo();
-            bool gepProducesPositivePointer = true;
-
-            // An address needs to be DW-aligned in order to be a base
-            // in a surface state.  In another word, a unaligned argument
-            // cannot be used as a surface base unless buffer_offset is
-            // used, in which "argument + buffer_offset" is instead used
-            // as a surface base. (argument + buffer_offset is the original
-            // base of buffer created on host side, the original buffer is
-            // guarantted to be DW-aligned.)
-            //
-            // Note that implicit arg is always aligned.
-            bool isAlignedPointee =
-                (IGC_IS_FLAG_DISABLED(UseSubDWAlignedPtrArg) || arg->isImplicitArg())
-                ? true
-                : (getPointeeAlign(DL, base) >= 4);
-
-            // If m_hasBufferOffsetArg is true, the offset argument is added to
-            // the final offset to make it definitely positive. Thus skip checking
-            // if an offset is positive.
-            //
-            // Howerver, if m_hasoptionalBufferOffsetArg is true, the buffer offset
-            // is not generated if all offsets can be proven positive (this has
-            // performance benefit as adding buffer offset is an additional add).
-            // Also, if an argument is unaligned, buffer offset must be ON and used;
-            // otherwise, no stateful conversion for the argument can be carried out.
-            //
-            // Note that offset should be positive for any implicit ptr argument,
-            // so no need to prove it!
-            if (!arg->isImplicitArg() &&
-                isAlignedPointee &&
-                (!m_hasBufferOffsetArg || m_hasOptionalBufferOffsetArg) &&
-                IGC_IS_FLAG_DISABLED(SToSProducesPositivePointer))
+            // This is for proving that the offset is positive.
+            for (int i = 0, sz = GEPs.size(); i < sz; ++i)
             {
-                // This is for proving that the offset is positive.
-                for (int i = 0, sz = GEPs.size(); i < sz; ++i)
+                GetElementPtrInst* tgep = GEPs[i];
+                for (auto U = tgep->idx_begin(), E = tgep->idx_end(); U != E; ++U)
                 {
-                    GetElementPtrInst* tgep = GEPs[i];
-                    for (auto U = tgep->idx_begin(), E = tgep->idx_end(); U != E; ++U)
-                    {
-                        Value* Idx = U->get();
-                        gepProducesPositivePointer &=
-                            valueIsPositive(Idx, &(F->getParent()->getDataLayout()), AC);
-                    }
-                }
-
-                if (m_hasOptionalBufferOffsetArg)
-                {
-                    updateArgInfo(arg, gepProducesPositivePointer);
+                    Value* Idx = U->get();
+                    gepProducesPositivePointer &=
+                        valueIsPositive(Idx, &(F->getParent()->getDataLayout()), AC);
                 }
             }
-            if ((m_hasBufferOffsetArg ||
-                 (gepProducesPositivePointer && isAlignedPointee)) &&
-                getOffsetFromGEP(F, GEPs, argNumber, arg->isImplicitArg(), offset))
+
+            if (m_hasOptionalBufferOffsetArg)
             {
-                return true;
+                updateArgInfo(arg, gepProducesPositivePointer);
             }
         }
+        if ((m_hasBufferOffsetArg ||
+             (gepProducesPositivePointer && isAlignedPointee)) &&
+            getOffsetFromGEP(F, GEPs, argNumber, arg->isImplicitArg(), offset))
+        {
+            return true;
+        }
     }
 
     return false;
@@ -446,12 +474,48 @@ bool StatelessToStatefull::pointerIsPositiveOffsetFromKernelArgument(
 
 void StatelessToStatefull::visitCallInst(CallInst& I)
 {
+    auto doPromoteUntypedAtomics = [](const GenISAIntrinsic::ID intrinID, const GenIntrinsicInst* Inst)-> bool
+    {
+        // Only promote if oprand0 and oprand1 are the same for 64bit-pointer atomics
+        if (intrinID == GenISAIntrinsic::GenISA_intatomicrawA64 ||
+            intrinID == GenISAIntrinsic::GenISA_icmpxchgatomicrawA64 ||
+            intrinID == GenISAIntrinsic::GenISA_floatatomicrawA64 ||
+            intrinID == GenISAIntrinsic::GenISA_fcmpxchgatomicrawA64)
+        {
+            if (Inst->getOperand(0) != Inst->getOperand(1))
+            {
+                return false;
+            }
+        }
+
+        // Qword untyped atomic int only support A64, so can't promote to statefull
+        if (Inst->getType()->isIntegerTy() && Inst->getType()->getScalarSizeInBits() == 64)
+        {
+            return false;
+        }
+
+        return true;
+    };
+
+    auto isUntypedAtomics = [](const GenISAIntrinsic::ID intrinID)-> bool
+    {
+        return (intrinID == GenISAIntrinsic::GenISA_intatomicraw ||
+            intrinID == GenISAIntrinsic::GenISA_floatatomicraw ||
+            intrinID == GenISAIntrinsic::GenISA_intatomicrawA64 ||
+            intrinID == GenISAIntrinsic::GenISA_floatatomicrawA64 ||
+            intrinID == GenISAIntrinsic::GenISA_icmpxchgatomicraw ||
+            intrinID == GenISAIntrinsic::GenISA_fcmpxchgatomicraw ||
+            intrinID == GenISAIntrinsic::GenISA_icmpxchgatomicrawA64 ||
+            intrinID == GenISAIntrinsic::GenISA_fcmpxchgatomicrawA64);
+    };
+
     if (auto Inst = dyn_cast<GenIntrinsicInst>(&I))
     {
         GenISAIntrinsic::ID const intrinID = Inst->getIntrinsicID();
 
         if (intrinID == GenISAIntrinsic::GenISA_simdBlockRead ||
-            intrinID == GenISAIntrinsic::GenISA_simdBlockWrite)
+            intrinID == GenISAIntrinsic::GenISA_simdBlockWrite ||
+            (isUntypedAtomics(intrinID) && doPromoteUntypedAtomics(intrinID, Inst)))
         {
             Module* M = Inst->getParent()->getParent()->getParent();
             Function* F = Inst->getParent()->getParent();
@@ -472,7 +536,7 @@ void StatelessToStatefull::visitCallInst(CallInst& I)
                 ModuleMetaData* modMD = getAnalysis<MetaDataUtilsWrapper>().getModuleMetaData();
                 FunctionMetaData* funcMD = &modMD->FuncMD[F];
                 ResourceAllocMD* resAllocMD = &funcMD->resAllocMD;
-                assert(resAllocMD->argAllocMDList.size() > 0 && "ArgAllocMDList is empty.");
+                IGC_ASSERT_MESSAGE(resAllocMD->argAllocMDList.size() > 0, "ArgAllocMDList is empty.");
                 ArgAllocMD* argAlloc = &resAllocMD->argAllocMDList[baseArgNumber];
 
                 Constant* resourceNumber = ConstantInt::get(int32Ty, argAlloc->indexType);
@@ -492,6 +556,34 @@ void StatelessToStatefull::visitCallInst(CallInst& I)
                     Inst->replaceAllUsesWith(simdMediaBlockRead);
                     Inst->eraseFromParent();
                 }
+                else if (isUntypedAtomics(intrinID))
+                {
+                    PointerType* pTy = PointerType::get(dyn_cast<PointerType>(ptr->getType())->getElementType(), addrSpace);
+                    Instruction* pPtrToInt = IntToPtrInst::Create(Instruction::IntToPtr, offset, pTy, "", Inst);
+                    Instruction* pIntrinInst = nullptr;
+                    if (intrinID == GenISAIntrinsic::GenISA_intatomicrawA64 ||
+                        intrinID == GenISAIntrinsic::GenISA_icmpxchgatomicrawA64 ||
+                        intrinID == GenISAIntrinsic::GenISA_floatatomicrawA64 ||
+                        intrinID == GenISAIntrinsic::GenISA_fcmpxchgatomicrawA64)
+                    {
+                        pIntrinInst = CallInst::Create(
+                            GenISAIntrinsic::getDeclaration(M, intrinID, { Inst->getType(), pTy, pTy }),
+                            { pPtrToInt, pPtrToInt, Inst->getOperand(2), Inst->getOperand(3) },
+                            "",
+                            Inst);
+                    }
+                    else
+                    {
+                        pIntrinInst = CallInst::Create(
+                            GenISAIntrinsic::getDeclaration(M, intrinID, { Inst->getType(), pTy }),
+                            { pPtrToInt, offset, Inst->getOperand(2), Inst->getOperand(3) },
+                            "",
+                            Inst);
+                    }
+                    pIntrinInst->setDebugLoc(DL);
+                    Inst->replaceAllUsesWith(pIntrinInst);
+                    Inst->eraseFromParent();
+                }
                 else
                 {
                     PointerType* pTy = PointerType::get(Inst->getOperand(1)->getType(), addrSpace);
@@ -513,7 +605,6 @@ void StatelessToStatefull::visitCallInst(CallInst& I)
             }
         }
     }
-
 }
 
 void StatelessToStatefull::visitLoadInst(LoadInst& I)
@@ -531,7 +622,7 @@ void StatelessToStatefull::visitLoadInst(LoadInst& I)
         ModuleMetaData* modMD = getAnalysis<MetaDataUtilsWrapper>().getModuleMetaData();
         FunctionMetaData* funcMD = &modMD->FuncMD[F];
         ResourceAllocMD* resAllocMD = &funcMD->resAllocMD;
-        assert(resAllocMD->argAllocMDList.size() > 0 && "ArgAllocMDList is empty.");
+        IGC_ASSERT_MESSAGE(resAllocMD->argAllocMDList.size() > 0, "ArgAllocMDList is empty.");
         ArgAllocMD* argAlloc = &resAllocMD->argAllocMDList[baseArgNumber];
 
         Constant* resourceNumber = ConstantInt::get(int32Ty, argAlloc->indexType);
@@ -544,7 +635,7 @@ void StatelessToStatefull::visitLoadInst(LoadInst& I)
         Instruction* pPtrToInt = IntToPtrInst::Create(Instruction::IntToPtr, offset, pTy, "", &I);
         pPtrToInt->setDebugLoc(DL);
 
-        Instruction* pLoad = new LoadInst(pPtrToInt, "", I.isVolatile(), MaybeAlign(I.getAlignment()), I.getOrdering(), IGCLLVM::getSyncScopeID(&I), &I);
+        Instruction* pLoad = new LoadInst(pPtrToInt->getType()->getPointerElementType(), pPtrToInt, "", I.isVolatile(), IGCLLVM::getCorrectAlign(I.getAlignment()), I.getOrdering(), I.getSyncScopeID(), &I);
         pLoad->setDebugLoc(DL);
 
         PointerType* ptrType = dyn_cast<PointerType>(ptr->getType());
@@ -582,7 +673,7 @@ void StatelessToStatefull::visitStoreInst(StoreInst& I)
             ModuleMetaData* modMD = getAnalysis<MetaDataUtilsWrapper>().getModuleMetaData();
             FunctionMetaData* funcMD = &modMD->FuncMD[F];
             ResourceAllocMD* resAllocMD = &funcMD->resAllocMD;
-            assert(resAllocMD->argAllocMDList.size() > 0 && "ArgAllocMDList is empty.");
+            IGC_ASSERT_MESSAGE(resAllocMD->argAllocMDList.size() > 0, "ArgAllocMDList is empty.");
             ArgAllocMD* argAlloc = &resAllocMD->argAllocMDList[baseArgNumber];
             Constant* resourceNumber = ConstantInt::get(int32Ty, argAlloc->indexType);
 
@@ -594,7 +685,7 @@ void StatelessToStatefull::visitStoreInst(StoreInst& I)
             Instruction* pPtrToInt = IntToPtrInst::Create(Instruction::IntToPtr, offset, pTy, "", &I);
             pPtrToInt->setDebugLoc(DL);
 
-            Instruction* pStore = new StoreInst(dataVal, pPtrToInt, I.isVolatile(), MaybeAlign(I.getAlignment()), I.getOrdering(), IGCLLVM::getSyncScopeID(&I), &I);
+            Instruction* pStore = new StoreInst(dataVal, pPtrToInt, I.isVolatile(), IGCLLVM::getCorrectAlign(I.getAlignment()), I.getOrdering(), I.getSyncScopeID(), &I);
             pStore->setDebugLoc(DL);
 
             I.eraseFromParent();
@@ -701,7 +792,7 @@ void StatelessToStatefull::finalizeArgInitialValue(Function* F)
         if (allOffsetPositive)
         {
             const KernelArg* offsetArg = getBufferOffsetKernelArg(kernelArg);
-            assert(offsetArg && "Missing BufferOffset arg!");
+            IGC_ASSERT_MESSAGE(offsetArg, "Missing BufferOffset arg!");
             Value* BufferOffsetArg = const_cast<Argument*>(offsetArg->getArg());
             BufferOffsetArg->replaceAllUsesWith(ZeroValue);
         }
