@@ -31,7 +31,7 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <llvm/IR/Function.h>
 #include <llvm/IR/Instructions.h>
 #include "common/LLVMWarningsPop.hpp"
-
+#include "Probe/Assertion.h"
 
 using namespace llvm;
 using namespace IGC;
@@ -235,13 +235,6 @@ bool SubGroupFuncsResolution::runOnFunction(Function& F)
     return m_changed;
 }
 
-// Debug line info helper function
-static void updateDebugLoc(Instruction* pOrigin, Instruction* pNew)
-{
-    assert(pOrigin && pNew && "Expect valid instructions");
-    pNew->setDebugLoc(pOrigin->getDebugLoc());
-}
-
 // Helps to obtain temporary index corresponding to the kernel argument.
 // This index will be used during codegen to resolve BTIs for Images (SRVs and UAVs).
 void SubGroupFuncsResolution::BTIHelper(llvm::CallInst& CI)
@@ -254,7 +247,7 @@ void SubGroupFuncsResolution::BTIHelper(llvm::CallInst& CI)
         int argNo = (*arg).getArgNo();
         FunctionMetaData* funcMD = &modMD->FuncMD[F];
         ResourceAllocMD* resAllocMD = &funcMD->resAllocMD;
-        assert((size_t)argNo < resAllocMD->argAllocMDList.size() && "ArgAllocMD List Out of Bounds");
+        IGC_ASSERT_MESSAGE((size_t)argNo < resAllocMD->argAllocMDList.size(), "ArgAllocMD List Out of Bounds");
         ArgAllocMD* argAlloc = &resAllocMD->argAllocMDList[argNo];
         m_argIndexMap[&(*arg)] = CImagesBI::ParamInfo(
             argAlloc->indexType,
@@ -292,7 +285,7 @@ WaveOps SubGroupFuncsResolution::GetWaveOp(StringRef funcName)
             return op.second;
         }
     }
-    assert(!"Function name does not contain spir-v operation type");
+    IGC_ASSERT_MESSAGE(0, "Function name does not contain spir-v operation type");
     return WaveOps::UNDEF;
 }
 
@@ -356,7 +349,7 @@ void SubGroupFuncsResolution::simdBlockRead(llvm::CallInst& CI)
     LLVMContext& C = CI.getCalledFunction()->getContext();
     Value* Ptr = CI.getArgOperand(0);
     PointerType* PtrTy = dyn_cast<PointerType>(Ptr->getType());
-    assert(PtrTy && "simdBlockRead has non-pointer type!");
+    IGC_ASSERT_MESSAGE(PtrTy, "simdBlockRead has non-pointer type!");
     SmallVector<Value*, 1> args;
     args.push_back(Ptr);
     SmallVector<Type*, 2>  types;
@@ -385,8 +378,8 @@ void SubGroupFuncsResolution::simdBlockRead(llvm::CallInst& CI)
         types[1] = (Type::getInt64PtrTy(C, AS));
         break;
     default:
-        assert("unrecognized bit width!");
-        //assert but continue code failsafe using default 32
+        IGC_ASSERT_MESSAGE(0, "unrecognized bit width!");
+        // assertion failed but continue code failsafe using default 32
     case 32:
         types[1] = (Type::getInt32PtrTy(C, AS));
         break;
@@ -432,7 +425,7 @@ void SubGroupFuncsResolution::simdBlockWrite(llvm::CallInst& CI)
     LLVMContext& C = CI.getCalledFunction()->getContext();
     Value* Ptr = CI.getArgOperand(0);
     PointerType* PtrTy = dyn_cast<PointerType>(Ptr->getType());
-    assert(PtrTy && "simdBlockWrite has non-pointer type!");
+    IGC_ASSERT_MESSAGE(PtrTy, "simdBlockWrite has non-pointer type!");
     ADDRESS_SPACE AS = (ADDRESS_SPACE)PtrTy->getAddressSpace();
     bool supportLocal = false;
     supportLocal = m_pCtx->platform.supportSLMBlockMessage();
@@ -461,8 +454,8 @@ void SubGroupFuncsResolution::simdBlockWrite(llvm::CallInst& CI)
         types.push_back(Type::getInt64PtrTy(C, AS));
         break;
     default:
-        assert("unrecognized bit width!");
-        //assert but continue code failsafe using default 32
+        IGC_ASSERT_MESSAGE(0, "unrecognized bit width!");
+        // assertion failed but continue code failsafe using default 32
     case 32:
         types.push_back(Type::getInt32PtrTy(C, AS));
         break;
@@ -508,46 +501,53 @@ void SubGroupFuncsResolution::pushMediaBlockArgs(llvm::SmallVector<llvm::Value*,
     args.push_back(isImageTypeUAV);
 }
 
-void SubGroupFuncsResolution::subGroupReduce(WaveOps op, CallInst& CI)
+void SubGroupFuncsResolution::subGroupArithmetic(CallInst& CI, WaveOps op, GroupOpType groupType)
 {
     IRBuilder<> IRB(&CI);
     Value* arg = CI.getArgOperand(0);
+    // GenISA_Wave* instrinsics do not support i1 type. Handle this with i8 version of instrinsic.
+    bool isBoolean = (arg->getType() == IRB.getInt1Ty());
+    if (isBoolean)
+    {
+        arg = IRB.CreateZExt(arg, IRB.getInt8Ty());
+    }
     Value* opVal = IRB.getInt8((uint8_t)op);
-    Value* args[2] = { arg, opVal };
-    Function* waveAll = GenISAIntrinsic::getDeclaration(CI.getCalledFunction()->getParent(),
-        GenISAIntrinsic::GenISA_WaveAll,
-        arg->getType());
-    Instruction* waveAllCall = IRB.CreateCall(waveAll, args);
-    CI.replaceAllUsesWith(waveAllCall);
-    CI.eraseFromParent();
-}
+    Value* waveCall = nullptr;
+    if (groupType == GroupOperationReduce)
+    {
+        Value* args[2] = { arg, opVal };
+        Function* waveAll = GenISAIntrinsic::getDeclaration(CI.getCalledFunction()->getParent(),
+            GenISAIntrinsic::GenISA_WaveAll,
+            arg->getType());
+        waveCall = IRB.CreateCall(waveAll, args);
+    }
+    else if (groupType == GroupOperationScan)
+    {
+        Value* args[4] = { arg, opVal, IRB.getInt1(false), IRB.getInt1(true) };
+        Function* waveScan = GenISAIntrinsic::getDeclaration(CI.getCalledFunction()->getParent(),
+            GenISAIntrinsic::GenISA_WavePrefix,
+            arg->getType());
+        waveCall = IRB.CreateCall(waveScan, args);
+    }
+    else if (groupType == GroupOperationClusteredReduce)
+    {
+        Value* clusterSize = CI.getOperand(1);
+        Value* args[3] = { arg, opVal, clusterSize };
+        Function* waveClustered = GenISAIntrinsic::getDeclaration(CI.getCalledFunction()->getParent(),
+            GenISAIntrinsic::GenISA_WaveClustered,
+            arg->getType());
+        waveCall = IRB.CreateCall(waveClustered, args);
+    }
+    else
+    {
+        IGC_ASSERT_MESSAGE(0, "Unsupported group operation type!");
+    }
 
-void SubGroupFuncsResolution::subGroupClusteredReduce(WaveOps op, CallInst& CI)
-{
-    IRBuilder<> IRB(&CI);
-    Value* arg = CI.getArgOperand(0);
-    Value* opVal = IRB.getInt8((uint8_t)op);
-    Value* clusterSize = CI.getOperand(1);
-    Value* args[3] = { arg, opVal, clusterSize };
-    Function* waveClustered = GenISAIntrinsic::getDeclaration(CI.getCalledFunction()->getParent(),
-        GenISAIntrinsic::GenISA_WaveClustered,
-        arg->getType());
-    Instruction* waveClusteredCall = IRB.CreateCall(waveClustered, args);
-    CI.replaceAllUsesWith(waveClusteredCall);
-    CI.eraseFromParent();
-}
-
-void SubGroupFuncsResolution::subGroupScan(WaveOps op, CallInst& CI)
-{
-    IRBuilder<> IRB(&CI);
-    Value* arg = CI.getArgOperand(0);
-    Value* opVal = IRB.getInt8((uint8_t)op);
-    Value* args[] = { arg, opVal, IRB.getInt1(false), IRB.getInt1(true) };
-    Function* waveScan = GenISAIntrinsic::getDeclaration(CI.getCalledFunction()->getParent(),
-        GenISAIntrinsic::GenISA_WavePrefix,
-        arg->getType());
-    Instruction* waveScanCall = IRB.CreateCall(waveScan, args);
-    CI.replaceAllUsesWith(waveScanCall);
+    if (isBoolean)
+    {
+        waveCall = IRB.CreateTrunc(waveCall, IRB.getInt1Ty());
+    }
+    CI.replaceAllUsesWith(waveCall);
     CI.eraseFromParent();
 }
 
@@ -881,15 +881,15 @@ void SubGroupFuncsResolution::visitCallInst(CallInst& CI)
     }
     else if (funcName.startswith(SubGroupFuncsResolution::SUB_GROUP_REDUCE))
     {
-        return subGroupReduce(GetWaveOp(funcName), CI);
+        return subGroupArithmetic( CI, GetWaveOp(funcName), GroupOperationReduce);
     }
     else if (funcName.startswith(SubGroupFuncsResolution::SUB_GROUP_SCAN))
     {
-        return subGroupScan(GetWaveOp(funcName), CI);
+        return subGroupArithmetic( CI, GetWaveOp(funcName), GroupOperationScan);
     }
     else if (funcName.startswith(SubGroupFuncsResolution::SUB_GROUP_CLUSTERED_REDUCE))
     {
-        return subGroupClusteredReduce(GetWaveOp(funcName), CI);
+        return subGroupArithmetic( CI, GetWaveOp(funcName), GroupOperationClusteredReduce);
     }
     else if (funcName.startswith(SubGroupFuncsResolution::SUB_GROUP_BARRIER))
     {
@@ -926,7 +926,7 @@ void SubGroupFuncsResolution::CheckMediaBlockInstError(llvm::GenIntrinsicInst* i
     const llvm::DataLayout* DL = &F->getParent()->getDataLayout();
 
     uint typeSize = isa<VectorType>(pDataType) ?
-        (uint)DL->getTypeSizeInBits(cast<VectorType>(pDataType)->getVectorElementType()) / 8 :
+        (uint)DL->getTypeSizeInBits(cast<VectorType>(pDataType)->getElementType()) / 8 :
         (uint)DL->getTypeSizeInBits(pDataType) / 8;
 
     uint widthInBytes = blockWidth * typeSize;
