@@ -23,6 +23,7 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 
 ======================= end_copyright_notice ==================================*/
+
 #include "RectListOptimizationPass.hpp"
 #include "IGCPassSupport.h"
 #include "GenISAIntrinsics/GenIntrinsicInst.h"
@@ -30,7 +31,6 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "Compiler/MetaDataUtilsWrapper.h"
 #include "Compiler/CISACodeGen/CollectGeometryShaderProperties.hpp"
 #include "ShaderTypesEnum.h"
-
 #include "common/LLVMWarningsPush.hpp"
 #include "llvm/IR/Function.h"
 #include <llvm/IR/InstVisitor.h>
@@ -40,10 +40,10 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "common/LLVMWarningsPop.hpp"
 #include "common/IGCIRBuilder.h"
 #include "common/igc_regkeys.hpp"
-
 #include <vector>
 #include <unordered_map>
 #include <set>
+#include "Probe/Assertion.h"
 
 using namespace llvm;
 using namespace IGC;
@@ -133,7 +133,7 @@ bool RectListOptimizationPass::isGSTriStripPrimTopology(Function& F)
 bool RectListOptimizationPass::isGSOutputVertValid(Function& F)
 {
     auto pMaxOutputVertices = F.getParent()->getGlobalVariable("GsMaxOutputVertices");
-    assert(pMaxOutputVertices != nullptr && "GsMaxOutputVertices must be defined");
+    IGC_ASSERT_MESSAGE(pMaxOutputVertices != nullptr, "GsMaxOutputVertices must be defined");
     const unsigned int maxOutputVertices = static_cast<unsigned int>(
         (llvm::cast<llvm::ConstantInt>(pMaxOutputVertices->getInitializer())->getZExtValue()));
     return (maxOutputVertices == m_NUM_OUTPUT_VERT_FOR_RECTLIST);
@@ -252,7 +252,7 @@ bool RectListOptimizationPass::isRectCoords(
                 Value* V = *it;
                 coordToVertMap::iterator coordToVertIt = rectCoordToVertM.find(V);
                 if (coordToVertIt == rectCoordToVertM.end()) {
-                    assert("Not able to find one of the XChannel Values");
+                    IGC_ASSERT_MESSAGE(0, "Not able to find one of the XChannel Values");
                     isRectCoords = false;
                     break;
                 }
@@ -286,7 +286,7 @@ bool RectListOptimizationPass::isConstantCoords(
         Value* uniqueValue = *ChannelValues.begin();
         coordToVertMap::iterator it = rectCoordToVertM.find(uniqueValue);
         if (it == rectCoordToVertM.end()) {
-            assert("Not able to find the value in the unique value in Vale to vertex List map");
+            IGC_ASSERT_MESSAGE(0, "Not able to find the value in the unique value in Vale to vertex List map");
             break;
         }
         if (it->second.size() != m_NUM_OUTPUT_VERT_FOR_RECTLIST) {
@@ -316,7 +316,7 @@ void RectListOptimizationPass::analyzeForRectList(Function& F, GeometryShaderCon
                 m_rectListPerAttrib.insert(std::make_pair(it->first, rectList));
                 arit = m_rectListPerAttrib.find(it->first);
             }
-            for (unsigned int ch_id = 0; ch_id < USC::NUM_SHADER_CHANNELS; ch_id++) {
+            for (unsigned int ch_id = 0; ch_id < IGC::NUM_SHADER_CHANNELS; ch_id++) {
                 Value* channelV = inst->getOperand(ch_id);
                 channelToCoordMap::iterator chtocoordit = arit->second.channelToCoordM.find(ch_id);
                 if (chtocoordit != arit->second.channelToCoordM.end()) {
@@ -346,13 +346,13 @@ void RectListOptimizationPass::analyzeForRectList(Function& F, GeometryShaderCon
     //   And ZY channel must be constant
     bool bPositionFound = false;
     bool isRectList = false;
-    assert(m_rectListPerAttrib.size() != 0 && "Empty attribute list, should not happen\n");
+    IGC_ASSERT_MESSAGE(m_rectListPerAttrib.size() != 0, "Empty attribute list, should not happen\n");
     for (atrribRectListMap::iterator it = m_rectListPerAttrib.begin(); it != m_rectListPerAttrib.end(); it++) {
         ATTRIB_SOURCELIST_ST* pAttrSrcList = &it->second;
-        channelToCoordMap::iterator ctCitXit = pAttrSrcList->channelToCoordM.find(USC::SHADER_CHANNEL_X);
-        channelToCoordMap::iterator ctCitYit = pAttrSrcList->channelToCoordM.find(USC::SHADER_CHANNEL_Y);
-        channelToCoordMap::iterator ctCitZit = pAttrSrcList->channelToCoordM.find(USC::SHADER_CHANNEL_Z);
-        channelToCoordMap::iterator ctCitWit = pAttrSrcList->channelToCoordM.find(USC::SHADER_CHANNEL_W);
+        channelToCoordMap::iterator ctCitXit = pAttrSrcList->channelToCoordM.find(IGC::SHADER_CHANNEL_X);
+        channelToCoordMap::iterator ctCitYit = pAttrSrcList->channelToCoordM.find(IGC::SHADER_CHANNEL_Y);
+        channelToCoordMap::iterator ctCitZit = pAttrSrcList->channelToCoordM.find(IGC::SHADER_CHANNEL_Z);
+        channelToCoordMap::iterator ctCitWit = pAttrSrcList->channelToCoordM.find(IGC::SHADER_CHANNEL_W);
         //reset the rectList to false for each iteration
         isRectList = false;
 
@@ -361,7 +361,7 @@ void RectListOptimizationPass::analyzeForRectList(Function& F, GeometryShaderCon
             ctCitZit == pAttrSrcList->channelToCoordM.end() ||
             ctCitWit == pAttrSrcList->channelToCoordM.end())
         {
-            assert("Not able to find one of the Channel X/Y/Z/W!!!");
+            IGC_ASSERT_MESSAGE(0, "Not able to find one of the Channel X/Y/Z/W!!!");
             break;
         }
 
@@ -422,17 +422,17 @@ bool RectListOptimizationPass::runOnFunction(Function& F)
         IGCMD::MetaDataUtils* pMdUtils = getAnalysis<MetaDataUtilsWrapper>().getMetaDataUtils();
         if (pMdUtils->findFunctionsInfoItem(&F) == pMdUtils->end_FunctionsInfo())
         {
-            assert("failed to find the function in metadata\n");
+            IGC_ASSERT_MESSAGE(0, "failed to find the function in metadata\n");
             break;
         }
         IGC::CodeGenContext* ctx = getAnalysis<IGC::CodeGenContextWrapper>().getCodeGenContext();
         if (!ctx || ctx->type != ShaderType::GEOMETRY_SHADER) {
-            assert("Failed to get the code gen context or not a Geometry shader\n");
+            IGC_ASSERT_MESSAGE(0, "Failed to get the code gen context or not a Geometry shader\n");
             break;
         }
         IGC::GeometryShaderContext* pgsctx = static_cast <IGC::GeometryShaderContext*>(ctx);
         if (!pgsctx) {
-            assert("Failed to get the GS code gen context \n");
+            IGC_ASSERT_MESSAGE(0, "Failed to get the GS code gen context \n");
             break;
         }
 
