@@ -32,7 +32,8 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "common/LLVMWarningsPush.hpp"
 #include "llvm/ADT/StringSwitch.h"
 #include "common/LLVMWarningsPop.hpp"
-
+#include "llvm/IR/InstIterator.h"
+#include <set>
 
 using namespace llvm;
 using namespace IGC;
@@ -116,6 +117,44 @@ void SPIRMetaDataTranslation::WarpFunctionMetadata(Module& M)
     }
 }
 
+bool SPIRMetaDataTranslation::isDoubleMathFunctionUsed(Module& M)
+{
+    StringRef mathFunctionNamePrefix = "__builtin_spirv_OpenCL_";
+    std::set<StringRef> mathFunctionNames = {
+        "acos", "acosh", "acospi", "asin", "asinh", "asinpi", "atan", "atan2",
+        "atanh", "atanpi", "cbrt", "cos", "cosh", "cospi", "erf", "erfc",
+        "exp", "exp10", "exp2", "expm1", "ln", "log10", "log1p", "log2",
+        "pow", "pown", "powr", "rootn", "sin", "sincos", "sinh", "sinpi",
+        "tan", "tanh", "tanpi"
+    };
+
+    for (Function& F : M)
+    {
+        for (inst_iterator i = inst_begin(&F), e = inst_end(&F); i != e; ++i)
+        {
+            if (auto CI = dyn_cast<CallInst>(&*i)) {
+
+                if (CI->getType()->isDoubleTy() ||
+                    (CI->getType()->isVectorTy() && CI->getType()->getContainedType(0)->isDoubleTy()))
+                {
+                    StringRef functionName = CI->getCalledFunction()->getName();
+
+                    if (functionName.startswith(mathFunctionNamePrefix))
+                    {
+                        functionName = functionName.ltrim(mathFunctionNamePrefix);
+                        functionName = functionName.take_front(functionName.find("_", 0));
+
+                        if (mathFunctionNames.find(functionName) != mathFunctionNames.end()) {
+                            return true;
+                        }
+                    }
+                }
+            }
+        }
+    }
+    return false;
+}
+
 bool SPIRMetaDataTranslation::runOnModule(Module& M)
 {
     WarpFunctionMetadata(M);
@@ -304,8 +343,9 @@ bool SPIRMetaDataTranslation::runOnModule(Module& M)
         for (; coi != coe; ++coi)
         {
             std::string co = *coi;
-            // Compiler options that originate from OpenCL are represented by the same name, without the "-cl" prefix.
-            if (co.find("-cl") == 0)
+            // Compiler options that originate from OpenCL/L0 are represented by the same name, without the "-cl"/"-ze" prefixes.
+            // L0 supports only "-opt-disable" from the options below
+            if (co.find("-cl") == 0 || co.find("-ze") == 0)
             {
                 co.erase(0, 3);
             }
@@ -322,6 +362,7 @@ bool SPIRMetaDataTranslation::runOnModule(Module& M)
                 FAST_RELAXED_MATH,
                 DASH_G,
                 RELAXED_BUILTINS,
+                MATCH_SINCOSPI,
                 NONE,
             };
             int igc_compiler_option = llvm::StringSwitch<OCL_OPTIONS>(co)
@@ -335,6 +376,7 @@ bool SPIRMetaDataTranslation::runOnModule(Module& M)
                 .Case("-fast-relaxed-math", FAST_RELAXED_MATH)
                 .Case("-g", DASH_G)
                 .Case("-relaxed-builtins", RELAXED_BUILTINS)
+                .Case("-match-sincospi", MATCH_SINCOSPI)
                 .Default(NONE);
 
 
@@ -356,11 +398,20 @@ bool SPIRMetaDataTranslation::runOnModule(Module& M)
                 break;
             case FAST_RELAXED_MATH:
             case RELAXED_BUILTINS:
-                modMD->compOpt.FastRelaxedMath = true;
-                modMD->compOpt.RelaxedBuiltins = true;
+                // Our implementations of double math built-in functions are precise only
+                // if we don't make any fast relaxed math optimizations.
+                // If there is any double-type math function call present in a module, we disable fast relaxed math,
+                // even if "-cl-fast-relaxed-math" build option is specified by the user.
+                if (!isDoubleMathFunctionUsed(M))
+                {
+                    modMD->compOpt.FastRelaxedMath = true;
+                    modMD->compOpt.RelaxedBuiltins = true;
+                }
                 break;
             case DASH_G: modMD->compOpt.DashGSpecified = true;
                 break;
+            case MATCH_SINCOSPI: modMD->compOpt.MatchSinCosPi = true;
+              break;
             default:
                 break;
             }
