@@ -24,21 +24,17 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 ======================= end_copyright_notice ==================================*/
 
-///=======================================================================================
-/// This file contains the implementation of the IGC IR Verification Pass.
-
 #include "Compiler/VerificationPass.hpp"
 #include "Compiler/IGCPassSupport.h"
 #include "Compiler/CodeGenPublic.h"
 #include "common/igc_regkeys.hpp"
-
 #include "common/LLVMWarningsPush.hpp"
 #include <llvm/IR/Module.h>
 #include <llvm/IR/InstIterator.h>
-
 #include <llvmWrapper/IR/Intrinsics.h>
-
+#include <llvmWrapper/IR/DerivedTypes.h>
 #include "common/LLVMWarningsPop.hpp"
+#include "Probe/Assertion.h"
 
 using namespace llvm;
 using namespace IGC;
@@ -127,11 +123,11 @@ bool VerificationPass::runOnModule(Module& M)
             }
         }
 
-        if (!success)
+        if (false == success)
         {
+            IGC_ASSERT_MESSAGE(0, "IGC IR Verification failed");
             printf("\nIGC IR verification failed:\n\n");
             errs() << m_messagesToDump.str();
-            assert(false && "IGC IR Verification failed");
         }
     }
 
@@ -223,7 +219,7 @@ bool VerificationPass::verifyInstCommon(Instruction& inst)
 bool VerificationPass::verifyInstCall(Instruction& inst)
 {
     CallInst* instCall = dyn_cast<CallInst>(&inst);
-    assert(instCall && "Unexpected instruction");
+    IGC_ASSERT_MESSAGE(instCall, "Unexpected instruction");
 
     if (GenIntrinsicInst::classof(instCall))
     {
@@ -257,10 +253,10 @@ bool VerificationPass::verifyVectorInst(llvm::Instruction& inst)
     {
         Value* val = inst.getOperand(i);
         Type* T = val->getType();
-        if (T->isVectorTy())
+        if (auto VT = dyn_cast<VectorType>(T))
         {
             // Insert and extract element support relaxed vector type
-            T = T->getVectorElementType();
+            T = VT->getElementType();
         }
         if (!verifyType(T, val))
         {
@@ -306,16 +302,17 @@ bool VerificationPass::verifyType(Type* type, Value* val)
         // All integer types are valide
         break;
 
-    case Type::VectorTyID:
+    case IGCLLVM::VectorTyID:
     {
-        unsigned int typeSize = type->getVectorNumElements();
+        auto VType = cast<VectorType>(type);
+        unsigned typeSize = (unsigned)VType->getNumElements();
         if (!m_IGC_IR_spec.vectorTypeSizes.count(typeSize))
         {
             m_messagesToDump << "Unexpected vector size found in value:\n";
             printValue(val);
             success = false;
         }
-        success &= verifyType(type->getVectorElementType(), val);
+        success &= verifyType(VType->getElementType(), val);
         break;
     }
 
