@@ -28,19 +28,17 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "GenISAIntrinsics/GenIntrinsicInst.h"
 #include "GenISAIntrinsics/GenIntrinsics.h"
 #include "Compiler/IGCPassSupport.h"
-
 #include "common/LLVMWarningsPush.hpp"
+#include "llvmWrapper/IR/DerivedTypes.h"
 #include <llvm/Support/CommandLine.h>
 #include <llvm/IR/Function.h>
 #include <llvm/ADT/SmallVector.h>
-
 #include <llvmWrapper/IR/Intrinsics.h>
-
 #include "common/LLVMWarningsPop.hpp"
-
 #include "Compiler/CISACodeGen/helper.h"
 #include "Compiler/CodeGenPublicEnums.h"
 #include "Compiler/CodeGenPublic.h"
+#include "Probe/Assertion.h"
 
 using namespace llvm;
 using namespace IGC;
@@ -236,6 +234,30 @@ void WorkaroundAnalysis::visitCallInst(llvm::CallInst& I)
 {
     CodeGenContext* pCodeGenCtx = m_pCtxWrapper->getCodeGenContext();
 
+    if (pCodeGenCtx->getModuleMetaData()->enableRangeReduce || IGC_IS_FLAG_ENABLED(EnableTrigFuncRangeReduction))
+    {
+        if (IntrinsicInst* intrinsicInst = dyn_cast<IntrinsicInst>(&I))
+        {
+            if (intrinsicInst->getIntrinsicID() == Intrinsic::sin || intrinsicInst->getIntrinsicID() == Intrinsic::cos)
+            {
+                m_builder->SetInsertPoint(intrinsicInst);
+                Value* angle = intrinsicInst->getOperand(0);
+                Value* int1_res_s1 = m_builder->CreateFCmpOGE(angle, llvm::ConstantFP::get(m_builder->getFloatTy(), 0.0));
+                Value* float_selRes_s = m_builder->CreateSelect(int1_res_s1, llvm::ConstantFP::get(m_builder->getFloatTy(), 6.283185f), llvm::ConstantFP::get(m_builder->getFloatTy(), -6.283185f));
+                Value* float_selRes_s4 = m_builder->CreateSelect(int1_res_s1, llvm::ConstantFP::get(m_builder->getFloatTy(), 0.159155f), llvm::ConstantFP::get(m_builder->getFloatTy(), -0.159155f));
+                Value* float_res_s5 = m_builder->CreateFMul(float_selRes_s4, angle);
+                Value* funcFloor = m_builder->CreateFPToSI(float_res_s5, m_builder->getInt32Ty());
+                Value* funcfloorfloat = m_builder->CreateSIToFP(funcFloor, m_builder->getFloatTy());
+                Value* float_res_s_i_s = m_builder->CreateFSub(float_res_s5, funcfloorfloat);
+                Value* float_res_s6 = m_builder->CreateFMul(float_res_s_i_s, float_selRes_s);
+                intrinsicInst->setOperand(0, float_res_s6);
+
+            }
+        }
+    }
+
+
+
     // TODO: Fix this for all Shaders once and for all
     if (pCodeGenCtx->type == ShaderType::VERTEX_SHADER && pCodeGenCtx->isPOSH())
     {
@@ -287,7 +309,7 @@ void WorkaroundAnalysis::visitCallInst(llvm::CallInst& I)
             uint bufferIndex = GetSampleCResourceIdx(I);
             if (bufferIndex == -1) break;
 
-            assert(bufferIndex < 256);
+            IGC_ASSERT(bufferIndex < 256);
 
             if (pCodeGenCtx->type == ShaderType::PIXEL_SHADER)
             {
@@ -354,7 +376,7 @@ void WorkaroundAnalysis::visitCallInst(llvm::CallInst& I)
 
                 Value* selW = m_builder->CreateSelect(cmpInst, valueW, one);
 
-                llvm::Value* newValue = llvm::UndefValue::get(llvm::VectorType::get(m_builder->getFloatTy(), 4));
+                llvm::Value* newValue = llvm::UndefValue::get(IGCLLVM::FixedVectorType::get(m_builder->getFloatTy(), 4));
                 newValue = m_builder->CreateInsertElement(newValue, valueX, m_builder->getInt32(0));
                 newValue = m_builder->CreateInsertElement(newValue, valueY, m_builder->getInt32(1));
                 newValue = m_builder->CreateInsertElement(newValue, valueZ, m_builder->getInt32(2));
