@@ -23,10 +23,11 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 
 ======================= end_copyright_notice ==================================*/
+
 #include "igc_workaround.h"
-#include "assert.h"
 #include <string.h>
 #include <stdlib.h>
+#include "Probe/Assertion.h"
 
 namespace IGC
 {
@@ -37,10 +38,14 @@ namespace IGC
         memset(&waTable, 0, sizeof(WA_TABLE));
         WA_INIT_PARAM      stWaInitParam = {};
         stWaInitParam.ePlatformType = platform->getPlatformInfo().ePlatformType;
+        if (IGC_GET_FLAG_VALUE(OverrideRevIdForWA))
+            platform->OverrideRevId(IGC_GET_FLAG_VALUE(OverrideRevIdForWA));
         stWaInitParam.usRevId = platform->getPlatformInfo().usRevId;
         stWaInitParam.usRevId_PCH = platform->getPlatformInfo().usRevId_PCH;
         GT_SYSTEM_INFO sysInfo = platform->GetGTSystemInfo();
         stWaInitParam.pGtSysInfo = &sysInfo;
+        if (IGC_GET_FLAG_VALUE(OverrideProductFamilyForWA))
+            platform->OverrideProductFamily(IGC_GET_FLAG_VALUE(OverrideProductFamilyForWA));
 
         switch (platform->getPlatformInfo().eProductFamily)
         {
@@ -73,23 +78,32 @@ namespace IGC
             break;
         case IGFX_ICELAKE:
             InitIclHwWaTable(&waTable, pSkuFeatureTable, &stWaInitParam);
+            InitIclSwWaTable(&waTable, pSkuFeatureTable, &stWaInitParam);
             break;
         case IGFX_ICELAKE_LP:
             InitIclLpHwWaTable(&waTable, pSkuFeatureTable, &stWaInitParam);
+            InitIclLpSwWaTable(&waTable, pSkuFeatureTable, &stWaInitParam);
             break;
         case IGFX_LAKEFIELD:
             InitLkfHwWaTable(&waTable, pSkuFeatureTable, &stWaInitParam);
+            InitLkfSwWaTable(&waTable, pSkuFeatureTable, &stWaInitParam);
             break;
         case IGFX_TIGERLAKE_LP:
             {
                 InitTglLpHwWaTable(&waTable, pSkuFeatureTable, &stWaInitParam);
+                InitTglLpSwWaTable(&waTable, pSkuFeatureTable, &stWaInitParam);
             }
             break;
         case IGFX_JASPERLAKE:
             InitJslHwWaTable(&waTable, pSkuFeatureTable, &stWaInitParam);
+            InitJslSwWaTable(&waTable, pSkuFeatureTable, &stWaInitParam);
+            break;
+        case IGFX_DG1:
+            InitDg1HwWaTable(&waTable, pSkuFeatureTable, &stWaInitParam);
+            InitDg1SwWaTable(&waTable, pSkuFeatureTable, &stWaInitParam);
             break;
         default:
-            assert(false);
+            IGC_ASSERT(0);
             break;
         }
 
@@ -121,6 +135,7 @@ namespace IGC
         SkuFeatureTable.FtrIoMmuPageFaulting = pUSCSkuFeatureTable->FtrIoMmuPageFaulting;
         SkuFeatureTable.FtrWddm2Svm = pUSCSkuFeatureTable->FtrWddm2Svm;
         SkuFeatureTable.FtrPooledEuEnabled = pUSCSkuFeatureTable->FtrPooledEuEnabled;
+        SkuFeatureTable.FtrLocalMemory = pUSCSkuFeatureTable->FtrLocalMemory;
     }
 
     void SetWorkaroundTable(const SUscSkuFeatureTable* pSkuFeatureTable, CPlatform* platform)
