@@ -57,8 +57,10 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "llvm/IR/Module.h"
 #include "common/LLVMWarningsPop.hpp"
 #include "Compiler/CodeGenPublicEnums.h"
+#include "common/EmUtils.h"
 #include "common/Types.hpp"
 #include "GenIntrinsics.h"
+#include "Probe/Assertion.h"
 
 namespace llvm {
 /// IntrinsicInst - A useful wrapper class for inspecting calls to intrinsic
@@ -98,7 +100,7 @@ public:
     }
 
     uint64_t getImm64Operand(unsigned idx) const {
-        assert(isa<ConstantInt>(getOperand(idx)));
+        IGC_ASSERT(isa<ConstantInt>(getOperand(idx)));
         return valueToImm64(getOperand(idx));
     }
 };
@@ -173,6 +175,7 @@ protected:
     void setOperand(unsigned i, Value* v) { return GenIntrinsicInst::setOperand(i, v); }
 
 public:
+    inline Value* getSource0Alpha()     const { return nullptr; }
     inline Value* getOMask()            const { return getOperand(0); }
     inline Value* getPMask()            const { return getOperand(1); }
     inline void setPMask(Value* mask)         { setOperand(1, mask);  }
@@ -487,7 +490,7 @@ public:
         return static_cast<unsigned int>(cast<ConstantInt>(getAlignmentValue())->getZExtValue());
     }
     inline bool isVolatile() const {
-        assert(isa<ConstantInt>(getOperand(3)));
+        IGC_ASSERT(isa<ConstantInt>(getOperand(3)));
         ConstantInt* val = dyn_cast<ConstantInt>(getOperand(3));
         const bool isVolatile = val ? val->getZExtValue() : false;
         return isVolatile;
@@ -532,13 +535,13 @@ public:
         return getOperand(0);
     }
     inline unsigned int getAlignment() const {
-        assert(isa<ConstantInt>(getAlignmentValue()));
+        IGC_ASSERT(isa<ConstantInt>(getAlignmentValue()));
         ConstantInt* val = dyn_cast<ConstantInt>(getAlignmentValue());
         const unsigned int alignment = val ? int_cast<unsigned int>(val->getZExtValue()) : 1;
         return alignment;
     }
     inline bool isVolatile() const {
-        assert(isa<ConstantInt>(getOperand(4)));
+        IGC_ASSERT(isa<ConstantInt>(getOperand(4)));
         ConstantInt* val = dyn_cast<ConstantInt>(getOperand(4));
         const bool isVolatile = val ? val->getZExtValue() : false;
         return isVolatile;
@@ -580,6 +583,28 @@ public:
     inline Value* getResourceValue() const {
         return getOperand(0);
     }
+
+    inline IGC::AtomicOp getAtomicOp() const {
+        GenISAIntrinsic::ID ID = getIntrinsicID();
+        switch (ID)
+        {
+        case GenISAIntrinsic::GenISA_intatomicraw:
+        case GenISAIntrinsic::GenISA_intatomicrawA64:
+        case GenISAIntrinsic::GenISA_floatatomicraw:
+        case GenISAIntrinsic::GenISA_floatatomicrawA64:
+            return static_cast<IGC::AtomicOp>(getImm64Operand(3));
+        case GenISAIntrinsic::GenISA_icmpxchgatomicraw:
+            return IGC::EATOMIC_CMPXCHG;
+        case GenISAIntrinsic::GenISA_icmpxchgatomicrawA64:
+            return IGC::EATOMIC_CMPXCHG64;
+        case GenISAIntrinsic::GenISA_fcmpxchgatomicraw:
+        case GenISAIntrinsic::GenISA_fcmpxchgatomicrawA64:
+            return IGC::EATOMIC_FCMPWR;
+        default:
+            IGC_ASSERT_MESSAGE(false, "Unexpected atomic raw intrinisc!");
+        }
+        return IGC::EATOMIC_UNDEF;
+    }
 };
 
 class AtomicStructuredIntrinsic : public GenIntrinsicInst {
