@@ -27,9 +27,8 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 #include <Compiler/CodeGenPublic.h>
 #include "GenIntrinsics.h"
-
 #include "common/LLVMWarningsPush.hpp"
-#include "llvm/IR/DerivedTypes.h"
+#include "llvmWrapper/IR/DerivedTypes.h"
 #include "llvm/IR/Function.h"
 #include "llvm/IR/Intrinsics.h"
 #include "llvm/IR/Module.h"
@@ -38,10 +37,9 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <llvm/ADT/StringMap.h>
 #include <llvm/CodeGen/ValueTypes.h>
 #include "common/LLVMWarningsPop.hpp"
-
 #include "../../inc/common/UFO/portable_compiler.h"
-
 #include <cstring>
+#include "Probe/Assertion.h"
 
 using namespace llvm;
 
@@ -89,17 +87,11 @@ struct IITDescriptor {
         AK_AnyPointer
     };
     unsigned getArgumentNumber() const {
-        assert(Kind == Argument || Kind == ExtendArgument ||
-            Kind == TruncArgument || Kind == HalfVecArgument ||
-            Kind == SameVecWidthArgument || Kind == PtrToArgument ||
-            Kind == VecOfPtrsToElt);
+        IGC_ASSERT(Kind == Argument || Kind == ExtendArgument || Kind == TruncArgument || Kind == HalfVecArgument || Kind == SameVecWidthArgument || Kind == PtrToArgument || Kind == VecOfPtrsToElt);
         return Argument_Info >> 3;
     }
     ArgKind getArgumentKind() const {
-        assert(Kind == Argument || Kind == ExtendArgument ||
-            Kind == TruncArgument || Kind == HalfVecArgument ||
-            Kind == SameVecWidthArgument || Kind == PtrToArgument ||
-            Kind == VecOfPtrsToElt);
+        IGC_ASSERT(Kind == Argument || Kind == ExtendArgument || Kind == TruncArgument || Kind == HalfVecArgument || Kind == SameVecWidthArgument || Kind == PtrToArgument || Kind == VecOfPtrsToElt);
         return (ArgKind)(Argument_Info & 7);
     }
 
@@ -178,13 +170,13 @@ static Type *DecodeFixedType(ArrayRef<GenISAIntrinsic::IITDescriptor> &Infos,
   case IITDescriptor::Integer:
     return IntegerType::get(Context, D.Integer_Width);
   case IITDescriptor::Vector:
-    return VectorType::get(DecodeFixedType(Infos, Tys, Context),D.Vector_Width);
+    return IGCLLVM::FixedVectorType::get(DecodeFixedType(Infos, Tys, Context),D.Vector_Width);
   case IITDescriptor::Pointer:
     return PointerType::get(DecodeFixedType(Infos, Tys, Context),
                             D.Pointer_AddressSpace);
   case IITDescriptor::Struct: {
     Type *Elts[5];
-    assert(D.Struct_NumElements <= 5 && "Can't handle this yet");
+    IGC_ASSERT_MESSAGE(D.Struct_NumElements <= 5, "Can't handle this yet");
     for (unsigned i = 0, e = D.Struct_NumElements; i != e; ++i)
       Elts[i] = DecodeFixedType(Infos, Tys, Context);
     return StructType::get(Context, makeArrayRef(Elts,D.Struct_NumElements));
@@ -205,7 +197,8 @@ static Type *DecodeFixedType(ArrayRef<GenISAIntrinsic::IITDescriptor> &Infos,
       return VectorType::getTruncatedElementVectorType(VTy);
 
     IntegerType *ITy = cast<IntegerType>(Ty);
-    assert(ITy->getBitWidth() % 2 == 0);
+    IGC_ASSERT(nullptr != ITy);
+    IGC_ASSERT(ITy->getBitWidth() % 2 == 0);
     return IntegerType::get(Context, ITy->getBitWidth() / 2);
   }
   case IITDescriptor::HalfVecArgument:
@@ -215,9 +208,9 @@ static Type *DecodeFixedType(ArrayRef<GenISAIntrinsic::IITDescriptor> &Infos,
     Type *EltTy = DecodeFixedType(Infos, Tys, Context);
     Type *Ty = Tys[D.getArgumentNumber()];
     if (VectorType *VTy = dyn_cast<VectorType>(Ty)) {
-      return VectorType::get(EltTy, int_cast<unsigned int>(VTy->getNumElements()));
+      return IGCLLVM::FixedVectorType::get(EltTy, int_cast<unsigned int>(VTy->getNumElements()));
     }
-    llvm_unreachable("unhandled");
+    IGC_ASSERT_EXIT_MESSAGE(0, "unhandled");
   }
   case IITDescriptor::PtrToArgument: {
     Type *Ty = Tys[D.getArgumentNumber()];
@@ -227,13 +220,14 @@ static Type *DecodeFixedType(ArrayRef<GenISAIntrinsic::IITDescriptor> &Infos,
       Type *Ty = Tys[D.getArgumentNumber()];
       VectorType *VTy = dyn_cast<VectorType>(Ty);
       if (!VTy)
-          llvm_unreachable("Expected an argument of Vector Type");
-      Type *EltTy = VTy->getVectorElementType();
-      return VectorType::get(PointerType::getUnqual(EltTy),
+          IGC_ASSERT_EXIT_MESSAGE(0, "Expected an argument of Vector Type");
+      Type *EltTy = cast<VectorType>(VTy)->getElementType();
+      return IGCLLVM::FixedVectorType::get(PointerType::getUnqual(EltTy),
           int_cast<unsigned int>(VTy->getNumElements()));
   }
  }
-  llvm_unreachable("unhandled");
+  IGC_ASSERT_EXIT_MESSAGE(0, "unhandled");
+  return nullptr;
 }
 
 
@@ -399,7 +393,7 @@ static void DecodeIITType(unsigned &NextElt, ArrayRef<unsigned char> Infos,
     return;
   }
   }
-  llvm_unreachable("unhandled");
+  IGC_ASSERT_EXIT_MESSAGE(0, "unhandled");
 }
 
 // This defines the "Intrinsic::getIntrinsicForGCCBuiltin()" method.
@@ -443,8 +437,8 @@ static std::string getMangledTypeStr(Type* Ty) {
     Result += "f";
   }
   else if (isa<VectorType>(Ty))
-    Result += "v" + utostr(Ty->getVectorNumElements()) +
-      getMangledTypeStr(Ty->getVectorElementType());
+    Result += "v" + utostr(cast<VectorType>(Ty)->getNumElements()) +
+      getMangledTypeStr(cast<VectorType>(Ty)->getElementType());
   else if (Ty)
     Result += EVT::getEVT(Ty).getEVTString();
   return Result;
@@ -452,7 +446,7 @@ static std::string getMangledTypeStr(Type* Ty) {
 
 
 std::string GenISAIntrinsic::getName(GenISAIntrinsic::ID id, ArrayRef<Type*> Tys) {
-  //assert(id < num_genisa_intrinsics && "Invalid intrinsic ID!");
+  //IGC_ASSERT_MESSAGE(id < num_genisa_intrinsics, "Invalid intrinsic ID!");
   if (id > (GenISAIntrinsic::ID)Intrinsic::num_intrinsics)
     id = (GenISAIntrinsic::ID)(id-Intrinsic::num_intrinsics);
   static const char * const Table[] = {
@@ -470,6 +464,13 @@ std::string GenISAIntrinsic::getName(GenISAIntrinsic::ID id, ArrayRef<Type*> Tys
   return Result;
 }
 
+GenISAIntrinsic::IntrinsicComments GenISAIntrinsic::getIntrinsicComments(
+    GenISAIntrinsic::ID id) {
+#define GET_INTRINSIC_COMMENTS_TABLE
+#include "IntrinsicGenISA.gen"
+#undef GET_INTRINSIC_COMMENTS_TABLE
+}
+
 FunctionType *GenISAIntrinsic::getType(LLVMContext &Context,
                                  ID id, ArrayRef<Type*> Tys) {
   SmallVector<IITDescriptor, 8> Table;
@@ -506,7 +507,7 @@ bool GenISAIntrinsic::isOverloaded(GenISAIntrinsic::ID id) {
 #include "IntrinsicGenISA.gen"
 #undef GET_INTRINSIC_GENERATOR_GLOBAL
 
-IGCLLVM::AttributeSet GenISAIntrinsic::getGenIntrinsicAttributes(
+AttributeList GenISAIntrinsic::getGenIntrinsicAttributes(
     LLVMContext& C, GenISAIntrinsic::ID id)
 {
     return getAttributes(C, (GenISAIntrinsic::ID)(id - 1));
@@ -520,7 +521,7 @@ Function *GenISAIntrinsic::getDeclaration(
     IGCLLVM::Module* M = (IGCLLVM::Module*)Module;
 
     Function *F =
-        cast<Function>(
+        dyn_cast<Function>(
             M->getOrInsertFunction(
                 getName((GenISAIntrinsic::ID)(id-Intrinsic::num_intrinsics), Tys),
                 getType(
@@ -529,6 +530,7 @@ Function *GenISAIntrinsic::getDeclaration(
                     Tys),
                 getAttributes(M->getContext(),(GenISAIntrinsic::ID)(id - 1))));
 
+    IGC_ASSERT_MESSAGE(F, "getOrInsertFunction probably returned constant expression!");
     // Since Function::isIntrinsic() will return true due to llvm.* prefix,
     // Module::getOrInsertFunction fails to add the attributes.
     // explicitly adding the attribute to handle this problem.
@@ -575,17 +577,21 @@ void GenISAIntrinsic::getIntrinsicInfoTableEntries(
 }
 
 GenISAIntrinsic::ID GenISAIntrinsic::getIntrinsicID(const Function *F) {
+    IGC_ASSERT(nullptr != F);
     IGC::LLVMContextWrapper::SafeIntrinsicIDCacheTy* safeIntrinsicIDCache =
         &(static_cast<IGC::LLVMContextWrapper*>(&F->getContext())->m_SafeIntrinsicIDCache);
+    IGC_ASSERT(nullptr != safeIntrinsicIDCache);
 
     //If you do not find the function ptr as key corresponding to the GenISAIntrinsic::ID add the new key
     auto it = (*safeIntrinsicIDCache).find(F);
     if (it == (*safeIntrinsicIDCache).end()) {
-        const ValueName *valueName = F->getValueName();
+        const ValueName* const valueName = F->getValueName();
+        IGC_ASSERT(nullptr != valueName);
         GenISAIntrinsic::ID Id = no_intrinsic;
         llvm::StringRef prefix = getGenIntrinsicPrefix();
         llvm::StringRef Name = valueName->getKey();
-        assert(Name.size()>prefix.size() && Name.startswith(prefix) && "Not a valid gen intrinsic name signature\n");
+        IGC_ASSERT_MESSAGE(Name.size() > prefix.size(), "Not a valid gen intrinsic name signature");
+        IGC_ASSERT_MESSAGE(Name.startswith(prefix), "Not a valid gen intrinsic name signature");
         Id = lookupGenIntrinsicID(Name.data(), Name.size());
         (*safeIntrinsicIDCache)[F] = Id;
         return Id;
