@@ -24,13 +24,16 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 ======================= end_copyright_notice ==================================*/
 
+#ifndef GENINTRINSICS_H
+#define GENINTRINSICS_H
+
 #pragma once
 
 #include "common/LLVMWarningsPush.hpp"
 
 #include "llvmWrapper/IR/Module.h"
 
-#include "llvmWrapper/IR/Attributes.h"
+#include "llvm/IR/Attributes.h"
 #include "llvm/IR/Intrinsics.h"
 #include "llvm/IR/Function.h"
 #include "common/LLVMWarningsPop.hpp"
@@ -52,6 +55,15 @@ namespace GenISAIntrinsic {
   std::string getName(ID id, ArrayRef<Type*> Tys = None);
 
 
+  struct IntrinsicComments
+  {
+      const char* funcDescription;
+      std::vector<const char*> outputs;
+      std::vector<const char*> inputs;
+  };
+
+  IntrinsicComments getIntrinsicComments(ID id);
+
   /// Intrinsic::getDeclaration(M, ID) - Create or insert an LLVM Function
   /// declaration for an intrinsic, and return it.
   ///
@@ -72,7 +84,7 @@ namespace GenISAIntrinsic {
 #else
   Function *getDeclaration(Module *M, ID id, ArrayRef<Type*> OverloadedTys = None);
 #endif
-  IGCLLVM::AttributeSet getGenIntrinsicAttributes(LLVMContext& C, GenISAIntrinsic::ID id);
+  AttributeList getGenIntrinsicAttributes(LLVMContext& C, GenISAIntrinsic::ID id);
 
   //Override of isIntrinsic method defined in Function.h
   inline const char * getGenIntrinsicPrefix() { return "llvm.genx."; }
@@ -86,3 +98,4 @@ namespace GenISAIntrinsic {
 
 } // namespace llvm
 
+#endif //GENINTRINSICS_H
