@@ -68,8 +68,6 @@ typedef int INT; */
 
 #define HANDLE void*
 
-#define C_ASSERT(e)
-
 #include "GlobalData.h"
 
 #ifdef VER_H
@@ -85,8 +83,6 @@ typedef int INT; */
 #undef UINT
 #undef HANDLE
 
-#undef C_ASSERT
-
 class IGILContext : public llvm::LLVMContext
 {
 public:
