@@ -923,6 +923,15 @@ namespace TC
         {
             pClangArgs->options.assign(pOptions);
         }
+#if defined(IGC_DEBUG_VARIABLES)
+        char debugOptions[1024];
+        if (FCL::FCLReadIGCRegistry("ExtraOCLOptions", debugOptions, sizeof(debugOptions)))
+        {
+            if (!pClangArgs->options.empty())
+                pClangArgs->options += ' ';
+            pClangArgs->options += debugOptions;
+        }
+#endif
 
         GetOclCVersionFromOptions(pOptions, pInternalOptions, pClangArgs->oclVersion, exceptString);
         EnsureProperPCH(pClangArgs, pInternalOptions, exceptString);
@@ -967,7 +976,7 @@ namespace TC
             if (pData != NULL)
             {
                 assert(pData[uiDataSize - 1] == '\0' && "Program source is not null terminated");
-                pClangArgs->pszProgramSource = (const char *)pData;
+                pClangArgs->pszProgramSource = pData;
             }
         }
 
@@ -1008,27 +1017,26 @@ namespace TC
 
         if (!extensions.empty())
         {
-            output << "-cl-ext=-all,";
-            output << "+" << extensions[0];
+            output << "-cl-ext=-all";
+            for (const std::string &extension : extensions)
+                output << ",+" << extension;
         }
 
-        for (unsigned i = 1; i < extensions.size(); i++)
-            output << ",+" << extensions[i];
-
         output.flush();
         return output.str();
     }
 
-    std::string GetListOfExtensionsFromInternalOptions(const std::string& internalOptions) {
-    size_t start_pos = 0, end_pos = 0;
-    std::string clextString = "";
+    // Extracts a substring starting with "prefix" and ending with space.
+    std::string GetSubstring(const std::string& str, const std::string& prefix) {
+        size_t start_pos = 0, end_pos = 0;
+        std::string returnString = "";
 
-    while ((start_pos = internalOptions.find("-cl-ext=", end_pos)) != std::string::npos) {
-      end_pos = internalOptions.find(" ", start_pos);
-      clextString += internalOptions.substr(start_pos, end_pos - start_pos) + " ";
-    }
+        while ((start_pos = str.find(prefix, end_pos)) != std::string::npos) {
+          end_pos = str.find(' ', start_pos);
+          returnString += str.substr(start_pos, end_pos - start_pos) + " ";
+        }
 
-    return clextString;
+        return returnString;
     }
 
     std::string GetCDefinesFromInternalOptions(const char *pInternalOptions) {
@@ -1069,26 +1077,26 @@ namespace TC
 
     // The expected extensions input string is in a form:
     // -cl-ext=-all,+supported_ext_name,+second_supported_ext_name
-    std::string GetCDefinesForExtensions(llvm::StringRef extensions, unsigned int oclStd) {
+    // -cl-feature=+__opencl_c_3d_image_writes,+__opencl_c_atomic_order_acq_rel
+    std::string GetCDefinesForEnableList(llvm::StringRef enableListStr, unsigned int oclStd, const StringRef prefix) {
 
-      std::string extDefines;
+      std::string definesStr;
 
-      // check for the last occurence of -all, as it invalidates previous occurances.
-      const StringRef clExtPrefix = "-cl-ext=-all,";
-      size_t pos = extensions.rfind(clExtPrefix);
+      // check for the last occurence of prefix, as it invalidates previous occurances.
+      size_t pos = enableListStr.rfind(prefix);
       if (pos == llvm::StringRef::npos) {
         // If this string does not exist the input string does not contain valid extension list
         // or it has all extensions disabled (-all without colon afterwards).
-        return extDefines;
+        return definesStr;
       }
 
-      extensions = extensions.substr(pos+clExtPrefix.size());
+      enableListStr = enableListStr.substr(pos+prefix.size());
 
       // string with defines should be similar in size, ",+" will change to "-D".
-      extDefines.reserve(extensions.size());
+      definesStr.reserve(enableListStr.size());
 
       llvm::SmallVector<StringRef, 0> v;
-      extensions.split(v, ',');
+      enableListStr.split(v, ',');
 
       for (auto ext : v) {
         if (ext.consume_front("+")) {
@@ -1097,11 +1105,11 @@ namespace TC
             // This is because clang will not allow declarations of extension's functions which use avc types otherwise.
             if (!(oclStd >= 120 || oclStd == 0)) continue;
           }
-          extDefines.append(" -D").append(ext);
+          definesStr.append(" -D").append(ext);
         }
       }
 
-      return extDefines;
+      return definesStr;
     }
 
     /*****************************************************************************\
@@ -1323,18 +1331,21 @@ namespace TC
                             (strcmp(pParam, "-cl-denorms-are-zero") == 0) ||
                             (strcmp(pParam, "-cl-fp32-correctly-rounded-divide-sqrt") == 0) ||
                             (strcmp(pParam, "-cl-opt-disable") == 0) ||
+                            (strcmp(pParam, "-ze-opt-disable") == 0) ||
                             (strcmp(pParam, "-cl-strict-aliasing") == 0) ||
                             (strcmp(pParam, "-cl-mad-enable") == 0) ||
                             (strcmp(pParam, "-cl-no-signed-zeros") == 0) ||
                             (strcmp(pParam, "-cl-unsafe-math-optimizations") == 0) ||
                             (strcmp(pParam, "-cl-finite-math-only") == 0) ||
                             (strcmp(pParam, "-cl-fast-relaxed-math") == 0) ||
+                            (strcmp(pParam, "-cl-match-sincospi") == 0) ||
                             (strcmp(pParam, "-w") == 0) ||
                             (strcmp(pParam, "-Werror") == 0) ||
                             (strcmp(pParam, "-cl-std=CL1.1") == 0) ||
                             (strcmp(pParam, "-cl-std=CL1.2") == 0) ||
                             (strcmp(pParam, "-cl-std=CL2.0") == 0) ||
                             (strcmp(pParam, "-cl-std=CL2.1") == 0) ||
+                            (strcmp(pParam, "-cl-std=CL3.0") == 0) ||
                             (strcmp(pParam, "-cl-uniform-work-group-size") == 0) || //it's work only for OCL version greater than 1.2
                             (strcmp(pParam, "-cl-kernel-arg-info") == 0) ||
                             (strncmp(pParam, "-x", 2) == 0) ||
@@ -1346,10 +1357,17 @@ namespace TC
                             (strcmp(pParam, "-dwarf-column-info") == 0) ||
                             (strcmp(pParam, "-cl-intel-no-prera-scheduling") == 0) || //temporary options
                             (strcmp(pParam, "-igc_opts") == 0) || //temporary options
+                            (strcmp(pParam, "-cl-intel-greater-than-4GB-buffer-required") == 0) || //temporary options
+                            (strcmp(pParam, "-ze-opt-greater-than-4GB-buffer-required") == 0) || //temporary options
+                            (strcmp(pParam, "-cl-intel-has-buffer-offset-arg") == 0) || //temporary options
+                            (strcmp(pParam, "-ze-opt-has-buffer-offset-arg") == 0) || //temporary options
                             (strcmp(pParam, "-cl-intel-debug-info") == 0) ||
                             (strncmp(pParam, "-dump-opt-llvm", 14) == 0) ||
                             (strcmp(pParam, "-cl-no-subgroup-ifp") == 0) ||
-                            (strcmp(pParam, "-cl-intel-disable-a64WA") == 0); //temporary options;
+                            (strcmp(pParam, "-cl-intel-disable-a64WA") == 0) || //temporary options
+                            (strcmp(pParam, "-ze-gtpin-rera") == 0) || //used by GTPin
+                            (strcmp(pParam, "-ze-gtpin-grf-info") == 0) || //used by GTPin
+                            (strcmp(pParam, "-ze-gtpin-scratch-area-size") == 0);  //used by GTPin
 
 
 
@@ -1375,7 +1393,7 @@ namespace TC
                             {
                                 checkBinaryType = true;
                             }
-                            else if (strcmp(pParam, "-cl-intel-num-thread-per-eu") == 0)
+                            else if (strcmp(pParam, "-ze-gtpin-scratch-area-size") == 0)
                             {
                                 // Next token is N, so ignore it
                                 ignoreNextToken = true;
@@ -1480,7 +1498,8 @@ namespace TC
             optionsEx += " -debug-info-kind=line-tables-only -dwarf-version=4";
         }
 
-        std::string extensionsFromInternalOptions = GetListOfExtensionsFromInternalOptions(pInternalOptions);
+        std::string extensionsFromInternalOptions = GetSubstring(pInternalOptions, "-cl-ext=");
+
         std::string extensions;
 
         // if extensions list is passed in via internal options, it will override the default ones.
@@ -1498,7 +1517,27 @@ namespace TC
         unsigned int oclStd = GetOclCVersionFromOptions(pInputArgs->options.data(), nullptr, pInputArgs->oclVersion, exceptString);
         // get additional -D flags from internal options
         optionsEx += " " + GetCDefinesFromInternalOptions(pInternalOptions);
-        optionsEx += " " + GetCDefinesForExtensions(extensions, oclStd);
+        optionsEx += " " + GetCDefinesForEnableList(extensions, oclStd, "-cl-ext=-all,");
+
+#if defined(_WIN32)
+        if (oclStd >= 300) {
+          auto featureMacrosFromInternalOptions = GetSubstring(pInternalOptions, "-cl-feature=");
+
+          // Workaround for Clang issue.
+          // For write_imagef with image3d_t OpenCL 3.0 spec says:
+          // "Requires support for OpenCL C 2.0, the __opencl_c_3d_image_writes feature macro, or the cl_khr_3d_image_writes extension".
+          // In case only cl_khr_3d_image_writes is enabled, Clang triggers error for write_imagef declaration.
+          // TODO: remove when Clang is fixed.
+          if (extensions.find("+cl_khr_3d_image_writes") != std::string::npos &&
+              featureMacrosFromInternalOptions.find("+__opencl_c_3d_image_writes") == std::string::npos) {
+              featureMacrosFromInternalOptions += ",+__opencl_c_3d_image_writes";
+          }
+
+          optionsEx += " " + GetCDefinesForEnableList(featureMacrosFromInternalOptions, oclStd, "-cl-feature=");
+          optionsEx += " " + featureMacrosFromInternalOptions;
+        }
+#endif
+
         optionsEx += " -D__IMAGE_SUPPORT__ -D__ENDIAN_LITTLE__";
 
         IOCLFEBinaryResult *pResultPtr = NULL;
@@ -1757,15 +1796,14 @@ namespace TC
             bool successTC = TranslateClang(&args, pOutputArgs, exceptString, pInputArgs->pInternalOptions);
 
 #if defined(IGC_DEBUG_VARIABLES)
-            if (pInputArgs->pOptions != NULL)
+            if (pInputArgs->pOptions != nullptr)
             {
-                const std::string& igc_optsName = "-igc_opts";
-                const std::string& optionsWithFlags = (const char*)pInputArgs->pOptions;
-                std::size_t found = optionsWithFlags.find(igc_optsName);
+                const std::string optionsWithFlags = pInputArgs->pOptions;
+                std::size_t found = optionsWithFlags.find("-igc_opts");
                 if (found != std::string::npos)
                 {
-                    std::size_t foundFirstSingleQuote = optionsWithFlags.find("'", found);
-                    std::size_t foundSecondSingleQuote = optionsWithFlags.find("'", foundFirstSingleQuote + 1);
+                    std::size_t foundFirstSingleQuote = optionsWithFlags.find('\'', found);
+                    std::size_t foundSecondSingleQuote = optionsWithFlags.find('\'', foundFirstSingleQuote + 1);
                     if (foundFirstSingleQuote != std::string::npos && foundSecondSingleQuote != std::string::npos)
                     {
                         FCL::RegKeysFlagsFromOptions = optionsWithFlags.substr(foundFirstSingleQuote + 1, foundSecondSingleQuote - foundFirstSingleQuote - 1);
@@ -1779,12 +1817,12 @@ namespace TC
                 // Works for all OSes. Creates dir if necessary.
                 const char *pOutputFolder = FCL::GetShaderOutputFolder();
                 stringstream ss;
-                char* pBuffer = (char *)pInputArgs->pInput;
+                const char* pBuffer = pInputArgs->pInput;
                 UINT  bufferSize = pInputArgs->InputSize;
 
                 // Create hash based on cclang binary output (currently llvm binary; later also spirv).
                 // Hash computed in fcl needs to be same as the one computed in igc.
-                // This is to ensure easy matching .cl files dumped in fcl with .ll/.dat/.asm/... files dumoed in igc.
+                // This is to ensure easy matching .cl files dumped in fcl with .ll/.dat/.asm/... files dumped in igc.
                 QWORD hash = iSTD::Hash(reinterpret_cast<const DWORD *>(pOutputArgs->pOutput), (DWORD)(pOutputArgs->OutputSize) / 4);
 
                 ss << pOutputFolder;
