@@ -27,14 +27,12 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #ifndef IGCLLVM_IR_DIBUILDER_H
 #define IGCLLVM_IR_DIBUILDER_H
 
-#include <llvm/IR/DIBuilder.h>
-#include <llvmWrapper/IR/DebugInfoMetadata.h>
+#include "llvm/Config/llvm-config.h"
+#include "llvm/IR/DIBuilder.h"
+#include "llvm/IR/DebugInfoMetadata.h"
 
 namespace IGCLLVM
 {
-#if LLVM_VERSION_MAJOR == 4
-    using llvm::DIBuilder;
-#elif LLVM_VERSION_MAJOR >= 7
     class DIBuilder : public llvm::DIBuilder
     {
     public:
@@ -75,7 +73,7 @@ namespace IGCLLVM
             return llvm::DIBuilder::createNameSpace(Scope, Name, ExportSymbols);
         }
 
-        inline IGCLLVM::DISubprogram* createFunction(
+        inline llvm::DISubprogram* createFunction(
             llvm::DIScope *Scope, llvm::StringRef Name, llvm::StringRef LinkageName, llvm::DIFile *File,
             unsigned LineNo, llvm::DISubroutineType *Ty, bool isLocalToUnit,
             bool isDefinition, unsigned ScopeLine,
@@ -83,11 +81,11 @@ namespace IGCLLVM
             llvm::DITemplateParameterArray TParams = nullptr,
             llvm::DISubprogram *Decl = nullptr, llvm::DITypeArray ThrownTypes = nullptr)
         {
-            return (IGCLLVM::DISubprogram*)llvm::DIBuilder::createFunction(
+            return llvm::DIBuilder::createFunction(
 #if LLVM_VERSION_MAJOR == 7
                 Scope, Name, LinkageName, File, LineNo, Ty, isLocalToUnit, isDefinition,
                 ScopeLine, Flags, isOptimized, TParams, Decl, ThrownTypes);
-#elif LLVM_VERSION_MAJOR >= 8
+#else
                 Scope, Name, LinkageName, File, LineNo, Ty, isLocalToUnit, Flags,
                 llvm::DISubprogram::SPFlagDefinition, TParams, Decl, ThrownTypes);
 #endif
@@ -106,7 +104,7 @@ namespace IGCLLVM
 #if LLVM_VERSION_MAJOR == 7
                 Scope, Name, LinkageName, File, LineNo, Ty, isLocalToUnit, isDefinition,
                 Virtuality, VTableIndex, ThisAdjustment, VTableHolder, Flags, isOptimized, TParams, ThrownTypes);
-#elif LLVM_VERSION_MAJOR >= 8
+#else
                 Scope, Name, LinkageName, File, LineNo, Ty, Virtuality, ThisAdjustment,
                 VTableHolder, Flags, llvm::DISubprogram::SPFlagDefinition, TParams, ThrownTypes);
 #endif
@@ -125,13 +123,37 @@ namespace IGCLLVM
 #if LLVM_VERSION_MAJOR == 7
                 Scope, Name, LinkageName, File, LineNo, Ty, isLocalToUnit, isDefinition,
                 ScopeLine, Flags, isOptimized, TParams, Decl, ThrownTypes);
-#elif LLVM_VERSION_MAJOR >= 8
+#else
                 Scope, Name, LinkageName, File, LineNo, Ty, ScopeLine,
                 Flags, llvm::DISubprogram::SPFlagDefinition, TParams, Decl, ThrownTypes);
 #endif
         }
-    };
+
+        inline llvm::DITemplateTypeParameter* createTemplateTypeParameter(
+            llvm::DIScope* Scope,
+            llvm::StringRef Name,
+            llvm::DIType* Ty)
+        {
+#if LLVM_VERSION_MAJOR <= 10
+            return llvm::DIBuilder::createTemplateTypeParameter(Scope, Name, Ty);
+#else
+            return llvm::DIBuilder::createTemplateTypeParameter(Scope, Name, Ty, true);
 #endif
+        }
+
+        inline llvm::DITemplateValueParameter* createTemplateValueParameter(
+            llvm::DIScope* Scope,
+            llvm::StringRef Name,
+            llvm::DIType* Ty,
+            llvm::Constant* Val)
+        {
+#if LLVM_VERSION_MAJOR <= 10
+            return llvm::DIBuilder::createTemplateValueParameter(Scope, Name, Ty, Val);
+#else
+            return llvm::DIBuilder::createTemplateValueParameter(Scope, Name, Ty, true, Val);
+#endif
+        }
+    };
 }
 
 #endif
