@@ -27,28 +27,31 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #ifndef IGCLLVM_IR_INSTRTYPES_H
 #define IGCLLVM_IR_INSTRTYPES_H
 
-#include <llvm/IR/InstrTypes.h>
+#include "llvm/Config/llvm-config.h"
+#include "llvm/IR/InstrTypes.h"
 #if LLVM_VERSION_MAJOR >= 8
-#include <llvm/IR/PatternMatch.h>
+#include "llvm/IR/PatternMatch.h"
 #endif
 
 namespace IGCLLVM
 {
 #if LLVM_VERSION_MAJOR <= 7
     using llvm::TerminatorInst;
-    using llvm::BinaryOperator;
-#elif LLVM_VERSION_MAJOR >= 8
+#else
     using TerminatorInst = llvm::Instruction;
-
-    class BinaryOperator : public llvm::BinaryOperator
-    {
-    public:
-         static inline bool isNot(const llvm::Value *V)
-         {
-             return llvm::PatternMatch::match(V, llvm::PatternMatch::m_Not(llvm::PatternMatch::m_Value()));
-         }
-    };
 #endif
+
+    namespace BinaryOperator
+    {
+        inline bool isNot(const llvm::Value *V)
+        {
+#if LLVM_VERSION_MAJOR <= 7
+            return llvm::BinaryOperator::isNot(V);
+#else
+            return llvm::PatternMatch::match(V, llvm::PatternMatch::m_Not(llvm::PatternMatch::m_Value()));
+#endif
+        }
+    }
 }
 
 #endif
