@@ -27,53 +27,63 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #ifndef IGCLLVM_IR_INSTRUCTIONS_H
 #define IGCLLVM_IR_INSTRUCTIONS_H
 
-#include <llvm/IR/Instructions.h>
+#include "llvm/Config/llvm-config.h"
+#include "llvm/IR/Instructions.h"
+#if LLVM_VERSION_MAJOR <= 7
+#include "llvm/Support/Casting.h"
+#endif
 
 namespace IGCLLVM
 {
-    template<typename T>
-    inline void CopySyncScopeID(T* LHS, T* RHS)
+
+    inline llvm::Value* getCalledValue(llvm::CallInst& CI)
     {
-#if LLVM_VERSION_MAJOR == 4
-        LHS->setSynchScope(RHS->getSynchScope());
-#elif LLVM_VERSION_MAJOR >= 7
-        LHS->setSyncScopeID(RHS->getSyncScopeID());
+#if LLVM_VERSION_MAJOR <= 10
+        return CI.getCalledValue();
+#else
+        return CI.getCalledOperand();
 #endif
     }
 
-    template<class T = llvm::Instruction>
-    inline
-#if LLVM_VERSION_MAJOR == 4
-        llvm::SynchronizationScope
-#elif LLVM_VERSION_MAJOR >= 7
-        llvm::SyncScope::ID
-#endif
-        getSyncScopeID(T* I)
+    inline llvm::Value* getCalledValue(llvm::CallInst* CI)
     {
-#if LLVM_VERSION_MAJOR == 4
-        return I->getSynchScope();
-#elif LLVM_VERSION_MAJOR >= 7
-        return I->getSyncScopeID();
+#if LLVM_VERSION_MAJOR <= 10
+        return CI->getCalledValue();
+#else
+        return CI->getCalledOperand();
 #endif
     }
 
-    template<class T = llvm::Instruction>
-    inline void DeleteInstruction(T*& I)
+    inline const llvm::Value* getCalledValue(const llvm::CallInst* CI)
     {
-#if LLVM_VERSION_MAJOR == 4
-        delete I;
-#elif LLVM_VERSION_MAJOR >= 7
-        I->deleteValue();
+#if LLVM_VERSION_MAJOR <= 10
+        return CI->getCalledValue();
+#else
+        return CI->getCalledOperand();
 #endif
     }
 
-    template<class T = llvm::Instruction>
-    inline unsigned getDestAlignment(T* I)
+    inline bool isIndirectCall(const llvm::CallInst& CI)
     {
-#if LLVM_VERSION_MAJOR == 4
-        return I->getAlignment();
-#elif LLVM_VERSION_MAJOR >= 7
-        return I->getDestAlignment();
+#if LLVM_VERSION_MAJOR == 7
+        const llvm::Value *V = CI.getCalledValue();
+        if (llvm::isa<llvm::Function>(V) || llvm::isa<llvm::Constant>(V))
+            return false;
+        if (CI.isInlineAsm())
+            return false;
+        return true;
+#else
+        return CI.isIndirectCall();
+#endif
+    }
+
+
+    inline llvm::Constant* getShuffleMaskForBitcode(llvm::ShuffleVectorInst* SVI)
+    {
+#if LLVM_VERSION_MAJOR < 11
+        return SVI->getMask();
+#else
+        return SVI->getShuffleMaskForBitcode();
 #endif
     }
 }
