@@ -27,8 +27,9 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #ifndef IGCLLVM_IR_MODULE_H
 #define IGCLLVM_IR_MODULE_H
 
-#include <llvm/IR/Module.h>
-#include "Attributes.h"
+#include "llvm/Config/llvm-config.h"
+#include "llvm/IR/Module.h"
+#include "llvm/IR/Attributes.h"
 
 namespace IGCLLVM
 {
@@ -46,9 +47,9 @@ namespace IGCLLVM
 		{
             return llvm::Module::getOrInsertFunction(Name, Ty).getCallee();
 		}
-        inline llvm::Value* getOrInsertFunction(llvm::StringRef Name, llvm::FunctionType *Ty, IGCLLVM::AttributeSet AttributeList)
+        inline llvm::Value* getOrInsertFunction(llvm::StringRef Name, llvm::FunctionType *Ty, llvm::AttributeList attributeList)
         {
-            return llvm::Module::getOrInsertFunction(Name, Ty, AttributeList).getCallee();
+            return llvm::Module::getOrInsertFunction(Name, Ty, attributeList).getCallee();
         }
     };
 #endif
