@@ -27,17 +27,26 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #ifndef IGCLLVM_TRANSFORMS_UTILS_CLONING_H
 #define IGCLLVM_TRANSFORMS_UTILS_CLONING_H
 
-#include <llvm/Transforms/Utils/Cloning.h>
+#include "llvm/Config/llvm-config.h"
+#include "llvm/Transforms/Utils/Cloning.h"
+
 namespace IGCLLVM
 {
-#if LLVM_VERSION_MAJOR == 4
-    using llvm::CloneModule;
-#elif LLVM_VERSION_MAJOR >= 7
-    inline std::unique_ptr<llvm::Module> CloneModule(const llvm::Module *M)
+    inline bool InlineFunction(llvm::CallInst* CB, llvm::InlineFunctionInfo& IFI,
+        llvm::AAResults* CalleeAAR = nullptr,
+        bool InsertLifetime = true,
+        llvm::Function* ForwardVarArgsTo = nullptr)
     {
-        return llvm::CloneModule(*M);
-    }
+        return llvm::InlineFunction(
+#if LLVM_VERSION_MAJOR >= 11
+            *
 #endif
+            CB, IFI, CalleeAAR, InsertLifetime, ForwardVarArgsTo)
+#if LLVM_VERSION_MAJOR >= 11
+            .isSuccess()
+#endif
+            ;
+    }
 }
 
 #endif
