@@ -1,8 +1,7 @@
+#pragma once
 
 #include "common/LLVMWarningsPush.hpp"
-
 #include "llvmWrapper/IR/IRBuilder.h"
-
 #include "llvm/ADT/ArrayRef.h"
 #include "llvm/IR/Instructions.h"
 #include "llvm/IR/Intrinsics.h"
@@ -15,7 +14,6 @@
 #include "llvm/IR/InstrTypes.h"
 #include "common/LLVMWarningsPop.hpp"
 
-#pragma once
 //This Builder class provides definitions for functions calls that were once available till LLVM version 3.6.0
 //===--------------------------------------------------------------------===
 // CreateCall Variations which are removed in 3.8 for API Simplification, which IGC still finds convenient to use
@@ -23,25 +21,25 @@
 //===--------------------------------------------------------------------===
 namespace llvm {
 
-    template<typename T = ConstantFolder, typename Inserter = IRBuilderDefaultInserter >
-    class IGCIRBuilder : public IGCLLVM::IRBuilder<T, Inserter>
+    template<typename T = ConstantFolder, typename InserterTyDef() = IRBuilderDefaultInserter >
+    class IGCIRBuilder : public IGCLLVM::IRBuilder<T, InserterTyDef()>
     {
     public:
-        IGCIRBuilder(LLVMContext &C, const T &F, Inserter I = Inserter(),
+        IGCIRBuilder(LLVMContext &C, const T &F, InserterTyDef() I = InserterTyDef()(),
             MDNode *FPMathTag = nullptr,
             ArrayRef<OperandBundleDef> OpBundles = None)
-            : IGCLLVM::IRBuilder<T, Inserter>(C, F, I, FPMathTag, OpBundles){}
+            : IGCLLVM::IRBuilder<T, InserterTyDef()>(C, F, I, FPMathTag, OpBundles){}
 
         explicit IGCIRBuilder(LLVMContext &C, MDNode *FPMathTag = nullptr,
             ArrayRef<OperandBundleDef> OpBundles = None)
-            : IGCLLVM::IRBuilder<T, Inserter>(C, FPMathTag, OpBundles){}
+            : IGCLLVM::IRBuilder<T, InserterTyDef()>(C, FPMathTag, OpBundles){}
 
         explicit IGCIRBuilder(BasicBlock *TheBB, MDNode *FPMathTag = nullptr)
-            : IGCLLVM::IRBuilder<T, Inserter>(TheBB, FPMathTag){}
+            : IGCLLVM::IRBuilder<T, InserterTyDef()>(TheBB, FPMathTag){}
 
         explicit IGCIRBuilder(Instruction *IP, MDNode *FPMathTag = nullptr,
             ArrayRef<OperandBundleDef> OpBundles = None)
-            : IGCLLVM::IRBuilder<T, Inserter>(IP, FPMathTag, OpBundles) {}
+            : IGCLLVM::IRBuilder<T, InserterTyDef()>(IP, FPMathTag, OpBundles) {}
 
         CallInst *CreateCall2(Value *Callee, Value *Arg1, Value *Arg2,
             const Twine &Name = "") {
@@ -79,28 +77,11 @@ namespace llvm {
             return ne0;
         }
 
-        inline Value* CreateAllValuesAreConstF(Value** values,
-            unsigned nvalues, Value* constVal)
-        {
-            if (nvalues)
-            {
-                assert(isa<ConstantFP>(constVal));
-                Value* aeq = this->CreateFCmpOEQ(values[0], constVal);
-                for (unsigned i = 1; i < nvalues; i++)
-                {
-                    aeq = this->CreateAnd(aeq,
-                        this->CreateFCmpOEQ(values[i], constVal));
-                }
-                return aeq;
-            }
-            return nullptr;
-        }
-
         inline Value* CreateAllValuesAreZeroF(Value** values, unsigned nvalues)
         {
             if (nvalues)
             {
-                return CreateAllValuesAreConstF(values, nvalues,
+                return CreateAllValuesAreConstantFP(values, nvalues,
                     ConstantFP::get(values[0]->getType(), 0.0));
             }
             return nullptr;
@@ -110,7 +91,7 @@ namespace llvm {
         {
             if (nvalues)
             {
-                return CreateAllValuesAreConstF(values, nvalues,
+                return CreateAllValuesAreConstantFP(values, nvalues,
                     ConstantFP::get(values[0]->getType(), 1.0));
             }
             return nullptr;
@@ -159,6 +140,24 @@ namespace llvm {
         {
             return CreateExtractElementOrPropagate(vec, this->getInt64(idx), name);
         }
+
+    private:
+
+        inline Value* CreateAllValuesAreConstantFP(Value** values,
+            unsigned nvalues, Value* constVal)
+        {
+            if (nvalues)
+            {
+                Value* aeq = this->CreateFCmpOEQ(values[0], constVal);
+                for (unsigned i = 1; i < nvalues; i++)
+                {
+                    aeq = this->CreateAnd(aeq,
+                        this->CreateFCmpOEQ(values[i], constVal));
+                }
+                return aeq;
+            }
+            return nullptr;
+        }
     };
 
 } // end namespace llvm
