@@ -43,6 +43,7 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <iomanip>
 #include <sstream>
 #include <iostream>
+#include "Probe/Assertion.h"
 
 #if GET_TIME_STATS
 // Functions exposed by VISA lib API
@@ -68,7 +69,8 @@ const char* g_cShaderStatItems[STATS_MAX_SHADER_STATS_ITEMS+1] =
 
 int ShaderStats::getShaderStats( SHADER_STATS_ITEMS compileInterval )
 {
-    assert( compileInterval >= 0 && compileInterval < STATS_MAX_SHADER_STATS_ITEMS );
+    IGC_ASSERT(0 <= compileInterval);
+    IGC_ASSERT(compileInterval < STATS_MAX_SHADER_STATS_ITEMS);
     return m_CompileShaderStats[compileInterval];
 }
 
@@ -386,7 +388,7 @@ void ShaderStats::parseIsaShader( ShaderHash hash, ShaderType shaderType, SIMDMo
         }
         else
         {
-            assert(0);
+            IGC_ASSERT(0);
         }
     }
 
@@ -418,7 +420,8 @@ void ShaderStats::parseIsaShader( ShaderHash hash, ShaderType shaderType, SIMDMo
 
 void ShaderStats::sumShaderStat( SHADER_STATS_ITEMS compileInterval, int count )
 {
-    assert( compileInterval >= 0 && compileInterval < STATS_MAX_SHADER_STATS_ITEMS );
+    IGC_ASSERT(0 <= compileInterval);
+    IGC_ASSERT(compileInterval < STATS_MAX_SHADER_STATS_ITEMS);
     m_CompileShaderStats[ compileInterval ] += count;
 };
 
@@ -455,7 +458,7 @@ std::string str(COMPILE_TIME_INTERVALS cti)
 #define DEFINE_TIME_STAT( enumName, stringName, parentEnum, isVISA, isUnacc, isCoarseTimer, isDashBoardTimer ) case enumName: return #enumName;
 #include "timeStats.def"
 #undef DEFINE_TIME_STAT
-    default: assert( 0 && "unreachable" ); break;
+    default: IGC_ASSERT_MESSAGE(0, "unreachable"); break;
     }
 
     return "";
@@ -471,7 +474,7 @@ COMPILE_TIME_INTERVALS interval( std::string const& str )
 #include "timeStats.def"
 #undef DEFINE_TIME_STAT
     //llvm::errs() << str;
-    assert( 0 && "unreachable" && "unknown COMPILE_TIME_INTERVALS name" );
+    IGC_ASSERT_MESSAGE(0, "unreachable, unknown COMPILE_TIME_INTERVALS name");
     return MAX_COMPILE_TIME_INTERVALS;
 }
 
@@ -482,7 +485,7 @@ bool isVISATimer( COMPILE_TIME_INTERVALS cti )
 #define DEFINE_TIME_STAT( enumName, stringName, parentEnum, isVISA, isUnacc, isCoarseTimer, isDashBoardTimer ) case enumName: return isVISA;
 #include "timeStats.def"
 #undef DEFINE_TIME_STAT
-    default: assert( 0 && "unreachable" ); break;
+    default: IGC_ASSERT_MESSAGE(0, "unreachable"); break;
     }
     return false;
 }
@@ -494,7 +497,7 @@ bool isUnaccounted( COMPILE_TIME_INTERVALS cti )
 #define DEFINE_TIME_STAT( enumName, stringName, parentEnum, isVISA, isUnacc, isCoarseTimer, isDashBoardTimer ) case enumName: return isUnacc;
 #include "timeStats.def"
 #undef DEFINE_TIME_STAT
-    default: assert( 0 && "unreachable" ); break;
+    default: IGC_ASSERT_MESSAGE(0, "unreachable"); break;
     }
     return false;
 }
@@ -506,7 +509,7 @@ bool isCoarseTimer( COMPILE_TIME_INTERVALS cti )
 #define DEFINE_TIME_STAT( enumName, stringName, parentEnum, isVISA, isUnacc, isCoarseTimer, isDashBoardTimer ) case enumName: return isCoarseTimer;
 #include "timeStats.def"
 #undef DEFINE_TIME_STAT
-default: assert(0 && "unreachable"); break;
+    default: IGC_ASSERT_MESSAGE(0, "unreachable"); break;
     }
     return true;
 }
@@ -518,7 +521,7 @@ bool isDashboardTimer( COMPILE_TIME_INTERVALS cti )
 #define DEFINE_TIME_STAT( enumName, stringName, parentEnum, isVISA, isUnacc, isCoarseTimer, isDashBoardTimer ) case enumName: return isDashBoardTimer;
 #include "timeStats.def"
 #undef DEFINE_TIME_STAT
-    default: assert( 0 && "unreachable" ); break;
+    default: IGC_ASSERT_MESSAGE(0, "unreachable"); break;
     }
 
     return true;
@@ -531,7 +534,7 @@ COMPILE_TIME_INTERVALS parentInterval( COMPILE_TIME_INTERVALS cti )
 #define DEFINE_TIME_STAT( enumName, stringName, parentEnum, isVISA, isUnacc, isCoarseTimer, isDashBoardTimer ) case enumName: return parentEnum;
 #include "timeStats.def"
 #undef DEFINE_TIME_STAT
-    default: assert( 0 && "unreachable" ); break;
+    default: IGC_ASSERT_MESSAGE(0, "unreachable"); break;
     }
     return MAX_COMPILE_TIME_INTERVALS;
 }
@@ -580,26 +583,30 @@ void TimeStats::recordVISATimers()
 
 void TimeStats::recordTimerStart( COMPILE_TIME_INTERVALS compileInterval )
 {
-    assert( compileInterval >= 0 && compileInterval < MAX_COMPILE_TIME_INTERVALS );
+    IGC_ASSERT(0 <= compileInterval);
+    IGC_ASSERT(compileInterval < MAX_COMPILE_TIME_INTERVALS);
     m_wallclockStart[ compileInterval ] = iSTD::GetTimestampCounter();
 }
 
 void TimeStats::recordTimerEnd( COMPILE_TIME_INTERVALS compileInterval )
 {
-    assert( compileInterval >= 0 && compileInterval < MAX_COMPILE_TIME_INTERVALS );
+    IGC_ASSERT(0 <= compileInterval);
+    IGC_ASSERT(compileInterval < MAX_COMPILE_TIME_INTERVALS);
     m_elapsedTime[ compileInterval ] += iSTD::GetTimestampCounter() - m_wallclockStart[ compileInterval ];
     m_hitCount[ compileInterval ]++;
 }
 
 uint64_t TimeStats::getCompileTime( COMPILE_TIME_INTERVALS compileInterval ) const
 {
-    assert( compileInterval >= 0 && compileInterval < MAX_COMPILE_TIME_INTERVALS );
+    IGC_ASSERT(0 <= compileInterval);
+    IGC_ASSERT(compileInterval < MAX_COMPILE_TIME_INTERVALS);
     return m_elapsedTime[ compileInterval ];
 }
 
 uint64_t TimeStats::getCompileHit( COMPILE_TIME_INTERVALS compileInterval ) const
 {
-    assert( compileInterval >= 0 && compileInterval < MAX_COMPILE_TIME_INTERVALS );
+    IGC_ASSERT(0 <= compileInterval);
+    IGC_ASSERT(compileInterval < MAX_COMPILE_TIME_INTERVALS);
     return m_hitCount[ compileInterval ];
 }
 
@@ -616,7 +623,7 @@ void TimeStats::sumWith( const TimeStats* pOther )
 
     m_PassTotalTicks += pOther->m_PassTotalTicks;
 
-    if (IGC::Debug::GetDebugFlag(IGC::Debug::DebugFlag::TIME_STATS_PER_PASS))
+    if (IGC_REGKEY_OR_FLAG_ENABLED(DumpTimeStatsPerPass, TIME_STATS_PER_PASS))
     {
         if (m_PassTimeStatsMap.empty())
         {
@@ -674,9 +681,17 @@ void TimeStats::printTime( ShaderType type, ShaderHash hash, void* context ) con
 
 void TimeStats::printSumTime() const
 {
+    // If using regkey to turn on timestats, CorpusName is not initialized properly
+    if (strlen(IGC::Debug::GetShaderCorpusName()) == 0)
+    {
+        std::stringstream corpusName;
+        corpusName << m_totalShaderCount << " shaders";
+        IGC::Debug::SetShaderCorpusName(corpusName.str().c_str());
+    }
+
     TimeStats pp = postProcess();
 
-    bool dumpCoarse = IGC::Debug::GetDebugFlag(IGC::Debug::DebugFlag::TIME_STATS_COARSE );
+    bool dumpCoarse = IGC_REGKEY_OR_FLAG_ENABLED(DumpTimeStatsCoarse, TIME_STATS_COARSE);
 
     if ( dumpCoarse )
     {
@@ -687,7 +702,7 @@ void TimeStats::printSumTime() const
         pp.printSumTimeCSV("c:\\Intel\\TimeStatSum.csv");
     }
 
-    if (IGC::Debug::GetDebugFlag(IGC::Debug::DebugFlag::TIME_STATS_PER_PASS))
+    if (IGC_REGKEY_OR_FLAG_ENABLED(DumpTimeStatsPerPass, TIME_STATS_PER_PASS))
     {
         pp.printPerPassSumTime(llvm::dbgs());
         pp.printPerPassSumTimeCSV("c:\\Intel\\TimeStatPerPassSum.csv");
@@ -699,7 +714,7 @@ void TimeStats::printSumTime() const
 bool TimeStats::skipTimer( int i ) const
 {
     const COMPILE_TIME_INTERVALS interval = static_cast<COMPILE_TIME_INTERVALS>(i);
-    if( IGC::Debug::GetDebugFlag(IGC::Debug::DebugFlag::TIME_STATS_COARSE ) && !isCoarseTimer( interval ) )
+    if( IGC_REGKEY_OR_FLAG_ENABLED(DumpTimeStatsCoarse, TIME_STATS_COARSE ) && !isCoarseTimer( interval ) )
     {
         return true;
     }
@@ -711,7 +726,7 @@ bool TimeStats::skipTimer( int i ) const
 }
 void TimeStats::printSumTimeCSV(const char* outputFile) const
 {
-    assert( m_isPostProcessed && "Print functions should only be called on a Post-Processed TimeStats object" );
+    IGC_ASSERT_MESSAGE(m_isPostProcessed, "Print functions should only be called on a Post-Processed TimeStats object");
 
     bool fileExist = false;
 
@@ -752,7 +767,7 @@ void TimeStats::printSumTimeCSV(const char* outputFile) const
         }
         fprintf(fileName, "\n");
 
-        if( !IGC::Debug::GetDebugFlag(IGC::Debug::DebugFlag::TIME_STATS_COARSE ) )
+        if( !IGC_REGKEY_OR_FLAG_ENABLED(DumpTimeStatsCoarse, TIME_STATS_COARSE))
         {
             // print secs
             fprintf(fileName, "seconds,," );
@@ -785,7 +800,7 @@ void TimeStats::printSumTimeCSV(const char* outputFile) const
 
 void TimeStats::printPerPassSumTimeCSV(const char* outputFile) const
 {
-    assert(m_isPostProcessed && "Print functions should only be called on a Post-Processed TimeStats object");
+    IGC_ASSERT_MESSAGE(m_isPostProcessed, "Print functions should only be called on a Post-Processed TimeStats object");
 
     if (m_PassTimeStatsMap.empty())
     {
@@ -832,7 +847,7 @@ void TimeStats::printPerPassSumTimeCSV(const char* outputFile) const
 
 void TimeStats::recordPerPassTimerStart(std::string PassName)
 {
-    assert(!PassName.empty());
+    IGC_ASSERT(!PassName.empty());
 
     std::map<std::string, PerPassTimeStat>::iterator iter = m_PassTimeStatsMap.find(PassName);
     if (iter == m_PassTimeStatsMap.end())
@@ -852,7 +867,7 @@ void TimeStats::recordPerPassTimerStart(std::string PassName)
 
 void TimeStats::recordPerPassTimerEnd(std::string PassName)
 {
-    assert(!PassName.empty());
+    IGC_ASSERT(!PassName.empty());
 
     std::map<std::string, PerPassTimeStat>::iterator iter = m_PassTimeStatsMap.find(PassName);
 
@@ -897,7 +912,7 @@ namespace {
 
 void TimeStats::printSumTimeTable( llvm::raw_ostream & OS ) const
 {
-    assert( m_isPostProcessed && "Print functions should only be called on a Post-Processed TimeStats object" );
+    IGC_ASSERT_MESSAGE(m_isPostProcessed, "Print functions should only be called on a Post-Processed TimeStats object");
 
     llvm::formatted_raw_ostream FS(OS);
 
@@ -927,7 +942,7 @@ void TimeStats::printSumTimeTable( llvm::raw_ostream & OS ) const
     FS << "\n";
 
     // table body
-    if (IGC::Debug::GetDebugFlag(IGC::Debug::DebugFlag::TIME_STATS_COARSE))
+    if (IGC_REGKEY_OR_FLAG_ENABLED(DumpTimeStatsCoarse, TIME_STATS_COARSE))
     {
         uint64_t timeNotInCoarse = getCompileTime(TIME_TOTAL);
         for (int i = 0; i < MAX_COMPILE_TIME_INTERVALS; i++)
@@ -1040,9 +1055,14 @@ void TimeStats::printPerPassSumTime(llvm::raw_ostream& OS) const
 
 void TimeStats::printTimeCSV( std::string const& corpusName ) const
 {
-    assert( m_isPostProcessed && "Print functions should only be called on a Post-Processed TimeStats object" );
+    IGC_ASSERT_MESSAGE(m_isPostProcessed, "Print functions should only be called on a Post-Processed TimeStats object");
 
-    const std::string outputFilePath = std::string("c:\\Intel\\") + "TimeStat_" + IGC::Debug::GetShaderCorpusName() + ".csv";
+    std::string subFile = "TimeStat_";
+    if (strlen(IGC::Debug::GetShaderCorpusName()) == 0)
+        subFile += "Shaders";
+    else
+        subFile += IGC::Debug::GetShaderCorpusName();
+    const std::string outputFilePath = std::string("c:\\Intel\\") + subFile + ".csv";
     const char *outputFile = outputFilePath.c_str();
 
     bool fileExist = false;
@@ -1089,14 +1109,19 @@ void TimeStats::printTimeCSV( std::string const& corpusName ) const
 
 void TimeStats::printPerPassTimeCSV(std::string const& corpusName) const
 {
-    assert(m_isPostProcessed && "Print functions should only be called on a Post-Processed TimeStats object");
+    IGC_ASSERT_MESSAGE(m_isPostProcessed, "Print functions should only be called on a Post-Processed TimeStats object");
 
     if (m_PassTimeStatsMap.empty())
     {
         return;
     }
 
-    const std::string outputFilePath = std::string("c:\\Intel\\") + "TimeStatPerPass_" + IGC::Debug::GetShaderCorpusName() + ".csv";
+    std::string subFile = "TimeStatPerPass_";
+    if (strlen(IGC::Debug::GetShaderCorpusName()) == 0)
+        subFile += "Shaders";
+    else
+        subFile += IGC::Debug::GetShaderCorpusName();
+    const std::string outputFilePath = std::string("c:\\Intel\\") + subFile + ".csv";
     const char* outputFile = outputFilePath.c_str();
     bool fileExist = false;
 
