@@ -25,11 +25,11 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 ======================= end_copyright_notice ==================================*/
 
 #include "SysUtils.hpp"
-
-#include <cassert>
 #include <sstream>
 #include <fstream>
 #include <sys/stat.h>
+#include "Probe/Assertion.h"
+
 using namespace std;
 
 #ifndef S_ISDIR
@@ -74,18 +74,16 @@ namespace IGC
 
 #if defined (__linux__)
             ifstream in(MakeStr("/proc/", getpid(), "/cmdline"), ios::in);
-            if (!in.is_open())
-                assert(0 && "Can not open cmdline pseudo file for current process");
+            IGC_ASSERT_MESSAGE(in.is_open(), "Can not open cmdline pseudo file for current process");
 
             //in cmdline arguments are separated with \0
             getline(in, ret, '\0');
-            if (!in.good())
-                assert(0 && "Error reading from cmdline pseudo file");
+            IGC_ASSERT_MESSAGE(in.good(), "Error reading from cmdline pseudo file");
 
 #elif defined(_WIN64) || defined(_WIN32)
             ret.resize(MAX_PATH);
             DWORD size = ::GetModuleFileNameA(NULL, &ret[0], ret.size());
-            assert(size < ret.size() && "Windows path can have MAX_PATH characters max");
+            IGC_ASSERT_MESSAGE(size < ret.size(), "Windows path can have MAX_PATH characters max");
             ret.resize(size);
 #endif
 
