@@ -48,7 +48,7 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "../AdaptorOCL/OCL/LoadBuffer.h"
 #include "Compiler/CISACodeGen/Platform.hpp"
 #include "common/debug/Debug.hpp"
-#include "Probe.h"
+#include "Probe/Assertion.h"
 
 namespace SIP
 {
@@ -69,7 +69,6 @@ namespace SIP
         GEN9_GLV_SIP_CSR,
         GEN11_SIP_CSR,
         GEN11_SIP_CSR_DEBUG,
-        GEN11_HP_SIP_CSR,
         GEN11_LKF_SIP_CSR,
         GEN12_LP_CSR,
         GEN12_LP_CSR_DEBUG,
