@@ -34,6 +34,7 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "3d/common/iStdLib/File.h"
 #include "secure_mem.h"
 #include "secure_string.h"
+#include "AdaptorCommon/customApi.hpp"
 
 #if defined(_WIN64) || defined(_WIN32)
 #include <devguid.h>  // for GUID_DEVCLASS_DISPLAY
@@ -47,11 +48,11 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <list>
 #include <vector>
 #include <string>
-#include <cassert>
 #include <utility>
 #include <fstream>
 #include <sstream>
 #include <mutex>
+#include "Probe/Assertion.h"
 
 // path for IGC registry keys
 #define IGC_REGISTRY_KEY "SOFTWARE\\INTEL\\IGFX\\IGC"
@@ -59,19 +60,82 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 SRegKeysList g_RegKeyList;
 
 #if defined(_WIN64) || defined(_WIN32)
-#if _DEBUG
-#define DBGOUT stdout  // change to stderr if debugger output is preferred
-#define DBG(fmt, ...) { \
-    fprintf_s(DBGOUT, fmt, __VA_ARGS__); \
+
+DEVICE_INFO::DEVICE_INFO(DEVINST deviceInstance)
+{
+    deviceID = 0;
+    revisionID = 0;
+    pciBus = 0;
+    pciDevice = 0;
+    pciFunction = 0;
+    get_device_property(deviceInstance, CM_DRP_DEVICEDESC);
+    get_device_property(deviceInstance, CM_DRP_COMPATIBLEIDS);
+    get_device_property(deviceInstance, CM_DRP_DRIVER);
+    get_device_property(deviceInstance, CM_DRP_LOCATION_INFORMATION);
+}
+
+void DEVICE_INFO::get_device_property(DEVINST deviceInstace, DWORD property)
+{
+    wchar_t propertyData[MAX_PATH];
+    DWORD   propertyDataLength = 0;
+
+    // First get the size of the data
+    CONFIGRET status = CM_Get_DevNode_Registry_PropertyW(
+        deviceInstace,
+        property,
+        NULL,
+        NULL,
+        &propertyDataLength,
+        0);
+
+    // Then fetch the actual property data
+    if(CR_BUFFER_SMALL == status)
+    {
+        status = CM_Get_DevNode_Registry_PropertyW(
+            deviceInstace,
+            property,
+            NULL,
+            &propertyData[0],
+            &propertyDataLength,
+            0);
     }
-#define DBGERR(fmt, ...) { \
-    fprintf_s(DBGOUT, "%s [%d] --> ", __FUNCTION__, __LINE__); \
-    fprintf_s(DBGOUT, fmt, __VA_ARGS__); \
+
+    if(CR_SUCCESS == status)
+    {
+        // Convert from UTF-16 (wide char) to UTF-8
+        DWORD convertedStringSize = WideCharToMultiByte(CP_UTF8, 0, &propertyData[0], sizeof(propertyData), NULL, 0, NULL, NULL);
+        std::string propertyString(convertedStringSize, 0);
+        WideCharToMultiByte(CP_UTF8, 0, &propertyData[0], sizeof(propertyData), &propertyString[0], convertedStringSize, NULL, NULL);
+
+        switch(property)
+        {
+        case CM_DRP_COMPATIBLEIDS:
+            // PCI\VEN_8086&DEV_1912&REV_06
+            sscanf_s(propertyString.c_str(), "PCI\\VEN_8086&DEV_%x&REV_%x", &deviceID, &revisionID);
+            break;
+        case CM_DRP_DEVICEDESC:
+            // Intel(R) HD Graphics 530
+            description = propertyString.c_str();
+            break;
+        case CM_DRP_DRIVER:
+            // {4d36e968-e325-11ce-bfc1-08002be10318}\0001
+            driverRegistryPath = propertyString.c_str();
+            break;
+        case CM_DRP_LOCATION_INFORMATION:
+            // PCI bus 0, device 2, function 0
+            sscanf_s(propertyString.c_str(), "PCI bus %u, device %u, function %u", &pciBus, &pciDevice, &pciFunction);
+            break;
+        }
     }
-#else
-#define DBG(fmt, ...)
-#define DBGERR(fmt, ...)
-#endif
+    else if(CR_NO_SUCH_VALUE == status)
+    {
+        IGC_ASSERT_MESSAGE(0, "No such property value");
+    }
+    else
+    {
+        IGC_ASSERT_MESSAGE(0, "Failed to get DevNode property");
+    }
+}
 
 /********************************************************************************************/
 /* Function: ConvertDoubleNullTermStringToVector                                            */
@@ -125,7 +189,7 @@ static bool GetPropertyFromDevice(
     CONFIGRET status = CM_Get_DevNode_PropertyW(devInst, &PropertyKey, &propertyType, NULL, &propertySize, 0);
     if (status != CR_BUFFER_SMALL)
     {
-        DBGERR("CM_Get_DevNode_PropertyW() failed with the error code 0x%02x\n", status);
+        IGC_ASSERT_MESSAGE(0, "CM_Get_DevNode_PropertyW() failed, status indicates error code");
         return false;
     }
 
@@ -137,7 +201,7 @@ static bool GetPropertyFromDevice(
     status = CM_Get_DevNode_PropertyW(devInst, &PropertyKey, &propertyType, &PropertyData[0], &propertySize, 0);
     if (status != CR_SUCCESS)
     {
-        DBGERR("CM_Get_DevNode_PropertyW() failed with the error code 0x%02x\n", status);
+        IGC_ASSERT_MESSAGE(0, "CM_Get_DevNode_PropertyW() failed, status indicates error code");
         PropertyData.clear();
         return false;
     }
@@ -252,7 +316,7 @@ static size_t GetIntelDriverPaths(std::vector<DEVINST>& drivers)
     HRESULT hr = ObtainDeviceInstances(&drivers);
     if (FAILED(hr))
     {
-        DBGERR("Failed to find any graphics device instances\n");
+        IGC_ASSERT_MESSAGE(0, "Failed to find any graphics device instances");
         return 0;
     }
     return drivers.size();
@@ -381,12 +445,12 @@ static const char* ConvertType(const char* flagType)
     }
     return flagType;
 }
+
 #define DECLARE_IGC_REGKEY( dataType, regkeyName, defaultValue, descriptionText, releaseMode ) \
-    fprintf(fp, "    <Key name=\"%s\" type=\"%s\" location=\"%s\\%s\" description=\"%s\" />\n", \
-        #regkeyName,                                                                             \
-        ConvertType(#dataType),                                                                               \
-        "HKLM\\" IGC_REGISTRY_KEY,                                                              \
-        "",                                                                                     \
+    fprintf(fp, "    <Key name=\"%s\" type=\"%s\" location=\"%s\" description=\"%s\" />\n",    \
+        #regkeyName,                                                                           \
+        ConvertType(#dataType),                                                                \
+        "HKLM\\" IGC_REGISTRY_KEY,                                                             \
         descriptionText);
 #define DECLARE_IGC_GROUP( groupName ) \
     if(!firstGroup)                    \
@@ -439,18 +503,17 @@ void DumpIGCRegistryKeyDefinitions3(std::string driverRegistryPath, unsigned lon
 
     if (driverRegistryPath.empty())
     {
-        assert(!"Failed to find the driver registry path, cannot create the debug variable XML file.");
+        IGC_ASSERT_MESSAGE(0, "Failed to find the driver registry path, cannot create the debug variable XML file.");
         return;
     }
 
     std::string registryKeyPath = "HKLM\\SYSTEM\\ControlSet001\\Control\\Class\\" + driverRegistryPath + "\\IGC";
 
 #define DECLARE_IGC_REGKEY( dataType, regkeyName, defaultValue, descriptionText, releaseMode ) \
-    fprintf(fp, "    <Key name=\"%s\" type=\"%s\" location=\"%s\\%s\" description=\"%s\" />\n", \
-        #regkeyName,                                                                             \
-        ConvertType(#dataType),                                                                               \
-        registryKeyPath.c_str(),                                                              \
-        "",                                                                                     \
+    fprintf(fp, "    <Key name=\"%s\" type=\"%s\" location=\"%s\" description=\"%s\" />\n",    \
+        #regkeyName,                                                                           \
+        ConvertType(#dataType),                                                                \
+        registryKeyPath.c_str(),                                                               \
         descriptionText);
 
 #define DECLARE_IGC_GROUP( groupName ) \
@@ -835,6 +898,18 @@ void LoadRegistryKeys(const std::string& options, bool *RegFlagNameError)
             IGC_SET_FLAG_VALUE(DumpPatchTokens, true);
         }
 
+        if (IGC_IS_FLAG_ENABLED(DumpTimeStatsPerPass) ||
+            IGC_IS_FLAG_ENABLED(DumpTimeStatsCoarse))
+        {
+            IGC_SET_FLAG_VALUE(DumpTimeStats, true);
+            IGC::Debug::SetDebugFlag(IGC::Debug::DebugFlag::TIME_STATS_PER_SHADER, true);
+        }
+
+        if (IGC_IS_FLAG_ENABLED(DumpTimeStats))
+        {
+            // Need to turn on this setting so per-shader .csv is generated
+            IGC::Debug::SetDebugFlag(IGC::Debug::DebugFlag::TIME_STATS_PER_SHADER, true);
+        }
 
         switch(IGC_GET_FLAG_VALUE(ForceOCLSIMDWidth))
         {
