@@ -39,6 +39,7 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "Common_ISA.h"
 #include "visa_igc_common_header.h"
 #include "IsaDescription.h"
+#include "Mem_Manager.h"
 
 #define CISA_INVALID_ADDR_ID     -1
 #define CISA_INVALID_PRED_ID     -1
@@ -61,10 +62,10 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #endif
 
 struct attr_gen_struct {
-    char* name;
+    const char* name;
     bool  isInt;
     int   value;
-    char* string_val;
+    const char* string_val;
     bool  attr_set;
 };
 
@@ -78,17 +79,11 @@ typedef struct _VISA_StateOpndHandle : VISA_opnd {} VISA_StateOpndHandle;
 
 class VISAKernel;
 class VISAKernelImpl;
-namespace vISA
-{
-    class IR_Builder;
-}
+class CISA_IR_Builder;
 
 namespace CisaFramework
 {
 
-class  CisaInst;
-class  CisaBinary;
-
 class CisaInst
 {
 public:
@@ -143,15 +138,13 @@ public:
     VISA_Exec_Size getExecSize     () const { return (VISA_Exec_Size)
                                                            (m_cisa_instruction.execsize & 0xF); }
 
-    void *operator new(size_t sz, vISA::Mem_Manager& m){ return m.alloc(sz); }
+    void *operator new(size_t sz, vISA::Mem_Manager& m) {return m.alloc(sz); }
 
 private:
     char* m_inline_cisa;
 
     vISA::Mem_Manager &m_mem;
     short m_size;
-    friend class vISA::IR_Builder;
-    friend class CISA_IR_Builder;
 
     // during emition to determin what the labelID is.
     std::string m_label_name;
@@ -163,46 +156,16 @@ class CisaBinary
 {
 public:
 
-    CisaBinary(std::string filename, Options *options) :
-        m_mem(4096),
-        m_header_size(0),
-        m_total_size(0),
-        m_bytes_written_cisa_buffer(0),
-        m_header_buffer(NULL),
-        m_filename(filename),
-        m_instId(0),
-        m_options(options)
-    {
-        memset(&m_header, 0, sizeof(common_isa_header));
-    }
-
-    CisaBinary(Options *options) :
-        m_mem(4096),
-        m_filename(""),
-        m_instId(0),
-        m_options(options)
-    {
-        memset(&m_header, 0, sizeof(common_isa_header));
-
-        m_header.num_kernels = 0;
-        m_header.num_functions = 0;
-        m_upper_bound_kernels = 0;
-        m_upper_bound_functions = 0;
-
-        m_header_size = 0;
-        m_total_size = 0;
-        m_bytes_written_cisa_buffer = 0;
-        m_header_buffer = NULL;
-    }
+    CisaBinary(CISA_IR_Builder* builder);
 
     virtual ~CisaBinary() { }
 
     void initCisaBinary(int numberKernels, int numberFunctions)
     {
-        m_header.kernels = (kernel_info_t * ) m_mem.alloc(sizeof(kernel_info_t) * numberKernels);
+        m_header.kernels = (kernel_info_t *) m_mem.alloc(sizeof(kernel_info_t) * numberKernels);
         memset(m_header.kernels, 0, sizeof(kernel_info_t) * numberKernels);
 
-        m_header.functions = (function_info_t * ) m_mem.alloc(sizeof(function_info_t) * numberFunctions);
+        m_header.functions = (function_info_t *) m_mem.alloc(sizeof(function_info_t) * numberFunctions);
         memset(m_header.functions, 0, sizeof(function_info_t) * numberFunctions);
 
         m_upper_bound_kernels = numberKernels;
@@ -217,49 +180,37 @@ public:
         genxBinariesSize = 0;
     }
 
-    void setMagicNumber ( unsigned int  v ) { m_header.magic_number  = v; }
-    void setMajorVersion( unsigned char v ) { m_header.major_version = v; }
-    void setMinorVersion( unsigned char v ) { m_header.minor_version = v; }
+    void setMagicNumber (unsigned int  v) { m_header.magic_number  = v; }
+    void setMajorVersion(unsigned char v) { m_header.major_version = v; }
+    void setMinorVersion(unsigned char v) { m_header.minor_version = v; }
     unsigned char getMajorVersion () const { return m_header.major_version; }
     unsigned char getMinorVersion () const { return m_header.minor_version; }
     unsigned int  getMagicNumber  () const { return m_header.magic_number;  }
 
     const common_isa_header& getIsaHeader() const { return m_header; }
 
-    void initKernel( int kernelIndex, VISAKernelImpl * kernel );
+    void initKernel(int kernelIndex, VISAKernelImpl * kernel);
     int finalizeCisaBinary();
     int dumpToFile(std::string binFileName);
     int dumpToStream(std::ostream *os);
 
-    unsigned       getInstId() const { return m_instId; }
-    void     incrementInstId() const { m_instId++;      }
+    void *operator new(size_t sz, vISA::Mem_Manager& m) {return m.alloc(sz); }
 
-    void *operator new(size_t sz, vISA::Mem_Manager& m){ return m.alloc(sz); }
-
-    void isaDumpVerify(std::list<VISAKernelImpl *>, Options *options);
+    int isaDump(std::list<VISAKernelImpl *>, Options *options);
     void writeIsaAsmFile(std::string filename, std::string isaasmStr) const;
 
-    unsigned long getHeaderSize(){ return m_header_size; }
-    unsigned long getKernelVisaBinarySize(int i){ return m_header.kernels[i].size; }
-    unsigned long getFunctionVisaBinarySize(int i){ return m_header.functions[i].size; }
-    unsigned short getNumberKernels(){ return m_header.num_kernels; }
-    unsigned short getNumberFunctions(){ return m_header.num_functions; }
+    unsigned long getKernelVisaBinarySize(int i) {return m_header.kernels[i].size; }
+    unsigned long getFunctionVisaBinarySize(int i) {return m_header.functions[i].size; }
+    unsigned short getNumberKernels() {return m_header.num_kernels; }
+    unsigned short getNumberFunctions() {return m_header.num_functions; }
 
-    char * getVisaHeaderBuffer(){ return m_header_buffer; }
-    char * getKernelVisaBinaryBuffer(int i){ return m_header.kernels[i].cisa_binary_buffer; }
-    char * getFunctionVisaBinaryBuffer(int i){ return m_header.functions[i].cisa_binary_buffer; }
-
-    std::string getFilename() const { return m_filename; }
-    void setKernelVisaGenxBinaryBuffer(int i, void * buffer){ m_header.kernels[i].genx_binary_buffer = (char * )buffer; }
-    void setKernelVisaGenxBinarySize(int i, unsigned short size){ m_header.kernels[i].binary_size = size; }
-
-    void setFunctionsVisaGenxBinaryBuffer(int i, void * buffer){ m_header.functions[i].genx_binary_buffer = (char * )buffer; }
-    void setFunctionsVisaGenxBinarySize(int i, unsigned short size){ m_header.functions[i].binary_size = size; }
+    char * getVisaHeaderBuffer() {return m_header_buffer; }
 
     void patchKernel(int index, unsigned int genxBufferSize, void * buffer, int platform);
     void patchFunction(int index, unsigned genxBufferSize);
+    void patchFunctionWithGenBinary(int index, unsigned int genxBufferSize, char* buffer);
 
-    Options *getOptions(){ return m_options; }
+    Options *getOptions() {return m_options; }
 
 private:
     /*
@@ -277,16 +228,16 @@ private:
 
     common_isa_header m_header;
     vISA::Mem_Manager m_mem;
-    unsigned long m_header_size;
-    unsigned long m_total_size;
+    uint32_t m_header_size;
+    uint32_t m_total_size;
     unsigned long m_bytes_written_cisa_buffer;
     char * m_header_buffer;
     int m_upper_bound_kernels;
     int m_upper_bound_functions;
 
-    std::string m_filename;
-    mutable unsigned m_instId;
     Options *m_options;
+
+    CISA_IR_Builder* parent;
 };
 
 }
