@@ -34,16 +34,16 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 vISAPreDefinedSurface vISAPreDefSurf[COMMON_ISA_NUM_PREDEFINED_SURF_VER_3_1] =
 {
-    { 0, PREDEF_SURF_0, "T0" },
+    { 0, PREDEF_SURF_0, "%slm" },
     { 1, PREDEF_SURF_1, "T1" },
     { 2, PREDEF_SURF_2, "T2" },
     { 3, PREDEF_SURF_3, "TSS" },
-    { 4, PREDEF_SURF_252, "T252" },
-    { 5, PREDEF_SURF_255, "T255" },
+    { 4, PREDEF_SURF_252, "%bss" },
+    { 5, PREDEF_SURF_255, "%scratch" },
 };
 
 
-const char* Common_ISA_Get_Align_Name( VISA_Align align )
+const char* Common_ISA_Get_Align_Name(VISA_Align align)
 {
     static const char* common_ISA_align_name[] = {
         " ",
@@ -52,32 +52,36 @@ const char* Common_ISA_Get_Align_Name( VISA_Align align )
         "qword",
         "oword",
         "GRF",
-        "2GRF",
+        "GRFx2", // "2GRF"
+        "hword",
+        "wordx32",
+        "wordx64",
         "byte",
     };
 
     return common_ISA_align_name[align];
 }
 
+
 const char* Common_ISA_Get_Modifier_Name(VISA_Modifier modifier)
 {
-    switch(modifier)
+    switch (modifier)
     {
-        case MODIFIER_NONE    : return "";
-        case MODIFIER_ABS     : return "(abs)";
-        case MODIFIER_NEG     : return "(-)";
-        case MODIFIER_NEG_ABS : return "(-abs)";
-        case MODIFIER_SAT     : return "sat";
-        case MODIFIER_NOT     : return "(~)";
+        case MODIFIER_NONE:    return "";
+        case MODIFIER_ABS:     return "(abs)";
+        case MODIFIER_NEG:     return "(-)";
+        case MODIFIER_NEG_ABS: return "(-abs)";
+        case MODIFIER_SAT:     return "sat";
+        case MODIFIER_NOT:     return "(~)";
         default:
             MUST_BE_TRUE(false, "Invalid modifier.");
             return "invalid_modifier";
     }
 }
 
-short Common_ISA_Get_Region_Value( Common_ISA_Region_Val val )
+short Common_ISA_Get_Region_Value(Common_ISA_Region_Val val)
 {
-    switch( val ) {
+    switch (val) {
         case REGION_NULL:
             return -1;
         case REGION_0:
@@ -100,9 +104,9 @@ short Common_ISA_Get_Region_Value( Common_ISA_Region_Val val )
     }
 }
 
-G4_opcode GetGenOpcodeFromVISAOpcode( ISA_Opcode opcode )
+G4_opcode GetGenOpcodeFromVISAOpcode(ISA_Opcode opcode)
 {
-    switch(opcode)
+    switch (opcode)
     {
         case ISA_RESERVED_0:
             return G4_illegal;
@@ -245,9 +249,9 @@ G4_opcode GetGenOpcodeFromVISAOpcode( ISA_Opcode opcode )
     return G4_illegal;
 }
 
-G4_Type GetGenTypeFromVISAType( VISA_Type type )
+G4_Type GetGenTypeFromVISAType(VISA_Type type)
 {
-    switch(type)
+    switch (type)
     {
     case ISA_TYPE_UD:
         return Type_UD;
@@ -284,9 +288,9 @@ G4_Type GetGenTypeFromVISAType( VISA_Type type )
     }
 }
 
-VISA_Type Get_Common_ISA_Type_From_G4_Type( G4_Type type )
+VISA_Type Get_Common_ISA_Type_From_G4_Type(G4_Type type)
 {
-    switch(type)
+    switch (type)
     {
     case Type_UD:
         return ISA_TYPE_UD;
@@ -323,61 +327,9 @@ VISA_Type Get_Common_ISA_Type_From_G4_Type( G4_Type type )
     }
 }
 
-G4_SubReg_Align Get_G4_SubRegAlign_From_Type( G4_Type ty )
+G4_SrcModifier GetGenSrcModFromVISAMod(VISA_Modifier mod)
 {
-    switch(ty)
-    {
-    case Type_B:
-    case Type_UB:
-    case Type_W:
-    case Type_UW:
-        return Any;
-    case Type_UD:
-    case Type_D:
-    case Type_F:
-        return Even_Word;
-    case Type_DF:
-        return Four_Word;
-    case Type_V:
-    case Type_VF:
-    case Type_UV:
-        return Eight_Word;
-    case Type_Q:
-    case Type_UQ:
-        return Four_Word;
-    default:
-        return Any;
-    }
-}
-
-// size is the number of byte
-G4_SubReg_Align Get_G4_SubRegAlign_From_Size( uint16_t size )
-{
-    switch( size )
-    {
-    case 1:
-    case 2:
-        return Any;
-    case 4:
-        return Even_Word;
-    case 8:
-        if (getGenxPlatform() != GENX_BXT)
-          return Four_Word;
-        // FALL THROUGH
-        // WA: It's a workaround where a potential HW issue needs
-        // identifying.
-    case 16:
-        return Eight_Word;
-    case 32:
-        return Sixteen_Word;
-    default:
-        return GRFALIGN;
-    }
-}
-
-G4_SrcModifier GetGenSrcModFromVISAMod( VISA_Modifier mod )
-{
-    switch( mod )
+    switch (mod)
     {
     case MODIFIER_NONE:
         return Mod_src_undef;
@@ -395,9 +347,9 @@ G4_SrcModifier GetGenSrcModFromVISAMod( VISA_Modifier mod )
     }
 }
 
-G4_CondModifier Get_G4_CondModifier_From_Common_ISA_CondModifier( VISA_Cond_Mod cmod )
+G4_CondModifier Get_G4_CondModifier_From_Common_ISA_CondModifier(VISA_Cond_Mod cmod)
 {
-    switch(cmod){
+    switch (cmod) {
         case ISA_CMP_E:
             return Mod_e;
         case ISA_CMP_NE:
@@ -413,20 +365,22 @@ G4_CondModifier Get_G4_CondModifier_From_Common_ISA_CondModifier( VISA_Cond_Mod
         case ISA_CMP_UNDEF:
             return Mod_cond_undef;
         default:
-            MUST_BE_TRUE( 0, "Invalid CISA Conditional Modifier." );
+            MUST_BE_TRUE(0, "Invalid CISA Conditional Modifier.");
             return Mod_cond_undef;
     }
 }
 
 bool hasPredicate(ISA_Opcode op)
 {
-    switch(ISA_Inst_Table[op].type)
+    switch (ISA_Inst_Table[op].type)
     {
     case ISA_Inst_Mov:
         return !(op == ISA_SETP || op == ISA_MOVS || op == ISA_FMINMAX);
     case ISA_Inst_Arith:
     case ISA_Inst_Logic:
+    {
         return true;
+    }
     case ISA_Inst_Compare:
     case ISA_Inst_Address:
     case ISA_Inst_Data_Port:
@@ -452,7 +406,7 @@ bool hasPredicate(ISA_Opcode op)
 
 bool hasExecSize(ISA_Opcode op, uint8_t subOp)
 {
-    switch(ISA_Inst_Table[op].type)
+    switch (ISA_Inst_Table[op].type)
     {
         case ISA_Inst_Mov:
         case ISA_Inst_Arith:
@@ -461,9 +415,9 @@ bool hasExecSize(ISA_Opcode op, uint8_t subOp)
         case ISA_Inst_Address:
             return true;
         case ISA_Inst_Data_Port:
-            if( op == ISA_MEDIA_LD || op == ISA_MEDIA_ST ){
+            if (op == ISA_MEDIA_LD || op == ISA_MEDIA_ST) {
                 return false;
-            }else
+            } else
                 return true;
         case ISA_Inst_SVM:
             if (subOp == SVM_BLOCK_LD || subOp == SVM_BLOCK_ST || subOp == 0) {
@@ -472,7 +426,7 @@ bool hasExecSize(ISA_Opcode op, uint8_t subOp)
                 return true;
         case ISA_Inst_Sampler:
         case ISA_Inst_Misc:
-            if( op == ISA_RAW_SEND || op == ISA_RAW_SENDS || op == ISA_3D_SAMPLE ||
+            if (op == ISA_RAW_SEND || op == ISA_RAW_SENDS || op == ISA_3D_SAMPLE ||
                 op == ISA_3D_LOAD || op == ISA_3D_GATHER4 || op == ISA_3D_URB_WRITE ||
                 op == ISA_3D_INFO)
             {
@@ -481,7 +435,7 @@ bool hasExecSize(ISA_Opcode op, uint8_t subOp)
             else
                 return false;
         case ISA_Inst_Flow:
-            if( op == ISA_SUBROUTINE || op == ISA_LABEL || op == ISA_FADDR)
+            if (op == ISA_SUBROUTINE || op == ISA_LABEL || op == ISA_FADDR)
             {
                 return false;
             }
@@ -497,14 +451,14 @@ bool hasExecSize(ISA_Opcode op, uint8_t subOp)
 
 bool hasLabelSrc(ISA_Opcode op)
 {
-    if( ISA_Inst_Table[op].type == ISA_Inst_Flow)
+    if (ISA_Inst_Table[op].type == ISA_Inst_Flow)
     {
         if (op == ISA_RET || op == ISA_FRET || op == ISA_IFCALL || op == ISA_FADDR)
             return false;
-        else //( op == ISA_SUBROUTINE || op == ISA_LABEL || op == ISA_JMP || op == ISA_CALL || op == ISA_FCALL )
+        else //(op == ISA_SUBROUTINE || op == ISA_LABEL || op == ISA_JMP || op == ISA_CALL || op == ISA_FCALL)
             return true;
     }
-    else if( op == ISA_GOTO )
+    else if (op == ISA_GOTO)
         return true;
     else
         return false;
@@ -571,9 +525,9 @@ unsigned Get_Common_ISA_SVM_Block_Size(VISA_SVM_Block_Type size)
     return 0;
 }
 
-unsigned Get_VISA_Oword_Num( VISA_Oword_Num num )
+unsigned Get_VISA_Oword_Num(VISA_Oword_Num num)
 {
-    switch(num){
+    switch (num) {
         case OWORD_NUM_1:
             return 1;
         case OWORD_NUM_2:
@@ -585,14 +539,14 @@ unsigned Get_VISA_Oword_Num( VISA_Oword_Num num )
         case OWORD_NUM_16:
             return 16;
         default:
-            MUST_BE_TRUE( false, "illegal Oword number (should be 0..3)." );
+            MUST_BE_TRUE(false, "illegal Oword number (should be 0..3).");
             return 0;
     }
 }
 
-unsigned Get_VISA_Exec_Size( VISA_Exec_Size size )
+unsigned Get_VISA_Exec_Size(VISA_Exec_Size size)
 {
-    switch(size){
+    switch (size) {
         case EXEC_SIZE_1:
             return 1;
         case EXEC_SIZE_2:
@@ -606,14 +560,14 @@ unsigned Get_VISA_Exec_Size( VISA_Exec_Size size )
         case EXEC_SIZE_32:
             return 32;
         default:
-            MUST_BE_TRUE( false, "illegal common ISA execsize (should be 0..5)." );
+            MUST_BE_TRUE(false, "illegal common ISA execsize (should be 0..5).");
             return 0;
     }
 }
 
 bool IsMathInst(ISA_Opcode op)
 {
-    switch(op){
+    switch (op) {
         case ISA_INV:
         case ISA_DIV:
         case ISA_MOD:
@@ -632,7 +586,7 @@ bool IsMathInst(ISA_Opcode op)
 
 bool IsIntType(VISA_Type type)
 {
-    switch(type){
+    switch (type) {
         case ISA_TYPE_UD:
         case ISA_TYPE_D:
         case ISA_TYPE_UW:
@@ -680,7 +634,7 @@ bool IsUnsignedIntType(VISA_Type type)
 
 unsigned short Get_Common_ISA_Region_Value(Common_ISA_Region_Val val)
 {
-    switch( val ){
+    switch (val) {
     case REGION_0:
         return 0;
     case REGION_1:
@@ -700,24 +654,25 @@ unsigned short Get_Common_ISA_Region_Value(Common_ISA_Region_Val val)
     }
 }
 
-Common_ISA_Region_Val Get_CISA_Region_Val( short val )
+Common_ISA_Region_Val Get_CISA_Region_Val(short val)
 {
-    if( val == (short)0x8000 ){
+    if (val == (short)0x8000) {
         return REGION_NULL;
-    }else{
-        switch(val){
-            case 0: return REGION_0;
-            case 1: return REGION_1;
-            case 2: return REGION_2;
-            case 4: return REGION_4;
-            case 8: return REGION_8;
-            case 16: return REGION_16;
-            case 32: return REGION_32;
-            case -1:
-                return REGION_NULL;
-            default:
-                MUST_BE_TRUE( 0, "Invalid Region value." );
-                return REGION_NULL;
+    }
+    else {
+        switch (val) {
+        case 0: return REGION_0;
+        case 1: return REGION_1;
+        case 2: return REGION_2;
+        case 4: return REGION_4;
+        case 8: return REGION_8;
+        case 16: return REGION_16;
+        case 32: return REGION_32;
+        case -1:
+            return REGION_NULL;
+        default:
+            MUST_BE_TRUE(0, "Invalid Region value.");
+            return REGION_NULL;
         }
     }
 }
@@ -751,7 +706,7 @@ unsigned Round_Up_Pow2(unsigned n)
 
 G4_opcode Get_Pseudo_Opcode(ISA_Opcode op)
 {
-    switch( op ){
+    switch (op) {
         case ISA_AND:
             return G4_pseudo_and;
         case ISA_OR:
@@ -766,7 +721,7 @@ G4_opcode Get_Pseudo_Opcode(ISA_Opcode op)
     return G4_illegal;
 }
 
-VISA_EMask_Ctrl Get_Next_EMask(VISA_EMask_Ctrl currEMask, int execSize)
+VISA_EMask_Ctrl Get_Next_EMask(VISA_EMask_Ctrl currEMask, G4_ExecSize execSize)
 {
     switch (execSize) {
     default: // Next eMask is only valid for SIMD4, SIMD8, and SIMD16.
@@ -813,10 +768,10 @@ VISA_EMask_Ctrl Get_Next_EMask(VISA_EMask_Ctrl currEMask, int execSize)
     return vISA_NUM_EMASK;
 }
 
-unsigned int Get_Gen4_Emask( VISA_EMask_Ctrl cisa_emask, int exec_size )
+G4_InstOpts Get_Gen4_Emask(VISA_EMask_Ctrl cisa_emask, G4_ExecSize exec_size)
 {
 
-    switch( exec_size )
+    switch (exec_size.value)
     {
     case 32:
         switch (cisa_emask)
@@ -836,7 +791,7 @@ unsigned int Get_Gen4_Emask( VISA_EMask_Ctrl cisa_emask, int exec_size )
         break;
     case 16:
         {
-            switch( cisa_emask )
+            switch (cisa_emask)
             {
                 case vISA_EMASK_M1:
                     return InstOpt_M0;
@@ -847,14 +802,14 @@ unsigned int Get_Gen4_Emask( VISA_EMask_Ctrl cisa_emask, int exec_size )
                 case vISA_EMASK_M5_NM:
                     return InstOpt_M16 | InstOpt_WriteEnable;
                 default:
-                    MUST_BE_TRUE( false, "Invalid emask for SIMD16 inst" );
+                    MUST_BE_TRUE(false, "Invalid emask for SIMD16 inst");
                     return InstOpt_NoOpt;
             }
         }
         break;
     case 8:
         {
-            switch( cisa_emask )
+            switch (cisa_emask)
             {
                 case vISA_EMASK_M1:
                     return InstOpt_M0;
@@ -873,7 +828,7 @@ unsigned int Get_Gen4_Emask( VISA_EMask_Ctrl cisa_emask, int exec_size )
                 case vISA_EMASK_M7_NM:
                     return InstOpt_M24 | InstOpt_WriteEnable;
                 default:
-                    MUST_BE_TRUE( false, "Invalid emask for SIMD8 inst" );
+                    MUST_BE_TRUE(false, "Invalid emask for SIMD8 inst");
                     return InstOpt_NoOpt;
             }
         }
@@ -915,18 +870,13 @@ unsigned int Get_Gen4_Emask( VISA_EMask_Ctrl cisa_emask, int exec_size )
                 case vISA_EMASK_M8_NM:
                     return InstOpt_M28 | InstOpt_WriteEnable;
                 default:
-                    MUST_BE_TRUE( false, "Invalid emask for SIMD4 inst." );
+                    MUST_BE_TRUE(false, "Invalid emask for SIMD4 inst.");
                     return InstOpt_NoOpt;
             }
         }
     }
 }
 
-Common_ISA_Operand_Class CISA_Opnd_Class( vector_opnd opnd )
-{
-    return (Common_ISA_Operand_Class)(opnd.tag & 0x7);
-}
-
 unsigned Get_Atomic_Op(VISAAtomicOps op) {
 
     switch (op) {
@@ -956,7 +906,7 @@ unsigned Get_Atomic_Op(VISAAtomicOps op) {
 
 uint16_t Get_VISA_Type_Size(VISA_Type type)
 {
-    switch(type)
+    switch (type)
     {
     case ISA_TYPE_UD:
     case ISA_TYPE_D:
@@ -978,30 +928,30 @@ uint16_t Get_VISA_Type_Size(VISA_Type type)
     case ISA_TYPE_UQ:
         return 8;
     default:
-        MUST_BE_TRUE( 0, "Invalid data type: size unknown." );
+        MUST_BE_TRUE(0, "Invalid data type: size unknown.");
         return 0;
 
 
     }
 }
 
-int Get_Size_Attribute_Info(attribute_info_t * attr)
+int attribute_info_t::getSizeInBinary() const
 {
-    return sizeof(attr->nameIndex) + sizeof(attr->size) + attr->size;
+    return sizeof(nameIndex) + sizeof(size) + size;
 }
 
-int Get_Size_Label_Info(label_info_t * lbl)
+int label_info_t::getSizeInBinary() const
 {
-    int size = sizeof(lbl->name_index) + sizeof(lbl->kind) + sizeof(lbl->attribute_count);
+    int size = sizeof(name_index) + sizeof(kind) + sizeof(attribute_count);
 
-    for(int i = 0; i< lbl->attribute_count; i++)
+    for (int i = 0; i < attribute_count; i++)
     {
-        size += Get_Size_Attribute_Info(&lbl->attributes[i]);
+        size += attributes[i].getSizeInBinary();
     }
     return size;
 }
 
-int Get_Size_Var_Info_CISA3(var_info_t * t)
+int var_info_t::getSizeInBinary() const
 {
     /*
         var_info {
@@ -1014,17 +964,17 @@ int Get_Size_Var_Info_CISA3(var_info_t * t)
     attribute_info[attribute_count];
     }
     */
-    int size = sizeof(t->name_index) + sizeof(t->bit_properties) + sizeof(t->num_elements) +
-        sizeof(t->alias_index) + sizeof(t->alias_offset) + sizeof(t->alias_scope_specifier) +sizeof(t->attribute_count);
+    int size = sizeof(name_index) + sizeof(bit_properties) + sizeof(num_elements) +
+        sizeof(alias_index) + sizeof(alias_offset) + sizeof(alias_scope_specifier) + sizeof(attribute_count);
 
-    for(int i = 0; i< t->attribute_count; i++)
+    for (int i = 0; i < attribute_count; i++)
     {
-        size += Get_Size_Attribute_Info(&t->attributes[i]);
+        size += attributes[i].getSizeInBinary();
     }
     return size;
 }
 
-int Get_Size_Addr_Info(addr_info_t * addr)
+int addr_info_t::getSizeInBinary() const
 {
     /*
     address_info {
@@ -1034,15 +984,15 @@ int Get_Size_Addr_Info(addr_info_t * addr)
     attribute_info[attribute_count];
     }
     */
-    int size = sizeof(addr->name_index) + sizeof(addr->num_elements) + sizeof(addr->attribute_count);
-    for(int i = 0; i< addr->attribute_count; i++)
+    int size = sizeof(name_index) + sizeof(num_elements) + sizeof(attribute_count);
+    for (int i = 0; i < attribute_count; i++)
     {
-        size += Get_Size_Attribute_Info(&addr->attributes[i]);
+        size += attributes[i].getSizeInBinary();
     }
     return size;
 }
 
-int Get_Size_Pred_Info(pred_info_t * pred)
+int pred_info_t::getSizeInBinary() const
 {
     /*
     predicate_info {
@@ -1052,15 +1002,15 @@ int Get_Size_Pred_Info(pred_info_t * pred)
     attribute_info[attribute_count];
     }
     */
-    int size = sizeof(pred->name_index) + sizeof(pred->num_elements) + sizeof(pred->attribute_count);
-    for(int i = 0; i< pred->attribute_count; i++)
+    int size = sizeof(name_index) + sizeof(num_elements) + sizeof(attribute_count);
+    for (int i = 0; i < attribute_count; i++)
     {
-        size += Get_Size_Attribute_Info(&pred->attributes[i]);
+        size += attributes[i].getSizeInBinary();
     }
     return size;
 }
 
-int Get_Size_Input_Info(input_info_t * input)
+int input_info_t::getSizeInBinary() const
 {
     /*
     input_info {
@@ -1070,42 +1020,42 @@ int Get_Size_Input_Info(input_info_t * input)
     uw size;
     }
     */
-    return sizeof(input->kind) + sizeof(input->index) + sizeof(input->offset) + sizeof(input->size);
+    return sizeof(kind) + sizeof(index) + sizeof(offset) + sizeof(size);
 }
 
-int Get_Size_Vector_Operand(vector_opnd * cisa_opnd)
+int vector_opnd::getSizeInBinary() const
 {
     int size = 0;
 
-    switch(cisa_opnd->tag & 0x7)
+    switch (tag & 0x7)
     {
     case OPERAND_GENERAL:
         {
-            size = sizeof(cisa_opnd->opnd_val.gen_opnd.index) + sizeof(cisa_opnd->opnd_val.gen_opnd.col_offset) +
-                sizeof(cisa_opnd->opnd_val.gen_opnd.row_offset) + sizeof(cisa_opnd->opnd_val.gen_opnd.region);
+            size = sizeof(opnd_val.gen_opnd.index) + sizeof(opnd_val.gen_opnd.col_offset) +
+                sizeof(opnd_val.gen_opnd.row_offset) + sizeof(opnd_val.gen_opnd.region);
             break;
         }
     case OPERAND_ADDRESS:
         {
-            size = sizeof(cisa_opnd->opnd_val.addr_opnd.index) + sizeof(cisa_opnd->opnd_val.addr_opnd.offset) +
-                sizeof(cisa_opnd->opnd_val.addr_opnd.width);
+            size = sizeof(opnd_val.addr_opnd.index) + sizeof(opnd_val.addr_opnd.offset) +
+                sizeof(opnd_val.addr_opnd.width);
             break;
         }
     case OPERAND_INDIRECT:
         {
-            size = sizeof(cisa_opnd->opnd_val.indirect_opnd.index) + sizeof(cisa_opnd->opnd_val.indirect_opnd.addr_offset) +
-                sizeof(cisa_opnd->opnd_val.indirect_opnd.indirect_offset) + sizeof(cisa_opnd->opnd_val.indirect_opnd.bit_property) +
-                sizeof(cisa_opnd->opnd_val.indirect_opnd.region);
+            size = sizeof(opnd_val.indirect_opnd.index) + sizeof(opnd_val.indirect_opnd.addr_offset) +
+                sizeof(opnd_val.indirect_opnd.indirect_offset) + sizeof(opnd_val.indirect_opnd.bit_property) +
+                sizeof(opnd_val.indirect_opnd.region);
             break;
         }
     case OPERAND_PREDICATE:
         {
-            size = sizeof(cisa_opnd->opnd_val.pred_opnd.index);
+            size = sizeof(opnd_val.pred_opnd.index);
             break;
         }
     case OPERAND_IMMEDIATE:
         {
-            switch (cisa_opnd->opnd_val.const_opnd.type) {
+            switch (opnd_val.const_opnd.type) {
             default:
                 size = sizeof(unsigned int);
                 break;
@@ -1116,30 +1066,28 @@ int Get_Size_Vector_Operand(vector_opnd * cisa_opnd)
                 break;
             }
 
-            size += sizeof(cisa_opnd->opnd_val.const_opnd.type);
+            size += sizeof(opnd_val.const_opnd.type);
             break;
         }
     case OPERAND_STATE:
         {
-            size = sizeof(cisa_opnd->opnd_val.state_opnd.index) + sizeof(cisa_opnd->opnd_val.state_opnd.offset) +
-                sizeof(cisa_opnd->opnd_val.state_opnd.opnd_class);
+            size = sizeof(opnd_val.state_opnd.index) + sizeof(opnd_val.state_opnd.offset) +
+                sizeof(opnd_val.state_opnd.opnd_class);
             break;
         }
     default:
         {
-            MUST_BE_TRUE( 0, "Invalid Vector Operand Class. Size cannot be determined." );
+            MUST_BE_TRUE(0, "Invalid Vector Operand Class. Size cannot be determined.");
             break;
         }
     }
 
-    size += sizeof(cisa_opnd->tag);
+    size += sizeof(tag);
 
     return size;
 }
 
-
 /*
-cisa3.0
 function_info {
     ub linkage; // MBZ
     ub name_len;
@@ -1150,17 +1098,17 @@ function_info {
     uw num_syms_function; // MBZ
 }
 */
-//for cisa 3.0
-unsigned long getSizeFunctionInfo(kernel_info_t * kernel_info)
+uint32_t function_info_t::getSizeInBinary() const
 {
-    unsigned int size = sizeof(kernel_info->linkage) + sizeof(kernel_info->name_len) +
-        kernel_info->name_len + sizeof(kernel_info->offset) + sizeof(kernel_info->size);
+    uint32_t size = sizeof(linkage) + sizeof(name_len) +
+        name_len + sizeof(offset) + sizeof(this->size);
 
-    size += sizeof(kernel_info->variable_reloc_symtab.num_syms);
-    size += sizeof(kernel_info->function_reloc_symtab.num_syms);
+    size += sizeof(variable_reloc_symtab.num_syms);
+    size += sizeof(function_reloc_symtab.num_syms);
 
     return size;
 }
+
 /*
     kernel_info {
     ub name_len;
@@ -1174,35 +1122,35 @@ unsigned long getSizeFunctionInfo(kernel_info_t * kernel_info)
     gen_binary_info gen_binaries[num_gen_binaries];
 }
 */
-unsigned long get_Size_Kernel_Info(kernel_info_t * kernel_info, int major_version, int minor_version)
+uint32_t kernel_info_t::getSizeInBinary() const
 {
-    unsigned long size = sizeof(kernel_info->name_len) + kernel_info->name_len
-        + sizeof(kernel_info->offset) + sizeof(kernel_info->size)
-        + sizeof(kernel_info->input_offset);
+    uint32_t size = sizeof(name_len) + name_len
+        + sizeof(offset) + sizeof(this->size)
+        + sizeof(input_offset);
 
-    size += sizeof(kernel_info->variable_reloc_symtab.num_syms);
-    size += sizeof(kernel_info->function_reloc_symtab.num_syms);
+    size += sizeof(variable_reloc_symtab.num_syms);
+    size += sizeof(function_reloc_symtab.num_syms);
 
-    size += sizeof(kernel_info->num_gen_binaries);
+    size += sizeof(num_gen_binaries);
 
-    for (int i = 0; i < kernel_info->num_gen_binaries; i++)
+    for (int i = 0; i < num_gen_binaries; i++)
     {
-        size += sizeof(kernel_info->gen_binaries->platform);
-        size += sizeof(kernel_info->gen_binaries->binary_offset);
-        size += sizeof(kernel_info->gen_binaries->binary_size);
+        size += sizeof(gen_binaries->platform);
+        size += sizeof(gen_binaries->binary_offset);
+        size += sizeof(gen_binaries->binary_size);
     }
 
     return size;
 }
 
-unsigned long get_Size_Isa_Header( common_isa_header * m_header, int major_version, int minor_version )
+uint32_t common_isa_header::getSizeInBinary() const
 {
-    unsigned long size = sizeof(m_header->magic_number) + sizeof(m_header->major_version)
-        + sizeof(m_header->minor_version) + sizeof(m_header->num_kernels);
+    uint32_t size = sizeof(magic_number) + sizeof(major_version)
+        + sizeof(minor_version) + sizeof(num_kernels);
 
-    for(int i = 0; i < m_header->num_kernels; i++)
+    for (int i = 0; i < num_kernels; i++)
     {
-        size += get_Size_Kernel_Info(&m_header->kernels[i], major_version, minor_version);
+        size += kernels[i].getSizeInBinary();
     }
     /*
     common_isa_header {
@@ -1222,19 +1170,19 @@ unsigned long get_Size_Isa_Header( common_isa_header * m_header, int major_versi
     // file-scope variables are no longer supported
     size += sizeof(uint16_t);
 
-    size += sizeof(m_header->num_functions);
+    size += sizeof(num_functions);
 
-    for (int i = 0; i < m_header->num_functions; i++)
+    for (int i = 0; i < num_functions; i++)
     {
-        size += getSizeFunctionInfo(&m_header->functions[i]);
+        size += functions[i].getSizeInBinary();
     }
 
     return size;
 }
 
-VISA_Cond_Mod Get_Common_ISA_CondModifier_From_G4_CondModifier(G4_CondModifier  cmod )
+VISA_Cond_Mod Get_Common_ISA_CondModifier_From_G4_CondModifier(G4_CondModifier  cmod)
 {
-    switch(cmod){
+    switch (cmod) {
         //case ISA_CMP_NONE:
         //    return Mod_z;
         case Mod_e:
@@ -1258,14 +1206,14 @@ VISA_Cond_Mod Get_Common_ISA_CondModifier_From_G4_CondModifier(G4_CondModifier
         case Mod_cond_undef:
             return ISA_CMP_UNDEF;
         default:
-            MUST_BE_TRUE( 0, "Invalid G4 Conditional Modifier." );
+            MUST_BE_TRUE(0, "Invalid G4 Conditional Modifier.");
             return ISA_CMP_UNDEF;
     }
 }
 
-VISA_Exec_Size Get_VISA_Exec_Size_From_Raw_Size( unsigned int size )
+VISA_Exec_Size Get_VISA_Exec_Size_From_Raw_Size(unsigned int size)
 {
-    switch(size){
+    switch (size) {
         case 1:
             return EXEC_SIZE_1;
         case 2:
@@ -1279,24 +1227,25 @@ VISA_Exec_Size Get_VISA_Exec_Size_From_Raw_Size( unsigned int size )
         case 32:
             return EXEC_SIZE_32;
         default:
-            MUST_BE_TRUE( false, "illegal common ISA execsize (should be 1, 2, 4, 8, 16, 32)." );
+            MUST_BE_TRUE(false, "illegal common ISA execsize (should be 1, 2, 4, 8, 16, 32).");
             return EXEC_SIZE_ILLEGAL;
     }
 }
-int Get_Size_State_Info(state_info_t * t)
-{
-    int size = sizeof(t->name_index) + sizeof(t->num_elements) + sizeof(t->attribute_count);
 
-    for(int i = 0; i< t->attribute_count; i++)
+int state_info_t::getSizeInBinary() const
+{
+    int size = sizeof(name_index) + sizeof(num_elements) + sizeof(attribute_count);
+
+    for (int i = 0; i < attribute_count; i++)
     {
-        size += Get_Size_Attribute_Info(&t->attributes[i]);
+        size += attributes[i].getSizeInBinary();
     }
     return size;
 }
 
-VISA_Oword_Num Get_VISA_Oword_Num_From_Number( unsigned num )
+VISA_Oword_Num Get_VISA_Oword_Num_From_Number(unsigned num)
 {
-    switch(num){
+    switch (num) {
         case 1:
             return OWORD_NUM_1;
         case 2:
@@ -1308,14 +1257,14 @@ VISA_Oword_Num Get_VISA_Oword_Num_From_Number( unsigned num )
         case 16:
             return OWORD_NUM_16;
         default:
-            MUST_BE_TRUE( false, "illegal Oword number." );
+            MUST_BE_TRUE(false, "illegal Oword number.");
             return OWORD_NUM_ILLEGAL;
     }
 }
 
-VISA_Modifier Get_Common_ISA_SrcMod_From_G4_Mod(G4_SrcModifier mod )
+VISA_Modifier Get_Common_ISA_SrcMod_From_G4_Mod(G4_SrcModifier mod)
 {
-    switch( mod )
+    switch (mod)
     {
     case Mod_src_undef:
         return MODIFIER_NONE ;
@@ -1334,7 +1283,7 @@ VISA_Modifier Get_Common_ISA_SrcMod_From_G4_Mod(G4_SrcModifier mod )
 }
 
 
-VISA_Type getVectorOperandType(const common_isa_header& isaHeader, const print_format_provider_t* header, const vector_opnd& opnd)
+VISA_Type getVectorOperandType(const print_format_provider_t* header, const vector_opnd& opnd)
 {
     unsigned numPreDefinedVars = Get_CISA_PreDefined_Var_Count();
     switch (opnd.getOperandClass())
@@ -1467,18 +1416,21 @@ int Get_PreDefined_Surf_Index(int index)
 
 const char* createStringCopy(const char* name, vISA::Mem_Manager &m_mem)
 {
-    if (strlen(name) == 0)
-    {
+    if (*name == '\0') {
         return "";
     }
-    size_t size = strlen(name) + 1;
-    if (size > 255)
+
+    // TODO: look into relaxing this
+    static const size_t MAX_VISA_BINARY_STRING_LENGTH = 256;
+
+    size_t copyLen = strlen(name);
+    if (copyLen >= MAX_VISA_BINARY_STRING_LENGTH)
     {
-        size = 255;
+        copyLen = MAX_VISA_BINARY_STRING_LENGTH - 1;
     }
-    char* str = (char*) m_mem.alloc(size);
-    strncpy_s(str, size, name, size);
-    return str;
+    char* copy = (char*)m_mem.alloc(copyLen + 1);
+    strncpy_s(copy, copyLen + 1, name, copyLen);
+    return copy;
 }
 
 std::string sanitizeLabelString(std::string str)
