@@ -32,15 +32,8 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "Common_BinaryEncoding.h"
 #include "RegAlloc.h"
 #include "GraphColor.h"
+#include <unordered_map>
 
-// G4IR instructions added by JIT that do not result from lowering
-// any CISA bytecode will be assigned CISA offset = 0xffffffff.
-// This includes pseudo nodes, G4_labels, mov introduced for copying
-// r0 for pre-emption support.
-constexpr int UNMAPPABLE_VISA_INDEX = IR_Builder::OrphanVISAIndex;
-
-#define CALLER_SAVE_START 1
-#define CALLEE_SAVE_START 60
 namespace vISA
 {
 class G4_Declare;
@@ -164,10 +157,10 @@ private:
     G4_Declare* fretVar;
 
     // Caller save/restore
-    // std::vector<std::pair<fcall inst, std::pair<first caller save, last caller restore>>>
+    // std::vector<std::pair<fcall inst BB, std::pair<first caller save, last caller restore>>>
     // One entry per fcall inst in current compilation unit
     typedef std::pair<std::vector<G4_INST*>, std::vector<G4_INST*> > SaveRestore;
-    std::map<G4_INST*, SaveRestore> callerSaveRestore;
+    std::unordered_map<G4_BB*, SaveRestore> callerSaveRestore;
     SaveRestore calleeSaveRestore;
 
     INST_LIST oldInsts;
@@ -255,16 +248,15 @@ public:
     G4_Declare* getFretVar() { return fretVar; }
     void setFretVar(G4_Declare* dcl) { fretVar = dcl; }
 
-    void addCallerSaveInst(G4_INST* fcall, G4_INST* inst);
-    void addCallerRestoreInst(G4_INST* fcall, G4_INST* inst);
+    void updateExpandedIntrinsic(G4_InstIntrinsic* spillOrFill, G4_INST* inst);
+    void addCallerSaveInst(G4_BB* fcallBB, G4_INST* inst);
+    void addCallerRestoreInst(G4_BB* fcallBB, G4_INST* inst);
     void addCalleeSaveInst(G4_INST* inst);
     void addCalleeRestoreInst(G4_INST* inst);
-    bool isFcallWithSaveRestore(G4_INST* inst);
-    void setFCallInst(G4_INST* fcall);
-    bool isFCallInst(G4_INST* inst);
+    bool isFcallWithSaveRestore(G4_BB* bb);
 
-    std::vector<G4_INST*>& getCallerSaveInsts(G4_INST* fcall);
-    std::vector<G4_INST*>& getCallerRestoreInsts(G4_INST* fcall);
+    std::vector<G4_INST*>& getCallerSaveInsts(G4_BB* fcallBB);
+    std::vector<G4_INST*>& getCallerRestoreInsts(G4_BB* fcallBB);
     std::vector<G4_INST*>& getCalleeSaveInsts();
     std::vector<G4_INST*>& getCalleeRestoreInsts();
 
@@ -285,6 +277,7 @@ public:
     void generateCISAByteOffsetFromOffset();
     void updateRelocOffset();
     void updateCallStackLiveIntervals();
+    void updateCallStackMain();
     void generateGenISAToVISAIndex();
 
     void computeDebugInfo(std::list<G4_BB*>& stackCallEntryBBs);
@@ -296,6 +289,16 @@ public:
         mapCISAOffset.insert(std::make_pair(a, b));
     }
 
+    unsigned int getGenOffsetFromVISAIndex(unsigned int v)
+    {
+        for (auto& item : mapCISAIndexGenOffset)
+        {
+            if (item.first == v)
+                return item.second;
+        }
+        return 0;
+    }
+
     std::vector<std::pair<unsigned int, unsigned int>>& getMapCISAOffsetGenOffset() { return mapCISAOffsetGenOffset; }
     std::vector<std::pair<unsigned int, unsigned int>>& getMapCISAIndexGenOffset() { return mapCISAIndexGenOffset; }
     std::vector<std::pair<unsigned int, unsigned int>>& getMapGenISAOffsetToCISAIndex() { return genISAOffsetToVISAIndex; }
@@ -385,7 +388,7 @@ public:
         absOffset = false;
     }
 
-    std::vector<SaveRestoreInfo>& getSRInfo(){ return srInfo; }
+    std::vector<SaveRestoreInfo>& getSRInfo() { return srInfo; }
 
     void sieveInstructions(CallerOrCallee c);
 
