@@ -83,6 +83,7 @@ class LiveRange
     G4_RegFileKind regKind;
     bool* forbidden = nullptr;
     int numForbidden = -1;
+    bool spilled = false;
 
     GlobalRA& gra;
     unsigned numRegNeeded;
@@ -116,7 +117,7 @@ public:
 
     LiveRange(G4_RegVar* v, GlobalRA&);
 
-    void* operator new(size_t sz, vISA::Mem_Manager& m){ return m.alloc(sz); }
+    void* operator new(size_t sz, vISA::Mem_Manager& m) { return m.alloc(sz); }
 
     void setDegree(unsigned d)  {degree = d;}
     unsigned getDegree()        {return degree;}
@@ -132,7 +133,7 @@ public:
     void setActive(bool v) {active = v;}
     bool getActive() {return active;}
 
-    virtual void emit(std::ostream& output, bool symbolreg=false)
+    virtual void emit(std::ostream& output, bool symbolreg = false)
     {
         output << getVar ()->getDeclare ()->getName();
         if (reg.phyReg != NULL)
@@ -245,6 +246,9 @@ public:
         void setRetIp() { retIp = true; }
         bool isRetIp() const { return retIp; }
 
+        bool isSpilled() const { return spilled; }
+        void setSpilled(bool v) { spilled = v; }
+
 private:
     //const Options *m_options;
     unsigned getForbiddenVectorSize();
@@ -357,7 +361,7 @@ namespace vISA
         // like dense matrix, interference is not symmetric (that is, if v1 and v2 interfere and v1 < v2,
         // we insert (v1, v2) but not (v2, v1)) for better cache behavior
         std::vector<std::unordered_set<uint32_t> > sparseMatrix;
-        const uint32_t denseMatrixLimit = 65536;
+        const uint32_t denseMatrixLimit = 0x20000;
 
         void updateLiveness(BitSet& live, uint32_t id, bool val)
         {
@@ -514,6 +518,8 @@ namespace vISA
         bool dumpIntf(const char*) const;
         void interferenceVerificationForSplit() const;
 
+        bool linearScanVerify();
+
         void buildInterferenceWithLocalRA(G4_BB* bb);
 
         void buildInterferenceAmongLiveIns();
@@ -524,6 +530,20 @@ namespace vISA
         bool isStrongEdgeBetween(G4_Declare*, G4_Declare*);
     };
 
+    // Class to compute reg chart dump and dump it to ostream.
+    // Used only when -dumpregchart is passed.
+    class RegChartDump
+    {
+    public:
+        GlobalRA& gra;
+        std::vector<G4_Declare*> sortedLiveIntervals;
+        std::unordered_map<G4_Declare*, std::pair<G4_INST*, G4_INST*>> startEnd;
+        void recordLiveIntervals(std::vector<G4_Declare*>& dcls);
+        void dumpRegChart(std::ostream&, LiveRange** lrs = nullptr, unsigned int numLRs = 0);
+
+        RegChartDump(GlobalRA& g) : gra(g) {}
+    };
+
     class GraphColor
     {
         GlobalRA& gra;
@@ -569,7 +589,7 @@ namespace vISA
         void removeConstrained();
         void relaxNeighborDegreeGRF(LiveRange* lr);
         void relaxNeighborDegreeARF(LiveRange* lr);
-        bool assignColors(ColorHeuristic heuristicGRF, bool doBankConflict, bool highInternalConflict);
+        bool assignColors(ColorHeuristic heuristicGRF, bool doBankConflict, bool highInternalConflict, bool honorHints = true);
 
         void clearSpillAddrLocSignature()
         {
@@ -607,22 +627,25 @@ namespace vISA
         void addCalleeStackSetupCode();
         void saveRegs(
             unsigned startReg, unsigned owordSize, G4_Declare* scratchRegDcl, G4_Declare* framePtr,
-            unsigned frameOffset, G4_BB* bb, INST_LIST_ITER insertIt);
+            unsigned frameOffset, G4_BB* bb, INST_LIST_ITER insertIt, std::unordered_set<G4_INST*>& group);
         void saveActiveRegs(
             std::vector<bool>& saveRegs, unsigned startReg,
-            unsigned frameOffset, G4_BB* bb, INST_LIST_ITER insertIt);
+            unsigned frameOffset, G4_BB* bb, INST_LIST_ITER insertIt, std::unordered_set<G4_INST*>& group);
         void restoreRegs(
             unsigned startReg, unsigned owordSize, G4_Declare* scratchRegDcl, G4_Declare* framePtr,
-            unsigned frameOffset, G4_BB* bb, INST_LIST_ITER insertIt);
+            unsigned frameOffset, G4_BB* bb, INST_LIST_ITER insertIt, std::unordered_set<G4_INST*>& group);
         void restoreActiveRegs(
             std::vector<bool>& restoreRegs, unsigned startReg,
-            unsigned frameOffset, G4_BB* bb, INST_LIST_ITER insertIt);
+            unsigned frameOffset, G4_BB* bb, INST_LIST_ITER insertIt, std::unordered_set<G4_INST*>& group);
         void OptimizeActiveRegsFootprint(std::vector<bool>& saveRegs);
         void OptimizeActiveRegsFootprint(std::vector<bool>& saveRegs, std::vector<bool>& retRegs);
         void dumpRegisterPressure();
         GlobalRA & getGRA() { return gra; }
         G4_SrcRegRegion* getScratchSurface() const;
         void stackCallProlog();
+        LiveRange** getLRs() {
+            return lrs;
+        }
     };
 
     class RAVarInfo
@@ -632,6 +655,7 @@ namespace vISA
         unsigned int bb_id = UINT_MAX;      // block local variable's block id.
         G4_Declare* splittedDCL = nullptr;
         LocalLiveRange* localLR = nullptr;
+        LSLiveRange* LSLR = nullptr;
         unsigned int numRefs = 0;
         BankConflict conflict = BANK_CONFLICT_NONE;      // used to indicate bank that should be assigned to dcl if possible
         G4_INST* startInterval = nullptr;
@@ -704,6 +728,7 @@ namespace vISA
     {
     public:
         VerifyAugmentation *verifyAugmentation = nullptr;
+        RegChartDump* regChart = nullptr;
         static bool useGenericAugAlign()
         {
             auto gen = getPlatformGeneration(getGenxPlatform());
@@ -720,7 +745,6 @@ namespace vISA
         static bool owordAligned(unsigned offset);
         template <class REGION_TYPE> bool isUnalignedRegion(REGION_TYPE * region, unsigned execSize);
         bool shouldPreloadDst(G4_INST* instContext, G4_BB* curBB);
-        static unsigned sendBlockSizeCode(unsigned owordSize);
         void updateDefSet(std::set<G4_Declare*>& defs, G4_Declare* referencedDcl);
         void detectUndefinedUses(LivenessAnalysis& liveAnalysis, G4_Kernel& kernel);
         void markBlockLocalVar(G4_RegVar* var, unsigned bbId);
@@ -729,11 +753,13 @@ namespace vISA
         void fixAlignment();
         void expandSpillIntrinsic(G4_BB*);
         void expandFillIntrinsic(G4_BB*);
-        void expandSpillFillIntrinsics();
+        void expandSpillFillIntrinsics(unsigned int);
 
         RAVarInfo defaultValues;
         std::vector<RAVarInfo> vars;
 
+        std::vector<G4_Declare *> UndeclaredVars;
+
         // fake declares for each GRF reg, used by HRA
         // note only GRFs that are used by LRA get a declare
         std::vector<G4_Declare*> GRFDclsForHRA;
@@ -759,27 +785,42 @@ namespace vISA
         // created in addStoreRestoreForFP
         G4_Declare* oldFPDcl = nullptr;
 
+        // instruction to save/restore vISA FP, only present in functions
+        G4_INST* saveBE_FPInst = nullptr;
+        G4_INST* restoreBE_FPInst = nullptr;
+
         // new temps for each reference of spilled address/flag decls
         std::unordered_set<G4_Declare*> addrFlagSpillDcls;
 
         // store iteration number for GRA loop
         unsigned int iterNo = 0;
 
+        uint32_t numGRFSpill = 0;
+        uint32_t numGRFFill = 0;
+
         void expandFillNonStackcall(uint32_t& numRows, uint32_t& offset, short& rowOffset, G4_SrcRegRegion* header, G4_DstRegRegion* resultRgn, G4_BB* bb, INST_LIST_ITER& instIt);
         void expandSpillNonStackcall(uint32_t& numRows, uint32_t& offset, short& rowOffset, G4_SrcRegRegion* header, G4_SrcRegRegion* payload, G4_BB* bb, INST_LIST_ITER& instIt);
         void expandFillStackcall(uint32_t& numRows, uint32_t& offset, short& rowOffset, G4_SrcRegRegion* header, G4_DstRegRegion* resultRgn, G4_BB* bb, INST_LIST_ITER& instIt);
         void expandSpillStackcall(uint32_t& numRows, uint32_t& offset, short& rowOffset, G4_SrcRegRegion* payload, G4_BB* bb, INST_LIST_ITER& instIt);
 
+    public:
+        static unsigned sendBlockSizeCode(unsigned owordSize);
+
+        // For current program, store caller/callee save/restore instructions
+        std::unordered_set<G4_INST*> calleeSaveInsts;
+        std::unordered_set<G4_INST*> calleeRestoreInsts;
+        std::unordered_map<G4_INST*, std::unordered_set<G4_INST*>> callerSaveInsts;
+        std::unordered_map<G4_INST*, std::unordered_set<G4_INST*>> callerRestoreInsts;
+
     public:
         G4_Kernel& kernel;
         IR_Builder& builder;
         PhyRegPool& regPool;
         PointsToAnalysis& pointsToAnalysis;
         FCALL_RET_MAP fcallRetMap;
-        VarSplitPass* splitPass = nullptr;
 
-        void setVarSplitPass(VarSplitPass* p) { splitPass = p; }
-        VarSplitPass* getVarSplitPass() { return splitPass; }
+
+        VarSplitPass* getVarSplitPass() { return kernel.getVarSplitPass(); }
 
         unsigned int getSubRetLoc(G4_BB* bb)
         {
@@ -811,22 +852,18 @@ namespace vISA
             const char* name = builder.getNameString(kernel.fg.mem, 24, "RET__loc%d", retLoc);
             G4_Declare* dcl = builder.createDeclareNoLookup(name, G4_GRF, 2, 1, Type_UD);
 
-            if (VISA_WA_CHECK(builder.getPWaTable(), WaSIMD16SIMD32CallDstAlign))
-            {
-                dcl->setSubRegAlign(Sixteen_Word);
-                setSubRegAlign(dcl, Sixteen_Word);
-            }
-            else
-            {
-                // call destination must still be QWord aligned
-                dcl->setSubRegAlign(Four_Word);
-                setSubRegAlign(dcl, Four_Word);
-            }
+
+            // call destination must still be QWord aligned
+            dcl->setSubRegAlign(Four_Word);
+            setSubRegAlign(dcl, Four_Word);
 
             retDecls[retLoc] = dcl;
             return dcl;
         }
 
+        G4_INST* getSaveBE_FPInst() const { return saveBE_FPInst; };
+        G4_INST* getRestoreBE_FPInst() const { return restoreBE_FPInst; };
+
         static unsigned int owordToGRFSize(unsigned int numOwords);
         static unsigned int hwordToGRFSize(unsigned int numHwords);
         static unsigned int GRFToHwordSize(unsigned int numGRFs);
@@ -847,6 +884,16 @@ namespace vISA
             addrFlagSpillDcls.insert(dcl);
         }
 
+        void addUndefinedDcl(G4_Declare* dcl)
+        {
+            UndeclaredVars.push_back(dcl);
+        }
+
+        bool isUndefinedDcl(G4_Declare* dcl)
+        {
+            return std::find(UndeclaredVars.begin(), UndeclaredVars.end(), dcl) != UndeclaredVars.end();
+        }
+
         unsigned int getSplitVarNum(G4_Declare* dcl)
         {
             auto dclid = dcl->getDeclId();
@@ -921,6 +968,39 @@ namespace vISA
             lr->setTopDcl(dcl);
         }
 
+        LSLiveRange* getSafeLSLR(G4_Declare* dcl) const
+        {
+            auto dclid = dcl->getDeclId();
+            assert(dclid <= vars.size());
+            return vars[dclid].LSLR;
+        }
+
+        LSLiveRange* getLSLR(G4_Declare* dcl) const
+        {
+            auto dclid = dcl->getDeclId();
+            if (dclid >= vars.size())
+            {
+                return nullptr;
+            }
+            return vars[dclid].LSLR;
+        }
+
+        void setLSLR(G4_Declare* dcl, LSLiveRange* lr)
+        {
+            auto dclid = dcl->getDeclId();
+            resize(dclid);
+            MUST_BE_TRUE(vars[dclid].LSLR == NULL, "Local live range already allocated for declaration");
+            vars[dclid].LSLR = lr;
+            lr->setTopDcl(dcl);
+        }
+
+        void resetLSLR(G4_Declare* dcl)
+        {
+            auto dclid = dcl->getDeclId();
+            resize(dclid);
+            vars[dclid].LSLR = nullptr;
+        }
+
         void resetLocalLR(G4_Declare* dcl)
         {
             auto dclid = dcl->getDeclId();
@@ -1110,11 +1190,13 @@ namespace vISA
                 {
                     unsigned int reg = bDcl->getRegVar()->getPhyReg()->asGreg()->getRegNum();
                     unsigned int bundle = get_bundle(reg, offset);
+                    unsigned int bundle1 = get_bundle(reg, offset + 1);
                     if (!(occupiedBundles & ((unsigned short)1 << bundle)))
                     {
                         bundleNum++;
                     }
                     occupiedBundles |= (unsigned short)1 << bundle;
+                    occupiedBundles |= (unsigned short)1 << bundle1;
                 }
             }
             if (bundleNum > 12)
@@ -1242,6 +1324,9 @@ namespace vISA
         {
             if (verifyAugmentation)
                 delete verifyAugmentation;
+
+            if (regChart)
+                delete regChart;
         }
 
         void emitFGWithLiveness(LivenessAnalysis& liveAnalysis);
@@ -1268,7 +1353,7 @@ namespace vISA
         int coloringRegAlloc();
         void addCallerSavePseudoCode();
         void addCalleeSavePseudoCode();
-        void addStoreRestoreForFP();
+        void addStoreRestoreToReturn();
         void markGraphBlockLocalVars();
         void verifyRA(LivenessAnalysis & liveAnalysis);
         void resetGlobalRAStates();
@@ -1471,4 +1556,23 @@ namespace vISA
     };
 }
 
+// TODO: Refactor code so that stack call related enums,
+// methods, etc. should be part of this class. Right now
+// code is scattered across FlowGraph and GraphColor.
+class StackCall
+{
+public:
+    // Following enum holds offsets of various fields in
+    // frame descriptor as per VISA ABI.
+    enum class FrameDescriptorOfsets
+    {
+        FE_FP = IR_Builder::SubRegs_Stackcall::FE_FP * 8,
+        FE_SP = IR_Builder::SubRegs_Stackcall::FE_SP * 8,
+        Ret_IP = IR_Builder::SubRegs_Stackcall::Ret_IP * 4,
+        Ret_EM = IR_Builder::SubRegs_Stackcall::Ret_EM * 4,
+        BE_FP = IR_Builder::SubRegs_Stackcall::BE_FP * 4,
+        BE_SP = IR_Builder::SubRegs_Stackcall::BE_SP * 4
+    };
+};
+
 #endif // __GRAPHCOLOR_H__
