@@ -34,8 +34,6 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 #include <map>
 
-struct AccInterval;
-
 namespace vISA
 {
     class HWConformity
@@ -43,9 +41,9 @@ namespace vISA
         IR_Builder& builder;
         G4_Kernel& kernel;
         vISA::Mem_Manager& mem;
-        int numAccSubDef = 0;
-        int numAccSubUse = 0;
 
+        // this must be set before calling the individual fix functions
+        G4_BB* curBB = nullptr;
 
         // This is added for data layout optimization.
         // Currently it only targets packed-byte pattern.
@@ -81,8 +79,25 @@ namespace vISA
         G4_DstRegRegion* insertMovAfter(INST_LIST_ITER& it, G4_DstRegRegion* dst, G4_Type type, G4_BB *bb, G4_SubReg_Align dstAlign = Any);
         G4_Operand* insertMovBefore(INST_LIST_ITER it, uint32_t srcNum, G4_Type type, G4_BB *bb,
             G4_SubReg_Align tmpAlign = Any);
+
+        // replace src <srcNum> for inst <*it> with a temp variable of type <type>
+        // This is used to satisfy various HW restrictions on src type/alignment/region/modifier/etc.
+        void replaceSrc(INST_LIST_ITER it, uint32_t srcNum, G4_Type type, G4_BB* bb, G4_SubReg_Align tmpAlign = Any)
+        {
+            G4_INST* inst = *it;
+            inst->setSrc(insertMovBefore(it, srcNum, type, bb, tmpAlign), srcNum);
+        }
+        // replace dst for inst <*it> with a temp variable of type <type>
+        // the original dst is now the dst of a new move instruction from the temp variable.
+        // This is used to satisfy various HW restrictions on dst type/alignment/etc.
+        void replaceDst(INST_LIST_ITER it, G4_Type type, G4_SubReg_Align dstAlign = Any)
+        {
+            G4_INST* inst = *it;
+            inst->setDest(insertMovAfter(it, inst->getDst(), type, curBB, dstAlign));
+        }
+
         G4_SrcRegRegion* insertCopyBefore(INST_LIST_ITER it, uint32_t srcNum, G4_SubReg_Align tmpAlign, G4_BB *bb);
-        G4_SrcRegRegion* insertCopyAtBBEntry(G4_BB* bb, uint8_t newExecSize, G4_Operand* src);
+        G4_SrcRegRegion* insertCopyAtBBEntry(G4_BB* bb, G4_ExecSize newExecSize, G4_Operand* src);
         void broadcast(G4_BB* bb, INST_LIST_ITER it, int srcPos, G4_SubReg_Align subAlign);
 
         G4_INST *splitInstWithByteDst(G4_INST *expand_op);
@@ -131,17 +146,13 @@ namespace vISA
         void insertMovAfter(INST_LIST_ITER& it, uint16_t stride, G4_BB* bb);
         void removeBadSrc(INST_LIST_ITER& it, G4_BB *bb, bool crossGRFDst, bool oneGRFSrc[3], bool badTwoGRFSrc[3]);
         uint8_t checkMinExecSize(G4_opcode op);
-        bool convertMAD2MAC(INST_LIST_ITER it, std::vector<G4_INST*> &madList, G4_BB *bb);
         void convertMAD2MulAdd(INST_LIST_ITER iter, G4_BB *bb);
-        G4_Type getAccType(G4_Type ty);
-        bool findHoistLocation(INST_LIST_ITER start, INST_LIST_ITER &end, uint16_t &movDist, G4_INST *boundary);
-        void convertComprInstSrcRegion(G4_INST *inst);
-        void addACCOpnd(G4_INST *inst, bool needACCSrc, int stride, G4_Type accTy);
         void maintainDU4TempMov(G4_INST *inst, G4_INST *movInst);
         void fixImm64(INST_LIST_ITER i, G4_BB* bb);
         bool checkSrcCrossGRF(INST_LIST_ITER &i, G4_BB* bb);
         G4_INST* checkSrcDefInst(G4_INST *inst, G4_INST *def_inst, uint32_t srcNum);
-        void fix64bInst(INST_LIST_ITER i, G4_BB* bb);
+        bool emulate64bMov(INST_LIST_ITER iter, G4_BB* bb);
+        bool fix64bInst(INST_LIST_ITER i, G4_BB* bb);
         bool fixPlaneInst(INST_LIST_ITER i, G4_BB* bb);
         void expandPlaneInst(INST_LIST_ITER i, G4_BB* bb);
         bool fixAddcSubb(G4_BB* bb);
@@ -188,6 +199,7 @@ namespace vISA
         bool generateFPMad(G4_BB* bb, INST_LIST_ITER iter);
         bool generateAlign1Mad(G4_BB* bb, INST_LIST_ITER iter);
         bool hasSameSubregOffset(G4_INST* inst) const;
+        bool hasSameSubregOffset(G4_INST* inst, uint32_t& byteOffset) const;
 
         void fixImmAndARFSrc(INST_LIST_ITER it, G4_BB *bb);
         void generateMacl(INST_LIST_ITER it, G4_BB *bb);
@@ -195,9 +207,8 @@ namespace vISA
 
         void fixSelCsel(INST_LIST_ITER it, G4_BB *bb);
 
+        void avoidDstSrcOverlap(INST_LIST_ITER i, G4_BB* bb);
         void* operator new(size_t sz, vISA::Mem_Manager& m) { return m.alloc(sz); }
-        void multiAccSubstitution(G4_BB* bb);
-
 
         bool checkSrcMod(INST_LIST_ITER it, G4_BB* bb, int srcPos);
 
@@ -207,6 +218,15 @@ namespace vISA
 
         void fixPredCtrl(INST_LIST_ITER it, G4_BB* bb);
 
+        // If alignment and region of all operands of any instruction are conformed
+        // by a dedicated function, return true.
+        // This is used to skip generic conformity functions, such as fixOpndTypeAlign().
+        bool hasDedicateAlignRegionConformity(INST_LIST_ITER it) const
+        {
+            return hasDedicateAlignRegionConformity(*it);
+        }
+        bool hasDedicateAlignRegionConformity(const G4_INST *I) const;
+
     public:
         HWConformity(IR_Builder& b, G4_Kernel &k, vISA::Mem_Manager& m) :
             builder(b), kernel(k), mem(m)
@@ -214,9 +234,6 @@ namespace vISA
         }
         void chkHWConformity();
         static void tryEliminateMadSrcModifier(IR_Builder &builder, G4_INST *inst);
-        int getNumAccSubDef() const { return numAccSubDef; }
-        int getNumAccSubUse() const { return numAccSubUse; }
-        void accSubstitution(G4_BB* bb);
         void localizeForAcc(G4_BB* bb);
     };
 }
