@@ -45,6 +45,7 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  */
 
+#include "IGC/common/StringMacros.hpp"
 #include <cstdint>
 #include <cctype>
 #include <list>
@@ -77,7 +78,8 @@ const char *printAsmName(const print_format_provider_t* header)
 {
     for (unsigned i = 0; i < header->getAttrCount(); i++)
     {
-        if (!strcmp(header->getString(header->getAttr(i)->nameIndex), "OutputAsmPath"))
+        const char* attrName = header->getString(header->getAttr(i)->nameIndex);
+        if (Attributes::isAttribute(Attributes::ATTR_OutputAsmPath, attrName))
             return header->getAttr(i)->value.stringVal;
     }
 
@@ -125,7 +127,7 @@ std::string printVariableDeclName(
     std::stringstream sstr;
 
     unsigned numPreDefinedVars = Get_CISA_PreDefined_Var_Count();
-    if (options->getOption(vISA_DumpIsaVarNames))
+    if (operand_prefix_kind == NOT_A_STATE_OPND)
     {
         sstr << getGenVarName(declID, *header);
     }
@@ -135,7 +137,7 @@ std::string printVariableDeclName(
         {   case STATE_OPND_SURFACE : sstr << printSurfaceName(declID); break;
             case STATE_OPND_SAMPLER : sstr << "S"   << declID; break;
             default                 :
-                if(options->getOption(vISA_easyIsaasm) == false ||
+                if (options->getOption(vISA_easyIsaasm) == false ||
                     options->getOption(vISA_PlatformIsSet) == false)
                 {
                     // If platform is not set then dcl instances are
@@ -156,7 +158,7 @@ std::string printVariableDeclName(
                 else
                 {
 
-                    if(declID < numPreDefinedVars)
+                    if (declID < numPreDefinedVars)
                     {
                         sstr << "V" << declID;
                     }
@@ -167,7 +169,7 @@ std::string printVariableDeclName(
                         std::string type =
                             G4_Type_Table[aliasDcl->getElemType()].str;
 
-                        while(aliasDcl->getAliasDeclare() != NULL)
+                        while (aliasDcl->getAliasDeclare() != NULL)
                         {
                             aliasOff += aliasDcl->getAliasOffset();
                             aliasDcl = aliasDcl->getAliasDeclare();
@@ -175,9 +177,9 @@ std::string printVariableDeclName(
 
                         // aliasDcl is top most dcl with aliasOff
                         // Lets find out declID of aliasDcl
-                        for(unsigned int i = 0; i < header->getVarCount(); i++)
+                        for (unsigned int i = 0; i < header->getVarCount(); i++)
                         {
-                            if(header->getVar(i)->dcl == aliasDcl)
+                            if (header->getVar(i)->dcl == aliasDcl)
                             {
                                 declID = i + numPreDefinedVars;
                                 break;
@@ -185,7 +187,7 @@ std::string printVariableDeclName(
                         }
 
                         sstr << "V"   << declID << "_" << type;
-                        if(aliasOff != 0)
+                        if (aliasOff != 0)
                         {
                             sstr << "_" << aliasOff;
                         }
@@ -202,8 +204,8 @@ static std::string printRegion(uint16_t region)
 {
     std::stringstream sstr;
     Common_ISA_Region_Val v_stride = (Common_ISA_Region_Val)(region & 0xF);
-    Common_ISA_Region_Val width = (Common_ISA_Region_Val)((region >> 4 ) & 0xF);
-    Common_ISA_Region_Val h_stride = (Common_ISA_Region_Val)((region >> 8 ) & 0xF);
+    Common_ISA_Region_Val width = (Common_ISA_Region_Val)((region >> 4) & 0xF);
+    Common_ISA_Region_Val h_stride = (Common_ISA_Region_Val)((region >> 8) & 0xF);
 
     if (width == REGION_NULL)
     {
@@ -305,9 +307,10 @@ std::string printVectorOperand(
         }
         case OPERAND_ADDRESSOF:
         {
-            sstr << "&" << printVariableDeclName(header, opnd.getOperandIndex(), opt, NOT_A_STATE_OPND)
-                 << (((short)opnd.opnd_val.addressof_opnd.addr_offset >= 0) ? "+" : "")
-                 << (((short)opnd.opnd_val.addressof_opnd.addr_offset));
+            sstr << "&" << printVariableDeclName(header, opnd.getOperandIndex(), opt, NOT_A_STATE_OPND);
+            if (opnd.opnd_val.addressof_opnd.addr_offset >= 0) {
+                sstr << "[" << (((short)opnd.opnd_val.addressof_opnd.addr_offset)) << "]";
+            }
             break;
         }
         case OPERAND_IMMEDIATE:
@@ -396,23 +399,40 @@ static void encodeStringLiteral(std::stringstream &ss, const char *str) {
   ss << '"';
 }
 
-std::string printAttribute(
-    const attribute_info_t* attr,
-    const print_format_provider_t* kernel,
-    bool isKernelAttr)
+std::string printAttributes(
+    const print_format_provider_t* header,
+    const int attr_count,
+    const attribute_info_t* attrs)
 {
     std::stringstream sstr;
 
-    if (attr->isInt && attr->size == 1 && attr->value.intVal == 0)
+    if (attr_count > 0)
     {
-        return sstr.str();
+        // decl's attr in the form: attr=<attr0, attr1, ...>
+        sstr << " attrs={" << printOneAttribute(header, &attrs[0]);
+        for (int j = 1; j < attr_count; j++)
+        {
+            sstr << ", " << printOneAttribute(header, &attrs[j]);
+        }
+        sstr << "}";
     }
 
-    std::string attrName = kernel->getString(attr->nameIndex);
-    sstr << "." << (isKernelAttr ? "kernel_" : "") << "attr " << attrName << "=";
+    return sstr.str();
+}
 
-    if (attr->isInt) {
-        if (isKernelAttr && attrName == "Target") {
+std::string printOneAttribute(
+    const print_format_provider_t* kernel,
+    const attribute_info_t* attr)
+{
+    std::stringstream sstr;
+    const char* attrName = kernel->getString(attr->nameIndex);
+    Attributes::ID aID = Attributes::getAttributeID(attrName);
+    MUST_BE_TRUE(aID != Attributes::ATTR_INVALID, "Invalid Attribute names!");
+
+    sstr << attrName;
+    if (attr->isInt && Attributes::isInt32(aID)) {
+        sstr << "=";
+        if (Attributes::isAttribute(Attributes::ATTR_Target, attrName)) {
             switch (attr->value.intVal) {
             case VISA_CM: sstr << "\"cm\""; break;
             case VISA_3D: sstr << "\"3d\""; break;
@@ -423,7 +443,8 @@ std::string printAttribute(
         } else {
             sstr << attr->value.intVal;
         }
-    } else if (attr->size > 0) {
+    } else if (Attributes::isCStr(aID) && attr->size > 0) {
+        sstr << "=";
         encodeStringLiteral(sstr, attr->value.stringVal);
     }
 
@@ -443,11 +464,7 @@ std::string printPredicateDecl(
          << "v_type=P "
          << "num_elts=" << pred->num_elements;
 
-    for (unsigned j = 0; j < pred->attribute_count; j++)
-    {
-        sstr << " " << printAttribute(&pred->attributes[j], header);
-    }
-
+    sstr << printAttributes(header, pred->attribute_count, pred->attributes);
     return sstr.str();
 }
 
@@ -466,10 +483,7 @@ std::string printAddressDecl(
          << "v_type=A "
          << "num_elts=" << addr->num_elements;
 
-    for (unsigned j = 0; j < addr->attribute_count; j++)
-    {
-        sstr << " " << printAttribute(&addr->attributes[j], header);
-    }
+    sstr << printAttributes(header, addr->attribute_count, addr->attributes);
 
     return sstr.str();
 }
@@ -484,10 +498,7 @@ std::string printSamplerDecl(
     sstr << ".decl S" << declID << " v_type=S";
     sstr << " num_elts=" << info->num_elements;
     sstr << " v_name=" << header->getString(info->name_index);
-    for (unsigned j = 0; j < info->attribute_count; j++)
-    {
-        sstr << " " << printAttribute(&info->attributes[j], header);
-    }
+    sstr << printAttributes(header, info->attribute_count, info->attributes);
     return sstr.str();
 }
 
@@ -503,10 +514,7 @@ std::string printSurfaceDecl(
     sstr << ".decl T" << declID + numPredefinedSurfaces << " v_type=T";
     sstr << " num_elts=" << info->num_elements;
     sstr << " v_name=" << header->getString(info->name_index);
-    for (unsigned j = 0; j < info->attribute_count; j++)
-    {
-        sstr << " " << printAttribute(&info->attributes[j], header);
-    }
+    sstr << printAttributes(header, info->attribute_count, info->attributes);
     return sstr.str();
 }
 
@@ -553,54 +561,34 @@ std::string printFuncInput(
 
 // declID is in the range of [0..#user-var], pre-defnied are not included
 std::string printVariableDecl(
-    const common_isa_header& isaHeader,
     const print_format_provider_t* header,
     unsigned declID,
-    bool isKernel,
-    unsigned int funcId,
     const Options *options)
 {
     MUST_BE_TRUE(header, "Argument Exception: argument header is NULL.");
     std::stringstream sstr;
 
-    const var_info_t *var = header->getVar(declID);
-    VISA_Type  isa_type = (VISA_Type)  ((var->bit_properties     ) & 0xF);
-    VISA_Align align    = var->getAlignment();
+    const var_info_t* var = header->getVar(declID);
+    VISA_Type  isa_type = (VISA_Type)((var->bit_properties) & 0xF);
+    VISA_Align align = var->getAlignment();
 
     unsigned numPreDefinedVars = Get_CISA_PreDefined_Var_Count();
-    sstr << ".decl " << printVariableDeclName(header, declID+numPreDefinedVars, options)
-         << " v_type=G"
-         << " type=" << CISATypeTable[isa_type].typeName
-         << " num_elts=" << var->num_elements;
+    sstr << ".decl " << printVariableDeclName(header, declID + numPreDefinedVars, options)
+        << " v_type=G"
+        << " type=" << CISATypeTable[isa_type].typeName
+        << " num_elts=" << var->num_elements;
 
-    if(align != ALIGN_UNDEF)
+    if (align != ALIGN_UNDEF)
         sstr << " align=" << Common_ISA_Get_Align_Name(align);
 
     if (var->alias_index)
     {
         sstr << " alias=<";
-        if (options->getOption(vISA_DumpIsaVarNames))
-        {
-            sstr << printVariableDeclName(header, var->alias_index, options);
-        }
-        else
-        {
-            if(options->getOption(vISA_easyIsaasm))
-            {
-                sstr << printVariableDeclName(header, var->alias_index, options);
-            }
-            else
-            {
-               sstr << 'V' << var->alias_index;
-            }
-        }
+        sstr << printVariableDeclName(header, var->alias_index, options);
         sstr << ", " << var->alias_offset << ">";
     }
 
-    for (unsigned j = 0; j < var->attribute_count; j++)
-    {
-        sstr << " " << printAttribute(&var->attributes[j], header);
-    }
+    sstr << printAttributes(header, var->attribute_count, var->attributes);
 
     return sstr.str();
 }
@@ -822,7 +810,7 @@ static std::string printInstructionCommon(
             ISA_Inst_Address == ISA_Inst_Table[opcode].type ||
             ISA_Inst_Compare == ISA_Inst_Table[opcode].type)
         {
-            bool saturate = (((VISA_Modifier)((getVectorOperand(inst, i).tag >> 3 ) & 0x7)) == MODIFIER_SAT);
+            bool saturate = (((VISA_Modifier)((getVectorOperand(inst, i).tag >> 3) & 0x7)) == MODIFIER_SAT);
             sstr << (saturate ? ".sat" : "");
         }
 
@@ -841,7 +829,7 @@ static std::string printInstructionCommon(
             {
                 const vector_opnd& curOpnd = getVectorOperand(inst, i);
 
-                if(curOpnd.getOperandClass() == OPERAND_ADDRESS)
+                if (curOpnd.getOperandClass() == OPERAND_ADDRESS)
                 {
                     sstr << printVectorOperand(header, curOpnd, opt, true);
                 }
@@ -851,23 +839,29 @@ static std::string printInstructionCommon(
 
                     unsigned opnd_index = curOpnd.getOperandIndex();
 
-                    if(curOpnd.getOperandClass() == OPERAND_GENERAL)
+                    if (curOpnd.getOperandClass() == OPERAND_GENERAL)
                     {
                         uint32_t numPredefined = Get_CISA_PreDefined_Var_Count();
-                        VISA_Type type = opnd_index < numPredefined ? getPredefinedVarType(mapExternalToInternalPreDefVar(opnd_index)) :
+                        VISA_Type type = opnd_index < numPredefined ?
+                            getPredefinedVarType(mapExternalToInternalPreDefVar(opnd_index)) :
                             header->getVar(opnd_index - numPredefined)->getType();
-                        int offset = curOpnd.opnd_val.gen_opnd.col_offset *
-                                     CISATypeTable[type].typeSize +
-                                     curOpnd.opnd_val.gen_opnd.row_offset * G4_GRF_REG_NBYTES;
-                        sstr << "&" << printVariableDeclName( header, opnd_index, opt) << "+"
-                             << offset;
+                        sstr << "&" << printVariableDeclName(header, opnd_index, opt);
+                        int offset =
+                            curOpnd.opnd_val.gen_opnd.col_offset * CISATypeTable[type].typeSize +
+                            curOpnd.opnd_val.gen_opnd.row_offset * numEltPerGRF(Type_UB);
+                        if (offset) {
+                            sstr << "[" << offset << "]";
+                        }
                     }
                     else if (curOpnd.getOperandClass() == OPERAND_STATE)
                     {
                         auto OpClass = curOpnd.getStateOpClass();
-                        sstr << "&" << printVariableDeclName(header, opnd_index, opt, OpClass) << "+"
-                             << curOpnd.opnd_val.state_opnd.offset *
-                                CISATypeTable[ISA_TYPE_D].typeSize;
+                        sstr << "&" << printVariableDeclName(header, opnd_index, opt, OpClass);
+                        int offset =
+                            curOpnd.opnd_val.state_opnd.offset * CISATypeTable[ISA_TYPE_D].typeSize;
+                        if (offset) {
+                            sstr << "[" << offset << "]";
+                        }
                     }
                     else
                     {
@@ -1319,7 +1313,7 @@ static std::string printInstructionMisc(
 
             sstr << ".";
 
-            if((VISAVarLifetime)(properties & 1) == LIFETIME_START)
+            if ((VISAVarLifetime)(properties & 1) == LIFETIME_START)
             {
                 sstr << "start ";
             }
@@ -1331,17 +1325,17 @@ static std::string printInstructionMisc(
             // Since variable id is in non-standard form, we cannot invoke
             // printOperand directly on it
             unsigned char type = (properties >> 4) & 0x3;
-            if(type == OPERAND_GENERAL)
+            if (type == OPERAND_GENERAL)
             {
                 // General variable
                 sstr << printVariableDeclName(header, varId, opt, NOT_A_STATE_OPND);
             }
-            else if(type == OPERAND_ADDRESS)
+            else if (type == OPERAND_ADDRESS)
             {
                 // Address variable
                 sstr << "A" << varId;
             }
-            else if(type == OPERAND_PREDICATE)
+            else if (type == OPERAND_PREDICATE)
             {
                 // Predicate variable
                 sstr << "P" << varId;
@@ -1400,14 +1394,14 @@ static std::string printInstructionSampler(
                 sampler = getPrimitiveOperand<uint8_t>(inst, i++);
 
             uint8_t surface   = getPrimitiveOperand<uint8_t>(inst, i++);
-            uint8_t channel   = (mod     ) & 0xF;
+            uint8_t channel   = (mod    ) & 0xF;
             uint8_t SIMD_mode = (mod >> 4) & 0x3;
 
-            if((unsigned)SIMD_mode == 0)
+            if ((unsigned)SIMD_mode == 0)
             {
                 SIMD_mode = 8;
             }
-            else if((unsigned)SIMD_mode == 1)
+            else if ((unsigned)SIMD_mode == 1)
             {
                 SIMD_mode = 16;
             }
@@ -1422,8 +1416,7 @@ static std::string printInstructionSampler(
             if (opcode == ISA_SAMPLE)
                 sstr << " S" << (unsigned)sampler;
 
-            sstr << " T"
-                 << (unsigned)surface;
+            sstr << " " << printSurfaceName(surface);
 
             /// u offset
             sstr << printOperand(header, inst, i++, opt);
@@ -1464,10 +1457,10 @@ static std::string printInstructionSampler(
             }
 
             uint8_t channels = getPrimitiveOperand<uint8_t>(inst, i++);
-            if( channels & 0x1) sstr << "R";
-            if( channels & 0x2) sstr << "G";
-            if( channels & 0x4) sstr << "B";
-            if( channels & 0x8) sstr << "A";
+            if (channels & 0x1) sstr << "R";
+            if (channels & 0x2) sstr << "G";
+            if (channels & 0x4) sstr << "B";
+            if (channels & 0x8) sstr << "A";
 
             sstr << " " << printExecutionSize(inst->opcode, inst->execsize) << " ";
 
@@ -1477,7 +1470,8 @@ static std::string printInstructionSampler(
             sstr << " S" << printOperand(header, inst, i++, opt);
 
             // surface
-            sstr << " T" << printOperand(header, inst, i++, opt);
+            uint8_t surface = getPrimitiveOperand<uint8_t>(inst, i++);
+            sstr << " " << printSurfaceName(surface);
 
             // dst
             sstr << printOperand(header, inst, i++, opt);
@@ -1505,17 +1499,18 @@ static std::string printInstructionSampler(
             }
 
             uint8_t channels = getPrimitiveOperand<uint8_t>(inst, i++);
-            if( channels & 0x1) sstr << "R";
-            if( channels & 0x2) sstr << "G";
-            if( channels & 0x4) sstr << "B";
-            if( channels & 0x8) sstr << "A";
+            if (channels & 0x1) sstr << "R";
+            if (channels & 0x2) sstr << "G";
+            if (channels & 0x4) sstr << "B";
+            if (channels & 0x8) sstr << "A";
 
             sstr << " " << printExecutionSize(inst->opcode, inst->execsize) << " ";
 
             sstr << printOperand(header, inst, i++, opt);
 
             // surface
-            sstr << " T" << printOperand(header, inst, i++, opt);
+            uint8_t surface = getPrimitiveOperand<uint8_t>(inst, i++);
+            sstr << " " << printSurfaceName(surface);
 
             // dst
             sstr << printOperand(header, inst, i++, opt);
@@ -1572,7 +1567,8 @@ static std::string printInstructionSampler(
             sstr << " S" << printOperand(header, inst, i++, opt);
 
             // surface
-            sstr << " T" << printOperand(header, inst, i++, opt);
+            uint8_t surface = getPrimitiveOperand<uint8_t>(inst, i++);
+            sstr << " " << printSurfaceName(surface);
 
             // dst
             sstr << printOperand(header, inst, i++, opt);
@@ -1591,17 +1587,19 @@ static std::string printInstructionSampler(
         {
             VISASampler3DSubOpCode subop = (VISASampler3DSubOpCode)getPrimitiveOperand<uint8_t>(inst, i++);
             sstr << SAMPLE_OP_3D_NAME[subop];
-            sstr << " " << printExecutionSize(inst->opcode, inst->execsize) << " ";
-
-            if (subop == VISA_3D_RESINFO)
+            if (subop == VISA_3D_RESINFO || subop == VISA_3D_SAMPLEINFO)
             {
                 // channelMask
                 uint8_t channels = getPrimitiveOperand<uint8_t>(inst, i++);
                 ChannelMask chMask = ChannelMask::createFromBinary(ISA_3D_INFO, channels);
-                sstr << chMask.getString();
+                sstr << "." << chMask.getString();
             }
+
+            sstr << " " << printExecutionSize(inst->opcode, inst->execsize) << " ";
+
             // surface
-            sstr << " T" << printOperand(header, inst, i++, opt);
+            uint8_t surface = getPrimitiveOperand<uint8_t>(inst, i++);
+            sstr << " " << printSurfaceName(surface);
 
             if (subop == VISA_3D_RESINFO)
             {
@@ -1620,14 +1618,13 @@ static std::string printInstructionSampler(
             uint8_t sampler = getPrimitiveOperand<uint8_t>(inst, i++);
             uint8_t surface = getPrimitiveOperand<uint8_t>(inst, i++);
             sstr << ISA_Inst_Table[opcode].str
-                 << "."
-                 << channel_mask_str[(channel & 0xf)]
-                 << "."
-                 << sampler_channel_output_str[ChannelMask::getChannelOutputFormat(channel)]
-                 << " S"
-                 << (unsigned)sampler
-                 << " T"
-                 << (unsigned)surface;
+                << "."
+                << channel_mask_str[(channel & 0xf)]
+                << "."
+                << sampler_channel_output_str[ChannelMask::getChannelOutputFormat(channel)]
+                << " S"
+                << (unsigned)sampler
+                << " " << printSurfaceName(surface);
 
             /// u offset
             sstr << printOperand(header, inst, i++, opt);
@@ -1653,12 +1650,11 @@ static std::string printInstructionSampler(
             uint8_t surface = getPrimitiveOperand<uint8_t>(inst, i++);
 
             sstr << ISA_Inst_Table[opcode].str
-                 << "."
-                 << channel_mask_str[channel]
-                 << " T"
-                 << (unsigned)surface
-                 << " S"
-                 << (unsigned)sampler;
+                << "."
+                << channel_mask_str[channel]
+                << " " << printSurfaceName(surface)
+                << " S"
+                << (unsigned)sampler;
 
             /// u offset
             sstr << printOperand(header, inst, i++, opt);
@@ -1714,7 +1710,7 @@ static std::string printInstructionSampler(
                      sstr << ISA_Inst_Table[opcode].str
                           << "."
                           << va_sub_names[subOpcode]
-                          << " T" << (unsigned)surface;
+                          << " " << printSurfaceName(surface);
 
                      /// u offset
                      sstr << printOperand(header, inst, i++, opt);
@@ -1746,7 +1742,7 @@ static std::string printInstructionSampler(
                      sstr << ISA_Inst_Table[opcode].str
                           << "."
                           << va_sub_names[subOpcode]
-                          << " T" << (unsigned)surface
+                          << " " << printSurfaceName(surface)
                           << " S" << (unsigned)sampler;
 
                      /// u offset
@@ -1774,12 +1770,12 @@ static std::string printInstructionSampler(
                 case BoolCentroid_FOPCODE:
                 case Centroid_FOPCODE:
                 {
-                     uint8_t surface = getPrimitiveOperand<uint8_t>(inst, i++);
+                    uint8_t surface = getPrimitiveOperand<uint8_t>(inst, i++);
 
-                     sstr << ISA_Inst_Table[opcode].str
-                          << "."
-                          << va_sub_names[subOpcode]
-                          << " T" << (unsigned)surface;
+                    sstr << ISA_Inst_Table[opcode].str
+                        << "."
+                        << va_sub_names[subOpcode]
+                        << " " << printSurfaceName(surface);
 
                      /// u offset
                      sstr << printOperand(header, inst, i++, opt);
@@ -1809,7 +1805,7 @@ static std::string printInstructionSampler(
                      sstr << ISA_Inst_Table[opcode].str
                           << "."
                           << va_sub_names[subOpcode]
-                          << " T" << (unsigned)surface
+                          << " " << printSurfaceName(surface)
                           << " S" << (unsigned)sampler;
 
                      /// u offset
@@ -1818,13 +1814,13 @@ static std::string printInstructionSampler(
                      /// v offset
                      sstr << printOperand(header, inst, i++, opt);
 
-                     uint8_t execMode   =  getPrimitiveOperand<uint8_t>(inst, i  ) & 0x3;
+                     uint8_t execMode   =  getPrimitiveOperand<uint8_t>(inst, i ) & 0x3;
                      uint8_t regionSize = (getPrimitiveOperand<uint8_t>(inst, i++) & 0xC) >> 0x2;
 
                      sstr << " "
                           << (Convolve_FOPCODE == subOpcode ?
                               conv_exec_mode [execMode]     :
-                              ed_exec_mode   [execMode]    );
+                              ed_exec_mode   [execMode]   );
 
                      if (Convolve_FOPCODE == subOpcode)
                          sstr << " " << (regionSize & 0x1 ? "31x31" : "15x15");
@@ -1850,9 +1846,9 @@ static std::string printInstructionSampler(
                      uint8_t surface = getPrimitiveOperand<uint8_t>(inst, i++);
 
                      sstr << ISA_Inst_Table[opcode].str
-                          << "."
-                          << va_sub_names[subOpcode]
-                          << " T" << (unsigned)surface;
+                         << "."
+                         << va_sub_names[subOpcode]
+                         << " " << printSurfaceName(surface);
 
                      /// u offset
                      sstr << printOperand(header, inst, i++, opt);
@@ -1876,10 +1872,10 @@ static std::string printInstructionSampler(
                      uint8_t surface = getPrimitiveOperand<uint8_t>(inst, i++);
 
                      sstr << ISA_Inst_Table[opcode].str
-                          << "."
-                          << va_sub_names[subOpcode]
-                          << " T" << (unsigned)surface
-                          << " S" << (unsigned)sampler;
+                         << "."
+                         << va_sub_names[subOpcode]
+                         << " " << printSurfaceName(surface)
+                         << " S" << (unsigned)sampler;
 
                      /// u offset
                      sstr << printOperand(header, inst, i++, opt);
@@ -1910,9 +1906,9 @@ static std::string printInstructionSampler(
                      uint8_t surface = getPrimitiveOperand<uint8_t>(inst, i++);
 
                      sstr << ISA_Inst_Table[opcode].str
-                          << "."
-                          << va_sub_names[subOpcode]
-                          << " T" << (unsigned)surface;
+                         << "."
+                         << va_sub_names[subOpcode]
+                         << " " << printSurfaceName(surface);
 
                      /// u offset
                      sstr << printOperand(header, inst, i++, opt);
@@ -1960,9 +1956,9 @@ static std::string printInstructionSampler(
                      uint8_t surface = getPrimitiveOperand<uint8_t>(inst, i++);
 
                      sstr << ISA_Inst_Table[opcode].str
-                          << "."
-                          << va_sub_names[subOpcode]
-                          << " T" << (unsigned)surface;
+                         << "."
+                         << va_sub_names[subOpcode]
+                         << " " << printSurfaceName(surface);
 
                      /// u offset
                      sstr << printOperand(header, inst, i++, opt);
@@ -2013,17 +2009,16 @@ static std::string printInstructionSampler(
                             uint8_t sampler = getPrimitiveOperand<uint8_t>(inst, i++);
                             uint8_t surface = getPrimitiveOperand<uint8_t>(inst, i++);
 
-                            sstr << " T" << (unsigned)surface
+                            sstr << " " << printSurfaceName(surface)
                                  << " S" << (unsigned)sampler;
                         }
                         else
                         {
                             /// surface
                             uint8_t surface = getPrimitiveOperand<uint8_t>(inst, i++);
-                            sstr << " T" << (unsigned)surface;
+                            sstr << " " << printSurfaceName(surface);
                         }
 
-
                         /// u offset
                         sstr << printOperand(header, inst, i++, opt);
 
@@ -2043,7 +2038,7 @@ static std::string printInstructionSampler(
 
                             if (subOpcode == ISA_HDC_CONV)
                             {
-                                isBigKernel = (pixel_size & ( 1 << 4 ));
+                                isBigKernel = (pixel_size & (1 << 4));
                                 pixel_size = pixel_size & 0xF;
                             }
                             sstr << " " << pixel_size_str[pixel_size];
@@ -2087,7 +2082,7 @@ static std::string printInstructionSampler(
                         /// dst surface
                         uint8_t dst_surface = getPrimitiveOperand<uint8_t>(inst, i++);
 
-                        sstr << " T" << (unsigned)dst_surface;
+                        sstr << " " << printSurfaceName(dst_surface);
 
                         /// x offset
                         sstr << printOperand(header, inst, i++, opt);
@@ -2102,27 +2097,12 @@ static std::string printInstructionSampler(
 
             break;
         }
-        default: ASSERT_USER( false, "illegal opcode for sampler instruction");
+        default: ASSERT_USER(false, "illegal opcode for sampler instruction");
     }
 
     return sstr.str();
 }
 
-static std::string printSurfaceIndex(uint8_t surface)
-{
-    std::stringstream sstr;
-    unsigned numPreDefinedSurfs = Get_CISA_PreDefined_Surf_Count();
-    if (surface < numPreDefinedSurfs)
-    {
-        sstr << " " << vISAPreDefSurf[surface].name;
-    }
-    else
-    {
-        sstr << " " << "T" << (unsigned)surface;
-    }
-    return sstr.str();
-}
-
 static std::string printInstructionDataport(
     const print_format_provider_t* header,
     const CISA_INST* inst,
@@ -2183,7 +2163,7 @@ static std::string printInstructionDataport(
             sstr << (unsigned)block_height;
             sstr << ")";
 
-            sstr << " T" << (unsigned)surface;
+            sstr << " " << printSurfaceName(surface);
             sstr << " " << (unsigned)plane;
 
             /// x offset
@@ -2214,7 +2194,7 @@ static std::string printInstructionDataport(
             sstr << " (" << num_oword << ")";
 
             surface = getPrimitiveOperand<uint8_t>(inst, i++);
-            sstr <<printSurfaceIndex(surface);
+            sstr << " " << printSurfaceName(surface);
 
             /// offset
             sstr << printOperand(header, inst, i++, opt);
@@ -2232,7 +2212,7 @@ static std::string printInstructionDataport(
 
             elt_size = getPrimitiveOperand<uint8_t>(inst, i++);
             elt_size = elt_size & 0x3;
-            switch((GATHER_SCATTER_ELEMENT_SIZE)elt_size)
+            switch ((GATHER_SCATTER_ELEMENT_SIZE)elt_size)
             {
                 case GATHER_SCATTER_BYTE:
                     elt_size = 1;
@@ -2244,7 +2224,7 @@ static std::string printInstructionDataport(
                     elt_size = 4;
                     break;
                 default:
-                    ASSERT_USER( 0, "Incorrect element size for Gather/Scatter CISA inst." );
+                    ASSERT_USER(0, "Incorrect element size for Gather/Scatter CISA inst.");
                     break;
             }
             if (ISA_GATHER == opcode)
@@ -2255,7 +2235,7 @@ static std::string printInstructionDataport(
             num_elts = getPrimitiveOperand<uint8_t>(inst, i++);
 
             // modifier
-            if ( ISA_GATHER == opcode && modifier & 0x1 )
+            if (ISA_GATHER == opcode && modifier & 0x1)
             {
                 sstr << ".mod";
             }
@@ -2274,7 +2254,7 @@ static std::string printInstructionDataport(
 
             //surface
             surface = getPrimitiveOperand<uint8_t>(inst, i++);
-            sstr <<printSurfaceIndex(surface);
+            sstr << " " << printSurfaceName(surface);
 
             /// global offset
             sstr << printOperand(header, inst, i++, opt);
@@ -2297,7 +2277,7 @@ static std::string printInstructionDataport(
             sstr << " " << printExecutionSize(inst->opcode, inst->execsize);
 
             surface = getPrimitiveOperand<uint8_t>(inst, i++);
-            sstr <<printSurfaceIndex(surface);
+            sstr << " " << printSurfaceName(surface);
 
             /// u offset
             sstr << printOperand(header, inst, i++, opt);
@@ -2330,7 +2310,7 @@ static std::string printInstructionDataport(
 
             /// surface
             surface = getPrimitiveOperand<uint8_t>(inst, i++);
-            sstr <<printSurfaceIndex(surface);
+            sstr << " " << printSurfaceName(surface);
 
             /// global offset
             sstr << printOperand(header, inst, i++, opt);
@@ -2361,7 +2341,7 @@ static std::string printInstructionDataport(
 
             /// surface
             surface = getPrimitiveOperand<uint8_t>(inst, i++);
-            sstr <<printSurfaceIndex(surface);
+            sstr << " " << printSurfaceName(surface);
 
             /// global offset
             sstr << printOperand(header, inst, i++, opt);
@@ -2379,26 +2359,27 @@ static std::string printInstructionDataport(
             uint16_t mode = getPrimitiveOperand<uint16_t>(inst, i++);
             uint8_t surface;
 
-            if( ( mode ) != 0 )
+            if ((mode) != 0)
             {
                 sstr << ".";
                 if (mode & (0x1 << 2)) sstr << "<RTI>";
-                if( mode & (0x1 << 0x3)) sstr << "<A>";
-                if( mode & (0x1 << 0x4)) sstr << "<O>";
-                if( mode & (0x1 << 0x5)) sstr << "<Z>";
-                if( mode & (0x1 << 0x6)) sstr << "<ST>";
-                if( mode & (0x1 << 0x7)) sstr << "<LRTW>";
-                if( mode & (0x1 << 0x8)) sstr << "<CPS>";
-                if( mode & (0x1 << 0x9)) sstr << "<PS>";
-                if( mode & (0x1 << 0x10)) sstr << "<CM>";
+                if (mode & (0x1 << 0x3)) sstr << "<A>";
+                if (mode & (0x1 << 0x4)) sstr << "<O>";
+                if (mode & (0x1 << 0x5)) sstr << "<Z>";
+                if (mode & (0x1 << 0x6)) sstr << "<ST>";
+                if (mode & (0x1 << 0x7)) sstr << "<LRTW>";
+                if (mode & (0x1 << 0x8)) sstr << "<CPS>";
+                if (mode & (0x1 << 0x9)) sstr << "<PS>";
+                if (mode & (0x1 << 0x10)) sstr << "<CM>";
                 if (mode & (0x1 << 0x11)) sstr << "<SI>";
+                if (mode & (0x1 << 0x12)) sstr << "<NULLRT>";
             }
 
-            sstr << " " << printExecutionSize(inst->opcode, inst->execsize) << " ";
+            sstr << " " << printExecutionSize(inst->opcode, inst->execsize);
 
             // surface
             surface = getPrimitiveOperand<uint8_t>(inst, i++);
-            sstr <<printSurfaceIndex(surface);
+            sstr << " " << printSurfaceName(surface);
 
             while (i < inst->opnd_count)
             {
@@ -2413,7 +2394,7 @@ static std::string printInstructionDataport(
 
             /// surface
             surface = getPrimitiveOperand<uint8_t>(inst, i++);
-            sstr <<printSurfaceIndex(surface);
+            sstr << " " << printSurfaceName(surface);
 
             /// offsets
             sstr << printOperand(header, inst, i++, opt);
@@ -2435,7 +2416,7 @@ static std::string printInstructionDataport(
 
             /// surface
             surface = getPrimitiveOperand<uint8_t>(inst, i++);
-            sstr << printSurfaceIndex(surface);
+            sstr << " " << printSurfaceName(surface);
 
             /// u
             sstr << printOperand(header, inst, i++, opt);
@@ -2485,7 +2466,7 @@ std::string printKernelHeader(
     sstr << ".version " << (unsigned)(major_version) << "." << (unsigned)(minor_version) << "\n";
 
     std::string name = header->getString(header->getNameIndex());
-    std::replace_if(name.begin(), name.end(), [] (char c) { return c == '.'; } , ' ');
+    std::replace_if (name.begin(), name.end(), [] (char c) { return c == '.'; } , ' ');
 
     sstr << (!isKernel ? ".global_function " : ".kernel ");
     encodeStringLiteral(sstr, name.c_str());
@@ -2533,7 +2514,7 @@ std::string printKernelHeader(
         //.decl  V<#> name=<name> type=<type> num_elts=<num_elements> [align=<align>] [alias=(<alias_index>,<alias_offset>)]
         for (unsigned i = 0; i < header->getVarCount(); i++)
         {
-            sstr << "\n" << printVariableDecl(isaHeader, header, i, isKernel, funcionId, options);
+            sstr << "\n" << printVariableDecl(header, i, options);
         }
         // address decls
         for (unsigned i = 0; i < header->getAddrCount(); i++)
@@ -2567,15 +2548,17 @@ std::string printKernelHeader(
     bool isTargetSet = false;
     for (unsigned i = 0; i < header->getAttrCount(); i++)
     {
-        sstr << "\n" << printAttribute(header->getAttr(i), header, true);
-        if (strcmp(header->getString(header->getAttr(i)->nameIndex), "Target") == 0)
+        sstr << "\n.kernel_attr " << printOneAttribute(header, header->getAttr(i));
+        const char* attrName = header->getString(header->getAttr(i)->nameIndex);
+        if (Attributes::isAttribute(Attributes::ATTR_Target, attrName))
         {
             isTargetSet = true;
         }
     }
     if (isTargetSet == false)
     {
-        sstr << "\n" << ".kernel_attr Target=";
+        const char* attrName = Attributes::getAttributeName(Attributes::ATTR_Target);
+        sstr << "\n" << ".kernel_attr " << attrName << "=";
         switch (options->getTarget()) {
         case VISA_CM: sstr << "\"cm\""; break;
         case VISA_3D: sstr << "\"3d\""; break;
@@ -2604,7 +2587,7 @@ std::string printInstruction(
             sstr << "    ";
         }
 
-        switch(ISA_Inst_Table[opcode].type)
+        switch (ISA_Inst_Table[opcode].type)
         {
             case ISA_Inst_Mov:
             case ISA_Inst_Sync:
