@@ -51,6 +51,7 @@ void LocalScheduler::localScheduling()
     const Options *m_options = fg.builder->getOptions();
     LatencyTable LT(fg.builder);
 
+    uint32_t totalCycles = 0;
     for (; ib != bend; ++ib)
     {
         unsigned instCountBefore = (uint32_t)(*ib)->size();
@@ -105,6 +106,7 @@ void LocalScheduler::localScheduling()
             bbInfo[i].staticCycle = schedule.sequentialCycle;
             bbInfo[i].sendStallCycle = schedule.sendStallCycle;
             bbInfo[i].loopNestLevel = (*ib)->getNestLevel();
+            totalCycles += schedule.sequentialCycle;
         }
 
         i++;
@@ -112,6 +114,8 @@ void LocalScheduler::localScheduling()
     FINALIZER_INFO* jitInfo = fg.builder->getJitInfo();
     jitInfo->BBInfo = bbInfo;
     jitInfo->BBNum = i;
+
+    fg.builder->getcompilerStats().SetI64(CompilerStats::numCyclesStr(), totalCycles, fg.getKernel()->getSimdSize());
 }
 
 void G4_BB_Schedule::dumpSchedule(G4_BB *bb)
@@ -212,7 +216,8 @@ G4_BB_Schedule::G4_BB_Schedule(G4_Kernel* k, Mem_Manager& m, G4_BB* block,
 
     DDD ddd(mem, bb, LT, k);
     // Generate pairs of TypedWrites
-    bool doMessageFuse = (k->fg.builder->fuseTypedWrites() && k->getSimdSize() >= 16) ||
+    bool doMessageFuse =
+        (k->fg.builder->fuseTypedWrites() && k->getSimdSize() >= g4::SIMD16) ||
         k->fg.builder->fuseURBMessage();
 
     if (doMessageFuse)
@@ -220,7 +225,14 @@ G4_BB_Schedule::G4_BB_Schedule(G4_Kernel* k, Mem_Manager& m, G4_BB* block,
         ddd.pairTypedWriteOrURBWriteNodes(bb);
     }
 
-    lastCycle = ddd.listSchedule(this);
+    if (getOptions()->getOption(vISA_ScheduleForReadSuppression) && ddd.getIsThreeSourceBlock())
+    {
+        lastCycle = ddd.listScheduleForSuppression(this);
+    }
+    else
+    {
+        lastCycle = ddd.listSchedule(this);
+    }
 
     if (getOptions()->getOption(vISA_DumpSchedule))
     {
@@ -360,7 +372,7 @@ void DDD::getBucketsForOperand(G4_INST* inst, Gen4_Operand_Number opnd_num,
 
     switch (phyReg->getKind()) {
     case G4_VarBase::VK_phyGReg:
-        startingBucket = opnd->getLinearizedStart() / G4_GRF_REG_NBYTES;
+        startingBucket = opnd->getLinearizedStart() / numEltPerGRF(Type_UB);
         canSpanMultipleBuckets = true;
         break;
     case G4_VarBase::VK_phyAReg: {
@@ -399,7 +411,7 @@ void DDD::getBucketsForOperand(G4_INST* inst, Gen4_Operand_Number opnd_num,
     if (startingBucket != UNINIT_BUCKET) {
         if (canSpanMultipleBuckets) {
             assert(base->isGreg());
-            unsigned divisor = G4_GRF_REG_NBYTES;
+            unsigned divisor = numEltPerGRF(Type_UB);
             int baseBucket = GRF_BUCKET;
             int endingBucket = baseBucket + opnd->getLinearizedEnd() / divisor;
             MUST_BE_TRUE(endingBucket >= startingBucket, "Ending bucket less than starting bucket");
@@ -784,7 +796,7 @@ bool DDD::hasReadSuppression(G4_INST *prevInst, G4_INST *nextInst, BitSet &liveD
     int nextInstRegs[2][G4_MAX_SRCS];
     bool isCurrScalar[G4_MAX_SRCS] = { false, false, false, false };
     bool isNextScalar[G4_MAX_SRCS] = { false, false, false, false };
-    int execSize = 0;
+    int opndSize = 0;
 
     //Get the sources of previous instruction
     for (unsigned i = 0; i < 3; i++)
@@ -800,14 +812,14 @@ bool DDD::hasReadSuppression(G4_INST *prevInst, G4_INST *nextInst, BitSet &liveD
                 srcOpnd->asSrcRegRegion()->getBase()->isRegVar())
             {
                 G4_RegVar* baseVar = static_cast<G4_RegVar*>(srcOpnd->asSrcRegRegion()->getBase());
-                execSize = srcOpnd->getLinearizedEnd() - srcOpnd->getLinearizedStart() + 1;
+                opndSize = srcOpnd->getLinearizedEnd() - srcOpnd->getLinearizedStart() + 1;
                 if (baseVar->isGreg()) {
                     uint32_t byteAddress = srcOpnd->getLinearizedStart();
-                    currInstRegs[0][i] = byteAddress / GENX_GRF_REG_SIZ;
+                    currInstRegs[0][i] = byteAddress / numEltPerGRF(Type_UB);
 
-                    if (execSize > 32)
+                    if (opndSize > getGRFSize())
                     {
-                        currInstRegs[1][i] = currInstRegs[0][i] + (execSize + GENX_GRF_REG_SIZ - 1) / GENX_GRF_REG_SIZ - 1;
+                        currInstRegs[1][i] = currInstRegs[0][i] + (opndSize + numEltPerGRF(Type_UB) - 1) / numEltPerGRF(Type_UB) - 1;
                     }
                     else if (srcOpnd->asSrcRegRegion()->isScalar()) //No Read suppression for SIMD 16/scalar src
                     {
@@ -833,16 +845,16 @@ bool DDD::hasReadSuppression(G4_INST *prevInst, G4_INST *nextInst, BitSet &liveD
                 srcOpnd->asSrcRegRegion()->getBase()->isRegVar())
             {
                 G4_RegVar* baseVar = static_cast<G4_RegVar*>(srcOpnd->asSrcRegRegion()->getBase());
-                execSize = srcOpnd->getLinearizedEnd() - srcOpnd->getLinearizedStart() + 1;
+                opndSize = srcOpnd->getLinearizedEnd() - srcOpnd->getLinearizedStart() + 1;
                 if (baseVar->isGreg()) {
                     uint32_t byteAddress = srcOpnd->getLinearizedStart();
-                    nextInstRegs[0][i] = byteAddress / GENX_GRF_REG_SIZ;
+                    nextInstRegs[0][i] = byteAddress / numEltPerGRF(Type_UB);
 
                     liveSrc.set(nextInstRegs[0][i], true);  //Set live
 
-                    if (execSize > 32)
+                    if (opndSize > getGRFSize())
                     {
-                        int reg = nextInstRegs[0][i] + (execSize + GENX_GRF_REG_SIZ - 1) / GENX_GRF_REG_SIZ - 1;
+                        int reg = nextInstRegs[0][i] + (opndSize + numEltPerGRF(Type_UB) - 1) / numEltPerGRF(Type_UB) - 1;
                         nextInstRegs[1][i] =  reg;
                         liveSrc.set(reg, true);  //Set live
                     }
@@ -869,17 +881,17 @@ bool DDD::hasReadSuppression(G4_INST *prevInst, G4_INST *nextInst, BitSet &liveD
         !nextDstOpnd->isIndirect() &&
         nextDstOpnd->isGreg())
     {
-        execSize = nextDstOpnd->getLinearizedEnd() - nextDstOpnd->getLinearizedStart() + 1;
+        opndSize = nextDstOpnd->getLinearizedEnd() - nextDstOpnd->getLinearizedStart() + 1;
         uint32_t byteAddress = nextDstOpnd->getLinearizedStart();
-        dstReg0 = byteAddress / GENX_GRF_REG_SIZ;
+        dstReg0 = byteAddress / numEltPerGRF(Type_UB);
         liveDst.set(dstReg0, true);
-        if (execSize <= 32)
+        if (opndSize <= getGRFSize())
         {
             nextInstSimd8 = true;
         }
         else
         {
-            dstReg1 = dstReg0 + (execSize + GENX_GRF_REG_SIZ - 1) / GENX_GRF_REG_SIZ - 1;
+            dstReg1 = dstReg0 + (opndSize + numEltPerGRF(Type_UB) - 1) / numEltPerGRF(Type_UB) - 1;
             liveDst.set(dstReg1, true);
         }
     }
@@ -893,22 +905,22 @@ bool DDD::hasReadSuppression(G4_INST *prevInst, G4_INST *nextInst, BitSet &liveD
         !dstOpnd->isIndirect() &&
         dstOpnd->isGreg())
     {
-        execSize = dstOpnd->getLinearizedEnd() - dstOpnd->getLinearizedStart() + 1;
+        opndSize = dstOpnd->getLinearizedEnd() - dstOpnd->getLinearizedStart() + 1;
         uint32_t byteAddress = dstOpnd->getLinearizedStart();
-        dstReg0 = byteAddress / GENX_GRF_REG_SIZ;
+        dstReg0 = byteAddress / numEltPerGRF(Type_UB);
         //If there is RAW and WAW dependence
         if (liveSrc.isSet(dstReg0) || liveDst.isSet(dstReg0))
         {
             return false;
         }
 
-        if (execSize <= 32)
+        if (opndSize <= getGRFSize())
         {
             curInstSimd8 = true;
         }
         else
         {
-            dstReg1 = dstReg0 + (execSize + GENX_GRF_REG_SIZ - 1) / GENX_GRF_REG_SIZ - 1;
+            dstReg1 = dstReg0 + (opndSize + numEltPerGRF(Type_UB) - 1) / numEltPerGRF(Type_UB) - 1;
 
             //If there is RAW and WAW dependence
             if (liveSrc.isSet(dstReg1) || liveDst.isSet(dstReg1))
@@ -968,7 +980,7 @@ bool DDD::hasReadSuppression(G4_INST *prevInst, G4_INST *nextInst, BitSet &liveD
     return false;
 }
 
-bool DDD::hasReadSuppression(G4_INST* prevInst, G4_INST* nextInst)
+bool DDD::hasReadSuppression(G4_INST* prevInst, G4_INST* nextInst, bool multiSuppression)
 {
     //Not three source
     if (nextInst->getNumSrc() != 3 || nextInst->isSend())
@@ -1012,7 +1024,7 @@ bool DDD::hasReadSuppression(G4_INST* prevInst, G4_INST* nextInst)
     int nextInstRegs[2][G4_MAX_SRCS];
     bool isCurrScalar[G4_MAX_SRCS] = { false, false, false, false };
     bool isNextScalar[G4_MAX_SRCS] = { false, false, false, false };
-    int execSize = 0;
+    int opndSize = 0;
 
     //Get the sources of previous instruction
     for (unsigned i = 0; i < 3; i++)
@@ -1028,14 +1040,14 @@ bool DDD::hasReadSuppression(G4_INST* prevInst, G4_INST* nextInst)
                 srcOpnd->asSrcRegRegion()->getBase()->isRegVar())
             {
                 G4_RegVar* baseVar = static_cast<G4_RegVar*>(srcOpnd->asSrcRegRegion()->getBase());
-                execSize = srcOpnd->getLinearizedEnd() - srcOpnd->getLinearizedStart() + 1;
+                opndSize = srcOpnd->getLinearizedEnd() - srcOpnd->getLinearizedStart() + 1;
                 if (baseVar->isGreg()) {
                     uint32_t byteAddress = srcOpnd->getLinearizedStart();
-                    currInstRegs[0][i] = byteAddress / GENX_GRF_REG_SIZ;
+                    currInstRegs[0][i] = byteAddress / numEltPerGRF(Type_UB);
 
-                    if (execSize > 32)
+                    if (opndSize > getGRFSize())
                     {
-                        currInstRegs[1][i] = currInstRegs[0][i] + (execSize + GENX_GRF_REG_SIZ - 1) / GENX_GRF_REG_SIZ - 1;
+                        currInstRegs[1][i] = currInstRegs[0][i] + (opndSize + numEltPerGRF(Type_UB) - 1) / numEltPerGRF(Type_UB) - 1;
                     }
                     else if (srcOpnd->asSrcRegRegion()->isScalar()) //No Read suppression for SIMD 16/scalar src
                     {
@@ -1061,14 +1073,14 @@ bool DDD::hasReadSuppression(G4_INST* prevInst, G4_INST* nextInst)
                 srcOpnd->asSrcRegRegion()->getBase()->isRegVar())
             {
                 G4_RegVar* baseVar = static_cast<G4_RegVar*>(srcOpnd->asSrcRegRegion()->getBase());
-                execSize = srcOpnd->getLinearizedEnd() - srcOpnd->getLinearizedStart() + 1;
+                opndSize = srcOpnd->getLinearizedEnd() - srcOpnd->getLinearizedStart() + 1;
                 if (baseVar->isGreg()) {
                     uint32_t byteAddress = srcOpnd->getLinearizedStart();
-                    nextInstRegs[0][i] = byteAddress / GENX_GRF_REG_SIZ;
+                    nextInstRegs[0][i] = byteAddress / numEltPerGRF(Type_UB);
 
-                    if (execSize > 32)
+                    if (opndSize > getGRFSize())
                     {
-                        int reg = nextInstRegs[0][i] + (execSize + GENX_GRF_REG_SIZ - 1) / GENX_GRF_REG_SIZ - 1;
+                        int reg = nextInstRegs[0][i] + (opndSize + numEltPerGRF(Type_UB) - 1) / numEltPerGRF(Type_UB) - 1;
                         nextInstRegs[1][i] = reg;
                     }
                     if (srcOpnd->asSrcRegRegion()->isScalar()) //No Read suppression for SIMD 16/scalar src
@@ -1094,16 +1106,16 @@ bool DDD::hasReadSuppression(G4_INST* prevInst, G4_INST* nextInst)
         !nextDstOpnd->isIndirect() &&
         nextDstOpnd->isGreg())
     {
-        execSize = nextDstOpnd->getLinearizedEnd() - nextDstOpnd->getLinearizedStart() + 1;
+        opndSize = nextDstOpnd->getLinearizedEnd() - nextDstOpnd->getLinearizedStart() + 1;
         uint32_t byteAddress = nextDstOpnd->getLinearizedStart();
-        dstReg0 = byteAddress / GENX_GRF_REG_SIZ;
-        if (execSize <= 32)
+        dstReg0 = byteAddress / numEltPerGRF(Type_UB);
+        if (opndSize <= getGRFSize())
         {
             nextInstSimd8 = true;
         }
         else
         {
-            dstReg1 = dstReg0 + (execSize + GENX_GRF_REG_SIZ - 1) / GENX_GRF_REG_SIZ - 1;
+            dstReg1 = dstReg0 + (opndSize + numEltPerGRF(Type_UB) - 1) / numEltPerGRF(Type_UB) - 1;
         }
     }
 
@@ -1116,17 +1128,17 @@ bool DDD::hasReadSuppression(G4_INST* prevInst, G4_INST* nextInst)
         !dstOpnd->isIndirect() &&
         dstOpnd->isGreg())
     {
-        execSize = dstOpnd->getLinearizedEnd() - dstOpnd->getLinearizedStart() + 1;
+        opndSize = dstOpnd->getLinearizedEnd() - dstOpnd->getLinearizedStart() + 1;
         uint32_t byteAddress = dstOpnd->getLinearizedStart();
-        dstReg0 = byteAddress / GENX_GRF_REG_SIZ;
+        dstReg0 = byteAddress / numEltPerGRF(Type_UB);
 
-        if (execSize <= 32)
+        if (opndSize <= getGRFSize())
         {
             curInstSimd8 = true;
         }
         else
         {
-            dstReg1 = dstReg0 + (execSize + GENX_GRF_REG_SIZ - 1) / GENX_GRF_REG_SIZ - 1;
+            dstReg1 = dstReg0 + (opndSize + numEltPerGRF(Type_UB) - 1) / numEltPerGRF(Type_UB) - 1;
         }
     }
 
@@ -1143,15 +1155,20 @@ bool DDD::hasReadSuppression(G4_INST* prevInst, G4_INST* nextInst)
         }
     }
 
+    int  suppressionSrcs = 0;
     //For TGL, src0 support read suppresion as well
     if (kernel->fg.builder->hasSrc0ReadSuppression())
     {
         if (currInstRegs[0][0] == nextInstRegs[0][0] && currInstRegs[0][0] != -1 && curInstSimd8 && nextInstSimd8)
         {
             //No scalar supperssion
-            if (!isCurrScalar[0] && !isNextScalar[0])
+            if (!((!kernel->fg.builder->hasScalarReadSuppression()) && (isCurrScalar[0] || isNextScalar[0])))
             {
-                return true;
+                if (!multiSuppression)
+                {
+                    return true;
+                }
+                suppressionSrcs++;
             }
         }
     }
@@ -1162,22 +1179,30 @@ bool DDD::hasReadSuppression(G4_INST* prevInst, G4_INST* nextInst)
         ((curInstSimd8 && nextInstSimd8) || (!curInstSimd8 && !nextInstSimd8)))
     {
         //No scalar supperssion
-        if (!isCurrScalar[1] && !isNextScalar[1])
+        if (!((!kernel->fg.builder->hasScalarReadSuppression()) && (isCurrScalar[1] || isNextScalar[1])))
         {
-            return true;
+            if (!multiSuppression)
+            {
+                return true;
+            }
+            suppressionSrcs++;
         }
     }
 
     if (currInstRegs[0][2] == nextInstRegs[0][2] && currInstRegs[0][2] != -1 && curInstSimd8 && nextInstSimd8)
     {
         //No scalar supperssion
-        if (!isCurrScalar[2] && !isNextScalar[2])
+        if (!((!kernel->fg.builder->hasScalarReadSuppression()) && (isCurrScalar[2] || isNextScalar[2])))
         {
-            return true;
+            if (!multiSuppression)
+            {
+                return true;
+            }
+            suppressionSrcs++;
         }
     }
 
-    return false;
+    return suppressionSrcs > 1;
 }
 
 
@@ -1198,6 +1223,7 @@ DDD::DDD(Mem_Manager& m, G4_BB* bb, const LatencyTable& lt, G4_Kernel* k)
     HWthreadsPerEU = k->getNumThreads();
     useMTLatencies = getBuilder()->useMultiThreadLatency();
     totalGRFNum = kernel->getNumRegTotal();
+    isThreeSouceBlock = false;
     bool BTIIsRestrict = getOptions()->getOption(vISA_ReorderDPSendToDifferentBti);
 
     GRF_BUCKET = 0;
@@ -1217,6 +1243,7 @@ DDD::DDD(Mem_Manager& m, G4_BB* bb, const LatencyTable& lt, G4_Kernel* k)
     std::list<G4_INST*>::reverse_iterator iInst(bb->rbegin()), iInstEnd(bb->rend());
     std::vector<BucketDescr> BDvec;
 
+    int threeSrcInstNUm = 0;
     for (int nodeId = (int)(bb->size() - 1); iInst != iInstEnd; ++iInst, nodeId--)
     {
         Node *node = nullptr;
@@ -1227,6 +1254,10 @@ DDD::DDD(Mem_Manager& m, G4_BB* bb, const LatencyTable& lt, G4_Kernel* k)
         bool hasIndir = false;
         BDvec.clear();
 
+        if (curInst->getNumSrc() == 3)
+        {
+            threeSrcInstNUm ++;
+        }
         if (getBuilder()->hasReadSuppression() &&
             getOptions()->getOption(vISA_EnableGroupScheduleForBC))
         {
@@ -1387,6 +1418,11 @@ DDD::DDD(Mem_Manager& m, G4_BB* bb, const LatencyTable& lt, G4_Kernel* k)
         // Insert this node into the graph.
         InsertNode(node);
     }
+
+    if (Nodes.size())
+    {
+        isThreeSouceBlock = ((float)threeSrcInstNUm / Nodes.size()) > THREE_SOURCE_BLOCK_HERISTIC;
+    }
 }
 
 void Node::deletePred(Node* pred)
@@ -1510,7 +1546,7 @@ static bool canAvoidDepCycles(Node *firstNode, Node *secondNode, bool isFirstLev
 // Return TRUE if INST is the the partN'th part {0,1,2,3} of a typedWrite
 static bool isTypedWritePart(G4_INST* inst, int partN) {
     return inst->isSend()
-        && inst->getExecSize() == 8
+        && inst->getExecSize() == g4::SIMD8
         && inst->getMsgDesc()->isHdcTypedSurfaceWrite()
         && inst->getMaskOffset() == partN * 8;
 };
@@ -1923,12 +1959,187 @@ struct criticalCmp
             else
             {
                 return (*n1->getInstructions()).front()->getLocalId()
-                     > (*n2->getInstructions()).front()->getLocalId();
+            > (*n2->getInstructions()).front()->getLocalId();
             }
         }
     }
 };
 
+struct criticalCmpForMad
+{
+    criticalCmpForMad() = default;
+    bool operator()(const Node* n1, const Node* n2)
+    {
+        return (n1->getInstructions()->front()->getCISAOff() > n2->getInstructions()->front()->getCISAOff());
+    }
+};
+
+// 1).The priority queue is ordered as original sequence order.
+// 2).If there is a mad instruction be scheduled, trying to search the candidate which has read suppression in src1and src2.
+// 3).The scheduling is only applied to the BB whose instructions are mostly mad.
+// 4).This scheduling is not for general instruction scheduling, it's controlled by option vISA_ScheduleForReadSuppression
+uint32_t DDD::listScheduleForSuppression(G4_BB_Schedule* schedule)
+{
+    if (getOptions()->getOption(vISA_DumpDagDot))
+    {
+        dumpDagDot(schedule->getBB());
+        dumpNodes(schedule->getBB());
+    }
+
+    // All nodes in root set have no dependency
+    // so they can be immediately scheduled,
+    // and hence are added to readyList.
+    std::priority_queue<Node*, std::vector<Node*>, criticalCmpForMad> readyList;
+
+    // Nodes with their predecessors scheduled are pushed into the preReadyQueue
+    // They only get pushed into the real readyList if they are ready to issue,
+    // that is their earliest cycle is >= than the current schedule cycle.
+    std::priority_queue<Node*, std::vector<Node*>, earlyCmp> preReadyQueue;
+
+    collectRoots();
+    for (auto N : Roots) {
+        preReadyQueue.push(N);
+    }
+
+    // The scheduler's clock.
+    // This counter is updated in each loop iteration and its
+    // final value represents number of cycles the kernel would
+    // take to execute on the GPU as per the model.
+    uint32_t currCycle = 0;
+
+    // Used to avoid WAW subreg hazards
+    Node* lastScheduled = nullptr;
+
+    // Keep scheduling until both readyList or preReadyQueue contain instrs.
+    while (!(readyList.empty() && preReadyQueue.empty()))
+    {
+        Node* readyCandidate = preReadyQueue.empty() ? nullptr : preReadyQueue.top();
+        // While non empty, move nodes from preReadyQueue to readyList
+        while (readyCandidate)
+        {
+            readyList.push(readyCandidate);
+            preReadyQueue.pop();
+            readyCandidate = preReadyQueue.empty() ? nullptr : preReadyQueue.top();
+        }
+        // Nothing to issue at this cycle, increment scheduler clock
+        if (readyList.empty())
+        {
+            // readyCandidate is not nullptr. Proof: If it was nullptr, then both
+            // preReadyQueue and readyList would be empty. But this cannot
+            // happen because the while() loop will only enter if at least
+            // one of them is non-empty.
+            assert(readyCandidate && "Both readyList and preReadyQ empty!");
+            currCycle = readyCandidate->earliest;
+            continue;
+        }
+
+        // Pointer to node to be scheduled.
+        Node* scheduled = readyList.top();
+        readyList.pop();
+        if (lastScheduled &&
+            kernel->fg.builder->hasReadSuppression() &&
+            (readyList.size() > 0))
+        {
+            if (lastScheduled->getInstructions()->size() == 1)
+            {
+                G4_INST* inst = lastScheduled->getInstructions()->front();
+                if (inst->opcode() == G4_mad ||
+                    inst->opcode() == G4_dp4a)
+                {
+                    std::vector<Node*> popped;
+                    const int searchSize = (int)readyList.size();
+
+                    G4_INST* scheduledInst = scheduled->getInstructions()->front();
+                    if (!((scheduledInst->opcode() == G4_mad ||
+                        scheduledInst->opcode() == G4_dp4a) &&
+                        hasReadSuppression(inst, scheduledInst, inst->getExecSize() == g4::SIMD8)))
+                    {
+                        for (int i = 0; i < searchSize; ++i)
+                        {
+                            Node* next = readyList.top();
+                            if ((next->getInstructions()->size() == 1))
+                            {
+                                G4_INST* nextInst = next->getInstructions()->front();
+                                readyList.pop();
+                                if ((nextInst->opcode() == G4_mad ||
+                                    nextInst->opcode() == G4_dp4a) &&
+                                    hasReadSuppression(inst, nextInst, inst->getExecSize() == g4::SIMD8))
+                                {
+                                    readyList.push(scheduled);
+                                    scheduled = next;
+                                    break;
+                                }
+                                else
+                                {
+                                    popped.push_back(next);
+                                }
+                            }
+                        }
+
+                        for (auto nodes : popped)
+                        {
+                            readyList.push(nodes);
+                        }
+                    }
+                }
+            }
+        }
+
+        assert(scheduled && "Must have found an instruction to schedule by now");
+
+        // Append the scheduled node to the end of the schedule.
+        schedule->scheduledNodes.push_back(scheduled);
+        lastScheduled = scheduled;
+
+        // Set the cycle at which this node is scheduled.
+        scheduled->schedTime = currCycle;
+
+        for (auto& curSucc : scheduled->succs)
+        {
+            Node* succ = curSucc.getNode();
+            // Recompute the earliest time for each successor.
+            if (scheduled->isLabel())
+            {
+                succ->earliest = 0;
+            }
+            else
+            {
+                // Update the earliest time of the successor and set its last scheduled
+                // predecessor with the largest latency to the currently scheduled node
+                // if the latency incurred by scheduling the successor right after the
+                // "scheduled" node is larger than successor's earliest time.
+                uint32_t latencyToAdd = 0;
+                uint32_t earliestNew = 0;
+                latencyToAdd = curSucc.getLatency() > scheduled->getOccupancy() ? curSucc.getLatency() : scheduled->getOccupancy();
+                earliestNew = scheduled->schedTime + latencyToAdd;
+
+                if (succ->earliest <= earliestNew || !succ->lastSchedPred)
+                {
+                    succ->lastSchedPred = scheduled;
+                }
+                succ->earliest = (succ->earliest > earliestNew) ? succ->earliest : earliestNew;
+            }
+            // Decrease the number of predecessors not scheduled for the successor node.
+            if ((--(succ->predsNotScheduled)) == 0)
+            {
+                preReadyQueue.push(succ);
+            }
+        }
+
+        // Increment the scheduler's clock after each scheduled node
+        currCycle += scheduled->getOccupancy();
+    }
+
+    // Sanity check: Make sure all nodes are scheduled
+#if defined(_DEBUG)
+    for (auto node : allNodes)
+    {
+        assert(node->schedTime != UINT_MAX && "Node not scheduled!");
+    }
+#endif
+    return currCycle;
+}
+
 // Perform local list scheduling
 uint32_t DDD::listSchedule(G4_BB_Schedule *schedule)
 {
@@ -1988,6 +2199,81 @@ uint32_t DDD::listSchedule(G4_BB_Schedule *schedule)
         // Pointer to node to be scheduled.
         Node *scheduled = readyList.top();
         readyList.pop();
+        if (lastScheduled &&
+            kernel->fg.builder->hasReadSuppression() &&
+            getOptions()->getuInt32Option(vISA_ReadSuppressionDepth) != 0 &&
+            (readyList.size() > 0))
+        {
+            if (lastScheduled->getInstructions()->size() == 1)
+            {
+                G4_INST* inst = lastScheduled->getInstructions()->front();
+                if (inst->opcode() == G4_mad ||
+                    inst->opcode() == G4_dp4a)
+                {
+                    std::vector<Node*> popped;
+                    const int searchSize = std::min((int)getOptions()->getuInt32Option(vISA_ReadSuppressionDepth), (int)readyList.size());
+                    for (int i = 0; i < searchSize; ++i)
+                    {
+                        Node* next = readyList.top();
+                        if ((next->getInstructions()->size() == 1))
+                        {
+                            G4_INST* nextInst = next->getInstructions()->front();
+                            readyList.pop();
+                            if ((nextInst->opcode() == G4_mad ||
+                                nextInst->opcode() == G4_dp4a) &&
+                                hasReadSuppression(inst, nextInst, false))
+                            {
+                                readyList.push(scheduled);
+                                scheduled = next;
+                                break;
+                            }
+                            else
+                            {
+                                popped.push_back(next);
+                            }
+                        }
+                    }
+                    for (auto nodes : popped)
+                    {
+                        readyList.push(nodes);
+                    }
+                }
+            }
+        }
+        // try to avoid bank conflict for Gen12
+        // Such as in following case:
+        // r40 and r56 are from same bundle and have conflict
+        // scheduling next instruction up can avoid the conflit.
+        //mad r10   r20  r33 r40        mad r10   r20  r33 r40
+        //mad r12   r56  r61  r70  -->  mad r14   r58 r 63  r72
+        //mad r14   r58 r 63  r72       mad r12   r56  r61  r70
+        else if (kernel->fg.builder->hasEarlyGRFRead() &&
+            lastScheduled && lastScheduled->hasConflict(scheduled) &&
+            (readyList.size() > 0))
+        {
+            std::vector<Node*> popped;
+            const int searchSize = std::min(3, (int)readyList.size());
+            for (int i = 0; i < searchSize; ++i)
+            {
+                Node* next = readyList.top();
+                readyList.pop();
+                if (!lastScheduled->hasConflict(next))
+                {
+                    readyList.push(scheduled);
+                    scheduled = next;
+                    break;
+                }
+                else
+                {
+                    popped.push_back(next);
+                }
+            }
+            for (auto nodes : popped)
+            {
+                readyList.push(nodes);
+            }
+        }
+        else
 
         // try to avoid b2b math if possible as there are pipeline stalls
         if (scheduled->getInstructions()->front()->isMath() &&
@@ -2296,3 +2582,166 @@ void G4_BB_Schedule::emit(std::ostream &out) {
     }
 }
 
+//Check conflicts according to registers read in same cycle
+//Two kinds conflict:
+//1. multiple registers from same bundle
+//2. all three souces from same bank
+//Input: regCandidates are the registers which will be read in same cycle
+static bool hasConflictForSchedule(int *regCandidates)
+{
+    int bundles[3];
+    int bankSrcs[2];
+
+    for (int i = 0; i < 2; i++)
+    {
+        bankSrcs[i] = 0;
+    }
+    for (int i = 0; i < 3; i++)
+    {
+        bundles[i] = -1;
+    }
+
+    for (int i = 0; i < 3; i++)
+    {
+        if (regCandidates[i] != -1)
+        {
+            int bundleID = regCandidates[i] % 16;  //8 bundles for each bank
+            int bankID = regCandidates[i] % 2;
+
+            bundles[i] = bundleID;
+            bankSrcs[bankID]++;
+        }
+    }
+
+    //If there is bundle conflict between the sources from different instructions
+    //Note that the instruction internal conflict is not considered here.
+    if (bundles[2] != -1 &&
+        (bundles[2] == bundles[0] ||
+        bundles[2] == bundles[1]))
+    {
+        return true;
+    }
+
+    //If all three sources are from same bank
+    if ((bankSrcs[0] > 2 ||
+        bankSrcs[1] > 2))
+    {
+        return true;
+    }
+
+    return false;
+}
+
+//Gen12 check BC between adjacent instructions (this, node2)
+//Scheduling can only solve the bank conflict between adjacent instructions
+//The src0 of second instruction is read together with src1 (and src2) of the first instruction.
+//We only handle the situation that first instruction is a three source instruction.
+bool Node::hasConflict(Node* node2)
+{
+    G4_INST *inst1 = getInstructions()->front();
+    G4_INST *inst2 = node2->getInstructions()->front();
+
+    if (inst1->isSend() || inst2->isSend())
+    {
+        return false;
+    }
+
+    //Don't consider two source instructions
+    if (inst1->getNumSrc() != 3)
+    {
+        return false;
+    }
+
+    int prevInstRegs[2][G4_MAX_SRCS];
+    int currInstReg;
+    int prevInstExecSize[G4_MAX_SRCS];
+    int firstRegCandidate[G4_MAX_SRCS];
+    int candidateNum = 0;
+
+    //Get the sources of current instruction
+    bool instSplit = false;
+    for (int i = 0; i < inst1->getNumSrc(); i++)
+    {
+        prevInstRegs[0][i] = -1;
+        prevInstRegs[1][i] = -1;
+        G4_Operand* srcOpnd = inst1->getSrc(i);
+        if (srcOpnd)
+        {
+            if (srcOpnd->isSrcRegRegion() &&
+                srcOpnd->asSrcRegRegion()->getBase() &&
+                srcOpnd->asSrcRegRegion()->getBase()->isRegVar())
+            {
+                G4_RegVar* baseVar = static_cast<G4_RegVar*>(srcOpnd->asSrcRegRegion()->getBase());
+                prevInstExecSize[i] = srcOpnd->getLinearizedEnd() - srcOpnd->getLinearizedStart() + 1;
+                if (baseVar->isGreg()) {
+                    uint32_t byteAddress = srcOpnd->getLinearizedStart();
+                    prevInstRegs[0][i] = byteAddress / numEltPerGRF(Type_UB);
+
+                    if (prevInstExecSize[i] > 32)
+                    {
+                        prevInstRegs[1][i] = prevInstRegs[0][i] + (prevInstExecSize[i] + numEltPerGRF(Type_UB) - 1) / numEltPerGRF(Type_UB) - 1;
+                        instSplit = true;
+                    }
+                    else
+                    {
+                        //The same regsiter is reused in both SIMD8 instructions
+                        prevInstRegs[1][i] = prevInstRegs[0][i];
+                    }
+                }
+            }
+        }
+    }
+
+    if (!instSplit)
+    {
+        if (prevInstRegs[0][1] != -1)
+        {
+            firstRegCandidate[candidateNum] = prevInstRegs[0][1];
+            candidateNum++;
+        }
+        if (prevInstRegs[0][2] != -1)
+        {
+            firstRegCandidate[candidateNum] = prevInstRegs[0][2];
+            candidateNum++;
+        }
+    }
+    else    //For SIMD16 and SIMD32, if the GRF1 of src1 or src2 of inst 1 is GRF regsiter
+    {
+        if (prevInstRegs[1][1] != -1)
+        {
+            firstRegCandidate[candidateNum] = prevInstRegs[1][1];
+            candidateNum++;
+        }
+        if (prevInstRegs[1][2] != -1)
+        {
+            firstRegCandidate[candidateNum] = prevInstRegs[1][2];
+            candidateNum++;
+        }
+    }
+
+    //Get the src0 of the second instruction node2
+    currInstReg = -1;
+    G4_Operand * srcOpnd = inst2->getSrc(0);
+    if (srcOpnd)
+    {
+        if (srcOpnd->isSrcRegRegion() &&
+            srcOpnd->asSrcRegRegion()->getBase() &&
+            srcOpnd->asSrcRegRegion()->getBase()->isRegVar())
+        {
+            G4_RegVar* baseVar = static_cast<G4_RegVar*>(srcOpnd->asSrcRegRegion()->getBase());
+            if (baseVar->isGreg()) {
+                uint32_t byteAddress = srcOpnd->getLinearizedStart();
+                firstRegCandidate[candidateNum] = byteAddress / numEltPerGRF(Type_UB);
+                candidateNum++;
+            }
+        }
+    }
+    else
+    {
+        return false;
+    }
+
+    return hasConflictForSchedule(firstRegCandidate);
+}
+
+
