@@ -40,14 +40,7 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "LatencyTable.h"
 #include "Dependencies_G4IR.h"
 
-//To be comptabile with send cycles, don't normalized them to 1
-#define UNCOMPR_LATENCY  2    // Latency of an uncompressed instruction
-#define COMPR_LATENCY    4    // Latency of a compressed instruction
-#define ACC_BUBBLE       4    // Accumulator back-to-back stall
-#define IVB_PIPELINE_LENGTH  14
-#define EDGE_LATENCY_MATH 22
-#define EDGE_LATENCY_MATH_TYPE2 30
-#define EDGE_LATENCY_SEND_WAR 36
+#define THREE_SOURCE_BLOCK_HERISTIC 0.5
 
 namespace vISA {
 
@@ -161,6 +154,8 @@ public:
     void addPairInstr(G4_INST *inst) { instVec.push_back(inst); }
     void clear() { instVec.clear(); }
     void deletePred(Node *pred);
+    //Gen12 check BC between adjacent instructions
+    bool hasConflict(Node* node2);
 
     friend class DDD;
     friend class G4_BB_Schedule;
@@ -184,8 +179,8 @@ struct Mask {
     }
 
     bool killsBucket(unsigned int bucket) const {
-        unsigned short bucketLB = bucket * G4_GRF_REG_NBYTES;
-        unsigned short bucketRB = (bucket + 1) * G4_GRF_REG_NBYTES - 1;
+        unsigned short bucketLB = bucket * numEltPerGRF(Type_UB);
+        unsigned short bucketRB = (bucket + 1) * numEltPerGRF(Type_UB) - 1;
         return !nonContiguousStride && !withSpecialAcc() &&
                LeftB <= bucketLB && RightB >= bucketRB;
     }
@@ -249,6 +244,7 @@ class DDD {
     Edge_Allocator depEdgeAllocator;
     int HWthreadsPerEU;
     bool useMTLatencies;
+    bool isThreeSouceBlock;
     const LatencyTable LT;
 
     int GRF_BUCKET;
@@ -274,7 +270,7 @@ public:
     void pairTypedWriteOrURBWriteNodes(G4_BB *bb);
 
     bool hasReadSuppression(G4_INST *curInst, G4_INST *nextInst, BitSet &liveDst, BitSet &liveSrc);
-    bool hasReadSuppression(G4_INST* prevInst, G4_INST* nextInst);
+    bool hasReadSuppression(G4_INST* prevInst, G4_INST* nextInst, bool multipSuppression);
 
     DDD(Mem_Manager& m, G4_BB* bb, const LatencyTable& lt, G4_Kernel* k);
     ~DDD()
@@ -293,6 +289,7 @@ public:
     void InsertNode(Node *node) { Nodes.push_back(node); }
     void dumpNodes(G4_BB *bb);
     void dumpDagDot(G4_BB *bb);
+    uint32_t listScheduleForSuppression(G4_BB_Schedule* schedule);
     uint32_t listSchedule(G4_BB_Schedule*);
     void setPriority(Node *pred, const Edge &edge);
     void createAddEdge(Node* pred, Node* succ, DepType d);
@@ -310,6 +307,7 @@ public:
     Mem_Manager* get_mem() { return &mem; }
     IR_Builder* getBuilder() const { return kernel->fg.builder; }
     const Options* getOptions() const { return kernel->getOptions(); }
+    bool getIsThreeSourceBlock() { return isThreeSouceBlock; }
 };
 
 class G4_BB_Schedule {
