@@ -136,6 +136,23 @@ namespace vISA
 
         void setOffset(unsigned short o) { offset = o; }
 
+        bool hasOverlap(SBFootprint* liveFootprint) const
+        {
+            SBFootprint* curFootprint2Ptr = liveFootprint;
+            while (curFootprint2Ptr)
+            {
+                // Negative of no overlap: !(LeftB > curFootprint2Ptr->RightB || RightB < curFootprint2Ptr->LeftB)
+                if (fType == curFootprint2Ptr->fType &&
+                    LeftB <= curFootprint2Ptr->RightB && RightB >= curFootprint2Ptr->LeftB)
+                {
+                    return true;
+                }
+                curFootprint2Ptr = curFootprint2Ptr->next;
+            }
+
+            return false;
+        }
+
         bool hasOverlap(SBFootprint *liveFootprint, unsigned short &internalOffset) const
         {
             SBFootprint *curFootprint2Ptr = liveFootprint;
@@ -178,8 +195,8 @@ namespace vISA
             while (curFootprint2Ptr)
             {
                 if (fType == curFootprint2Ptr->fType &&
-                    ((LeftB / G4_GRF_REG_NBYTES) <= (curFootprint2Ptr->RightB / G4_GRF_REG_NBYTES)) &&
-                    ((RightB / G4_GRF_REG_NBYTES) >= (curFootprint2Ptr->LeftB / G4_GRF_REG_NBYTES)))
+                    ((LeftB / numEltPerGRF(Type_UB)) <= (curFootprint2Ptr->RightB / numEltPerGRF(Type_UB))) &&
+                    ((RightB / numEltPerGRF(Type_UB)) >= (curFootprint2Ptr->LeftB / numEltPerGRF(Type_UB))))
                 {
                     return true;
                 }
@@ -194,8 +211,8 @@ namespace vISA
                 while (curFootprint2Ptr)
                 {
                     if (fType == curFootprint2Ptr->fType &&
-                        ((curFootprintPtr->LeftB  / G4_GRF_REG_NBYTES) <= (curFootprint2Ptr->RightB  / G4_GRF_REG_NBYTES)) &&
-                        ((curFootprintPtr->RightB  / G4_GRF_REG_NBYTES) >= (curFootprint2Ptr->LeftB  / G4_GRF_REG_NBYTES)))
+                        ((curFootprintPtr->LeftB  / numEltPerGRF(Type_UB)) <= (curFootprint2Ptr->RightB  / numEltPerGRF(Type_UB))) &&
+                        ((curFootprintPtr->RightB  / numEltPerGRF(Type_UB)) >= (curFootprint2Ptr->LeftB  / numEltPerGRF(Type_UB))))
                     {
                         return true;
                     }
@@ -802,8 +819,8 @@ namespace vISA
             SBFootprint *footprint = bucketNode->node->getFirstFootprint(bucketNode->opndNum);
             while (footprint)
             {
-                unsigned int startBucket = footprint->LeftB / G4_GRF_REG_NBYTES;
-                unsigned int endBucket = footprint->RightB / G4_GRF_REG_NBYTES;
+                unsigned int startBucket = footprint->LeftB / numEltPerGRF(Type_UB);
+                unsigned int endBucket = footprint->RightB / numEltPerGRF(Type_UB);
                 if (footprint->fType == ACC_T)
                 {
                     startBucket = startBucket + aregOffset;
@@ -984,14 +1001,11 @@ namespace vISA
         SBFootprint* getFootprintForFlag(G4_Operand* opnd,
             Gen4_Operand_Number opnd_num,
             G4_INST* inst);
-        void getGRFBuckets(SBNode *node,
-            SBFootprint* footprint,
-            Gen4_Operand_Number opndNum,
-            std::vector<SBBucketDescr>& BDvec);
         bool getFootprintForOperand(SBNode *node,
             G4_INST *inst,
             G4_Operand* opnd,
             Gen4_Operand_Number opnd_num);
+        void getGRFBuckets(SBNode* node, SBFootprint* footprint, Gen4_Operand_Number opndNum, std::vector<SBBucketDescr>& BDvec, bool GRFOnly);
         bool getGRFFootPrintOperands(SBNode *node,
             G4_INST *inst,
             Gen4_Operand_Number first_opnd,
@@ -1014,9 +1028,11 @@ namespace vISA
             std::vector<SBBucketDescr>& BDvec,
             bool GRFOnly);
 
-        void setDistance(SBFootprint * footprint, SBNode *node, SBNode *liveNode);
+        void setDistance(SBFootprint * footprint, SBNode *node, SBNode *liveNode, bool dstDep);
         void footprintMerge(SBNode * node, SBNode * nextNode);
 
+        void pushItemToQueue(std::vector<unsigned>* nodeIDQueue, unsigned nodeID);
+
         //Local distance dependence analysis and assignment
         void SBDDD(G4_BB* bb,
             LiveGRFBuckets* &LB,
@@ -1054,8 +1070,6 @@ namespace vISA
 
     typedef std::vector<G4_BB_SB *> BB_SWSB_VECTOR;
     typedef BB_SWSB_VECTOR::iterator BB_SWSB_VECTOR_ITER;
-    typedef std::vector<SWSB_LOOP> LOOP_SWSB_VECTOR;
-    typedef LOOP_SWSB_VECTOR::iterator LOOP_SWSB_VECTOR_ITER;
 
     class Dom
     {
@@ -1087,13 +1101,93 @@ namespace vISA
         void updateImmDom();
     };
 
+    class SWSB_TOKEN_PROFILE {
+        uint32_t tokenInstructionCount;
+        uint32_t tokenReuseCount;
+        uint32_t AWTokenReuseCount;
+        uint32_t ARTokenReuseCount;
+        uint32_t AATokenReuseCount;
+        uint32_t mathInstCount;
+        uint32_t syncInstCount;
+        uint32_t mathReuseCount;
+        uint32_t ARSyncInstCount;
+        uint32_t AWSyncInstCount;
+        uint32_t ARSyncAllCount;
+        uint32_t AWSyncAllCount;
+        uint32_t prunedDepEdges;
+        uint32_t prunedGlobalEdgeNum;
+        uint32_t prunedDiffBBEdgeNum;
+        uint32_t prunedDiffBBSameTokenEdgeNum;
+    public:
+        SWSB_TOKEN_PROFILE()
+            : tokenInstructionCount(0), tokenReuseCount(0),
+            AWTokenReuseCount(0), ARTokenReuseCount(0), AATokenReuseCount(0),
+            mathInstCount(0), syncInstCount(0), mathReuseCount(0),
+            ARSyncInstCount(0), AWSyncInstCount(0), ARSyncAllCount(0), AWSyncAllCount(0),
+            prunedDepEdges(0), prunedGlobalEdgeNum(0), prunedDiffBBEdgeNum(0), prunedDiffBBSameTokenEdgeNum(0) {
+            ;
+        }
+
+        ~SWSB_TOKEN_PROFILE()
+        {
+        }
+        void* operator new(size_t sz, vISA::Mem_Manager& m) { return m.alloc(sz); }
+
+        void setTokenInstructionCount(int count) { tokenInstructionCount = count; }
+        uint32_t getTokenInstructionCount() { return tokenInstructionCount; }
+
+        void setTokenReuseCount(int count) { tokenReuseCount = count; }
+        uint32_t getTokenReuseCount() { return tokenReuseCount; }
+
+        void setAWTokenReuseCount(int count) { AWTokenReuseCount = count; }
+        uint32_t getAWTokenReuseCount() { return AWTokenReuseCount; }
+
+        void setARTokenReuseCount(int count) { ARTokenReuseCount = count; }
+        uint32_t getARTokenReuseCount() { return ARTokenReuseCount; }
+
+        void setAATokenReuseCount(int count) { AATokenReuseCount = count; }
+        uint32_t getAATokenReuseCount() { return AATokenReuseCount; }
+
+        void setMathInstCount(int count) { mathInstCount = count; }
+        uint32_t getMathInstCount() { return mathInstCount; }
+
+        void setSyncInstCount(int count) { syncInstCount = count; }
+        uint32_t getSyncInstCount() { return syncInstCount; }
+
+        void setMathReuseCount(int count) { mathReuseCount = count; }
+        uint32_t getMathReuseCount() { return mathReuseCount; }
+
+        void setARSyncInstCount(int count) { ARSyncInstCount = count; }
+        uint32_t getARSyncInstCount() { return ARSyncInstCount; }
+
+        void setAWSyncInstCount(int count) { AWSyncInstCount = count; }
+        uint32_t getAWSyncInstCount() { return AWSyncInstCount; }
+
+        void setARSyncAllCount(int count) { ARSyncAllCount = count; }
+        uint32_t getARSyncAllCount() { return ARSyncAllCount; }
+
+        void setAWSyncAllCount(int count) { AWSyncAllCount = count; }
+        uint32_t getAWSyncAllCount() { return AWSyncAllCount; }
+
+        void setPrunedEdgeNum(int num) { prunedDepEdges = num; }
+        uint32_t getPrunedEdgeNum() { return prunedDepEdges; }
+
+        void setPrunedGlobalEdgeNum(int num) { prunedGlobalEdgeNum = num; }
+        uint32_t getPrunedGlobalEdgeNum() { return prunedGlobalEdgeNum; }
+
+        void setPrunedDiffBBEdgeNum(int num) { prunedDiffBBEdgeNum = num; }
+        uint32_t getPrunedDiffBBEdgeNum() { return prunedDiffBBEdgeNum; }
+
+        void setPrunedDiffBBSameTokenEdgeNum(int num) { prunedDiffBBSameTokenEdgeNum = num; }
+        uint32_t getPrunedDiffBBSameTokenEdgeNum() { return prunedDiffBBSameTokenEdgeNum; }
+    };
+
     class SWSB {
         G4_Kernel &kernel;
         FlowGraph&  fg;
         vISA::Mem_Manager& mem;
 
         BB_SWSB_VECTOR BBVector;    // The basic block vector, ordered with ID of the BB
-        LOOP_SWSB_VECTOR loopVector;
         SBNODE_VECT SBNodes;        // All instruction nodes
         SBNODE_VECT SBSendNodes;    // All out-of-order instruction nodes
         SBNODE_VECT SBSendUses;    // All out-of-order instruction nodes
@@ -1128,6 +1222,7 @@ namespace vISA
 
         std::map<G4_Label*, G4_BB_SB*> labelToBlockMap;
         BitSet   **allTokenNodesMap;
+        SWSB_TOKEN_PROFILE* tokenProfile;
 
         //Global dependence analysis
         bool globalDependenceDefReachAnalysis(G4_BB* bb);
@@ -1142,6 +1237,8 @@ namespace vISA
 
         void setDefaultDistanceAtFirstInstruction();
 
+        void quickTokenAllocation();
+
         //Token allocation
         void tokenAllocation();
         void buildLiveIntervals();
@@ -1154,7 +1251,6 @@ namespace vISA
         void addGlobalDependenceWithReachingDef(unsigned globalSendNum, SBBUCKET_VECTOR* globalSendOpndList, SBNODE_VECT* SBNodes, PointsToAnalysis& p, bool afterWrite);
         void expireLocalIntervals(unsigned startID, unsigned BBID);
         void assignTokenToPred(SBNode* node, SBNode* pred, G4_BB* bb);
-        void assignTokenToSucc(SBNode* node, G4_BB* bb);
         bool assignTokenWithPred(SBNode* node, G4_BB* bb);
         void allocateToken(G4_BB* bb);
         void tokenAllocationBB(G4_BB* bb);
@@ -1170,7 +1266,6 @@ namespace vISA
         //Assign Token
         void assignToken(SBNode *node, unsigned short token, uint32_t &AWTokenReuseCount, uint32_t &ARTokenReuseCount, uint32_t &AATokenReuseCount);
         void assignDepToken(SBNode *node);
-        bool insertSyncToken(G4_BB *bb, SBNode *node, G4_INST *inst, INST_LIST_ITER inst_it, int newInstID, BitSet *dstTokens, BitSet *srcTokens, bool removeAllTokens);
         void insertSync(G4_BB* bb, SBNode* node, G4_INST* inst, INST_LIST_ITER inst_it, int newInstID, BitSet* dstTokens, BitSet* srcTokens);
         void insertTest();
 
@@ -1181,7 +1276,10 @@ namespace vISA
         G4_INST* insertSyncAllRDInstruction(G4_BB *bb, unsigned int SBIDs, INST_LIST_ITER nextIter, int CISAOff, int lineNo);
         G4_INST *insertSyncAllWRInstruction(G4_BB *bb, unsigned int SBIDs, INST_LIST_ITER nextIter, int CISAOff, int lineNo);
 
+        bool insertSyncToken(G4_BB* bb, SBNode* node, G4_INST* inst, INST_LIST_ITER inst_it, int newInstID, BitSet* dstTokens, BitSet* srcTokens, bool& keepDst, bool removeAllToken);
+
         void SWSBDepDistanceGenerator(PointsToAnalysis& p, LiveGRFBuckets &LB, LiveGRFBuckets &globalSendsLB);
+        void handleIndirectCall();
         void SWSBGlobalTokenGenerator(PointsToAnalysis& p, LiveGRFBuckets &LB, LiveGRFBuckets &globalSendsLB);
         void SWSBBuildSIMDCFG();
         void addSIMDEdge(G4_BB_SB *pred, G4_BB_SB* succ);
