@@ -35,6 +35,7 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "Timer.h"
 #include "G4_Opcode.h"          // temporarily add to support G4_MAX_SRCS == 3
 #include "DebugInfo.h"
+#include "IGC/common/StringMacros.hpp"
 
 #ifdef _MSC_VER
 // MAX : Disable security enhancements warning (VC2005)
@@ -46,16 +47,6 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 // reset to the default value after the builder is destroyed.  We will move these options to be
 // part of the builder in the near future.
 
-void Options::setGTPin()
-{
-    // GTPin mode, we need to
-    // 1. reserve 16 GRF's worth of spill memory starting at 0
-    // 3. produce asm output
-    // spillMemOffset = 16 * G4_GRF_REG_NBYTES;
-    m_vISAOptions.setUint32(vISA_SpillMemOffset, (uint32_t) 16 * G4_GRF_REG_NBYTES);
-    m_vISAOptions.setBool(vISA_outputToFile, true);         //outputToFile = true;
-}
-
 bool Options::get_isaasm(int argc, const char *argv[])
 {
     return true;
@@ -87,9 +78,9 @@ bool Options::parseOptions(int argc, const char* argv[])
         argString << argv[i] << " ";
     }
 
-    for(int i = startPos; i < argc; i++)
+    for (int i = startPos; i < argc; i++)
     {
-        if( argv[i] == NULL )
+        if (argv[i] == NULL)
         {
             return false;
         }
@@ -105,7 +96,7 @@ bool Options::parseOptions(int argc, const char* argv[])
         // Else if int32,int64, or cstr, set parse argv[i+1] and set the value.
         vISAOptions vISAOpt = it->second;
         EntryType type = m_vISAOptions.getType(vISAOpt);
-        switch(type) {
+        switch (type) {
         case ET_BOOL: {
             bool val = ! m_vISAOptions.getDefaultBool(vISAOpt);
             m_vISAOptions.setBool(vISAOpt, val);
@@ -122,7 +113,7 @@ bool Options::parseOptions(int argc, const char* argv[])
                 std::cout << errorMsg << std::endl;
                 return false;
             }
-            switch(type) {
+            switch (type) {
             case ET_INT32:
                 m_vISAOptions.setUint32(vISAOpt, atoi(argv[i]));
                 m_vISAOptions.setArgSetByUser(vISAOpt);
@@ -193,15 +184,12 @@ bool Options::parseOptions(int argc, const char* argv[])
             return false;
         }
         int status = SetStepping(stepping);
-        if( status != 0 ){
+        if (status != 0) {
             std::cout << "unrecognized stepping string: "
                       << stepping << std::endl;
             return false;
         }
     }
-    if (m_vISAOptions.isArgSetByUser(vISA_Gtpin)) {
-        setGTPin();
-    }
     if (m_vISAOptions.isArgSetByUser(vISA_DumpPasses)) {
         m_vISAOptions.setBool(vISA_DumpDotAll, true);
     }
@@ -215,11 +203,6 @@ bool Options::parseOptions(int argc, const char* argv[])
         target = VISA_3D;
     }
 
-    if (m_vISAOptions.isArgSetByUser(vISA_SpillMemOffset)) {
-        assert((m_vISAOptions.getUint32(vISA_SpillMemOffset)
-                & (GENX_GRF_REG_SIZ - 1)) == 0
-               && "-spilloffset is mis-aligned");
-    }
     if (m_vISAOptions.isArgSetByUser(vISA_ReservedGRFNum)) {
         if (m_vISAOptions.getUint32(vISA_ReservedGRFNum)) {
             m_vISAOptions.setBool(vISA_LocalBankConflictReduction, false);
@@ -254,12 +237,19 @@ bool Options::parseOptions(int argc, const char* argv[])
         }
 
         int status = SetPlatform(platformStr);
-        if( status != 0 ){
+        if (status != 0) {
             std::cout << "unrecognized platform string: "
                       << platformStr << std::endl;
             return false;
         }
         m_vISAOptions.setBool(vISA_PlatformIsSet, true);   //platformIsSet = true;
+        if (GetStepping() == Step_A)
+        {
+            if (getGenxPlatform() == GENX_TGLLP)
+            {
+                m_vISAOptions.setBool(vISA_HasEarlyGRFRead, true);
+            }
+        }
     }
 
     if (m_vISAOptions.isArgSetByUser(vISA_GetvISABinaryName)) {
@@ -281,13 +271,10 @@ bool Options::parseOptions(int argc, const char* argv[])
         m_vISAOptions.setBool(vISA_DisableSpillCoalescing, true);
     }
 
-    if (getPlatformGeneration(getGenxPlatform()) >= PlatformGen::GEN12)
+    if (m_vISAOptions.isArgSetByUser(vISA_forceDebugSWSB))
     {
-        if (m_vISAOptions.isArgSetByUser(vISA_forceDebugSWSB))
-        {
-            m_vISAOptions.setUint32(vISA_SWSBInstStall, 0);
-            m_vISAOptions.setUint32(vISA_SWSBTokenBarrier, 0);
-        }
+        m_vISAOptions.setUint32(vISA_SWSBInstStall, 0);
+        m_vISAOptions.setUint32(vISA_SWSBTokenBarrier, 0);
     }
 
 #if (defined(_DEBUG) || defined(_INTERNAL))
@@ -330,7 +317,7 @@ void Options::showUsage(std::ostream& output)
 void Options::initialize_vISAOptionsToStr(void) {
    #undef DEF_VISA_OPTION
    #define DEF_VISA_OPTION(ENUM, TYPE, STR, ERROR_MSG, DEFAULT_VAL) \
-    vISAOptionsToStr[ENUM] = #ENUM;
+    vISAOptionsToStr[ENUM] = IGC_MANGLE(#ENUM);
    #include "VISAOptions.def"
 }
 
