@@ -5,112 +5,30 @@
 
 PreDefinedVarInfo preDefinedVarTable[static_cast<int>(PreDefinedVarsInternal::VAR_LAST)] =
 {
-    { PreDefinedVarsInternal::VAR_NULL,      ISA_TYPE_UD, 1, true,  false,  0, 1,   "null" },
-    { PreDefinedVarsInternal::X,             ISA_TYPE_UW, 1, true,  false,  4,  1,   "thread_x" },
-    { PreDefinedVarsInternal::Y,             ISA_TYPE_UW, 1, true,  false,  6,  1,   "thread_y" },
-    { PreDefinedVarsInternal::LOCAL_ID_X,    ISA_TYPE_UD, 2, false, true,   16, 1,   "local_id_x" },
-    { PreDefinedVarsInternal::LOCAL_ID_Y,    ISA_TYPE_UD, 2, false, true,   20, 1,   "local_id_y" },
-    { PreDefinedVarsInternal::LOCAL_SIZE_X,  ISA_TYPE_UD, 2, false, true,   0,  1,   "local_size_x" },
-    { PreDefinedVarsInternal::LOCAL_SIZE_Y,  ISA_TYPE_UD, 2, false, true,   4,  1,   "local_size_y" },
-    { PreDefinedVarsInternal::GROUP_ID_X,    ISA_TYPE_UD, 2, true,  true,   4,  1,   "group_id_x" },
-    { PreDefinedVarsInternal::GROUP_ID_Y,    ISA_TYPE_UD, 2, true,  true,   24, 1,   "group_id_y" },
-    { PreDefinedVarsInternal::GROUP_ID_Z,    ISA_TYPE_UD, 3, true,  true,   28, 1,   "group_id_z" },
-    { PreDefinedVarsInternal::GROUP_COUNT_X, ISA_TYPE_UD, 2, false, true,   8,  1,   "group_count_x" },
-    { PreDefinedVarsInternal::GROUP_COUNT_Y, ISA_TYPE_UD, 2, false, true,   12, 1,   "group_count_y" },
-    { PreDefinedVarsInternal::TSC,           ISA_TYPE_UD, 2, false, false,  0,  5,   "tsc" },
-    { PreDefinedVarsInternal::R0,            ISA_TYPE_UD, 2, true,  false,  0,  8,   "r0" },
-    { PreDefinedVarsInternal::ARG,           ISA_TYPE_UD, 3, false, false,  0,  256, "arg" },
-    { PreDefinedVarsInternal::RET,           ISA_TYPE_UD, 3, false, false,  0,  96,  "retval" },
-    { PreDefinedVarsInternal::FE_SP,         ISA_TYPE_UD, 3, false, false,  0,  1,   "sp" },
-    { PreDefinedVarsInternal::FE_FP,         ISA_TYPE_UD, 3, false, false,  0,  1,   "fp" },
-    { PreDefinedVarsInternal::HW_TID,        ISA_TYPE_UD, 3, false, false,  0,  1,   "hw_id" },
-    { PreDefinedVarsInternal::SR0,           ISA_TYPE_UD, 3, false, false,  0,  4,   "sr0" },
-    { PreDefinedVarsInternal::CR0,           ISA_TYPE_UD, 3, false, false,  0,  1,   "cr0" },
-    { PreDefinedVarsInternal::CE0,           ISA_TYPE_UD, 3, false, false,  0,  1,   "ce0" },
-    { PreDefinedVarsInternal::DBG,           ISA_TYPE_UD, 3, false, false,  0,  2,   "dbg0" },
-    { PreDefinedVarsInternal::COLOR,         ISA_TYPE_UW, 3, true,  false,  0,  1,   "color" },
-};
-
-enum class PreDefinedVarsInternal_3_1
-{
-    VAR_NULL = 0,
-    X = 1,
-    Y = 2,
-    LOCAL_ID_X = 3,
-    LOCAL_ID_Y = 4,
-    LOCAL_SIZE_X = 5,
-    LOCAL_SIZE_Y = 6,
-    GROUP_ID_X = 7,
-    GROUP_ID_Y = 8,
-    GROUP_COUNT_X = 9,
-    GROUP_COUNT_Y = 10,
-    TSC = 11,
-    R0 = 12,
-    ARG = 13,
-    RET = 14,
-    FE_SP = 15,
-    FE_FP = 16,
-    HW_TID = 17,
-    SR0 = 18,
-    CR0 = 19,
-    VAR_LAST = CR0
-};
-
-enum class PreDefinedVarsInternal_3_2
-{
-    VAR_NULL = 0,
-    X = 1,
-    Y = 2,
-    LOCAL_ID_X = 3,
-    LOCAL_ID_Y = 4,
-    LOCAL_SIZE_X = 5,
-    LOCAL_SIZE_Y = 6,
-    GROUP_ID_X = 7,
-    GROUP_ID_Y = 8,
-    GROUP_COUNT_X = 9,
-    GROUP_COUNT_Y = 10,
-    TSC = 11,
-    R0 = 12,
-    ARG = 13,
-    RET = 14,
-    FE_SP = 15,
-    FE_FP = 16,
-    HW_TID = 17,
-    SR0 = 18,
-    CR0 = 19,
-    CE0 = 20,
-    DBG = 21,
-    COLOR = 22, // Officially not supported for vISA3.2
-    VAR_LAST = COLOR
-};
-
-enum class PreDefinedVarsInternal_3_3
-{
-    VAR_NULL = 0,
-    X = 1,
-    Y = 2,
-    LOCAL_ID_X = 3,
-    LOCAL_ID_Y = 4,
-    LOCAL_SIZE_X = 5,
-    LOCAL_SIZE_Y = 6,
-    GROUP_ID_X = 7,
-    GROUP_ID_Y = 8,
-    GROUP_COUNT_X = 9,
-    GROUP_COUNT_Y = 10,
-    TSC = 11,
-    R0 = 12,
-    ARG = 13,
-    RET = 14,
-    FE_SP = 15,
-    FE_FP = 16,
-    HW_TID = 17,
-    SR0 = 18,
-    CR0 = 19,
-    CE0 = 20,
-    DBG = 21,
-    COLOR = 22,
-    GROUP_ID_Z = 23,
-    VAR_LAST = GROUP_ID_Z
+    { PreDefinedVarsInternal::VAR_NULL,      ISA_TYPE_UD, 1, true,  false,  0, 1,   "%null" },
+    { PreDefinedVarsInternal::X,             ISA_TYPE_UW, 1, true,  false,  4,  1,   "%thread_x" },
+    { PreDefinedVarsInternal::Y,             ISA_TYPE_UW, 1, true,  false,  6,  1,   "%thread_y" },
+    { PreDefinedVarsInternal::LOCAL_ID_X,    ISA_TYPE_UD, 2, false, true,   16, 1,   "%local_id_x" },
+    { PreDefinedVarsInternal::LOCAL_ID_Y,    ISA_TYPE_UD, 2, false, true,   20, 1,   "%local_id_y" },
+    { PreDefinedVarsInternal::LOCAL_SIZE_X,  ISA_TYPE_UD, 2, false, true,   0,  1,   "%local_size_x" },
+    { PreDefinedVarsInternal::LOCAL_SIZE_Y,  ISA_TYPE_UD, 2, false, true,   4,  1,   "%local_size_y" },
+    { PreDefinedVarsInternal::GROUP_ID_X,    ISA_TYPE_UD, 2, true,  true,   4,  1,   "%group_id_x" },
+    { PreDefinedVarsInternal::GROUP_ID_Y,    ISA_TYPE_UD, 2, true,  true,   24, 1,   "%group_id_y" },
+    { PreDefinedVarsInternal::GROUP_ID_Z,    ISA_TYPE_UD, 3, true,  true,   28, 1,   "%group_id_z" },
+    { PreDefinedVarsInternal::GROUP_COUNT_X, ISA_TYPE_UD, 2, false, true,   8,  1,   "%group_count_x" },
+    { PreDefinedVarsInternal::GROUP_COUNT_Y, ISA_TYPE_UD, 2, false, true,   12, 1,   "%group_count_y" },
+    { PreDefinedVarsInternal::TSC,           ISA_TYPE_UD, 2, false, false,  0,  5,   "%tsc" },
+    { PreDefinedVarsInternal::R0,            ISA_TYPE_UD, 2, true,  false,  0,  8,   "%r0" },
+    { PreDefinedVarsInternal::ARG,           ISA_TYPE_UD, 3, false, false,  0,  256, "%arg" },
+    { PreDefinedVarsInternal::RET,           ISA_TYPE_UD, 3, false, false,  0,  96,  "%retval" },
+    { PreDefinedVarsInternal::FE_SP,         ISA_TYPE_UQ, 3, false, false,  0,  1,   "%sp" },
+    { PreDefinedVarsInternal::FE_FP,         ISA_TYPE_UQ, 3, false, false,  0,  1,   "%fp" },
+    { PreDefinedVarsInternal::HW_TID,        ISA_TYPE_UD, 3, false, false,  0,  1,   "%hw_id" },
+    { PreDefinedVarsInternal::SR0,           ISA_TYPE_UD, 3, false, false,  0,  4,   "%sr0" },
+    { PreDefinedVarsInternal::CR0,           ISA_TYPE_UD, 3, false, false,  0,  1,   "%cr0" },
+    { PreDefinedVarsInternal::CE0,           ISA_TYPE_UD, 3, false, false,  0,  1,   "%ce0" },
+    { PreDefinedVarsInternal::DBG,           ISA_TYPE_UD, 3, false, false,  0,  2,   "%dbg0" },
+    { PreDefinedVarsInternal::COLOR,         ISA_TYPE_UW, 3, true,  false,  0,  1,   "%color" },
 };
 
 enum class PreDefinedVarsInternal_3_4
