@@ -59,8 +59,8 @@ namespace vISA
                         srcOpnd->isSrcRegRegion())
                     {
                         auto topdcl = srcOpnd->asSrcRegRegion()->getTopDcl();
-                        unsigned int startRow = srcOpnd->getLeftBound() / G4_GRF_REG_NBYTES;
-                        unsigned int endRow = srcOpnd->getRightBound() / G4_GRF_REG_NBYTES;
+                        unsigned int startRow = srcOpnd->getLeftBound() / numEltPerGRF(Type_UB);
+                        unsigned int endRow = srcOpnd->getRightBound() / numEltPerGRF(Type_UB);
                         if (topdcl)
                         {
                             auto dclIt = operations.find(topdcl);
@@ -180,7 +180,7 @@ namespace vISA
 
                     auto dupOp = samplerHeaderMov->cloneInst();
 
-                    bb->insert(instIt, dupOp);
+                    bb->insertBefore(instIt, dupOp);
                 }
             }
 
@@ -477,6 +477,11 @@ namespace vISA
             uniqueDef->first->getCondMod())
             return false;
 
+        // It is illegal to rematerialize intrinsic.split instruction as it
+        // is dependent on an earlier send.
+        if (uniqueDef->first->isSplitIntrinsic())
+            return false;
+
         ref = uniqueDef;
 
         // Check whether op1 can be recomputed
@@ -500,19 +505,10 @@ namespace vISA
         if (!inSameSubroutine(bb, uniqueDefBB))
             return false;
 
-#if 0
-        // idom currently not computed
-
-        // Def must be in a dominating BB
-        auto defDomsUse = doms.dominates(uniqueDefBB, bb);
-        if (!defDomsUse)
-            return false;
-#endif
-
         // If uniqueDefBB is not under SIMD CF, current BB is under SIMD CF
         // then we can remat only if def has NoMask option set.
-        if (!uniqueDefBB->isInSimdFlow() &&
-            bb->isInSimdFlow() &&
+        if (uniqueDefBB->isAllLaneActive() &&
+            !bb->isAllLaneActive() &&
             !uniqueDefInst->isWriteEnableInst())
         {
             return false;
@@ -532,7 +528,7 @@ namespace vISA
             // range then skip remating this operation.
             // Be less aggressive if this is SIMD8 since we run the
             // chance of perf penalty with this.
-            if (kernel.getSimdSize() == 8 ||
+            if ((kernel.getSimdSize() == 8 && rpe.getRegisterPressure(srcInst) < (float)rematLoopRegPressure * 1.6f) ||
                 rematCandidates[topdcl->getRegVar()->getId()] == false ||
                 rpe.getRegisterPressure(srcInst) < rematLoopRegPressure)
                 return false;
@@ -541,7 +537,13 @@ namespace vISA
             {
                 // Restrict non-SIMD1 remats to a low percent of loop instructions.
                 float loopInstToTotalInstRatio = (float)getNumRematsInLoop() / (float)loopInstsBeforeRemat*100.0f;
-                if (loopInstToTotalInstRatio > 1.75f)
+                if (rpe.getMaxRP() < rematRegPressure * 1.4f)
+                {
+                    // If max RPE is not very high, dont sink too many instructions in loop
+                    if(loopInstToTotalInstRatio > 1.75f)
+                        return false;
+                }
+                else if (loopInstToTotalInstRatio > 3.75f)
                     return false;
             }
         }
@@ -750,7 +752,7 @@ namespace vISA
                     auto srcOpndUniqueDef = findUniqueDef(srcOpndRefs, srcOpndRgn);
 
                     bool isSrcAvailble = false;
-                    if (kernel.getOptions()->getTarget() == VISA_CM &&
+                    if (kernel.getInt32KernelAttr(Attributes::ATTR_Target) == VISA_CM &&
                         uniqueDefBB == bb)
                     {
                         isSrcAvailble = checkLocalWAR(uniqueDefInst, bb, instIter);
@@ -792,7 +794,8 @@ namespace vISA
                 {
                     G4_SrcRegRegion* srcRgn = uniqueDefInst->getSrc(i)->asSrcRegRegion();
 
-                    if (srcRgn->getTopDcl()->getNumElems() > 1)
+                    if (srcRgn->getTopDcl()->getNumElems() > 1 &&
+                        getNumUses(srcRgn->getTopDcl()) < 15)
                     {
                         // Extending non-scalar operands can be expensive
                         return false;
@@ -845,20 +848,19 @@ namespace vISA
 
         if (!isSampler)
         {
-            unsigned int diffBound = dst->getRightBound() - (dst->getRegOff() * G4_GRF_REG_NBYTES);
+            unsigned int diffBound = dst->getRightBound() - (dst->getRegOff() * numEltPerGRF(Type_UB));
             unsigned numElems = (diffBound + 1) / G4_Type_Table[dst->getType()].byteSize;
             auto newTemp = kernel.fg.builder->createTempVar(numElems, dst->getType(), Any, "REMAT_");
             newTemp->copyAlign(dst->getTopDcl());
             gra.copyAlignment(newTemp, dst->getTopDcl());
             G4_DstRegRegion* newDst = kernel.fg.builder->createDst(newTemp->getRegVar(), 0,
-                (dst->getLeftBound() % G4_GRF_REG_NBYTES) / G4_Type_Table[dst->getType()].byteSize,
+                (dst->getLeftBound() % numEltPerGRF(Type_UB)) / G4_Type_Table[dst->getType()].byteSize,
                 dst->getHorzStride(), dst->getType());
             G4_INST* dupOp = nullptr;
 
             dupOp = dstInst->cloneInst();
             dupOp->setDest(newDst);
-
-            dupOp->setLineNo(dstInst->getLineNo());
+            dupOp->setLocation(dstInst->getLocation());
             dupOp->setCISAOff(dstInst->getCISAOff());
 
             rematSrc = createSrcRgn(src, dst, newTemp);
@@ -905,7 +907,7 @@ namespace vISA
                 MUST_BE_TRUE(ops != operations.end(), "Didnt find record in map");
                 MUST_BE_TRUE((*ops).second.numUses == 1, "Expecting src0 to be used only in sampler");
 
-                auto newSrc0Dcl = kernel.fg.builder->createTempVar(src0TopDcl->getNumElems(),
+                auto newSrc0Dcl = kernel.fg.builder->createTempVar(src0TopDcl->getTotalElems(),
                     src0TopDcl->getElemType(), gra.getSubRegAlign(src0TopDcl));
 
                 // Clone all defining instructions for sampler's msg header
@@ -915,7 +917,8 @@ namespace vISA
 
                     auto dupOp = headerDefInst->cloneInst();
                     auto headerDefDst = headerDefInst->getDst();
-                    dupOp->setDest(kernel.fg.builder->createDstRegRegion(headerDefDst->getRegAccess(),
+                    assert(!headerDefDst->isIndirect()); // we dont allow send header to be defined indirectly
+                    dupOp->setDest(kernel.fg.builder->createDst(
                         newSrc0Dcl->getRegVar(), headerDefDst->getRegOff(), headerDefDst->getSubRegOff(),
                         headerDefDst->getHorzStride(), headerDefDst->getType()));
                     newInst.push_back(dupOp);
@@ -944,8 +947,8 @@ namespace vISA
                 kernel.fg.builder->duplicateOperand(src0)->asSrcRegRegion(),
                 kernel.fg.builder->duplicateOperand(dstInst->getSrc(1))->asSrcRegRegion(),
                 kernel.fg.builder->duplicateOperand(dstInst->asSendInst()->getMsgDescOperand()), dstInst->getOption(),
-                newMsgDesc, kernel.fg.builder->duplicateOperand(dstInst->getSrc(3)), dstInst->getLineNo());
-            dupOp->setLineNo(dstInst->getLineNo());
+                newMsgDesc, kernel.fg.builder->duplicateOperand(dstInst->getSrc(3)), true);
+            dupOp->setLocation(dstInst->getLocation());
             dupOp->setCISAOff(dstInst->getCISAOff());
 
             newInst.push_back(dupOp);
@@ -962,8 +965,8 @@ namespace vISA
     {
         G4_SrcRegRegion* rematSrc = nullptr;
 
-        unsigned row = (srcToRemat->getLeftBound() / G4_GRF_REG_NBYTES) - (uniqueDef->getLeftBound() / G4_GRF_REG_NBYTES);
-        unsigned subReg = (srcToRemat->getLeftBound() % G4_GRF_REG_NBYTES) / G4_Type_Table[srcToRemat->getType()].byteSize;
+        unsigned row = (srcToRemat->getLeftBound() / numEltPerGRF(Type_UB)) - (uniqueDef->getLeftBound() / numEltPerGRF(Type_UB));
+        unsigned subReg = (srcToRemat->getLeftBound() % numEltPerGRF(Type_UB)) / G4_Type_Table[srcToRemat->getType()].byteSize;
 
         rematSrc = kernel.fg.builder->createSrcRegRegion(srcToRemat->getModifier(), Direct,
             rematTemp->getRegVar(), (short)row, (short)subReg, srcToRemat->getRegion(), srcToRemat->getType());
@@ -1042,20 +1045,13 @@ namespace vISA
 
     void Rematerialization::run()
     {
-        //unsigned int before = getNumSamplers(kernel);
-
-        if (kernel.fg.builder->getOption(vISA_DumpDotAll))
-        {
-            kernel.dumpDotFile("before.remat");
-        }
-
         populateRefs();
 
         auto firstProgInst = kernel.fg.getEntryBB()->getFirstInst();
 
         for (auto bb : kernel.fg)
         {
-            if (kernel.getOptions()->getTarget() == VISATarget::VISA_3D)
+            if (kernel.getInt32KernelAttr(Attributes::ATTR_Target) == VISATarget::VISA_3D)
             {
                 // For Cm, assume cr0 def is live across BBs
                 // For IGC, assume cr0 is reset at each BB entry
@@ -1154,7 +1150,7 @@ namespace vISA
                                 rematSrc = rematerialize(src->asSrcRegRegion(), bb, uniqueDef, newInsts, cacheInst);
                                 while (!newInsts.empty())
                                 {
-                                    bb->insert(instIt, newInsts.front());
+                                    bb->insertBefore(instIt, newInsts.front());
                                     newInsts.pop_front();
                                 }
 
@@ -1181,46 +1177,4 @@ namespace vISA
 
         //unsigned int after = getNumSamplers(kernel);
     }
-
-    void Dominators::computeDominators()
-    {
-        // Compute all doms for given bb.
-        // Flowgraph already has idoms for each bb.
-        for (auto&& bb : fg)
-        {
-            std::pair<G4_BB*, std::set<G4_BB*>> domBB;
-            domBB.first = bb;
-            auto idom = bb->getIDom();
-            while (idom)
-            {
-                domBB.second.insert(bb->getIDom());
-                idom = idom->getIDom();
-            }
-        }
-    }
-    bool Dominators::dominates(G4_BB* def, G4_BB* use)
-    {
-        auto dIt = dom.find(use);
-        if (dIt == dom.end())
-            return false;
-
-        auto bbIt = dIt->second.find(def);
-        if (bbIt == dIt->second.end())
-            return false;
-
-        return true;
-    }
-
-    void Dominators::dump()
-    {
-        for (auto&& bb : dom)
-        {
-            printf("BB%d:", bb.first->getId());
-            for (auto&& d : bb.second)
-            {
-                printf("BB%d, ", d->getId());
-            }
-            printf("\n\n");
-        }
-    }
 }
