@@ -145,8 +145,18 @@ namespace vISA
               FlagDefPerBB(nullptr),
               WAce0Read(false)
         {
-            WAce0Read = VISA_WA_CHECK(Builder->getPWaTable(), Wa_1406950495) ;
-            initDMaskModInfo();
+            // Using "dmask (sr0.2) And ce0.0" for emask for each BB is not very safe
+            // for the following reasons:
+            //    1) shader may use vmask; or
+            //    2) reading ce might be unsafe (for example, hardware issue)
+            // Thus, we set WAce0Read to true to force using "cmp r0, r0" to create emask
+            // always, and no dmask would be used at all!
+            //WAce0Read = VISA_WA_CHECK(Builder->getPWaTable(), Wa_1406950495) ;
+            WAce0Read = true;
+            if (!WAce0Read)
+            {
+                initDMaskModInfo();
+            }
         }
 
         bool run(G4_BB* BB);
@@ -272,13 +282,6 @@ bool SendFusion::isAtomicCandidate(G4_SendMsgDescriptor* msgDesc)
     }
     else
     {
-        // For now, comment this out and rely solely on
-        // vISA_EnableAtomicFusion
-        //if (!Builder->getOption(vISA_unsafeMath))
-        //{
-        //    return false;
-        //}
-
         switch (atomicOp)
         {
         default:
@@ -313,7 +316,7 @@ bool SendFusion::simplifyAndCheckCandidate(INST_LIST_ITER Iter)
     // If needed, more messages can be handled later.
     G4_opcode opc = I->opcode();
     if ((opc != G4_send && opc != G4_sends) ||
-        I->getExecSize() > 8 ||
+        I->getExecSize() > g4::SIMD8 ||
         I->getPredicate() != nullptr ||
         !(I->is1QInst() || I->isWriteEnableInst()))
     {
@@ -351,7 +354,7 @@ bool SendFusion::simplifyAndCheckCandidate(INST_LIST_ITER Iter)
     uint16_t msgLen = msgDesc->MessageLength();
     uint16_t rspLen = msgDesc->ResponseLength();
     uint16_t extMsgLen = msgDesc->extMessageLength();
-    if (I->getExecSize() < 8 &&
+    if (I->getExecSize() < g4::SIMD8 &&
         !(I->isWriteEnableInst() &&
           ((msgDesc->isDataPortWrite() && (msgLen + extMsgLen) == 2) ||
            (msgDesc->isDataPortRead() && msgLen == 1 && rspLen == 1))))
@@ -452,7 +455,7 @@ bool SendFusion::simplifyAndCheckCandidate(INST_LIST_ITER Iter)
     auto funcID = msgDesc->getFuncId();
     switch (funcID) {
     case SFID::DP_DC:
-        switch(msgDesc->getHdcMessageType())
+        switch (msgDesc->getHdcMessageType())
         {
         case DC_DWORD_SCATTERED_READ:
         case DC_DWORD_SCATTERED_WRITE:
@@ -462,7 +465,7 @@ bool SendFusion::simplifyAndCheckCandidate(INST_LIST_ITER Iter)
         }
         break;
     case SFID::DP_DC1:
-        switch(msgDesc->getHdcMessageType())
+        switch (msgDesc->getHdcMessageType())
         {
         case DC1_UNTYPED_SURFACE_READ:
         case DC1_UNTYPED_SURFACE_WRITE:
@@ -470,7 +473,7 @@ bool SendFusion::simplifyAndCheckCandidate(INST_LIST_ITER Iter)
         }
         break;
     case SFID::DP_DC2:
-        switch(msgDesc->getHdcMessageType())
+        switch (msgDesc->getHdcMessageType())
         {
         case DC2_BYTE_SCATTERED_READ:
         case DC2_BYTE_SCATTERED_WRITE:
@@ -532,7 +535,7 @@ void SendFusion::simplifyMsg(INST_LIST_ITER SendIter)
 
     if (nDefs != 1 || addI->opcode() != G4_add ||
         !addI->getSrc(1)->isImm() || !addI->isWriteEnableInst() ||
-        addI->getExecSize() != 1) {
+        addI->getExecSize() != g4::SIMD1) {
         return;
     }
 
@@ -548,7 +551,7 @@ void SendFusion::simplifyMsg(INST_LIST_ITER SendIter)
     }
     if (nDefs != 1 || movI->opcode() != G4_mov ||
         !movI->getSrc(0)->isImm() || !movI->isWriteEnableInst() ||
-        movI->getExecSize() != 1) {
+        movI->getExecSize() != g4::SIMD1) {
         return;
     }
 
@@ -625,7 +628,7 @@ bool SendFusion::canFusion(INST_LIST_ITER IT0, INST_LIST_ITER IT1)
     G4_INST* I0 = *IT0;
     G4_INST* I1 = *IT1;
     G4_opcode opc = I0->opcode();
-    assert( (opc == G4_send || opc == G4_sends) && opc == I1->opcode() &&
+    assert((opc == G4_send || opc == G4_sends) && opc == I1->opcode() &&
             "Arguments to canFusion must be the same kind of Send Messages!");
 
     // Current implementation uses split send to replace two sends. For simplicity,
@@ -792,7 +795,7 @@ void SendFusion::doSink(
     {
         int j = 1;
         G4_INST *Inst = InstToBeSinked[j];
-        for (INST_LIST_ITER IT = StartIT; IT != EndIT; )
+        for (INST_LIST_ITER IT = StartIT; IT != EndIT;)
         {
             G4_INST *tmp = *IT;
             if (tmp == Inst) {
@@ -816,7 +819,7 @@ void SendFusion::doSink(
 
         for (int i = 1; i < numToBeSinked; ++i)
         {
-            CurrBB->insert(InsertBeforePos, InstToBeSinked[i]);
+            CurrBB->insertBefore(InsertBeforePos, InstToBeSinked[i]);
         }
     }
 }
@@ -835,7 +838,7 @@ void SendFusion::doHoist(
     {
         int j = numToBeHoisted-1;
         G4_INST *Inst = InstToBeHoisted[j];
-        for (INST_LIST_ITER IT = StartIT; IT != EndIT; )
+        for (INST_LIST_ITER IT = StartIT; IT != EndIT;)
         {
             G4_INST *tmp = *IT;
             if (tmp == Inst) {
@@ -859,7 +862,7 @@ void SendFusion::doHoist(
 
         for (int i = numToBeHoisted - 1; i > 0; --i)
         {
-            CurrBB->insert(InsertBeforePos, InstToBeHoisted[i]);
+            CurrBB->insertBefore(InsertBeforePos, InstToBeHoisted[i]);
         }
     }
 }
@@ -888,7 +891,7 @@ void SendFusion::packPayload(
     G4_BB* bb, INST_LIST_ITER InsertBeforePos)
 {
     // Both Send0 and Send1 have the same MsgDesc.
-    unsigned char ExecSize = Send0->getExecSize();
+    G4_ExecSize execSize = Send0->getExecSize();
     G4_SendMsgDescriptor* origDesc = Send0->getMsgDesc();
     int option = Send0->getOption();
     int16_t msgLen = origDesc->MessageLength();
@@ -897,7 +900,7 @@ void SendFusion::packPayload(
     // mov (ES) DVar<D_regoff, D_sregoff>:Ty<1> SVar<S_regoff, S_sregoff>:Ty(1;1,0)
     auto copyRegOpnd = [&](
         G4_VarBase* DVar, G4_VarBase* SVar,
-        G4_Type Ty, unsigned char ES,
+        G4_Type Ty, G4_ExecSize ES,
         int16_t D_regoff, int16_t D_sregoff,
         int16_t S_regoff, int16_t S_sregoff) -> G4_INST*
     {
@@ -907,7 +910,7 @@ void SendFusion::packPayload(
         G4_DstRegRegion* D = Builder->createDst(
             DVar, D_regoff, D_sregoff, 1, Ty);
         G4_INST* nInst = Builder->createMov(ES, D, S, option, false);
-        bb->insert(InsertBeforePos, nInst);
+        bb->insertBefore(InsertBeforePos, nInst);
         return nInst;
     };
 
@@ -923,7 +926,7 @@ void SendFusion::packPayload(
     int16_t s1_roff = S1->getRegOff();
 
     // Special case for exec_size = 1|2|4
-    if (ExecSize < 8)
+    if (execSize < g4::SIMD8)
     {
         assert((origDesc->isDataPortWrite() ||
                 (msgLen == 1 && origDesc->ResponseLength() == 1)) &&
@@ -932,15 +935,15 @@ void SendFusion::packPayload(
             "Internal Error (SendFusion): unexpected write message!");
 
         /// Address payload size (in unit of GRF)
-        //    A64: 2*ExecSize UQ (1 GRF for ExecSize=1|2; 2 GRF for ExecSize=4)
-        //    Otherwise: 2*ExecSize DW, which is 1 GRF.
-        G4_INST* Inst0 = copyRegOpnd(Dst0, V0, Ty, ExecSize, d_roff, 0, s0_roff, 0);
+        //    A64: 2*execSize UQ (1 GRF for execSize=1|2; 2 GRF for execSize=4)
+        //    Otherwise: 2*execSize DW, which is 1 GRF.
+        G4_INST* Inst0 = copyRegOpnd(Dst0, V0, Ty, execSize, d_roff, 0, s0_roff, 0);
         G4_INST* Inst1;
-        if (origDesc->isA64Message() && ExecSize == 4) {
-            Inst1 = copyRegOpnd(Dst0, V1, Ty, ExecSize, d_roff + 1, 0, s1_roff, 0);
+        if (origDesc->isA64Message() && execSize == 4) {
+            Inst1 = copyRegOpnd(Dst0, V1, Ty, execSize, d_roff + 1, 0, s1_roff, 0);
         }
         else {
-            Inst1 = copyRegOpnd(Dst0, V1, Ty, ExecSize, d_roff, ExecSize, s1_roff, 0);
+            Inst1 = copyRegOpnd(Dst0, V1, Ty, execSize, d_roff, execSize, s1_roff, 0);
         }
 
         // Update DefUse
@@ -956,7 +959,7 @@ void SendFusion::packPayload(
         }
 
         // Source payload
-        //    Either in src0 or src1, but not both as ExecSize <= 4
+        //    Either in src0 or src1, but not both as execSize <= 4
         Ty = Type_UD; // source payload
         Gen4_Operand_Number opn = (msgLen > 1) ? Opnd_src0 : Opnd_src1;
         G4_SrcRegRegion* Reg0 = Send0->getOperand(opn)->asSrcRegRegion();
@@ -969,13 +972,13 @@ void SendFusion::packPayload(
         s1_roff = Reg1->getRegOff();
         if (opn == Opnd_src0) {
             // source payload follows address payload in Opnd_src0.
-            d_roff += ((origDesc->isA64Message() && ExecSize == 4) ? 2 : 1);
+            d_roff += ((origDesc->isA64Message() && execSize == 4) ? 2 : 1);
             s0_roff += 1;
             s1_roff += 1;
         }
 
-        Inst0 = copyRegOpnd(Dst, Var0, Ty, ExecSize, d_roff, 0, s0_roff, 0);
-        Inst1 = copyRegOpnd(Dst, Var1, Ty, ExecSize, d_roff, ExecSize, s1_roff, 0);
+        Inst0 = copyRegOpnd(Dst, Var0, Ty, execSize, d_roff, 0, s0_roff, 0);
+        Inst1 = copyRegOpnd(Dst, Var1, Ty, execSize, d_roff, execSize, s1_roff, 0);
 
         // Update DefUse
         Inst0->addDefUse(FusedSend, opn);
@@ -986,15 +989,15 @@ void SendFusion::packPayload(
         return;
     }
 
-    // Now, ExecSize = 8
+    // Now, execSize = 8
     // the number of grf for address payload in the original send
     const int16_t addrLen = (origDesc->isA64Message() ? 2 : 1);
 
     ///
     /// 1. copy address payload
     ///
-    G4_INST* Inst0 = copyRegOpnd(Dst0, V0, Ty, ExecSize, d_roff,   0, s0_roff, 0);
-    G4_INST* Inst1 = copyRegOpnd(Dst0, V1, Ty, ExecSize, d_roff+addrLen, 0, s1_roff, 0);
+    G4_INST* Inst0 = copyRegOpnd(Dst0, V0, Ty, execSize, d_roff,   0, s0_roff, 0);
+    G4_INST* Inst1 = copyRegOpnd(Dst0, V1, Ty, execSize, d_roff+addrLen, 0, s1_roff, 0);
 
     // Update DefUse
     Inst0->addDefUse(FusedSend, Opnd_src0);
@@ -1039,8 +1042,8 @@ void SendFusion::packPayload(
         for (int i = 0; i < nMov; ++i)
         {
             // copy operands of both send0 and send1 to Dst
-            G4_INST* I0 = copyRegOpnd(Dst, Var0, Ty, ExecSize, Off + 2 * i, 0, Off0 + i, 0);
-            G4_INST* I1 = copyRegOpnd(Dst, Var1, Ty, ExecSize, Off + 2 * i + 1, 0, Off1 + i, 0);
+            G4_INST* I0 = copyRegOpnd(Dst, Var0, Ty, execSize, Off + 2 * i, 0, Off0 + i, 0);
+            G4_INST* I1 = copyRegOpnd(Dst, Var1, Ty, execSize, Off + 2 * i + 1, 0, Off1 + i, 0);
 
             // Update DefUse
             I0->addDefUse(FusedSend, opn);
@@ -1075,7 +1078,7 @@ void SendFusion::unpackPayload(
     assert(G4_Type_Table[Ty].byteSize == 4 && "Unexpected Type!");
 
     // Use the original option for mov instructions
-    unsigned char ExecSize = Send0->getExecSize();
+    G4_ExecSize execSize = Send0->getExecSize();
     int option = Send0->getOption();
     int32_t nMov = Send0->getMsgDesc()->ResponseLength();
 
@@ -1084,7 +1087,7 @@ void SendFusion::unpackPayload(
     // Note that the code is designed for exec_size=8. It also
     // works for exec_size=1|2|4 with minor change (keep in mind
     // that nMov = 1 for exec_size=1|2|4)
-    assert((ExecSize == 8 || nMov == 1) &&
+    assert((execSize == g4::SIMD8 || nMov == 1) &&
            "Internal Error(SendFusion) : unexpected message response length!");
 
     G4_VarBase* Payload = FusedSend->getDst()->getBase();
@@ -1103,8 +1106,8 @@ void SendFusion::unpackPayload(
             Mod_src_undef, Direct, Payload, 2*i, 0, stride1, Ty);
         D = Builder->createDst(
             Dst0, Off0 + i, 0, 1, Ty);
-        G4_INST* Inst0 = Builder->createMov(ExecSize, D, S, option, false);
-        bb->insert(InsertBeforePos, Inst0);
+        G4_INST* Inst0 = Builder->createMov(execSize, D, S, option, false);
+        bb->insertBefore(InsertBeforePos, Inst0);
 
         // Update DefUse
         FusedSend->addDefUse(Inst0, Opnd_src0);
@@ -1116,12 +1119,12 @@ void SendFusion::unpackPayload(
     {
         S = Builder->createSrcRegRegion(
             Mod_src_undef, Direct, Payload,
-            (ExecSize == 8 ? 2*i + 1 : 2*i),
-            (ExecSize == 8) ? 0 : ExecSize,
+            (execSize == 8 ? 2*i + 1 : 2*i),
+            (execSize == 8) ? 0 : execSize,
             stride1, Ty);
         D = Builder->createDst(Dst1, Off1 + i, 0, 1, Ty);
-        G4_INST* Inst1 = Builder->createMov(ExecSize, D, S, option, false);
-        bb->insert(InsertBeforePos, Inst1);
+        G4_INST* Inst1 = Builder->createMov(execSize, D, S, option, false);
+        bb->insertBefore(InsertBeforePos, Inst1);
 
         // Update DefUse
         FusedSend->addDefUse(Inst1, Opnd_src0);
@@ -1165,8 +1168,8 @@ void SendFusion::createDMask(G4_BB* bb, INST_LIST_ITER InsertBeforePos)
         Mod_src_undef, Direct, sr0, 0, 2, Builder->getRegionScalar(), Type_UD);
     G4_DstRegRegion* Dst = Builder->createDst(
         dmaskDecl->getRegVar(), 0, 0, 1, Type_UD);
-    G4_INST* Inst = Builder->createMov(1, Dst, Src, InstOpt_WriteEnable, false);
-    bb->insert(InsertBeforePos, Inst);
+    G4_INST* Inst = Builder->createMov(g4::SIMD1, Dst, Src, InstOpt_WriteEnable, false);
+    bb->insertBefore(InsertBeforePos, Inst);
 
     // update DefUse info
     CFG->globalOpndHT.addGlobalOpnd(Dst);
@@ -1186,8 +1189,8 @@ void SendFusion::createDMask(G4_BB* bb, INST_LIST_ITER InsertBeforePos)
             Mod_src_undef, Direct, sr0, 0, 2, Builder->getRegionScalar(), Type_UD);
         G4_DstRegRegion* D = Builder->createDst(
             dmaskDecl->getRegVar(), 0, 0, 1, Type_UD);
-        G4_INST* Inst = Builder->createMov(1, D, S, InstOpt_WriteEnable, false);
-        BB->insert(InsertPos, Inst);
+        G4_INST* Inst = Builder->createMov(g4::SIMD1, D, S, InstOpt_WriteEnable, false);
+        BB->insertBefore(InsertPos, Inst);
     }
 }
 
@@ -1221,9 +1224,9 @@ void SendFusion::createFlagPerBB(G4_BB* bb, INST_LIST_ITER InsertBeforePos)
         G4_DstRegRegion* flag = Builder->createDst(
             flagVar, 0, 0, 1, Type_UW);
 
-        G4_INST* I0 = Builder->createMov(1, flag,
+        G4_INST* I0 = Builder->createMov(g4::SIMD1, flag,
             Builder->createImm(0, Type_UW), InstOpt_WriteEnable, false);
-        bb->insert(InsertBeforePos, I0);
+        bb->insertBefore(InsertBeforePos, I0);
 
         G4_SrcRegRegion *r0_0 = Builder->createSrcRegRegion(
             Mod_src_undef, Direct,
@@ -1236,9 +1239,10 @@ void SendFusion::createFlagPerBB(G4_BB* bb, INST_LIST_ITER InsertBeforePos)
         G4_CondMod* flagCM = Builder->createCondMod(Mod_e, flagVar, 0);
         G4_DstRegRegion *nullDst = Builder->createNullDst(Type_UW);
         // Hard-coded simd8 here!
-        G4_INST* I1 = Builder->createInternalInst(NULL, G4_cmp, flagCM, false, 8,
+        G4_INST* I1 = Builder->createInternalInst(
+            NULL, G4_cmp, flagCM, g4::NOSAT, g4::SIMD8,
             nullDst, r0_0, r0_1, InstOpt_M0);
-        bb->insert(InsertBeforePos, I1);
+        bb->insertBefore(InsertBeforePos, I1);
 
         G4_SrcRegRegion *flagSrc = Builder->createSrcRegRegion(
             Mod_src_undef, Direct,
@@ -1246,8 +1250,8 @@ void SendFusion::createFlagPerBB(G4_BB* bb, INST_LIST_ITER InsertBeforePos)
             Builder->getRegionScalar(), Type_UW);
         G4_DstRegRegion* tmpDst1 = Builder->createDst(
             tmpDecl->getRegVar(), 0, 0, 1, Type_UW);
-        Inst0 = Builder->createMov( 1, tmpDst1, flagSrc, InstOpt_WriteEnable, false);
-        bb->insert(InsertBeforePos, Inst0);
+        Inst0 = Builder->createMov(g4::SIMD1, tmpDst1, flagSrc, InstOpt_WriteEnable, false);
+        bb->insertBefore(InsertBeforePos, Inst0);
 
         // update DefUse
         I1->addDefUse(Inst0, Opnd_src0);
@@ -1255,16 +1259,16 @@ void SendFusion::createFlagPerBB(G4_BB* bb, INST_LIST_ITER InsertBeforePos)
     }
     else
     {
-        // (W) and (|M0) tmp<1>:ud ce0.0<0;1,0>:ud DMaskUD:ud
+        // (W) and (1|M0) tmp<1>:ud ce0.0<0;1,0>:ud DMaskUD:ud
         G4_SrcRegRegion *ce0Src = Builder->createSrcRegRegion(
             Mod_src_undef, Direct, Builder->phyregpool.getMask0Reg(), 0, 0, scalar, Type_UD);
         G4_SrcRegRegion *dmaskSrc = Builder->createSrcRegRegion(
             Mod_src_undef, Direct, DMaskUD, 0, 0, scalar, Type_UD);
         G4_DstRegRegion* tmpDst = Builder->createDst(
             tmpDecl->getRegVar(), 0, 0, 1, Type_UD);
-        Inst0 = Builder->createInternalInst(
-            NULL, G4_and, NULL, false, 1, tmpDst, ce0Src, dmaskSrc, InstOpt_WriteEnable);
-        bb->insert(InsertBeforePos, Inst0);
+        Inst0 = Builder->createBinOp(
+            G4_and, g4::SIMD1, tmpDst, ce0Src, dmaskSrc, InstOpt_WriteEnable, false);
+        bb->insertBefore(InsertBeforePos, Inst0);
     }
 
     //  Duplicate 8-bit mask to the next 8 bits
@@ -1275,8 +1279,8 @@ void SendFusion::createFlagPerBB(G4_BB* bb, INST_LIST_ITER InsertBeforePos)
         Mod_src_undef, Direct, tmpUBDecl->getRegVar(), 0, 0, scalar, Type_UB);
     G4_DstRegRegion* D = Builder->createDst(
         tmpUBDecl->getRegVar(), 0, 0, 1, Type_UB);
-    G4_INST* Inst1 = Builder->createMov(2, D, S, InstOpt_WriteEnable, false);
-    bb->insert(InsertBeforePos, Inst1);
+    G4_INST* Inst1 = Builder->createMov(g4::SIMD2, D, S, InstOpt_WriteEnable, false);
+    bb->insertBefore(InsertBeforePos, Inst1);
 
     // update DefUse
     Inst0->addDefUse(Inst1, Opnd_src0);
@@ -1288,8 +1292,8 @@ void SendFusion::createFlagPerBB(G4_BB* bb, INST_LIST_ITER InsertBeforePos)
         Mod_src_undef, Direct, tmpUW->getRegVar(), 0, 0, scalar, Type_UW);
     G4_DstRegRegion* flag = Builder->createDst(
         FlagPerBB, 0, 0, 1, Type_UW);
-    FlagDefPerBB = Builder->createMov(1, flag, Src, InstOpt_WriteEnable, false);
-    bb->insert(InsertBeforePos, FlagDefPerBB);
+    FlagDefPerBB = Builder->createMov(g4::SIMD1, flag, Src, InstOpt_WriteEnable, false);
+    bb->insertBefore(InsertBeforePos, FlagDefPerBB);
 
     // update DefUse
     Inst1->addDefUse(FlagDefPerBB, Opnd_src0);
@@ -1335,7 +1339,7 @@ void SendFusion::doFusion(
 
     // Use I0 as both I0 and I1 have the same properties
     G4_SendMsgDescriptor* desc = I0->getMsgDesc();
-    unsigned char ExecSize = I0->getExecSize();
+    G4_ExecSize execSize = I0->getExecSize();
     bool isWrtEnable = I0->isWriteEnableInst();
     bool isSplitSend = I0->isSplitSend();
 
@@ -1379,10 +1383,10 @@ void SendFusion::doFusion(
     uint32_t newMsgLen = 2*msgLen;
     uint32_t newRspLen = 2*rspLen;
     uint32_t newExtMsgLen = 2*extMsgLen;
-    if (ExecSize < 8)
+    if (execSize < 8)
     {
         // Re-adjust length, note that msgLen should be > 0
-        if (ExecSize == 4 && desc->isA64Message() && msgLen > 0)
+        if (execSize == 4 && desc->isA64Message() && msgLen > 0)
         {
             // addr : 2 GRF; source : 1 GRF
             newMsgLen = (msgLen == 1 ? 2 : 3);
@@ -1424,7 +1428,7 @@ void SendFusion::doFusion(
     // safely set bti to nullptr here.
     //G4_Operand* bti = (desc->getBti() ? Builder->duplicateOperand(desc->getBti()) : nullptr);
     G4_Operand* bti = nullptr;
-    uint32_t newFC = (ExecSize < 8 ? desc->getFuncCtrl()
+    uint32_t newFC = (execSize < 8 ? desc->getFuncCtrl()
                                    : getFuncCtrlWithSimd16(desc));
 
     // In general, we have the following
@@ -1452,7 +1456,7 @@ void SendFusion::doFusion(
     //           <O1>
 
     // Special case of two reads whose payloads can be concatenated using split send.
-    if (!isSplitSend && ExecSize == 8 && rspLen > 0 &&
+    if (!isSplitSend && execSize == 8 && rspLen > 0 &&
         (msgLen == 1 || (msgLen == 2 && desc->isA64Message())) &&
         extMsgLen == 0)
     {
@@ -1469,15 +1473,15 @@ void SendFusion::doFusion(
         G4_SrcRegRegion* Src0 = Builder->createSrcRegRegion(*s0);
         G4_SrcRegRegion* Src1 = Builder->createSrcRegRegion(*s1);
         G4_INST* sendInst = Builder->createSplitSendInst(
-            Pred, G4_sends, 16, Dst, Src0, Src1,
+            Pred, G4_sends, g4::SIMD16, Dst, Src0, Src1,
             Builder->createImm(newDesc->getDesc(), Type_UD),
-            InstOpt_WriteEnable, newDesc, nullptr, 0);
+            InstOpt_WriteEnable, newDesc, nullptr, true);
 
         if (!IsSink)
         {   // move depInst first if doing hoisting
             doHoist(IT0, IT1, InsertBeforePos);
         }
-        CurrBB->insert(InsertBeforePos, sendInst);
+        CurrBB->insertBefore(InsertBeforePos, sendInst);
 
         // Update DefUse
         if (Pred)
@@ -1535,17 +1539,18 @@ void SendFusion::doFusion(
         P1 = Builder->createTempVar(newExtMsgLen * 8, P1Ty, Any, "payload1");
         G4_SrcRegRegion* Src1 = Builder->Create_Src_Opnd_From_Dcl(P1, region);
         sendInst = Builder->createSplitSendInst(
-            Pred, G4_sends, ExecSize*2, Dst, Src0, Src1,
+            Pred, G4_sends, G4_ExecSize(execSize*2), Dst, Src0, Src1,
             Builder->createImm(newDesc->getDesc(), Type_UD),
-            InstOpt_WriteEnable, newDesc, nullptr, 0);
+            InstOpt_WriteEnable, newDesc, nullptr, true);
     }
     else
     {
         sendInst = Builder->createSendInst(
-            Pred, G4_send, ExecSize*2, Dst, Src0,
+            Pred, G4_send, G4_ExecSize(execSize*2), Dst, Src0,
             Builder->createImm(newDesc->getDesc(), Type_UD),
             InstOpt_WriteEnable,
-            newDesc);
+            newDesc,
+            true);
     }
 
     if (!IsSink) {
@@ -1562,7 +1567,7 @@ void SendFusion::doFusion(
         FlagDefPerBB->addDefUse(sendInst, Opnd_pred);
     }
 
-    CurrBB->insert(InsertBeforePos, sendInst);
+    CurrBB->insertBefore(InsertBeforePos, sendInst);
 
     if (rspLen > 0)
     {
@@ -1611,7 +1616,7 @@ bool SendFusion::run(G4_BB* BB)
         G4_INST* inst1 = nullptr;
         INST_LIST_ITER II1 = II0;
         ++II1;
-        while(II1 != IE)
+        while (II1 != IE)
         {
             G4_INST* tmp = *II1;
             if (simplifyAndCheckCandidate(II1))
