@@ -36,6 +36,8 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 using namespace vISA;
 
+void splice(G4_BB* bb, INST_LIST_ITER iter, INST_LIST& instList, unsigned int CISAOff);
+
 //
 // create a declare to hold the spill value of var
 //
@@ -123,7 +125,7 @@ G4_Declare* SpillManager::createNewTempAddrDeclare(G4_Declare* dcl, uint16_t num
                  type == Type_D, "addr reg's type should be UW or UD");
     MUST_BE_TRUE(dcl->getNumRows() == 1, "Temp_ADDR should be only 1 row");
     MUST_BE_TRUE(dcl->getNumElems() <= getNumAddrRegisters(), "Temp_ADDR exceeds 16 bytes");
-    G4_Declare* sp = builder.createDeclareNoLookup( name,
+    G4_Declare* sp = builder.createDeclareNoLookup(name,
                                             G4_ADDRESS,
                                             num_reg,
                                             1, // 1 row
@@ -164,17 +166,17 @@ void SpillManager::genRegMov(G4_BB* bb,
             */
             G4_Type type = Type_W;
             const RegionDesc* srcRgn = NULL;
-            unsigned execSize = i;
-            if(src->isFlag() || dst->isFlag())
+            G4_ExecSize execSize {i};
+            if (src->isFlag() || dst->isFlag())
             {
 
                 type = Type_UW;
-                if(i == 2)
+                if (i == 2)
                 {
                     type = Type_UD;
-                    execSize = 1;
+                    execSize = g4::SIMD1;
                 }
-                else if(i > 2)
+                else if (i > 2)
                 {
                     ASSERT_USER(false, "unsupported flag width");
                 }
@@ -186,22 +188,18 @@ void SpillManager::genRegMov(G4_BB* bb,
                 srcRgn = (i== 1) ? builder.getRegionScalar() : builder.getRegionStride1();
             }
 
-            G4_SrcRegRegion* s = builder.createSrcRegRegion(Mod_src_undef,
-                    Direct,
-                    src,
-                    0,
-                    sSubRegOff,
-                    srcRgn,
-                    type);
+            G4_SrcRegRegion* s = builder.createSrcRegRegion(
+                Mod_src_undef,
+                Direct,
+                src,
+                0,
+                sSubRegOff,
+                srcRgn,
+                type);
             //
             // create a0.aOff<1>
             //
-            G4_DstRegRegion* d = builder.createDst(
-                                dst,
-                                0,
-                                dSubRegOff,
-                                1,
-                                type);
+            G4_DstRegRegion* d = builder.createDst(dst, 0, dSubRegOff, 1, type);
 
             if (execSize != kernel.getSimdSize())
             {
@@ -209,7 +207,7 @@ void SpillManager::genRegMov(G4_BB* bb,
                 useNoMask = true;
             }
             // mov (nRegs)  a0.aOff<1>  loc(0,locOff)<4;4,1>
-            builder.createMov((uint8_t) execSize, d, s,
+            builder.createMov(execSize, d, s,
                 useNoMask ? InstOpt_WriteEnable : InstOpt_NoOpt, true);
 
             sSubRegOff += i;
@@ -223,7 +221,7 @@ void SpillManager::genRegMov(G4_BB* bb,
     //
     // insert newly created insts from builder to instList
     //
-    bb->splice(it,builder.instList);
+    splice(bb, it, builder.instList, currCISAOffset);
 }
 //
 // check if dst is spilled & insert spill code
@@ -250,12 +248,12 @@ void SpillManager::replaceSpilledDst(G4_BB* bb,
         {
 
             G4_DstRegRegion rgn(*dst, spDcl->getRegVar()); // using spDcl as new base
-            if( rgn.getHorzStride() == UNDEFINED_SHORT &&
-                dst->isFlag() )
+            if (rgn.getHorzStride() == UNDEFINED_SHORT &&
+                dst->isFlag())
             {
                 // Flag as destination has undefined hstride
                 // For replacing it with spill range, make hstride 1
-                rgn.setHorzStride( 1 );
+                rgn.setHorzStride(1);
             }
             G4_DstRegRegion* d = builder.createDstRegRegion(rgn);
             inst->setDest(d);
@@ -273,20 +271,20 @@ void SpillManager::replaceSpilledDst(G4_BB* bb,
             G4_Declare* tmpDcl = NULL;
             bool match_found = false;
 
-            for(unsigned int j = 0; j < G4_MAX_SRCS; j++)
+            for (unsigned int j = 0; j < G4_MAX_SRCS; j++)
             {
                 G4_SrcRegRegion* analyzed_src = (G4_SrcRegRegion*) operands_analyzed[j];
-                if( analyzed_src != NULL &&
+                if (analyzed_src != NULL &&
                     analyzed_src->getBase()->asRegVar()->getDeclare() == dst->getBase()->asRegVar()->getDeclare() &&
                     analyzed_src->getSubRegOff() == dst->getSubRegOff() &&
-                    !analyzed_src->getRegion()->isRegionWH() )
+                    !analyzed_src->getRegion()->isRegionWH())
                 {
                     tmpDcl = declares_created[j];
                     match_found = true;
                 }
             }
 
-            if( !match_found )
+            if (!match_found)
             {
                 tmpDcl = createNewTempAddrDeclare(spDcl);
                 //
@@ -299,19 +297,12 @@ void SpillManager::replaceSpilledDst(G4_BB* bb,
             }
 
             G4_DstRegRegion rgn(*dst, tmpDcl->getRegVar()); // using tmpDcl as new base
-            G4_DstRegRegion* d = builder.createDstRegRegion(rgn);
-
-            //src has optimization to bring back only what is used.
-            //so it will always start from 0
-            if( match_found )
-            {
-                d->setSubRegOff(0);
-            }
+            G4_DstRegRegion* d = match_found ? builder.createDstWithNewSubRegOff(&rgn, 0) : builder.createDstRegRegion(rgn);
             inst->setDest(d);
 
-            if( !match_found )
+            if (!match_found)
             {
-                pointsToAnalysis.insertAndMergeFilledAddr( dst->getBase()->asRegVar(), tmpDcl->getRegVar() );
+                pointsToAnalysis.insertAndMergeFilledAddr(dst->getBase()->asRegVar(), tmpDcl->getRegVar());
             }
         }
         else
@@ -356,8 +347,8 @@ void SpillManager::replaceSpilledSrc(G4_BB* bb,
                 // (W) mov (1) tmpDcl<1>:ud spDcl<0;1,0>:ud
                 auto movSrc = builder.Create_Src_Opnd_From_Dcl(spDcl, builder.getRegionScalar());
                 auto movDst = builder.Create_Dst_Opnd_From_Dcl(tmpDcl, 1);
-                G4_INST* movInst = builder.createMov(1, movDst, movSrc, InstOpt_WriteEnable, false);
-                bb->insert(it, movInst);
+                G4_INST* movInst = builder.createMov(g4::SIMD1, movDst, movSrc, InstOpt_WriteEnable, false);
+                bb->insertBefore(it, movInst);
 
                 s = builder.createSrcRegRegion(
                     Mod_src_undef,
@@ -420,7 +411,7 @@ void SpillManager::replaceSpilledSrc(G4_BB* bb,
                 genRegMov(bb, it,
                     spDcl->getRegVar(), ss->getSubRegOff(),
                     tmpDcl->getRegVar(),
-                    tmpDcl->getNumElems(), getGenxPlatform() >= GENX_CNL ? false : true);
+                    tmpDcl->getNumElems(), builder.getPlatform() >= GENX_ICLLP ? false : true);
             }
 
             // create new src from the temp address variable, with offset 0
@@ -500,7 +491,7 @@ void SpillManager::replaceSpilledFlagDst(G4_BB*         bb,
             // Need to pre-load the spill GRF if the inst isn't going to write the full
             // spilled GRF variable.
             if (flagDcl->getNumberFlagElements() > inst->getExecSize() ||
-                (bb->isInSimdFlow() && !inst->isWriteEnableInst()))
+                (!bb->isAllLaneActive() && !inst->isWriteEnableInst()))
             {
                 genRegMov(bb, it,
                     spDcl->getRegVar(), 0,
@@ -562,11 +553,11 @@ void SpillManager::createSpillLocations(G4_Kernel& kernel)
 
 bool isSpillCandidateForLifetimeOpRemoval(G4_INST* inst)
 {
-    if(inst->isPseudoKill())
+    if (inst->isPseudoKill())
     {
         return inst->getDst()->isSpilled();
     }
-    else if(inst->isLifeTimeEnd())
+    else if (inst->isLifeTimeEnd())
     {
         return inst->getSrc(0)->asSrcRegRegion()->isSpilled();
     }
@@ -600,6 +591,7 @@ void SpillManager::insertSpillCode()
         {
             G4_INST* inst = *inst_it;
 
+            currCISAOffset = inst->getCISAOff();
 
             G4_Operand * operands_analyzed[G4_MAX_SRCS] = {NULL, NULL, NULL};
             G4_Declare * declares_created[G4_MAX_SRCS] = {NULL, NULL, NULL};
@@ -615,7 +607,7 @@ void SpillManager::insertSpillCode()
             // Process predicate
             //
             G4_Predicate* predicate = inst->getPredicate();
-            if(predicate != NULL) {
+            if (predicate != NULL) {
                 replaceSpilledPredicate(bb, inst_it, inst);
             }
 
@@ -623,8 +615,8 @@ void SpillManager::insertSpillCode()
             // Process condMod
             //
             G4_CondMod* mod = inst->getCondMod();
-            if( mod != NULL &&
-                mod->getBase() != NULL ) {
+            if (mod != NULL &&
+                mod->getBase() != NULL) {
                 replaceSpilledFlagDst(bb, inst_it, inst);
             }
             inst_it++;
