@@ -51,6 +51,7 @@ class IR_Builder;
 class LivenessAnalysis;
 class Interference;
 class LiveRange;
+class LSLiveRange;
 class PointsToAnalysis;
 class GlobalRA;
 }
@@ -66,6 +67,7 @@ public:
 
     typedef std::list < G4_Declare * > DECLARE_LIST;
     typedef std::list < LiveRange * > LR_LIST;
+    typedef std::list < LSLiveRange* > LSLR_LIST;
     typedef std::list < G4_INST *, INST_LIST_NODE_ALLOCATOR > INST_LIST;
     typedef struct Edge
     {
@@ -83,8 +85,7 @@ public:
         const LivenessAnalysis * lvInfo,
         LiveRange **             lrInfo,
         Interference *           intf,
-        std::vector<EDGE> &      prevIntfEdges,
-        LR_LIST &                spilledLRs,
+        LR_LIST *                spilledLRs,
         unsigned                 iterationNo,
         bool                     useSpillReg,
         unsigned                 spillRegSize,
@@ -93,14 +94,30 @@ public:
         bool                     useScratchMsg
     );
 
+    SpillManagerGRF(
+        GlobalRA& g,
+        unsigned spillAreaOffset,
+        unsigned varIdCount,
+        const LivenessAnalysis* lvInfo,
+        LSLR_LIST* spilledLSLRs,
+        bool enableSpillSpaceCompression,
+        bool useScratchMsg);
+
     ~SpillManagerGRF () {}
 
+
     bool
     insertSpillFillCode    (
         G4_Kernel*         kernel,
         PointsToAnalysis& pointsToAnalysis
     );
 
+    void expireRanges(unsigned int idx, std::list<LSLiveRange*>* liveList);
+
+    void updateActiveList(LSLiveRange* lr, std::list<LSLiveRange*>* liveList);
+
+    bool spillLiveRanges(G4_Kernel* kernel);
+
     unsigned getNumGRFSpill() const { return numGRFSpill; }
     unsigned getNumGRFFill() const { return numGRFFill; }
     unsigned getNumGRFMove() const { return numGRFMove; }
@@ -125,21 +142,22 @@ public:
 
     static unsigned int createSpillSendMsgDescOWord(
         unsigned int height,
-        unsigned int& execSize);
+        G4_ExecSize& execSize);
 
 private:
 
     // Methods
 
     bool handleAddrTakenSpills( G4_Kernel * kernel, PointsToAnalysis& pointsToAnalysis );
+    unsigned int handleAddrTakenLSSpills(G4_Kernel* kernel, PointsToAnalysis& pointsToAnalysis);
     void insertAddrTakenSpillFill( G4_Kernel * kernel, PointsToAnalysis& pointsToAnalysis );
+    void insertAddrTakenLSSpillFill(G4_Kernel* kernel, PointsToAnalysis& pointsToAnalysis);
     void insertAddrTakenSpillAndFillCode( G4_Kernel* kernel, G4_BB* bb, INST_LIST::iterator inst_it,
         G4_Operand* opnd, PointsToAnalysis& pointsToAnalysis, bool spill, unsigned int bbid);
+    void insertAddrTakenLSSpillAndFillCode(G4_Kernel* kernel, G4_BB* bb, INST_LIST::iterator inst_it, G4_Operand* opnd, PointsToAnalysis& pointsToAnalysis, bool spill, unsigned int bbid);
     void prunePointsTo( G4_Kernel* kernel, PointsToAnalysis& pointsToAnalysis );
 
-    void
-    computeSpillIntf (
-    );
+    void prunePointsToLS(G4_Kernel* kernel, PointsToAnalysis& pointsToAnalysis);
 
     unsigned
     getMaxExecSize (
@@ -237,165 +255,81 @@ private:
     ) const;
 
     template <class REGION_TYPE>
-    G4_RegFileKind
-    getRFType (
-        REGION_TYPE * region
-    ) const;
+    G4_RegFileKind getRFType(REGION_TYPE * region) const;
 
     template <class REGION_TYPE>
-    unsigned
-    getRegionOriginOffset (
-        REGION_TYPE * region
-    ) const;
+    unsigned getRegionOriginOffset(REGION_TYPE * region) const;
 
-    unsigned
-    grfMask() const;
+    unsigned grfMask() const;
 
-    unsigned
-    hwordMask () const;
+    unsigned hwordMask() const;
 
-    unsigned
-    owordMask () const;
+    unsigned owordMask() const;
 
-    unsigned
-    dwordMask () const;
+    unsigned dwordMask() const;
 
-    bool
-    owordAligned (
-        unsigned offset
-    ) const;
+    bool owordAligned(unsigned offset) const;
 
-    bool
-    dwordAligned (
-        unsigned offset
-    ) const;
+    bool dwordAligned(unsigned offset) const;
 
-    static unsigned
-    cdiv (
-        unsigned dvd,
-        unsigned dvr
-    );
+    static unsigned cdiv(unsigned dvd, unsigned dvr);
 
-    G4_RegVar *
-    getRegVar (
-        unsigned id
-    ) const;
+    G4_RegVar *getRegVar(unsigned id) const;
 
-    bool
-    shouldSpillRegister (
-        G4_RegVar * regVar
-    ) const;
+    bool shouldSpillRegister(G4_RegVar * regVar) const;
 
-    unsigned
-    getByteSize (
-        G4_RegVar * regVar
-    ) const;
+    unsigned getByteSize(G4_RegVar * regVar) const;
 
     template <class REGION_TYPE>
-    unsigned
-    getSegmentDisp (
+    unsigned getSegmentDisp(REGION_TYPE * region, G4_ExecSize   execSize);
+
+    unsigned getDisp(G4_RegVar *   lRange);
+
+    template <class REGION_TYPE>
+    unsigned getRegionDisp(REGION_TYPE * region);
+
+    bool spillMemLifetimeInterfere(unsigned i, unsigned j) const;
+
+    unsigned calculateSpillDisp(G4_RegVar * lRange) const;
+
+    unsigned calculateSpillDispForLS(G4_RegVar* regVar) const;
+
+    template <class REGION_TYPE>
+    unsigned getMsgType(REGION_TYPE * region, G4_ExecSize   execSize);
+
+    template <class REGION_TYPE>
+    bool isUnalignedRegion(REGION_TYPE * region, G4_ExecSize   execSize);
+
+    template <class REGION_TYPE>
+    void calculateEncAlignedSegment(
         REGION_TYPE * region,
-        unsigned      execSize
-    ) ;
-
-    unsigned
-    getDisp (
-        G4_RegVar *   lRange
-    ) ;
-
-    template <class REGION_TYPE>
-    unsigned
-    getRegionDisp (
-        REGION_TYPE * region
-    ) ;
-
-    bool
-    spillMemLifetimeInterfere (
-        unsigned i,
-        unsigned j
-    ) const;
-
-    unsigned
-    calculateSpillDisp (
-        G4_RegVar * lRange
-    ) const;
-
-    template <class REGION_TYPE>
-    unsigned
-    getMsgType (
-        REGION_TYPE * region,
-        unsigned      execSize
-    );
-
-    template <class REGION_TYPE>
-    bool
-    isUnalignedRegion (
-        REGION_TYPE * region,
-        unsigned      execSize
-    );
-
-    template <class REGION_TYPE>
-    void
-    calculateEncAlignedSegment (
-        REGION_TYPE * region,
-        unsigned      execSize,
+        G4_ExecSize   execSize,
         unsigned &    start,
         unsigned &    end,
-        unsigned &    type
-    );
+        unsigned &    type);
 
     template <class REGION_TYPE>
-    unsigned
-    getEncAlignedSegmentByteSize (
-        REGION_TYPE * region,
-        unsigned      execSize
-    );
+    unsigned getEncAlignedSegmentByteSize(REGION_TYPE * region, G4_ExecSize   execSize);
 
     template <class REGION_TYPE>
-    unsigned
-    getEncAlignedSegmentDisp (
-        REGION_TYPE * region,
-        unsigned      execSize
-    );
+    unsigned getEncAlignedSegmentDisp(REGION_TYPE * region, G4_ExecSize   execSize);
 
     template <class REGION_TYPE>
-    unsigned
-    getEncAlignedSegmentMsgType (
-        REGION_TYPE * region,
-        unsigned      execSize
-    );
+    unsigned getEncAlignedSegmentMsgType(REGION_TYPE * region, G4_ExecSize   execSize);
 
     template <class REGION_TYPE>
-    unsigned
-    getSegmentByteSize (
-        REGION_TYPE * region,
-        unsigned      execSize
-    );
+    unsigned getSegmentByteSize(REGION_TYPE * region, G4_ExecSize   execSize);
 
-    unsigned
-    getRegionByteSize (
-        G4_DstRegRegion * region,
-        unsigned          execSize
-    ) const;
+    unsigned getRegionByteSize (G4_DstRegRegion * region, G4_ExecSize execSize) const;
 
-    unsigned
-    getRegionByteSize (
-        G4_SrcRegRegion * region,
-        unsigned          execSize
-    ) const;
+    unsigned getRegionByteSize(G4_SrcRegRegion * region, G4_ExecSize execSize) const;
 
-    bool
-    isScalarReplication (
-        G4_SrcRegRegion * region
-    ) const;
+    bool isScalarReplication(G4_SrcRegRegion * region) const;
 
-    bool
-    repeatSIMD16or32Source (
-        G4_SrcRegRegion * region
-    ) const;
+    bool repeatSIMD16or32Source(G4_SrcRegRegion * region) const;
 
     G4_Declare *
-    createRangeDeclare (
+    createRangeDeclare(
         const char*    name,
         G4_RegFileKind regFile,
         unsigned short nElems,
@@ -404,36 +338,31 @@ private:
         DeclareType    kind,
         G4_RegVar *    base,
         G4_Operand *   repRegion,
-        unsigned       execSize
-    );
+        G4_ExecSize    execSize);
 
     template <class REGION_TYPE>
     G4_Declare *
-    createTransientGRFRangeDeclare (
+    createTransientGRFRangeDeclare(
         REGION_TYPE * region,
         const char  * name,
         unsigned      index,
-        unsigned      execSize,
+        G4_ExecSize   execSize,
         G4_INST     * inst
     );
 
     G4_Declare *
-    createPostDstSpillRangeDeclare (
-        G4_INST *         sendOut,
-        G4_DstRegRegion * spillRegion
-    );
+    createPostDstSpillRangeDeclare(G4_INST * sendOut);
 
     G4_Declare *
-    createSpillRangeDeclare (
+    createSpillRangeDeclare(
         G4_DstRegRegion * spillRegion,
-        unsigned          execSize,
-        G4_INST         * inst
-    );
+        G4_ExecSize       execSize,
+        G4_INST         * inst);
 
     G4_Declare *
     createGRFFillRangeDeclare (
         G4_SrcRegRegion * fillRegion,
-        unsigned          execSize,
+        G4_ExecSize       execSize,
         G4_INST         * inst
     );
 
@@ -444,126 +373,93 @@ private:
     );
 
     G4_Declare *
-    createTemporaryRangeDeclare (
+    createTemporaryRangeDeclare(
         G4_DstRegRegion * fillRegion,
-        unsigned          execSize,
-        bool              forceSegmentAlignment = false
-    );
+        G4_ExecSize       execSize,
+        bool              forceSegmentAlignment = false);
 
     G4_DstRegRegion *
-    createSpillRangeDstRegion (
+    createSpillRangeDstRegion(
         G4_RegVar *       spillRangeRegVar,
         G4_DstRegRegion * spilledRegion,
-        unsigned          execSize,
-        unsigned          regOff = 0
-    );
+        G4_ExecSize       execSize,
+        unsigned          regOff = 0);
 
     G4_SrcRegRegion *
-    createFillRangeSrcRegion (
+    createFillRangeSrcRegion(
         G4_RegVar *       fillRangeRegVar,
         G4_SrcRegRegion * filledRegion,
-        unsigned          execSize
-    );
+        G4_ExecSize       execSize);
 
     G4_SrcRegRegion *
-    createTemporaryRangeSrcRegion (
+    createTemporaryRangeSrcRegion(
         G4_RegVar *       tmpRangeRegVar,
         G4_DstRegRegion * spilledRegion,
-        uint16_t          execSize,
-        unsigned          regOff = 0
-    );
+        G4_ExecSize       execSize,
+        unsigned          regOff = 0);
 
     G4_SrcRegRegion *
-    createBlockSpillRangeSrcRegion (
+    createBlockSpillRangeSrcRegion(
         G4_RegVar *       spillRangeRegVar,
-        unsigned          regOff            = 0,
-        unsigned          subregOff         = 0
-    );
+        unsigned          regOff = 0,
+        unsigned          subregOff = 0);
 
     G4_Declare *
-    createMRangeDeclare (
-        G4_RegVar *          regVar
-    );
+    createMRangeDeclare(G4_RegVar * regVar);
 
     G4_Declare *
-    createMRangeDeclare (
-        G4_DstRegRegion * region,
-        unsigned          execSize
-    );
+    createMRangeDeclare(G4_DstRegRegion * region, G4_ExecSize execSize);
 
     G4_Declare *
-    createMRangeDeclare (
-        G4_SrcRegRegion * region,
-        unsigned          execSize
-    );
+    createMRangeDeclare(G4_SrcRegRegion * region, G4_ExecSize execSize);
 
     G4_DstRegRegion *
-    createMPayloadBlockWriteDstRegion (
+    createMPayloadBlockWriteDstRegion(
         G4_RegVar *       grfRange,
         unsigned          regOff = 0,
-        unsigned          subregOff = 0
-    );
+        unsigned          subregOff = 0);
 
     G4_DstRegRegion *
-    createMHeaderInputDstRegion (
+    createMHeaderInputDstRegion(
         G4_RegVar *       grfRange,
-        unsigned          subregOff = 0
-    );
+        unsigned          subregOff = 0);
 
     G4_DstRegRegion *
-    createMHeaderBlockOffsetDstRegion (
-        G4_RegVar *          grfRange
-    );
+    createMHeaderBlockOffsetDstRegion(G4_RegVar * grfRange);
 
 
-    G4_SrcRegRegion *
-    createInputPayloadSrcRegion ();
+    G4_SrcRegRegion * createInputPayloadSrcRegion();
 
-    G4_Declare *
-    initMHeader (
-        G4_Declare *      mRangeDcl
-    );
+    G4_Declare * initMHeader(G4_Declare * mRangeDcl);
 
-    G4_Declare *
-    createAndInitMHeader (
-        G4_RegVar *          regVar
-    );
+    G4_Declare * createAndInitMHeader(G4_RegVar * regVar);
 
     template <class REGION_TYPE>
-    G4_Declare *
-    initMHeader (
+    G4_Declare * initMHeader(
         G4_Declare *      mRangeDcl,
-        REGION_TYPE *      region,
-        unsigned          execSize
-    );
+        REGION_TYPE *     region,
+        G4_ExecSize       execSize);
 
     template <class REGION_TYPE>
-    G4_Declare *
-    createAndInitMHeader (
+    G4_Declare * createAndInitMHeader(
         REGION_TYPE *      region,
-        unsigned          execSize
-    );
+        G4_ExecSize        execSize);
 
-    void
-    sendInSpilledRegVarPortions (
+    void sendInSpilledRegVarPortions(
         G4_Declare *      fillRangeDcl,
         G4_Declare *      mRangeDcl,
         unsigned          regOff,
         unsigned          height,
-        unsigned          srcRegOff = 0
-    );
+        unsigned          srcRegOff = 0);
 
-    void
-    sendOutSpilledRegVarPortions (
+    void sendOutSpilledRegVarPortions(
         G4_Declare *      spillRangeDcl,
         G4_Declare *      mRangeDcl,
         unsigned          regOff,
         unsigned          height,
-        unsigned          srcRegOff = 0
-    );
+        unsigned          srcRegOff = 0);
 
-    void
-    initMWritePayload (
+    void initMWritePayload(
         G4_Declare *      spillRangeDcl,
         G4_Declare *      mRangeDcl,
         unsigned          regOff,
@@ -575,159 +471,174 @@ private:
         G4_Declare *      spillRangeDcl,
         G4_Declare *      mRangeDcl,
         G4_DstRegRegion * spilledRangeRegion,
-        unsigned          execSize,
-        unsigned          regOff = 0
-    );
+        G4_ExecSize       execSize,
+        unsigned          regOff = 0);
 
-    static unsigned
-    blockSendBlockSizeCode (
-        unsigned        regOff
-    );
+    static unsigned blockSendBlockSizeCode(unsigned regOff);
 
-    unsigned
-    scatterSendBlockSizeCode (
-        unsigned        regOff
-    ) const;
+    unsigned scatterSendBlockSizeCode(unsigned regOff) const;
 
-    G4_Imm *
-    createSpillSendMsgDesc (
+    G4_Imm * createSpillSendMsgDesc(
         unsigned        regOff,
         unsigned        height,
-        unsigned &      execSize,
-        G4_RegVar*      base
-    );
+        G4_ExecSize &   execSize,
+        G4_RegVar*      base = NULL);
 
-    G4_Imm *
-    createSpillSendMsgDesc (
+    G4_Imm * createSpillSendMsgDesc(
         G4_DstRegRegion * spilledRangeRegion,
-        unsigned &        execSize
+        G4_ExecSize &     execSize);
+
+    G4_Imm * createSpillSendMsgDesc(int size, int offset);
+
+    G4_INST * createAddFPInst(
+        G4_ExecSize       execSize,
+        G4_DstRegRegion * dst,
+        G4_Operand *      src
     );
 
-    G4_INST *
-    createAddFPInst (
-        unsigned char      execSize,
-        G4_DstRegRegion *      dst,
-        G4_Operand *      src,
-        G4_Predicate *    predicate = NULL
-    );
-
-    G4_INST *
-    createMovInst (
-        unsigned char      execSize,
-        G4_DstRegRegion *      dst,
+    G4_INST * createMovInst(
+        G4_ExecSize       execSize,
+        G4_DstRegRegion * dst,
         G4_Operand *      src,
         G4_Predicate *    predicate = NULL,
-        unsigned int      options = InstOpt_WriteEnable
+        G4_InstOpts       options = InstOpt_WriteEnable
     );
 
-    G4_INST *
-        createSendInst(
-        unsigned char      execSize,
+    G4_INST * createSendInst(
+        G4_ExecSize       execSize,
         G4_DstRegRegion * postDst,
         G4_SrcRegRegion * payload,
         G4_Imm *          desc,
-        SFID funcID,
+        SFID              funcID,
         bool              isWrite,
-        unsigned          option
-        );
+        G4_InstOpts       option);
 
-    bool
-    shouldPreloadSpillRange (
-        G4_INST *         instContext
-    );
+    bool shouldPreloadSpillRange(
+        G4_INST* instContext,
+        G4_BB* parentBB);
 
-    void
-    preloadSpillRange (
+    void preloadSpillRange(
         G4_Declare *      spillRangeDcl,
         G4_Declare *      mRangeDcl,
         G4_DstRegRegion * spilledRangeRegion,
-        uint8_t     execSize
-    );
+        G4_ExecSize       execSize);
 
-    G4_INST *
-    createSpillSendInstr (
+    G4_INST * createSpillSendInstr(
         G4_Declare *      spillRangeDcl,
         G4_Declare *      mRangeDcl,
         unsigned          regOff,
         unsigned          height,
-        unsigned          srcRegOff = 0
-    );
+        unsigned          spillOff);
 
-    G4_INST *
-    createSpillSendInstr (
+    G4_INST *createSpillSendInstr (
         G4_Declare *      spillRangeDcl,
         G4_Declare *      mRangeDcl,
         G4_DstRegRegion * spilledRangeRegion,
-        unsigned          execSize,
-        unsigned          option
+        G4_ExecSize       execSize,
+        unsigned          option);
+
+    G4_Imm *createFillSendMsgDesc (
+        unsigned          regOff,
+        unsigned          height,
+        G4_ExecSize &     execSize,
+        G4_RegVar* base = NULL);
+
+    template <class REGION_TYPE>
+    G4_Imm * createFillSendMsgDesc (
+        REGION_TYPE *     filledRangeRegion,
+        G4_ExecSize &     execSize
     );
 
-    G4_INST *
-        createFillSendInstr (
+    G4_Imm * createFillSendMsgDesc(
+        int size,
+        int memOffset);
+
+    G4_INST * createFillSendInstr (
         G4_Declare *      fillRangeDcl,
         G4_Declare *      mRangeDcl,
         unsigned          regOff,
         unsigned          height,
-        unsigned          srcRegOff = 0
-    );
+        unsigned          spillOff);
 
-    G4_INST *
-    createFillSendInstr (
+    G4_INST * createFillSendInstr(
         G4_Declare *       fillRangeDcl,
         G4_Declare *       mRangeDcl,
         G4_SrcRegRegion *  filledRangeRegion,
-        unsigned           execSize,
-        unsigned           regOff = 0
-    );
+        G4_ExecSize        execSize);
 
-    G4_INST* createFillInstr(G4_Declare* fillRangeDcl, G4_Declare* mRangeDcl, unsigned regOff, unsigned height, unsigned srcRegOff = 0);
-    G4_INST* createFillInstr(G4_Declare* fillRangeDcl, G4_Declare* mRangeDcl, G4_SrcRegRegion* filledRangeRegion, unsigned execSize);
+    G4_INST* createFillInstr(
+        G4_Declare* fillRangeDcl,
+        G4_Declare* mRangeDcl,
+        unsigned regOff,
+        unsigned height,
+        unsigned srcRegOff);
 
-    void
-    replaceSpilledRange (
+    void replaceSpilledRange(
         G4_Declare *      spillRangeDcl,
         G4_DstRegRegion * spilledRegion,
-        G4_INST *         spilledInst
-    );
+        G4_INST *         spilledInst);
 
-    void
-    replaceFilledRange (
+    void replaceFilledRange(
         G4_Declare *      fillRangeDcl,
         G4_SrcRegRegion * filledRegion,
-        G4_INST *         filledInst
-    );
+        G4_INST *         filledInst);
 
-    INST_LIST::iterator
-    insertSpillRangeCode (
+    INST_LIST::iterator insertSpillRangeCode(
         INST_LIST::iterator spilledInstIter,
         G4_BB* bb
     );
 
-    INST_LIST::iterator
-    insertSendFillRangeCode (
+    INST_LIST::iterator insertSendFillRangeCode(
         G4_SrcRegRegion *   filledRegion,
         INST_LIST::iterator filledInstIter,
-        G4_BB* bb
-    );
+        G4_BB* bb);
 
-    INST_LIST::iterator
-    insertFillGRFRangeCode (
+    INST_LIST::iterator insertFillGRFRangeCode(
         G4_SrcRegRegion *   filledRegion,
         INST_LIST::iterator filledInstIter,
-        G4_BB* bb
-    );
+        G4_BB* bb);
 
-    void *
-    allocMem (
-        unsigned size
-    ) const;
+    void *allocMem (unsigned size) const;
 
-    SpillManagerGRF (
-        const SpillManagerGRF & other
-    );
+    SpillManagerGRF(const SpillManagerGRF & other);
 
     bool useSplitSend() const;
 
+    int getHWordEncoding(int numHWord)
+    {
+        switch (numHWord)
+        {
+        case 1:
+            return 0;
+        case 2:
+            return 1;
+        case 4:
+            return 2;
+        case 8:
+            return 3;
+        default:
+            MUST_BE_TRUE(false, "only 1/2/4/8 HWords are supported");
+            return 0;
+        }
+    }
+
+    ChannelMask getChMaskForSpill(int numHWord) const
+    {
+        switch (numHWord)
+        {
+        case 1:
+        case 2:
+            return ChannelMask::createFromAPI(CHANNEL_MASK_R);
+        case 4:
+            return ChannelMask::createFromAPI(CHANNEL_MASK_RG);
+        case 8:
+            return ChannelMask::createFromAPI(CHANNEL_MASK_RGBA);
+        default:
+            assert(false && "illegal spill size");
+            return ChannelMask::createFromAPI(CHANNEL_MASK_R);
+        }
+    }
+
     // Data
     GlobalRA&                gra;
     IR_Builder *             builder_;
@@ -735,8 +646,8 @@ private:
     unsigned                 latestImplicitVarIdCount_;
     const LivenessAnalysis * lvInfo_;
     LiveRange **             lrInfo_;
-    std::vector<EDGE> &      prevIntfEdges_;
-    LR_LIST &                spilledLRs_;
+    LR_LIST *                spilledLRs_;
+    LSLR_LIST*               spilledLSLRs_;
     unsigned *               spillRangeCount_;
     unsigned *               fillRangeCount_;
     unsigned *               tmpRangeCount_;
@@ -746,14 +657,13 @@ private:
     unsigned                 iterationNo_;
     unsigned                 bbId_;
     unsigned                 spillAreaOffset_;
-    bool                     inSIMDCFContext_;
     bool                     doSpillSpaceCompression;
 
     bool                     failSafeSpill_;
     unsigned                 spillRegStart_;
     unsigned                 indrSpillRegStart_;
     unsigned                 spillRegOffset_;
-
+    LSLR_LIST                activeLR_;
 
     Interference *             spillIntf_;
     vISA::Mem_Manager              mem_;
@@ -774,21 +684,27 @@ private:
 
     bool useScratchMsg_;
 
+
     bool headerNeeded()
     {
         bool needed = true;
 
-        if (useScratchMsg_ && getGenxPlatform() >= GENX_SKL)
+        if (useScratchMsg_ && builder_->getPlatform() >= GENX_SKL)
             needed = false;
 
         if (builder_->kernel.fg.getHasStackCalls() ||
             builder_->kernel.fg.getIsStackCallFunc())
             needed = false;
 
+
         return needed;
     }
 
-    void saveRestoreA0(G4_BB*);
+    // return true if offset for spill/fill message needs to be 32byte aligned
+    bool need32ByteAlignedOffset() const
+    {
+        return useScratchMsg_ || useSplitSend();
+    }
 };
 }
 bool isDisContRegion (
@@ -821,4 +737,6 @@ bool isPartialRegion (
     }
 }
 
+G4_SrcRegRegion* getSpillFillHeader(IR_Builder& builder, G4_Declare* decl);
+
 #endif // __SPILLMANAGERGMRF_H__
