@@ -27,26 +27,31 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #ifndef VISA_KERNEL_H
 #define VISA_KERNEL_H
 #include "VISABuilderAPIDefinition.h"
-#include "DebugInfo.h"
+#include "Common_ISA_util.h"
+#include "BuildCISAIR.h"
 #include "visa_wa.h"
-
+#include "Mem_Manager.h"
+#include "JitterDataStruct.h"
+//#include "Gen4_IR.hpp"  // for PhyRegPool
+#include "Attributes.hpp"
 #include "CompilerStats.h"
 
+#include <list>
+#include <map>
+#include <string>
+#include <vector>
+#include <unordered_set>
+
 #define MAX_ERROR_MSG_LEN            511
 #define vISA_NUMBER_OF_OPNDS_IN_POOL 47
 
 //forward declaration
 namespace vISA
 {
-class G4_Declare;
-class G4_Inst;
 class G4_Kernel;
-class IR_Builder;
 class DebugInfoFormat;
-class BinaryEncoding;
 class BinaryEncodingBase;
 }
-class CISA_IR_Builder;
 
 // Class hierarchy is as follows:
 // VISAKernel -> Abstract class that declares virtual functions to build a kernel object
@@ -60,7 +65,6 @@ public:
     VISAKernelImpl(CISA_IR_Builder* cisaBuilder, VISA_BUILDER_OPTION buildOption, Options *option)
         : m_mem(4096), m_CISABuilder(cisaBuilder), m_options(option)
     {
-        //CisaBinary* module = NULL;
         mBuildOption = buildOption;
         m_magic_number = COMMON_ISA_MAGIC_NUM;
         m_major_version = 0;
@@ -76,7 +80,7 @@ public:
 
         m_string_pool_size = 0;
         m_var_info_size = 0;
-        m_adress_info_size = 0;
+        m_address_info_size = 0;
         m_predicate_info_size = 0;
         m_label_info_size = 0;
         m_input_info_size = 0;
@@ -101,7 +105,7 @@ public:
         m_kernel = NULL;
         m_builder = NULL;
         m_globalMem = NULL;
-        m_phyRegPool = NULL;
+
         m_kernelID = 0;
         m_inputSize = 0;
         m_opndCounter = 0;
@@ -117,8 +121,7 @@ public:
         predicateNameCount = COMMON_ISA_NUM_PREDEFINED_PRED;
         surfaceNameCount = COMMON_ISA_NUM_PREDEFINED_SURF_VER_3_1;
         samplerNameCount = 0;
-        mTargetAttributeSet = false;
-
+        unknownNameCount = 0;
         m_functionId = 0;
         m_vISAInstCount = -1;
 
@@ -127,25 +130,28 @@ public:
         mIsFCComposableKernel = false;
 
         // Initialize first level scope of the map
-        m_GenNamedVarMap.push_back(GenDeclNameToVarMap());
+        // m_GenNamedVarMap.push_back(GenDeclNameToVarMap());
+        m_GenNamedVarMap.emplace_back();
+
+        createKernelAttributes();
+        createReservedKeywordSet();
     }
 
     void* alloc(size_t sz) { return m_mem.alloc(sz); }
 
     virtual ~VISAKernelImpl();
 
-    void *operator new(size_t sz, vISA::Mem_Manager& m){ return m.alloc(sz); };
+    void *operator new(size_t sz, vISA::Mem_Manager& m) { return m.alloc(sz); };
     int InitializeKernel(const char *kernel_name);
     int CISABuildPreDefinedDecls();
-    void setVersion(unsigned char major_ver, unsigned char minor_ver){
+    void setVersion(unsigned char major_ver, unsigned char minor_ver) {
         m_major_version = major_ver;
         m_minor_version = minor_ver;
     }
 
-    void setPWaTable(PVISA_WA_TABLE pWaTable){
-        m_pWaTable = pWaTable;
-    }
-
+    vISA::Attributes* getKernelAttributes() { return m_kernelAttrs; }
+    // Temporary function to move options to attributes!
+    void finalizeAttributes();
     void finalizeKernel();
     unsigned long writeInToCisaBinaryBuffer(const void * value, int size);
     unsigned long getBytesWritten() { return m_bytes_written_cisa_buffer; }
@@ -213,7 +219,7 @@ public:
     void addPendingLabelNames(std::string name) { m_pending_label_names.push_back(name); }
     void setIsKernel(bool isKernel) { m_isKernel = isKernel; };
     bool getIsKernel() const { return m_isKernel; }
-    unsigned long getCodeOffset(){ return m_cisa_kernel.entry; }
+    unsigned long getCodeOffset() { return m_cisa_kernel.entry; }
 
     CISA_GEN_VAR * getDeclFromName(const std::string &name);
     bool setNameIndexMap(const std::string &name, CISA_GEN_VAR *, bool unique = false);
@@ -230,8 +236,6 @@ public:
 
     int getVISAOffset() const;
 
-    void processAttributes();
-
     /***************** START EXPOSED APIS *************************/
     VISA_BUILDER_API int CreateVISAGenVar(VISA_GenVar *& decl, const char *varName, int numberElements, VISA_Type dataType,
         VISA_Align varAlign, VISA_GenVar *parentDecl = NULL, int aliasOffset = 0);
@@ -325,7 +329,7 @@ public:
     VISA_BUILDER_API int AppendVISAArithmeticInst(ISA_Opcode opcode, VISA_PredOpnd *pred, bool satMode, VISA_EMask_Ctrl emask,
         VISA_Exec_Size executionSize, VISA_VectorOpnd *tmpDst, VISA_VectorOpnd *src0, VISA_VectorOpnd *src1, VISA_VectorOpnd *src2);
 
-    VISA_BUILDER_API int AppendVISAArithmeticInst(ISA_Opcode opcode, VISA_PredOpnd *pred, VISA_EMask_Ctrl emask,
+    VISA_BUILDER_API int AppendVISATwoDstArithmeticInst(ISA_Opcode opcode, VISA_PredOpnd *pred, VISA_EMask_Ctrl emask,
         VISA_Exec_Size executionSize, VISA_VectorOpnd *dst1, VISA_VectorOpnd *carry_borrow, VISA_VectorOpnd *src0, VISA_VectorOpnd *src1);
 
     VISA_BUILDER_API int AppendVISALogicOrShiftInst(ISA_Opcode opcode, VISA_PredOpnd *pred, bool satMode, VISA_EMask_Ctrl emask,
@@ -481,6 +485,8 @@ public:
 
     VISA_BUILDER_API int AppendVISADebugLinePlaceholder();
 
+    VISA_BUILDER_API int AppendVISALLVMInst(void *inst);
+
     VISA_BUILDER_API int AppendVISAMiscRawSend(VISA_PredOpnd *pred, VISA_EMask_Ctrl emask, VISA_Exec_Size executionSize, unsigned char modifiers,
         unsigned int exMsgDesc, unsigned char srcSize, unsigned char dstSize, VISA_VectorOpnd *desc,
         VISA_RawOpnd *src, VISA_RawOpnd *dst);
@@ -575,6 +581,13 @@ public:
     VISA_BUILDER_API int AppendVISALifetime(VISAVarLifetime startOrEnd, VISA_VectorOpnd *varId);
 
 
+
+
+
+
+
+
+
     /********** APPEND INSTRUCTION APIS END   ******************/
 
     /********** APPEND 3D Instructions START ******************/
@@ -594,7 +607,8 @@ public:
         int numMsgSpecificOpnds,
         VISA_RawOpnd **opndArray);
 
-    VISA_BUILDER_API int AppendVISA3dLoad(VISASampler3DSubOpCode subOpcode,
+    VISA_BUILDER_API int AppendVISA3dLoad(
+        VISASampler3DSubOpCode subOpcode,
         bool pixelNullMask,
         VISA_PredOpnd *pred,
         VISA_EMask_Ctrl emask,
@@ -606,7 +620,8 @@ public:
         int numMsgSpecificOpnds,
         VISA_RawOpnd ** opndArray);
 
-    VISA_BUILDER_API int AppendVISA3dGather4(VISASampler3DSubOpCode subOpcode,
+    VISA_BUILDER_API int AppendVISA3dGather4(
+        VISASampler3DSubOpCode subOpcode,
         bool pixelNullMask,
         VISA_PredOpnd *pred,
         VISA_EMask_Ctrl emask,
@@ -621,15 +636,18 @@ public:
 
     VISA_BUILDER_API int AppendVISA3dInfo(VISASampler3DSubOpCode subOpcode, VISA_EMask_Ctrl emask, VISA_Exec_Size executionSize, VISAChannelMask srcChannel, VISA_StateOpndHandle *surface, VISA_RawOpnd *lod, VISA_RawOpnd *dst);
 
-    VISA_BUILDER_API int AppendVISA3dRTWrite(VISA_PredOpnd *pred, VISA_EMask_Ctrl emask, VISA_Exec_Size executionSize, VISA_VectorOpnd* renderTargetIndex, vISA_RT_CONTROLS cntrls,
+    VISA_BUILDER_API int AppendVISA3dRTWrite(
+        VISA_PredOpnd *pred, VISA_EMask_Ctrl emask, VISA_Exec_Size executionSize, VISA_VectorOpnd* renderTargetIndex, vISA_RT_CONTROLS cntrls,
         VISA_StateOpndHandle *surface, VISA_RawOpnd *r1HeaderOpnd, VISA_VectorOpnd *sampleIndex,
         uint8_t numMsgSpecificOpnds, VISA_RawOpnd **opndArray);
 
-    VISA_BUILDER_API int AppendVISA3dRTWriteCPS(VISA_PredOpnd *pred, VISA_EMask_Ctrl emask, VISA_Exec_Size executionSize, VISA_VectorOpnd* renderTargetIndex, vISA_RT_CONTROLS cntrls,
+    VISA_BUILDER_API int AppendVISA3dRTWriteCPS(
+        VISA_PredOpnd *pred, VISA_EMask_Ctrl emask, VISA_Exec_Size executionSize, VISA_VectorOpnd* renderTargetIndex, vISA_RT_CONTROLS cntrls,
         VISA_StateOpndHandle *surface, VISA_RawOpnd *r1HeaderOpnd, VISA_VectorOpnd *sampleIndex,
         VISA_VectorOpnd *cPSCounter, uint8_t numMsgSpecificOpnds, VISA_RawOpnd **opndArray);
 
-    VISA_BUILDER_API int AppendVISA3dURBWrite(VISA_PredOpnd *pred, VISA_EMask_Ctrl emask,
+    VISA_BUILDER_API int AppendVISA3dURBWrite(
+        VISA_PredOpnd *pred, VISA_EMask_Ctrl emask,
         VISA_Exec_Size executionSize, unsigned char numberOutputParams,
         VISA_RawOpnd *channelMask, unsigned short globalOffset, VISA_RawOpnd *URBHandle,
         VISA_RawOpnd *perSLotOffset, VISA_RawOpnd *vertexData);
@@ -644,13 +662,16 @@ public:
     /********** APPEND 3D Instructions END ******************/
 
     /********** MISC APIs START *************************/
-    VISA_BUILDER_API int GetGenxBinary(void *&buffer, int &size);
+    VISA_BUILDER_API int GetGenxBinary(void *&buffer, int &size) const;
     VISA_BUILDER_API int GetJitInfo(FINALIZER_INFO *&jitInfo);
     VISA_BUILDER_API int GetCompilerStats(CompilerStats &compilerStats);
     VISA_BUILDER_API int GetErrorMessage(const char *&errorMsg) const;
-    VISA_BUILDER_API virtual int GetGenxDebugInfo(void *&buffer, unsigned int &size, void*&, unsigned int&);
+    VISA_BUILDER_API virtual int GetGenxDebugInfo(void *&buffer, unsigned int &size, void*&, unsigned int&) const;
     /// GetGenRelocEntryBuffer -- allocate and return a buffer of all GenRelocEntry that are created by vISA
     VISA_BUILDER_API int GetGenRelocEntryBuffer(void *&buffer, unsigned int &byteSize, unsigned int &numEntries);
+    /// GetRelocations -- add vISA created relocations into given relocation list
+    /// This get the same information as GetGenRelocEntryBuffer, but in different foramt
+    VISA_BUILDER_API int GetRelocations(RelocListType &relocs);
     VISA_BUILDER_API int GetGTPinBuffer(void*& buffer, unsigned int& size);
     VISA_BUILDER_API int SetGTPinInit(void* buffer);
     VISA_BUILDER_API int GetFreeGRFInfo(void*& buffer, unsigned int& size);
@@ -681,6 +702,9 @@ public:
     ///Gets gen binary size within instruction heap
     VISA_BUILDER_API int64_t getGenSize() const;
 
+    /// Get global function name
+    VISA_BUILDER_API const char* getFunctionName() const;
+
     //Gets the VISA string format for the variable
     VISA_BUILDER_API std::string getVarName(VISA_GenVar* decl) const;
     VISA_BUILDER_API std::string getVarName(VISA_PredVar* decl) const;
@@ -725,7 +749,16 @@ public:
         unsigned int numMsgSpecificOpnds,
         VISA_RawOpnd **opndArray);
 
-    int AddAttributeToVarGeneric(CISA_GEN_VAR *decl, const char* varName, unsigned int size, void *val);
+    attribute_info_t* allocAttribute(CISA_GEN_VAR* Dcl)
+    {
+        return allocAttributeImpl(Dcl, 0);
+    }
+    attribute_info_t* resizeAttribute(CISA_GEN_VAR* Dcl, uint32_t AllocMaxNum)
+    {
+        return allocAttributeImpl(Dcl, AllocMaxNum);
+    }
+
+    int AddAttributeToVarGeneric(CISA_GEN_VAR *decl, const char* varName, unsigned int size, const void *val);
 
     int CreateStateVar(CISA_GEN_VAR *&decl, Common_ISA_Var_Class type, const char* name, unsigned int numberElements);
 
@@ -737,9 +770,9 @@ public:
 
     int CreateDummyLabelOperand(VISA_LabelOpnd *& opnd, char *name, VISA_Label_Kind kind);
 
-    void setGenxBinaryBuffer(void *buffer, int size){ m_genx_binary_buffer = (char *)buffer; m_genx_binary_size = size; }
-    void setJitInfo(FINALIZER_INFO* jitInfo){ m_jitInfo = jitInfo; }
-    // char * getErrorMsgPtr(){ return errorMessage; }
+    void setGenxBinaryBuffer(void *buffer, int size) { m_genx_binary_buffer = (char *)buffer; m_genx_binary_size = size; }
+    void setJitInfo(FINALIZER_INFO* jitInfo) { m_jitInfo = jitInfo; }
+    // char * getErrorMsgPtr() { return errorMessage; }
 
     std::string getOutputAsmPath() const { return m_asmName; }
 
@@ -749,8 +782,6 @@ public:
     unsigned char m_major_version;
     unsigned char m_minor_version;
 
-    PVISA_WA_TABLE m_pWaTable;
-
     void* compilePostOptimize(unsigned int& binarySize);
 
     void setInputSize(uint8_t size);
@@ -837,14 +868,25 @@ public:
         return (*it);
     }
 
+    std::string getVarName(CISA_GEN_VAR* decl) const
+    {
+        assert(m_GenVarToNameMap.count(decl) && "Can't find the decl's name");
+        return m_GenVarToNameMap.find(decl)->second;
+    }
+
     Options * getOptions() { return m_options; }
 
     bool IsAsmWriterMode() const { return m_options->getOption(vISA_IsaAssembly); }
 
-    void computeAndEmitDebugInfo(std::list<VISAKernelImpl*>& functions);
+    typedef std::list<VISAKernelImpl*> VISAKernelImplListTy;
+    void computeAndEmitDebugInfo(VISAKernelImplListTy& functions);
 
 private:
-    void setDefaultVariableName(Common_ISA_Var_Class Ty, const char *&varName);
+    void createReservedKeywordSet();
+    bool isReservedName(const std::string &nm) const;
+    void ensureVariableNameUnique(const char *&varName);
+    void generateVariableName(Common_ISA_Var_Class Ty, const char *&varName);
+
     void dumpDebugFormatFile(std::vector<vISA::DebugInfoFormat>& debugSymbols, std::string filename);
     void patchLabels();
     int InitializeFastPath();
@@ -852,8 +894,11 @@ private:
     int predefinedVarRegAssignment();
     int calculateTotalInputSize();
     int compileTillOptimize();
+    void recordFinalizerInfo();
+
+    // Re-adjust indirect call target after swsb
+    void adjustIndirectCallOffset();
 
-    void getHeightWidth(G4_Type type, unsigned int numberElements, unsigned short &dclWidth, unsigned short &dclHeight, int &totalByteSize);
     CisaFramework::CisaInst* AppendVISASvmGeneralScatterInst(VISA_PredOpnd* pred,
         VISA_EMask_Ctrl emask, VISA_Exec_Size execSize, unsigned char blockSize,
         unsigned char numBlocks, VISA_RawOpnd* address, VISA_RawOpnd *srcDst, bool isRead);
@@ -876,19 +921,17 @@ private:
 
     kernel_format_t m_cisa_kernel;
 
-
-    bool mTargetAttributeSet;
     unsigned int m_num_pred_vars;
     //size of various arrays in kernel header.
     //used for buffer size allocation.
-    unsigned int m_string_pool_size; //done
-    unsigned int m_var_info_size; //done
-    unsigned int m_adress_info_size;
-    unsigned int m_predicate_info_size; //done
-    unsigned int m_label_info_size; //done
-    unsigned int m_input_info_size; //done
-    unsigned int m_attribute_info_size; //done
-    unsigned int m_instruction_size; //done
+    unsigned int m_string_pool_size;
+    unsigned int m_var_info_size;
+    unsigned int m_address_info_size;
+    unsigned int m_predicate_info_size;
+    unsigned int m_label_info_size;
+    unsigned int m_input_info_size;
+    unsigned int m_attribute_info_size;
+    unsigned int m_instruction_size;
     unsigned int m_surface_info_size;
     unsigned int m_sampler_info_size;
 
@@ -945,7 +988,7 @@ private:
 
     unsigned int m_input_count;
     std::vector<input_info_t *> m_input_info_list;
-    //std::map<unsigned int, unsigned int> m_declID_to_inputID_map;
+    // std::map<unsigned int, unsigned int> m_declID_to_inputID_map;
 
     unsigned int m_attribute_count;
     std::list<attribute_info_t *> m_attribute_info_list;
@@ -953,7 +996,7 @@ private:
     unsigned int m_label_count;
     std::vector<label_info_t *> m_label_info_list;
 
-    //list of cisa operands representing labels that need to be resolved
+    // list of cisa operands representing labels that need to be resolved
     std::list<VISA_opnd *> m_pending_labels;
     std::list<std::string> m_pending_label_names;
 
@@ -964,8 +1007,12 @@ private:
     std::vector<GenDeclNameToVarMap> m_GenNamedVarMap;
     GenDeclNameToVarMap m_UniqueNamedVarMap;
 
-    std::map<std::string, VISA_LabelOpnd *> m_label_name_to_index_map;
-    std::map<std::string, VISA_LabelOpnd *> m_funcName_to_labelID_map;
+    // reverse map from a GenVar to its declared name, used in inline assembly
+    // Note that name is only unique within the same scope
+    std::map<CISA_GEN_VAR*, std::string> m_GenVarToNameMap;
+
+    std::unordered_map<std::string, VISA_LabelOpnd *> m_label_name_to_index_map;
+    std::unordered_map<std::string, VISA_LabelOpnd *> m_funcName_to_labelID_map;
 
     char errorMessage[MAX_ERROR_MSG_LEN];
 
@@ -975,7 +1022,6 @@ private:
     vISA::IR_Builder* m_builder;
     vISA::Mem_Manager *m_globalMem;
     vISA::Mem_Manager *m_kernelMem;
-    vISA::PhyRegPool *m_phyRegPool;
     //customized allocator for allocating
     //It is very important that the same allocator is used by all instruction lists
     //that might be joined/spliced.
@@ -990,6 +1036,12 @@ private:
     unsigned int predicateNameCount;
     unsigned int surfaceNameCount;
     unsigned int samplerNameCount;
+    unsigned int unknownNameCount;
+
+    // TODO: this should be merged and re-worked to fit into the symbol table
+    // scheme
+    std::unordered_set<std::string> varNames;
+    std::unordered_set<std::string> reservedNames;
 
     int m_vISAInstCount;
     print_decl_index_t m_printDeclIndex;
@@ -1003,7 +1055,21 @@ private:
     //memory managed by the entity that creates vISA Kernel object
     Options *m_options;
 
-    bool getIntKernelAttributeValue(const char* attrName, int& value);
+    void createKernelAttributes() {
+        void* pmem = m_mem.alloc(sizeof(vISA::Attributes));
+        m_kernelAttrs = new (pmem) vISA::Attributes();
+    }
+    void destroyKernelAttributes() {
+        // Even though attributes is allocated from m_mem, need to invoke dtor.
+        if (m_kernelAttrs) {
+            m_kernelAttrs->~Attributes();
+            m_kernelAttrs = nullptr;
+        }
+    }
+    attribute_info_t* allocAttributeImpl(CISA_GEN_VAR* Dcl, uint32_t AllocNum);
+
+    // Shared with G4_kernel
+    vISA::Attributes* m_kernelAttrs;
 };
 
 class VISAKernel_format_provider : public print_format_provider_t
