@@ -435,9 +435,7 @@ namespace {
             std::string id = "LABEL__EMPTYBB__" + toString(BB->getId());
             G4_Label *label = IRB->createLabel(id, LABEL_BLOCK);
             G4_INST *inst =
-                IRB->createInternalInst(nullptr, G4_label, nullptr, false,
-                                UNDEFINED_EXEC_SIZE, nullptr,
-                                label, nullptr, 0);
+                IRB->createLabelInst(label, false);
             BB->push_back(inst);
         }
 
@@ -566,7 +564,7 @@ void IfConverter::fullConvert(IfConvertible &IC) {
                 I->setPredicate(fg.builder->createPredicate(pred));
             }
         }
-        head->insert(pos, I);
+        head->insertBefore(pos, I);
     }
     markEmptyBB(fg.builder, s0);
     // Merge predicated 'else' into header.
@@ -598,7 +596,7 @@ void IfConverter::fullConvert(IfConvertible &IC) {
                     I->setPredicate(fg.builder->createPredicate(pred));
                 }
             }
-            head->insert(pos, I);
+            head->insertBefore(pos, I);
         }
         markEmptyBB(fg.builder, s1);
     }
