@@ -6,16 +6,6 @@
 #include "common/ged_compact_mapping_table.h"
 #include "common/ged_ins_encoding_masks.h"
 #include "common/ged_interpreter_types.h"
-#include "common/ged_ins_disassembly_table.h"
-
-
-# if GED_DISASSEMBLY
-enum GED_NUMERIC_TYPE
-{
-    GED_NUMERIC_TYPE_INT = 0,
-    GED_NUMERIC_TYPE_FP  = 1
-};
-# endif // GED_DISASSEMBLY
 
 
 union SignedDNum
@@ -35,9 +25,6 @@ union SignedQNum
 typedef const ged_ins_decoding_table_t (*InsFieldTableFunc)(uint32_t, GED_INS_TYPE);
 typedef const ged_compact_mapping_table_t (*MappingTableFunc)(uint32_t);
 typedef const ged_instruction_masks_table_t (*EncodingMasksTableFunc)(uint32_t, GED_INS_TYPE);
-# if GED_DISASSEMBLY
-typedef const ged_disassembly_table_t (*DisassemblyTableFunc)(uint32_t);
-# endif // GED_DISASSEMBLY
 
 struct OpcodeTables
 {
@@ -62,15 +49,6 @@ struct ModelData
     const ged_collector_info_t* collectionTables;
 };
 
-# if GED_DISASSEMBLY
-struct ModelDisassemblyData
-{
-    DisassemblyTableFunc DisassemblyTable;
-};
-
-extern ModelDisassemblyData ModelsDisassemblyArray[];
-# endif // GED_DISASSEMBLY
-
 
 extern ModelData ModelsArray[];
 
@@ -85,11 +63,6 @@ extern GED_FIELD_TYPE fieldTypesByField[];
 extern GED_FIELD_TYPE pseudoFieldTypesByField[];
 
 
-# if GED_DISASSEMBLY // TODO: This should be guarded by a different define, probably GED_LOG.
-extern const char** stringGettersByField[];
-extern const char** stringGettersByPseudoField[];
-# endif // GED_DISASSEMBLY
-
 # if GED_VALIDATION_API
 extern const char* fieldNameByField[];
 extern const char* fieldNameByPseudoField[];
