@@ -26,11 +26,98 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #ifndef _IGA_BACKEND_GED_DECODER_HPP_
 #define _IGA_BACKEND_GED_DECODER_HPP_
 
-#include "DecoderCommon.hpp"
 #include "GEDBitProcessor.hpp"
 #include "GEDToIGATranslation.hpp"
 #include "ged.h"
 
+#ifdef GED_PRINT_BIT_LOCATION
+#undef GED_PRINT_BIT_LOCATION
+#endif
+#if GED_VALIDATION_API
+#define GED_PRINT_BIT_LOCATION(FIELD) \
+    do { \
+        std::cout << "FIELD: " << #FIELD << "\n"; \
+        GED_PrintFieldBitLocation(&m_currGedInst, GED_INS_FIELD_ ## FIELD); \
+    } while(0)
+#else
+#define GED_PRINT_BIT_LOCATION(FIELD)
+#endif
+
+#define GED_DECODE_TO(FIELD, TRANS, DST) \
+    do { \
+        GED_RETURN_VALUE _status; \
+        GED_PRINT_BIT_LOCATION(FIELD); \
+        DST = TRANS(GED_Get ## FIELD(&m_currGedInst, &_status)); \
+        if (_status != GED_RETURN_VALUE_SUCCESS) { \
+            handleGedDecoderError(__LINE__, #FIELD, _status); \
+        } \
+    } while (0)
+
+#define GED_DECODE_RAW_TO(FIELD, DST) \
+    do { \
+        GED_RETURN_VALUE _status; \
+        GED_PRINT_BIT_LOCATION(FIELD); \
+        DST = GED_Get ## FIELD(&m_currGedInst, &_status); \
+        if (_status != GED_RETURN_VALUE_SUCCESS) { \
+            handleGedDecoderError(__LINE__, #FIELD, _status); \
+        } \
+    } while (0)
+
+#define GED_DECODE_RAW(GED_TYPE, ID, FIELD) \
+    GED_TYPE ID; \
+    GED_DECODE_RAW_TO(FIELD, ID);
+
+#define GED_DECODE(IGA_TYPE, GED_TYPE, ID, FIELD) \
+    GED_DECODE_RAW(GED_TYPE, GED_ ## ID, FIELD); \
+    IGA_TYPE ID = translate(GED_ ## ID);
+
+#define GED_DECODE_RAW_TO_SRC(DST, TYPE, FIELD) \
+    do { \
+        GED_RETURN_VALUE _STATUS; \
+        GED_PRINT_BIT_LOCATION(FIELD); \
+        DST = GED_Get ## FIELD(&m_currGedInst, &_STATUS); \
+        if (_STATUS != GED_RETURN_VALUE_SUCCESS) { \
+            handleGedDecoderError(__LINE__, #FIELD, _STATUS); \
+        } \
+    } while (0)
+
+#define RETURN_GED_DECODE_RAW_TO_SRC(TYPE, FIELD) { \
+        TYPE _DST; \
+        GED_DECODE_RAW_TO_SRC(_DST, TYPE, FIELD); \
+        return _DST; \
+    }
+// the extra namespace declaration is needed to make g++ happy
+#define DEFINE_SOURCE_ACCESSOR(TYPE, FIELD, I) \
+    namespace iga { \
+        template <> TYPE Decoder::decodeSrc ## FIELD <SourceIndex::SRC##I>() { \
+            RETURN_GED_DECODE_RAW_TO_SRC(TYPE, Src ## I ## FIELD); \
+        } \
+    }
+
+/* #define DEFINE_SOURCE_ACCESSOR_INLINE(TYPE, FIELD, I) \
+template <> TYPE decodeSrc ## FIELD <SourceIndex::SRC##I>() { \
+RETURN_GED_DECODE_RAW_TO_SRC(TYPE, Src ## I ## FIELD); \
+}
+*/
+/* #define DEFINE_GED_SOURCE_ACCESSORS_INLINE_01(TYPE, FIELD) \
+DEFINE_SOURCE_ACCESSOR_INLINE(TYPE, FIELD, 0) \
+DEFINE_SOURCE_ACCESSOR_INLINE(TYPE, FIELD, 1)
+*/
+#define DEFINE_GED_SOURCE_ACCESSORS_01(TYPE, FIELD) \
+    DEFINE_SOURCE_ACCESSOR(TYPE, FIELD, 0) \
+    DEFINE_SOURCE_ACCESSOR(TYPE, FIELD, 1)
+
+/* #define DEFINE_GED_SOURCE_ACCESSORS_INLINE_012(TYPE, FIELD) \
+DEFINE_SOURCE_ACCESSOR_INLINE(TYPE, FIELD, 0) \
+DEFINE_SOURCE_ACCESSOR_INLINE(TYPE, FIELD, 1) \
+DEFINE_SOURCE_ACCESSOR_INLINE(TYPE, FIELD, 2)
+*/
+#define DEFINE_GED_SOURCE_ACCESSORS_012(TYPE, FIELD) \
+    DEFINE_SOURCE_ACCESSOR(TYPE, FIELD, 0) \
+    DEFINE_SOURCE_ACCESSOR(TYPE, FIELD, 1) \
+    DEFINE_SOURCE_ACCESSOR(TYPE, FIELD, 2)
+
+
 namespace iga
 {
     struct FlagRegInfo {
@@ -44,11 +131,11 @@ namespace iga
         Type      type = Type::INVALID;
     };
 
-    class DecoderBase : public GEDBitProcessor
+    class Decoder : public GEDBitProcessor
     {
     public:
         // Constructs a new decoder with an error handler and an empty kernel
-        DecoderBase(const Model &model, ErrorHandler &errHandler);
+        Decoder(const Model &model, ErrorHandler &errHandler);
 
         // the main entry point for decoding a kernel
         Kernel *decodeKernelBlocks(
@@ -61,9 +148,14 @@ namespace iga
         // Set the SWSB endcoding mode, if not set, derived from platform
         void setSWSBEncodingMode(SWSB_ENCODE_MODE mode)
         {
-            m_SWSBEncodeMode = mode;
+            if (mode != SWSB_ENCODE_MODE::SWSBInvalidMode)
+            {
+                m_SWSBEncodeMode = mode;
+            }
         }
 
+        bool isMacro() const;
+
     private:
         Kernel *decodeKernel(
             const void *binary,
@@ -139,11 +231,11 @@ namespace iga
         }
         template <SourceIndex S>
         void decodeSourceBasicAlign1(Instruction *inst) {
-            decodeSourceBasicAlign1<S>(inst,S);
+            decodeSourceBasicAlign1<S>(inst, S);
         }
         template <SourceIndex S>
         void decodeSourceBasicAlign16(Instruction *inst) {
-            decodeSourceBasicAlign16<S>(inst,S);
+            decodeSourceBasicAlign16<S>(inst, S);
         }
         template <SourceIndex S>
         void decodeSourceBasicAlign1(Instruction *inst, SourceIndex toSrcIx);
@@ -170,7 +262,8 @@ namespace iga
         GED_ADDR_MODE decodeSendSource0AddressMode();
         void decodeSendSource0(Instruction *inst);
         void decodeSendSource1(Instruction *inst);
-        void decodeSendInstructionOptions(Instruction *inst);
+        SendDesc decodeSendExDesc();
+        SendDesc decodeSendDesc();
 
         ///////////////////////////////////////////////////////////////////////
         // BRANCH INSTRUCTIONS
@@ -201,12 +294,12 @@ namespace iga
         int decodeDestinationRegNumAccBitsFromChEn();
         MathMacroExt decodeDestinationMathMacroRegFromChEn();
 
-        virtual unsigned decodeOpGroup(Op op);
-                bool hasImm64Src0Overlap();
+        Subfunction decodeSubfunction();
+        bool hasImm64Src0Overlap();
 
-        virtual void decodeDstDirSubRegNum(DirRegOpInfo& dri);
-        virtual bool hasImplicitScalingType(Type& type, DirRegOpInfo& dri);
-        virtual void decodeNextInstructionEpilog(Instruction *inst);
+        void decodeDstDirSubRegNum(DirRegOpInfo& dri);
+        bool hasImplicitScalingType(Type& type, DirRegOpInfo& dri);
+        void decodeNextInstructionEpilog(Instruction *inst);
 
         void decodeThreadOptions(Instruction *inst, GED_THREAD_CTRL trdCntrl);
 
@@ -244,24 +337,24 @@ namespace iga
 
 
         template <SourceIndex S> Type decodeSrcType() {
-            return GEDToIGATranslation::translate(decodeSrcDataType<S>());
+            return translate(decodeSrcDataType<S>());
         }
 
         template <SourceIndex S> Region decodeSrcRegionVWH() {
-            return GEDToIGATranslation::transateGEDtoIGARegion(
+            return transateGEDtoIGARegion(
                 decodeSrcVertStride<S>(),
                 decodeSrcWidth<S>(),
                 decodeSrcHorzStride<S>());
         }
 
-        template <SourceIndex S> Region decodeSrcRegionTernaryAlign1();
+        template <SourceIndex S> Region decodeSrcRegionTernaryAlign1(const OpSpec &);
 
         template <SourceIndex S> MathMacroExt decodeSrcMathMacroReg() {
-            return GEDToIGATranslation::translate(decodeSrcMathMacroExt<S>());
+            return translate(decodeSrcMathMacroExt<S>());
         }
         template <SourceIndex S> SrcModifier decodeSrcModifier() {
             if (m_opSpec->supportsSourceModifiers()) {
-                return GEDToIGATranslation::translate(decodeSrcSrcMod<S>());
+                return translate(decodeSrcSrcMod<S>());
             }
             return SrcModifier::NONE;
         }
@@ -273,8 +366,8 @@ namespace iga
             RegName regName = RegName::INVALID;
             GED_REG_FILE regFile = decodeSrcRegFile<S>();
             decodeReg((int)S, regFile, regNumBits, regName, regRef);
-            if (!m_opSpec->isSendOrSendsFamily() && !m_opSpec->isMacro()) {
-                regRef.subRegNum = (uint8_t)decodeSrcSubRegNum<S>();
+            if (!m_opSpec->isSendOrSendsFamily() && !isMacro()) {
+                regRef.subRegNum = (uint16_t)decodeSrcSubRegNum<S>();
             } else {
                 regRef.subRegNum = 0;
             }
@@ -298,7 +391,7 @@ namespace iga
                     scalingType = Type::D;
             }
 
-            dri.regRef.subRegNum = binNumToSubRegNum(
+            dri.regRef.subRegNum = (uint16_t)binNumToSubRegNum(
                 dri.regRef.subRegNum, dri.regName,
                 scalingType);
 
@@ -315,14 +408,6 @@ namespace iga
         void decodeChSelToSwizzle(uint32_t chanSel, GED_SWIZZLE swizzle[4]);
         template <SourceIndex S> bool isChanSelPacked();
 
-#if GED_VALIDATION_API
-        static const bool print_ged_debug = true;
-#else
-        static const bool print_ged_debug = false;
-        static GED_RETURN_VALUE GED_PrintFieldBitLocation(const ged_ins_t* ins, const GED_INS_FIELD field) { return GED_RETURN_VALUE_SUCCESS; };
-#endif
-
-
         void decodeOptions(Instruction *inst);
 
         // Translate Sub register from binary encoding number to asm number
@@ -339,6 +424,7 @@ namespace iga
         // info about the instruction being converted to IGA IR
         ged_ins_t                     m_currGedInst;
         const OpSpec                 *m_opSpec;
+        Subfunction                   m_subfunc;
         const void                   *m_binary;
 
         // SWSB encoding mode
@@ -367,7 +453,7 @@ namespace iga
 
 namespace iga
 {
-    typedef DecoderBase Decoder;
+    typedef Decoder Decoder;
 }
 
 #endif //end: _IGA_DECODER_H_
