@@ -26,32 +26,72 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #ifndef _IGA_BACKEND_GED_ENCODER_H_
 #define _IGA_BACKEND_GED_ENCODER_H_
 
-#include "EncoderCommon.hpp"
 #include "GEDBitProcessor.hpp"
 #include "IGAToGEDTranslation.hpp"
 #include "../EncoderOpts.hpp"
+#include "../../ErrorHandler.hpp"
 #include "../../IR/Instruction.hpp"
 #include "../../IR/Block.hpp"
 #include "../../IR/Kernel.hpp"
-#include "../../ErrorHandler.hpp"
 #include "../../Timer/Timer.hpp"
 
 #include <list>
 #include <map>
 
 
-
 namespace iga
 {
+#define GED_ENCODE(FUNC, ARG) \
+    GED_ENCODE_TO(FUNC, ARG, &m_gedInst)
+#if defined(GED_TIMER) || defined(_DEBUG)
+#define START_GED_TIMER() startIGATimer(TIMER_GED);
+#define STOP_GED_TIMER()  stopIGATimer(TIMER_GED);
+#else
+#define START_GED_TIMER()
+#define STOP_GED_TIMER()
+#endif
+
+#if defined(TOTAL_ENCODE_TIMER) || defined (_DEBUG)
+#define START_ENCODER_TIMER() startIGATimer(TIMER_TOTAL);
+#define STOP_ENCODER_TIMER()  stopIGATimer(TIMER_TOTAL);
+#else
+#define START_ENCODER_TIMER()
+#define STOP_ENCODER_TIMER()
+#endif
+
+#ifdef GED_PRINT_BIT_LOCATION
+#undef GED_PRINT_BIT_LOCATION
+#endif
+#if GED_VALIDATION_API
+#define GED_PRINT_BIT_LOCATION(FIELD, VAL) \
+    do { \
+        std::cout << "FIELD: " << #FIELD << " <= " \
+            "0x" << std::hex << std::uppercase << (uint64_t)(VAL) << "\n"; \
+            GED_PrintFieldBitLocation(&m_currGedInst, GED_INS_FIELD_ ## FIELD); \
+    } while(0)
+#else
+#define GED_PRINT_BIT_LOCATION(FIELD, VAL)
+#endif
+
+#define GED_ENCODE_TO(FUNC, ARG, GED) \
+    do { \
+        GED_RETURN_VALUE _status; \
+        GED_PRINT_BIT_LOCATION(FIELD, ARG); \
+        START_GED_TIMER() \
+        _status = GED_Set ## FUNC (GED, ARG); \
+        STOP_GED_TIMER() \
+        if (_status != GED_RETURN_VALUE_SUCCESS) { \
+            handleGedError(__LINE__, #FUNC, _status); \
+        } \
+    } while (0)
+
+
     typedef list<Block *, std_arena_based_allocator<Block*>> BlockList;
 
-    class EncoderBase : protected GEDBitProcessor
+    class Encoder : protected GEDBitProcessor
     {
     public:
-        EncoderBase(
-            const Model& model,
-            ErrorHandler& errHandler,
-            const EncoderOpts& eos);
+        Encoder(const Model& model, ErrorHandler& eh, const EncoderOpts& eos);
 
         void encodeKernel(
             Kernel& k,
@@ -72,19 +112,19 @@ namespace iga
     protected:
 
         // TODO: phase these out
-                void encodeKernelPreProcess(Kernel &k);
-        virtual void doEncodeKernelPreProcess(Kernel &k);
-        virtual void encodeFC(const OpSpec &os);
-        virtual void encodeTernaryInstruction(const Instruction& inst, GED_ACCESS_MODE accessMode);
-                void encodeTernaryAlign1Instruction(const Instruction& inst);
-                void encodeTernaryAlign16Instruction(const Instruction& inst);
-        virtual void encodeSendInstructionProcessSFID(const OpSpec&);
-        virtual void encodeSendDirectDestination(const Operand& dst);
-        virtual void encodeSendDestinationDataType(const Operand& dst);
-                void encodeOptionsThreadControl(const Instruction& inst);
+        void encodeKernelPreProcess(Kernel &k);
+        void doEncodeKernelPreProcess(Kernel &k);
+        void encodeFC(const Instruction &i);
+        void encodeTernaryInstruction(const Instruction& inst, GED_ACCESS_MODE accessMode);
+        void encodeTernaryAlign1Instruction(const Instruction& inst);
+        void encodeTernaryAlign16Instruction(const Instruction& inst);
+        void encodeSendInstructionProcessSFID(const Instruction& inst);
+        void encodeSendDirectDestination(const Operand& dst);
+        void encodeSendDestinationDataType(const Operand& dst);
+        void encodeOptionsThreadControl(const Instruction& inst);
 
     protected:
-        void encodeDstReg(RegName regName, uint8_t regNum); // just the regnum
+        void encodeDstReg(RegName regName, uint16_t regNum); // just the regnum
 
         void encodeImmVal(const ImmVal &val, Type type);
 
@@ -98,10 +138,10 @@ namespace iga
 
         template <SourceIndex S> void encodeSrcMathMacroReg(MathMacroExt a);
 
-        template <SourceIndex S> void encodeSrcReg(RegName regName, uint8_t regNum);
+        template <SourceIndex S> void encodeSrcReg(RegName regName, uint16_t regNum);
         template <SourceIndex S> void encodeSrcAddrImm(int32_t addrImm);
         template <SourceIndex S> void encodeSrcAddrSubRegNum(uint32_t addrSubReg);
-        template <SourceIndex S> void encodeSrcRegion(const Region& r);
+        template <SourceIndex S> void encodeSrcRegion(const Region& r, bool hasRgnWi = true);
         template <SourceIndex S> void encodeSrcRegionWidth(Region::Width w);
         template <SourceIndex S> void encodeSrcRepCtrl(GED_REP_CTRL rep);
         template <SourceIndex S> void encodeSrcChanSel(
@@ -115,14 +155,10 @@ namespace iga
         void encodeTernaryDestinationAlign16(const Instruction& inst);
 
         template <SourceIndex S>
-        void encodeTernarySourceAlign1(
-            const Instruction& inst,
-            GED_EXECUTION_DATA_TYPE execDataType);
-        void encodeTernaryDestinationAlign1(
-            const Instruction& inst,
-            GED_EXECUTION_DATA_TYPE execDataType);
+        void encodeTernarySourceAlign1(const Instruction& inst);
+        void encodeTernaryDestinationAlign1(const Instruction& inst);
 
-        virtual void encodeTernarySrcRegionVert(SourceIndex S, Region::Vert v);
+        void encodeTernarySrcRegionVert(SourceIndex S, Region::Vert v);
 
         template <SourceIndex S>
         void encodeTernaryImmVal(const ImmVal &val, Type type);
@@ -133,7 +169,7 @@ namespace iga
         void handleGedError(int line, const char *setter, GED_RETURN_VALUE status);
 
         // state that is valid over instance life
-        struct EncoderOpts                        m_opts;
+        EncoderOpts                               m_opts;
 
         // state that is valid over encodeInst()
         ged_ins_t                                 m_gedInst;
@@ -142,8 +178,8 @@ namespace iga
         size_t                                    m_numberInstructionsEncoded;
 
     private:
-        void operator delete(void*foo, MemManager* m) { };
-        void *operator new(size_t sz, MemManager* m) { return m->alloc(sz); };
+        void operator delete(void*, MemManager*) { };
+        void *operator new(size_t sz, MemManager* m) {return m->alloc(sz);};
 
         void encodeBlock(Block* blk);
         void encodeInstruction(Instruction& inst);
@@ -166,14 +202,14 @@ namespace iga
         ///////////////////////////////////////////////////////////////////////
         void encodeBranchingInstruction(const Instruction& inst);
         void encodeBranchingInstructionSimplified(const Instruction& inst);
-        void encodeBranchDestination(const Instruction& inst, const Operand& dst);
-        void encodeBranchSource(const Instruction& inst, const Operand& src);
+        void encodeBranchDestination(const Operand& dst);
+        void encodeBranchSource(const Operand& src);
         ///////////////////////////////////////////////////////////////////////
         // SEND INSTRUCTIONS
         ///////////////////////////////////////////////////////////////////////
         void encodeSendInstruction(const Instruction& inst);
         void encodeSendSource0(const Operand& src);
-        void encodeSendDestination(const Instruction& inst, const Operand& dst);
+        void encodeSendDestination(const Operand& dst);
         void encodeSendsSource0(const Operand& src);
         void encodeSendsSource1(const Operand& src);
         void encodeSendsDestination(const Operand& dst);
@@ -194,7 +230,7 @@ namespace iga
         // handles encoding ARF registers as well as the easy GRF case
         // caller actually pulls the trigger and encodes the bits, but this
         // call can raise the encoding error
-        uint32_t         translateRegNum(int opIx, RegName reg, uint8_t regNum);
+        uint32_t         translateRegNum(int opIx, RegName reg, uint16_t regNum);
         uint32_t         mathMacroRegToBits(int src, MathMacroExt mme); // ChSel and SubReg
         GED_DST_CHAN_EN  mathMacroRegToChEn(MathMacroExt mme);
 
@@ -212,9 +248,13 @@ namespace iga
         //
         // GEN7p5 implicitly scales PC offsets by QW except for a few instructions
         bool arePcsInQWords(const OpSpec &os) const;
-        // Older GENs need a <2;2,1> on the src operand.
+
+        // Call need to have src0 region be set to:
+        // SKL and before: <2;2,1>
+        // ICL: <2;4,1>
         // Later GENs ignore the region completely
-        bool callNeedsSrcRegion221(const Instruction &inst) const;
+        bool callNeedsSrc0Region221(const Instruction &inst) const;
+        bool callNeedsSrc0Region241(const Instruction &inst) const;
 
         /////////////////////////////////////////////////////////////
         // state valid over encodeKernel()
@@ -237,30 +277,30 @@ namespace iga
         {
             uint64_t value = 0;
             switch (type) {
-            case iga::Type::UD:
-            case iga::Type::F:
-            case iga::Type::V:
-            case iga::Type::UV:
-            case iga::Type::VF:
+            case Type::UD:
+            case Type::F:
+            case Type::V:
+            case Type::UV:
+            case Type::VF:
                 value = (uint64_t)val.u32;
                 break;
-            case iga::Type::D:
+            case Type::D:
                 value = (uint64_t)val.s32;
                 break;
-            case iga::Type::W:
+            case Type::W:
                 value = (uint64_t)val.s16;
                 break;
-            case iga::Type::UW:
-            case iga::Type::HF:
+            case Type::UW:
+            case Type::HF:
                 value = (uint64_t)val.u16;
                 break;
-            case iga::Type::DF:
-            case iga::Type::UQ:
-            case iga::Type::Q:
+            case Type::DF:
+            case Type::UQ:
+            case Type::Q:
                 value = val.u64;
                 break;
-            case iga::Type::B:
-            case iga::Type::UB:
+            case Type::B:
+            case Type::UB:
                 // technically not reachable since we don't permit byte moves
                 // from immediates
                 value = val.u64;
@@ -306,9 +346,9 @@ namespace iga
         }
 
 
-    }; //end: class definition EncoderBase
+    }; //end: class definition Encoder
 
-    template <SourceIndex S> void EncoderBase::encodeSrcRegFile(GED_REG_FILE rf) {
+    template <SourceIndex S> void Encoder::encodeSrcRegFile(GED_REG_FILE rf) {
         if (S == SourceIndex::SRC0) {
             GED_ENCODE(Src0RegFile, rf);
         } else if (S == SourceIndex::SRC1) {
@@ -318,25 +358,25 @@ namespace iga
         }
     }
 
-    template <SourceIndex S> void EncoderBase::encodeSrcRegionVert(Region::Vert v) {
+    template <SourceIndex S> void Encoder::encodeSrcRegionVert(Region::Vert v) {
         if (S == SourceIndex::SRC0) {
-            GED_ENCODE(Src0VertStride, IGAToGEDTranslation::lowerRegionVert(v));
+            GED_ENCODE(Src0VertStride, lowerRegionVert(v));
         } else { // (S == SourceIndex::SRC1)
-            GED_ENCODE(Src1VertStride, IGAToGEDTranslation::lowerRegionVert(v));
+            GED_ENCODE(Src1VertStride, lowerRegionVert(v));
         } // S != SRC2 since ternary Align1 doesn't have bits for that
     }
 
-    template <SourceIndex S> void EncoderBase::encodeSrcType(Type t) {
+    template <SourceIndex S> void Encoder::encodeSrcType(Type t) {
         if (S == SourceIndex::SRC0) {
-            GED_ENCODE(Src0DataType, IGAToGEDTranslation::lowerDataType(t));
+            GED_ENCODE(Src0DataType, lowerDataType(t));
         } else if (S == SourceIndex::SRC1) {
-            GED_ENCODE(Src1DataType, IGAToGEDTranslation::lowerDataType(t));
+            GED_ENCODE(Src1DataType, lowerDataType(t));
         } else {
-            GED_ENCODE(Src2DataType, IGAToGEDTranslation::lowerDataType(t));
+            GED_ENCODE(Src2DataType, lowerDataType(t));
         }
     }
 
-    template <SourceIndex S> void EncoderBase::encodeSrcAddrMode(GED_ADDR_MODE x) {
+    template <SourceIndex S> void Encoder::encodeSrcAddrMode(GED_ADDR_MODE x) {
         if (S == SourceIndex::SRC0) {
             GED_ENCODE(Src0AddrMode, x);
         } else {
@@ -344,17 +384,17 @@ namespace iga
         }
     }
 
-    template <SourceIndex S> void EncoderBase::encodeSrcModifier(SrcModifier x) {
+    template <SourceIndex S> void Encoder::encodeSrcModifier(SrcModifier x) {
         if (S == SourceIndex::SRC0) {
-            GED_ENCODE(Src0SrcMod, IGAToGEDTranslation::lowerSrcMod(x));
+            GED_ENCODE(Src0SrcMod, lowerSrcMod(x));
         } else if (S == SourceIndex::SRC1) {
-            GED_ENCODE(Src1SrcMod, IGAToGEDTranslation::lowerSrcMod(x));
+            GED_ENCODE(Src1SrcMod, lowerSrcMod(x));
         } else {
-            GED_ENCODE(Src2SrcMod, IGAToGEDTranslation::lowerSrcMod(x));
+            GED_ENCODE(Src2SrcMod, lowerSrcMod(x));
         }
     }
 
-    template <SourceIndex S> void EncoderBase::encodeSrcSubRegNum(
+    template <SourceIndex S> void Encoder::encodeSrcSubRegNum(
         std::pair<bool, uint32_t> subReg, bool isTernaryOrBranch) {
         if (S == SourceIndex::SRC0) {
             GED_ENCODE(Src0SubRegNum, subReg.second);
@@ -366,19 +406,19 @@ namespace iga
     }
 
     template <SourceIndex S>
-    void EncoderBase::encodeSrcMathMacroReg(MathMacroExt a)
+    void Encoder::encodeSrcMathMacroReg(MathMacroExt a)
     {
         if (S == SourceIndex::SRC0) {
-            GED_ENCODE(Src0MathMacroExt, IGAToGEDTranslation::lowerSpecialAcc(a));
+            GED_ENCODE(Src0MathMacroExt, lowerSpecialAcc(a));
         } else if (S == SourceIndex::SRC1) {
-            GED_ENCODE(Src1MathMacroExt, IGAToGEDTranslation::lowerSpecialAcc(a));
+            GED_ENCODE(Src1MathMacroExt, lowerSpecialAcc(a));
         } else {
-            GED_ENCODE(Src2MathMacroExt, IGAToGEDTranslation::lowerSpecialAcc(a));
+            GED_ENCODE(Src2MathMacroExt, lowerSpecialAcc(a));
         }
     }
-    template <SourceIndex S> void EncoderBase::encodeSrcReg(
+    template <SourceIndex S> void Encoder::encodeSrcReg(
         RegName regName,
-        uint8_t regNum)
+        uint16_t regNum)
     {
         uint32_t regBits = 0;
         if (regName == RegName::GRF_R) {
@@ -388,8 +428,9 @@ namespace iga
             if (!ri) {
                 error("src%d: unexpected register on this platform", (int)S);
             } else {
-                ri->encode((int)regNum,regNum);
-                regBits = regNum; // widen for GED
+                uint8_t reg8;
+                ri->encode((int)regNum, reg8);
+                regBits = reg8; // widen for GED
             }
         }
         if (S == SourceIndex::SRC0) {
@@ -401,14 +442,15 @@ namespace iga
         }
     }
 
-    template <SourceIndex S> void EncoderBase::encodeSrcAddrImm(int32_t addrImm) {
+    template <SourceIndex S> void Encoder::encodeSrcAddrImm(int32_t addrImm) {
         if (S == SourceIndex::SRC0) {
             GED_ENCODE(Src0AddrImm, addrImm);
         } else {
             GED_ENCODE(Src1AddrImm, addrImm);
         }
     }
-    template <SourceIndex S> void EncoderBase::encodeSrcAddrSubRegNum(uint32_t addrSubReg) {
+    template <SourceIndex S>
+    void Encoder::encodeSrcAddrSubRegNum(uint32_t addrSubReg) {
         if (S == SourceIndex::SRC0) {
             GED_ENCODE(Src0AddrSubRegNum, addrSubReg);
         } else {
@@ -416,8 +458,7 @@ namespace iga
         }
     }
     template <SourceIndex S>
-    void EncoderBase::encodeSrcRegion(const Region &rgn)
-    {
+    void Encoder::encodeSrcRegion(const Region &rgn, bool hasRgnWi) {
         uint32_t v = 0;
         if (rgn.getVt() == Region::Vert::VT_VxH) {
             v = 0x3;
@@ -446,27 +487,37 @@ namespace iga
 
         if (S == SourceIndex::SRC0) {
             GED_ENCODE(Src0VertStride, v);
-            GED_ENCODE(Src0Width, w);
+            if (hasRgnWi) {
+                GED_ENCODE(Src0Width, w);
+            } else {
+                // some ops have an implicit width region
+                // (e.g. some specialized instructions poaches Src0.RgnWi)
+                //
+                // Within the IR we use 1 so logic that depends on regioning
+                // gets the correct behavior (hardware assumes w=1).
+                w = 1;
+            }
             GED_ENCODE(Src0HorzStride, h);
         } else if (S == SourceIndex::SRC1) {
             GED_ENCODE(Src1VertStride, v);
             GED_ENCODE(Src1Width, w);
             GED_ENCODE(Src1HorzStride, h);
         } else {
-            IGA_ASSERT_FALSE("EncoderBase::encodeSrcRegion only works on src0 and src1");
+            IGA_ASSERT_FALSE("Encoder::encodeSrcRegion: only works on src0 and src1");
         }
     }
 
-    template <SourceIndex S> void EncoderBase::encodeSrcRegionWidth(Region::Width w) {
+    template <SourceIndex S>
+    void Encoder::encodeSrcRegionWidth(Region::Width w) {
         if (S == SourceIndex::SRC0) {
-            GED_ENCODE(Src0Width, IGAToGEDTranslation::lowerRegionWidth(w));
+            GED_ENCODE(Src0Width, lowerRegionWidth(w));
         } else { // (S == SourceIndex::SRC1)
-            GED_ENCODE(Src1Width, IGAToGEDTranslation::lowerRegionWidth(w));
+            GED_ENCODE(Src1Width, lowerRegionWidth(w));
         } // S != SRC2 since ternary Align1 doesn't have bits for that
     }
 
     template <SourceIndex S>
-    void EncoderBase::encodeTernaryImmVal(const ImmVal &val, Type type) {
+    void Encoder::encodeTernaryImmVal(const ImmVal &val, Type type) {
         if (S == SourceIndex::SRC0) {
             GED_ENCODE(Src0TernaryImm, typeConvesionHelper(val, type));
         } else if (S == SourceIndex::SRC2) {
@@ -476,19 +527,20 @@ namespace iga
         }
     }
 
-    template <SourceIndex S> void EncoderBase::encodeSrcRegionHorz(Region::Horz s) {
+    template <SourceIndex S>
+    void Encoder::encodeSrcRegionHorz(Region::Horz s) {
         if (S == SourceIndex::SRC0) {
-            GED_ENCODE(Src0HorzStride, IGAToGEDTranslation::lowerRegionHorz(s));
+            GED_ENCODE(Src0HorzStride, lowerRegionHorz(s));
         } else if (S == SourceIndex::SRC1) {
-            GED_ENCODE(Src1HorzStride, IGAToGEDTranslation::lowerRegionHorz(s));
+            GED_ENCODE(Src1HorzStride, lowerRegionHorz(s));
         } else {
-            GED_ENCODE(Src2HorzStride, IGAToGEDTranslation::lowerRegionHorz(s));
+            GED_ENCODE(Src2HorzStride, lowerRegionHorz(s));
         }
     }
 } // end: namespace iga*
 
 namespace iga
 {
-    typedef EncoderBase Encoder;
+    typedef Encoder Encoder;
 }
 #endif
