@@ -29,9 +29,12 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "../../api/iga_bxml_ops.hpp"
 #include "../../asserts.hpp"
 #include "../../bits.hpp"
+#include "../../strings.hpp"
 
 #include <cstdint>
 #include <cstring>
+#include <iomanip>
+#include <ostream>
 #include <string>
 
 #if (__cplusplus >= 201402L) || (defined(_MSC_VER) && (_MSVC_LANG >= 201402L))
@@ -40,8 +43,8 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 // to opt-in based if our compiler is C++14 or better.  C++11 allows us to
 // return constexpr expressions only, but C++14 allows more statements.
 //
-// MSVS defines __cplusplus as 199711L, but they do use _MSVC_LANG
-// Maybe there's a better way.
+// MSVS defines __cplusplus as 199711L, but they do use _MSVC_LANG in those
+// release.  Maybe there's a better way.
 #define CONSTEXPR_IF_CPP14 constexpr
 #else
 #define CONSTEXPR_IF_CPP14
@@ -80,15 +83,22 @@ namespace iga
         constexpr Fragment() :
             Fragment(nullptr, NO_OFFSET, 0, Kind::INVALID) { }
 
-        bool overlaps(const Fragment &f) const {
+        // We could lower this to C++11 if we really tried.
+        //        (it's a pure boolean expression underneath)
+        CONSTEXPR_IF_CPP14 bool overlaps(const Fragment &f) const {
             if (!isEncoded() || !f.isEncoded())
-                return false;
-            if (length < f.length)
-                return f.overlaps(*this);
-            // test the end points of the smaller interval within the larger
-            return (offset >= f.offset && offset < (f.offset + f.length)) ||
-                ((offset + length - 1) >= f.offset &&
-                 (offset + length - 1) < (f.offset + f.length));
+                return false; // pseudo-fragments (unmapped) don't collide
+
+            // arrange intervals as: smaller or equal, and larger
+            const Fragment &smaller = length <= f.length ? *this : f;
+            const Fragment &larger  = length <= f.length ? f : *this;
+            int sLo = smaller.offset,
+                sHi = smaller.offset + smaller.length - 1;
+            int lLo = larger.offset,
+                lHi = larger.offset + larger.length - 1;
+            // determine if the ends of smaller fit in larger
+            return (sLo >= lLo && sLo <= lHi) ||
+                (sHi >= lLo && sHi <= lHi);
         }
 
         constexpr bool isEncoded() const {return kind == Kind::ENCODED;}
@@ -96,7 +106,7 @@ namespace iga
         constexpr bool isZeroWires() const {return kind == Kind::ZERO_WIRES;}
         constexpr bool isInvalid() const {return kind == Kind::INVALID;}
 
-        uint64_t getMask() const {
+        constexpr uint64_t getMask() const {
             return getFieldMaskUnshifted<uint64_t>(length);
         }
     };
@@ -198,8 +208,8 @@ namespace iga
             const char *name2, int offset2, int length2)
             : Field(_name,
                 name0, offset0, length0, Fragment::Kind::ENCODED,
-                name0, offset0, length0, Fragment::Kind::ENCODED,
-                name0, offset0, length0, Fragment::Kind::ENCODED)
+                name1, offset1, length1, Fragment::Kind::ENCODED,
+                name2, offset2, length2, Fragment::Kind::ENCODED)
         {
         }
         // a field a fragment of intrinsically defined 0's and top bits encoded
@@ -270,7 +280,7 @@ namespace iga
         }
     }; // Field
 
-    static bool operator== (const Fragment &f1, const Fragment &f2)
+    static inline bool operator== (const Fragment &f1, const Fragment &f2)
     {
         return
             f1.kind == f2.kind &&
@@ -278,7 +288,7 @@ namespace iga
             f1.length == f2.length &&
             strcmp(f1.name, f2.name) == 0;
     }
-    static bool operator< (const Fragment &f1, const Fragment &f2)
+    static inline bool operator< (const Fragment &f1, const Fragment &f2)
     {
         if (f1.kind != f2.kind) {
             return f1.kind < f2.kind;
@@ -345,7 +355,7 @@ namespace iga
     static inline int ToFieldOperandArrayIndex(OpIx IX) {
         return (IX & OpIx::OP_IX_MASK);
     }
-    static ::std::string ToStringOpIx(OpIx ix) {
+    static inline std::string ToStringOpIx(OpIx ix) {
         switch (ix & OpIx::OP_IX_TYPE_MASK) {
         case OpIx::IX_DST:  return "dst";
         case OpIx::IX_SRC0: return "src0";
@@ -375,6 +385,26 @@ namespace iga
             return entrySizeBits;
         }
         bool isSrcImmField() const {return mappings == nullptr;}
+
+        // emits output such as  "0`001`1`0`001"
+        // for SrcImm compacted fields it just emits the value
+        void emitBinary(std::ostream &os, uint64_t val) const {
+            if (mappings == nullptr) {
+                // srcimm field
+                os << std::setw(16) << "(" << fmtHex(val) << ")";
+            } else {
+                int bitOff = (int)countNumBitsMapped();
+                for (int mIx = 0; mIx < (int)numMappings; ++mIx) {
+                    if (mIx != 0) {
+                        os << "`";
+                    }
+                    const Field *mf = mappings[mIx];
+                    bitOff -= mf->length();
+                    auto bs = iga::getBits(val, bitOff, mf->length());
+                    fmtBinaryDigits(os, bs, mf->length());
+                }
+            }
+        }
     };
 
     // groups all CompactionMappings for a given platform
