@@ -35,10 +35,6 @@ namespace iga
 {
     // machine instruction
     struct MInst {
-        MInst() {
-            qw0 = qw1 = 0;
-        }
-
         union {
             struct {uint32_t dw0, dw1, dw2, dw3; };
             struct {uint32_t dws[4];};
@@ -46,6 +42,21 @@ namespace iga
             struct {uint64_t qws[2];};
         };
 
+        MInst() {
+            qw0 = qw1 = 0;
+        }
+
+        bool operator==(const MInst &rhs) const {
+            return qw0 == rhs.qw0 && qw1 == rhs.qw1;
+        }
+        bool operator!=(const MInst &rhs) const {
+            return !(*this == rhs);
+        }
+        bool operator<(const MInst &rhs) const {
+            return (qw1 < rhs.qw1) || (qw1 == rhs.qw1 && qw0 < rhs.qw0);
+        }
+
+
         bool testBit(int off) const {
             return getBits(off, 1) != 0;
         }
@@ -89,6 +100,10 @@ namespace iga
             //
             return bits;
         }
+        int64_t getSignedField(const Field &f) const {
+            auto bits = getField(f);
+            return getSignedBits(bits, 0, f.length());
+        }
 
         // gets a fragmented field from an array of fields
         // the fields are ordered low bit to high bit
