@@ -23,11 +23,12 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 
 ======================= end_copyright_notice ==================================*/
-#ifndef _IGA_IRTYPES_TO_STRING_HPP
-#define _IGA_IRTYPES_TO_STRING_HPP
+#ifndef _IGA_FRONTEND_IRTOSTRING_HPP
+#define _IGA_FRONTEND_IRTOSTRING_HPP
 
 #include "../IR/Types.hpp"
 #include "../strings.hpp"
+#include "../api/iga_bxml_enums.hpp"
 
 #include <string>
 #include <sstream>
@@ -35,7 +36,7 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 namespace iga {
 
-static std::string MakeErrorString(const char *pfx, int t) {
+static inline std::string MakeErrorString(const char *pfx, int t) {
     std::stringstream ss;
     ss << pfx << "::" << t << "?";
     return ss.str();
@@ -44,27 +45,12 @@ static std::string MakeErrorString(const char *pfx, int t) {
 #define MAKE_CASE(C,S) case C::S: return #C "::" #S
 #define MAKE_DEFAULT_CASE(C) default: return MakeErrorString(#C, (int)x)
 
-static std::string ToSymbol(Platform x) {
-    switch (x) {
-    MAKE_CASE(Platform, INVALID);
-    MAKE_CASE(Platform, GEN6);
-    MAKE_CASE(Platform, GEN7);
-    MAKE_CASE(Platform, GEN7P5);
-    MAKE_CASE(Platform, GEN8);
-    MAKE_CASE(Platform, GEN8LP);
-    MAKE_CASE(Platform, GEN9);
-    MAKE_CASE(Platform, GEN9LP);
-    MAKE_CASE(Platform, GEN9P5);
-    MAKE_CASE(Platform, GEN10);
-    MAKE_CASE(Platform, GEN11);
-    MAKE_CASE(Platform, GEN12P1);
-    MAKE_CASE(Platform, GENNEXT);
-    MAKE_DEFAULT_CASE(Platform);
-    }
-}
+
+// e.g. "Platform::GEN11"
+std::string ToSymbol(Platform x);
 
 
-static std::string ToSymbol(PredCtrl x) {
+static inline std::string ToSymbol(PredCtrl x) {
     switch (x) {
     MAKE_CASE(PredCtrl, NONE);
     MAKE_CASE(PredCtrl, SEQ);
@@ -85,19 +71,18 @@ static std::string ToSymbol(PredCtrl x) {
 }
 
 
-static std::string ToSymbol(Op op) {
-    // TODO: expand this manually, this is a crude approximation
+static inline std::string ToSymbol(Op op) {
     std::stringstream ss;
-    ss << "op#" << (int)op;
+    ss << "Op::0x" << std::hex << std::uppercase << (int)op;
     return ss.str();
 }
 
 
-static std::string ToSymbol(bool x) {
+static inline std::string ToSymbol(bool x) {
     return x ? "true" : "false";
 }
 
-static std::string ToSymbol(BranchCntrl x) {
+static inline std::string ToSymbol(BranchCntrl x) {
     switch (x) {
     MAKE_CASE(BranchCntrl, ON);
     MAKE_CASE(BranchCntrl, OFF);
@@ -106,7 +91,7 @@ static std::string ToSymbol(BranchCntrl x) {
 }
 
 
-static std::string ToSymbol(ExecSize x) {
+static inline std::string ToSymbol(ExecSize x) {
     switch (x) {
     MAKE_CASE(ExecSize, SIMD1);
     MAKE_CASE(ExecSize, SIMD2);
@@ -119,7 +104,7 @@ static std::string ToSymbol(ExecSize x) {
 }
 
 
-static std::string ToSymbol(ChannelOffset x) {
+static inline std::string ToSymbol(ChannelOffset x) {
     switch (x) {
     MAKE_CASE(ChannelOffset, M0);
     MAKE_CASE(ChannelOffset, M4);
@@ -134,7 +119,7 @@ static std::string ToSymbol(ChannelOffset x) {
 }
 
 
-static std::string ToSymbol(MaskCtrl x) {
+static inline std::string ToSymbol(MaskCtrl x) {
     switch (x) {
     MAKE_CASE(MaskCtrl, NORMAL);
     MAKE_CASE(MaskCtrl, NOMASK);
@@ -143,7 +128,7 @@ static std::string ToSymbol(MaskCtrl x) {
 }
 
 
-static std::string ToSymbol(SrcModifier x) {
+static inline std::string ToSymbol(SrcModifier x) {
     switch (x) {
     MAKE_CASE(SrcModifier, NONE);
     MAKE_CASE(SrcModifier, ABS);
@@ -154,7 +139,8 @@ static std::string ToSymbol(SrcModifier x) {
 }
 
 
-static std::string ToSymbol(DstModifier x) {
+static inline std::string ToSymbol(DstModifier x)
+{
     switch (x) {
     MAKE_CASE(DstModifier, NONE);
     MAKE_CASE(DstModifier, SAT);
@@ -163,7 +149,8 @@ static std::string ToSymbol(DstModifier x) {
 }
 
 
-static std::string ToSymbol(RegName x) {
+static inline std::string ToSymbol(RegName x)
+{
     switch (x) {
     MAKE_CASE(RegName, INVALID);
     MAKE_CASE(RegName, ARF_NULL);
@@ -188,7 +175,8 @@ static std::string ToSymbol(RegName x) {
 }
 
 
-static std::string ToSymbol(MathMacroExt x) {
+static inline std::string ToSymbol(MathMacroExt x)
+{
     switch (x) {
     MAKE_CASE(MathMacroExt, INVALID);
     MAKE_CASE(MathMacroExt, MME0);
@@ -205,7 +193,8 @@ static std::string ToSymbol(MathMacroExt x) {
 }
 
 
-static std::string ToSymbol(Type x) {
+static inline std::string ToSymbol(Type x)
+{
     switch (x) {
     MAKE_CASE(Type, INVALID);
 
@@ -233,7 +222,8 @@ static std::string ToSymbol(Type x) {
 }
 
 
-static std::string ToSymbol(FlagModifier x) {
+static inline std::string ToSymbol(FlagModifier x)
+{
     switch (x) {
     MAKE_CASE(FlagModifier, NONE);
     MAKE_CASE(FlagModifier, EQ);
@@ -250,7 +240,8 @@ static std::string ToSymbol(FlagModifier x) {
 }
 
 
-static std::string ToSymbol(Region x) {
+static inline std::string ToSymbol(Region x)
+{
     if (x == Region::INVALID)
         return "Region::INVALID";
     else if (x == Region::SRC010)
@@ -269,6 +260,8 @@ static std::string ToSymbol(Region x) {
         return "Region::DST1";
     else if (x == Region::SRC0X0)
         return "Region::SRC0X0";
+    else if (x == Region::SRC1X0)
+        return "Region::SRC1X0";
     else if (x == Region::SRC2X1)
         return "Region::SRC2X1";
     else if (x == Region::SRC4X1)
@@ -323,7 +316,8 @@ static std::string ToSymbol(Region x) {
 }
 
 
-static std::string ToSymbol(InstOpt x) {
+static inline std::string ToSymbol(InstOpt x)
+{
     switch (x) {
     MAKE_CASE(InstOpt, ACCWREN);
     MAKE_CASE(InstOpt, ATOMIC);
@@ -341,7 +335,8 @@ static std::string ToSymbol(InstOpt x) {
 }
 
 
-static std::string ToSymbol(const RegRef& x) {
+static inline std::string ToSymbol(const RegRef& x)
+{
     if (x == REGREF_INVALID) {
         return "RegRef::INVALID";
     } else {
@@ -356,12 +351,12 @@ static std::string ToSymbol(const RegRef& x) {
 }
 
 
-static std::string ToSymbol(const SendDescArg& x) {
+static inline std::string ToSymbol(const SendDesc& x) {
     std::stringstream ss;
     ss << "{";
-    ss << (x.type == SendDescArg::IMM ? "IMM" : "REG32A");
+    ss << (x.isImm() ? "IMM" : "REG32A");
     ss << ",";
-    if (x.type == SendDescArg::IMM) {
+    if (x.isImm()) {
         ss << ToSymbol(x.reg);
     } else {
         fmtHex(ss, x.imm);
@@ -371,7 +366,7 @@ static std::string ToSymbol(const SendDescArg& x) {
 }
 
 
-static std::string ToSyntax(Type ot) {
+static inline std::string ToSyntax(Type ot) {
     switch (ot) {
     case Type::UV: return ":uv";
     case Type::UB: return ":ub";
@@ -398,12 +393,12 @@ static std::string ToSyntax(Type ot) {
 }
 
 
-static std::string ToSyntax(MaskCtrl mc) {
+static inline std::string ToSyntax(MaskCtrl mc) {
     return mc == MaskCtrl::NOMASK ? "W" : "";
 }
 
 
-static std::string ToSyntax(PredCtrl pc) {
+static inline std::string ToSyntax(PredCtrl pc) {
     switch (pc) {
     case PredCtrl::NONE: return "";
     case PredCtrl::SEQ: return "";
@@ -424,7 +419,7 @@ static std::string ToSyntax(PredCtrl pc) {
 }
 
 
-static std::string ToSyntax(const Predication& pred) {
+static inline std::string ToSyntax(const Predication& pred) {
     std::stringstream ss;
     ss << (pred.inverse ? "~" : "");
     ss << ToSyntax(pred.function);
@@ -432,7 +427,7 @@ static std::string ToSyntax(const Predication& pred) {
 }
 
 
-static std::string ToSyntax(ExecSize es) {
+static inline std::string ToSyntax(ExecSize es) {
     switch (es) {
     case ExecSize::SIMD1: return "1";
     case ExecSize::SIMD2: return "2";
@@ -445,22 +440,22 @@ static std::string ToSyntax(ExecSize es) {
 }
 
 
-static std::string ToSyntax(ChannelOffset es) {
+static inline std::string ToSyntax(ChannelOffset es) {
     switch (es) {
-    case ChannelOffset::M0: return "|M0";
-    case ChannelOffset::M4: return "|M4";
-    case ChannelOffset::M8: return "|M8";
-    case ChannelOffset::M12: return "|M12";
-    case ChannelOffset::M16: return "|M16";
-    case ChannelOffset::M20: return "|M20";
-    case ChannelOffset::M24: return "|M24";
-    case ChannelOffset::M28: return "|M28";
-    default: return MakeErrorString("|ChannelOffset", (int)es);
+    case ChannelOffset::M0: return "M0";
+    case ChannelOffset::M4: return "M4";
+    case ChannelOffset::M8: return "M8";
+    case ChannelOffset::M12: return "M12";
+    case ChannelOffset::M16: return "M16";
+    case ChannelOffset::M20: return "M20";
+    case ChannelOffset::M24: return "M24";
+    case ChannelOffset::M28: return "M28";
+    default: return MakeErrorString("ChannelOffset", (int)es);
     }
 }
 
 
-static std::string ToSyntax(FlagModifier fm) {
+static inline std::string ToSyntax(FlagModifier fm) {
     switch (fm) {
     case FlagModifier::NONE: return ""; // 0
     case FlagModifier::EQ: return "eq";
@@ -477,19 +472,23 @@ static std::string ToSyntax(FlagModifier fm) {
 }
 
 
-static bool IsBitwise(Op op) {
+static inline bool IsBitwise(Op op) {
     switch (op) {
     case Op::AND:
     case Op::OR:
     case Op::XOR:
     case Op::NOT:
-      return true;
+    // what about these?
+    // case Op::CBIT:
+    // case Op::LZD:
+        return true;
     default:
       return false;
     }
 }
 
-static std::string ToSyntax(BranchCntrl bc) {
+
+static inline std::string ToSyntax(BranchCntrl bc) {
     switch (bc) {
     case BranchCntrl::OFF: return "";
     case BranchCntrl::ON: return ".b";
@@ -497,7 +496,7 @@ static std::string ToSyntax(BranchCntrl bc) {
     }
 }
 
-static std::string ToSyntax(Op op, SrcModifier sm) {
+static inline std::string ToSyntax(Op op, SrcModifier sm) {
     switch (sm) {
     case SrcModifier::NONE: return "";
     case SrcModifier::ABS: return "(abs)";
@@ -508,7 +507,7 @@ static std::string ToSyntax(Op op, SrcModifier sm) {
 }
 
 
-static std::string ToSyntax(RegName regName) {
+static inline std::string ToSyntax(RegName regName) {
     switch (regName) {
     case RegName::ARF_NULL:  return "null";  // null
     case RegName::ARF_A:     return "a";
@@ -532,7 +531,7 @@ static std::string ToSyntax(RegName regName) {
 }
 
 
-static std::string ToSyntax(MathMacroExt MathMacroReg) {
+static inline std::string ToSyntax(MathMacroExt MathMacroReg) {
     switch (MathMacroReg) {
     case MathMacroExt::MME0:  return ".mme0";
     case MathMacroExt::MME1:  return ".mme1";
@@ -549,7 +548,7 @@ static std::string ToSyntax(MathMacroExt MathMacroReg) {
 }
 
 
-static std::string ToSyntax(const Region &rgn) {
+static inline std::string ToSyntax(const Region &rgn) {
     std::stringstream ss;
 
     if (rgn.getVt() != Region::Vert::VT_INVALID &&
@@ -583,7 +582,7 @@ static std::string ToSyntax(const Region &rgn) {
 }
 
 
-static std::string ToSyntax(const InstOpt &i) {
+static inline std::string ToSyntax(const InstOpt &i) {
     switch (i) {
     case InstOpt::ACCWREN:     return "AccWrEn";
     case InstOpt::ATOMIC:      return "Atomic";
@@ -602,11 +601,11 @@ static std::string ToSyntax(const InstOpt &i) {
 }
 
 
-static void ToSyntaxNoBraces(
+static inline void ToSyntaxNoBraces(
     std::ostream &os,
     const InstOptSet &instOpts)
 {
-    static const InstOpt ALL_INST_OPTS[] = {
+    static const InstOpt ALL_INST_OPTS[] {
         InstOpt::ACCWREN,
         InstOpt::ATOMIC,
         InstOpt::BREAKPOINT,
@@ -618,7 +617,7 @@ static void ToSyntaxNoBraces(
         InstOpt::NOPREEMPT,
         InstOpt::NOSRCDEPSET,
         InstOpt::SWITCH,
-        InstOpt::SERIALIZE
+        InstOpt::SERIALIZE,
       };
 
 
@@ -638,16 +637,36 @@ static void ToSyntaxNoBraces(
     }
 }
 
-static std::string ToSyntax(const InstOptSet &inst_opts) {
+static inline std::string ToSyntax(const InstOptSet &instOpts) {
     std::stringstream ss;
 
     ss << "{";
-    ToSyntaxNoBraces(ss, inst_opts);
+    ToSyntaxNoBraces(ss, instOpts);
     ss << "}";
 
     return ss.str();
 }
 
-} // namespace
+/////////////////////////////////////////////////////
+// manually defined enumerations from BXML
+template <typename T> T FromSyntax(std::string str);
 
-#endif // _IGA_IRTYPES_TO_STRING_HPP
+
+std::string ToSyntax(MathFC sfc);
+template <> MathFC FromSyntax<MathFC>(std::string str);
+
+
+std::string ToSyntax(SFID sfid);
+template <> SFID FromSyntax<SFID>(std::string str);
+
+
+std::string ToSyntax(SyncFC sfc);
+template <> SyncFC FromSyntax<SyncFC>(std::string str);
+
+
+
+
+
+} // iga::
+
+#endif // _IGA_FRONTEND_IRTOSTRING_HPP
