@@ -158,11 +158,11 @@ bool RegSet::addSourceInputs(const Instruction &i, RegSet &rs)
     if (i.getOpSpec().isSendOrSendsFamily()) {
         // send register descriptors touch a0.#
         auto desc = i.getMsgDescriptor();
-        if (desc.type == SendDescArg::REG32A) {
+        if (desc.isReg()) {
             added |= rs.add(RS_ARF_A, desc.reg.subRegNum, 4);
         }
         auto exDesc = i.getExtMsgDescriptor();
-        if (exDesc.type == SendDescArg::REG32A) {
+        if (exDesc.isReg()) {
             added |= rs.add(RS_ARF_A, desc.reg.subRegNum, 4);
         }
     }
@@ -235,27 +235,18 @@ bool RegSet::addSourceInputs(const Instruction &i, RegSet &rs)
             if (i.getOpSpec().isSendOrSendsFamily()) {
                 if (op.getDirRegName() == RegName::GRF_R) {
                     // send source GRF (not null reg)
-                    int nregs = 1;
-
+                    int nregs = 0;
                     if (srcIx == 0) {
-                        // wlen
-                        auto desc = i.getMsgDescriptor();
-                        if (desc.type == SendDescArg::IMM) {
-                            nregs = (desc.imm >> 25) & 0xF; // desc[28:25]
-                        } else {
-                            // since we don't know the length must be conservative
-                            nregs = op.getDirRegName() == RegName::GRF_R ? 31 : 0;
-                        }
+                        // mlen
+                        nregs = i.getSrc0Length();
+                        if (nregs < 0)
+                            nregs = 31;
                     } else {
-                        auto ex_desc = i.getExtMsgDescriptor();
-                        if (ex_desc.type == SendDescArg::IMM) {
-                            nregs = ((ex_desc.imm >> 6) & 0xF);
-                        } else {
-                            // indirect register usage, assume worst if not the null reg
-                            nregs = op.getDirRegName() == RegName::GRF_R ? 31 : 0;
-                        }
+                        nregs = i.getSrc1Length();
+                        if (nregs < 0)
+                            nregs = 8;
                     }
-                    uint8_t regNum = op.getDirRegRef().regNum;
+                    uint16_t regNum = op.getDirRegRef().regNum;
                     for (int i = 0; i < nregs; i++) {
                         if ((regNum + i) >= RS_GRF_R.numRegisters) {
                             break;
@@ -351,7 +342,7 @@ bool RegSet::addDestinationOutputs(const Instruction &i, RegSet &rs)
     }
     bool added = false;
 
-    int execOff = 4 * (static_cast<int>(i.getChannelOffset()));
+    unsigned execOff = 4 * (static_cast<int>(i.getChannelOffset()));
     int execSize = static_cast<int>(i.getExecSize());
     auto op = i.getDestination();
     auto tType = op.getType();
@@ -359,10 +350,9 @@ bool RegSet::addDestinationOutputs(const Instruction &i, RegSet &rs)
 
     if (i.hasInstOpt(InstOpt::ACCWREN) /* || i.getDestination().getDirRegName() == RegName::ARF_ACC*/) { // AccWrEn
         auto elemsPerAccReg = 8*RS_ARF_ACC.bytesPerRegister / typeSizeBits; // e.g. 8 subreg elems for :f
-        RegRef ar = {
-            (uint8_t)(execOff / elemsPerAccReg),
-            (uint8_t)(execOff % elemsPerAccReg)
-        };
+        RegRef ar(
+            execOff / elemsPerAccReg,
+            execOff % elemsPerAccReg);
         added |= rs.setDstRegion(
             RegName::ARF_ACC,
             ar,
@@ -377,20 +367,15 @@ bool RegSet::addDestinationOutputs(const Instruction &i, RegSet &rs)
         if (i.getOpSpec().isSendOrSendsFamily() &&
             op.getDirRegName() == RegName::GRF_R)
         {
-            int nregs = 1;
-
-            auto desc = i.getMsgDescriptor();
-            if (desc.type == SendDescArg::IMM) {
-                nregs = (desc.imm >> 20) & 0x1F; // desc[24:20] => rlen
-            } else {
+            int nregs = i.getDstLength();
+            if (nregs < 0)
                 nregs = 31;
-            }
-            for (int i = 0; i < nregs; i++) {
-                uint8_t regNum = op.getDirRegRef().regNum;
-                if ((regNum + i) >= RS_GRF_R.numRegisters) {
+            for (int ri = 0; ri < nregs; ri++) {
+                uint16_t regNum = op.getDirRegRef().regNum;
+                if ((regNum + ri) >= RS_GRF_R.numRegisters) {
                     break;
                 }
-                added |= rs.addFullReg(RS_GRF_R, regNum + i);
+                added |= rs.addFullReg(RS_GRF_R, regNum + ri);
             }
             rgn = Region::DST1;
         } else {
