@@ -1,8 +1,12 @@
 #include "Models.hpp"
 
+// this must precede model*.hpp inclusion below
+#include "../strings.hpp"
+
+
 // for bxml/Model operand type mappings
 #define TYPE(T) \
-    ENUM_BITSET_VALUE(T,uint32_t)
+    ENUM_BITSET_VALUE(T, uint32_t)
 
 #include "bxml/Model7P5.hpp"
 #include "bxml/Model8.hpp"
@@ -11,7 +15,6 @@
 #include "bxml/Model11.hpp"
 #include "bxml/Model12P1.hpp"
 #include "../bits.hpp"
-#include "../bits.hpp"
 #include "../Backend/Native/MInst.hpp"
 
 #include <sstream>
@@ -20,7 +23,6 @@
 using namespace iga;
 
 
-
 // full "constructor"
 #define UNWRAP_TUPLE(...) {__VA_ARGS__}
 #define IGA_REGISTER_SPEC(\
@@ -46,14 +48,14 @@ using namespace iga;
 static const struct RegInfo REGISTER_SPECIFICATIONS[] = {
     IGA_REGISTER_SPEC_LE(
         Platform::GEN11,
-        RegName::GRF_R,"r","General",
+        RegName::GRF_R, "r", "General",
         0, 0,
         1,
         128,(0)),
 
     IGA_REGISTER_SPEC_GE(
         Platform::GEN12P1,
-        RegName::GRF_R,"r","General",
+        RegName::GRF_R, "r", "General",
         0,0, // regNum7_4, regNumBase
         1,   // accGran
         256,(0)),
@@ -224,9 +226,8 @@ const OpSpec& Model::lookupOpSpecByCode(unsigned opcode) const
     //     opsByCodeValid = true;
     // }
     for (int i = (int)Op::FIRST_OP; i <= (int)Op::LAST_OP; i++) {
-        // FIXME: BXML needs to default invalid fields to -1
         if (opsArray[i].op != Op::INVALID &&
-            opsArray[i].code == opcode)
+            opsArray[i].opcode == opcode)
         {
             return opsArray[i];
         }
@@ -252,164 +253,6 @@ static unsigned getBitsFromFragments(const uint64_t *qws, const Fragment ff[N])
     return bits;
 }
 
-// determines if a given group op has ternary subops
-// this assumes all subops are the same format (all ternary or not)
-static bool groupOpsAreTernaryFamily(
-    const Model &model,
-    const OpSpec *osGroup)
-{
-    for (int i = 0; osGroup->subopsLength; i++) {
-        // find the first valid subop on this platform (model)
-        const OpSpec &child = model.lookupOpSpec((Op)((int)osGroup->subopStart + i));
-        if (child.isGroup()) {
-            return groupOpsAreTernaryFamily(model, &child);
-        } else if (child.isValid()) {
-            return child.isTernary();
-        }
-    }
-    // all are invalid: something is probably wrong
-    return false;
-}
-
-
-const OpSpec& LookupOpSpecByBitsNative(
-    const Model &model,
-    const MInst *mi,
-    OpSpecMissInfo &missInfo,
-    const OpSpec *parentOp)
-{
-    while (parentOp->isGroup()) {
-        auto sfBits = mi->getField(parentOp->functionControlField);
-        const OpSpec &subOp = model.lookupGroupSubOp(parentOp->op, (int)sfBits);
-        if (!subOp.isValid()) {
-            missInfo.parent = parentOp;
-            missInfo.opcode = sfBits;
-            return subOp;
-        }
-        parentOp = &subOp;
-    }
-    return *parentOp;
-}
-
-const OpSpec& LookupOpSpecSubfunctionByBitsCompacted(
-    const Model &model,
-    const MInst *miCmp,
-    OpSpecMissInfo &missInfo,
-    const CompactionMapping **compactedFields,
-    size_t numCompactedFields,
-    const OpSpec *parentOp)
-{
-    // HARD CASE: we have to extract the subop from one or more
-    // compacted fields. E.g. we have Op::MATH and need to
-    // resolve this to Op::MATH_INV.  But we need to get the
-    // subfunction (MathFC) from the FlagMod bits.
-    //
-    // The generalized form is a bit harder and we describe it in
-    // detail.
-    //
-    // NOTE: below {X,Y} means simple bitwise concatenation
-    //       (X<<(8*sizeof(Y)) | Y)
-    // NOTE: native means an uncompacted 128b encoding
-    //
-    // However, the OpSpec::functionControlField, which tells us where
-    // the native encoding can find subfunction field fragments
-    // may contain multiple fragments.  Moreover, they don't have to map
-    // exactly onto existing native fields.  For instance, they might be
-    // a subset or even superset of native fields.  The spec just lists
-    // the pieces that exist. E.g.
-    //    <Property Name="Subfunctions" Value="MathFC[27:24]" ... />
-    //                                         ^^^^ native field
-    // Moreover, this can be multiple fragments (made up example)
-    //    <Property Name="Subfunctions" Value="Foo[102,89:86,77:74]" ... />
-    // would mean fragments {102,89:64,27:24}
-    // And 102 could just be a subset of some compacted field.
-    //
-    // Visually:
-    //  So in general a subfunction value might consist of bits
-    //    {TTTT,SSSS}
-    // where these are drawn from the uncompacted format
-    //   ......TTTT.........SSSS..................128b
-    // and the uncompacted fields that we compact might be A, B, and C
-    //   ......CCCC.........BBAA..................128b
-    //         TTTT         SSSS < uses two fields
-    //
-    // *********************************************************************
-    // The easiest way to solve this is to just partially uncompact enough
-    // of the instruction parts we need and then pull the subfunction fields
-    // from from that dummy instruction.
-    MInst partialUncompact;
-    partialUncompact.qw0 = partialUncompact.qw1 = 0;
-
-    // checks if a native field is used by our subfunction bits
-    auto fragmentOverlapsSubfunction = [&] (const Fragment &nf) {
-        for (const Fragment &sfFrag :
-            parentOp->functionControlField.fragments)
-        {
-            if (sfFrag.length == 0) // end of the subfunction fragments
-                break;
-            if (sfFrag.overlaps(nf))
-                return true;
-        }
-        return false;
-    };
-    for (size_t cfIx = 0; cfIx < numCompactedFields; cfIx++ ) {
-        const CompactionMapping &cf = *compactedFields[cfIx];
-        uint64_t cmpIndex = miCmp->getField(cf.index);
-        uint64_t mappedValue = cf.values[cmpIndex];
-
-        int offsetWithinCompactedValue = 0;
-        for (int k = (int)cf.numMappings - 1; k >= 0; --k) {
-            // iterate all the packed native fields within compacted value
-            // starting from low bits in the mapping and iterate on up to
-            // the higher ones
-            //
-            // e.g. CMP_SRIX_3SRC has mappings
-            // { Src2.SubRegNum[4:0], Src1.SubRegNum[4:0],
-            //   Src0.SubRegNum[4:0], Dst.SubRegNum[4:0] }
-            // we go fro Dst.SubReg[4:0] upwards
-            const Fragment &nativeFrag = *cf.mappings[k];
-            if (fragmentOverlapsSubfunction(nativeFrag)) {
-                uint64_t nativeVal =
-                    getBits(
-                        mappedValue,
-                        offsetWithinCompactedValue,
-                        nativeFrag.length);
-                partialUncompact.setFragment(nativeFrag, nativeVal);
-            }
-
-            offsetWithinCompactedValue += nativeFrag.length;
-        }
-    }
-
-    // extract from the paritally uncompacted native instruction
-    // the bits in parentOp->functionControlFields will be unpacked
-    //
-    // A static test should be run to ensure model consistency
-    // I.e. subfunction bits may not come from unpacked fields (e.g. Src0.Reg)
-    // since we won't look there (you could enable this if needed with
-    // modifications)
-    auto sfBits = partialUncompact.getField(parentOp->functionControlField);
-
-    // lookup the subfunction bits
-    const OpSpec *subOp = &model.lookupGroupSubOp(parentOp->op, (unsigned)sfBits);
-    if (!subOp->isValid()) {
-        missInfo.parent = parentOp;
-        missInfo.opcode = sfBits;
-        return *subOp;
-    } else if (subOp->isGroup()) {
-        // subop of a subop (not used currently)
-        return LookupOpSpecSubfunctionByBitsCompacted(
-            model,
-            miCmp,
-            missInfo,
-            compactedFields,
-            numCompactedFields,
-            subOp);
-    } else {
-        // valid lookup
-        return *subOp;
-    }
-}
 
 const OpSpec& Model::lookupOpSpecFromBits(
     const void *bits,
@@ -419,55 +262,11 @@ const OpSpec& Model::lookupOpSpecFromBits(
     //
     const MInst *mi = (const MInst *)bits;
     auto opc = mi->getFragment(F_OPCODE);
-    missInfo.parent = nullptr;
     missInfo.opcode = opc;
     const OpSpec *os = &lookupOpSpecByCode((unsigned)opc);
-    if (mi->isCompact()) {
-        if (!os->isGroup()) {
-            return *os;
-        } else {
-            const CompactionMapping **compactedFields = nullptr;
-            size_t numCompactedFields = 0;
-            if (compactedFields)
-              return LookupOpSpecSubfunctionByBitsCompacted(
-                  *this, mi, missInfo, compactedFields, numCompactedFields, os);
-
-            compactedFields = nullptr;
-            IGA_ASSERT_FALSE("no compaction tables on this platform (yet)");
-            return *os;
-        } // else(!os->isGroup())
-    } else {
-        return LookupOpSpecByBitsNative(*this, mi, missInfo, os);
-    }
+    return *os;
 }
 
-const OpSpec &Model::lookupGroupSubOp(Op op, unsigned fcBits) const
-{
-    const OpSpec& os = opsArray[(int)op];
-    IGA_ASSERT(os.format == OpSpec::GROUP, "must be group op");
-    for (int i = 0; i < os.subopsLength; i++) {
-        const OpSpec &sos = opsArray[(int)os.subopStart + i];
-        if (sos.functionControlValue == fcBits) {
-            return sos;
-        }
-    }
-    // return invalid without asserting
-    //
-    // std::stringstream ss;
-    // ss << "unable to find subop of ";
-    // auto mne = lookupOpSpec(op).mnemonic;
-    // mne = mne ? mne : "?";
-    // ss << std::hex << "0x" << (int)op << " (" << mne << ")";
-    // IGA_ASSERT_FALSE(ss.str().c_str());
-    return opsArray[static_cast<int>(Op::INVALID)];
-}
-
-const OpSpec* Model::lookupSubOpParent(const OpSpec &os) const
-{
-    if (os.groupOp == Op::INVALID)
-        return nullptr;
-    return &opsArray[static_cast<int>(os.groupOp)];
-}
 
 const RegInfo *Model::lookupRegInfoByRegName(RegName name) const
 {
@@ -510,7 +309,7 @@ const RegInfo* Model::lookupArfRegInfoByRegNum(uint8_t regNum7_0) const
     for (const RegInfo &ri : REGISTER_SPECIFICATIONS) {
         if (ri.regName == RegName::GRF_R) {
             continue; // GRF will be in the table as 0000b
-        } else if (ri.regNum7_4 == (regNum7_0 >> 4) && // RegNum[7:4] matches AND
+        } else if (ri.regNum7_4 == ((uint32_t)regNum7_0 >> 4) && // RegNum[7:4] matches AND
             ri.supportedOn(platform))           // platform matches
         {
             int shiftedRegNum = regNum - ri.regNumBase;
@@ -553,7 +352,7 @@ bool RegInfo::encode(int reg, uint8_t &regNumBits) const
     }
 
     if (regName == RegName::GRF_R) {
-        regNumBits = reg;
+        regNumBits = (uint8_t)reg;
     } else {
         // ARF
         // RegNum[7:4] = high bits from the spec
@@ -561,7 +360,7 @@ bool RegInfo::encode(int reg, uint8_t &regNumBits) const
         // this assert would suggest that something is busted in
         // the RegInfo table
         IGA_ASSERT(reg <= 0xF, "ARF encoding overflowed");
-        regNumBits = regNum7_4 << 4;
+        regNumBits = (uint8_t)(regNum7_4 << 4);
         regNumBits |= (uint8_t)reg;
     }
     return true;
@@ -601,3 +400,18 @@ const Model *Model::LookupModel(Platform p)
         return nullptr;
     }
 }
+
+const PlatformEntry iga::ALL_PLATFORMS[] {
+    {iga::Platform::GEN7,     "7",   "ivb"},
+    {iga::Platform::GEN7P5,   "7p5", "hsw"},
+    {iga::Platform::GEN8,     "8",   "bdw"},
+    {iga::Platform::GEN8LP,   "8lp",  "chv"},
+    {iga::Platform::GEN9,     "9",    "skl"},
+    {iga::Platform::GEN9LP,   "9lp",  "bxt"},
+    {iga::Platform::GEN9P5,   "9p5",  "kbl"},
+    {iga::Platform::GEN10,    "10",   "cnl"},
+    {iga::Platform::GEN11,    "11",   "icl", "icllp"},
+    {iga::Platform::GEN12P1,  "12p1", "tgl", "tgllp", "dg1"},
+};
+
+const size_t iga::ALL_PLATFORMS_LEN = sizeof(ALL_PLATFORMS)/sizeof(ALL_PLATFORMS[0]);
