@@ -37,12 +37,13 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 
 
-#include <stdint.h>
+#include <cstdint>
 #include <string>
-#include "Timer.hpp"
 #include <fstream>
 #include <iostream>
 
+#include "Timer.hpp"
+
 // actually do real timing
 #ifdef _MSC_VER
     // MSVC way of declaring a thread-local variable
@@ -58,7 +59,7 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #endif
 
 
-static const char* timerNames[TIMER_NUM_TIMERS] = {"Total", "GED"};
+static const char* timerNames[TIMER_NUM_TIMERS] {"Total", "GED"};
 
 #ifdef _WIN32
 static int64_t CurrTicks() {
@@ -125,20 +126,20 @@ void setIGAKernelName(const char *name)
 }
 
 
-int createIGANewTimer( const char* name )
+int createIGANewTimer(const char* name)
 {
     timers[numTimers].name = name;
     return numTimers++;
 }
 
 
-void startIGATimer( int timer )
+void startIGATimer(int timer)
 {
 #ifdef MEASURE_COMPILATION_TIME
-    if( timer < TIMER_NUM_TIMERS )
+    if (timer < TIMER_NUM_TIMERS)
     {
 #if defined(_DEBUG) && defined(CHECK_TIMER)
-        if(timers[timer].started)
+        if (timers[timer].started)
         {
             std::cerr << "***********************************************\n";
             std::cerr << "Timer already started.\n";
@@ -160,10 +161,10 @@ void startIGATimer( int timer )
 #endif
 }
 
-void stopIGATimer( int timer )
+void stopIGATimer(int timer)
 {
 #ifdef MEASURE_COMPILATION_TIME
-    if(timer < TIMER_NUM_TIMERS)
+    if (timer < TIMER_NUM_TIMERS)
     {
         int64_t stop = CurrTicks();
         timers[timer].time += (double)((stop - timers[timer].currentStart) / (double)proc_freq);
@@ -223,7 +224,7 @@ void dumpAllIGATimers(bool outputTime)
 
     for (unsigned int i = 0; i < getIGATotalTimers(); i++)
     {
-        if(outputTime)
+        if (outputTime)
         {
             krnlOutput << timers[i].time << "\t";
         }
