@@ -28,13 +28,14 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #define IGA_STRINGS_HPP
 
 #include <cstdarg>
-#include <cstdio>
+#include <cstring>
 #include <cstdint>
+#include <cstdio>
 #include <functional>
 #include <iostream>
 #include <ostream>
-#include <string>
 #include <sstream>
+#include <string>
 #include <vector>
 
 #ifdef _MSC_VER
@@ -63,7 +64,26 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
        vsnprintf(B, BLEN, __VA_ARGS__)
 #endif
 
+
+#define IGA_MODEL_STRING(X) X
+
+
+
+
+
+
+
 namespace iga {
+    ///////////////////////////////////////////////////////////////////////////
+    struct ModelString {
+        const char* text;
+        inline constexpr ModelString(const char *s) : text(s == nullptr ? "" : s) { }
+        inline constexpr ModelString() : ModelString(nullptr) { }
+        inline std::string str() const {return text;}
+        inline operator std::string() const {return str();}
+    };
+
+    ///////////////////////////////////////////////////////////////////////////
     // takes a printf-style pattern and converts it to a string
     std::string format(const char *pat, ...);
     std::string formatv(const char *pat, va_list &va);
@@ -108,7 +128,7 @@ namespace iga {
     //   Intercalator comma(os, ",");
     //   for (...) {
     //     comma.insert();
-    //     emitElement(...);
+    //     os << element;
     //   }
     class Intercalator {
         std::ostream &os;
