@@ -30,12 +30,9 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "VISADefines.h"
 #include "VISAOptions.h"
 #include "CompilerStats.h"
+#include "JitterDataStruct.h"
 
-typedef enum
-{
-    LIFETIME_START = 0,
-    LIFETIME_END = 1
-} VISAVarLifetime;
+#include "visa/include/RelocationInfo.h"
 
 class VISAKernel
 {
@@ -237,37 +234,51 @@ public:
     /// AppendVISAArithmeticTwoDstInst -- append a two-dst, two-source vISA arithmetic instruction to this kernel
     /// [pred] op (emask, execSize) dst0 dst1 src0 src1
     /// This is used by addc and subb
-    VISA_BUILDER_API virtual int AppendVISAArithmeticInst(ISA_Opcode opcode, VISA_PredOpnd *pred, VISA_EMask_Ctrl emask,
+    VISA_BUILDER_API virtual int AppendVISATwoDstArithmeticInst(ISA_Opcode opcode, VISA_PredOpnd *pred, VISA_EMask_Ctrl emask,
                                           VISA_Exec_Size executionSize, VISA_VectorOpnd *dst1, VISA_VectorOpnd *carry_borrow, VISA_VectorOpnd *src0, VISA_VectorOpnd *src1) = 0;
 
+
+
+
+
+
     /// FIXME: we should probably have separate API for logic and shift instructions, as the arguments they expect are quite different
     /// AppendVISALogicOrShiftInst -- append a two-dst, two-source vISA arithmetic instruction to this kernel
     /// [pred] op[.sat] (emask, execSize) dst src0 src1 [src2 src3]
     /// src2 is used only for bfe and bfi
     /// src3 is used only for bfi
-    VISA_BUILDER_API virtual int AppendVISALogicOrShiftInst(ISA_Opcode opcode, VISA_PredOpnd *pred, bool satMode, VISA_EMask_Ctrl emask,
-                                            VISA_Exec_Size executionSize, VISA_VectorOpnd *dst, VISA_VectorOpnd *src0, VISA_VectorOpnd *src1, VISA_VectorOpnd *src2 = NULL, VISA_VectorOpnd *src3 = NULL) = 0;
+    VISA_BUILDER_API virtual int AppendVISALogicOrShiftInst(
+        ISA_Opcode opcode, VISA_PredOpnd *pred, bool satMode,
+        VISA_EMask_Ctrl emask, VISA_Exec_Size executionSize,
+        VISA_VectorOpnd *dst,
+        VISA_VectorOpnd *src0, VISA_VectorOpnd *src1,
+        VISA_VectorOpnd *src2 = NULL, VISA_VectorOpnd *src3 = NULL) = 0;
 
     ///AppendVISALogicOrShiftInst
     ///Used to perform logic operations on predicates.
     ///The operand is constructed internally.
     ///Enforces none matching types
-    VISA_BUILDER_API virtual int AppendVISALogicOrShiftInst(ISA_Opcode opcode, VISA_EMask_Ctrl emask,
-                                            VISA_Exec_Size executionSize, VISA_PredVar *dst, VISA_PredVar *src0, VISA_PredVar *src1) = 0;
+    VISA_BUILDER_API virtual int AppendVISALogicOrShiftInst(
+        ISA_Opcode opcode, VISA_EMask_Ctrl emask, VISA_Exec_Size executionSize,
+        VISA_PredVar *dst, VISA_PredVar *src0, VISA_PredVar *src1) = 0;
     /// FIXME: why not use the addressof_opnd if we had introduced it earlier?
     /// AppendVISAAddrAddInst -- append an address add instruction to this kernel
     /// addr_add (emask, execSize) dst src0 src1
-    VISA_BUILDER_API virtual int AppendVISAAddrAddInst(VISA_EMask_Ctrl emask, VISA_Exec_Size executionSize, VISA_VectorOpnd *dst, VISA_VectorOpnd *src0, VISA_VectorOpnd *src1) = 0;
+    VISA_BUILDER_API virtual int AppendVISAAddrAddInst(
+        VISA_EMask_Ctrl emask, VISA_Exec_Size executionSize,
+        VISA_VectorOpnd *dst, VISA_VectorOpnd *src0, VISA_VectorOpnd *src1) = 0;
 
     /// AppendVISADataMovementInst -- append a one-source data movement instruction to this kernel
     /// [pred] op[.sat] (emask, execSize) dst src0
-    VISA_BUILDER_API virtual int AppendVISADataMovementInst(ISA_Opcode opcode, VISA_PredOpnd *pred, bool satMod, VISA_EMask_Ctrl emask,
-                                            VISA_Exec_Size executionSize, VISA_VectorOpnd *dst, VISA_VectorOpnd *src0) = 0;
+    VISA_BUILDER_API virtual int AppendVISADataMovementInst(
+        ISA_Opcode opcode, VISA_PredOpnd *pred, bool satMod, VISA_EMask_Ctrl emask,
+        VISA_Exec_Size executionSize, VISA_VectorOpnd *dst, VISA_VectorOpnd *src0) = 0;
 
     /// AppendVISADataMovementInst -- append a two-source data movement instruction to this kernel
     /// [pred] op[.sat] (emask, execSize) dst src0 src1
-    VISA_BUILDER_API virtual int AppendVISADataMovementInst(ISA_Opcode opcode, VISA_PredOpnd *pred, bool satMod, VISA_EMask_Ctrl emask,
-                                            VISA_Exec_Size executionSize, VISA_VectorOpnd *dst, VISA_VectorOpnd *src0, VISA_VectorOpnd *src1) = 0;
+    VISA_BUILDER_API virtual int AppendVISADataMovementInst(
+        ISA_Opcode opcode, VISA_PredOpnd *pred, bool satMod, VISA_EMask_Ctrl emask,
+        VISA_Exec_Size executionSize, VISA_VectorOpnd *dst, VISA_VectorOpnd *src0, VISA_VectorOpnd *src1) = 0;
 
     ///AppendVISAPredicateMove
     ///Moves the context of the Predicate in to a Vector Operand.
@@ -281,19 +292,24 @@ public:
 
     /// AppendVISAMinMaxInst -- append a two-source data movement instruction to this kernel
     /// min/max[.sat] (emask, execSize) dst src0 src1
-    VISA_BUILDER_API virtual int AppendVISAMinMaxInst(CISA_MIN_MAX_SUB_OPCODE subOpcode, bool satMod, VISA_EMask_Ctrl emask,
-                                            VISA_Exec_Size executionSize, VISA_VectorOpnd *dst, VISA_VectorOpnd *src0, VISA_VectorOpnd *src1) = 0;
+    VISA_BUILDER_API virtual int AppendVISAMinMaxInst(
+        CISA_MIN_MAX_SUB_OPCODE subOpcode, bool satMod, VISA_EMask_Ctrl emask,
+        VISA_Exec_Size executionSize, VISA_VectorOpnd *dst, VISA_VectorOpnd *src0, VISA_VectorOpnd *src1) = 0;
 
     /// AppendVISAComparisonInst -- append a two-source comparison instruction to this kernel
     ///                             Destination is a flag register.
     /// cmp.sub_op.f# (emask, execSize) nullDst src0 src1
-    VISA_BUILDER_API virtual int AppendVISAComparisonInst(VISA_Cond_Mod sub_op, VISA_EMask_Ctrl emask, VISA_Exec_Size executionSize, VISA_PredVar *dst, VISA_VectorOpnd *src0, VISA_VectorOpnd *src1) = 0;
+    VISA_BUILDER_API virtual int AppendVISAComparisonInst(
+        VISA_Cond_Mod sub_op, VISA_EMask_Ctrl emask, VISA_Exec_Size executionSize,
+        VISA_PredVar *dst, VISA_VectorOpnd *src0, VISA_VectorOpnd *src1) = 0;
 
     /// AppendVISAComparisonInst -- append a two-source comparison instruction to this kernel.
     ///                             Destination is a GRF register.
     ///                             Flag register is updated but live range is only this instruction.
     /// cmp.sub_op.f# (emask, execSize) dst src0 src1
-    VISA_BUILDER_API virtual int AppendVISAComparisonInst(VISA_Cond_Mod sub_op, VISA_EMask_Ctrl emask, VISA_Exec_Size executionSize, VISA_VectorOpnd *dst, VISA_VectorOpnd *src0, VISA_VectorOpnd *src1) = 0;
+    VISA_BUILDER_API virtual int AppendVISAComparisonInst(
+        VISA_Cond_Mod sub_op, VISA_EMask_Ctrl emask, VISA_Exec_Size executionSize,
+        VISA_VectorOpnd *dst, VISA_VectorOpnd *src0, VISA_VectorOpnd *src1) = 0;
 
     /// AppendVISACFGotoInst -- append a possibly divergent goto instruction to this kernel
     /// [pred] goto (emask, execSize) label
@@ -357,8 +373,10 @@ public:
     /// FIXME: why not have separate functions for them?
     /// AppendVISASurfAccessGatherScatterInst -- append gather/scatter instruction to this kernel
     /// globalOffset and elementOffset are both in unit of element size
-    VISA_BUILDER_API virtual int AppendVISASurfAccessGatherScatterInst(ISA_Opcode opcode, VISA_EMask_Ctrl emask, GATHER_SCATTER_ELEMENT_SIZE elementSize,
-                                                      VISA_Exec_Size executionSize, VISA_StateOpndHandle *surface, VISA_VectorOpnd *globalOffset, VISA_RawOpnd *elementOffset, VISA_RawOpnd *srcDst) = 0;
+    VISA_BUILDER_API virtual int AppendVISASurfAccessGatherScatterInst(
+        ISA_Opcode opcode, VISA_EMask_Ctrl emask, GATHER_SCATTER_ELEMENT_SIZE elementSize,
+        VISA_Exec_Size executionSize, VISA_StateOpndHandle *surface,
+        VISA_VectorOpnd *globalOffset, VISA_RawOpnd *elementOffset, VISA_RawOpnd *srcDst) = 0;
 
     /// AppendVISASurfAccessGather4Scatter4TypedInst -- append a typed dword gather4/scatter4 instruction to this kernel
     /// uOffset, vOffset, and rOffset are all in unit of pixels.
@@ -499,6 +517,8 @@ public:
 
     VISA_BUILDER_API virtual int AppendVISADebugLinePlaceholder() = 0;
 
+    VISA_BUILDER_API virtual int AppendVISALLVMInst(void *inst) = 0;
+
     /// AppendVISAMiscRawSend -- create a GEN send instruction
     /// [pred] send/sendc (esize) <dst> <src> <exMsgDesc> <desc> {emask}
     /// bit 0 of modifiers controls whether it's send (0) or sendc (1)
@@ -715,7 +735,7 @@ public:
     /// This function may only be called after Compile() is called
     /// If finalization fails, buffer will be set to NULL and size will be set to 0
     /// buffer must be de-allocated using freeBLock API.
-    VISA_BUILDER_API virtual int GetGenxBinary(void *&buffer, int &size) = 0;
+    VISA_BUILDER_API virtual int GetGenxBinary(void *&buffer, int &size) const = 0;
 
     /// GetJitInfo -- returns auxiliary information collected during finalization
     /// This function may only be called after Compile() is called
@@ -734,11 +754,16 @@ public:
     /// buffer must be de-allocated using freeBLock API.
     /// VISAMap contains VISA->GenISA mapping which also needs to be freed using
     /// freeBlock API. numEntries determines entries populated in VISAMap.
-    VISA_BUILDER_API virtual int GetGenxDebugInfo(void *&buffer, unsigned int &size, void*& VISAMap, unsigned int& numEntries) = 0;
+    VISA_BUILDER_API virtual int GetGenxDebugInfo(void *&buffer, unsigned int &size, void*& VISAMap, unsigned int& numEntries) const = 0;
 
     /// GetGenRelocEntryBuffer -- allocate and return a buffer of all GenRelocEntry that are created by vISA
     VISA_BUILDER_API virtual int GetGenRelocEntryBuffer(void *&buffer, unsigned int &byteSize, unsigned int &numEntries) = 0;
 
+    /// GetRelocations -- add vISA created relocations into given relocation list
+    /// This get the same information as GetGenRelocEntryBuffer, but in different foramt
+    typedef std::vector<vISA::ZERelocEntry> RelocListType;
+    VISA_BUILDER_API virtual int GetRelocations(RelocListType &relocs) = 0;
+
     /// SetGTPinInit -- pass igc_init_t struct instance
     /// VISA decodes this struct and enables options accordingly
     VISA_BUILDER_API virtual int SetGTPinInit(void* buffer) = 0;
@@ -782,6 +807,9 @@ public:
     //Gets the VISA string format for the operand
     VISA_BUILDER_API virtual std::string getVectorOperandName(VISA_VectorOpnd* opnd, bool showRegion) const = 0;
     VISA_BUILDER_API virtual std::string getPredicateOperandName(VISA_PredOpnd* opnd) const = 0;
+
+    /// getGenSize -- Get gen binary size of this kernel/function
+    VISA_BUILDER_API virtual int64_t getGenSize() const = 0;
 };
 
 class VISAFunction : public VISAKernel
@@ -802,8 +830,8 @@ public:
     /// getGenOffset -- Get gen binary offset of this function
     VISA_BUILDER_API virtual int64_t getGenOffset() const = 0;
 
-    /// getGenOffset -- Get gen binary size of this function
-    VISA_BUILDER_API virtual int64_t getGenSize() const = 0;
+    /// getFunctionName -- Get function name.
+    VISA_BUILDER_API virtual const char* getFunctionName() const = 0;
 };
 
 typedef enum
@@ -830,7 +858,8 @@ public:
     VISA_BUILDER_API virtual int WriteVISAHeader() = 0;
     VISA_BUILDER_API virtual std::stringstream& GetAsmTextStream() = 0;
     VISA_BUILDER_API virtual std::stringstream& GetAsmTextHeaderStream() = 0;
-    VISA_BUILDER_API virtual VISAKernel* GetVISAKernel() = 0;
+    VISA_BUILDER_API virtual VISAKernel* GetVISAKernel(const std::string& kernelName = "") = 0;
     VISA_BUILDER_API virtual int ClearAsmTextStreams() = 0;
+    VISA_BUILDER_API virtual std::string GetCriticalMsg() = 0;
 };
 #endif
