@@ -130,6 +130,9 @@ typedef enum {
     ALIGN_OWORD = 0x4,
     ALIGN_GRF = 0x5,
     ALIGN_2_GRF = 0x6,
+    ALIGN_HWORD = 0x7,
+    ALIGN_32WORD = 0x8,
+    ALIGN_64WORD = 0x9,
     ALIGN_BYTE = 0x0
 } VISA_Align;
 
@@ -621,8 +624,16 @@ typedef struct _vISA_RT_CONTROLS
     unsigned isCoarseMode:1; //controls coasrse mode bit inmsg descriptor
     unsigned isSampleIndex : 1; //controls whether sampleIndex is used.
     unsigned isHeaderMaskfromCe0 : 1;
+    unsigned isNullRT : 1;   // null render target
 } vISA_RT_CONTROLS;
 
+typedef enum
+{
+    LIFETIME_START = 0,
+    LIFETIME_END = 1
+} VISAVarLifetime;
+
+
 
 // FixedFunctionID: these are hardware FFID values
 enum FFID
