@@ -30,21 +30,23 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "Common_ISA_framework.h"
 #include "JitterDataStruct.h"
 #include "VISAKernel.h"
-#include "BuildCISAIR.h"
 #include "VISADefines.h"
-#include "visa_wa.h"
+#include "inc/common/sku_wa.h"
 
 
 extern "C"
-VISA_BUILDER_API int CreateVISABuilder(VISABuilder* &builder, vISABuilderMode mode, VISA_BUILDER_OPTION builderOption, TARGET_PLATFORM platform, int numArgs, const char* flags[], PVISA_WA_TABLE pWaTable)
+VISA_BUILDER_API int CreateVISABuilder(VISABuilder* &builder, vISABuilderMode mode,
+    VISA_BUILDER_OPTION builderOption, TARGET_PLATFORM platform, int numArgs,
+    const char* flags[], const PWA_TABLE pWaTable)
 {
-    if( builder != NULL )
+    if (builder)
     {
         return VISA_FAILURE;
     }
-    CISA_IR_Builder *cisa_builder = NULL;
-    int status = CISA_IR_Builder::CreateBuilder(cisa_builder, mode, builderOption, platform, numArgs, flags, pWaTable);
-    builder = static_cast< VISABuilder* >(cisa_builder);
+    CISA_IR_Builder* cisa_builder = NULL;
+    int status = CISA_IR_Builder::CreateBuilder(cisa_builder, mode, builderOption, platform,
+        numArgs, flags, pWaTable);
+    builder = static_cast<VISABuilder*>(cisa_builder);
 
     return status;
 }
@@ -53,7 +55,7 @@ extern "C"
 VISA_BUILDER_API int DestroyVISABuilder(VISABuilder *&builder)
 {
     CISA_IR_Builder *cisa_builder = (CISA_IR_Builder *) builder;
-    if( cisa_builder == NULL )
+    if (cisa_builder == NULL)
     {
         return VISA_FAILURE;
     }
